@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -35,7 +36,7 @@
 #include "amanda.h"
 #include "amfeatures.h"
 #include "am_sl.h"
-#include "util.h"		/* for bstrncmp() */
+#include "amutil.h"		/* for bstrncmp() */
 
 typedef struct g_option_s {
     char *str;
@@ -44,6 +45,8 @@ typedef struct g_option_s {
     char *auth;
     int maxdumps;
     char *config;
+    char *timestamp;
+    char *data_shm_control_name;
 } g_option_t;
 
 
