@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +16,14 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "amar.h"
+#include "file.h"
 
 /* Each block in an archive is made up of one or more records, where each
  * record is either a header record or a data record.  The two are
@@ -37,19 +39,19 @@ typedef struct header_s {
     /* magic is HEADER_MAGIC + ' ' + decimal version, NUL padded */
     char     magic[28];
 } header_t;
-#define HEADER_SIZE (SIZEOF(header_t))
+#define HEADER_SIZE (sizeof(header_t))
 
 typedef struct record_s {
     guint16  filenum;
     guint16  attrid;
     guint32  size;
 } record_t;
-#define RECORD_SIZE (SIZEOF(record_t))
+#define RECORD_SIZE (sizeof(record_t))
 #define MAX_RECORD_DATA_SIZE (4*1024*1024)
 
 #define MKRECORD(ptr, f, a, s, eoa) do { \
     record_t r; \
-    uint32_t size = s; \
+    guint32  size = s; \
     if (eoa) size |= EOA_BIT; \
     r.filenum = htons(f); \
     r.attrid = htons(a); \
@@ -111,6 +113,7 @@ struct amar_s {
     guint16   maxfilenum;	/* Next file number to allocate		*/
     header_t  hdr;		/* pre-constructed header		*/
     off_t     position;		/* current position in the archive	*/
+    off_t     record;		/* record number			*/
     GHashTable *files;		/* List of all amar_file_t		*/
     gboolean  seekable;		/* does lseek() work on this fd?	*/
 
@@ -243,7 +246,8 @@ amar_new(
     mode_t mode,
     GError **error)
 {
-    amar_t *archive = malloc(SIZEOF(amar_t));
+    amar_t *archive = malloc(sizeof(amar_t));
+    assert(archive != NULL);
 
     /* make some sanity checks first */
     g_assert(fd >= 0);
@@ -289,7 +293,7 @@ amar_close(
     /* verify all files are done */
     g_assert(g_hash_table_size(archive->files) == 0);
 
-    if (!flush_buffer(archive, error))
+    if (archive->mode == O_WRONLY && !flush_buffer(archive, error))
 	success = FALSE;
 
     g_hash_table_destroy(archive->files);
@@ -306,6 +310,13 @@ amar_size(
     return archive->position;
 }
 
+off_t
+amar_record(
+    amar_t *archive)
+{
+    return archive->record;
+}
+
 /*
  * Writing
  */
@@ -354,12 +365,17 @@ amar_new_file(
 
 	/* see if this fileid is already in use */
 	filenum = archive->maxfilenum;
-	if (g_hash_table_lookup(archive->files, &filenum))
-	    continue;
+	if (!g_hash_table_lookup(archive->files, &filenum))
+	    break;
 
-    } while (0);
+    } while (1);
 
     file = g_new0(amar_file_t, 1);
+    if (!file) {
+	g_set_error(error, amar_error_quark(), ENOSPC,
+		    "No more memory");
+	return NULL;
+    }
     file->archive = archive;
     file->filenum = archive->maxfilenum;
     file->size = 0;
@@ -415,7 +431,7 @@ foreach_attr_close(
 	return;
 
     if (!attr->wrote_eoa) {
-	amar_attr_close_no_remove(attr, error);
+	(void)amar_attr_close_no_remove(attr, error);
     }
 }
 
@@ -462,7 +478,12 @@ amar_new_attr(
     g_assert(attrid >= AMAR_ATTR_APP_START);
     g_assert(g_hash_table_lookup(file->attributes, &attrid_gint) == NULL);
 
-    attribute = malloc(SIZEOF(amar_attr_t));
+    attribute = malloc(sizeof(amar_attr_t));
+    if (attribute == NULL) {
+	g_set_error(error, amar_error_quark(), ENOSPC,
+		    "No more memory");
+	return NULL;
+    }
     attribute->file = file;
     attribute->size = 0;
     attribute->attrid = attrid;
@@ -603,41 +624,63 @@ amar_attr_add_data_fd(
     gboolean eoa,
     GError **error)
 {
-    amar_file_t   *file    = attribute->file;
-    amar_t        *archive = file->archive;
-    gssize size;
+    amar_file_t *file = attribute->file;
+    amar_t *archive = file->archive;
+    gsize size;
+    int read_error;
+    gboolean short_read;
     off_t filesize = 0;
     gpointer buf = g_malloc(MAX_RECORD_DATA_SIZE);
 
     g_assert(!attribute->wrote_eoa);
 
     /* read and write until reaching EOF */
-    while ((size = full_read(fd, buf, MAX_RECORD_DATA_SIZE)) >= 0) {
+    while (1) {
+	/*
+	 * NOTE: we want to write everything we read, even if the last read
+	 * returned an error. We check read_error only outside of the loop for
+	 * this reason, and exit early only on EOF (read_fully() returns 0).
+	 */
+
+	size = read_fully(fd, buf, MAX_RECORD_DATA_SIZE, &read_error);
+
+	if (size == 0) {
+	    if (eoa && !attribute->wrote_eoa) {
+		if (!write_record(archive, file, attribute->attrid,
+				  1, buf, size, error)) {
+		    filesize = -1;
+		}
+	    }
+	    break;
+	}
+
+	short_read = (size < MAX_RECORD_DATA_SIZE);
+
 	if (!write_record(archive, file, attribute->attrid,
-			    eoa && (size < MAX_RECORD_DATA_SIZE), buf, size, error))
-	    goto error_exit;
+	    eoa && short_read, buf, size, error)) {
+	    filesize = -1;
+	    break;
+	}
 
 	filesize += size;
 	attribute->size += size;
 
-	if (size < MAX_RECORD_DATA_SIZE)
+	if (short_read)
 	    break;
     }
 
-    if (size < 0) {
-	g_set_error(error, amar_error_quark(), errno,
-		    "Error reading from fd %d: %s", fd, strerror(errno));
-	goto error_exit;
-    }
     g_free(buf);
 
-    attribute->wrote_eoa = eoa;
+    if (read_error) {
+	g_set_error(error, amar_error_quark(), read_error,
+	    "Error reading from fd %d: %s", fd, strerror(read_error));
+	filesize = -1;
+    }
+
+    if (filesize != -1)
+	attribute->wrote_eoa = eoa;
 
     return filesize;
-
-error_exit:
-    g_free(buf);
-    return -1;
 }
 
 /*
@@ -716,9 +759,9 @@ buf_atleast_(
     else
 	to_read = hp->buf_size - hp->buf_offset - hp->buf_len;
 
-    bytes_read = full_read(archive->fd,
+    bytes_read = read_fully(archive->fd,
 			   hp->buf+hp->buf_offset+hp->buf_len,
-			   to_read);
+			   to_read, NULL);
     if (bytes_read < to_read)
 	hp->got_eof = TRUE;
     hp->just_lseeked = FALSE;
@@ -744,6 +787,7 @@ buf_skip_(
     /* easy case of buf_len > skipbytes is taken care of by the macro, below,
      * so we know we're clearing out the entire buffer here */
 
+    archive->position += hp->buf_len;
     skipbytes -= hp->buf_len;
     hp->buf_len = 0;
 
@@ -760,16 +804,18 @@ retry:
 	    hp->got_eof = TRUE;
 	    return FALSE;
 	}
+	archive->position += skipbytes;
     } else {
 	while (skipbytes) {
 	    gsize toread = MIN(skipbytes, hp->buf_size);
-	    gsize bytes_read = full_read(archive->fd, hp->buf, toread);
+	    gsize bytes_read = read_fully(archive->fd, hp->buf, toread, NULL);
 
 	    if (bytes_read < toread) {
 		hp->got_eof = TRUE;
 		return FALSE;
 	    }
 
+	    archive->position += bytes_read;
 	    skipbytes -= bytes_read;
 	}
     }
@@ -781,6 +827,7 @@ retry:
     (((skipbytes) <= (hp)->buf_len) ? \
 	((hp)->buf_len -= (skipbytes), \
 	 (hp)->buf_offset += (skipbytes), \
+	 archive->position += (skipbytes), \
 	 TRUE) \
       : buf_skip_((archive), (hp), (skipbytes)))
 
@@ -948,8 +995,9 @@ void amar_start_read(
     amar_t   *archive)
 {
     if (!archive->hp->event_read_extract) {
-	archive->hp->event_read_extract = event_register(archive->fd, EV_READFD,
-						         amar_read_cb, archive);
+	archive->hp->event_read_extract = event_create(archive->fd, EV_READFD,
+						       amar_read_cb, archive);
+	event_activate(archive->hp->event_read_extract);
     }
 }
 
@@ -964,6 +1012,7 @@ amar_read_cb(
     guint16  attrid;
     guint32  datasize;
     gboolean eoa;
+    gboolean record_processed = FALSE;
     file_state_t *fs = NULL;
     attr_state_t *as = NULL;
     GSList *iter;
@@ -979,7 +1028,8 @@ amar_read_cb(
 	int save_errno = errno;
 	g_debug("failed to read archive: %s", strerror(save_errno));
 	g_set_error(hp->error, amar_error_quark(), save_errno,
-			"failed to read archive: %s", strerror(save_errno));
+			"failed to read archive, position = %lld: %s",
+			(long long)archive->position, strerror(save_errno));
     }
     hp->buf_len += count;
 
@@ -1000,14 +1050,16 @@ amar_read_cb(
 
 	    if (sscanf(buf_ptr(hp), HEADER_MAGIC " %d", &vers) != 1) {
 		g_set_error(hp->error, amar_error_quark(), EINVAL,
-			    "Invalid archive header");
+			    "Invalid archive header, position = %lld",
+			    (long long)archive->position);
 		read_done(archive->hp);
 		return;
 	    }
 
 	    if (vers > HEADER_VERSION) {
 		g_set_error(hp->error, amar_error_quark(), EINVAL,
-			    "Archive version %d is not supported", vers);
+			    "Archive version %d is not supported, position = %lld",
+			    vers, (long long)archive->position);
 		read_done(archive->hp);
 		return;
 	    }
@@ -1015,12 +1067,13 @@ amar_read_cb(
 	    /* skip the header block */
 	    hp->buf_offset += HEADER_SIZE;
 	    hp->buf_len    -= HEADER_SIZE;
+	    record_processed = TRUE;
 	    continue; /* go to next record */
 
 	} else if (datasize > MAX_RECORD_DATA_SIZE) {
 	    g_set_error(hp->error, amar_error_quark(), EINVAL,
-			"Invalid record: data size must be less than %d",
-			MAX_RECORD_DATA_SIZE);
+			"Invalid record: data size must be less than %d, position = %lld",
+			MAX_RECORD_DATA_SIZE, (long long)archive->position);
 	    read_done(archive->hp);
 	    return;
 
@@ -1030,6 +1083,7 @@ amar_read_cb(
 	    break;
 	}
 
+	record_processed = TRUE;
 	/* find the file_state_t, if it exists */
 	for (iter = hp->file_states; iter; iter = iter->next) {
 	    if (((file_state_t *)iter->data)->filenum == filenum) {
@@ -1043,7 +1097,8 @@ amar_read_cb(
 	    if (attrid == AMAR_ATTR_EOF) {
 		if (datasize != 0) {
 		    g_set_error(hp->error, amar_error_quark(), EINVAL,
-				"Archive contains an EOF record with nonzero size");
+				"Archive contains an EOF record with nonzero size, position = %lld",
+				(long long)archive->position);
 		    read_done(archive->hp);
 		    return;
 		}
@@ -1091,8 +1146,8 @@ amar_read_cb(
 			break;
 		    }
 		    g_set_error(hp->error, amar_error_quark(), EINVAL,
-				"Archive file %d has an empty filename",
-				(int)filenum);
+				"Archive file %d has an empty filename, position = %lld",
+				(int)filenum, (long long)archive->position);
 		    read_done(archive->hp);
 		    return;
 		}
@@ -1100,7 +1155,8 @@ amar_read_cb(
 		if (!eoa) {
 		    g_set_error(hp->error, amar_error_quark(), EINVAL,
 				"Filename record for fileid %d does "
-				"not have its EOA bit set", (int)filenum);
+				"not have its EOA bit set, position = %lld",
+				(int)filenum, (long long)archive->position);
 		    hp->buf_offset += (RECORD_SIZE + datasize);
 		    hp->buf_len    -= (RECORD_SIZE + datasize);
 		    read_done(archive->hp);
@@ -1126,8 +1182,8 @@ amar_read_cb(
 
 	    } else {
 		g_set_error(hp->error, amar_error_quark(), EINVAL,
-			    "Unknown attribute id %d in archive file %d",
-			    (int)attrid, (int)filenum);
+			    "Unknown attribute id %d in archive file %d, position = %lld",
+			    (int)attrid, (int)filenum, (long long)archive->position);
 		read_done(archive->hp);
 		return;
 	    }
@@ -1239,10 +1295,11 @@ amar_read_cb(
 	hp->buf_offset = 0;
     }
 
-    if (count == -1 || count == 0) {
+    if (count == -1 || (count == 0 && (hp->buf_len == 0 || !record_processed))) {
 	if (count == 0 && hp->buf_len != 0) {
 	    g_set_error(hp->error, amar_error_quark(), EINVAL,
-			    "Archive ended with a partial record");
+			    "Archive ended with a partial record, position = %lld, buf_len = %zu",
+			(long long)archive->position, hp->buf_len );
 	}
 	hp->got_eof = TRUE;
 	amar_stop_read(archive);
@@ -1335,7 +1392,8 @@ amar_read(
 	GETRECORD(buf_ptr(&hp), filenum, attrid, datasize, eoa);
 	if (filenum != MAGIC_FILENUM) {
 	    g_set_error(error, amar_error_quark(), EINVAL,
-			"Archive read does not begin at a header record");
+			"Archive read does not begin at a header record, position = %lld",
+			(long long)archive->position);
 	    return FALSE;
 	}
     }
@@ -1346,6 +1404,7 @@ amar_read(
 
 	GETRECORD(buf_ptr(&hp), filenum, attrid, datasize, eoa);
 
+	archive->record++;
 	/* handle headers specially */
 	if (G_UNLIKELY(filenum == MAGIC_FILENUM)) {
 	    int vers;
@@ -1356,13 +1415,15 @@ amar_read(
 
 	    if (sscanf(buf_ptr(&hp), HEADER_MAGIC " %d", &vers) != 1) {
 		g_set_error(error, amar_error_quark(), EINVAL,
-			    "Invalid archive header");
+			    "Invalid archive header, position = %lld",
+			    (long long)archive->position);
 		return FALSE;
 	    }
 
 	    if (vers > HEADER_VERSION) {
 		g_set_error(error, amar_error_quark(), EINVAL,
-			    "Archive version %d is not supported", vers);
+			    "Archive version %d is not supported, position = %lld", vers,
+			    (long long)archive->position);
 		return FALSE;
 	    }
 
@@ -1375,8 +1436,8 @@ amar_read(
 
 	if (datasize > MAX_RECORD_DATA_SIZE) {
 	    g_set_error(error, amar_error_quark(), EINVAL,
-			"Invalid record: data size must be less than %d",
-			MAX_RECORD_DATA_SIZE);
+			"Invalid record: data size must be less than %d, position = %lld",
+			MAX_RECORD_DATA_SIZE, (long long)archive->position);
 	    return FALSE;
 	}
 
@@ -1396,12 +1457,13 @@ amar_read(
 	    if (attrid == AMAR_ATTR_EOF) {
 		if (datasize != 0) {
 		    g_set_error(error, amar_error_quark(), EINVAL,
-				"Archive contains an EOF record with nonzero size");
+				"Archive contains an EOF record with nonzero size, position = %lld",
+				(long long)archive->position);
 		    return FALSE;
 		}
 		if (fs) {
-		    success = finish_file(&hp, fs, FALSE);
 		    hp.file_states = g_slist_remove(hp.file_states, fs);
+		    success = finish_file(&hp, fs, FALSE);
 		    as = NULL;
 		    g_free(fs);
 		    fs = NULL;
@@ -1416,8 +1478,8 @@ amar_read(
 
 		if (fs) {
 		    /* TODO: warn - previous file did not end correctly */
-		    success = finish_file(&hp, fs, TRUE);
 		    hp.file_states = g_slist_remove(hp.file_states, fs);
+		    success = finish_file(&hp, fs, TRUE);
 		    as = NULL;
 		    g_free(fs);
 		    fs = NULL;
@@ -1443,15 +1505,16 @@ amar_read(
 			break;
 		    }
 		    g_set_error(error, amar_error_quark(), EINVAL,
-				"Archive file %d has an empty filename",
-				(int)filenum);
+				"Archive file %d has an empty filename, position = %lld",
+				(int)filenum, (long long)archive->position);
 		    return FALSE;
 		}
 
 		if (!eoa) {
 		    g_set_error(error, amar_error_quark(), EINVAL,
 				"Filename record for fileid %d does "
-				"not have its EOA bit set", (int)filenum);
+				"not have its EOA bit set, position = %lld",
+				(int)filenum, (long long)archive->position);
 		    return FALSE;
 		}
 
@@ -1472,8 +1535,8 @@ amar_read(
 		continue;
 	    } else {
 		g_set_error(error, amar_error_quark(), EINVAL,
-			    "Unknown attribute id %d in archive file %d",
-			    (int)attrid, (int)filenum);
+			    "Unknown attribute id %d in archive file %d, position = %lld",
+			    (int)attrid, (int)filenum, (long long)archive->position);
 		return FALSE;
 	    }
 	}
