@@ -2,6 +2,7 @@
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
  * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -29,9 +30,10 @@
  *
  * BSDTAR-PATH     (default /usr/bin/tar))
  * STATE-DIR       (default $AMSTATDIR/bsdtar)
- * DIRECTORY       (no default, if set, the backup will be from that directory
- *		    instead of from the --device)
+ * TARGET (DIRECTORY)  (no default, if set, the backup will be from that
+ *			directory instead of from the --device)
  * ONE-FILE-SYSTEM (default YES)
+ * SPARSE	   (default YES) -S on restore
  * INCLUDE-FILE
  * INCLUDE-LIST
  * INCLUDE-OPTIONAL
@@ -52,13 +54,14 @@
 #include "pipespawn.h"
 #include "amfeatures.h"
 #include "clock.h"
-#include "util.h"
+#include "amutil.h"
 #include "getfsent.h"
 #include "client_util.h"
 #include "conffile.h"
 #include "getopt.h"
 #include "event.h"
 #include "security-file.h"
+#include "ammessage.h"
 
 int debug_application = 1;
 #define application_debug(i, ...) do {	\
@@ -88,6 +91,8 @@ static amregex_t init_re_table[] = {
   AM_NORMAL_RE(": Cannot add file .*: No such file or directory$"),
   AM_NORMAL_RE(": Error exit delayed from previous errors"),
 
+  AM_ERROR_RE("ambsdtar: error"),
+
   /* catch-all: DMP_STRANGE is returned for all other lines */
   AM_STRANGE_RE(NULL)
 };
@@ -120,9 +125,10 @@ static void ambsdtar_backup(application_argument_t *argument);
 static void ambsdtar_restore(application_argument_t *argument);
 static void ambsdtar_validate(application_argument_t *argument);
 static void ambsdtar_index(application_argument_t *argument);
-static void ambsdtar_build_exinclude(dle_t *dle, int verbose,
+static void ambsdtar_build_exinclude(dle_t *dle,
 				     int *nb_exclude, char **file_exclude,
-				     int *nb_include, char **file_include);
+				     int *nb_include, char **file_include,
+				     char *dirname, messagelist_t *mlist);
 static char *ambsdtar_get_timestamps(application_argument_t *argument,
 				     int level,
 				     FILE *mesgstream, int command);
@@ -134,18 +140,22 @@ static GPtrArray *ambsdtar_build_argv(char *bsdtar_realpath,
 				      application_argument_t *argument,
 				      char *timestamps,
 				      char **file_exclude, char **file_include,
-				      int command);
+				      int command, messagelist_t *mlist);
+static char *command = NULL;
 static char *bsdtar_path;
 static char *state_dir;
-static char *bsdtar_directory;
+static char *bsdtar_target;
 static int bsdtar_onefilesystem;
+static int bsdtar_sparse;
 static GSList *normal_message = NULL;
 static GSList *ignore_message = NULL;
 static GSList *strange_message = NULL;
 static char   *exit_handling;
 static int    exit_value[256];
-static int    exit_status = 0;
 static off_t  gblocksize = 0;
+static int    ambsdtar_exit_value = 0;
+static FILE  *mesgstream;
+static gboolean restore_ok = TRUE;
 
 static struct option long_options[] = {
     {"config"          , 1, NULL,  1},
@@ -175,10 +185,37 @@ static struct option long_options[] = {
     {"tar-blocksize"   , 1, NULL, 28},
     {"no-unquote"      , 1, NULL, 29},
     {"command-options" , 1, NULL, 33},
-    {"verbose"          , 1, NULL, 36},
+    {"verbose"         , 1, NULL, 36},
+    {"sparse"          , 1, NULL, 37},
+    {"target"          , 1, NULL, 38},
     {NULL, 0, NULL, 0}
 };
 
+static message_t *
+ambsdtar_print_message(
+    message_t *message)
+{
+    if (strcasecmp(command, "selfcheck") == 0) {
+	return print_message(message);
+    }
+    if (message_get_severity(message) <= MSG_INFO) {
+	if (g_str_equal(command, "estimate")) {
+	    fprintf(stdout, "OK %s\n", get_message(message));
+	} else if (g_str_equal(command, "backup")) {
+	    fprintf(mesgstream, "| %s\n", get_message(message));
+	}
+    } else {
+	ambsdtar_exit_value = 1;
+	if (g_str_equal(command, "estimate")) {
+	    fprintf(stdout, "ERROR %s\n", get_message(message));
+	} else if (g_str_equal(command, "backup")) {
+	    fprintf(mesgstream, "sendbackup: error [%s]\n", get_message(message));
+	}
+    }
+    return message;
+}
+
+
 static char *
 escape_tar_glob(
     char *str,
@@ -269,9 +306,9 @@ main(
     char **	argv)
 {
     int c;
-    char *command;
     application_argument_t argument;
     int i;
+    char *bsdtar_onefilesystem_value = NULL;
 
 #ifdef BSDTAR
     bsdtar_path = g_strdup(BSDTAR);
@@ -279,10 +316,13 @@ main(
     bsdtar_path = NULL;
 #endif
     state_dir = NULL;
-    bsdtar_directory = NULL;
+    bsdtar_target = NULL;
     bsdtar_onefilesystem = 1;
+    bsdtar_sparse = 1;
     exit_handling = NULL;
 
+    glib_init();
+
     /* initialize */
 
     /*
@@ -310,6 +350,8 @@ main(
     safe_fd(3, 2);
 
     set_pname("ambsdtar");
+    set_pcomponent("application");
+    set_pmodule("ambsdtar");
 
     /* Don't die when child closes pipe */
     signal(SIGPIPE, SIG_IGN);
@@ -324,7 +366,7 @@ main(
     startclock();
     g_debug(_("version %s"), VERSION);
 
-    config_init(CONFIG_INIT_CLIENT, NULL);
+    config_init(CONFIG_INIT_CLIENT|CONFIG_INIT_GLOBAL, NULL);
 
     //check_running_as(RUNNING_AS_DUMPUSER_PREFERRED);
     //root for amrecover
@@ -333,6 +375,10 @@ main(
     /* parse argument */
     command = argv[1];
 
+    if (strcasecmp(command,"selfcheck") == 0) {
+	fprintf(stdout, "MESSAGE JSON\n");
+    }
+
     argument.config     = NULL;
     argument.host       = NULL;
     argument.message    = 0;
@@ -381,12 +427,8 @@ main(
 	case 11: amfree(state_dir);
 		 state_dir = g_strdup(optarg);
 		 break;
-	case 12: if (strcasecmp(optarg, "NO") == 0)
-		     bsdtar_onefilesystem = 0;
-		 else if (strcasecmp(optarg, "YES") == 0)
-		     bsdtar_onefilesystem = 1;
-		 else if (strcasecmp(command, "selfcheck") == 0)
-		     printf(_("ERROR [%s: bad ONE-FILE-SYSTEM property value (%s)]\n"), get_pname(), optarg);
+	case 12: amfree(bsdtar_onefilesystem_value);
+		 bsdtar_onefilesystem_value = g_strdup(optarg);
 		 break;
 	case 16: argument.dle.include_file =
 			 append_sl(argument.dle.include_file, optarg);
@@ -404,8 +446,8 @@ main(
 		 break;
 	case 21: argument.dle.exclude_optional = 1;
 		 break;
-	case 22: amfree(bsdtar_directory);
-		 bsdtar_directory = g_strdup(optarg);
+	case 22: amfree(bsdtar_target);
+		 bsdtar_target = g_strdup(optarg);
 		 break;
 	case 23: normal_message =
 			 g_slist_append(normal_message, optarg);
@@ -432,17 +474,45 @@ main(
 	case 36: if (strcasecmp(optarg, "YES") == 0)
 		     argument.verbose = 1;
 		 break;
+	case 37: if (strcasecmp(optarg, "NO") == 0)
+		     bsdtar_sparse = 0;
+		 break;
+	case 38: amfree(bsdtar_target);
+		 bsdtar_target = g_strdup(optarg);
+		 break;
 	case ':':
 	case '?':
 		break;
 	}
     }
 
+    if (g_str_equal(command, "backup")) {
+	mesgstream = fdopen(3, "w");
+	if (!mesgstream) {
+	    error(_("error mesgstream(%d): %s\n"), 3, strerror(errno));
+	}
+    }
+
     if (!argument.dle.disk && argument.dle.device)
 	argument.dle.disk = g_strdup(argument.dle.device);
     if (!argument.dle.device && argument.dle.disk)
 	argument.dle.device = g_strdup(argument.dle.disk);
 
+    if (bsdtar_onefilesystem_value) {
+	if (strcasecmp(bsdtar_onefilesystem_value, "NO") == 0) {
+	    bsdtar_onefilesystem = 0;
+	} else if (strcasecmp(bsdtar_onefilesystem_value, "YES") == 0) {
+	    bsdtar_onefilesystem = 1;
+	} else {
+	    delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702007, MSG_ERROR, 4,
+			"value", bsdtar_onefilesystem_value,
+			"disk", argument.dle.disk,
+			"device", argument.dle.device,
+			"hostname", argument.host)));
+	}
+    }
+
     argument.argc = argc - optind;
     argument.argv = argv + optind;
 
@@ -491,12 +561,12 @@ main(
 	g_debug("BSDTAR-PATH is not set");
     }
     if (state_dir) {
-	    g_debug("STATE-DIR %s", state_dir);
+	g_debug("STATE-DIR %s", state_dir);
     } else {
 	g_debug("STATE-DIR is not set");
     }
-    if (bsdtar_directory) {
-	g_debug("DIRECTORY %s", bsdtar_directory);
+    if (bsdtar_target) {
+	g_debug("TARGET %s", bsdtar_target);
     }
     g_debug("ONE-FILE-SYSTEM %s", bsdtar_onefilesystem? "yes":"no");
     {
@@ -541,7 +611,7 @@ main(
 
     dbclose();
 
-    return exit_status;
+    return ambsdtar_exit_value;
 }
 
 static char *validate_command_options(
@@ -572,6 +642,7 @@ ambsdtar_support(
     fprintf(stdout, "INDEX-LINE YES\n");
     fprintf(stdout, "INDEX-XML NO\n");
     fprintf(stdout, "MESSAGE-LINE YES\n");
+    fprintf(stdout, "MESSAGE-SELFCHECK-JSON YES\n");
     fprintf(stdout, "MESSAGE-XML NO\n");
     fprintf(stdout, "RECORD YES\n");
     fprintf(stdout, "INCLUDE-FILE YES\n");
@@ -591,26 +662,65 @@ ambsdtar_selfcheck(
     application_argument_t *argument)
 {
     char *option;
+    messagelist_t mlist = NULL;
+    messagelist_t mesglist = NULL;
+    char *dirname;
 
     if (argument->dle.disk) {
-	char *qdisk = quote_string(argument->dle.disk);
-	fprintf(stdout, "OK disk %s\n", qdisk);
-	amfree(qdisk);
+	delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702000, MSG_INFO, 3,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
     }
 
-    printf("OK ambsdtar version %s\n", VERSION);
-    ambsdtar_build_exinclude(&argument->dle, 1, NULL, NULL, NULL, NULL);
+    delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702001, MSG_INFO, 4,
+			"version", VERSION,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
 
-    printf("OK ambsdtar\n");
+    if (bsdtar_target) {
+	dirname = bsdtar_target;
+    } else {
+	dirname = argument->dle.device;
+    }
+
+    ambsdtar_build_exinclude(&argument->dle, NULL, NULL, NULL, NULL,
+                             dirname, &mlist);
+    for (mesglist = mlist; mesglist != NULL; mesglist = mesglist->next){
+	message_t *message = mesglist->data;
+	if (message_get_severity(message) > MSG_INFO)
+	    ambsdtar_print_message(message);
+	delete_message(message);
+    }
+    g_slist_free(mlist);
+
+    delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702004, MSG_INFO, 3,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
 
     if ((option = validate_command_options(argument))) {
-	fprintf(stdout, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
+	delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702014, MSG_ERROR, 4,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host,
+			"command-options", option)));
     }
 
     if (bsdtar_path) {
-	if (check_file(bsdtar_path, X_OK)) {
-	    char *bsdtar_realpath;
-	    if (check_exec_for_suid("BSDTAR_PATH", bsdtar_path, stdout, &bsdtar_realpath)) {
+	char *bsdtar_realpath;
+	message_t *message;
+	if ((message = check_exec_for_suid_message("BSDTAR_PATH", bsdtar_path, &bsdtar_realpath))) {
+	    delete_message(ambsdtar_print_message(message));
+	} else {
+	    message = ambsdtar_print_message(check_file_message(bsdtar_path, X_OK));
+	    if (message && message_get_severity(message) <= MSG_INFO) {
+	    } else {
 		char *bsdtar_version;
 		GPtrArray *argv_ptr = g_ptr_array_new();
 
@@ -624,39 +734,59 @@ ambsdtar_selfcheck(
 		    for (tv = bsdtar_version; *tv && !g_ascii_isdigit(*tv); tv++);
 		    for (bv = tv; *bv && *bv != ' '; bv++);
 		    if (*bv) *bv = '\0';
-		    printf("OK ambsdtar bsdtar-version %s\n", tv);
+		    delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702002, MSG_INFO, 4,
+			"bsdtar-version", bv,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
 		} else {
-		    printf(_("ERROR [Can't get %s version]\n"), bsdtar_path);
+		    delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702003, MSG_ERROR, 4,
+			"bsdtar-path", bsdtar_path,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
 		}
 
 		g_ptr_array_free(argv_ptr, TRUE);
 		amfree(bsdtar_version);
 	    }
-	    amfree(bsdtar_realpath);
+	    if (message)
+		delete_message(message);
 	}
+	amfree(bsdtar_realpath);
     } else {
-	printf(_("ERROR [BSDTAR program not available]\n"));
+	delete_message(ambsdtar_print_message(build_message(
+			AMANDA_FILE, __LINE__, 3702005, MSG_ERROR, 3,
+			"disk", argument->dle.disk,
+			"device", argument->dle.device,
+			"hostname", argument->host)));
     }
 
     set_root_privs(1);
     if (state_dir && strlen(state_dir) == 0)
 	state_dir = NULL;
     if (state_dir) {
-	check_dir(state_dir, R_OK|W_OK);
+	delete_message(ambsdtar_print_message(check_dir_message(state_dir, R_OK|W_OK)));
     } else {
-	printf(_("ERROR [No STATE-DIR]\n"));
+	delete_message(ambsdtar_print_message(build_message(
+                        AMANDA_FILE, __LINE__, 3702020, MSG_ERROR, 3,
+                        "disk", argument->dle.disk,
+                        "device", argument->dle.device,
+                        "hostname", argument->host)));
     }
 
-    if (bsdtar_directory) {
-	check_dir(bsdtar_directory, R_OK);
+    if (bsdtar_target) {
+	delete_message(ambsdtar_print_message(check_dir_message(bsdtar_target, R_OK)));
     } else if (argument->dle.device) {
-	check_dir(argument->dle.device, R_OK);
+	delete_message(ambsdtar_print_message(check_dir_message(argument->dle.device, R_OK)));
     }
 
     if (argument->calcsize) {
 	char *calcsize = g_strjoin(NULL, amlibexecdir, "/", "calcsize", NULL);
-	check_file(calcsize, X_OK);
-	check_suid(calcsize);
+	delete_message(ambsdtar_print_message(check_file_message(calcsize, X_OK)));
+	delete_message(ambsdtar_print_message(check_suid_message(calcsize)));
 	amfree(calcsize);
     }
     set_root_privs(0);
@@ -687,6 +817,7 @@ ambsdtar_estimate(
     GString   *strbuf;
     char      *option;
     char      *bsdtar_realpath;
+    message_t *message;
 
     if (!argument->level) {
         fprintf(stderr, "ERROR No level argument\n");
@@ -706,19 +837,44 @@ ambsdtar_estimate(
 	error("Invalid '%s' COMMAND-OPTIONS", option);
     }
 
+    if (state_dir && strlen(state_dir) == 0)
+	state_dir = NULL;
+    if (state_dir) {
+	message = check_dir_message(state_dir, R_OK|W_OK);
+	if (message) {
+	    if (message_get_severity(message) > MSG_INFO) {
+		ambsdtar_print_message(message);
+	    }
+	    delete_message(message);
+	}
+    } else {
+        fprintf(mesgstream, "ERROR No STATE-DIR\n");
+        exit(1);
+    }
+
     if (argument->calcsize) {
 	char *dirname;
 	int   nb_exclude;
 	int   nb_include;
+	messagelist_t mlist = NULL;
+	messagelist_t mesglist = NULL;
 
-	if (bsdtar_directory) {
-	    dirname = bsdtar_directory;
+	if (bsdtar_target) {
+	    dirname = bsdtar_target;
 	} else {
 	    dirname = argument->dle.device;
 	}
-	ambsdtar_build_exinclude(&argument->dle, 1,
+	ambsdtar_build_exinclude(&argument->dle,
 				 &nb_exclude, &file_exclude,
-				 &nb_include, &file_include);
+				 &nb_include, &file_include, dirname, &mlist);
+	for (mesglist = mlist; mesglist != NULL; mesglist = mesglist->next){
+	    message_t *message = mesglist->data;
+	    if (message_get_severity(message) > MSG_INFO)
+		fprintf(stdout, "ERROR %s\n", get_message(message));
+	    delete_message(message);
+	}
+	g_slist_free(mlist);
+	mlist = NULL;
 
 	run_calcsize(argument->config, "BSDTAR", argument->dle.disk, dirname,
 		     argument->level, file_exclude, file_include);
@@ -740,24 +896,32 @@ ambsdtar_estimate(
 	goto common_error;
     }
 
-    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
-	errmsg = g_strdup_printf("'%s' binary is not secure", bsdtar_path);
-	goto common_error;
-    }
-
-    if (!state_dir) {
-	errmsg = g_strdup(_("STATE-DIR not defined"));
+    if ((message = check_exec_for_suid_message("BSDTAR_PATH", bsdtar_path, &bsdtar_realpath))) {
+	errmsg = g_strdup(get_message(message));
+	delete_message(message);
 	goto common_error;
     }
 
     qdisk = quote_string(argument->dle.disk);
     for (levels = argument->level; levels != NULL; levels = levels->next) {
 	char *timestamps;
+	messagelist_t mlist = NULL;
+	messagelist_t mesglist = NULL;
+
 	level = GPOINTER_TO_INT(levels->data);
 	timestamps = ambsdtar_get_timestamps(argument, level, stdout, CMD_ESTIMATE);
 	argv_ptr = ambsdtar_build_argv(bsdtar_realpath,
 				       argument, timestamps, &file_exclude,
-				       &file_include, CMD_ESTIMATE);
+				       &file_include, CMD_ESTIMATE, &mlist);
+	for (mesglist = mlist; mesglist != NULL; mesglist = mesglist->next){
+	    message_t *message = mesglist->data;
+	    if (message_get_severity(message) > MSG_INFO)
+		fprintf(stdout, "ERROR %s\n", get_message(message));
+	    delete_message(message);
+	}
+	g_slist_free(mlist);
+	mlist = NULL;
+
 	amfree(timestamps);
 
 	start_time = curclock();
@@ -768,7 +932,7 @@ ambsdtar_estimate(
 	    goto common_exit;
 	}
 
-	tarpid = pipespawnv(bsdtar_path, STDERR_PIPE, 1,
+	tarpid = pipespawnv(bsdtar_realpath, STDERR_PIPE, 1,
 			    &nullfd, &nullfd, &pipefd,
 			    (char **)argv_ptr->pdata);
 
@@ -832,7 +996,7 @@ ambsdtar_estimate(
 		 level,
 		 (long long)size);
 
-	kill(-tarpid, SIGTERM);
+	(void)kill(-tarpid, SIGTERM);
 
 	g_debug(_("waiting for %s \"%s\" child"), bsdtar_path, qdisk);
 	waitpid(tarpid, &wait_status, 0);
@@ -842,7 +1006,7 @@ ambsdtar_estimate(
                 bsdtar_path, WTERMSIG(wait_status), dbfn());
 	    g_free(errmsg);
             errmsg = g_string_free(strbuf, FALSE);
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	} else if (WIFEXITED(wait_status)) {
 	    if (exit_value[WEXITSTATUS(wait_status)] == 1) {
                 strbuf = g_string_new(errmsg);
@@ -850,14 +1014,14 @@ ambsdtar_estimate(
                     bsdtar_path, WEXITSTATUS(wait_status), dbfn());
 		g_free(errmsg);
                 errmsg = g_string_free(strbuf, FALSE);
-		exit_status = 1;
+		ambsdtar_exit_value = 1;
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
 	    errmsg = g_strdup_printf(_("%s got bad exit: see %s"),
 				bsdtar_path, dbfn());
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	}
 	g_debug(_("after %s %s wait"), bsdtar_path, qdisk);
 
@@ -897,7 +1061,7 @@ common_error:
     amfree(qdisk);
     g_debug("%s", errmsg);
     fprintf(stdout, "ERROR %s\n", qerrmsg);
-    exit_status = 1;
+    ambsdtar_exit_value = 1;
     amfree(file_exclude);
     amfree(file_include);
     amfree(errmsg);
@@ -920,7 +1084,6 @@ typedef struct filter_s {
 static off_t dump_size = -1;
 static int   dataf = 1;
 static int   index_in;
-static FILE *mesgstream;
 static FILE *indexstream = NULL;
 
 static void read_fd(int fd, char *name, event_fn_t fn);
@@ -936,8 +1099,9 @@ read_fd(
     filter_t *filter = g_new0(filter_t, 1);
     filter->fd = fd;
     filter->name = g_strdup(name);
-    filter->event = event_register((event_id_t)filter->fd, EV_READFD,
-				   fn, filter);
+    filter->event = event_create((event_id_t)filter->fd, EV_READFD,
+				 fn, filter);
+    event_activate(filter->event);
 }
 
 static void
@@ -1097,7 +1261,6 @@ ambsdtar_backup(
     int         dumpin;
     char      *qdisk;
     char      *timestamps;
-    int        mesgf = 3;
     int        indexf = 4;
     int        outf;
     int        data_out;
@@ -1114,42 +1277,51 @@ ambsdtar_backup(
     char       new_timestamps[64];
     char      *option;
     char      *bsdtar_realpath;
-
-    mesgstream = fdopen(mesgf, "w");
-    if (!mesgstream) {
-	error(_("error mesgstream(%d): %s\n"), mesgf, strerror(errno));
-    }
+    messagelist_t mlist = NULL;
+    messagelist_t mesglist = NULL;
+    message_t *message;
 
     if (!bsdtar_path) {
-	fprintf(mesgstream, "? BSDTAR-PATH not defined");
-	error(_("BSDTAR-PATH not defined"));
+	fprintf(mesgstream, "sendbackup: error [BSDTAR-PATH not defined]\n");
+	exit(1);
     }
 
-    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
-	fprintf(mesgstream, "? '%s' binary is not secure\n", bsdtar_path);
-	error("'%s' binary is not secure", bsdtar_path);
+    if ((message = check_exec_for_suid_message("BSDTAR_PATH", bsdtar_path, &bsdtar_realpath))) {
+	fprintf(mesgstream, "sendbackup: error [%s]", get_message(message));
+	exit(1);
     }
 
     if ((option = validate_command_options(argument))) {
-	fprintf(mesgstream, "? Invalid '%s' COMMAND-OPTIONS\n", option);
-	error("Invalid '%s' COMMAND-OPTIONS", option);
+	fprintf(mesgstream, "sendbackup: error [Invalid '%s' COMMAND-OPTIONS]\n", option);
+	exit(1);
     }
 
-    if (!state_dir) {
-	error(_("STATE-DIR not defined"));
+    if (state_dir && strlen(state_dir) == 0)
+	state_dir = NULL;
+    if (state_dir) {
+	message = check_dir_message(state_dir, R_OK|W_OK);
+	if (message) {
+	    if (message_get_severity(message) > MSG_INFO) {
+		ambsdtar_print_message(message);
+	    }
+	    delete_message(message);
+	}
+    } else {
+        fprintf(mesgstream, "sendbackup: error [STATE-DIR not defined]\n");
+        exit(1);
     }
 
     if (!argument->level) {
-        fprintf(mesgstream, "? No level argument\n");
-        error(_("No level argument"));
+        fprintf(mesgstream, "sendbackup: error [No level argument]\n");
+        exit(1);
     }
     if (!argument->dle.disk) {
-        fprintf(mesgstream, "? No disk argument\n");
-        error(_("No disk argument"));
+        fprintf(mesgstream, "sendbackup: error [No disk argument]\n");
+        exit(1);
     }
     if (!argument->dle.device) {
-        fprintf(mesgstream, "? No device argument\n");
-        error(_("No device argument"));
+        fprintf(mesgstream, "sendbackup: error [No device argument]\n");
+        exit(1);
     }
 
     qdisk = quote_string(argument->dle.disk);
@@ -1162,7 +1334,18 @@ ambsdtar_backup(
 				   mesgstream, CMD_BACKUP);
     argv_ptr = ambsdtar_build_argv(bsdtar_realpath,
 				   argument, timestamps, &file_exclude,
-				   &file_include, CMD_BACKUP);
+				   &file_include, CMD_BACKUP, &mlist);
+    for (mesglist = mlist; mesglist != NULL; mesglist = mesglist->next){
+	message_t *message = mesglist->data;
+	if (message_get_severity(message) <= MSG_INFO) {
+	    fprintf(mesgstream, "| %s\n", get_message(message));
+	} else {
+	    fprintf(mesgstream, "? %s\n", get_message(message));
+	}
+	delete_message(message);
+    }
+    g_slist_free(mlist);
+    mlist = NULL;
 
     if (argument->dle.create_index) {
 	tarpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
@@ -1187,7 +1370,7 @@ ambsdtar_backup(
 	read_fd(data_out , "data out" , &read_data_out);
 	read_fd(outf     , "data err" , &read_text);
 	read_fd(index_out, "index out", &read_text);
-	read_fd(index_err, "index_err", &read_text);
+	read_fd(index_err, "index err", &read_text);
     } else {
 	tarpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDERR_PIPE, 1,
 			&dumpin, &dataf, &outf, (char **)argv_ptr->pdata);
@@ -1203,38 +1386,38 @@ ambsdtar_backup(
     if (WIFSIGNALED(wait_status)) {
 	errmsg = g_strdup_printf(_("%s terminated with signal %d: see %s"),
 			    bsdtar_path, WTERMSIG(wait_status), dbfn());
-	exit_status = 1;
+	ambsdtar_exit_value = 1;
     } else if (WIFEXITED(wait_status)) {
 	if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 	    errmsg = g_strdup_printf(_("%s exited with status %d: see %s"),
 				bsdtar_path, WEXITSTATUS(wait_status), dbfn());
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	} else {
 	    /* Normal exit */
 	}
     } else {
 	errmsg = g_strdup_printf(_("%s got bad exit: see %s"),
 			    bsdtar_path, dbfn());
-	exit_status = 1;
+	ambsdtar_exit_value = 1;
     }
     if (argument->dle.create_index) {
 	waitpid(indexpid, &wait_status, 0);
 	if (WIFSIGNALED(wait_status)) {
 	    errmsg = g_strdup_printf(_("'%s index' terminated with signal %d: see %s"),
 			    bsdtar_path, WTERMSIG(wait_status), dbfn());
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	} else if (WIFEXITED(wait_status)) {
 	    if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 		errmsg = g_strdup_printf(_("'%s index' exited with status %d: see %s"),
 				bsdtar_path, WEXITSTATUS(wait_status), dbfn());
-		exit_status = 1;
+		ambsdtar_exit_value = 1;
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
 	    errmsg = g_strdup_printf(_("'%s index' got bad exit: see %s"),
 			    bsdtar_path, dbfn());
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	}
     }
     g_debug(_("after %s %s wait"), bsdtar_path, qdisk);
@@ -1242,7 +1425,7 @@ ambsdtar_backup(
     if (errmsg) {
 	g_debug("%s", errmsg);
 	g_fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
-	exit_status = 1;
+	ambsdtar_exit_value = 1;
     } else if (argument->dle.record) {
 	ambsdtar_set_timestamps(argument,
 				GPOINTER_TO_INT(argument->level->data),
@@ -1253,8 +1436,6 @@ ambsdtar_backup(
 
     g_debug("sendbackup: size %lld", (long long)dump_size);
     fprintf(mesgstream, "sendbackup: size %lld\n", (long long)dump_size);
-    g_debug("sendbackup: end");
-    fprintf(mesgstream, "sendbackup: end\n");
 
     if (argument->dle.create_index)
 	fclose(indexstream);
@@ -1338,7 +1519,7 @@ handle_restore_stdin(
 	/* process the complete buffer */
 	int nwrite = full_write(filter->out , filter->buffer, nread);
 	if (nwrite < nread) {
-	    exit_status = 1;
+	    ambsdtar_exit_value = 1;
 	    g_debug("wrote only %d bytes", nwrite);
 	    event_release(filter->event);
 	    filter->event = NULL;
@@ -1406,7 +1587,18 @@ handle_restore_stderr(
 	if (*b == 'x' && *(b+1) == ' ') {
 	    g_fprintf(stdout, "%s\n", b+2);
 	} else {
+	    gboolean line_is_error = TRUE;
+	    if (restore_ok)  {
+		if ((strncmp(b, "bsdtar: copyfile unpack (./.) failed: Operation not permitted", 61) == 0) ||
+		    (strncmp(b, "bsdtar: Error exit delayed from previous errors.", 48) == 0)) {
+		    line_is_error = FALSE;
+		} else {
+		    restore_ok = FALSE;
+		}
+	    }
+	    if (line_is_error) {
 	    g_fprintf(stderr, "%s\n", b);
+	    }
 	}
 	len = p - b + 1;
 	filter->first += len;
@@ -1435,16 +1627,20 @@ ambsdtar_restore(
     char       *errmsg = NULL;
     amwait_t    wait_status;
     char       *bsdtar_realpath;
+    message_t  *message;
 
+    restore_ok = TRUE;
     if (!bsdtar_path) {
 	error(_("BSDTAR-PATH not defined"));
     }
 
-    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
-	error("'%s' binary is not secure", bsdtar_path);
+    if ((message = check_exec_for_suid_message("BSDTAR_PATH", bsdtar_path, &bsdtar_realpath))) {
+	fprintf(stderr, "%s\n", get_message(message));
+	delete_message(message);
+	exit(1);
     }
 
-    if (!security_allow_to_restore(NULL)) {
+    if (!security_allow_to_restore()) {
 	error("The user is not allowed to restore files");
     }
 
@@ -1457,22 +1653,25 @@ ambsdtar_restore(
     }
     g_ptr_array_add(argv_ptr, g_strdup("-xvf"));
     g_ptr_array_add(argv_ptr, g_strdup("-"));
-    if (bsdtar_directory) {
+    if (bsdtar_sparse) {
+	g_ptr_array_add(argv_ptr, g_strdup("-S"));
+    }
+    if (bsdtar_target) {
 	struct stat stat_buf;
-	if(stat(bsdtar_directory, &stat_buf) != 0) {
-	    fprintf(stderr,"can not stat directory %s: %s\n", bsdtar_directory, strerror(errno));
+	if(stat(bsdtar_target, &stat_buf) != 0) {
+	    fprintf(stderr,"can not stat directory %s: %s\n", bsdtar_target, strerror(errno));
 	    exit(1);
 	}
 	if (!S_ISDIR(stat_buf.st_mode)) {
-	    fprintf(stderr,"%s is not a directory\n", bsdtar_directory);
+	    fprintf(stderr,"%s is not a directory\n", bsdtar_target);
 	    exit(1);
 	}
-	if (access(bsdtar_directory, W_OK) != 0) {
-	    fprintf(stderr, "Can't write to %s: %s\n", bsdtar_directory, strerror(errno));
+	if (access(bsdtar_target, W_OK) != 0) {
+	    fprintf(stderr, "Can't write to %s: %s\n", bsdtar_target, strerror(errno));
 	    exit(1);
 	}
 	g_ptr_array_add(argv_ptr, g_strdup("--directory"));
-	g_ptr_array_add(argv_ptr, g_strdup(bsdtar_directory));
+	g_ptr_array_add(argv_ptr, g_strdup(bsdtar_target));
     }
 
     if (argument->dle.exclude_list &&
@@ -1632,12 +1831,15 @@ ambsdtar_restore(
     err_buf.size = 0;
     err_buf.allocated_size = 0;
 
-    in_buf.event = event_register((event_id_t)0, EV_READFD,
+    in_buf.event = event_create((event_id_t)0, EV_READFD,
 			    handle_restore_stdin, &in_buf);
-    out_buf.event = event_register((event_id_t)tarout, EV_READFD,
+    out_buf.event = event_create((event_id_t)tarout, EV_READFD,
 			    handle_restore_stdout, &out_buf);
-    err_buf.event = event_register((event_id_t)tarerr, EV_READFD,
+    err_buf.event = event_create((event_id_t)tarerr, EV_READFD,
 			    handle_restore_stderr, &err_buf);
+    event_activate(in_buf.event);
+    event_activate(out_buf.event);
+    event_activate(err_buf.event);
 
     event_loop(0);
 
@@ -1645,11 +1847,13 @@ ambsdtar_restore(
     if (WIFSIGNALED(wait_status)) {
 	errmsg = g_strdup_printf(_("%s terminated with signal %d: see %s"),
 				 bsdtar_path, WTERMSIG(wait_status), dbfn());
-	exit_status = 1;
+	ambsdtar_exit_value = 1;
     } else if (WIFEXITED(wait_status) && WEXITSTATUS(wait_status) > 0) {
-	errmsg = g_strdup_printf(_("%s exited with status %d: see %s"),
-				 bsdtar_path, WEXITSTATUS(wait_status), dbfn());
-	exit_status = 1;
+	if (!restore_ok || WEXITSTATUS(wait_status) != 1) {
+	    errmsg = g_strdup_printf(_("%s exited with status %d: see %s"),
+				     bsdtar_path, WEXITSTATUS(wait_status), dbfn());
+	    ambsdtar_exit_value = 1;
+	}
     }
 
     g_debug(_("ambsdtar: %s: pid %ld"), bsdtar_path, (long)tarpid);
@@ -1699,6 +1903,7 @@ ambsdtar_validate(
     e = strerror(errno);
     g_debug("failed to execute %s: %s; Piping to /dev/null", cmd, e);
     fprintf(stderr,"failed to execute %s: %s; Piping to /dev/null\n", cmd, e);
+    free_env(env);
 pipe_to_null:
     while (read(0, buf, 32768) > 0) {
     }
@@ -1782,11 +1987,12 @@ ambsdtar_index(
 static void
 ambsdtar_build_exinclude(
     dle_t  *dle,
-    int     verbose,
     int    *nb_exclude,
     char  **file_exclude,
     int    *nb_include,
-    char  **file_include)
+    char  **file_include,
+    char   *dirname,
+    messagelist_t *mlist)
 {
     int n_exclude = 0;
     int n_include = 0;
@@ -1798,8 +2004,8 @@ ambsdtar_build_exinclude(
     if (dle->include_file) n_include += dle->include_file->nb_element;
     if (dle->include_list) n_include += dle->include_list->nb_element;
 
-    if (n_exclude > 0) exclude = build_exclude(dle, verbose);
-    if (n_include > 0) include = build_include(dle, verbose);
+    if (n_exclude > 0) exclude = build_exclude(dle, mlist);
+    if (n_include > 0) include = build_include(dle, dirname, mlist);
 
     if (nb_exclude)
 	*nb_exclude = n_exclude;
@@ -1875,7 +2081,7 @@ ambsdtar_get_timestamps(
 		    if (command == CMD_ESTIMATE) {
 			fprintf(mesgstream, "ERROR %s\n", errmsg);
 		    } else {
-			fprintf(mesgstream, "? %s\n", errmsg);
+			fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
 		    }
 		    exit(1);
 		}
@@ -1894,7 +2100,7 @@ ambsdtar_get_timestamps(
 	    if (command == CMD_ESTIMATE) {
 		fprintf(mesgstream, "ERROR %s\n", errmsg);
 	    } else {
-		fprintf(mesgstream, "? %s\n", errmsg);
+		fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
 	    }
 	    fclose(tt);
 	    exit(1);
@@ -1914,7 +2120,7 @@ ambsdtar_get_timestamps(
 	if (command == CMD_ESTIMATE) {
 		fprintf(mesgstream, "ERROR %s\n", errmsg);
 	} else {
-		fprintf(mesgstream, "? %s\n", errmsg);
+		fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
 	}
 	exit(1);
     }
@@ -1963,7 +2169,7 @@ ambsdtar_set_timestamps(
     } else {
 	errmsg =  _("STATE-DIR is not defined");
 	g_debug("%s", errmsg);
-	fprintf(mesgstream, "? %s\n", errmsg);
+	fprintf(mesgstream, "sendabckup: error [%s]\n", errmsg);
 	exit(1);
     }
 }
@@ -1975,7 +2181,8 @@ ambsdtar_build_argv(
     char  *timestamps,
     char **file_exclude,
     char **file_include,
-    int    command)
+    int    command,
+    messagelist_t *mlist)
 {
     int        nb_exclude = 0;
     int        nb_include = 0;
@@ -1984,16 +2191,16 @@ ambsdtar_build_argv(
     GPtrArray *argv_ptr = g_ptr_array_new();
     GSList    *copt;
 
-    ambsdtar_build_exinclude(&argument->dle, 1,
-			     &nb_exclude, file_exclude,
-			     &nb_include, file_include);
-
-    if (bsdtar_directory) {
-	dirname = bsdtar_directory;
+    if (bsdtar_target) {
+	dirname = bsdtar_target;
     } else {
 	dirname = argument->dle.device;
     }
 
+    ambsdtar_build_exinclude(&argument->dle,
+			     &nb_exclude, file_exclude,
+			     &nb_include, file_include, dirname, mlist);
+
     g_ptr_array_add(argv_ptr, g_strdup(bsdtar_realpath));
 
     g_ptr_array_add(argv_ptr, g_strdup("--create"));
