@@ -1,5 +1,6 @@
 #!@PERL@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -35,7 +36,7 @@ use Amanda::Constants;
 
 sub new {
     my $class = shift;
-    my ($execute_where, $config, $host, $disk, $device, $level, $index, $message, $collection, $record, $logfile, $text) = @_;
+    my ($execute_where, $config, $host, $disk, $device, $level, $index, $timestamp, $message, $collection, $record, $logfile, $text) = @_;
     my $self = $class->SUPER::new($execute_where, $config);
 
     $self->{execute_where} = $execute_where;
@@ -45,6 +46,7 @@ sub new {
     $self->{device}        = $device;
     $self->{level}         = [ @{$level} ]; # Copy the array
     $self->{index}         = $index;
+    $self->{timestamp}     = $timestamp;
     $self->{message}       = $message;
     $self->{collection}    = $collection;
     $self->{record}        = $record;
@@ -83,6 +85,7 @@ sub command_support {
    print "MESSAGE-LINE YES\n";
    print "MESSAGE-XML NO\n";
    print "EXECUTE-WHERE YES\n";
+   print "TIMESTAMP YES\n";
 }
 
 #define a execute_on_* function for every execute_on you want the script to do
@@ -271,6 +274,7 @@ EOF
     exit(1);
 }
 
+my $opt_version;
 my $opt_execute_where;
 my $opt_config;
 my $opt_host;
@@ -278,6 +282,7 @@ my $opt_disk;
 my $opt_device;
 my @opt_level;
 my $opt_index;
+my $opt_timestamp;
 my $opt_message;
 my $opt_collection;
 my $opt_record;
@@ -288,6 +293,7 @@ my @orig_argv = @ARGV;
 
 Getopt::Long::Configure(qw{bundling});
 GetOptions(
+    'version'         => \$opt_version,
     'execute-where=s' => \$opt_execute_where,
     'config=s'        => \$opt_config,
     'host=s'          => \$opt_host,
@@ -295,6 +301,7 @@ GetOptions(
     'device=s'        => \$opt_device,
     'level=s'         => \@opt_level,
     'index=s'         => \$opt_index,
+    'timestamp=s'     => \$opt_timestamp,
     'message=s'       => \$opt_message,
     'collection=s'    => \$opt_collection,
     'record=s'        => \$opt_record,
@@ -302,9 +309,14 @@ GetOptions(
     'text=s'          => \$opt_text
 ) or usage();
 
-my $script = Amanda::Script::amlog_script->new($opt_execute_where, $opt_config, $opt_host, $opt_disk, $opt_device, \@opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, $opt_logfile, $opt_text);
+if (defined $opt_version) {
+    print "amlog_script-" . $Amanda::Constants::VERSION , "\n";
+    exit(0);
+}
+
+my $script = Amanda::Script::amlog_script->new($opt_execute_where, $opt_config, $opt_host, $opt_disk, $opt_device, \@opt_level, $opt_index, $opt_timestamp, $opt_message, $opt_collection, $opt_record, $opt_logfile, $opt_text);
 
 Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
 
 $script->do($ARGV[0]);
-
+# NOTREACHED
