@@ -1,5 +1,6 @@
 #!@PERL@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -75,11 +76,16 @@ sub new {
         'pg-cleanupwal' => 'yes',
 	'pg-max-wal-wait' => 60,
 	'pg-full-wal' => 'incr',
+	'pg-incremental' => 'no',
+	'pg-remove-full-wal' => 'yes',
+	'pg-remove-incremental-wal' => 'no',
     };
 
     my @PROP_NAMES = qw(pg-host pg-port pg-db pg-user pg-password pg-passfile
 			psql-path pg-datadir pg-archivedir pg-cleanupwal
-			pg-max-wal-wait  pg-full-wal);
+			pg-max-wal-wait  pg-full-wal
+			pg-remove-full-wal pg-remove-incremental-wal
+			pg-incremental );
 
     # config is loaded by Amanda::Application (and Amanda::Script_App)
     my $conf_props = getconf($CNF_PROPERTY);
@@ -110,8 +116,8 @@ sub new {
 	$pdumpname =~ s/^pg-//g;
 	$self->{'props'}->{$pname} = $self->{'args'}->{$pdumpname}
 				 if defined $self->{'args'}->{$pdumpname};
-debug("prop $pname set from dumpname $pdumpname: $self->{'args'}->{$pdumpname}")
-if defined $self->{'args'}->{$pdumpname};
+	debug("prop $pname set from dumpname $pdumpname: $self->{'args'}->{$pdumpname}")
+	      if defined $self->{'args'}->{$pdumpname};
     }
 
     unless ($self->{'props'}->{'psql-path'}) {
@@ -169,6 +175,8 @@ COLLECTION NO
 CLIENT-ESTIMATE YES
 MULTI-ESTIMATE NO
 CALCSIZE NO
+EXCLUDE-FILE YES
+EXCLUDE-LIST YES
 EOF
 }
 
@@ -734,7 +742,21 @@ sub _get_pg_version {
     waitpid($pid, 0);
     $self->{'die_cb'}->("could not run psql to determine version") if (($? >> 8) != 0);
 
-    my ($maj, $min, $pat) = ($lines[0] =~ / ([0-9]+)\.([0-9]+)\.([0-9]+).*$/);
+    my ($maj, $min, $pat);
+    if ($lines[0] =~ /([0-9]+)\.([0-9]+)\.([0-9]+)/) {
+        $maj = $1;
+        $min = $2;
+        $pat = $3;
+    } elsif ($lines[0] =~ /([0-9]+)\.([0-9]+)/) {
+        $maj = $1;
+        $min = $2;
+        $pat = 0;
+    } elsif ($lines[0] =~ /([0-9]+)/) {
+        $maj = $1;
+        $min = 0;
+        $pat = 0;
+    }
+
     return $maj * 10000 + $min * 100 + $pat;
 }
 
@@ -787,6 +809,26 @@ sub _wait_for_wal {
     $self->{'die_cb'}->("WAL file $wal was not archived in $maxwait seconds");
 }
 
+sub _remove_wal_files {
+    my $self = shift;
+    my $start_wal = shift;
+    my $last_end_wal = shift;
+
+    if (defined $start_wal || defined $last_end_wal) {
+	my $adir = new IO::Dir($self->{'props'}->{'pg-archivedir'});
+	$adir or $self->{'die_cb'}->("Could not open archive WAL directory");
+	while (defined(my $fname = $adir->read())) {
+	    if ($fname =~ /^$_WAL_FILE_PAT$/) {
+		if ((defined $start_wal and $fname lt $start_wal) ||
+		    (defined $last_end_wal and $fname lt $last_end_wal)) {
+		    $self->{'unlink_cb'}->("$self->{'props'}->{'pg-archivedir'}/$fname");
+		}
+            }
+	}
+	$adir->close();
+    }
+}
+
 sub _base_backup {
    my ($self) = @_;
 
@@ -815,12 +857,28 @@ sub _base_backup {
        }
        $old_die_cb->($msg);
    };
-   my $size = _run_tar_totals($self, '--dereference', '--file', "-",
+
+   my @args = ('--dereference', '--file', "-",
        '--directory', $self->{'props'}->{'pg-datadir'},
        '--exclude', 'postmaster.pid',
        '--exclude', 'pg_xlog/*', # contains WAL files; will be handled below
-       '--transform', "s,^./pg_tblspc,./pg_tblspc_data,S;s,^,$_DATA_DIR_RESTORE/,S",
-       ".");
+       '--exclude', 'postmaster.pid',
+       '--exclude', 'postmaster.opts',
+       '--exclude', 'pg_replslot/*');
+   if (defined $self->{args}->{'exclude-file'}) {
+       foreach my $excl (@{$self->{args}->{'exclude-file'}}) {
+           push @args, '--exclude', $excl;
+       }
+   }
+   if (defined $self->{args}->{'exclude-list'}) {
+       foreach my $excl (@{$self->{args}->{'exclude-list'}}) {
+           push @args, '--exclude-from', $excl;
+       }
+   }
+   push @args, '--transform', "s,^./pg_tblspc,./pg_tblspc_data,S;s,^,$_DATA_DIR_RESTORE/,S",
+               '.';
+
+   my $size = _run_tar_totals($self,@args);
 
    # tar the symlink in pg_tblspc
    $size += _run_tar_totals($self, '--file', "-",
@@ -858,6 +916,8 @@ sub _base_backup {
    }
 
    # now grab all of the WAL files, *inclusive* of $start_wal
+   my $cleanup_wal_val = $self->{'props'}->{'pg-cleanupwal'} || 'yes';
+   my $cleanup_wal = string_to_boolean($cleanup_wal_val);
    my @wal_files;
    my $adir = new IO::Dir($self->{'props'}->{'pg-archivedir'});
    $adir or $self->{'die_cb'}->("Could not open archive WAL directory");
@@ -871,7 +931,9 @@ sub _base_backup {
                push @wal_files, $fname;
                debug("will store: $fname");
            } elsif (defined $start_wal and $fname lt $start_wal) {
-               $self->{'unlink_cb'}->("$self->{'props'}->{'pg-archivedir'}/$fname");
+               if ($cleanup_wal) {
+                   $self->{'unlink_cb'}->("$self->{'props'}->{'pg-archivedir'}/$fname");
+               }
            }
        }
    }
@@ -894,22 +956,15 @@ sub _base_backup {
        rmtree($dummydir);
    }
 
+    # should be done only on confirmation the backup succeeded from the server
     $self->{'state_cb'}->($self, $end_wal);
 
     # remove older WAL files.
+    my $remove_full_wal_val = $self->{'props'}->{'pg-remove-full-wal'} || 'yes';
+    my $remove_full_wal = string_to_boolean($remove_full_wal_val);
     # should be done only on confirmation the backup succeeded from the server
-    if (defined $start_wal || defined $last_end_wal) {
-	my $adir = new IO::Dir($self->{'props'}->{'pg-archivedir'});
-	$adir or $self->{'die_cb'}->("Could not open archive WAL directory");
-	while (defined(my $fname = $adir->read())) {
-	    if ($fname =~ /^$_WAL_FILE_PAT$/) {
-		if ((defined $start_wal and $fname lt $start_wal) ||
-		    (defined $last_end_wal and $fname le $last_end_wal)) {
-		    $self->{'unlink_cb'}->("$self->{'props'}->{'pg-archivedir'}/$fname");
-		}
-            }
-	}
-	$adir->close();
+    if ($remove_full_wal) {
+	$self->_remove_wal_files($start_wal, $last_end_wal);
     }
 
     $self->{'done_cb'}->($size);
@@ -920,6 +975,9 @@ sub _incr_backup {
 
    debug("running _incr_backup");
 
+   my $incremental_val = $self->{'props'}->{'pg-incremental'} || 'yes';
+   my $incremental = string_to_boolean($incremental_val);
+
    if ($self->{'action'} eq 'backup') {
       _run_psql_command($self, "SELECT file_name from pg_xlogfile_name_offset(pg_switch_xlog())");
       if (defined($self->{'switch_xlog_filename'})) {
@@ -927,7 +985,18 @@ sub _incr_backup {
       }
    }
 
-   my $end_wal = _get_prev_state($self);
+   my $end_wal;
+   if ($incremental) {
+	$end_wal = _get_prev_state($self, $self->{'args'}->{'level'});
+	for (my $level = $self->{'args'}->{'level'}-1; $level >= 0; $level--) {
+	    my $xend_wal = _get_prev_state($self, $level);
+	    if (!$end_wal || $xend_wal lt $end_wal) {
+		$end_wal = $xend_wal;
+	    }
+	}
+   } else {
+	$end_wal = _get_prev_state($self);
+   }
    if ($end_wal) {
        debug("previously ended at: $end_wal");
    } else {
@@ -947,8 +1016,6 @@ sub _incr_backup {
        }
    }
 
-   $self->{'state_cb'}->($self, $max_wal ? $max_wal : $end_wal);
-
    my $size;
    if (@wal_files) {
        $size = _run_tar_totals($self, '--dereference', '--file', '-',
@@ -960,6 +1027,20 @@ sub _incr_backup {
        rmtree($dummydir);
    }
 
+   # should be done only on confirmation the backup succeeded from the server
+   $self->{'state_cb'}->($self, $max_wal ? $max_wal : $end_wal);
+
+   my $remove_incremental_wal_val = $self->{'props'}->{'pg-remove-incremental-wal'} || 'no';
+   my $remove_incremental_wal = string_to_boolean($remove_incremental_wal_val);
+   # should be done only on confirmation the backup succeeded from the server
+   if ($remove_incremental_wal) {
+	my $end_wal = pop @wal_files;   # Do not unlink the latest wal files
+	foreach my $wal_file (@wal_files) {
+	    $end_wal = $wal_file if $wal_file gt $end_wal;
+	}
+	$self->_remove_wal_files(undef, $end_wal);
+   }
+
    $self->{'done_cb'}->($size);
 }
 
@@ -982,7 +1063,6 @@ sub command_backup {
 
        $self->{'index_h'}->print("/PostgreSQL-Database-$self->{'args'}->{'level'}\n");
 
-       $msg_fd->print("sendbackup: end\n");
    };
    $self->{'die_cb'} = sub {
        my $msg = shift @_;
@@ -1000,22 +1080,18 @@ sub command_backup {
    my $cleanup_wal = string_to_boolean($cleanup_wal_val);
    if (!defined($cleanup_wal)) {
        $self->{'die_cb'}->("couldn't interpret PG-CLEANUPWAL value '$cleanup_wal_val' as a boolean");
-   } elsif ($cleanup_wal) {
-       $self->{'unlink_cb'} = sub {
-           my $filename = shift @_;
-	   debug("unlinking WAL file $filename");
-           if (unlink($filename) == 0) {
-               debug("Failed to unlink '$filename': $!");
-               $self->print_to_server("Failed to unlink '$filename': $!",
-                                      $Amanda::Script_App::ERROR);
-           }
-       };
-   } else {
-       $self->{'unlink_cb'} = sub {
-           # do nothing
-       };
    }
 
+   $self->{'unlink_cb'} = sub {
+       my $filename = shift @_;
+       debug("unlinking WAL file $filename");
+       if (unlink($filename) == 0) {
+           debug("Failed to unlink '$filename': $!");
+           $self->print_to_server("Failed to unlink '$filename': $!",
+                                  $Amanda::Script_App::ERROR);
+       }
+   };
+
    if ($self->{'args'}->{'level'} > 0) {
        _incr_backup($self, \*STDOUT);
    } else {
@@ -1057,8 +1133,8 @@ sub command_restore {
 	if ($uid == 0) {
 	    opendir(my $dh, $_ARCHIVE_DIR_RESTORE) || die;
 	    while (my $name = readdir $dh) {
-		next if $name == '.';
-		next if $name == '..';
+		next if $name eq '.';
+		next if $name eq '..';
 		my $uid = (stat "$_ARCHIVE_DIR_RESTORE/$name")[4];
 		my $gid = (stat "$_ARCHIVE_DIR_RESTORE/$name")[5];
 		chown $uid, $gid, $_ARCHIVE_DIR_RESTORE;
@@ -1145,7 +1221,17 @@ sub command_validate {
        $self->print_to_server_and_die("$self->{gnutar} returned error",
 				      $Amanda::Application::ERROR);
    }
-   exit($self->{error_status});
+}
+
+sub command_index {
+    my $self = shift;
+    my $buffer;
+
+    print "/PostgreSQL-Database-$self->{'args'}->{'level'}\n";
+
+    do {
+	sysread STDIN, $buffer, 1048576;
+    } while (defined $buffer and length($buffer) > 0);
 }
 
 package main;
@@ -1175,9 +1261,10 @@ GetOptions(
     'collection=s',
     'record',
     'calcsize',
+    'exclude-file=s@',
     'exclude-list=s@',
     'include-list=s@',
-    'directory=s',
+    'directory|target=s',
     # ampgsql-specific
     'statedir=s',
     'tmpdir=s',
@@ -1192,7 +1279,10 @@ GetOptions(
     'port=s',
     'user=s',
     'psql-path=s',
-    'verbose=s'
+    'verbose=s',
+    'incremental=s',
+    'remove-full-wal=s',
+    'remove-incremental-wal=s'
 ) or usage();
 
 if (defined $opt_version) {
@@ -1205,3 +1295,4 @@ my $application = Amanda::Application::ampgsql->new($opts);
 Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
 
 $application->do($ARGV[0]);
+# NOTREACHED
