@@ -1,5 +1,6 @@
 #!@PERL@ 
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -42,7 +43,7 @@ use Amanda::MainLoop qw( :GIOCondition );
 
 sub new {
     my $class = shift;
-    my ($config, $host, $disk, $device, $level, $index, $message, $collection, $record, $calcsize, $gnutar_path, $smbclient_path, $amandapass, $exclude_file, $exclude_list, $exclude_optional, $include_file, $include_list, $include_optional, $recover_mode, $allow_anonymous, $directory, $regex_match) = @_;
+    my ($config, $host, $disk, $device, $level, $index, $message, $collection, $record, $calcsize, $gnutar_path, $smbclient_path, $amandapass, $exclude_file, $exclude_list, $exclude_optional, $include_file, $include_list, $include_optional, $recover_mode, $allow_anonymous, $target, $regex_match) = @_;
     my $self = $class->SUPER::new($config);
 
     if (defined $gnutar_path) {
@@ -87,7 +88,7 @@ sub new {
     $self->{include_optional} = $include_optional;
     $self->{recover_mode}     = $recover_mode;
     $self->{allow_anonymous}  = $allow_anonymous;
-    $self->{directory}        = $directory;
+    $self->{target}           = $target;
     if ($regex_match =~ /^yes$/i) {
 	$self->{'regex_match'} = 1;
     } else {
@@ -212,7 +213,7 @@ sub validate_inexclude {
 }
 
 # on entry:
-#   $self->{directory} == //host/share/subdir           \\host\share\subdir
+#   $self->{target} == //host/share/subdir           \\host\share\subdir
 #   or
 #   $self->{device}    == //host/share/subdir		\\host\share\subdir
 # on exit:
@@ -222,7 +223,7 @@ sub validate_inexclude {
 #   $self->{subdir}     = subdir			subdir
 sub parsesharename {
     my $self = shift;
-    my $to_parse = $self->{directory};
+    my $to_parse = $self->{target};
     $to_parse = $self->{device} if !defined $to_parse;;
 
     return if !defined $to_parse;
@@ -309,6 +310,7 @@ sub findpass {
 		if ($password =~ /^6G\!dr(.*)/) {
 		    my $base64 = $1;
 		    $password = MIME::Base64::decode($base64);
+		    chomp($password);
 		}
 	        $self->{password} = $password;
 		$self->{password} = undef if (defined $password && $password eq "");
@@ -402,7 +404,7 @@ sub command_selfcheck {
 
     print "OK " . $self->{share} . "\n";
     print "OK " . $self->{device} . "\n";
-    print "OK " . $self->{directory} . "\n" if defined $self->{directory};
+    print "OK " . $self->{target} . "\n" if defined $self->{target};
 
     my ($password_rdr, $password_wtr);
     if (defined $self->{password}) {
@@ -439,13 +441,15 @@ sub command_selfcheck {
 	push @ARGV, "-c", "quit";
 	debug("execute: " . $self->{smbclient} . " " .
 	      join(" ", @ARGV));
+	$ENV{'LC_CTYPE'} = 'en_US.UTF-8';
 	exec {$self->{smbclient}} @ARGV;
     }
     #parent
     if (defined $self->{password}) {
         my $ff = $password_wtr->fileno;
         debug("password_wtr $ff");
-        $password_wtr->print($self->{password});
+        $password_wtr->print("$self->{password}\n");
+        $password_wtr->print("$self->{password}\n");
         $password_wtr->close();
         $password_rdr->close();
     } else {
@@ -457,6 +461,7 @@ sub command_selfcheck {
 	chomp;
 	debug("stderr: " . $_);
 	next if /^Domain=/;
+	next if /^WARNING/g;
 	# message if samba server is configured with 'security = share'
 	next if /Server not using user level security and no password supplied./;
 	$self->print_to_server("smbclient: $_",
@@ -516,14 +521,15 @@ sub command_estimate {
 	}
 	debug("execute: " . $self->{smbclient} . " " .
 	      join(" ", @ARGV));
+	$ENV{'LC_CTYPE'} = 'en_US.UTF-8';
 	exec {$self->{smbclient}} @ARGV;
     }
     #parent
     if (defined $self->{password}) {
         my $ff = $password_wtr->fileno;
         debug("password_wtr $ff");
-        debug("password $self->{password}");
-        $password_wtr->print($self->{password});
+        $password_wtr->print("$self->{password}\n");
+        $password_wtr->print("$self->{password}\n");
         $password_wtr->close();
         $password_rdr->close();
     }
@@ -547,6 +553,7 @@ sub parse_estimate {
 	next if /^\s*$/;
 	next if /^Domain=/;
 	next if /dumped \d+ files and directories/;
+	next if /^WARNING/g;
 	# message if samba server is configured with 'security = share'
 	next if /Server not using user level security and no password supplied./;
 	debug("stderr: $_");
@@ -658,13 +665,15 @@ sub command_backup {
 	push @ARGV, "-c", $comm;
 	debug("execute: " . $self->{smbclient} . " " .
 	      join(" ", @ARGV));
+	$ENV{'LC_CTYPE'} = 'en_US.UTF-8';
 	exec {$self->{smbclient}} @ARGV;
     }
 
     if (defined $self->{password}) {
         my $ff = $password_wtr->fileno;
         debug("password_wtr $ff");
-        $password_wtr->print($self->{password});
+        $password_wtr->print("$self->{password}\n");
+        $password_wtr->print("$self->{password}\n");
         $password_wtr->close();
         $password_rdr->close();
     } else {
@@ -672,11 +681,12 @@ sub command_backup {
     }
     close($smbclient_wtr);
 
-    #index process 
-    my $index_rdr;
+    #index process
     my $index_wtr;
+    my $index_rdr;
+    my $index_err = Symbol::gensym;;
     debug("$self->{gnutar} -tf -");
-    my $pid_index1 = open2($index_rdr, $index_wtr, $self->{gnutar}, "-tf", "-");
+    my $pid_index1 = open3($index_wtr, $index_rdr, $index_err, $self->{gnutar}, "-tf", "-");
     my $size = -1;
     my $index_fd = $index_rdr->fileno;
     debug("index $index_fd");
@@ -694,6 +704,8 @@ sub command_backup {
 				$G_IO_IN|$G_IO_HUP|$G_IO_ERR);
     my $index_tar_stdout_src = Amanda::MainLoop::fd_source($index_rdr,
 				$G_IO_IN|$G_IO_HUP|$G_IO_ERR);
+    my $index_tar_stderr_src = Amanda::MainLoop::fd_source($index_err,
+				$G_IO_IN|$G_IO_HUP|$G_IO_ERR);
 
     my $smbclient_stdout_done = 0;
     my $smbclient_stderr_done = 0;
@@ -742,16 +754,20 @@ sub command_backup {
 	}
 	chomp $line;
 	debug("stderr: " . $line);
-	return if $line =~ /^Domain=/;
-	return if $line =~ /^tarmode is now /;
-	return if $line =~ /^tar_re_search set/;
+	return if $line =~ /Domain=/;
+	return if $line =~ /tarmode is now /;
+	return if $line =~ /tar_re_search set/;
+	return if $line =~ /WARNING/g;
 	if ($line =~ /dumped (\d+) files and directories/) {
 	    $nb_files = $1;
 	    return;
 	}
 	# message if samba server is configured with 'security = share'
 	return if $line =~  /Server not using user level security and no password supplied./;
-	if ($line =~ /^Total bytes written: (\d*)/) {
+	if ($line =~ /Total bytes written: (\d*)/) {
+	    $size = $1;
+	    return;
+	} elsif ($line =~ /Total bytes received: (\d*)/) {
 	    $size = $1;
 	    return;
 	}
@@ -777,6 +793,27 @@ sub command_backup {
 	    }
 	} else {
 	    chomp $line;
+	    if ($line =~ /Ignoring unknown extended header keyword/) {
+		debug("tar stderr: $line");
+	    } else {
+		$self->print_to_server($line, $Amanda::Script_App::ERROR);
+	    }
+	}
+    });
+
+    $index_tar_stderr_src->set_callback(sub {
+	my $line = <$index_err>;
+	if (!defined $line) {
+	    $file_to_close--;
+	    $index_tar_stderr_src->remove();
+	    close($index_err);
+	    Amanda::MainLoop::quit() if $file_to_close == 0;
+	    return;
+	}
+	chomp $line;
+	if ($line =~ /Ignoring unknown extended header keyword/) {
+	    debug("tar stderr: $line");
+	} else {
 	    $self->print_to_server($line, $Amanda::Script_App::ERROR);
 	}
     });
@@ -789,7 +826,6 @@ sub command_backup {
 	    $ksize = 32;
 	}
 	print {$self->{mesgout}} "sendbackup: size $ksize\n";
-	print {$self->{mesgout}} "sendbackup: end\n";
     }
 
     waitpid $pid, 0;
@@ -797,7 +833,6 @@ sub command_backup {
 	$self->print_to_server_and_die("smbclient returned error",
 				       $Amanda::Script_App::ERROR);
     }
-    exit 0;
 }
 
 sub parse_backup {
@@ -821,7 +856,6 @@ sub parse_backup {
 
 sub command_index_from_output {
    index_from_output(0, 1);
-   exit 0;
 }
 
 sub index_from_output {
@@ -835,13 +869,13 @@ sub index_from_output {
    }
 }
 
-sub command_index_from_image {
-   my $self = shift;
-   my $index_fd;
-   open($index_fd, "$self->{gnutar} --list --file - |") ||
-      $self->print_to_server_and_die("Can't run $self->{gnutar}: $!",
-				     $Amanda::Script_App::ERROR);
-   index_from_output($index_fd, 1);
+sub command_index {
+    my $self = shift;
+    my $index_fd;
+    open2($index_fd, ">&0", $self->{gnutar}, "--list", "--file", "-") ||
+	$self->print_to_server_and_die("Can't run $self->{gnutar}: $!",
+				       $Amanda::Script_App::ERROR);
+    index_from_output($index_fd, \*STDOUT);
 }
 
 sub command_restore {
@@ -901,15 +935,15 @@ sub command_restore {
 	die("Can't exec '", $cmd[0], "'");
     } else {
 	push @cmd, $self->{gnutar}, "-xpvf", "-";
-	if (defined $self->{directory}) {
-	    if (!-d $self->{directory}) {
+	if (defined $self->{target}) {
+	    if (!-d $self->{target}) {
 		$self->print_to_server_and_die(
-				       "Directory $self->{directory}: $!",
+				       "Directory $self->{target}: $!",
 				       $Amanda::Script_App::ERROR);
 	    }
-	    if (!-w $self->{directory}) {
+	    if (!-w $self->{target}) {
 		$self->print_to_server_and_die(
-				       "Directory $self->{directory}: $!",
+				       "Directory $self->{target}: $!",
 				       $Amanda::Script_App::ERROR);
 	    }
 	    push @cmd, "--directory", $self->{directory};
@@ -948,7 +982,6 @@ sub command_validate {
 	$self->print_to_server_and_die("$self->{gnutar} returned error",
 				       $Amanda::Script_App::ERROR);
    }
-   exit(0);
 }
 
 sub command_print_command {
@@ -985,7 +1018,7 @@ my @opt_include_list;
 my $opt_include_optional;
 my $opt_recover_mode;
 my $opt_allow_anonymous;
-my $opt_directory;
+my $opt_target;
 my $opt_regex_match;
 
 my @orig_argv = @ARGV;
@@ -1014,7 +1047,7 @@ GetOptions(
     'include-optional=s' => \$opt_include_optional,
     'recover-mode=s'     => \$opt_recover_mode,
     'allow-anonymous=s'  => \$opt_allow_anonymous,
-    'directory=s'        => \$opt_directory,
+    'target|directory=s' => \$opt_target,
     'regex-match=s'      => \$opt_regex_match,
 ) or usage();
 
@@ -1023,8 +1056,9 @@ if (defined $opt_version) {
     exit(0);
 }
 
-my $application = Amanda::Application::Amsamba->new($opt_config, $opt_host, $opt_disk, $opt_device, \@opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, $opt_calcsize, $opt_gnutar_path, $opt_smbclient_path, $opt_amandapass, \@opt_exclude_file, \@opt_exclude_list, $opt_exclude_optional, \@opt_include_file, \@opt_include_list, $opt_include_optional, $opt_recover_mode, $opt_allow_anonymous, $opt_directory, $opt_regex_match);
+my $application = Amanda::Application::Amsamba->new($opt_config, $opt_host, $opt_disk, $opt_device, \@opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, $opt_calcsize, $opt_gnutar_path, $opt_smbclient_path, $opt_amandapass, \@opt_exclude_file, \@opt_exclude_list, $opt_exclude_optional, \@opt_include_file, \@opt_include_list, $opt_include_optional, $opt_recover_mode, $opt_allow_anonymous, $opt_target, $opt_regex_match);
 
 Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
 
 $application->do($ARGV[0]);
+# NOTREACHED
