@@ -1,5 +1,6 @@
 #!@PERL@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -40,7 +41,7 @@ use Amanda::Util qw( :constants quote_string );
 
 sub new {
     my $class = shift;
-    my ($config, $host, $disk, $device, $level, $index, $message, $collection, $record, $exclude_list, $exclude_optional,  $include_list, $include_optional, $bsize, $ext_attrib, $ext_header, $ignore, $normal, $strange, $error_exp, $directory, $suntar_path) = @_;
+    my ($config, $host, $disk, $device, $level, $index, $message, $collection, $record, $exclude_list, $exclude_optional,  $include_list, $include_optional, $bsize, $ext_attrib, $ext_header, $ignore, $normal, $strange, $error_exp, $target, $suntar_path) = @_;
     my $self = $class->SUPER::new($config);
 
     $self->{suntar}            = $Amanda::Constants::SUNTAR;
@@ -74,8 +75,8 @@ sub new {
     $self->{include_optional}  = $include_optional;
     $self->{block_size}        = $bsize;
     $self->{extended_header}   = $ext_header;
-    $self->{extended_attrib}   = $ext_attrib; 
-    $self->{directory}         = $directory;
+    $self->{extended_attrib}   = $ext_attrib;
+    $self->{target}            = $target;
 
     $self->{regex} = ();
     my $regex;
@@ -164,7 +165,7 @@ sub command_selfcheck {
       return;
    }
    print "OK " . $self->{device} . "\n";
-   print "OK " . $self->{directory} . "\n" if defined $self->{directory};
+   print "OK " . $self->{target} . "\n" if defined $self->{target};
    $self->validate_inexclude();
 }
 
@@ -213,7 +214,6 @@ sub command_estimate() {
         }
     }
     output_size($level, $msgsize);
-    exit 0;
 }
 
 
@@ -286,10 +286,7 @@ sub command_backup {
 
    my($ksize) = int ($size/1024);
    print {$self->{mesgout}} "sendbackup: size $ksize\n";
-   print {$self->{mesgout}} "sendbackup: end\n";
    debug("sendbackup: size $ksize "); 
-
-   exit 0;
 }
 
 sub parse_backup {
@@ -406,7 +403,6 @@ sub validate_inexclude {
 
 sub command_index_from_output {
    index_from_output(0, 1);
-   exit 0;
 }
 
 sub index_from_output {
@@ -420,29 +416,38 @@ sub index_from_output {
    }
 }
 
-sub command_index_from_image {
-   my $self = shift;
-   my $index_fd;
-   open($index_fd, "$self->{suntar} -tf - |") ||
-      $self->print_to_server_and_die("Can't run $self->{suntar}: $!",
-				     $Amanda::Script_App::ERROR);
-   index_from_output($index_fd, 1);
+sub command_index {
+    my $self = shift;
+    my $program;
+    my $index_fd;
+
+    if (-e $self->{suntar}) {
+	$program = $self->{suntar};
+    } elsif (-e $self->{gnutar}) {
+	$program = $self->{gnutar};
+    }
+
+    open2($index_fd, ">&0", $program, "tf", "-") ||
+	$self->print_to_server_and_die("Can't run $program: $!",
+				       $Amanda::Script_App::ERROR);
+    index_from_output($index_fd, \*STDOUT);
+
 }
 
 sub command_restore {
    my $self = shift;
 
    chdir(Amanda::Util::get_original_cwd());
-   if (defined $self->{directory}) {
-      if (!-d $self->{directory}) {
-         $self->print_to_server_and_die("Directory $self->{directory}: $!",
+   if (defined $self->{target}) {
+      if (!-d $self->{target}) {
+         $self->print_to_server_and_die("Directory $self->{target}: $!",
 				        $Amanda::Script_App::ERROR);
       }
-      if (!-w $self->{directory}) {
-         $self->print_to_server_and_die("Directory $self->{directory}: $!",
+      if (!-w $self->{target}) {
+         $self->print_to_server_and_die("Directory $self->{target}: $!",
 				        $Amanda::Script_App::ERROR);
       }
-      chdir($self->{directory});
+      chdir($self->{target});
    }
 
    my $cmd = "-xpv";
@@ -503,7 +508,6 @@ sub command_validate {
 	$self->print_to_server_and_die("$program returned error",
 				       $Amanda::Script_App::ERROR);
    }
-   exit(0);
 }
 
 sub build_command {
@@ -538,8 +542,8 @@ sub build_command {
       $cmd .= "f";
       push @optparams,"-";
    }
-   if ($self->{directory}) {
-      push @optparams, "-C", $self->{directory};
+   if ($self->{target}) {
+      push @optparams, "-C", $self->{target};
    } else {
       push @optparams, "-C", $self->{device};
    }
@@ -563,6 +567,7 @@ EOF
     exit(1);
 }
 
+my $opt_version;
 my $opt_config;
 my $opt_host;
 my $opt_disk;
@@ -584,44 +589,51 @@ my @opt_normal;
 my @opt_strange;
 my @opt_error;
 my $opt_lang;
-my $opt_directory;
+my $opt_target;
 my $opt_suntar_path;
 
 my @orig_argv = @ARGV;
 
 Getopt::Long::Configure(qw{bundling});
 GetOptions(
-    'config=s'     	  => \$opt_config,
-    'host=s'       	  => \$opt_host,
-    'disk=s'       	  => \$opt_disk,
-    'device=s'     	  => \$opt_device,
-    'level=s'      	  => \$opt_level,
-    'index=s'      	  => \$opt_index,
-    'message=s'    	  => \$opt_message,
-    'collection=s' 	  => \$opt_collection,
-    'exclude-list=s'      => \@opt_exclude_list,
-    'exclude-optional=s'  => \$opt_exclude_optional,
-    'include-list=s'      => \@opt_include_list,
-    'include-optional=s'  => \$opt_include_optional,
-    'record'       	  => \$opt_record,
-    'block-size=s'        => \$opt_bsize,
-    'extended-attributes=s'  => \$opt_ext_attrib,
-    'extended-headers=s'     => \$opt_ext_head,
-    'ignore=s'               => \@opt_ignore,
-    'normal=s'               => \@opt_normal,
-    'strange=s'              => \@opt_strange,
-    'error=s'                => \@opt_error,
-    'lang=s'                 => \$opt_lang,
-    'directory=s'            => \$opt_directory,
-    'suntar-path=s'          => \$opt_suntar_path,
+    'version'		    => \$opt_version,
+    'config=s'		    => \$opt_config,
+    'host=s'		    => \$opt_host,
+    'disk=s'		    => \$opt_disk,
+    'device=s'		    => \$opt_device,
+    'level=s'		    => \$opt_level,
+    'index=s'		    => \$opt_index,
+    'message=s'		    => \$opt_message,
+    'collection=s'	    => \$opt_collection,
+    'exclude-list=s'        => \@opt_exclude_list,
+    'exclude-optional=s'    => \$opt_exclude_optional,
+    'include-list=s'        => \@opt_include_list,
+    'include-optional=s'    => \$opt_include_optional,
+    'record'		    => \$opt_record,
+    'block-size=s'          => \$opt_bsize,
+    'extended-attributes=s' => \$opt_ext_attrib,
+    'extended-headers=s'    => \$opt_ext_head,
+    'ignore=s'              => \@opt_ignore,
+    'normal=s'              => \@opt_normal,
+    'strange=s'             => \@opt_strange,
+    'error=s'               => \@opt_error,
+    'lang=s'                => \$opt_lang,
+    'target|directory=s'    => \$opt_target,
+    'suntar-path=s'         => \$opt_suntar_path,
 ) or usage();
 
+if (defined $opt_version) {
+    print "amsuntar-" . $Amanda::Constants::VERSION , "\n";
+    exit(0);
+}
+
 if (defined $opt_lang) {
     $ENV{LANG} = $opt_lang;
 }
 
-my $application = Amanda::Application::Amsuntar->new($opt_config, $opt_host, $opt_disk, $opt_device, $opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, \@opt_exclude_list, $opt_exclude_optional, \@opt_include_list, $opt_include_optional,$opt_bsize,$opt_ext_attrib,$opt_ext_head, \@opt_ignore, \@opt_normal, \@opt_strange, \@opt_error, $opt_directory, $opt_suntar_path);
+my $application = Amanda::Application::Amsuntar->new($opt_config, $opt_host, $opt_disk, $opt_device, $opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, \@opt_exclude_list, $opt_exclude_optional, \@opt_include_list, $opt_include_optional,$opt_bsize,$opt_ext_attrib,$opt_ext_head, \@opt_ignore, \@opt_normal, \@opt_strange, \@opt_error, $opt_target, $opt_suntar_path);
 
 Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
 
 $application->do($ARGV[0]);
+# NOTREACHED
