@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -146,8 +147,54 @@ start_backup(
     char *config;
     am_level_t *alevel = (am_level_t *)dle->levellist->data;
     int      level  = alevel->level;
+    int        native_pipe[2];
+    int        client_pipe[2];
+    int        data_out = dataf;
 
-    g_snprintf(level_str, SIZEOF(level_str), "%d", level);
+    have_filter = FALSE;
+    crc32_init(&native_crc.crc);
+    crc32_init(&client_crc.crc);
+
+    /* create pipes to compute the native CRC */
+    if (pipe(native_pipe) < 0) {
+	char  *errmsg;
+	char  *qerrmsg;
+	errmsg = g_strdup_printf(_("Program '%s': can't create pipe"),
+				 dle->program);
+	qerrmsg = quote_string(errmsg);
+	fdprintf(mesgf, _("sendbackup: error [%s]\n"), errmsg);
+	dbprintf(_("ERROR %s\n"), qerrmsg);
+	amfree(qerrmsg);
+	amfree(errmsg);
+	return;
+    }
+
+    if (dle->encrypt == ENCRYPT_CUST ||
+        dle->compress == COMP_FAST ||
+        dle->compress == COMP_BEST ||
+        dle->compress == COMP_CUST) {
+
+        have_filter = TRUE;
+
+        /* create pipes to compute the client CRC */
+        if (pipe(client_pipe) < 0) {
+            char  *errmsg;
+            char  *qerrmsg;
+            errmsg = g_strdup_printf(_("Application '%s': can't create pipe"),
+                                     dle->program);
+            qerrmsg = quote_string(errmsg);
+            fdprintf(mesgf, _("sendbackup: error [%s]\n"), errmsg);
+            dbprintf(_("ERROR %s\n"), qerrmsg);
+            amfree(qerrmsg);
+            amfree(errmsg);
+            return;
+        }
+        data_out = client_pipe[1];
+    } else {
+        data_out = dataf;
+    }
+
+    g_snprintf(level_str, sizeof(level_str), "%d", level);
 
     qdisk = quote_string(dle->disk);
     dbprintf(_("start: %s:%s lev %d\n"), host, qdisk, level);
@@ -159,16 +206,15 @@ start_backup(
     /*  apply client-side encryption here */
     if (dle->encrypt == ENCRYPT_CUST ) {
         encpid = pipespawn(dle->clnt_encrypt, STDIN_PIPE, 0,
-	                   &compout, &dataf, &mesgf,
+	                   &compout, &data_out, &mesgf,
 	                   dle->clnt_encrypt, encryptopt, NULL);
         dbprintf(_("gnutar: pid %ld: %s\n"), (long)encpid, dle->clnt_encrypt);
+	aclose(data_out);
     } else {
-        compout = dataf;
+        compout = data_out;
         encpid = -1;
     }
     /*  now do the client-side compression */
-
-
     if(dle->compress == COMP_FAST || dle->compress == COMP_BEST) {
 	compopt = skip_argument;
 
@@ -187,6 +233,7 @@ start_backup(
 	    dbprintf(" %s", compopt);
 	}
 	dbprintf("\n");
+	aclose(compout);
      } else if (dle->compress == COMP_CUST) {
         compopt = skip_argument;
 	comppid = pipespawn(dle->compprog, STDIN_PIPE, 0,
@@ -198,8 +245,10 @@ start_backup(
 	    dbprintf(" %s", compopt);
 	}
 	dbprintf("\n");
+	aclose(compout);
     } else {
 	dumpout = compout;
+	compout = -1;
 	comppid = -1;
     }
 
@@ -210,14 +259,14 @@ start_backup(
     dbprintf(_("dumping device '%s' with '%s'\n"), device, fstype);
 
 #if defined(USE_RUNDUMP) || !defined(DUMP)
-    cmd = vstralloc(amlibexecdir, "/", "rundump", NULL);
+    cmd = g_strjoin(NULL, amlibexecdir, "/", "rundump", NULL);
     cmdX = cmd;
     if (g_options->config)
 	config = g_options->config;
     else
 	config = "NOCONFIG";
 #else
-    cmd = stralloc(DUMP);
+    cmd = g_strdup(DUMP);
     cmdX = skip_argument;
     config = skip_argument;
 #endif
@@ -226,13 +275,14 @@ start_backup(
     /* normal dump */
 #ifdef XFSDUMP						/* { */
 #ifdef DUMP						/* { */
-    if (strcmp(amname_to_fstype(dle->device), "xfs") == 0)
+    if (g_str_equal(amname_to_fstype(dle->device), "xfs"))
 #else							/* } { */
     if (1)
 #endif							/* } */
     {
-        char *progname = cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump",
-					    NULL);
+        char *progname;
+        g_free(cmd);
+        progname = cmd = g_strconcat(amlibexecdir, "/", "rundump", NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
@@ -242,21 +292,16 @@ start_backup(
 	program->backup_name  = XFSDUMP;
 	program->restore_name = XFSRESTORE;
 
-	indexcmd = vstralloc(XFSRESTORE,
-			     " -t",
-			     " -v", " silent",
-			     " -",
-			     " 2>/dev/null",
-			     " | sed",
-			     " -e", " \'s/^/\\//\'",
-			     NULL);
+	indexcmd = g_strjoin(NULL, XFSRESTORE, " -t", " -v", " silent", " -",
+	    " 2>/dev/null", " | sed", " -e", " \'s/^/\\//\'", NULL);
+
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
-	dumpkeys = stralloc(level_str);
+	dumpkeys = g_strdup(level_str);
 	dumppid = pipespawn(progname, STDIN_PIPE, 0,
-			    &dumpin, &dumpout, &mesgf,
+			    &dumpin, &native_pipe[1], &mesgf,
 			    cmdX, config,
 			    "xfsdump",
 			    !dle->record ? "-J" : skip_argument,
@@ -270,34 +315,37 @@ start_backup(
 #endif							/* } */
 #ifdef VXDUMP						/* { */
 #ifdef DUMP
-    if (strcmp(amname_to_fstype(dle->device), "vxfs") == 0)
+    if (g_str_equal(amname_to_fstype(dle->device), "vxfs"))
 #else
     if (1)
 #endif
     {
+char *progname;
 #ifdef USE_RUNDUMP
-        char *progname = cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump",
-					    NULL);
+        g_free(cmd);
+        progname = cmd = g_strconcat(amlibexecdir, "/", "rundump", NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
 	else
 	    config = "NOCONFIG";
 #else
-	char *progname = cmd = newvstralloc(cmd, VXDUMP, NULL);
+	char *progname;
+        g_free(cmd);
+        progname = cmd = g_strdup(VXDUMP);
 	cmdX = skip_argument;
 	config = skip_argument;
 #endif
 	program->backup_name  = VXDUMP;
 	program->restore_name = VXRESTORE;
 
-	dumpkeys = vstralloc(level_str,
+	dumpkeys = g_strjoin(NULL, level_str,
 			     !dle->record ? "" : "u",
 			     "s",
 			     "f",
 			     NULL);
 
-	indexcmd = vstralloc(VXRESTORE,
+	indexcmd = g_strjoin(NULL, VXRESTORE,
 			     " -tvf", " -",
 			     " 2>/dev/null",
 			     " | ",
@@ -305,10 +353,10 @@ start_backup(
 			     NULL);
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
 	dumppid = pipespawn(progname, STDIN_PIPE, 0,
-			    &dumpin, &dumpout, &mesgf, 
+			    &dumpin, &native_pipe[1], &mesgf,
 			    cmdX, config,
 			    "vxdump",
 			    dumpkeys,
@@ -322,28 +370,30 @@ start_backup(
 
 #ifdef VDUMP						/* { */
 #ifdef DUMP
-    if (strcmp(amname_to_fstype(dle->device), "advfs") == 0)
+    if (g_str_equal(amname_to_fstype(dle->device), "advfs"))
 #else
     if (1)
 #endif
     {
-        cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump", NULL);
+        g_free(cmd);
+        cmd = g_strconcat(amlibexecdir, "/", "rundump", NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
 	else
 	    config = "NOCONFIG";
-	device = newstralloc(device, amname_to_dirname(dle->device));
+	g_free(device);
+	device = g_strdup(amname_to_dirname(dle->device));
 	program->backup_name  = VDUMP;
 	program->restore_name = VRESTORE;
 
-	dumpkeys = vstralloc(level_str,
+	dumpkeys = g_strjoin(NULL, level_str,
 			     !dle->record ? "" : "u",
 			     "b",
 			     "f",
 			     NULL);
 
-	indexcmd = vstralloc(VRESTORE,
+	indexcmd = g_strjoin(NULL, VRESTORE,
 			     " -tvf", " -",
 			     " 2>/dev/null",
 			     " | ",
@@ -351,10 +401,10 @@ start_backup(
 			     NULL);
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
 	dumppid = pipespawn(cmd, STDIN_PIPE, 0,
-			    &dumpin, &dumpout, &mesgf, 
+			    &dumpin, &native_pipe[1], &mesgf,
 			    cmdX, config,
 			    "vdump",
 			    dumpkeys,
@@ -387,7 +437,7 @@ start_backup(
 # define FREEBSD_EXTRA_KEYS ""
 #endif
 
-	dumpkeys = vstralloc(level_str,
+	dumpkeys = g_strjoin(NULL, level_str,
 			     !dle->record ? "" : "u",
 			     FREEBSD_EXTRA_KEYS,
 			     "s",
@@ -395,7 +445,7 @@ start_backup(
 			     "f",
 			     NULL);
 
-	indexcmd = vstralloc(RESTORE,
+	indexcmd = g_strjoin(NULL, RESTORE,
 			     " -tvf", " -",
 			     " 2>&1",
 			     /* not to /dev/null because of DU's dump */
@@ -404,10 +454,10 @@ start_backup(
 			     NULL);
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
 	dumppid = pipespawn(cmd, STDIN_PIPE, 0,
-			    &dumpin, &dumpout, &mesgf, 
+			    &dumpin, &native_pipe[1], &mesgf,
 			    cmdX, config,
 			    "dump",
 			    dumpkeys,
@@ -424,13 +474,13 @@ start_backup(
     }
 #else							/* } { */
     /* AIX backup program */
-    dumpkeys = vstralloc("-",
+    dumpkeys = g_strjoin(NULL, "-",
 			 level_str,
 			 !dle->record ? "" : "u",
 			 "f",
 			 NULL);
 
-    indexcmd = vstralloc(RESTORE,
+    indexcmd = g_strjoin(NULL, RESTORE,
 			 " -B",
 			 " -tvf", " -",
 			 " 2>/dev/null",
@@ -442,7 +492,7 @@ start_backup(
     start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
 
     dumppid = pipespawn(cmd, STDIN_PIPE, 0,
-			&dumpin, &dumpout, &mesgf, 
+			&dumpin, &native_pipe[1], &mesgf,
 			cmdX, config,
 			"backup",
 			dumpkeys,
@@ -460,12 +510,45 @@ start_backup(
     /* close the write ends of the pipes */
 
     aclose(dumpin);
-    aclose(dumpout);
-    aclose(compout);
-    aclose(dataf);
+    aclose(native_pipe[1]);
     aclose(mesgf);
     if (dle->create_index)
 	aclose(indexf);
+
+    if (shm_control_name) {
+	shm_ring = shm_ring_link(shm_control_name);
+	shm_ring_producer_set_size(shm_ring, NETWORK_BLOCK_BYTES*16, NETWORK_BLOCK_BYTES*4);
+	native_crc.in  = native_pipe[0];
+	if (!have_filter) {
+	    native_crc.out = dumpout;
+	    native_crc.shm_ring = shm_ring;
+	    native_crc.thread = g_thread_create(handle_crc_to_shm_ring_thread,
+				(gpointer)&native_crc, TRUE, NULL);
+	} else {
+	    native_crc.out = dumpout;
+	    native_crc.thread = g_thread_create(handle_crc_thread,
+				(gpointer)&native_crc, TRUE, NULL);
+	    close(client_pipe[1]);
+	    client_crc.in  = client_pipe[0];
+	    client_crc.out = dumpout;
+	    client_crc.shm_ring = shm_ring;
+	    client_crc.thread = g_thread_create(handle_crc_to_shm_ring_thread,
+				(gpointer)&client_crc, TRUE, NULL);
+	}
+    } else {
+	native_crc.in  = native_pipe[0];
+	native_crc.out = dumpout;
+	native_crc.thread = g_thread_create(handle_crc_thread,
+				(gpointer)&native_crc, TRUE, NULL);
+
+	if (have_filter) {
+	    close(client_pipe[1]);
+	    client_crc.in  = client_pipe[0];
+	    client_crc.out = dataf;
+	    client_crc.thread = g_thread_create(handle_crc_thread,
+				(gpointer)&client_crc, TRUE, NULL);
+	}
+    }
 }
 
 static void
