@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -34,7 +35,7 @@
 #include "sendbackup.h"
 #include "amandates.h"
 #include "clock.h"
-#include "util.h"
+#include "amutil.h"
 #include "getfsent.h"			/* for amname_to_dirname lookup */
 #include "conffile.h"
 
@@ -126,6 +127,7 @@ time_t cur_dumptime;
 
 static char *gnutar_list_dir = NULL;
 static char *incrname = NULL;
+
 /*
  *  doing similar to $ gtar | compression | encryption 
  */
@@ -159,8 +161,54 @@ start_backup(
     char *amandates_file = NULL;
     am_level_t *alevel = (am_level_t *)dle->levellist->data;
     int      level  = alevel->level;
+    int        native_pipe[2];
+    int        client_pipe[2];
+    int        data_out;
 
-    error_pn = stralloc2(get_pname(), "-smbclient");
+    have_filter = FALSE;
+    crc32_init(&native_crc.crc);
+    crc32_init(&client_crc.crc);
+
+    /* create pipes to compute the native CRC */
+    if (pipe(native_pipe) < 0) {
+	char  *errmsg;
+	char  *qerrmsg;
+	errmsg = g_strdup_printf(_("Program '%s': can't create pipe"),
+				 dle->program);
+	qerrmsg = quote_string(errmsg);
+	fdprintf(mesgf, _("sendbackup: error [%s]\n"), errmsg);
+	dbprintf(_("ERROR %s\n"), qerrmsg);
+	amfree(qerrmsg);
+	amfree(errmsg);
+	return;
+    }
+
+    if (dle->encrypt == ENCRYPT_CUST ||
+        dle->compress == COMP_FAST ||
+        dle->compress == COMP_BEST ||
+        dle->compress == COMP_CUST) {
+
+        have_filter = TRUE;
+
+        /* create pipes to compute the client CRC */
+        if (pipe(client_pipe) < 0) {
+            char  *errmsg;
+            char  *qerrmsg;
+            errmsg = g_strdup_printf(_("Application '%s': can't create pipe"),
+                                     dle->program);
+            qerrmsg = quote_string(errmsg);
+            fdprintf(mesgf, _("sendbackup: error [%s]\n"), errmsg);
+            dbprintf(_("ERROR %s\n"), qerrmsg);
+            amfree(qerrmsg);
+            amfree(errmsg);
+            return;
+        }
+        data_out = client_pipe[1];
+    } else {
+        data_out = dataf;
+    }
+
+    error_pn = g_strconcat(get_pname(), "-smbclient", NULL);
 
     qdisk = quote_string(dle->disk);
     dbprintf(_("start: %s:%s lev %d\n"), host, qdisk, level);
@@ -170,17 +218,18 @@ start_backup(
 
      /*  apply client-side encryption here */
      if ( dle->encrypt == ENCRYPT_CUST ) {
-         encpid = pipespawn(dle->clnt_encrypt, STDIN_PIPE, 0, 
-			&compout, &dataf, &mesgf, 
+        encpid = pipespawn(dle->clnt_encrypt, STDIN_PIPE, 0,
+			&compout, &data_out, &mesgf,
 			dle->clnt_encrypt, encryptopt, NULL);
-         dbprintf(_("gnutar: pid %ld: %s\n"), (long)encpid, dle->clnt_encrypt);
+        dbprintf(_("gnutar: pid %ld: %s\n"), (long)encpid, dle->clnt_encrypt);
+	aclose(data_out);
     } else {
-       compout = dataf;
-       encpid = -1;
-    } 
-     /*  now do the client-side compression */
-    if(dle->compress == COMP_FAST || dle->compress == COMP_BEST) {
-          compopt = skip_argument;
+        compout = data_out;
+        encpid = -1;
+    }
+    /*  now do the client-side compression */
+    if (dle->compress == COMP_FAST || dle->compress == COMP_BEST) {
+        compopt = skip_argument;
 #if defined(COMPRESS_BEST_OPT) && defined(COMPRESS_FAST_OPT)
 	if(dle->compress == COMP_BEST) {
 	    compopt = COMPRESS_BEST_OPT;
@@ -198,6 +247,7 @@ start_backup(
 	} else {
 	    dbprintf(_("pid %ld: %s\n"), (long)comppid, COMPRESS_PATH);
 	}
+	aclose(compout);
      } else if (dle->compress == COMP_CUST) {
         compopt = skip_argument;
 	comppid = pipespawn(dle->compprog, STDIN_PIPE, 0,
@@ -209,8 +259,10 @@ start_backup(
 	} else {
 	    dbprintf(_("pid %ld: %s\n"), (long)comppid, dle->compprog);
 	}
+	aclose(compout);
     } else {
 	dumpout = compout;
+	compout = -1;
 	comppid = -1;
     }
 
@@ -230,15 +282,15 @@ start_backup(
 	int baselevel;
 	char *sdisk = sanitise_filename(dle->disk);
 
-	basename = vstralloc(gnutar_list_dir,
+	basename = g_strjoin(NULL, gnutar_list_dir,
 			     "/",
 			     host,
 			     sdisk,
 			     NULL);
 	amfree(sdisk);
 
-	g_snprintf(number, SIZEOF(number), "%d", level);
-	incrname = vstralloc(basename, "_", number, ".new", NULL);
+	g_snprintf(number, sizeof(number), "%d", level);
+	incrname = g_strjoin(NULL, basename, "_", number, ".new", NULL);
 	unlink(incrname);
 
 	/*
@@ -250,11 +302,12 @@ start_backup(
 	infd = -1;
 	while (infd == -1) {
 	    if (--baselevel >= 0) {
-		g_snprintf(number, SIZEOF(number), "%d", baselevel);
-		inputname = newvstralloc(inputname,
-					 basename, "_", number, NULL);
+		g_snprintf(number, sizeof(number), "%d", baselevel);
+		g_free(inputname);
+		inputname = g_strconcat(basename, "_", number, NULL);
 	    } else {
-		inputname = newstralloc(inputname, "/dev/null");
+		g_free(inputname);
+		inputname = g_strdup("/dev/null");
 	    }
 	    if ((infd = open(inputname, O_RDONLY)) == -1) {
 		int save_errno = errno;
@@ -279,7 +332,7 @@ start_backup(
 	    /*NOTREACHED*/
 	}
 
-	while ((nb = read(infd, &buf, SIZEOF(buf))) > 0) {
+	while ((nb = read(infd, &buf, sizeof(buf))) > 0) {
 	    if (full_write(outfd, &buf, (size_t)nb) < (size_t)nb) {
 		error(_("error [writing to '%s': %s]"), incrname,
 		       strerror(errno));
@@ -336,7 +389,7 @@ start_backup(
 	free_amandates();
 
 	gmtm = gmtime(&prev_dumptime);
-	g_snprintf(dumptimestr, SIZEOF(dumptimestr),
+	g_snprintf(dumptimestr, sizeof(dumptimestr),
 		    "%04d-%02d-%02d %2d:%02d:%02d GMT",
 		    gmtm->tm_year + 1900, gmtm->tm_mon+1, gmtm->tm_mday,
 		    gmtm->tm_hour, gmtm->tm_min, gmtm->tm_sec);
@@ -349,13 +402,13 @@ start_backup(
 
     cur_dumptime = time(0);
     cur_level = level;
-    cur_disk = stralloc(dle->disk);
+    cur_disk = g_strdup(dle->disk);
 #ifdef GNUTAR
 #  define PROGRAM_GNUTAR GNUTAR
 #else
 #  define PROGRAM_GNUTAR "tar"
 #endif
-    indexcmd = vstralloc(
+    indexcmd = g_strjoin(NULL, 
 			 PROGRAM_GNUTAR,
 			 " -tf", " -",
 			 " 2>/dev/null",
@@ -430,7 +483,7 @@ start_backup(
 	    /*NOTREACHED*/
 	}
 
-	taropt = stralloc("-T");
+	taropt = g_strdup("-T");
 	if(dle->exclude_file && dle->exclude_file->nb_element == 1) {
 	    strappend(taropt, "X");
 	}
@@ -451,10 +504,10 @@ start_backup(
 	}
 
 	program->backup_name = program->restore_name = SAMBA_CLIENT;
-	cmd = stralloc(program->backup_name);
+	cmd = g_strdup(program->backup_name);
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
 	if (pwtext_len > 0) {
 	    pw_fd_env = "PASSWD_FD";
@@ -462,7 +515,7 @@ start_backup(
 	    pw_fd_env = "dummy_PASSWD_FD";
 	}
 	dumppid = pipespawn(cmd, STDIN_PIPE|PASSWD_PIPE, 0,
-			    &dumpin, &dumpout, &mesgf,
+			    &dumpin, &native_pipe[1], &mesgf,
 			    pw_fd_env, &passwdf,
 			    "smbclient",
 			    sharename,
@@ -512,44 +565,46 @@ start_backup(
 	GPtrArray *argv_ptr = g_ptr_array_new();
 	char *file_exclude = NULL;
 	char *file_include = NULL;
+	messagelist_t mlist = NULL;
 
 	if (dle->exclude_file) nb_exclude+=dle->exclude_file->nb_element;
 	if (dle->exclude_list) nb_exclude+=dle->exclude_list->nb_element;
 	if (dle->include_file) nb_include+=dle->include_file->nb_element;
 	if (dle->include_list) nb_include+=dle->include_list->nb_element;
 
-	if (nb_exclude > 0) file_exclude = build_exclude(dle, 0);
-	if (nb_include > 0) file_include = build_include(dle, 0);
+	if (nb_exclude > 0) file_exclude = build_exclude(dle, &mlist);
+	if (nb_include > 0) file_include = build_include(dle, dirname, &mlist);
+	g_slist_free(mlist); // MUST also free the message
 
-	cmd = vstralloc(amlibexecdir, "/", "runtar", NULL);
+	cmd = g_strjoin(NULL, amlibexecdir, "/", "runtar", NULL);
 	info_tapeheader(dle);
 
-	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, native_pipe[1], mesgf, indexf, indexcmd);
 
-	g_ptr_array_add(argv_ptr, stralloc("runtar"));
+	g_ptr_array_add(argv_ptr, g_strdup("runtar"));
 	if (g_options->config)
-	    g_ptr_array_add(argv_ptr, stralloc(g_options->config));
+	    g_ptr_array_add(argv_ptr, g_strdup(g_options->config));
 	else
-	    g_ptr_array_add(argv_ptr, stralloc("NOCONFIG"));
+	    g_ptr_array_add(argv_ptr, g_strdup("NOCONFIG"));
 #ifdef GNUTAR
-	g_ptr_array_add(argv_ptr, stralloc(GNUTAR));
+	g_ptr_array_add(argv_ptr, g_strdup(GNUTAR));
 #else
-	g_ptr_array_add(argv_ptr, stralloc("tar"));
+	g_ptr_array_add(argv_ptr, g_strdup("tar"));
 #endif
-	g_ptr_array_add(argv_ptr, stralloc("--create"));
-	g_ptr_array_add(argv_ptr, stralloc("--file"));
-	g_ptr_array_add(argv_ptr, stralloc("-"));
-	g_ptr_array_add(argv_ptr, stralloc("--directory"));
+	g_ptr_array_add(argv_ptr, g_strdup("--create"));
+	g_ptr_array_add(argv_ptr, g_strdup("--file"));
+	g_ptr_array_add(argv_ptr, g_strdup("-"));
+	g_ptr_array_add(argv_ptr, g_strdup("--directory"));
 	canonicalize_pathname(dirname, tmppath);
-	g_ptr_array_add(argv_ptr, stralloc(tmppath));
-	g_ptr_array_add(argv_ptr, stralloc("--one-file-system"));
+	g_ptr_array_add(argv_ptr, g_strdup(tmppath));
+	g_ptr_array_add(argv_ptr, g_strdup("--one-file-system"));
 	if (gnutar_list_dir && incrname) {
-	    g_ptr_array_add(argv_ptr, stralloc("--listed-incremental"));
-	    g_ptr_array_add(argv_ptr, stralloc(incrname));
+	    g_ptr_array_add(argv_ptr, g_strdup("--listed-incremental"));
+	    g_ptr_array_add(argv_ptr, g_strdup(incrname));
 	} else {
-	    g_ptr_array_add(argv_ptr, stralloc("--incremental"));
-	    g_ptr_array_add(argv_ptr, stralloc("--newer"));
-	    g_ptr_array_add(argv_ptr, stralloc(dumptimestr));
+	    g_ptr_array_add(argv_ptr, g_strdup("--incremental"));
+	    g_ptr_array_add(argv_ptr, g_strdup("--newer"));
+	    g_ptr_array_add(argv_ptr, g_strdup(dumptimestr));
 	}
 #ifdef ENABLE_GNUTAR_ATIME_PRESERVE
 	/* --atime-preserve causes gnutar to call
@@ -557,27 +612,27 @@ start_backup(
 	 * adjust their atime.  However, utime()
 	 * updates the file's ctime, so incremental
 	 * dumps will think the file has changed. */
-	g_ptr_array_add(argv_ptr, stralloc("--atime-preserve"));
+	g_ptr_array_add(argv_ptr, g_strdup("--atime-preserve"));
 #endif
-	g_ptr_array_add(argv_ptr, stralloc("--sparse"));
-	g_ptr_array_add(argv_ptr, stralloc("--ignore-failed-read"));
-	g_ptr_array_add(argv_ptr, stralloc("--totals"));
+	g_ptr_array_add(argv_ptr, g_strdup("--sparse"));
+	g_ptr_array_add(argv_ptr, g_strdup("--ignore-failed-read"));
+	g_ptr_array_add(argv_ptr, g_strdup("--totals"));
 
 	if(file_exclude) {
-	    g_ptr_array_add(argv_ptr, stralloc("--exclude-from"));
-	    g_ptr_array_add(argv_ptr, stralloc(file_exclude));
+	    g_ptr_array_add(argv_ptr, g_strdup("--exclude-from"));
+	    g_ptr_array_add(argv_ptr, g_strdup(file_exclude));
 	}
 
 	if(file_include) {
-	    g_ptr_array_add(argv_ptr, stralloc("--files-from"));
-	    g_ptr_array_add(argv_ptr, stralloc(file_include));
+	    g_ptr_array_add(argv_ptr, g_strdup("--files-from"));
+	    g_ptr_array_add(argv_ptr, g_strdup(file_include));
 	}
 	else {
-	    g_ptr_array_add(argv_ptr, stralloc("."));
+	    g_ptr_array_add(argv_ptr, g_strdup("."));
 	}
 	    g_ptr_array_add(argv_ptr, NULL);
 	dumppid = pipespawnv(cmd, STDIN_PIPE, 0,
-			     &dumpin, &dumpout, &mesgf,
+			     &dumpin, &native_pipe[1], &mesgf,
 			     (char **)argv_ptr->pdata);
 	tarpid = dumppid;
 	amfree(file_exclude);
@@ -595,12 +650,45 @@ start_backup(
     /* close the write ends of the pipes */
 
     aclose(dumpin);
-    aclose(dumpout);
-    aclose(compout);
-    aclose(dataf);
+    aclose(native_pipe[1]);
     aclose(mesgf);
     if (dle->create_index)
 	aclose(indexf);
+
+    if (shm_control_name) {
+	shm_ring = shm_ring_link(shm_control_name);
+	shm_ring_producer_set_size(shm_ring, NETWORK_BLOCK_BYTES*16, NETWORK_BLOCK_BYTES*4);
+	native_crc.in  = native_pipe[0];
+	if (!have_filter) {
+	    native_crc.out = dumpout;
+	    native_crc.shm_ring = shm_ring;
+	    native_crc.thread = g_thread_create(handle_crc_to_shm_ring_thread,
+					(gpointer)&native_crc, TRUE, NULL);
+	} else {
+	    native_crc.out = dumpout;
+	    native_crc.thread = g_thread_create(handle_crc_thread,
+					(gpointer)&native_crc, TRUE, NULL);
+	    close(client_pipe[1]);
+	    client_crc.in  = client_pipe[0];
+	    client_crc.out = dataf;
+	    client_crc.shm_ring = shm_ring;
+	    client_crc.thread = g_thread_create(handle_crc_to_shm_ring_thread,
+					(gpointer)&client_crc, TRUE, NULL);
+	}
+    } else {
+	native_crc.in  = native_pipe[0];
+	native_crc.out = dumpout;
+	native_crc.thread = g_thread_create(handle_crc_thread,
+					(gpointer)&native_crc, TRUE, NULL);
+
+	if (have_filter) {
+	    close(client_pipe[1]);
+	    client_crc.in  = client_pipe[0];
+	    client_crc.out = dataf;
+	    client_crc.thread = g_thread_create(handle_crc_thread,
+					(gpointer)&client_crc, TRUE, NULL);
+	}
+    }
 }
 
 static void
@@ -613,8 +701,8 @@ end_backup(
     if(dle->record && !goterror) {
 	if (incrname != NULL && strlen(incrname) > 4) {
 	    char *nodotnew;
-	
-	    nodotnew = stralloc(incrname);
+
+	    nodotnew = g_strdup(incrname);
 	    nodotnew[strlen(nodotnew)-4] = '\0';
 	    if (rename(incrname, nodotnew)) {
 		g_fprintf(stderr, _("%s: warning [renaming %s to %s: %s]\n"), 
