@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -167,7 +168,55 @@ am_init_feature_set(void)
 	am_add_feature(f, fe_application_client_name);
 	am_add_feature(f, fe_script_client_name);
 	am_add_feature(f, fe_dumptype_property);
+	am_add_feature(f, fe_sendbackup_crc);
+	am_add_feature(f, fe_amrecover_crc_in_header);
+	am_add_feature(f, fe_amrecover_data_status);
+	am_add_feature(f, fe_amrecover_data_crc);
 	am_add_feature(f, fe_sendbackup_state);
+	am_add_feature(f, fe_amrecover_storage_in_marshall);
+	am_add_feature(f, fe_amidxtaped_storage_in_marshall);
+	am_add_feature(f, fe_amindexd_STORAGE);
+	am_add_feature(f, fe_sendbackup_retry);
+	am_add_feature(f, fe_sendbackup_retry_delay);
+	am_add_feature(f, fe_sendbackup_retry_level);
+	am_add_feature(f, fe_sendbackup_retry_message);
+	am_add_feature(f, fe_selfcheck_message);
+	//am_add_feature(f, fe_sendsize_message);
+	//am_add_feature(f, fe_sendbackup_message);
+	am_add_feature(f, fe_sendbackup_rep_retry);
+	am_add_feature(f, fe_sendbackup_stream_state);
+	am_add_feature(f, fe_amrecover_stream_state);
+	am_add_feature(f, fe_amidxtaped_dar);
+	am_add_feature(f, fe_restore_mesg_json);
+	am_add_feature(f, fe_restore_state_stream);
+	am_add_feature(f, fe_amrecover_header_send_size);
+	am_add_feature(f, fe_amrecover_header_ready);
+	am_add_feature(f, fe_amrecover_header_done);
+	am_add_feature(f, fe_amrecover_state_send);
+	am_add_feature(f, fe_amrecover_state_ready);
+	am_add_feature(f, fe_amrecover_state_done);
+	am_add_feature(f, fe_amrecover_data_send);
+	am_add_feature(f, fe_amrecover_data_ready);
+	//am_add_feature(f, fe_amrecover_data_done);
+	am_add_feature(f, fe_restore_header_send_size);
+	am_add_feature(f, fe_restore_header_ready);
+	am_add_feature(f, fe_restore_header_done);
+	am_add_feature(f, fe_restore_state_send);
+	am_add_feature(f, fe_restore_state_ready);
+	am_add_feature(f, fe_restore_state_done);
+	am_add_feature(f, fe_restore_data_send);
+	am_add_feature(f, fe_restore_data_ready);
+	//am_add_feature(f, fe_amrecover_data_done);
+	am_add_feature(f, fe_restore_dar);
+	am_add_feature(f, fe_restore_datapath);
+	am_add_feature(f, fe_restore_include);
+	am_add_feature(f, fe_restore_include_glob);
+	am_add_feature(f, fe_restore_exclude);
+	am_add_feature(f, fe_restore_exclude_glob);
+	am_add_feature(f, fe_restore_prev_next_level);
+
+	am_add_feature(f, fe_sendbackup_req_options_data_shm_control_name);
+	am_add_feature(f, fe_req_options_timestamp);
     }
     return f;
 }
@@ -236,11 +285,11 @@ am_allocate_feature_set(void)
     size_t			nbytes;
     am_feature_t		*result;
 
-    result = (am_feature_t *)alloc(SIZEOF(*result));
-    memset(result, 0, SIZEOF(*result));
+    result = (am_feature_t *)g_malloc(sizeof(*result));
+    memset(result, 0, sizeof(*result));
     nbytes = (((size_t)last_feature) + 8) >> 3;
     result->size = nbytes;
-    result->bytes = (unsigned char *)alloc(nbytes);
+    result->bytes = (unsigned char *)g_malloc(nbytes);
     memset(result->bytes, 0, nbytes);
     return result;
 }
@@ -383,9 +432,9 @@ am_feature_to_string(
     size_t			i;
 
     if (f == NULL) {
-	result = stralloc(_("UNKNOWNFEATURE"));
+	result = g_strdup(_("UNKNOWNFEATURE"));
     } else {
-	result = alloc((f->size * 2) + 1);
+	result = g_malloc((f->size * 2) + 1);
 	for (i = 0; i < f->size; i++) {
 	    g_snprintf(result + (i * 2), 2 + 1, "%02x", f->bytes[i]);
 	}
@@ -423,7 +472,7 @@ am_string_to_feature(
     int				ch1, ch2;
     char *			orig = s;
 
-    if (s != NULL && strcmp(s,"UNKNOWNFEATURE") != 0) {
+    if (s != NULL && !g_str_equal(s, "UNKNOWNFEATURE")) {
 	f = am_allocate_feature_set();
 	for (i = 0; i < f->size && (ch1 = *s++) != '\0'; i++) {
 	    if (isdigit(ch1)) {
