@@ -34,7 +34,7 @@
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "event.h"
 #include "packet.h"
 #include "security.h"
@@ -75,15 +75,23 @@ const security_driver_t bsdtcp_security_driver = {
     tcpma_stream_accept,
     tcpma_stream_client,
     tcpma_stream_close,
+    tcpma_stream_close_async,
     sec_stream_auth,
     sec_stream_id,
     tcpm_stream_write,
+    tcpm_stream_write_async,
     tcpm_stream_read,
     tcpm_stream_read_sync,
+    tcpm_stream_read_to_shm_ring,
     tcpm_stream_read_cancel,
+    tcpm_stream_pause,
+    tcpm_stream_resume,
     tcpm_close_connection,
     NULL,
-    NULL
+    NULL,
+    generic_data_write,
+    generic_data_write_non_blocking,
+    generic_data_read
 };
 
 static int newhandle = 1;
@@ -94,7 +102,7 @@ static int newhandle = 1;
 static void bsdtcp_fn_connect(void              *cookie,
 			      security_handle_t *security_handle,
 			      security_status_t  status);
-static int runbsdtcp(struct sec_handle *, in_port_t port);
+static int runbsdtcp(struct sec_handle *, const char *src_ip, in_port_t port);
 
 
 /*
@@ -113,6 +121,7 @@ bsdtcp_connect(
     int result;
     char *canonname;
     char *service;
+    char *src_ip;
     in_port_t port;
     struct addrinfo *res = NULL;
 
@@ -125,6 +134,7 @@ bsdtcp_connect(
 
     rh = g_new0(struct sec_handle, 1);
     security_handleinit(&rh->sech, &bsdtcp_security_driver);
+    rh->dle_hostname = g_strdup(hostname);
     rh->hostname = NULL;
     rh->rs = NULL;
     rh->ev_timeout = NULL;
@@ -133,37 +143,45 @@ bsdtcp_connect(
     result = resolve_hostname(hostname, SOCK_STREAM, &res, &canonname);
     if(result != 0) {
 	dbprintf(_("resolve_hostname(%s): %s\n"), hostname, gai_strerror(result));
-	security_seterror(&rh->sech, _("resolve_hostname(%s): %s\n"), hostname,
+	security_seterror(&rh->sech, _("resolve_hostname(%s): %s"), hostname,
 			  gai_strerror(result));
 	(*fn)(arg, &rh->sech, S_ERROR);
+	if (res) freeaddrinfo(res);
 	return;
     }
     if (canonname == NULL) {
 	dbprintf(_("resolve_hostname(%s) did not return a canonical name\n"), hostname);
 	security_seterror(&rh->sech,
-	        _("resolve_hostname(%s) did not return a canonical name\n"), hostname);
+	        _("resolve_hostname(%s) did not return a canonical name"), hostname);
 	(*fn)(arg, &rh->sech, S_ERROR);
-       return;
+	if (res) freeaddrinfo(res);
+	return;
     }
 
     rh->hostname = canonname;	/* will be replaced */
     canonname = NULL; /* steal reference */
     rh->rs = tcpma_stream_client(rh, newhandle++);
+    if (rh->rc == NULL)
+	goto error;
+
     rh->rc->recv_security_ok = &bsd_recv_security_ok;
     rh->rc->prefix_packet = &bsd_prefix_packet;
+    rh->rc->need_priv_port = 1;
 
     if (rh->rs == NULL)
 	goto error;
 
     amfree(rh->hostname);
-    rh->hostname = stralloc(rh->rs->rc->hostname);
+    rh->hostname = g_strdup(rh->rs->rc->hostname);
 
     if (conf_fn) {
 	service = conf_fn("client_port", datap);
 	if (!service || strlen(service) <= 1)
-	    service = "amanda";
+	    service = AMANDA_SERVICE_NAME;
+	src_ip = conf_fn("src_ip", datap);
     } else {
-	service = "amanda";
+	service = AMANDA_SERVICE_NAME;
+	src_ip = NULL;
     }
     port = find_port_for_service(service, "tcp");
     if (port == 0) {
@@ -178,11 +196,12 @@ bsdtcp_connect(
      */
     rh->res = res;
     rh->next_res = res;
+    rh->src_ip = src_ip;
     rh->port = port;
-    if (rh->rc->read == -1) {
+    if(rh->rc->read == -1) {
 	int result = -1;
 	while (rh->next_res) {
-	    result = runbsdtcp(rh, rh->port);
+	    result = runbsdtcp(rh, rh->src_ip, rh->port);
 	    if (result >=0 )
 		break;
 	}
@@ -203,10 +222,14 @@ bsdtcp_connect(
     rh->arg = rh;
     rh->connect_callback = fn;
     rh->connect_arg = arg;
-    rh->rs->ev_read = event_register((event_id_t)(rh->rs->rc->write),
+    g_mutex_lock(security_mutex);
+    rh->rs->rc->ev_write = event_create((event_id_t)(rh->rs->rc->write),
 	EV_WRITEFD, sec_connect_callback, rh);
-    rh->ev_timeout = event_register(CONNECT_TIMEOUT, EV_TIME,
+    rh->ev_timeout = event_create(CONNECT_TIMEOUT, EV_TIME,
 	sec_connect_timeout, rh);
+    event_activate(rh->rs->rc->ev_write);
+    event_activate(rh->ev_timeout);
+    g_mutex_unlock(security_mutex);
 
     return;
 
@@ -231,8 +254,9 @@ bsdtcp_fn_connect(
     if (status == S_OK) {
 	int so_errno;
 	socklen_t error_len = sizeof(so_errno);
-	getsockopt(rh->rc->write, SOL_SOCKET, SO_ERROR, &so_errno, &error_len);
-	if ( rh->next_res && so_errno == ECONNREFUSED) {
+	if (getsockopt(rh->rc->write, SOL_SOCKET, SO_ERROR, &so_errno, &error_len) == -1) {
+	    status = S_ERROR;
+	} else if (rh->next_res && so_errno == ECONNREFUSED) {
 	    status = S_ERROR;
 	}
     }
@@ -242,14 +266,16 @@ bsdtcp_fn_connect(
     case S_ERROR:
 	if (rh->next_res) {
 	    while (rh->next_res) {
-		result = runbsdtcp(rh, rh->port);
+		result = runbsdtcp(rh, rh->src_ip, rh->port);
 		if (result >= 0) {
 		    rh->rc->refcnt++;
-		    rh->rs->ev_read = event_register(
+		    rh->rs->rc->ev_write = event_create(
 				(event_id_t)(rh->rs->rc->write),
 				EV_WRITEFD, sec_connect_callback, rh);
-		    rh->ev_timeout = event_register(CONNECT_TIMEOUT, EV_TIME,
+		    rh->ev_timeout = event_create(CONNECT_TIMEOUT, EV_TIME,
 				sec_connect_timeout, rh);
+		    event_activate(rh->rs->rc->ev_write);
+		    event_activate(rh->ev_timeout);
 		    return;
 		}
 	    }
@@ -291,24 +317,23 @@ bsdtcp_accept(
 
     len = sizeof(sin);
     if (getpeername(in, (struct sockaddr *)&sin, &len) < 0) {
-	dbprintf(_("getpeername returned: %s\n"), strerror(errno));
-	return;
+	errmsg = g_strdup_printf("getpeername returned: %s", strerror(errno));
+	goto return_error;
     }
     if ((result = getnameinfo((struct sockaddr *)&sin, len,
 			      hostname, NI_MAXHOST, NULL, 0, 0) != 0)) {
-	dbprintf(_("getnameinfo failed: %s\n"),
-		  gai_strerror(result));
-	return;
+	errmsg = g_strdup_printf("getnameinfo failed: %s", gai_strerror(result));
+	goto return_error;
     }
     if (check_name_give_sockaddr(hostname,
 				 (struct sockaddr *)&sin, &errmsg) < 0) {
-	amfree(errmsg);
-	return;
+	goto return_error;
     }
 
-    rc = sec_tcp_conn_get(hostname, 0);
+    rc = sec_tcp_conn_get(NULL, hostname, 0);
     rc->recv_security_ok = &bsd_recv_security_ok;
     rc->prefix_packet = &bsd_prefix_packet;
+    rc->need_priv_port = 1;
     copy_sockaddr(&rc->peer, &sin);
     rc->read = in;
     rc->write = out;
@@ -317,6 +342,34 @@ bsdtcp_accept(
     rc->conf_fn = conf_fn;
     rc->datap = datap;
     sec_tcp_conn_read(rc);
+    return;
+
+return_error:
+    {
+	char *errstr = g_strjoin(" ", errmsg, NULL);
+	size_t len = strlen(errmsg);
+	guint32 *nethandle = g_malloc(sizeof(guint32));
+	guint32 *netlength = g_malloc(sizeof(guint32));
+	struct iovec iov[3];
+	errstr[0] = P_NAK;
+
+	g_debug("%s", errmsg);
+	*netlength = htonl(len);
+	iov[0].iov_base = (void *)netlength;
+	iov[0].iov_len = sizeof(*netlength);
+
+	*nethandle = htonl((guint32)1);
+	iov[1].iov_base = (void *)nethandle;
+	iov[1].iov_len = sizeof(*nethandle);
+
+	iov[2].iov_base = (void *)errstr;
+	iov[2].iov_len = len;
+
+	full_writev(out, iov, 3);
+	g_free(errstr);
+	g_free(errmsg);
+	return;
+    }
 }
 
 /*
@@ -326,27 +379,31 @@ bsdtcp_accept(
 static int
 runbsdtcp(
     struct sec_handle *	rh,
+    const char *src_ip,
     in_port_t port)
 {
     int			server_socket;
     in_port_t		my_port;
     struct tcp_conn *	rc = rh->rc;
+    char	       *stream_msg = NULL;
 
-    set_root_privs(1);
-
-    server_socket = stream_client_addr(rh->next_res,
-				       port,
-				       STREAM_BUFSIZE,
-				       STREAM_BUFSIZE,
-				       &my_port,
-				       0,
-				       1);
-    set_root_privs(0);
+    server_socket = stream_client_addr(src_ip,
+				     rh->next_res,
+				     port,
+				     STREAM_BUFSIZE,
+				     STREAM_BUFSIZE,
+				     &my_port,
+				     0,
+				     1, &stream_msg);
     rh->next_res = rh->next_res->ai_next;
 
-    if(server_socket < 0) {
-	security_seterror(&rh->sech,
-	    "%s", strerror(errno));
+    if (stream_msg) {
+	security_seterror(&rh->sech, "%s", stream_msg);
+	g_free(stream_msg);
+	return -1;
+    }
+    if (server_socket < 0) {
+	security_seterror(&rh->sech, "%s", strerror(errno));
 	return -1;
     }
 
