@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-2000 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -34,7 +35,7 @@
 #define CONFFILE_H
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 
 /* Getting Configuration Values
  * ============================
@@ -91,6 +92,20 @@ typedef struct seen_s {
     int linenum;
 } seen_t;
 
+/* boolean and no_yes_all */
+typedef enum {
+    VALUE_NO,
+    VALUE_YES,
+    VALUE_ALL
+} no_yes_all_t;
+
+/* priority types */
+typedef enum {
+    PRIORITY_LOW,               /* Always direct to tape  */
+    PRIORITY_MEDIUM,            /* If possible            */
+    PRIORITY_HIGH               /* Always to holding disk */
+} priority_t;
+
 /* holdingdisk types */
 typedef enum {
     HOLD_NEVER,                 /* Always direct to tape  */
@@ -139,6 +154,11 @@ typedef struct autolabel_s {
     autolabel_set_t  autolabel;
 } autolabel_t;
 
+typedef struct labelstr_s {
+    char     *template;
+    gboolean  match_autolabel;
+} labelstr_s;
+
 /* Dump strategies */
 typedef enum {
     DS_SKIP,        /* Don't do any dumps at all */
@@ -165,32 +185,35 @@ typedef enum {
 } taperalgo_t;
 
 /* execute_on types */
-#define EXECUTE_ON_PRE_AMCHECK         1<<0
-#define EXECUTE_ON_PRE_DLE_AMCHECK     1<<1
-#define EXECUTE_ON_PRE_HOST_AMCHECK    1<<2
-#define EXECUTE_ON_POST_AMCHECK        1<<3
-#define EXECUTE_ON_POST_DLE_AMCHECK    1<<4
-#define EXECUTE_ON_POST_HOST_AMCHECK   1<<5
-#define EXECUTE_ON_PRE_ESTIMATE        1<<6
-#define EXECUTE_ON_PRE_DLE_ESTIMATE    1<<7
-#define EXECUTE_ON_PRE_HOST_ESTIMATE   1<<8
-#define EXECUTE_ON_POST_ESTIMATE       1<<9
-#define EXECUTE_ON_POST_DLE_ESTIMATE   1<<10
-#define EXECUTE_ON_POST_HOST_ESTIMATE  1<<11
-#define EXECUTE_ON_PRE_BACKUP          1<<12
-#define EXECUTE_ON_PRE_DLE_BACKUP      1<<13
-#define EXECUTE_ON_PRE_HOST_BACKUP     1<<14
-#define EXECUTE_ON_POST_BACKUP         1<<15
-#define EXECUTE_ON_POST_DLE_BACKUP     1<<16
-#define EXECUTE_ON_POST_HOST_BACKUP    1<<17
-#define EXECUTE_ON_PRE_RECOVER         1<<18
-#define EXECUTE_ON_POST_RECOVER        1<<19
-#define EXECUTE_ON_PRE_LEVEL_RECOVER   1<<20
-#define EXECUTE_ON_POST_LEVEL_RECOVER  1<<21
-#define EXECUTE_ON_INTER_LEVEL_RECOVER 1<<22
+#define EXECUTE_ON_PRE_AMCHECK         (1 << 0)
+#define EXECUTE_ON_PRE_DLE_AMCHECK     (1 << 1)
+#define EXECUTE_ON_PRE_HOST_AMCHECK    (1 << 2)
+#define EXECUTE_ON_POST_AMCHECK        (1 << 3)
+#define EXECUTE_ON_POST_DLE_AMCHECK    (1 << 4)
+#define EXECUTE_ON_POST_HOST_AMCHECK   (1 << 5)
+#define EXECUTE_ON_PRE_ESTIMATE        (1 << 6)
+#define EXECUTE_ON_PRE_DLE_ESTIMATE    (1 << 7)
+#define EXECUTE_ON_PRE_HOST_ESTIMATE   (1 << 8)
+#define EXECUTE_ON_POST_ESTIMATE       (1 << 9)
+#define EXECUTE_ON_POST_DLE_ESTIMATE   (1 << 10)
+#define EXECUTE_ON_POST_HOST_ESTIMATE  (1 << 11)
+#define EXECUTE_ON_PRE_BACKUP          (1 << 12)
+#define EXECUTE_ON_PRE_DLE_BACKUP      (1 << 13)
+#define EXECUTE_ON_PRE_HOST_BACKUP     (1 << 14)
+#define EXECUTE_ON_POST_BACKUP         (1 << 15)
+#define EXECUTE_ON_POST_DLE_BACKUP     (1 << 16)
+#define EXECUTE_ON_POST_HOST_BACKUP    (1 << 17)
+#define EXECUTE_ON_PRE_RECOVER         (1 << 18)
+#define EXECUTE_ON_POST_RECOVER        (1 << 19)
+#define EXECUTE_ON_PRE_LEVEL_RECOVER   (1 << 20)
+#define EXECUTE_ON_POST_LEVEL_RECOVER  (1 << 21)
+#define EXECUTE_ON_INTER_LEVEL_RECOVER (1 << 22)
 typedef int execute_on_t;
 
-typedef int execute_where_t;
+typedef enum {
+    EXECUTE_WHERE_CLIENT,
+    EXECUTE_WHERE_SERVER,
+} execute_where_t;
 
 typedef enum {
     SEND_AMREPORT_ALL,
@@ -212,6 +235,7 @@ typedef struct exinclude_s {
 
 typedef struct {
     int append;
+    int visible;
     int priority;
     GSList* values;
     seen_t seen;
@@ -221,6 +245,31 @@ typedef GHashTable* proplist_t;
 /* A GSlist where each element is a 'char*' */
 typedef GSList* identlist_t;
 
+typedef enum {
+    TAG_NAME,
+    TAG_ALL,
+    TAG_OTHER
+} dumptype_tag_t;
+
+typedef enum {
+    LEVEL_ALL,
+    LEVEL_FULL,
+    LEVEL_INCR
+} level_type_t;
+
+typedef struct {
+    dumptype_tag_t  tag_type;
+    char           *tag;
+    level_type_t    level;
+} dump_selection_t;
+typedef GSList* dump_selection_list_t;
+
+typedef struct {
+    char *storage;
+    int  days;
+} vault_el_t;
+typedef GSList* vault_list_t;
+
 /* part_cache_types */
 typedef enum {
     PART_CACHE_TYPE_NONE,
@@ -264,10 +313,13 @@ typedef enum {
     CONFTYPE_IDENTLIST,
     CONFTYPE_DATA_PATH,
     CONFTYPE_AUTOLABEL,
+    CONFTYPE_LABELSTR,
     CONFTYPE_PART_CACHE_TYPE,
     CONFTYPE_HOST_LIMIT,
     CONFTYPE_NO_YES_ALL,
     CONFTYPE_STR_LIST,
+    CONFTYPE_DUMP_SELECTION,
+    CONFTYPE_VAULT_LIST,
 } conftype_t;
 
 typedef enum {
@@ -293,7 +345,10 @@ typedef struct val_s {
 	estimatelist_t  estimatelist;
 	identlist_t     identlist;
         autolabel_t     autolabel;
+        labelstr_s      labelstr;
 	host_limit_t    host_limit;
+	dump_selection_list_t   dump_selection;
+	vault_list_t    vault_list;
     } v;
     seen_t seen;
     conftype_t type;
@@ -320,7 +375,7 @@ dump_holdingdisk_t    val_t_to_holding  (val_t *);
 estimatelist_t        val_t_to_estimatelist (val_t *);
 strategy_t            val_t_to_strategy (val_t *);
 taperalgo_t           val_t_to_taperalgo(val_t *);
-int                   val_t_to_priority (val_t *);
+int                   val_t_to_priority (val_t *); /* priority_t or int */
 float                *val_t_to_rate     (val_t *); /* array of two floats */
 exinclude_t           val_t_to_exinclude(val_t *);
 int                  *val_t_to_intrange (val_t *); /* array of two ints */
@@ -330,9 +385,12 @@ execute_on_t          val_t_to_execute_on(val_t *);
 execute_where_t       val_t_to_execute_where(val_t *);
 send_amreport_t       val_t_to_send_amreport(val_t *);
 data_path_t           val_t_to_data_path(val_t *);
-autolabel_t           val_t_to_autolabel(val_t *);
+autolabel_t          *val_t_to_autolabel(val_t *);
+labelstr_s           *val_t_to_labelstr(val_t *);
 part_cache_type_t     val_t_to_part_cache_type(val_t *);
 host_limit_t         *val_t_to_host_limit(val_t *);
+dump_selection_list_t val_t_to_dump_selection(val_t *);
+vault_list_t          val_t_to_vault_list(val_t *);
 
 /* Has the given val_t been seen in a configuration file or config overwrite?
  *
@@ -355,36 +413,39 @@ host_limit_t         *val_t_to_host_limit(val_t *);
  * (in the macro name) to the corresponding union field.  The macros work
  * as lvalues, too.
  */
-#define val_t__seen(val)          ((val)->seen)
-#define val_t__int(val)           ((val)->v.i)
-#define val_t__int64(val)         ((val)->v.int64)
-#define val_t__real(val)          ((val)->v.r)
-#define val_t__str(val)           ((val)->v.s)
-#define val_t__ident(val)         ((val)->v.s)
-#define val_t__identlist(val)     ((val)->v.identlist)
-#define val_t__time(val)          ((val)->v.t)
-#define val_t__size(val)          ((val)->v.size)
-#define val_t__boolean(val)       ((val)->v.i)
-#define val_t__no_yes_all(val)    ((val)->v.i)
-#define val_t__compress(val)      ((val)->v.i)
-#define val_t__encrypt(val)       ((val)->v.i)
-#define val_t__holding(val)       ((val)->v.i)
-#define val_t__estimatelist(val)  ((val)->v.estimatelist)
-#define val_t__strategy(val)      ((val)->v.i)
-#define val_t__taperalgo(val)     ((val)->v.i)
-#define val_t__send_amreport(val) ((val)->v.i)
-#define val_t__priority(val)      ((val)->v.i)
-#define val_t__rate(val)          ((val)->v.rate)
-#define val_t__exinclude(val)     ((val)->v.exinclude)
-#define val_t__intrange(val)      ((val)->v.intrange)
-#define val_t__proplist(val)      ((val)->v.proplist)
-#define val_t__application(val)   ((val)->v.application)
-#define val_t__execute_on(val)    ((val)->v.i)
-#define val_t__execute_where(val) ((val)->v.i)
-#define val_t__data_path(val)     ((val)->v.i)
-#define val_t__autolabel(val)     ((val)->v.autolabel)
+#define val_t__seen(val)            ((val)->seen)
+#define val_t__int(val)             ((val)->v.i)
+#define val_t__int64(val)           ((val)->v.int64)
+#define val_t__real(val)            ((val)->v.r)
+#define val_t__str(val)             ((val)->v.s)
+#define val_t__ident(val)           ((val)->v.s)
+#define val_t__identlist(val)       ((val)->v.identlist)
+#define val_t__time(val)            ((val)->v.t)
+#define val_t__size(val)            ((val)->v.size)
+#define val_t__boolean(val)         ((val)->v.i)
+#define val_t__no_yes_all(val)      ((val)->v.i)
+#define val_t__compress(val)        ((val)->v.i)
+#define val_t__encrypt(val)         ((val)->v.i)
+#define val_t__holding(val)         ((val)->v.i)
+#define val_t__estimatelist(val)    ((val)->v.estimatelist)
+#define val_t__strategy(val)        ((val)->v.i)
+#define val_t__taperalgo(val)       ((val)->v.i)
+#define val_t__send_amreport(val)   ((val)->v.i)
+#define val_t__priority(val)        ((val)->v.i)
+#define val_t__rate(val)            ((val)->v.rate)
+#define val_t__exinclude(val)       ((val)->v.exinclude)
+#define val_t__intrange(val)        ((val)->v.intrange)
+#define val_t__proplist(val)        ((val)->v.proplist)
+#define val_t__application(val)     ((val)->v.application)
+#define val_t__execute_on(val)      ((val)->v.i)
+#define val_t__execute_where(val)   ((val)->v.i)
+#define val_t__data_path(val)       ((val)->v.i)
+#define val_t__autolabel(val)       (&((val)->v.autolabel))
+#define val_t__labelstr(val)        (&((val)->v.labelstr))
 #define val_t__part_cache_type(val) ((val)->v.i)
-#define val_t__host_limit(val)    ((val)->v.host_limit)
+#define val_t__host_limit(val)      ((val)->v.host_limit)
+#define val_t__dump_selection(val)  ((val)->v.dump_selection)
+#define val_t__vault_list(val)      ((val)->v.vault_list)
 
 /*
  * Parameters
@@ -503,6 +564,25 @@ typedef enum {
     CNF_REPORT_USE_MEDIA,
     CNF_REPORT_NEXT_MEDIA,
     CNF_REPORT_FORMAT,
+    CNF_ACTIVE_STORAGE,
+    CNF_STORAGE,
+    CNF_VAULT_STORAGE,
+    CNF_CMDFILE,
+    CNF_COMPRESS_INDEX,
+    CNF_SORT_INDEX,
+    CNF_REST_API_PORT,
+    CNF_REST_SSL_CERT,
+    CNF_REST_SSL_KEY,
+    CNF_SSL_FINGERPRINT_FILE,
+    CNF_SSL_CERT_FILE,
+    CNF_SSL_KEY_FILE,
+    CNF_SSL_CA_CERT_FILE,
+    CNF_SSL_CIPHER_LIST,
+    CNF_SSL_CHECK_HOST,
+    CNF_SSL_CHECK_CERTIFICATE_HOST,
+    CNF_SSL_DIR,
+    CNF_SSL_CHECK_FINGERPRINT,
+    CNF_HOSTNAME,
     CNF_CNF /* sentinel */
 } confparm_key;
 
@@ -553,6 +633,7 @@ val_t *getconf(confparm_key key);
 #define getconf_proplist(key)     (val_t_to_proplist(getconf((key))))
 #define getconf_send_amreport(key) (val_t_to_send_amreport(getconf((key))))
 #define getconf_autolabel(key)    (val_t_to_autolabel(getconf((key))))
+#define getconf_labelstr(key)     (val_t_to_labelstr(getconf((key))))
 #define getconf_part_cache_type(key) (val_t_to_part_cache_type(getconf((key))))
 #define getconf_recovery_limit(key) (val_t_to_host_limit(getconf((key))))
 
@@ -569,6 +650,8 @@ GSList *getconf_list(char *listname);
  */
 val_t *getconf_byname(char *key);
 
+char *confparm_key_to_name(int parm);
+
 /*
  * Derived values
  *
@@ -583,6 +666,8 @@ val_t *getconf_byname(char *key);
  */
 long int getconf_unit_divisor(void);
 
+extern char *prepend_prefix;
+
 /* If any of these globals are true, the corresponding component will
  * send verbose debugging output to the debug file.  The options are
  * set during config_init, but can be modified at will after that if 
@@ -651,6 +736,8 @@ val_t *tapetype_getconf(tapetype_t *ttyp, tapetype_key key);
  */
 char *tapetype_name(tapetype_t *ttyp);
 
+char *tapetype_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this tapetype?  This
  * applies to the specific parameter *within* the tapetype.
  *
@@ -731,6 +818,16 @@ typedef enum {
     DUMPTYPE_RECOVERY_LIMIT,
     DUMPTYPE_DUMP_LIMIT,
     DUMPTYPE_MAX_WARNINGS,
+    DUMPTYPE_RETRY_DUMP,
+    DUMPTYPE_TAG,
+    DUMPTYPE_SSL_FINGERPRINT_FILE,
+    DUMPTYPE_SSL_CERT_FILE,
+    DUMPTYPE_SSL_KEY_FILE,
+    DUMPTYPE_SSL_CA_CERT_FILE,
+    DUMPTYPE_SSL_CIPHER_LIST,
+    DUMPTYPE_SSL_CHECK_HOST,
+    DUMPTYPE_SSL_CHECK_CERTIFICATE_HOST,
+    DUMPTYPE_SSL_CHECK_FINGERPRINT,
     DUMPTYPE_DUMPTYPE /* sentinel */
 } dumptype_key;
 
@@ -760,6 +857,8 @@ val_t *dumptype_getconf(dumptype_t *dtyp, dumptype_key key);
  */
 char *dumptype_name(dumptype_t *dtyp);
 
+char *dumptype_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this dumptype?  This
  * applies to the specific parameter *within* the dumptype.
  *
@@ -823,6 +922,16 @@ char *dumptype_name(dumptype_t *dtyp);
 #define dumptype_get_recovery_limit(dtyp)      (val_t_to_host_limit(dumptype_getconf((dtyp), DUMPTYPE_RECOVERY_LIMIT)))
 #define dumptype_get_dump_limit(dtyp)          (val_t_to_host_limit(dumptype_getconf((dtyp), DUMPTYPE_DUMP_LIMIT)))
 #define dumptype_get_max_warnings(dtyp)        (val_t_to_int(dumptype_getconf((dtyp), DUMPTYPE_MAX_WARNINGS)))
+#define dumptype_get_retry_dump(dtyp)          (val_t_to_int(dumptype_getconf((dtyp), DUMPTYPE_RETRY_DUMP)))
+#define dumptype_get_tags(dtyp)                (val_t_to_str_list(dumptype_getconf((dtyp), DUMPTYPE_TAG)))
+#define dumptype_get_ssl_fingerprint_file(dtyp) (val_t_to_str(dumptype_getconf((dtyp), DUMPTYPE_SSL_FINGERPRINT_FILE)))
+#define dumptype_get_ssl_cert_file(dtyp)        (val_t_to_str(dumptype_getconf((dtyp), DUMPTYPE_SSL_CERT_FILE)))
+#define dumptype_get_ssl_key_file(dtyp)         (val_t_to_str(dumptype_getconf((dtyp), DUMPTYPE_SSL_KEY_FILE)))
+#define dumptype_get_ssl_ca_cert_file(dtyp)     (val_t_to_str(dumptype_getconf((dtyp), DUMPTYPE_SSL_CA_CERT_FILE)))
+#define dumptype_get_ssl_cipher_list(dtyp)      (val_t_to_str(dumptype_getconf((dtyp), DUMPTYPE_SSL_CIPHER_LIST)))
+#define dumptype_get_ssl_check_host(dtyp)       (val_t_to_boolean(dumptype_getconf((dtyp), DUMPTYPE_SSL_CHECK_HOST)))
+#define dumptype_get_ssl_check_certificate_host(dtyp) (val_t_to_boolean(dumptype_getconf((dtyp), DUMPTYPE_SSL_CHECK_CERTIFICATE_HOST)))
+#define dumptype_get_ssl_check_fingerprint(dtyp)(val_t_to_boolean(dumptype_getconf((dtyp), DUMPTYPE_SSL_CHECK_FINGERPRINT)))
 
 /*
  * Interface parameter access
@@ -831,6 +940,7 @@ char *dumptype_name(dumptype_t *dtyp);
 typedef enum {
     INTER_COMMENT,
     INTER_MAXUSAGE,
+    INTER_SRC_IP,
     INTER_INTER /* sentinel */
 } interface_key;
 
@@ -860,6 +970,8 @@ val_t *interface_getconf(interface_t *iface, interface_key key);
  */
 char *interface_name(interface_t *iface);
 
+char *interface_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this interface?  This
  * applies to the specific parameter *within* the interface.
  *
@@ -876,6 +988,7 @@ char *interface_name(interface_t *iface);
  */
 #define interface_get_comment(iface)    (val_t_to_str(interface_getconf((iface), INTER_COMMENT)))
 #define interface_get_maxusage(iface)   (val_t_to_int(interface_getconf((iface), INTER_MAXUSAGE)))
+#define interface_get_src_ip(iface)     (val_t_to_str(interface_getconf((iface), INTER_SRC_IP)))
 
 /*
  * Holdingdisk parameter access
@@ -921,6 +1034,8 @@ val_t *holdingdisk_getconf(holdingdisk_t *hdisk, holdingdisk_key key);
  */
 char *holdingdisk_name(holdingdisk_t *hdisk);
 
+char *holdingdisk_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this holdingdisk?  This
  * applies to the specific parameter *within* the holdingdisk.
  *
@@ -976,6 +1091,8 @@ val_t *application_getconf(application_t *app, application_key key);
  */
 char *application_name(application_t *app);
 
+char *application_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this application?  This
  * applies to the specific parameter *within* the application.
  *
@@ -1035,6 +1152,8 @@ val_t *pp_script_getconf(pp_script_t *pps, pp_script_key key);
  */
 char *pp_script_name(pp_script_t *pps);
 
+char *pp_script_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this pp_script?  This
  * applies to the specific parameter *within* the pp_script.
  *
@@ -1096,6 +1215,8 @@ val_t *device_config_getconf(device_config_t *devconf, device_config_key key);
  */
 char *device_config_name(device_config_t *devconf);
 
+char *device_config_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this device_config?  This
  * applies to the specific parameter *within* the device_config.
  *
@@ -1130,6 +1251,8 @@ typedef enum {
 } changer_config_key;
 
 /* opaque object */
+char **get_changer_list(void);
+
 typedef struct changer_config_s changer_config_t;
 
 /* Given the name of the changer, return a changer_config_t object.  Returns NULL
@@ -1156,6 +1279,8 @@ val_t *changer_config_getconf(changer_config_t *devconf, changer_config_key key)
  */
 char *changer_config_name(changer_config_t *devconf);
 
+char *changer_config_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this changer_config?  This
  * applies to the specific parameter *within* the changer_config.
  *
@@ -1215,6 +1340,8 @@ val_t *interactivity_getconf(interactivity_t *app, interactivity_key key);
  */
 char *interactivity_name(interactivity_t *app);
 
+char *interactivity_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this interactivity?
  * This applies to the specific parameter *within* the interactivity.
  *
@@ -1269,6 +1396,8 @@ val_t *taperscan_getconf(taperscan_t *app, taperscan_key key);
  */
 char *taperscan_name(taperscan_t *app);
 
+char *taperscan_key_to_name(int parm);
+
 /* (convenience macro) has this parameter been seen in this taperscan?
  * This applies to the specific parameter *within* the taperscan.
  *
@@ -1283,11 +1412,186 @@ char *taperscan_name(taperscan_t *app);
  * @param ttyp: the taperscan to examine
  * @returns: various
  */
-#define taperscan_get_comment(taperscan)  (val_t_to_str(taperscan_getconf((taperscan), TAPERSCAN_COMMENT))
+#define taperscan_get_comment(taperscan)  (val_t_to_str(taperscan_getconf((taperscan), TAPERSCAN_COMMENT)))
 #define taperscan_get_plugin(taperscan)   (val_t_to_str(taperscan_getconf((taperscan), TAPERSCAN_PLUGIN)))
 #define taperscan_get_property(taperscan) (val_t_to_proplist(taperscan_getconf((taperscan), TAPERSCAN_PROPERTY)))
 
 
+/* A policy interface */
+typedef enum policy_e  {
+    POLICY_COMMENT,
+    POLICY_RETENTION_TAPES,
+    POLICY_RETENTION_DAYS,
+    POLICY_RETENTION_RECOVER,
+    POLICY_RETENTION_FULL,
+    POLICY_POLICY
+} policy_key;
+
+
+/* opaque object */
+typedef struct policy_s policy_s;
+
+/* Given the name of the policy, return a policy object.
+ *  Returns NULL if no matching policy exists.
+ *  Note that the match is case-insensitive.
+ *
+ * @param identifier: name of the desired policy
+ * @returns: object or NULL
+ */
+
+policy_s *lookup_policy(char *identifier);
+
+/* Given a policy and a key, return a pointer to the corresponding val_t.
+ *
+ * @param ttyp: the policy to examine
+ * @param key: policy (one of the TAPERSCAN_* constants)
+ * @returns: pointer to value
+ */
+val_t *policy_getconf(policy_s *app, policy_key key);
+
+/* Get the name of this policy.
+ *
+ * @param ttyp: the policy to examine
+ * @returns: name of the policy
+ */
+char *policy_name(policy_s *app);
+
+char *policy_key_to_name(int parm);
+
+/* (convenience macro) has this parameter been seen in this policy?
+ * This applies to the specific parameter *within* the policy.
+ *
+ * @param key: policy_key
+ * @returns: boolean
+ */
+#define policy_seen(app, key)       (val_t_seen(policy_getconf((app), (key))))
+
+/* (convenience macros)
+ * fetch a particular parameter; caller must know the correct type.
+ *
+ * @param ttyp: the policy to examine
+ * @returns: various
+ */
+#define policy_get_comment(policy)  (val_t_to_str(policy_getconf((policy), POLICY_COMMENT)))
+#define policy_get_retention_tapes(policy)  (val_t_to_int(policy_getconf((policy), POLICY_RETENTION_TAPES)))
+#define policy_get_retention_days(policy)  (val_t_to_int(policy_getconf((policy), POLICY_RETENTION_DAYS)))
+#define policy_get_retention_recover(policy)  (val_t_to_int(policy_getconf((policy), POLICY_RETENTION_RECOVER)))
+#define policy_get_retention_full(policy)  (val_t_to_int(policy_getconf((policy), POLICY_RETENTION_FULL)))
+
+
+/* A storage interface */
+typedef enum storage_e  {
+    STORAGE_COMMENT,
+    STORAGE_POLICY,
+    STORAGE_TAPEDEV,
+    STORAGE_TPCHANGER,
+    STORAGE_LABELSTR,
+    STORAGE_AUTOLABEL,
+    STORAGE_META_AUTOLABEL,
+    STORAGE_TAPEPOOL,
+    STORAGE_RUNTAPES,
+    STORAGE_TAPERSCAN,
+    STORAGE_TAPETYPE,
+    STORAGE_MAX_DLE_BY_VOLUME,
+    STORAGE_TAPERALGO,
+    STORAGE_TAPER_PARALLEL_WRITE,
+    STORAGE_EJECT_VOLUME,
+    STORAGE_ERASE_VOLUME,
+    STORAGE_DEVICE_OUTPUT_BUFFER_SIZE,
+    STORAGE_AUTOFLUSH,
+    STORAGE_FLUSH_THRESHOLD_DUMPED,
+    STORAGE_FLUSH_THRESHOLD_SCHEDULED,
+    STORAGE_TAPERFLUSH,
+    STORAGE_REPORT_USE_MEDIA,
+    STORAGE_REPORT_NEXT_MEDIA,
+    STORAGE_INTERACTIVITY,
+    STORAGE_SET_NO_REUSE,
+    STORAGE_DUMP_SELECTION,
+    STORAGE_ERASE_ON_FAILURE,
+    STORAGE_ERASE_ON_FULL,
+    STORAGE_VAULT_LIST,
+    STORAGE_STORAGE
+} storage_key;
+
+
+/* opaque object */
+typedef struct storage_s storage_t;
+
+storage_t *get_first_storage(void);
+storage_t *get_next_storage(storage_t *st);
+char **get_storage_list(void);
+
+/* Given the name of the storage, return a storage object.
+ *  Returns NULL if no matching storage exists.
+ *  Note that the match is case-insensitive.
+ *
+ * @param identifier: name of the desired storage
+ * @returns: object or NULL
+ */
+
+storage_t *lookup_storage(char *identifier);
+
+/* Given a storage and a key, return a pointer to the corresponding val_t.
+ *
+ * @param ttyp: the storage to examine
+ * @param key: storage (one of the TAPERSCAN_* constants)
+ * @returns: pointer to value
+ */
+val_t *storage_getconf(storage_t *app, storage_key key);
+
+/* Get the name of this storage.
+ *
+ * @param ttyp: the storage to examine
+ * @returns: name of the storage
+ */
+char *storage_name(storage_t *app);
+
+char *storage_key_to_name(int parm);
+
+/* (convenience macro) has this parameter been seen in this storage?
+ * This applies to the specific parameter *within* the storage.
+ *
+ * @param key: storage_key
+ * @returns: boolean
+ */
+#define storage_seen(app, key)       (val_t_seen(storage_getconf((app), (key))))
+
+/* (convenience macros)
+ * fetch a particular parameter; caller must know the correct type.
+ *
+ * @param ttyp: the storage to examine
+ * @returns: various
+ */
+#define storage_get_comment(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_COMMENT)))
+#define storage_get_policy(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_POLICY)))
+#define storage_get_tapedev(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_TAPEDEV)))
+#define storage_get_tpchanger(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_TPCHANGER)))
+#define storage_get_labelstr(storage)  (val_t_to_labelstr(storage_getconf((storage), STORAGE_LABELSTR)))
+#define storage_get_autolabel(storage)  (val_t_to_autolabel(storage_getconf((storage), STORAGE_AUTOLABEL)))
+#define storage_get_meta_autolabel(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_META_AUTOLABEL)))
+#define storage_get_tapepool(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_TAPEPOOL)))
+#define storage_get_runtapes(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_RUNTAPES)))
+#define storage_get_taperscan(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_TAPERSCAN)))
+#define storage_get_tapetype(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_TAPETYPE)))
+#define storage_get_max_dle_by_volume(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_MAX_DLE_BY_VOLUME)))
+#define storage_get_taperalgo(storage)  (val_t_to_taperalgo(storage_getconf((storage), STORAGE_TAPERALGO)))
+#define storage_get_taper_parallel_write(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_TAPER_PARALLEL_WRITE)))
+#define storage_get_eject_volume(storage)  (val_t_to_boolean(storage_getconf((storage), STORAGE_EJECT_VOLUME)))
+#define storage_get_erase_volume(storage)  (val_t_to_boolean(storage_getconf((storage), STORAGE_ERASE_VOLUME)))
+#define storage_get_device_output_buffer_size(storage)  (val_t_to_size(storage_getconf((storage), STORAGE_DEVICE_OUTPUT_BUFFER_SIZE)))
+#define storage_get_autoflush(storage)  (val_t_to_no_yes_all(storage_getconf((storage), STORAGE_AUTOFLUSH)))
+#define storage_get_flush_threshold_dumped(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_FLUSH_THRESHOLD_DUMPED)))
+#define storage_get_flush_threshold_scheduled(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_FLUSH_THRESHOLD_SCHEDULED)))
+#define storage_get_taperflush(storage)  (val_t_to_int(storage_getconf((storage), STORAGE_TAPERFLUSH)))
+#define storage_get_report_use_media(storage)  (val_t_to_bool(storage_getconf((storage), STORAGE_REPORT_USE_MEDIA)))
+#define storage_get_report_next_media(storage)  (val_t_to_bool(storage_getconf((storage), STORAGE_REPORT_NEXT_MEDIA)))
+#define storage_get_interactivity(storage)  (val_t_to_str(storage_getconf((storage), STORAGE_INTERACTIVITY)))
+#define storage_get_set_no_reuse(storage)  (val_t_to_bool(storage_getconf((storage), STORAGE_SET_NO_REUSE)))
+#define storage_get_dump_selection(storage)  (val_t_to_dump_selection(storage_getconf((storage), STORAGE_DUMP_SELECTION)))
+#define storage_get_erase_on_failure(storage)  (val_t_to_boolean(storage_getconf((storage), STORAGE_ERASE_ON_FAILURE)))
+#define storage_get_erase_on_full(storage)  (val_t_to_boolean(storage_getconf((storage), STORAGE_ERASE_ON_FULL)))
+#define storage_get_vault_list(storage)  (val_t_to_vault_list(storage_getconf((storage), STORAGE_VAULT_LIST)))
+
 /*
  * Error Handling
  */
@@ -1325,6 +1629,7 @@ void config_clear_errors(void);
  * applications.
  */
 void config_print_errors(void);
+void config_print_errors_as_message(void);
 
 /* Add an error message to the list of errors, and make sure tha the
  * error level is at least LEVEL.  This is used by the diskfile module
@@ -1422,6 +1727,12 @@ typedef enum {
     /* New configuration should "overlay" existing configuration; this
      * is used by clients to load multiple amanda-client.conf files. */
     CONFIG_INIT_OVERLAY = 1 << 3,
+
+    /* Load the global config */
+    CONFIG_INIT_GLOBAL = 1 << 4,
+
+    /* Do not generate errors (internal use) */
+    CONFIG_OVERRDIDE_NO_ERROR = 1 << 5,
 } config_init_flags;
 
 /* Initialize this application's configuration, with the specific actions
@@ -1445,6 +1756,9 @@ typedef enum {
 cfgerr_level_t config_init(
 	 config_init_flags flags,
 	 char *arg_config_name);
+cfgerr_level_t config_init_with_global(
+	 config_init_flags flags,
+	 char *arg_config_name);
 
 /* Free all memory allocated for the configuration.  This effectively
  * reverses the effects of config_init().
@@ -1481,6 +1795,8 @@ char *get_config_dir(void);
 /* Get the config filename */
 char *get_config_filename(void);
 
+char *get_running_on(void);
+
 /*
  * Utilities
  */
@@ -1544,7 +1860,7 @@ dumptype_t *read_dumptype(char *name, FILE *from, char *fname, int *linenum);
 char *anonymous_value(void);
 
 /* Extend a relative filename with the current config_dir; if filename is already
- * absolute, this is equivalent to stralloc.
+ * absolute, this is equivalent to g_strdup.
  *
  * @param filename: filename to extend
  * @returns: newly allocated filename
@@ -1594,4 +1910,31 @@ void free_property_t(gpointer p);
  */
 gchar *amandaify_property_name(const gchar *name);
 
+/**
+ * Returns a string representation of a CONFTYPE_EXECUTE_ON property with a
+ * given separator. It is up to the caller to free that string if it so desires.
+ *
+ * @param flags: the CONFTYPE_EXECUTE_ON property.
+ * @param separator: the separator to use.
+ * @returns: the string representation.
+ */
+
+char *execute_on_to_string(int flags, char *separator);
+
+char *custom_escape(char *str);
+
+val_t *getconf_human(confparm_key key);
+val_t *dumptype_getconf_human(dumptype_t *typ, dumptype_key key);
+val_t *tapetype_getconf_human(tapetype_t *typ, tapetype_key key);
+val_t *application_getconf_human(application_t *typ, application_key key);
+val_t *device_config_getconf_human(device_config_t *typ, device_config_key key);
+val_t *changer_config_getconf_human(changer_config_t *typ, changer_config_key key);
+val_t *storage_getconf_human(storage_t *typ, storage_key key);
+val_t *pp_script_getconf_human(pp_script_t *typ, pp_script_key key);
+val_t *holdingdisk_getconf_human(holdingdisk_t *typ, holdingdisk_key key);
+val_t *interface_getconf_human(interface_t *typ, interface_key key);
+val_t *interactivity_getconf_human(interactivity_t *typ, interactivity_key key);
+val_t *taperscan_getconf_human(taperscan_t *typ, taperscan_key key);
+val_t *policy_getconf_human(policy_s *typ, policy_key key);
+
 #endif /* ! CONFFILE_H */
