@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -85,7 +86,7 @@ typedef void (*event_fn_t)(void *);
  * count on the time events being too accurate.  They depend on the
  * caller calling event_loop() often enough.
  */
-event_handle_t *event_register(event_id_t, event_type_t, event_fn_t, void *);
+//event_handle_t *event_register(event_id_t, event_type_t, event_fn_t, void *);
 event_handle_t *event_create(event_id_t, event_type_t, event_fn_t, void *);
 void event_activate(event_handle_t *);
 
@@ -115,6 +116,19 @@ void event_wait(event_handle_t *evt);
  */
 void event_loop(int nonblock);
 
+/*
+ * Process events, not even stopping when there are no more event API events
+ * left (there may be other GMainLoop events pending); similar to
+ * g_main_loop_run, but compatible with the event API.
+ */
+void event_loop_run(void);
+
+/*
+ * Stop an event_loop_run invocation; similar to g_main_loop_quit, but
+ * compatible with the event API
+ */
+void event_loop_quit(void);
+
 /*
  * Get the default GMainLoop object.  Applications which use the Glib
  * main loop directly should use this object for calls to e.g.,
