@@ -1,8 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1997-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
- * All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc  All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -30,12 +29,11 @@
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "timestamp.h"
-#include "arglist.h"
 #include "file.h"
 
-static void areads_getbuf(const char *s, int l, int fd);
+static struct areads_buffer *areads_getbuf(const char *s, int l, int fd);
 static char *original_cwd = NULL;
 
 /*
@@ -58,16 +56,19 @@ mkpdir(
     rc = 0;
 
     /* Remove last member of file, put the result in dir */
-    dir = stralloc(file); /* make a copy we can play with */
+    dir = g_strdup(file); /* make a copy we can play with */
     p = strrchr(dir, '/');
-    *p = '\0';
+    if (p)
+	*p = '\0';
 
     rc = mkdir(dir, mode);
     if (rc != 0) {
 	if (errno == ENOENT) { /* create parent directory */
 	    rc = mkpdir(dir, mode, uid, gid);
-	    if (rc != 0)
+	    if (rc != 0) {
+		amfree(dir);
 		return rc;
+	    }
 	    rc = mkdir(dir, mode);
 	}
 	if (rc != 0 && errno == EEXIST) {
@@ -105,7 +106,7 @@ rmpdir(
     int rc;
     char *p, *dir;
 
-    if(strcmp(file, topdir) == 0) return 0; /* all done */
+    if(g_str_equal(file, topdir)) return 0; /* all done */
 
     rc = rmdir(file);
     if (rc != 0) switch(errno) {
@@ -126,7 +127,7 @@ rmpdir(
 
     if(rc != 0) return -1; /* unexpected error */
 
-    dir = stralloc(file);
+    dir = g_strdup(file);
 
     p = strrchr(dir, '/');
     if (p == NULL || p == dir) {
@@ -183,11 +184,11 @@ safe_cd(void)
 
     if (client_uid != (uid_t) -1) {
 #if defined(AMANDA_DBGDIR)
-	d = stralloc2(AMANDA_DBGDIR, "/.");
+	d = g_strconcat(AMANDA_DBGDIR, "/.", NULL);
 	(void) mkpdir(d, (mode_t)0700, client_uid, client_gid);
 	amfree(d);
 #endif
-	d = stralloc2(AMANDA_TMPDIR, "/.");
+	d = g_strconcat(AMANDA_TMPDIR, "/.", NULL);
 	(void) mkpdir(d, (mode_t)0700, client_uid, client_gid);
 	amfree(d);
     }
@@ -237,6 +238,62 @@ void
 safe_fd(
     int		fd_start,
     int		fd_count)
+{
+    safe_fd3(fd_start, fd_count, 0, 0);
+}
+
+/*
+ *=====================================================================
+ * Close all file descriptors except stdin, stdout and stderr.  Make
+ * sure they are open.
+ *
+ * void safe_fd2 (fd_start, fd_count, fd1)
+ *
+ * entry:	fd_start - start of fd-s to leave alone (or -1)
+ *		fd_count - count of fd-s to leave alone
+ *		fd1      - do not close
+ * exit:	none
+ *
+ * On exit, all three standard file descriptors will be open and pointing
+ * someplace (either what we were handed or /dev/null) and all other
+ * file descriptors (up to FD_SETSIZE) will be closed.
+ *=====================================================================
+ */
+
+void
+safe_fd2(
+    int		fd_start,
+    int		fd_count,
+    int		fd1)
+{
+    safe_fd3(fd_start, fd_count, fd1, 0);
+}
+
+/*
+ *=====================================================================
+ * Close all file descriptors except stdin, stdout and stderr.  Make
+ * sure they are open.
+ *
+ * void safe_fd3 (fd_start, fd_count, fd1, fd2)
+ *
+ * entry:	fd_start - start of fd-s to leave alone (or -1)
+ *		fd_count - count of fd-s to leave alone
+ *		fd1      - do not close
+ *		fd2      - do not close
+ * exit:	none
+ *
+ * On exit, all three standard file descriptors will be open and pointing
+ * someplace (either what we were handed or /dev/null) and all other
+ * file descriptors (up to FD_SETSIZE) will be closed.
+ *=====================================================================
+ */
+
+void
+safe_fd3(
+    int		fd_start,
+    int		fd_count,
+    int		fd1,
+    int		fd2)
 {
     int			fd;
 
@@ -265,7 +322,9 @@ safe_fd(
 	     * descriptor, which in turn might be used as an index into
 	     * an array (e.g. an fd_set).
 	     */
-	    if (fd < fd_start || fd >= fd_start + fd_count) {
+	    if ((fd < fd_start || fd >= fd_start + fd_count) &&
+		(fd != fd1) &&
+		(fd != fd2)) {
 		close(fd);
 	    }
 	}
@@ -306,7 +365,7 @@ save_core(void)
 	ts = get_datestamp_from_time(sbuf.st_mtime);
         suffix[0] = 'z';
         suffix[1] = '\0';
-        old = vstralloc("core", ts, suffix, NULL);
+        old = g_strjoin(NULL, "core", ts, suffix, NULL);
         new = NULL;
         while(ts[0] != '\0') {
             amfree(new);
@@ -318,7 +377,7 @@ save_core(void)
             } else {
                 suffix[0]--;
             }
-            old = vstralloc("core", ts, suffix, NULL);
+            old = g_strjoin(NULL, "core", ts, suffix, NULL);
             (void)rename(old, new);         /* it either works ... */
         }
 	amfree(ts);
@@ -342,25 +401,8 @@ char *
 sanitise_filename(
     char *	inp)
 {
-    char *buf;
-    size_t buf_size;
-    char *s, *d;
-    int ch;
-
-    buf_size = strlen(inp) + 1;		/* worst case */
-    buf = alloc(buf_size);
-    d = buf;
-    s = inp;
-    while((ch = *s++) != '\0') {
-	if((ch == '/') || (ch == ':') || (ch == '\\')) {
-	    ch = '_';	/* convert "bad" to "_" */
-	}
-	*d++ = (char)ch;
-    }
-    assert(d < buf + buf_size);
-    *d = '\0';
-
-    return buf;
+    char *ret = g_strdup(inp);
+    return g_strdelimit(ret, "/:\\", '_');
 }
 
 /* duplicate '_' */
@@ -374,7 +416,7 @@ old_sanitise_filename(
     int ch;
 
     buf_size = 2*strlen(inp) + 1;		/* worst case */
-    buf = alloc(buf_size);
+    buf = g_malloc(buf_size);
     d = buf;
     s = inp;
     while((ch = *s++) != '\0') {
@@ -410,7 +452,7 @@ canonicalize_pathname(char *pathname, char *result_buf)
  * char *agets (FILE *stream)
  *
  * entry:	stream  -  stream to read
- * exit:	returns a pointer to an alloc'd string or NULL
+ * exit:	returns a pointer to an g_malloc'd string or NULL
  *		at EOF or error.  The functions ferror(stream) and
  *		feof(stream) should be checked by caller to determine
  *		stream status.
@@ -431,7 +473,7 @@ debug_agets(
     FILE *	stream)
 {
     int	ch;
-    char *line = alloc(AGETS_LINE_INCR);
+    char *line = g_malloc(AGETS_LINE_INCR);
     size_t line_size = 0;
     size_t loffset = 0;
     int	inquote = 0;
@@ -478,9 +520,9 @@ debug_agets(
 
 	    /*
 	     * Reallocate input line.
-	     * alloc() never return NULL pointer.
+	     * g_malloc() never return NULL pointer.
 	     */
-	    tmpline = alloc(line_size + AGETS_LINE_INCR);
+	    tmpline = g_malloc(line_size + AGETS_LINE_INCR);
 	    memcpy(tmpline, line, line_size);
 	    amfree(line);
 	    line = tmpline;
@@ -503,6 +545,59 @@ debug_agets(
 }
 
 
+char *
+debug_pgets(
+    const char *sourcefile,
+    int		lineno,
+    FILE *	stream)
+{
+    char *line = g_malloc(AGETS_LINE_INCR);
+    char *l;
+    char *cline;
+    char *untainted_line;
+    char *ul;
+    size_t line_size = AGETS_LINE_INCR;
+    size_t loffset = 0;
+
+    (void)sourcefile;	/* Quiet unused parameter warning if not debugging */
+    (void)lineno;	/* Quiet unused parameter warning if not debugging */
+    line[0] = '\0';
+
+    cline = fgets(line, line_size, stream);
+    if (!cline) {
+	g_free(line);
+	return cline;
+    }
+    loffset = strlen(line);
+    while (cline && loffset == line_size-1 && line[loffset-1] != '\n') {
+	char *tmpline;
+	char *pline;
+
+	line_size *= 2;
+	tmpline = g_malloc(line_size);
+	memcpy(tmpline, line, loffset+1);
+	amfree(line);
+	line = tmpline;
+
+	pline = line + loffset;
+	cline = fgets(pline, line_size-loffset, stream);
+	loffset += strlen(pline);
+    }
+
+    if (line[loffset-1] == '\n')
+	line[loffset-1] = '\0';
+
+    untainted_line = ul = g_malloc(loffset+1);
+    for (l = line; *l != '\0'; l++) {
+	*ul++ = *l;
+    }
+    *ul = '\0';
+    g_free(line);
+
+    return untainted_line;
+}
+
+
 /*
  *=====================================================================
  * Find/create a buffer for a particular file descriptor for use with
@@ -516,43 +611,54 @@ debug_agets(
  *=====================================================================
  */
 
+GMutex *file_mutex = NULL;
 static struct areads_buffer {
     char *buffer;
     char *endptr;
     size_t bufsize;
-} *areads_buffer = NULL;
+} **areads_buffer = NULL;
 static int areads_bufcount = 0;
 static size_t areads_bufsize = BUFSIZ;		/* for the test program */
 
-static void
+static struct areads_buffer *
 areads_getbuf(
-    const char *s,
-    int		l,
+    const char *s G_GNUC_UNUSED,
+    int		l G_GNUC_UNUSED,
     int		fd)
 {
-    struct areads_buffer *new;
-    size_t size;
+    struct areads_buffer *ptr;
 
     assert(fd >= 0);
-    if(fd >= areads_bufcount) {
-	size = (size_t)(fd + 1) * SIZEOF(*areads_buffer);
-	new = (struct areads_buffer *) debug_alloc(s, l, size);
-	memset((char *)new, 0, size);
-	if(areads_buffer) {
-	    size = areads_bufcount * SIZEOF(*areads_buffer);
+
+    g_mutex_lock(file_mutex);
+    if (fd >= areads_bufcount) {
+	struct areads_buffer **new;
+	int afd = 30;
+	int i;
+
+	if (afd < fd * 2)
+	    afd = fd * 2;
+	new = g_new0(struct areads_buffer *, (size_t)afd);
+	if (areads_buffer) {
+	    size_t size = areads_bufcount * sizeof(*areads_buffer);
 	    memcpy(new, areads_buffer, size);
 	}
+	for (i = areads_bufcount; i < afd; i++){
+	    new[i] = g_new0(struct areads_buffer, 1);
+	}
 	amfree(areads_buffer);
 	areads_buffer = new;
-	areads_bufcount = fd + 1;
+	areads_bufcount = afd;
     }
-    if(areads_buffer[fd].buffer == NULL) {
-	areads_buffer[fd].bufsize = areads_bufsize;
-	areads_buffer[fd].buffer = debug_alloc(s, l,
-					       areads_buffer[fd].bufsize + 1);
-	areads_buffer[fd].buffer[0] = '\0';
-	areads_buffer[fd].endptr = areads_buffer[fd].buffer;
+    ptr = areads_buffer[fd];
+    g_mutex_unlock(file_mutex);
+    if (ptr->buffer == NULL) {
+	ptr->bufsize = areads_bufsize;
+	ptr->buffer = g_malloc(ptr->bufsize + 1);
+	ptr->buffer[0] = '\0';
+	ptr->endptr = ptr->buffer;
     }
+    return ptr;
 }
 
 /*
@@ -571,11 +677,35 @@ areads_dataready(
     int	fd)
 {
     ssize_t r = 0;
+    SELECT_ARG_TYPE ready;
+    struct timeval  to;
+    int             nfound;
 
-    if(fd >= 0 && fd < areads_bufcount && areads_buffer[fd].buffer != NULL) {
-	r = (ssize_t) (areads_buffer[fd].endptr - areads_buffer[fd].buffer);
+    if (fd < 0)
+	return 0;
+
+    g_mutex_lock(file_mutex);
+
+    if (fd >= 0 && fd < areads_bufcount && areads_buffer[fd]->buffer != NULL) {
+	r = (ssize_t) (areads_buffer[fd]->endptr - areads_buffer[fd]->buffer);
     }
-    return r;
+    g_mutex_unlock(file_mutex);
+    if (r) {
+        return r;
+    }
+
+    FD_ZERO(&ready);
+    FD_SET(fd, &ready);
+    to.tv_sec = 0;
+    to.tv_usec = 0;
+
+    nfound = select(fd+1, &ready, NULL, NULL, &to);
+    if (nfound > 0 && FD_ISSET(fd, &ready)) {
+        return 1;
+    } else {
+	return 0;
+    }
+
 }
 
 /*
@@ -593,11 +723,13 @@ void
 areads_relbuf(
     int fd)
 {
+    g_mutex_lock(file_mutex);
     if(fd >= 0 && fd < areads_bufcount) {
-	amfree(areads_buffer[fd].buffer);
-	areads_buffer[fd].endptr = NULL;
-	areads_buffer[fd].bufsize = 0;
+	amfree(areads_buffer[fd]->buffer);
+	areads_buffer[fd]->endptr = NULL;
+	areads_buffer[fd]->bufsize = 0;
     }
+    g_mutex_unlock(file_mutex);
 }
 
 /*
@@ -607,7 +739,7 @@ areads_relbuf(
  * char *areads (int fd)
  *
  * entry:	fd = file descriptor to read
- * exit:	returns a pointer to an alloc'd string or NULL at EOF
+ * exit:	returns a pointer to an g_malloc'd string or NULL at EOF
  *		or error (errno will be zero on EOF).
  *
  * Notes:	the newline, if read, is removed from the string
@@ -629,51 +761,60 @@ debug_areads (
     size_t buflen;
     size_t size;
     ssize_t r;
+    struct areads_buffer *ptr;
 
     if(fd < 0) {
 	errno = EBADF;
 	return NULL;
     }
-    areads_getbuf(s, l, fd);
-    buffer = areads_buffer[fd].buffer;
-    endptr = areads_buffer[fd].endptr;
-    buflen = areads_buffer[fd].bufsize - (size_t)(endptr - buffer);
+    ptr = areads_getbuf(s, l, fd);
+
+    buffer = ptr->buffer;
+    endptr = ptr->endptr;
+    buflen = ptr->bufsize - (size_t)(endptr - buffer);
     while((nl = strchr(buffer, '\n')) == NULL) {
 	/*
 	 * No newline yet, so get more data.
 	 */
 	if (buflen == 0) {
-	    if ((size = areads_buffer[fd].bufsize) < 256 * areads_bufsize) {
+	    if ((size = ptr->bufsize) < 256 * areads_bufsize) {
 		size *= 2;
 	    } else {
 		size += 256 * areads_bufsize;
 	    }
-	    newbuf = debug_alloc(s, l, size + 1);
-	    memcpy (newbuf, buffer, areads_buffer[fd].bufsize + 1);
-	    amfree(areads_buffer[fd].buffer);
-	    areads_buffer[fd].buffer = newbuf;
-	    areads_buffer[fd].endptr = newbuf + areads_buffer[fd].bufsize;
-	    areads_buffer[fd].bufsize = size;
-	    buffer = areads_buffer[fd].buffer;
-	    endptr = areads_buffer[fd].endptr;
-	    buflen = areads_buffer[fd].bufsize - (size_t)(endptr - buffer);
+	    newbuf = g_malloc(size + 1);
+	    memcpy (newbuf, buffer, ptr->bufsize + 1);
+	    amfree(ptr->buffer);
+	    ptr->buffer = newbuf;
+	    ptr->endptr = newbuf + ptr->bufsize;
+	    ptr->bufsize = size;
+	    buffer = ptr->buffer;
+	    endptr = ptr->endptr;
+	    buflen = ptr->bufsize - (size_t)(endptr - buffer);
 	}
 	if ((r = read(fd, endptr, buflen)) <= 0) {
 	    if(r == 0) {
-		errno = 0;		/* flag EOF instead of error */
+		if (buffer == endptr || *(endptr-1) == '\n') {
+		    errno = 0;		/* flag EOF instead of error */
+		    return NULL;
+		}
+		*endptr = '\n';
+		r = 1;
+	    } else {
+		return NULL;
 	    }
-	    return NULL;
+	} else {
+	    endptr[r] = '\0';		/* we always leave room for this */
+	    endptr += r;
+	    buflen -= r;
 	}
-	endptr[r] = '\0';		/* we always leave room for this */
-	endptr += r;
-	buflen -= r;
     }
     *nl++ = '\0';
-    line = stralloc(buffer);
+    line = g_strdup(buffer);
     size = (size_t)(endptr - nl);	/* data still left in buffer */
     memmove(buffer, nl, size);
-    areads_buffer[fd].endptr = buffer + size;
-    areads_buffer[fd].endptr[0] = '\0';
+    ptr->endptr = buffer + size;
+    ptr->endptr[0] = '\0';
     return line;
 }
 
@@ -732,7 +873,7 @@ int robust_open(const char * pathname, int flags, mode_t mode) {
 
 #ifdef F_SETFD
     if (result >= 0) {
-        fcntl(result, F_SETFD, 1); /* Throw away result. */
+        (void)fcntl(result, F_SETFD, 1); /* Throw away result. */
     }
 #endif
 
@@ -803,6 +944,63 @@ get_original_cwd(void)
     return original_cwd;
 }
 
+/**
+ * Read up to "count" bytes from a file descriptor, optionally collecting the
+ * read operation status (0 on success, not 0 otherwise).
+ *
+ * This function exists to overcome the confusing behavior of full_read():
+ *
+ * - unlike read(2), full_read() does not return -1 on failure;
+ * - errno needs to be explicitly checked for each time the number of bytes
+ *   actually read is less than the "count" argument.
+ *
+ * With this function, a full read becomes a one time process. Error collecting
+ * is optional.
+ *
+ * @param fd: the file descriptor to read from
+ * @param buf: the buffer to write data into
+ * @param count: the number of bytes to write into the buffer
+ * @param err (output): 0 if "count" bytes have been read; errno (as set by
+ *                      full_read()) otherwise
+ * @returns: the number of bytes read from the file descriptor
+ */
+
+gsize read_fully(int fd, void *buf, gsize count, int *err)
+{
+    gsize ret = full_read(fd, buf, count);
+
+    if (err)
+        *err = (ret == count) ? 0 : errno;
+
+    return ret;
+}
+
+char *
+untaint_fgets(
+    char *s,
+    int size,
+    FILE *stream)
+{
+    char *untainted_line = malloc(size);
+    char *line = fgets(untainted_line, size, stream);
+    char *s1 = s;
+
+    if (!line) {
+	g_free(untainted_line);
+	return NULL;
+    }
+
+    while (*line != '\0') {
+	*s1 = *line;
+	line++;
+	s1++;
+    }
+    *s1 = '\0';
+    g_free(untainted_line);
+    return s;
+}
+
+
 #ifdef TEST
 
 int
@@ -817,6 +1015,8 @@ main(
 	char *file;
 	char *line;
 
+	glib_init();
+
 	/*
 	 * Configure program for internationalization:
 	 *   1) Only set the message locale for now.
