@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -34,6 +35,7 @@
 
 #include <glib.h>
 #include <stdio.h>
+#include "amutil.h"
 
 #define STRMAX		256
 
@@ -72,6 +74,9 @@ typedef struct file_s {
     int totalparts; /* -1 == UNKNOWN */
     size_t blocksize;
     off_t  orig_size;
+    crc_t  native_crc;
+    crc_t  client_crc;
+    crc_t  server_crc;
 } dumpfile_t;
 
 /* local functions */
