@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -25,7 +26,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: genversion.c 5713 2014-04-25 14:20:01Z martinea $
+ * $Id: genversion.c 7469 2017-11-08 17:08:47Z martinea $
  *
  * dump the current Amanda version info
  */
@@ -42,7 +43,11 @@
 
 static size_t linelen;
 
-#define	startline(title)  g_printf("  \"%-*s", LMARGIN, title); linelen = 0
+#define	startline(title)  do { \
+    g_printf("  \"%-*s", LMARGIN, title); \
+    linelen = 0; \
+} while (0)
+
 #define	endline()	  g_printf("\\n\",\n")
 
 static void prstr(const char *);
@@ -83,9 +88,9 @@ prvar(
     size_t new_len;
 
     new_len = strlen(var)
-	      + SIZEOF("=\\\"")
+	      + sizeof("=\\\"")
 	      + strlen(val)
-	      + SIZEOF("\\\"")
+	      + sizeof("\\\"")
 	      + 1;
     if(new_len > buf_len) {
 	free(buf);
@@ -109,7 +114,7 @@ prundefvar(
     size_t new_len;
 
     new_len = strlen(var)
-	      + SIZEOF("=UNDEF")
+	      + sizeof("=UNDEF")
 	      + 1;
     if(new_len > buf_len) {
 	free(buf);
@@ -134,9 +139,9 @@ prnum(
     static char number[NUM_STR_SIZE];
     size_t new_len;
 
-    g_snprintf(number, SIZEOF(number), "%ld", val);
+    g_snprintf(number, sizeof(number), "%ld", val);
     new_len = strlen(var)
-	      + SIZEOF("=")
+	      + sizeof("=")
 	      + strlen(number)
 	      + 1;
     if(new_len > buf_len) {
@@ -179,7 +184,7 @@ main(
 
     startline("build:");
     v = VERSION;
-    v_len = SIZEOF("Amanda-") + strlen(v) + 1;
+    v_len = sizeof("Amanda-") + strlen(v) + 1;
     verstr = malloc(v_len);
     if (!verstr) {
 	g_fprintf(stderr, _("genversion: Not enough memory"));
@@ -243,12 +248,6 @@ main(
     prundefvar("DEV_PREFIX");
 #endif
 
-#ifdef RDEV_PREFIX
-    prvar("RDEV_PREFIX", RDEV_PREFIX);
-#else
-    prundefvar("RDEV_PREFIX");
-#endif
-
 #ifdef DUMP
     prvar("DUMP", DUMP);
     prvar("RESTORE", RESTORE);
@@ -305,14 +304,14 @@ main(
     prundefvar("UNCOMPRESS_PATH");
 #endif
 
-#ifdef LPRCMD
-    prvar("LPRCMD", LPRCMD);
+#ifdef LPR
+    prvar("LPR", LPR);
 #else
-    prundefvar(" LPRCMD");
+    prundefvar(" LPR");
 #endif
 
-#ifdef MAILER
-    prvar("MAILER", MAILER);
+#ifdef DEFAULT_MAILER
+    prvar("MAILER", DEFAULT_MAILER);
 #else
     prundefvar(" MAILER");
 #endif
