@@ -34,7 +34,7 @@
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "event.h"
 #include "packet.h"
 #include "security.h"
@@ -58,7 +58,7 @@
 #ifdef HAVE_ON_EXIT
 #define atexit(func)    on_exit(func, 0)
 #else
-#define atexit(func)    (you must to resolve lack of atexit)
+#error "You don't have atexit() or on_exit()"
 #endif  /* HAVE_ON_EXIT */
 #endif  /* ! HAVE_ATEXIT */
 #endif
@@ -166,7 +166,7 @@ static int k5_encrypt(void *cookie, void *buf, ssize_t buflen,
 static int k5_decrypt(void *cookie, void *buf, ssize_t buflen,
 		      void **encbuf, ssize_t *encbuflen);
 
-static ssize_t krb5_tcpm_recv_token(struct tcp_conn *rc, int fd, int *handle,
+static ssize_t krb5_tcpm_recv_token(struct tcp_conn *rc, int *handle,
 				    char **errmsg, char **buf, ssize_t *size,
 				    int timeout);
 /*
@@ -185,15 +185,23 @@ const security_driver_t krb5_security_driver = {
     tcpma_stream_accept,
     tcpma_stream_client,
     tcpma_stream_close,
+    tcpma_stream_close_async,
     sec_stream_auth,
     sec_stream_id,
     tcpm_stream_write,
+    tcpm_stream_write_async,
     tcpm_stream_read,
     tcpm_stream_read_sync,
+    tcpm_stream_read_to_shm_ring,
     tcpm_stream_read_cancel,
+    tcpm_stream_pause,
+    tcpm_stream_resume,
     tcpm_close_connection,
     k5_encrypt,
     k5_decrypt,
+    generic_data_write,
+    generic_data_write_non_blocking,
+    generic_data_read
 };
 
 static int newhandle = 1;
@@ -229,8 +237,9 @@ krb5_connect(
 
     krb5_init();
 
-    rh = alloc(sizeof(*rh));
+    rh = g_malloc(sizeof(*rh));
     security_handleinit(&rh->sech, &krb5_security_driver);
+    rh->dle_hostname = g_strdup(hostname);
     rh->hostname = NULL;
     rh->rs = NULL;
     rh->ev_timeout = NULL;
@@ -239,7 +248,7 @@ krb5_connect(
     result = resolve_hostname(hostname, 0, NULL, &canonname);
     if(result != 0) {
 	dbprintf(_("resolve_hostname(%s): %s\n"), hostname, gai_strerror(result));
-	security_seterror(&rh->sech, _("resolve_hostname(%s): %s\n"), hostname,
+	security_seterror(&rh->sech, _("resolve_hostname(%s): %s"), hostname,
 			  gai_strerror(result));
 	(*fn)(arg, &rh->sech, S_ERROR);
 	return;
@@ -247,7 +256,7 @@ krb5_connect(
     if (canonname == NULL) {
 	dbprintf(_("resolve_hostname(%s) did not return a canonical name\n"), hostname);
 	security_seterror(&rh->sech,
-	        _("resolve_hostname(%s) did not return a canonical name\n"), hostname);
+	        _("resolve_hostname(%s) did not return a canonical name"), hostname);
 	(*fn)(arg, &rh->sech, S_ERROR);
        return;
     }
@@ -255,16 +264,20 @@ krb5_connect(
     rh->hostname = canonname;        /* will be replaced */
     canonname = NULL; /* steal reference */
     rh->rs = tcpma_stream_client(rh, newhandle++);
+    if (rh->rc == NULL)
+	goto error;
+
     rh->rc->conf_fn = conf_fn;
     rh->rc->datap = datap;
     rh->rc->recv_security_ok = NULL;
     rh->rc->prefix_packet = NULL;
+    rh->rc->need_priv_port = 0;
 
     if (rh->rs == NULL)
 	goto error;
 
     amfree(rh->hostname);
-    rh->hostname = stralloc(rh->rs->rc->hostname);
+    rh->hostname = g_strdup(rh->rs->rc->hostname);
 
 #ifdef AMANDA_KEYTAB
     keytab_name = AMANDA_KEYTAB;
@@ -300,12 +313,16 @@ krb5_connect(
      * Overload rh->rs->ev_read to provide a write event handle.
      * We also register a timeout.
      */
+    g_mutex_lock(security_mutex);
     rh->fn.connect = fn;
     rh->arg = arg;
-    rh->rs->ev_read = event_register((event_id_t)(rh->rs->rc->write),
+    rh->rs->rc->ev_write = event_create((event_id_t)(rh->rs->rc->write),
 	EV_WRITEFD, sec_connect_callback, rh);
-    rh->ev_timeout = event_register(CONNECT_TIMEOUT, EV_TIME,
+    rh->ev_timeout = event_create(CONNECT_TIMEOUT, EV_TIME,
 	sec_connect_timeout, rh);
+    event_activate(rh->rs->rc->ev_write);
+    event_activate(rh->ev_timeout);
+    g_mutex_unlock(security_mutex);
 
     amfree(canonname);
     return;
@@ -360,11 +377,12 @@ krb5_accept(
     }
 
 
-    rc = sec_tcp_conn_get(hostname, 0);
+    rc = sec_tcp_conn_get(NULL, hostname, 0);
     rc->conf_fn = conf_fn;
     rc->datap = datap;
     rc->recv_security_ok = NULL;
     rc->prefix_packet = NULL;
+    rc->need_priv_port = 0;
     copy_sockaddr(&rc->peer, &sin);
     rc->read = in;
     rc->write = out;
@@ -378,7 +396,10 @@ krb5_accept(
      *(making the userid equal to the dumpuser)
      */
     pw = getpwnam(CLIENT_LOGIN);
-    setreuid(pw->pw_uid, pw->pw_uid);
+    if (setreuid(pw->pw_uid, pw->pw_uid) == -1) {
+	g_critical("setreuid failed: %s", strerror(errno));
+	exit(1);
+    }
 }
 
 /*
@@ -389,16 +410,13 @@ static int
 runkrb5(
     struct sec_handle *	rh)
 {
-    struct servent *	sp;
     int			server_socket;
     in_port_t		my_port, port;
     struct tcp_conn *	rc = rh->rc;
     const char *err;
+    char *stream_msg = NULL;
 
-    if ((sp = getservbyname(AMANDA_KRB5_SERVICE_NAME, "tcp")) == NULL)
-	port = htons(AMANDA_KRB5_DEFAULT_PORT);
-    else
-	port = sp->s_port;
+    port = find_port_for_service(AMANDA_KRB5_SERVICE_NAME, "tcp");
 
     if ((err = get_tgt(keytab_name, principal_name)) != NULL) {
         security_seterror(&rh->sech, "%s: could not get TGT: %s",
@@ -406,19 +424,20 @@ runkrb5(
         return -1;
     }
 
-    set_root_privs(1);
-    server_socket = stream_client(rc->hostname,
-				     (in_port_t)(ntohs(port)),
+    server_socket = stream_client_privileged(NULL, rc->hostname,
+				     port,
 				     STREAM_BUFSIZE,
 				     STREAM_BUFSIZE,
 				     &my_port,
-				     0);
-    set_root_privs(0);
+				     0, &stream_msg);
 
-    if(server_socket < 0) {
-	security_seterror(&rh->sech,
-	    "%s", strerror(errno));
-	
+    if (stream_msg) {
+	security_seterror(&rh->sech, "%s", stream_msg);
+	g_free(stream_msg);
+	return -1;
+    }
+    if (server_socket < 0) {
+	security_seterror(&rh->sech, "%s", strerror(errno));
 	return -1;
     }
 
@@ -455,7 +474,7 @@ gss_client(
 
     auth_debug(1, "gss_client\n");
 
-    send_tok.value = vstralloc("host/", rs->rc->hostname, NULL);
+    send_tok.value = g_strjoin(NULL, "host/", rs->rc->hostname, NULL);
     send_tok.length = strlen(send_tok.value) + 1;
     maj_stat = gss_import_name(&min_stat, &send_tok, GSS_C_NULL_OID,
 	&gss_name);
@@ -514,7 +533,7 @@ gss_client(
 	/*
 	 * Send back the response
 	 */
-	if (send_tok.length != 0 && tcpm_send_token(rc, rc->write, rs->handle, &errmsg, send_tok.value, send_tok.length) < 0) {
+	if (send_tok.length != 0 && tcpm_send_token(rc, rs->handle, &errmsg, send_tok.value, send_tok.length) < 0) {
 	    security_seterror(&rh->sech, "%s", rc->errmsg);
 	    gss_release_buffer(&min_stat, &send_tok);
 	    goto done;
@@ -527,7 +546,7 @@ gss_client(
 	if (maj_stat != (OM_uint32)GSS_S_CONTINUE_NEEDED)
 	    break;
 
-        rvalue = krb5_tcpm_recv_token(rc, rc->read, &rc->handle,
+        rvalue = krb5_tcpm_recv_token(rc, &rc->handle,
 				 &rc->errmsg,
 				 (void *)&recv_tok.value,
 				 (ssize_t *)&recv_tok.length, 60);
@@ -576,13 +595,13 @@ gss_server(
      * gss_accept_context() thanks to the replay cache code.
      */
     if (!set_root_privs(0)) {
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("can't take root privileges to read krb5 host key: %s"), strerror(errno));
 	goto out;
     }
 
     rc->gss_context = GSS_C_NO_CONTEXT;
-    send_tok.value = vstralloc("host/", myhostname, NULL);
+    send_tok.value = g_strjoin(NULL, "host/", myhostname, NULL);
     send_tok.length = strlen(send_tok.value) + 1;
     for (p = send_tok.value; *p != '\0'; p++) {
 	if (isupper((int)*p))
@@ -592,7 +611,7 @@ gss_server(
 	&gss_name);
     if (maj_stat != (OM_uint32)GSS_S_COMPLETE) {
 	set_root_privs(0);
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("can't import name %s: %s"), (char *)send_tok.value,
 	    gss_error(maj_stat, min_stat));
 	amfree(send_tok.value);
@@ -605,7 +624,7 @@ gss_server(
     maj_stat = gss_acquire_cred(&min_stat, gss_name, 0,
 	GSS_C_NULL_OID_SET, GSS_C_ACCEPT, &gss_creds, NULL, NULL);
     if (maj_stat != (OM_uint32)GSS_S_COMPLETE) {
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("can't acquire creds for host key host/%s: %s"), myhostname,
 	    gss_error(maj_stat, min_stat));
 	gss_release_name(&min_stat, &gss_name);
@@ -616,18 +635,18 @@ gss_server(
 
     for (recv_tok.length = 0;;) {
 	recv_tok.value = NULL;
-        rvalue = krb5_tcpm_recv_token(rc, rc->read, &rc->handle,
+        rvalue = krb5_tcpm_recv_token(rc, &rc->handle,
 				 &rc->errmsg,
 				 /* (void *) is to avoid type-punning warning */
 				 (char **)(void *)&recv_tok.value,
 				 (ssize_t *)&recv_tok.length, 60);
 	if (rvalue <= 0) {
 	    if (rvalue < 0) {
-		g_snprintf(errbuf, SIZEOF(errbuf),
+		g_snprintf(errbuf, sizeof(errbuf),
 		    _("recv error in gss loop: %s"), rc->errmsg);
 		amfree(rc->errmsg);
 	    } else
-		g_snprintf(errbuf, SIZEOF(errbuf), _("EOF in gss loop"));
+		g_snprintf(errbuf, sizeof(errbuf), _("EOF in gss loop"));
 	    goto out;
 	}
 
@@ -637,16 +656,16 @@ gss_server(
 
 	if (maj_stat != (OM_uint32)GSS_S_COMPLETE &&
 	    maj_stat != (OM_uint32)GSS_S_CONTINUE_NEEDED) {
-	    g_snprintf(errbuf, SIZEOF(errbuf),
+	    g_snprintf(errbuf, sizeof(errbuf),
 		_("error accepting context: %s"), gss_error(maj_stat, min_stat));
 	    amfree(recv_tok.value);
 	    goto out;
 	}
 	amfree(recv_tok.value);
 
-	if (send_tok.length != 0 && tcpm_send_token(rc, rc->write, 0, &errmsg, send_tok.value, send_tok.length) < 0) {
-	    strncpy(errbuf, rc->errmsg, SIZEOF(errbuf) - 1);
-	    errbuf[SIZEOF(errbuf) - 1] = '\0';
+	if (send_tok.length != 0 && tcpm_send_token(rc, 0, &errmsg, send_tok.value, send_tok.length) < 0) {
+	    strncpy(errbuf, rc->errmsg, sizeof(errbuf) - 1);
+	    errbuf[sizeof(errbuf) - 1] = '\0';
 	    amfree(rc->errmsg);
 	    gss_release_buffer(&min_stat, &send_tok);
 	    goto out;
@@ -664,7 +683,7 @@ gss_server(
 
     maj_stat = gss_display_name(&min_stat, gss_name, &send_tok, &doid);
     if (maj_stat != (OM_uint32)GSS_S_COMPLETE) {
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("can't display gss name: %s"), gss_error(maj_stat, min_stat));
 	gss_release_name(&min_stat, &gss_name);
 	goto out;
@@ -673,7 +692,7 @@ gss_server(
 
     /* get rid of the realm */
     if ((p = strchr(send_tok.value, '@')) == NULL) {
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("malformed gss name: %s"), (char *)send_tok.value);
 	amfree(send_tok.value);
 	goto out;
@@ -685,7 +704,7 @@ gss_server(
      * If the principal doesn't match, complain
      */
     if ((msg = krb5_checkuser(rc->hostname, send_tok.value, realm)) != NULL) {
-	g_snprintf(errbuf, SIZEOF(errbuf),
+	g_snprintf(errbuf, sizeof(errbuf),
 	    _("access not allowed from %s: %s"), (char *)send_tok.value, msg);
 	amfree(send_tok.value);
 	goto out;
@@ -696,7 +715,7 @@ gss_server(
 out:
     set_root_privs(0);
     if (rval != 0) {
-	rc->errmsg = stralloc(errbuf);
+	rc->errmsg = g_strdup(errbuf);
     } else {
 	rc->auth = 1;
     }
@@ -719,7 +738,7 @@ krb5_init(void)
     beenhere = 1;
 
 #ifndef BROKEN_MEMORY_CCACHE
-    putenv(stralloc(KRB5_ENV_CCNAME"=MEMORY:amanda_ccache"));
+    putenv(g_strdup(KRB5_ENV_CCNAME"=MEMORY:amanda_ccache"));
 #else
     /*
      * MEMORY ccaches seem buggy and cause a lot of internal heap
@@ -731,17 +750,15 @@ krb5_init(void)
      */
     atexit(cleanup);
     {
-	char *ccache;
-	ccache = malloc(128);
-	g_snprintf(ccache, SIZEOF(ccache),
-		 KRB5_ENV_CCNAME"=FILE:/tmp/amanda_ccache.%ld.%ld",
-		 (long)geteuid(), (long)getpid());
+	char *ccache = g_strdup_printf(
+		KRB5_ENV_CCNAME"=FILE:/tmp/amanda_ccache.%ld.%ld",
+		(long)geteuid(), (long)getpid());
 	putenv(ccache);
     }
 #endif
 
-    gethostname(myhostname, SIZEOF(myhostname) - 1);
-    myhostname[SIZEOF(myhostname) - 1] = '\0';
+    gethostname(myhostname, sizeof(myhostname) - 1);
+    myhostname[sizeof(myhostname) - 1] = '\0';
 
     /*
      * In case it isn't fully qualified, do a DNS lookup.  Ignore
@@ -749,8 +766,8 @@ krb5_init(void)
      */
     if (resolve_hostname(myhostname, SOCK_STREAM, NULL, &myfqhostname) == 0
 	&& myfqhostname != NULL) {
-	strncpy(myhostname, myfqhostname, SIZEOF(myhostname)-1);
-	myhostname[SIZEOF(myhostname)-1] = '\0';
+	strncpy(myhostname, myfqhostname, sizeof(myhostname)-1);
+	myhostname[sizeof(myhostname)-1] = '\0';
 	amfree(myfqhostname);
     }
 
@@ -770,7 +787,7 @@ cleanup(void)
 {
 #ifdef KDESTROY_VIA_UNLINK
     char ccache[64];
-    g_snprintf(ccache, SIZEOF(ccache), "/tmp/amanda_ccache.%ld.%ld",
+    g_snprintf(ccache, sizeof(ccache), "/tmp/amanda_ccache.%ld.%ld",
         (long)geteuid(), (long)getpid());
     unlink(ccache);
 #else
@@ -806,7 +823,7 @@ get_tgt(
 	error = NULL;
     }
     if ((ret = krb5_init_context(&context)) != 0) {
-	error = vstrallocf(_("error initializing krb5 context: %s"),
+	error = g_strdup_printf(_("error initializing krb5 context: %s"),
 	    error_message(ret));
 	return (error);
     }
@@ -814,12 +831,12 @@ get_tgt(
     /*krb5_init_ets(context);*/
 
     if(!keytab_name) {
-        error = vstrallocf(_("error  -- no krb5 keytab defined"));
+        error = g_strdup(_("error  -- no krb5 keytab defined"));
         return(error);
     }
 
     if(!principal_name) {
-        error = vstrallocf(_("error  -- no krb5 principal defined"));
+        error = g_strdup(_("error  -- no krb5 principal defined"));
         return(error);
     }
 
@@ -827,7 +844,7 @@ get_tgt(
      * Resolve keytab file into a keytab object
      */
     if ((ret = krb5_kt_resolve(context, keytab_name, &keytab)) != 0) {
-	error = vstrallocf(_("error resolving keytab %s: %s"), keytab_name, 
+	error = g_strdup_printf(_("error resolving keytab %s: %s"), keytab_name,
 	    error_message(ret));
 	return (error);
     }
@@ -838,7 +855,7 @@ get_tgt(
      */
     ret = krb5_parse_name(context, principal_name, &client);
     if (ret != 0) {
-	error = vstrallocf(_("error parsing %s: %s"), principal_name,
+	error = g_strdup_printf(_("error parsing %s: %s"), principal_name,
 	    error_message(ret));
 	return (error);
     }
@@ -861,18 +878,18 @@ get_tgt(
 	0);
 #endif
     if (ret != 0) {
-	error = vstrallocf(_("error while building server name: %s"),
+	error = g_strdup_printf(_("error while building server name: %s"),
 	    error_message(ret));
 	return (error);
     }
 
     ret = krb5_timeofday(context, &now);
     if (ret != 0) {
-	error = vstrallocf(_("error getting time of day: %s"), error_message(ret));
+	error = g_strdup_printf(_("error getting time of day: %s"), error_message(ret));
 	return (error);
     }
 
-    memset(&creds, 0, SIZEOF(creds));
+    memset(&creds, 0, sizeof(creds));
     creds.times.starttime = 0;
     creds.times.endtime = now + AMANDA_TKT_LIFETIME;
 
@@ -886,21 +903,21 @@ get_tgt(
 	keytab, 0, &creds, 0);
 
     if (ret != 0) {
-	error = vstrallocf(_("error getting ticket for %s: %s"),
+	error = g_strdup_printf(_("error getting ticket for %s: %s"),
 	    principal_name, error_message(ret));
 	goto cleanup2;
     }
 
     if ((ret = krb5_cc_default(context, &ccache)) != 0) {
-	error = vstrallocf(_("error initializing ccache: %s"), error_message(ret));
+	error = g_strdup_printf(_("error initializing ccache: %s"), error_message(ret));
 	goto cleanup;
     }
     if ((ret = krb5_cc_initialize(context, ccache, client)) != 0) {
-	error = vstrallocf(_("error initializing ccache: %s"), error_message(ret));
+	error = g_strdup_printf(_("error initializing ccache: %s"), error_message(ret));
 	goto cleanup;
     }
     if ((ret = krb5_cc_store_cred(context, ccache, &creds)) != 0) {
-	error = vstrallocf(_("error storing creds in ccache: %s"), 
+	error = g_strdup_printf(_("error storing creds in ccache: %s"), 
 	    error_message(ret));
 	/* FALLTHROUGH */
     }
@@ -1066,10 +1083,10 @@ krb5_checkuser( char *	host,
     char *	realm)
 {
 #ifdef AMANDA_PRINCIPAL
-    if(strcmp(name, AMANDA_PRINCIPAL) == 0) {
+    if(g_str_equal(name, AMANDA_PRINCIPAL)) {
 	return(NULL);
     } else {
-	return(vstrallocf(_("does not match compiled in default")));
+	return(g_strdup(_("does not match compiled in default")));
     }
 #else
     struct passwd *pwd;
@@ -1085,62 +1102,60 @@ krb5_checkuser( char *	host,
     assert( name != NULL);
 
     if((pwd = getpwnam(CLIENT_LOGIN)) == NULL) {
-	result = vstrallocf(_("can not find user %s"), CLIENT_LOGIN);
+	result = g_strdup_printf(_("can not find user %s"), CLIENT_LOGIN);
+	return result;
     }
     localuid = pwd->pw_uid;
 
 #ifdef USE_AMANDAHOSTS
-    ptmp = stralloc2(pwd->pw_dir, "/.k5amandahosts");
+    ptmp = g_strconcat(pwd->pw_dir, "/.k5amandahosts", NULL);
 #else
-    ptmp = stralloc2(pwd->pw_dir, "/.k5login");
+    ptmp = g_strconcat(pwd->pw_dir, "/.k5login", NULL);
 #endif
 
     if(!ptmp) {
-	result = vstrallocf(_("could not find home directory for %s"), CLIENT_LOGIN);
-	goto common_exit;
-   }
-
-   /*
-    * check to see if the ptmp file does nto exist.
-    */
-   if(access(ptmp, R_OK) == -1 && errno == ENOENT) {
-	/*
-	 * in this case we check to see if the principal matches
-	 * the destination user mimicing the .k5login functionality.
-	 */
-	 if(strcmp(name, CLIENT_LOGIN) != 0) {
-		result = vstrallocf(_("%s does not match %s"),
-			name, CLIENT_LOGIN);
-		return result;
-	}
-	result = NULL;
+	result = g_strdup_printf(_("could not find home directory for %s"), CLIENT_LOGIN);
 	goto common_exit;
     }
 
     auth_debug(1, _("opening ptmp: %s\n"), (ptmp)?ptmp: "NULL!");
-    if((fp = fopen(ptmp, "r")) == NULL) {
-	result = vstrallocf(_("can not open %s"), ptmp);
+    if ((fp = fopen(ptmp, "r")) == NULL) {
+	/* check to see if the ptmp file does nto exist. */
+	if (errno == ENOENT) {
+	    /*
+	     * in this case we check to see if the principal matches
+	     * the destination user mimicing the .k5login functionality.
+	     */
+	     if (!g_str_equal(name, CLIENT_LOGIN)) {
+		result = g_strdup_printf(_("%s does not match %s"),
+					 name, CLIENT_LOGIN);
+		return result;
+	    }
+	    result = NULL;
+	    goto common_exit;
+	}
+	result = g_strdup_printf(_("can not open %s"), ptmp);
 	return result;
     }
     auth_debug(1, _("opened ptmp\n"));
 
     if (fstat(fileno(fp), &sbuf) != 0) {
-	result = vstrallocf(_("cannot fstat %s: %s"), ptmp, strerror(errno));
+	result = g_strdup_printf(_("cannot fstat %s: %s"), ptmp, strerror(errno));
 	goto common_exit;
     }
 
     if (sbuf.st_uid != localuid) {
-	result = vstrallocf(_("%s is owned by %ld, should be %ld"),
+	result = g_strdup_printf(_("%s is owned by %ld, should be %ld"),
 		ptmp, (long)sbuf.st_uid, (long)localuid);
 	goto common_exit;
     }
     if ((sbuf.st_mode & 077) != 0) {
-	result = vstrallocf(
+	result = g_strdup_printf(
 	    _("%s: incorrect permissions; file must be accessible only by its owner"), ptmp);
 	goto common_exit;
     }
 
-    while ((line = agets(fp)) != NULL) {
+    while ((line = pgets(fp)) != NULL) {
 	if (line[0] == '\0') {
 	    amfree(line);
 	    continue;
@@ -1161,7 +1176,7 @@ krb5_checkuser( char *	host,
 	    filehost = NULL;
 	}
 
-	if(filehost && strcmp(filehost, host) != 0) {
+	if(filehost && !g_str_equal(filehost, host)) {
 	    amfree(line);
 	    continue;
 	} else {
@@ -1181,9 +1196,9 @@ krb5_checkuser( char *	host,
 	 * anyway...
 	 */
 	auth_debug(1, _("comparing %s %s\n"), fileuser, name);
-	if(strcmp(fileuser, name) == 0) {
+	if(g_str_equal(fileuser, name)) {
 		auth_debug(1, _("found a match!\n"));
-		if(realm && filerealm && (strcmp(realm, filerealm)!=0)) {
+		if(realm && filerealm && (!g_str_equal(realm, filerealm))) {
 			amfree(line);
 			continue;
 		}
@@ -1193,7 +1208,7 @@ krb5_checkuser( char *	host,
 	}
 	amfree(line);
     }
-    result = vstrallocf(_("no match in %s"), ptmp);
+    result = g_strdup_printf(_("no match in %s"), ptmp);
 
 common_exit:
     afclose(fp);
@@ -1211,7 +1226,6 @@ common_exit:
 static ssize_t
 krb5_tcpm_recv_token(
     struct tcp_conn    *rc,
-    int		fd,
     int *	handle,
     char **	errmsg,
     char **	buf,
@@ -1220,19 +1234,22 @@ krb5_tcpm_recv_token(
 {
     unsigned int netint[2];
 
-    assert(SIZEOF(netint) == 8);
+    assert(sizeof(netint) == 8);
 
-    switch (net_read(fd, &netint, SIZEOF(netint), timeout)) {
+    switch (net_read(rc->read, &netint, sizeof(netint), timeout)) {
     case -1:
-	if (errmsg)
-	    *errmsg = newvstrallocf(*errmsg, _("recv error: %s"), strerror(errno));
+	if (errmsg) {
+	    g_free(*errmsg);
+	    *errmsg = g_strdup_printf(_("recv error: %s"), strerror(errno));
+	}
 	auth_debug(1, _("krb5_tcpm_recv_token: A return(-1)\n"));
 	return (-1);
     case 0:
 	*size = 0;
 	*handle = H_EOF;
-	*errmsg = newvstrallocf(*errmsg, _("SOCKET_EOF"));
-	auth_debug(1, _("krb5_tcpm_recv_token: A return(0)\n"));
+	g_free(*errmsg);
+	*errmsg = g_strdup("SOCKET_EOF");
+	auth_debug(1, "krb5_tcpm_recv_token: A return(0)\n");
 	return (0);
     default:
 	break;
@@ -1262,7 +1279,7 @@ krb5_tcpm_recv_token(
 	    s[7] = (*handle      ) & 0xFF;
 	    i = 8; s[i] = ' ';
 	    while(i<100 && isprint((int)s[i]) && s[i] != '\n') {
-		switch(net_read(fd, &s[i], 1, 0)) {
+		switch(net_read(rc->read, &s[i], 1, 0)) {
 		case -1: s[i] = '\0'; break;
 		case  0: s[i] = '\0'; break;
 		default:
@@ -1271,33 +1288,40 @@ krb5_tcpm_recv_token(
 		}
 	    }
 	    s[i] = '\0';
-	    *errmsg = newvstrallocf(*errmsg, _("krb5_tcpm_recv_token: invalid size: %s"), s);
+	    g_free(*errmsg);
+	    *errmsg = g_strdup_printf(_("krb5_tcpm_recv_token: invalid size: %s"),
+                                      s);
 	    dbprintf(_("krb5_tcpm_recv_token: invalid size %s\n"), s);
 	} else {
-	    *errmsg = newvstrallocf(*errmsg, _("krb5_tcpm_recv_token: invalid size"));
-	    dbprintf(_("krb5_tcpm_recv_token: invalid size %zd\n"), *size);
+	    g_free(*errmsg);
+	    *errmsg = g_strdup("krb5_tcpm_recv_token: invalid size");
+	    dbprintf("krb5_tcpm_recv_token: invalid size %zd\n", *size);
 	}
 	*size = -1;
 	return -1;
     }
     amfree(*buf);
-    *buf = alloc((size_t)*size);
+    *buf = g_malloc((size_t)*size);
 
     if(*size == 0) {
-	auth_debug(1, _("krb5_tcpm_recv_token: read EOF from %d\n"), *handle);
-	*errmsg = newvstrallocf(*errmsg, _("EOF"));
+	auth_debug(1, "krb5_tcpm_recv_token: read EOF from %d\n", *handle);
+	g_free(*errmsg);
+	*errmsg = g_strdup("EOF");
 	return 0;
     }
-    switch (net_read(fd, *buf, (size_t)*size, timeout)) {
+    switch (net_read(rc->read, *buf, (size_t)*size, timeout)) {
     case -1:
-	if (errmsg)
-	    *errmsg = newvstrallocf(*errmsg, _("recv error: %s"), strerror(errno));
+	if (errmsg) {
+	    g_free(*errmsg);
+	    *errmsg = g_strdup_printf(_("recv error: %s"), strerror(errno));
+	}
 	auth_debug(1, _("krb5_tcpm_recv_token: B return(-1)\n"));
 	return (-1);
     case 0:
 	*size = 0;
-	*errmsg = newvstrallocf(*errmsg, _("SOCKET_EOF"));
-	auth_debug(1, _("krb5_tcpm_recv_token: B return(0)\n"));
+	g_free(*errmsg);
+	*errmsg = g_strdup("SOCKET_EOF");
+	auth_debug(1, "krb5_tcpm_recv_token: B return(0)\n");
 	return (0);
     default:
 	break;
