@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -172,7 +173,7 @@ test_match(void)
 	matched = match_no_newline(t->expr, t->str);
 	if (!!matched != !!t->should_match_no_newline) {
 	    ok = FALSE;
-	    if (t->should_match) {
+	    if (t->should_match_no_newline) {
 		g_fprintf(stderr, "%s should have matched (no_newline) regular expr %s\n",
 			t->str, t->expr);
 	    } else {
@@ -244,7 +245,7 @@ test_glob_to_regex(void)
 
     for (t = tests; t->glob; t++) {
 	char *regex = glob_to_regex(t->glob);
-	if (0 != strcmp(regex, t->regex)) {
+	if (!g_str_equal(regex, t->regex)) {
 	    ok = FALSE;
 	    g_fprintf(stderr, "glob_to_regex(\"%s\") returned \"%s\"; expected \"%s\"\n",
 		    t->glob, regex, t->regex);
@@ -488,6 +489,7 @@ test_make_exact_host_expression(void)
 			    expr, test_strs[i], test_strs[j]);
 		}
 	    }
+	    amfree(expr);
 	}
     }
 
@@ -644,6 +646,7 @@ test_make_exact_disk_expression(void)
 			    expr, test_strs[i], test_strs[j]);
 		}
 	    }
+	    amfree(expr);
 	}
     }
 
@@ -988,6 +991,8 @@ main(int argc, char **argv)
 	TU_END()
     };
 
+    glib_init();
+
     return testutils_run_tests(argc, argv, tests);
 }
 
