@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -30,7 +31,6 @@
  * Routines for modifying the amanda protocol packet type
  */
 #include "amanda.h"
-#include "arglist.h"
 #include "packet.h"
 
 /*
@@ -46,7 +46,7 @@ static const struct {
     { "ACK", P_ACK },
     { "NAK", P_NAK }
 };
-#define	NPKTYPES	(int)(SIZEOF(pktypes) / SIZEOF(pktypes[0]))
+#define NPKTYPES G_N_ELEMENTS(pktypes)
 
 /*
  * Initialize a packet
@@ -56,29 +56,28 @@ void pkt_init_empty(
     pktype_t type)
 {
     assert(pkt != NULL);
-    assert(strcmp(pkt_type2str(type), "BOGUS") != 0);
+    assert(!g_str_equal(pkt_type2str(type), "BOGUS"));
 
     pkt->type = type;
     pkt->packet_size = 1000;
-    pkt->body = alloc(pkt->packet_size);
+    pkt->body = g_malloc(pkt->packet_size);
     pkt->body[0] = '\0';
     pkt->size = strlen(pkt->body);
 }
 
-printf_arglist_function2(void pkt_init, pkt_t *, pkt, pktype_t, type,
-    const char *, fmt)
+void pkt_init(pkt_t *pkt, pktype_t type, const char *fmt, ...)
 {
     va_list	argp;
     int         len;
 
     assert(pkt != NULL);
-    assert(strcmp(pkt_type2str(type), "BOGUS") != 0);
+    assert(!g_str_equal(pkt_type2str(type), "BOGUS"));
     if(fmt == NULL)
 	fmt = "";
 
     pkt->type = type;
     pkt->packet_size = 1000;
-    pkt->body = alloc(pkt->packet_size);
+    pkt->body = g_malloc(pkt->packet_size);
     while(1) {
 	arglist_start(argp, fmt);
 	len = g_vsnprintf(pkt->body, pkt->packet_size, fmt, argp);
@@ -87,7 +86,7 @@ printf_arglist_function2(void pkt_init, pkt_t *, pkt, pktype_t, type,
 	    break;
 	pkt->packet_size *= 2;
 	amfree(pkt->body);
-	pkt->body = alloc(pkt->packet_size);
+	pkt->body = g_malloc(pkt->packet_size);
     }
     pkt->size = strlen(pkt->body);
 }
@@ -95,7 +94,7 @@ printf_arglist_function2(void pkt_init, pkt_t *, pkt, pktype_t, type,
 /*
  * Append data to a packet
  */
-printf_arglist_function1(void pkt_cat, pkt_t *, pkt, const char *, fmt)
+void pkt_cat(pkt_t *pkt, const char *fmt, ...)
 {
     size_t	len;
     int         lenX;
@@ -114,7 +113,7 @@ printf_arglist_function1(void pkt_cat, pkt_t *, pkt, const char *, fmt)
 	if (lenX > -1 && lenX < (int)(pkt->packet_size - len - 1))
 	    break;
 	pkt->packet_size *= 2;
-	pktbody = alloc(pkt->packet_size);
+	pktbody = g_malloc(pkt->packet_size);
 	strncpy(pktbody, pkt->body, len);
 	pktbody[len] = '\0';
 	amfree(pkt->body);
@@ -130,12 +129,12 @@ pktype_t
 pkt_str2type(
     const char *typestr)
 {
-    int i;
+    guint i;
 
     assert(typestr != NULL);
 
-    for (i = 0; i < (int)NPKTYPES; i++)
-	if (strcmp(typestr, pktypes[i].name) == 0)
+    for (i = 0; i < NPKTYPES; i++)
+	if (g_str_equal(typestr, pktypes[i].name))
 	    return (pktypes[i].type);
     return ((pktype_t)-1);
 }
@@ -147,9 +146,9 @@ const char *
 pkt_type2str(
     pktype_t	type)
 {
-    int i;
+    guint i;
 
-    for (i = 0; i < (int)NPKTYPES; i++)
+    for (i = 0; i < NPKTYPES; i++)
 	if (pktypes[i].type == type)
 	    return (pktypes[i].name);
     return ("BOGUS");
