@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -17,9 +17,11 @@
 /*
  */
 #include "amanda.h"
+#include "ammessage.h"
 
-gboolean check_security_file_permission(FILE *verbose);
+message_t *check_security_file_permission_message(void);
 
-gboolean security_allow_program_as_root(char *name, char *path, FILE *verbose);
-gboolean security_allow_to_restore(FILE *verbose);
+message_t *security_allow_program_as_root(char *name, char *path);
+gboolean security_allow_to_restore(void);
+gboolean security_allow_bind(int s, sockaddr_union *addr);
 
