@@ -34,7 +34,7 @@
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "event.h"
 #include "packet.h"
 #include "security.h"
@@ -80,15 +80,23 @@ const security_driver_t ssh_security_driver = {
     tcpma_stream_accept,
     tcpma_stream_client,
     tcpma_stream_close,
+    tcpma_stream_close_async,
     sec_stream_auth,
     sec_stream_id,
     tcpm_stream_write,
+    tcpm_stream_write_async,
     tcpm_stream_read,
     tcpm_stream_read_sync,
+    tcpm_stream_read_to_shm_ring,
     tcpm_stream_read_cancel,
+    tcpm_stream_pause,
+    tcpm_stream_resume,
     tcpm_close_connection,
     NULL,
-    NULL
+    NULL,
+    generic_data_write,
+    generic_data_write_non_blocking,
+    generic_data_read
 };
 
 static int newhandle = 1;
@@ -122,6 +130,7 @@ ssh_connect(
 
     rh = g_new0(struct sec_handle, 1);
     security_handleinit(&rh->sech, &ssh_security_driver);
+    rh->dle_hostname = g_strdup(hostname);
     rh->hostname = NULL;
     rh->rs = NULL;
     rh->ev_timeout = NULL;
@@ -129,6 +138,8 @@ ssh_connect(
 
     rh->hostname = g_strdup(hostname);
     rh->rs = tcpma_stream_client(rh, newhandle++);
+    if (rh->rc == NULL)
+	goto error;
     rh->rc->conf_fn = conf_fn;
     rh->rc->datap = datap;
 
@@ -136,7 +147,7 @@ ssh_connect(
 	goto error;
 
     amfree(rh->hostname);
-    rh->hostname = stralloc(rh->rs->rc->hostname);
+    rh->hostname = g_strdup(rh->rs->rc->hostname);
 
     /*
      * We need to open a new connection.
@@ -171,12 +182,16 @@ ssh_connect(
      * Overload rh->rs->ev_read to provide a write event handle.
      * We also register a timeout.
      */
+    g_mutex_lock(security_mutex);
     rh->fn.connect = fn;
     rh->arg = arg;
-    rh->rs->ev_read = event_register((event_id_t)rh->rs->rc->write, EV_WRITEFD,
+    rh->rs->rc->ev_write = event_create((event_id_t)rh->rs->rc->write, EV_WRITEFD,
 	sec_connect_callback, rh);
-    rh->ev_timeout = event_register((event_id_t)CONNECT_TIMEOUT, EV_TIME,
+    rh->ev_timeout = event_create((event_id_t)CONNECT_TIMEOUT, EV_TIME,
 	sec_connect_timeout, rh);
+    event_activate(rh->rs->rc->ev_write);
+    event_activate(rh->ev_timeout);
+    g_mutex_unlock(security_mutex);
 
     return;
 
@@ -195,7 +210,7 @@ ssh_accept(
     void       *datap)
 {
     struct sec_handle *rh;
-    struct tcp_conn *rc = sec_tcp_conn_get("", 0);
+    struct tcp_conn *rc = sec_tcp_conn_get(NULL, "", 0);
     char *ssh_connection, *p;
     char *errmsg = NULL;
     sockaddr_union addr;
@@ -253,8 +268,7 @@ ssh_accept(
     }
 
 done:
-    if (ssh_connection)
-	g_free(ssh_connection);
+    g_free(ssh_connection);
 
     rc->read = in;
     rc->write = out;
@@ -277,6 +291,29 @@ error:
     (*fn)(&rh->sech, NULL);
 }
 
+static void
+ssh_child_watch_callback(
+    pid_t pid,
+    gint status,
+    gpointer data)
+{
+    struct tcp_conn *rc = (struct tcp_conn *)data;
+
+    if (pid != rc->pid)
+	return;
+
+    g_assert(pid == rc->pid);
+    rc->pid = -1 ; /* it's gone now.. */
+
+    if (WIFEXITED(status)) {
+	int exitcode = WEXITSTATUS(status);
+	g_debug("ssh exited with status %d", exitcode);
+    } else if (WIFSIGNALED(status)) {
+	int signal = WTERMSIG(status);
+	g_debug("ssh died on signal %d", signal);
+    }
+}
+
 /*
  * Forks a ssh to the host listed in rc->hostname
  * Returns negative on error, with an errmsg in rc->errmsg.
@@ -299,15 +336,16 @@ runssh(
     gchar **ssh_option;
     gchar *cmd;
 
-    memset(rpipe, -1, SIZEOF(rpipe));
-    memset(wpipe, -1, SIZEOF(wpipe));
+    memset(rpipe, -1, sizeof(rpipe));
+    memset(wpipe, -1, sizeof(wpipe));
     if (pipe(rpipe) < 0 || pipe(wpipe) < 0) {
-	rc->errmsg = newvstrallocf(rc->errmsg, _("pipe: %s"), strerror(errno));
+	g_free(rc->errmsg);
+	rc->errmsg = g_strdup_printf(_("pipe: %s"), strerror(errno));
 	return (-1);
     }
 
     if(!xamandad_path || strlen(xamandad_path) <= 1) 
-	xamandad_path = vstralloc(amlibexecdir, "/", "amandad", NULL);
+	xamandad_path = g_strjoin(NULL, amlibexecdir, "/", "amandad", NULL);
     if(!xclient_username || strlen(xclient_username) <= 1)
 	xclient_username = CLIENT_LOGIN;
     if(!xclient_port || strlen(xclient_port) <= 1)
@@ -339,7 +377,8 @@ runssh(
 
     switch (rc->pid = fork()) {
     case -1:
-	rc->errmsg = newvstrallocf(rc->errmsg, _("fork: %s"), strerror(errno));
+	g_free(rc->errmsg);
+	rc->errmsg = g_strdup_printf(_("fork: %s"), strerror(errno));
 	aclose(rpipe[0]);
 	aclose(rpipe[1]);
 	aclose(wpipe[0]);
@@ -354,6 +393,12 @@ runssh(
 	aclose(rpipe[1]);
 	rc->write = wpipe[1];
 	aclose(wpipe[0]);
+	rc->child_watch = new_child_watch_source(rc->pid);
+	g_source_set_callback(rc->child_watch,
+            (GSourceFunc)ssh_child_watch_callback, rc, NULL);
+	g_source_attach(rc->child_watch, NULL);
+	g_source_unref(rc->child_watch);
+
 	return (0);
     }
 
