@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -33,16 +34,17 @@
 #include "amanda.h"
 #include "dgram.h"
 #include "stream.h"
-#include "util.h"
+#include "amutil.h"
 #include "conffile.h"
 #include "security-util.h"
 #include "sockaddr-util.h"
 
 /* local functions */
 static void try_socksize(int sock, int which, size_t size);
-static int stream_client_internal(const char *hostname, in_port_t port,
+static int stream_client_internal(const char *src_ip,
+		const char *hostname, in_port_t port,
 		size_t sendsize, size_t recvsize, in_port_t *localport,
-		int nonblock, int priv);
+		int nonblock, int priv, char **stream_msg);
 
 int
 stream_server(
@@ -53,6 +55,7 @@ stream_server(
     int    priv)
 {
     int server_socket, retries;
+    int new_s;
     socklen_t_equiv len;
 #if defined(SO_KEEPALIVE) || defined(USE_REUSEADDR)
     const int on = 1;
@@ -63,6 +66,7 @@ stream_server(
     int *portrange;
     socklen_t_equiv socklen;
     int socket_family;
+    char *bind_msg = NULL;
 
     *portp = USHRT_MAX;				/* in case we error exit */
     if (family == -1) {
@@ -125,6 +129,7 @@ stream_server(
      * is within the range it requires.
      */
     for (retries = 0; ; retries++) {
+	amfree(bind_msg);
 	if (priv) {
 	    portrange = getconf_intrange(CNF_RESERVED_TCP_PORT);
 	} else {
@@ -132,13 +137,17 @@ stream_server(
 	}
 
 	if (portrange[0] != 0 && portrange[1] != 0) {
-	    if (bind_portrange(server_socket, &server, (in_port_t)portrange[0],
-			       (in_port_t)portrange[1], "tcp") == 0)
+	    if ((new_s = bind_portrange(server_socket, &server, (in_port_t)portrange[0],
+			       (in_port_t)portrange[1], "tcp", priv, &bind_msg)) >= 0)
 		goto out;
-	    g_debug(_("stream_server: Could not bind to port in range: %d - %d."),
-		      portrange[0], portrange[1]);
+	    g_debug("stream_server: Could not bind to port in range: %d - %d: %s",
+		      portrange[0], portrange[1], bind_msg);
+	    if (new_s == -1) {
+		break;
+	    }
 	} else {
 	    socklen = SS_LEN(&server);
+	    new_s = server_socket;
 	    if (bind(server_socket, (struct sockaddr *)&server, socklen) == 0)
 		goto out;
 	    g_debug(_("stream_server: Could not bind to any port: %s"),
@@ -148,41 +157,52 @@ stream_server(
 	if (retries >= BIND_CYCLE_RETRIES)
 	    break;
 
-	g_debug(_("stream_server: Retrying entire range after 10 second delay."));
+	g_debug(_("stream_server: Retrying entire range after 15 second delay."));
 
 	sleep(15);
     }
 
     save_errno = errno;
     g_debug(_("stream_server: bind(in6addr_any) failed: %s"),
-		  strerror(save_errno));
+		  bind_msg);
+    g_free(bind_msg);
     aclose(server_socket);
     errno = save_errno;
     return -1;
 
 out:
-    listen(server_socket, 1);
+    if (server_socket != new_s)
+	aclose(server_socket);
+
+    if (listen(new_s, 1) == -1) {
+	save_errno = errno;
+	g_debug(_("stream_server: listen() failed: %s"),
+		  strerror(save_errno));
+	aclose(new_s);
+	errno = save_errno;
+	return -1;
+    }
 
     /* find out what port was actually used */
 
-    len = SIZEOF(server);
-    if(getsockname(server_socket, (struct sockaddr *)&server, &len) == -1) {
+    len = sizeof(server);
+    if(getsockname(new_s, (struct sockaddr *)&server, &len) == -1) {
 	save_errno = errno;
 	g_debug(_("stream_server: getsockname() failed: %s"),
 		  strerror(save_errno));
-	aclose(server_socket);
+	aclose(new_s);
 	errno = save_errno;
 	return -1;
     }
 
 #ifdef SO_KEEPALIVE
-    r = setsockopt(server_socket, SOL_SOCKET, SO_KEEPALIVE,
+    r = setsockopt(new_s, SOL_SOCKET, SO_KEEPALIVE,
 	(void *)&on, (socklen_t_equiv)sizeof(on));
     if(r == -1) {
 	save_errno = errno;
 	g_debug(_("stream_server: setsockopt(SO_KEEPALIVE) failed: %s"),
 		  strerror(save_errno));
-        aclose(server_socket);
+        aclose(new_s);
 	errno = save_errno;
         return -1;
     }
@@ -191,18 +211,20 @@ out:
     *portp = SU_GET_PORT(&server);
     g_debug(_("stream_server: waiting for connection: %s"),
 	      str_sockaddr(&server));
-    return server_socket;
+    return new_s;
 }
 
 int
 stream_client_addr(
+    const char *src_ip,
     struct addrinfo *res,
     in_port_t port,
     size_t sendsize,
     size_t recvsize,
     in_port_t *localport,
     int nonblock,
-    int priv)
+    int priv,
+    char **stream_msg)
 {
     sockaddr_union svaddr, claddr;
     int save_errno = 0;
@@ -215,6 +237,9 @@ stream_client_addr(
 
     SU_INIT(&claddr, SU_GET_FAMILY(&svaddr));
     SU_SET_INADDR_ANY(&claddr);
+    if (src_ip) {
+	SU_SET_INADDR(&claddr, src_ip);
+    }
 
     /*
      * If a privileged port range was requested, we try to get a port in
@@ -233,7 +258,7 @@ stream_client_addr(
     }
     client_socket = connect_portrange(&claddr, (in_port_t)portrange[0],
 				      (in_port_t)portrange[1],
-				      "tcp", &svaddr, nonblock);
+				      "tcp", &svaddr, nonblock, priv, stream_msg);
     save_errno = errno;
 
     if (client_socket < 0) {
@@ -253,13 +278,15 @@ stream_client_addr(
 
 static int
 stream_client_internal(
+    const char *src_ip,
     const char *hostname,
     in_port_t port,
     size_t sendsize,
     size_t recvsize,
     in_port_t *localport,
     int nonblock,
-    int priv)
+    int priv,
+    char **stream_msg)
 {
     sockaddr_union svaddr, claddr;
     int save_errno = 0;
@@ -287,6 +314,9 @@ stream_client_internal(
 
 	SU_INIT(&claddr, SU_GET_FAMILY(&svaddr));
 	SU_SET_INADDR_ANY(&claddr);
+	if (src_ip) {
+	    SU_SET_INADDR(&claddr, src_ip);
+	}
 
 	/*
 	 * If a privileged port range was requested, we try to get a port in
@@ -305,15 +335,20 @@ stream_client_internal(
 	}
 	client_socket = connect_portrange(&claddr, (in_port_t)portrange[0],
 					  (in_port_t)portrange[1],
-					  "tcp", &svaddr, nonblock);
+					  "tcp", &svaddr, nonblock, priv, stream_msg);
 	save_errno = errno;
-	if (client_socket > 0)
+	if (*stream_msg) {
+	    aclose(client_socket);
+	    client_socket = -1;
+	    break;
+	}
+	if (client_socket >= 0)
 	    break;
     }
 
     freeaddrinfo(res);
-					  
-    if (client_socket > 0)
+
+    if (client_socket >= 0)
 	goto out;
 
     g_debug(_("stream_client: Could not bind to port in range %d-%d."),
@@ -332,38 +367,44 @@ out:
 
 int
 stream_client_privileged(
+    const char *src_ip,
     const char *hostname,
     in_port_t port,
     size_t sendsize,
     size_t recvsize,
     in_port_t *localport,
-    int nonblock)
+    int nonblock,
+    char **stream_msg)
 {
-    return stream_client_internal(hostname,
+    return stream_client_internal(src_ip,
+				  hostname,
 				  port,
 				  sendsize,
 				  recvsize,
 				  localport,
 				  nonblock,
-				  1);
+				  1, stream_msg);
 }
 
 int
 stream_client(
+    const char *src_ip,
     const char *hostname,
     in_port_t port,
     size_t sendsize,
     size_t recvsize,
     in_port_t *localport,
-    int nonblock)
+    int nonblock,
+    char **stream_msg)
 {
-    return stream_client_internal(hostname,
+    return stream_client_internal(src_ip,
+				  hostname,
 				  port,
 				  sendsize,
 				  recvsize,
 				  localport,
 				  nonblock,
-				  0);
+				  0, stream_msg);
 }
 
 /* don't care about these values */
