@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -61,6 +62,11 @@ dump_tapelist(
     dbprintf("dump_tapelist(%p):\n", tapelist);
     for(cur_tape = tapelist ; cur_tape != NULL ; cur_tape = cur_tape->next) {
 	dbprintf("  %p->next     = %p\n", cur_tape, cur_tape->next);
+	if (cur_tape->storage) {
+	    dbprintf("  %p->storage  = %s\n", cur_tape, cur_tape->storage);
+	} else {
+	    dbprintf("  %p->storage  = %s\n", cur_tape, "NULL");
+	}
 	dbprintf("  %p->label    = %s\n", cur_tape, cur_tape->label);
 	dbprintf("  %p->isafile  = %d\n", cur_tape, cur_tape->isafile);
 	dbprintf("  %p->numfiles = %d\n", cur_tape, cur_tape->numfiles);
@@ -81,6 +87,7 @@ dump_tapelist(
 tapelist_t *
 append_to_tapelist(
     tapelist_t *tapelist,
+    char *	storage,
     char *	label,
     off_t	file,
     int 	partnum,
@@ -89,20 +96,27 @@ append_to_tapelist(
     tapelist_t *new_tape, *cur_tape;
     int c;
 
-    dbprintf("append_to_tapelist(tapelist=%p, label='%s', file=%lld, partnum=%d,  isafile=%d)\n",
-		tapelist, label, (long long)file, partnum, isafile);
+    if (storage) {
+	dbprintf("append_to_tapelist(tapelist=%p, storage='%s', label='%s', file=%lld, partnum=%d,  isafile=%d)\n",
+		 tapelist, storage, label, (long long)file, partnum, isafile);
+    } else {
+	dbprintf("append_to_tapelist(tapelist=%p, storage='%s', label='%s', file=%lld, partnum=%d,  isafile=%d)\n",
+		 tapelist, "NULL", label, (long long)file, partnum, isafile);
+    }
 
     /* see if we have this tape already, and if so just add to its file list */
     for(cur_tape = tapelist; cur_tape; cur_tape = cur_tape->next) {
-	if(strcmp(label, cur_tape->label) == 0) {
+	if ((!storage || cur_tape->storage ||
+	     g_str_equal(storage, cur_tape->storage)) &&
+	    g_str_equal(label, cur_tape->label)) {
 	    int d_idx = 0;
 	    off_t *newfiles;
 	    int   *newpartnum;
 
 	    if(file >= (off_t)0) {
-		newfiles = alloc(SIZEOF(*newfiles) *
+		newfiles = g_malloc(sizeof(*newfiles) *
 				 (cur_tape->numfiles + 1));
-		newpartnum = alloc(SIZEOF(*newpartnum) *
+		newpartnum = g_malloc(sizeof(*newpartnum) *
 				 (cur_tape->numfiles + 1));
 		for(c = 0; c < cur_tape->numfiles ; c++) {
 		    if(cur_tape->files[c] > file && c == d_idx) {
@@ -128,13 +142,13 @@ append_to_tapelist(
 	}
     }
 
-    new_tape = alloc(SIZEOF(tapelist_t));
-    memset(new_tape, 0, SIZEOF(tapelist_t));
-    new_tape->label = stralloc(label);
+    new_tape = g_new0(tapelist_t, 1);
+    new_tape->storage = g_strdup(storage);
+    new_tape->label = g_strdup(label);
     if(file >= (off_t)0){
-	new_tape->files = alloc(SIZEOF(*(new_tape->files)));
+	new_tape->files = g_malloc(sizeof(*(new_tape->files)));
 	new_tape->files[0] = file;
-	new_tape->partnum = alloc(SIZEOF(*(new_tape->partnum)));
+	new_tape->partnum = g_malloc(sizeof(*(new_tape->partnum)));
 	new_tape->partnum[0] = partnum;
 	new_tape->numfiles = 1;
 	new_tape->isafile = isafile;
@@ -166,7 +180,7 @@ escape_label(
 
     if(!label) return(NULL);
 
-    temp_str = alloc(strlen(label) * 2);
+    temp_str = g_malloc(strlen(label) * 2);
 
     do{
 	if(label[s_idx] == ',' || label[s_idx] == '\\' ||
@@ -180,7 +194,7 @@ escape_label(
     } while(label[s_idx] != '\0');
     temp_str[d_idx] = '\0';
 
-    cooked_str = stralloc(temp_str);
+    cooked_str = g_strdup(temp_str);
     amfree(temp_str);
     
     return(cooked_str);
@@ -198,7 +212,7 @@ unescape_label(
     
     if(!label) return(NULL);
 
-    temp_str = alloc(strlen(label));
+    temp_str = g_malloc(strlen(label)+1);
 
     do{
 	if(label[s_idx] == '\\' && !prev_esc){
@@ -213,7 +227,7 @@ unescape_label(
     } while(label[s_idx] != '\0');
     temp_str[d_idx] = '\0';
 
-    cooked_str = stralloc(temp_str);
+    cooked_str = g_strdup(temp_str);
     amfree(temp_str);
     
     return(cooked_str);
@@ -225,39 +239,59 @@ unescape_label(
 char *
 marshal_tapelist(
     tapelist_t *tapelist,
-    int		do_escape)
+    int		do_escape,
+    int         with_storage)
 {
     tapelist_t *cur_tape;
-    char *str = NULL;
+    char *str;
+    GPtrArray *strarray = g_ptr_array_new();
+    GString *strbuf;
+    gchar **strings;
 
-    for(cur_tape = tapelist; cur_tape; cur_tape = cur_tape->next){
-	char *esc_label;
-	char *files_str = NULL;
+    for (cur_tape = tapelist; cur_tape; cur_tape = cur_tape->next) {
+        GPtrArray *array = g_ptr_array_new();
+        char *p;
 	int c;
 
-	if(do_escape) esc_label = escape_label(cur_tape->label);
-	else esc_label = stralloc(cur_tape->label);
-
-	for(c = 0; c < cur_tape->numfiles ; c++){
-	    char num_str[NUM_STR_SIZE];
-	    g_snprintf(num_str, SIZEOF(num_str), "%lld",
-			(long long)cur_tape->files[c]);
-	    if (!files_str)
-		files_str = stralloc(num_str);
-	    else
-		vstrextend(&files_str, ",", num_str, NULL);
+	strbuf = g_string_new("");
+	if (with_storage) {
+            p = escape_label(cur_tape->storage);
+	    g_string_append(strbuf, p);
+	    g_free(p);
+	    g_string_append_c(strbuf, ':');
 	}
 
-	if (!str)
-	    str = vstralloc(esc_label, ":", files_str, NULL);
-	else
-	    vstrextend(&str, ";", esc_label, ":", files_str, NULL);
+        p = (do_escape) ? escape_label(cur_tape->label)
+            : g_strdup(cur_tape->label);
+        g_string_append(strbuf, p);
+        g_free(p);
 
-	amfree(esc_label);
-	amfree(files_str);
+        g_string_append_c(strbuf, ':');
+
+        for (c = 0; c < cur_tape->numfiles; c++) {
+            p = g_strdup_printf("%lld", (long long)cur_tape->files[c]);
+            g_ptr_array_add(array, p);
+        }
+
+        g_ptr_array_add(array, NULL);
+
+        strings = (gchar **)g_ptr_array_free(array, FALSE);
+        p = g_strjoinv(",", strings);
+        g_strfreev(strings);
+
+        g_string_append(strbuf, p);
+        g_free(p);
+
+        g_ptr_array_add(strarray, g_string_free(strbuf, FALSE));
     }
 
-    return(str);
+    g_ptr_array_add(strarray, NULL);
+
+    strings = (gchar **)g_ptr_array_free(strarray, FALSE);
+    str = g_strjoinv(";", strings);
+    g_strfreev(strings);
+
+    return str;
 }
 
 /*
@@ -266,9 +300,10 @@ marshal_tapelist(
  */
 tapelist_t *
 unmarshal_tapelist_str(
-    char *	tapelist_str)
+    char *	tapelist_str,
+    int		with_storage)
 {
-    char *temp_label, *temp_filenum;
+    char *temp_storage, *temp_label, *temp_filenum;
     int l_idx, n_idx;
     size_t input_length;
     tapelist_t *tapelist = NULL;
@@ -277,11 +312,29 @@ unmarshal_tapelist_str(
 
     input_length = strlen(tapelist_str);
 
-    temp_label = alloc(input_length+1);
-    temp_filenum = alloc(input_length+1);
+    temp_label = g_malloc(input_length+1);
+    temp_storage = g_malloc(input_length+1);
+    temp_filenum = g_malloc(input_length+1);
 
-    do{
-	/* first, read the label part */
+    do {
+	/* first, read the storage part */
+	if (with_storage) {
+	    memset(temp_storage, '\0', input_length+1);
+            l_idx = 0;
+	    while(*tapelist_str != ':' && *tapelist_str != '\0'){
+		if(*tapelist_str == '\\')
+		    tapelist_str++; /* skip escapes */
+		temp_storage[l_idx] = *tapelist_str;
+		if(*tapelist_str == '\0')
+		    break; /* bad format, should kvetch */
+		tapelist_str++;
+		l_idx++;
+	    }
+	    if(*tapelist_str != '\0')
+		tapelist_str++;
+	}
+
+	/* then, read the label part */
 	memset(temp_label, '\0', input_length+1);
         l_idx = 0;
 	while(*tapelist_str != ':' && *tapelist_str != '\0'){
@@ -295,7 +348,7 @@ unmarshal_tapelist_str(
 	}
 	if(*tapelist_str != '\0')
 	    tapelist_str++;
-	tapelist = append_to_tapelist(tapelist, temp_label, (off_t)-1, -1, 0);
+	tapelist = append_to_tapelist(tapelist, temp_storage, temp_label, (off_t)-1, -1, 0);
 
 	/* now read the list of file numbers */
 	while(*tapelist_str != ';' && *tapelist_str != '\0'){
@@ -311,7 +364,7 @@ unmarshal_tapelist_str(
 	    }
 	    filenum = OFF_T_ATOI(temp_filenum);
 
-	    tapelist = append_to_tapelist(tapelist, temp_label, filenum, -1, 0);
+	    tapelist = append_to_tapelist(tapelist, temp_storage, temp_label, filenum, -1, 0);
 	    if(*tapelist_str != '\0' && *tapelist_str != ';')
 		tapelist_str++;
 	}
@@ -321,6 +374,7 @@ unmarshal_tapelist_str(
     } while(*tapelist_str != '\0');
 
     amfree(temp_label);
+    amfree(temp_storage);
     amfree(temp_filenum);
 
     return(tapelist);
@@ -337,6 +391,7 @@ free_tapelist(
     tapelist_t *prev = NULL;
 
     for(cur_tape = tapelist ; cur_tape ; cur_tape = cur_tape->next){
+	amfree(cur_tape->storage);
 	amfree(cur_tape->label);
 	amfree(cur_tape->files);
 	amfree(cur_tape->partnum);
