@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 /* The Device API abstracts device workings, interaction, properties, and
@@ -47,6 +48,15 @@ typedef enum {
     ACCESS_APPEND
 } DeviceAccessMode;
 
+/* Different write result modes */
+typedef enum {
+    WRITE_SUCCEED,	/* everything is written                      */
+    WRITE_FAILED,	/* something was written                      */
+    WRITE_SPACE,	/* nothing was written, global space is full
+			   freeing space could make the write succeed */
+    WRITE_FULL		/* nothing was written, volume is full        */
+} DeviceWriteResult;
+
 #define IS_WRITABLE_ACCESS_MODE(mode) ((mode) == ACCESS_WRITE || \
                                        (mode) == ACCESS_APPEND)
 
@@ -185,16 +195,18 @@ struct _DeviceClass {
     gboolean (* start) (Device * self, DeviceAccessMode mode,
                         char * label, char * timestamp);
     gboolean (* start_file) (Device * self, dumpfile_t * info);
-    gboolean (* write_block) (Device * self, guint size, gpointer data);
+    DeviceWriteResult (* write_block) (Device * self, guint size,
+				       gpointer data);
     gboolean (* finish_file) (Device * self);
+    gboolean (* init_seek_file) (Device * self, guint file);
     dumpfile_t* (* seek_file) (Device * self, guint file);
     gboolean (* seek_block) (Device * self, guint64 block);
-    int (* read_block) (Device * self, gpointer buf, int * size);
+    int (* read_block) (Device * self, gpointer buf, int * size, int max_block);
     gboolean (* property_get_ex) (Device * self, DevicePropertyId id,
 				  GValue * val,
 				  PropertySurety *surety,
 				  PropertySource *source);
-    gboolean (* property_set_ex) (Device * self,
+    char * (* property_set_ex) (Device * self,
 				  DevicePropertyId id,
 				  GValue * val,
 				  PropertySurety surety,
@@ -204,6 +216,7 @@ struct _DeviceClass {
     gboolean (* eject) (Device * self);
     gboolean (* finish) (Device * self);
     guint64  (* get_bytes_read) (Device * self);
+    guint64  (* clear_bytes_read) (Device * self);
     guint64  (* get_bytes_written) (Device * self);
 
     gboolean (* listen)(Device *self, gboolean for_writing, DirectTCPAddr **addrs);
@@ -228,6 +241,13 @@ struct _DeviceClass {
 			GMutex *abort_mutex, GCond *abort_cond);
 
     gboolean (* use_connection)(Device *self, DirectTCPConnection *conn);
+    gboolean (* check_writable)(Device *self);
+    gboolean (* have_set_reuse)(Device *self);
+    gboolean (* set_reuse)(Device *self);
+    gboolean (* set_no_reuse)(Device *self, char *label, char *datestamp);
+
+    gboolean (* create) (Device * self);
+    gboolean (* sync_catalog) (Device * self, int request, int wait, char **slot_names);
 
     /* array of DeviceProperty objects for this class, keyed by ID */
     GArray *class_properties;
@@ -276,6 +296,9 @@ gboolean device_configure(Device *self, gboolean use_global_config);
  * Error Handling
  */
 
+/* remove error */
+void device_reset(Device * self);
+
 /* return the error message or the string "Unknown Device error".  The
  * string remains the responsibility of the Device, and should not
  * be freed by the caller. */
@@ -319,19 +342,22 @@ gboolean 	device_start	(Device * self,
                                  DeviceAccessMode mode, char * label,
                                  char * timestamp);
 gboolean 	device_finish	(Device * self);
+void		device_clear_bytes_read	(Device * self);
 guint64 	device_get_bytes_read	(Device * self);
 guint64 	device_get_bytes_written(Device * self);
 gboolean        device_start_file       (Device * self,
                                          dumpfile_t * jobInfo);
-gboolean 	device_write_block	(Device * self,
+DeviceWriteResult device_write_block	(Device * self,
                                          guint size,
                                          gpointer data);
 gboolean 	device_finish_file	(Device * self);
+gboolean	device_init_seek_file	(Device * self,
+					guint file);
 dumpfile_t* 	device_seek_file	(Device * self,
 					guint file);
 gboolean 	device_seek_block	(Device * self,
 					guint64 block);
-int 	device_read_block	(Device * self, gpointer buffer, int * size);
+int 	device_read_block	(Device * self, gpointer buffer, int * size, int max_block);
 const GSList *	device_property_get_list	(Device * self);
 gboolean 	device_property_get_ex	(Device * self,
                                          DevicePropertyId id,
@@ -340,7 +366,7 @@ gboolean 	device_property_get_ex	(Device * self,
 					 PropertySource *source);
 #define		device_property_get(self, id, val) \
     device_property_get_ex((self), (id), (val), NULL, NULL)
-gboolean 	device_property_set_ex	(Device * self,
+char *	 	device_property_set_ex	(Device * self,
                                          DevicePropertyId id,
                                          GValue * val,
 					 PropertySurety surety,
@@ -370,6 +396,13 @@ int device_read_to_connection(Device *self, guint64 size,
 			GMutex *abort_mutex, GCond *abort_cond);
 gboolean device_use_connection(Device *self, DirectTCPConnection *conn);
 gboolean device_allow_take_scribe_from(Device *self);
+gboolean device_check_writable(Device *self);
+gboolean device_have_set_reuse(Device *self);
+gboolean device_set_reuse(Device *self);
+gboolean device_set_no_reuse(Device *self, char *label, char *datestamp);
+gboolean device_create(Device *self);
+gboolean device_sync_catalog(Device * self, int request, int wait,
+			     char **slot_names);
 
 /* Protected methods. Don't call these except in subclass implementations. */
 
