@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,12 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
 #include "device.h"
 #include "directtcp.h"
 #include "stream.h"
@@ -51,6 +52,7 @@ struct NdmpDevice_ {
 
     /* true if tape service is open on the NDMP server */
     gboolean tape_open;
+    guint64      bytes_moved_before;
 
     /* addresses the object is listening on, and how the connection
      * was opened */
@@ -557,10 +559,17 @@ read_err:
 	return dself->status;
     }
 
+    if (!header) {
+	device_set_error(dself,
+			 g_strdup(_("no header set")),
+			 DEVICE_STATUS_VOLUME_UNLABELED);
+	return dself->status;
+    }
+
     /* handle a "weird" label */
     if (header->type != F_TAPESTART) {
 	device_set_error(dself,
-		stralloc(_("No tapestart header -- unlabeled device?")),
+		g_strdup(_("No tapestart header -- unlabeled device?")),
 			 DEVICE_STATUS_VOLUME_UNLABELED);
 	return dself->status;
     }
@@ -631,7 +640,7 @@ ndmp_device_start(
 	header_buf = device_build_amanda_header(dself, header, NULL);
 	if (header_buf == NULL) {
 	    device_set_error(dself,
-		stralloc(_("Tapestart header won't fit in a single block!")),
+		g_strdup(_("Tapestart header won't fit in a single block!")),
 		DEVICE_STATUS_DEVICE_ERROR);
 	    dumpfile_free(header);
 	    return FALSE;
@@ -648,7 +657,7 @@ ndmp_device_start(
 		/* this would be an odd error to see writing the tape label, but
 		 * oh well */
 		device_set_error(dself,
-		    stralloc(_("No space left on device")),
+		    g_strdup(_("No space left on device")),
 		    DEVICE_STATUS_VOLUME_ERROR);
 		dself->is_eom = TRUE;
 		/* fall through */
@@ -668,8 +677,10 @@ ndmp_device_start(
 	    return FALSE;
 	}
 
-	dself->volume_label = newstralloc(dself->volume_label, label);
-	dself->volume_time = newstralloc(dself->volume_time, timestamp);
+	g_free(dself->volume_label);
+	dself->volume_label = g_strdup(label);
+	g_free(dself->volume_time);
+	dself->volume_time = g_strdup(timestamp);
 	dumpfile_free(dself->volume_header);
 	dself->volume_header = header;
 
@@ -715,6 +726,11 @@ ndmp_device_eject(
     NdmpDevice *self = NDMP_DEVICE(dself);
     if (device_in_error(dself)) return FALSE;
 
+    if (!open_connection(self)) {
+	/* error was set by open_connection */
+	return FALSE;
+    }
+
     if (!single_ndmp_mtio(self, NDMP9_MTIO_OFF)) {
 	/* error was set by single_ndmp_mtio */
 	return FALSE;
@@ -749,7 +765,7 @@ ndmp_device_start_file(
     header_buf = device_build_amanda_header(dself, header, NULL);
     if (header_buf == NULL) {
 	device_set_error(dself,
-	    stralloc(_("Amanda file header won't fit in a single block!")),
+	    g_strdup(_("Amanda file header won't fit in a single block!")),
 	    DEVICE_STATUS_DEVICE_ERROR);
 	return FALSE;
     }
@@ -766,7 +782,7 @@ ndmp_device_start_file(
 	    /* this would be an odd error to see writing the tape label, but
 	     * oh well */
 	    device_set_error(dself,
-		stralloc(_("No space left on device")),
+		g_strdup(_("No space left on device")),
 		DEVICE_STATUS_VOLUME_ERROR);
 	    dself->is_eom = TRUE;
 	    /* fall through */
@@ -793,7 +809,7 @@ ndmp_device_start_file(
     return TRUE;
 }
 
-static gboolean
+static DeviceWriteResult
 ndmp_device_write_block(
     Device   *dself,
     guint     size,
@@ -802,7 +818,7 @@ ndmp_device_write_block(
     NdmpDevice *self = NDMP_DEVICE(dself);
     gchar *replacement_buffer = NULL;
 
-    if (device_in_error(self)) return FALSE;
+    if (device_in_error(self)) return WRITE_FAILED;
 
     /* zero out to the end of a short block -- tape devices only write
      * whole blocks. */
@@ -812,7 +828,7 @@ ndmp_device_write_block(
 	    device_set_error(dself,
 		g_strdup(_("Cannot allocate memory")),
 			DEVICE_STATUS_DEVICE_ERROR);
-	    return FALSE;
+	    return WRITE_FAILED;
 	}
         memcpy(replacement_buffer, data, size);
         bzero(replacement_buffer+size, dself->block_size-size);
@@ -833,7 +849,7 @@ ndmp_device_write_block(
 	    /* this would be an odd error to see writing the tape label, but
 	     * oh well */
 	    device_set_error(dself,
-		stralloc(_("No space left on device")),
+		g_strdup(_("No space left on device")),
 		DEVICE_STATUS_VOLUME_ERROR);
 	    dself->is_eom = TRUE;
 	    /* fall through */
@@ -841,7 +857,7 @@ ndmp_device_write_block(
 	case ROBUST_WRITE_ERROR:
 	    /* error was set by robust_write or above */
 	    if (replacement_buffer) g_free(replacement_buffer);
-	    return FALSE;
+	    return WRITE_FAILED;
     }
 
     dself->block++;
@@ -850,7 +866,7 @@ ndmp_device_write_block(
     g_mutex_unlock(dself->device_mutex);
 
     if (replacement_buffer) g_free(replacement_buffer);
-    return TRUE;
+    return WRITE_SUCCEED;
 }
 
 static gboolean
@@ -949,7 +965,7 @@ incomplete_bsf:
 	/* if we didn't seek all the way there, then we're past the tapeend */
 	if (resid > 0) {
 	    device_set_error(dself,
-		vstrallocf(_("Could not seek forward to file %d"), file),
+		g_strdup_printf(_("Could not seek forward to file %d"), file),
 		DEVICE_STATUS_VOLUME_ERROR);
 	    return NULL;
 	}
@@ -1010,7 +1026,7 @@ ndmp_device_seek_block(
 }
 
 static int
-ndmp_device_read_block (Device * dself, gpointer data, int *size_req) {
+ndmp_device_read_block (Device * dself, gpointer data, int *size_req, int max_block G_GNUC_UNUSED) {
     NdmpDevice *self = NDMP_DEVICE(dself);
     guint64 requested, actual;
     gsize read_block_size = ndmp_device_read_size(self);
@@ -1113,9 +1129,9 @@ listen_impl(
 	    return indirecttcp_listen(self, addrs);
 	}
 	if (!ndmp_connection_mover_set_window(self->ndmp, 0, 0)) {
-	    /* NDMP4_ILLEGAL_ARGS_ERR means the NDMP server doesn't like a zero-byte
+	    /* NDMP9_ILLEGAL_ARGS_ERR means the NDMP server doesn't like a zero-byte
 	     * mover window, so we'll ignore it */
-	    if (ndmp_connection_err_code(self->ndmp) != NDMP4_ILLEGAL_ARGS_ERR) {
+	    if (ndmp_connection_err_code(self->ndmp) != NDMP9_ILLEGAL_ARGS_ERR) {
 		set_error_from_ndmp(self);
 		return FALSE;
 	    }
@@ -1148,9 +1164,10 @@ listen_impl(
     return TRUE;
 }
 
+
 static gpointer
 accept_wait_cond(
-    gpointer data)
+        gpointer data)
 {
     NdmpDevice *self = NDMP_DEVICE(data);
 
@@ -1205,7 +1222,7 @@ accept_impl(
     ndmp9_mover_mode mode;
     ndmp9_mover_halt_reason mover_halt_reason  = NDMP9_MOVER_HALT_NA;
     ndmp9_mover_pause_reason mover_pause_reason = NDMP9_MOVER_PAUSE_NA;
-    guint64 seek_position;
+    guint64  seek_position;
     GThread *wait_thread;
     int      result;
     char    *err;
@@ -1495,10 +1512,11 @@ indirecttcp_start_writing(
     /* tell mover to start listening */
     g_assert(self->for_writing);
     if (!ndmp_connection_mover_listen(self->ndmp,
-		NDMP4_MOVER_MODE_READ,
-		NDMP4_ADDR_TCP,
+		NDMP9_MOVER_MODE_READ,
+		NDMP9_ADDR_TCP,
 		&real_addrs)) {
 	set_error_from_ndmp(self);
+	close(conn_sock);
 	return FALSE;
     }
 
@@ -1517,6 +1535,7 @@ indirecttcp_start_writing(
 	    device_set_error(DEVICE(self),
 		g_strdup_printf("writing to indirecttcp socket: %s", strerror(errno)),
 		DEVICE_STATUS_DEVICE_ERROR);
+	    close(conn_sock);
 	    return FALSE;
 	}
     }
@@ -1596,7 +1615,8 @@ write_from_connection_impl(
      */
     if (!ndmp_connection_mover_set_window(self->ndmp,
 		nconn->offset,
-		size? size : G_MAXUINT64 - nconn->offset)) {
+		size && size != G_MAXUINT64 ? size
+					    : G_MAXUINT64 - nconn->offset)) {
 	set_error_from_ndmp(self);
 	return 1;
     }
@@ -1741,7 +1761,8 @@ read_to_connection_impl(
 
 	if (!ndmp_connection_mover_set_window(self->ndmp,
 		nconn->offset,
-		size? size : G_MAXUINT64 - nconn->offset)) {
+		size && size != G_MAXUINT64 ? size
+					    : G_MAXUINT64 - nconn->offset)) {
 	    set_error_from_ndmp(self);
 	    return 1;
 	}
@@ -1812,12 +1833,13 @@ read_to_connection_impl(
 	set_error_from_ndmp(self);
 	return 1;
     }
-    size = bytes_moved_after - bytes_moved_before;
+    size = bytes_moved_after - self->bytes_moved_before;
     nconn->offset += size;
 
     if (actual_size) {
-	*actual_size = bytes_moved_after - bytes_moved_before;
+	*actual_size = size;
     }
+    self->bytes_moved_before = bytes_moved_after;
 
     if (eow) {
         ; /* mover finished the whole part -- nothing to report! */
@@ -2163,7 +2185,7 @@ ndmp_device_get_type(void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (NdmpDeviceClass),
             (GBaseInitFunc) NULL,
@@ -2191,7 +2213,7 @@ ndmp_device_factory(
     char *device_node)
 {
     Device *rval;
-    g_assert(0 == strcmp(device_type, NDMP_DEVICE_NAME));
+    g_assert(g_str_equal(device_type, NDMP_DEVICE_NAME));
     rval = DEVICE(g_object_new(TYPE_NDMP_DEVICE, NULL));
 
     device_open_device(rval, device_name, device_type, device_node);
@@ -2282,10 +2304,10 @@ directtcp_connection_ndmp_close(DirectTCPConnection *dself)
     }
 
 error:
-    if (self->ndmp) {
-	g_object_unref(self->ndmp);
-	self->ndmp = NULL;
-    }
+
+    /* self->ndmp is always set */
+    g_object_unref(self->ndmp);
+    self->ndmp = NULL;
 
     return rv;
 }
@@ -2303,7 +2325,7 @@ directtcp_connection_ndmp_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (DirectTCPConnectionNDMPClass),
             (GBaseInitFunc) NULL,
