@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -66,7 +67,8 @@ static gboolean null_device_start (Device * self, DeviceAccessMode mode,
                                    char * label, char * timestamp);
 static gboolean null_device_finish (Device * pself);
 static gboolean null_device_start_file(Device * self, dumpfile_t * jobInfo);
-static gboolean null_device_write_block (Device * self, guint size, gpointer data);
+static DeviceWriteResult null_device_write_block(Device * self, guint size,
+						 gpointer data);
 static gboolean null_device_finish_file(Device * self);
 static Device* null_device_factory(char * device_name, char * device_type, char * device_node);
 
@@ -83,7 +85,7 @@ null_device_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (NullDeviceClass),
             (GBaseInitFunc) null_device_base_init,
@@ -197,7 +199,7 @@ null_device_base_init (NullDeviceClass * c)
 
 static Device* null_device_factory(char * device_name, char * device_type, char * device_node) {
     Device * device;
-    g_assert(0 == strcmp(device_type, "null"));
+    g_assert(g_str_equal(device_type, "null"));
     device = DEVICE(g_object_new(TYPE_NULL_DEVICE, NULL));
     device_open_device(device, device_name, device_type, device_node);
     return device;
@@ -210,7 +212,7 @@ null_device_read_label(Device * dself) {
     if (device_in_error(dself)) return FALSE;
 
     device_set_error(dself,
-	stralloc(_("Can't open NULL device for reading or appending.")),
+	g_strdup(_("Can't open NULL device for reading or appending.")),
 	DEVICE_STATUS_VOLUME_UNLABELED | DEVICE_STATUS_VOLUME_ERROR);
     return FALSE;
 }
@@ -242,13 +244,15 @@ null_device_start (Device * pself, DeviceAccessMode mode,
     g_mutex_unlock(pself->device_mutex);
 
     if (mode == ACCESS_WRITE) {
-        pself->volume_label = newstralloc(pself->volume_label, label);
-        pself->volume_time = newstralloc(pself->volume_time, timestamp);
+        g_free(pself->volume_label);
+        pself->volume_label = g_strdup(label);
+        g_free(pself->volume_time);
+        pself->volume_time = g_strdup(timestamp);
 	pself->header_block_size = 32768;
 	return TRUE;
     } else {
 	device_set_error(pself,
-	    stralloc(_("Can't open NULL device for reading or appending.")),
+	    g_strdup(_("Can't open NULL device for reading or appending.")),
 	    DEVICE_STATUS_VOLUME_UNLABELED | DEVICE_STATUS_VOLUME_ERROR);
         return FALSE;
     }
@@ -281,17 +285,17 @@ null_device_start_file(Device * d_self,
     return TRUE;
 }
 
-static gboolean
+static DeviceWriteResult
 null_device_write_block (Device * pself, guint size G_GNUC_UNUSED,
 	    gpointer data G_GNUC_UNUSED) {
     NullDevice * self;
     self = NULL_DEVICE(pself);
 
-    if (device_in_error(self)) return FALSE;
+    if (device_in_error(self)) return WRITE_FAILED;
 
     pself->block++;
 
-    return TRUE;
+    return WRITE_SUCCEED;
 }
 
 static gboolean
