@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -247,6 +248,30 @@ void device_property_init(void) {
     device_property_fill_and_register(&device_property_leom,
                                      G_TYPE_BOOLEAN, "leom",
        "Does this device support LEOM?");
+    device_property_fill_and_register(&device_property_reuse_connection,
+                                     G_TYPE_BOOLEAN, "reuse_connection",
+       "Whether to reuse connection");
+    device_property_fill_and_register(&device_property_max_send_speed,
+                                     G_TYPE_UINT64, "max_send_speed",
+       "Maximum average upload speed (bytes/sec)");
+    device_property_fill_and_register(&device_property_max_recv_speed,
+                                     G_TYPE_UINT64, "max_recv_speed",
+       "Maximum average download speed (bytes/sec)");
+    device_property_fill_and_register(&device_property_nb_threads_backup,
+                                     G_TYPE_UINT64, "nb_threads_backup",
+       "Number of writer thread");
+    device_property_fill_and_register(&device_property_nb_threads_recovery,
+                                     G_TYPE_UINT64, "nb_threads_recovery",
+       "Number of reader thread");
+    device_property_fill_and_register(&device_property_multi_part_upload,
+                                     G_TYPE_BOOLEAN, "multi_part_upload",
+       "Whether to use multi part upload");
+    device_property_fill_and_register(&device_property_ssl_ca_info,
+                                      G_TYPE_STRING, "ssl_ca_info",
+       "Path to certificate authority certificate");
+    device_property_fill_and_register(&device_property_proxy,
+                                      G_TYPE_STRING, "proxy",
+       "The proxy");
 }
 
 DevicePropertyBase device_property_concurrency;
@@ -267,3 +292,11 @@ DevicePropertyBase device_property_enforce_max_volume_usage;
 DevicePropertyBase device_property_comment;
 DevicePropertyBase device_property_leom;
 DevicePropertyBase device_property_verbose;
+DevicePropertyBase device_property_reuse_connection;
+DevicePropertyBase device_property_max_send_speed;
+DevicePropertyBase device_property_max_recv_speed;
+DevicePropertyBase device_property_nb_threads_backup;
+DevicePropertyBase device_property_nb_threads_recovery;
+DevicePropertyBase device_property_multi_part_upload;
+DevicePropertyBase device_property_ssl_ca_info;
+DevicePropertyBase device_property_proxy;
