@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 
@@ -186,7 +187,7 @@ s3_compute_sha256_hash(
     int len)
 {
     unsigned char hash[SHA256_DIGEST_LENGTH];
-    char *ret = malloc(64);
+    char *ret = malloc(SHA256_DIGEST_LENGTH*2+1);
     int i;
 
     SHA256_CTX sha256;
@@ -220,7 +221,7 @@ s3_uri_encode(
 		g_string_append_c(ret, ch);
 	    }
 	} else {
-	    g_string_append_printf(ret, "%02x", ch);
+	    g_string_append_printf(ret, "%%%02X", ch);
 	}
     }
     return g_string_free(ret, FALSE);
@@ -238,7 +239,13 @@ EncodeHMACSHA256(
     unsigned char tk[SHA256_DIGEST_LENGTH];
 
     // Initialise HMACh
+#if (defined OPENSSL_VERSION_NUMBER && OPENSSL_VERSION_NUMBER < 0x10100000L) \
+    || defined LIBRESSL_VERSION_NUMBER
+
     HMAC_CTX HMAC;
+#else
+    HMAC_CTX *HMAC;
+#endif
     unsigned int hmaclength = 32;
     memset(hmachash, 0, hmaclength);
 
@@ -249,11 +256,21 @@ EncodeHMACSHA256(
     }
 
     // Digest the key and message using SHA256
+#if (defined OPENSSL_VERSION_NUMBER && OPENSSL_VERSION_NUMBER < 0x10100000L) \
+    || defined LIBRESSL_VERSION_NUMBER
     HMAC_CTX_init(&HMAC);
     HMAC_Init_ex(&HMAC, key, keylen, EVP_sha256(),NULL);
     HMAC_Update(&HMAC, datatohash, datalen);
     HMAC_Final(&HMAC, hmachash, &hmaclength);
     HMAC_CTX_cleanup(&HMAC);
+#else
+    HMAC = HMAC_CTX_new();
+    HMAC_CTX_reset(HMAC);
+    HMAC_Init_ex(HMAC, key, keylen, EVP_sha256(),NULL);
+    HMAC_Update(HMAC, datatohash, datalen);
+    HMAC_Final(HMAC, hmachash, &hmaclength);
+    HMAC_CTX_free(HMAC);
+#endif
 
     return hmachash;
 }
