@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #ifndef __S3_H__
@@ -33,11 +34,22 @@ typedef enum {
    S3_API_S3,
    S3_API_SWIFT_1,
    S3_API_SWIFT_2,
+   S3_API_SWIFT_3,
    S3_API_OAUTH2,
    S3_API_CASTOR,
    S3_API_AWS4
 } S3_api;
 
+extern char *S3_name[];
+extern char *S3_bucket_name[];
+
+typedef enum {
+   S3_SC_STANDARD,
+   S3_SC_STANDARD_IA,
+   S3_SC_REDUCED_REDUNDANCY,
+   S3_SC_GLACIER
+} StorageClass;
+
 /* An opaque handle.  S3Handles should only be accessed from a single
  * thread at any given time, although it is fine to use different handles
  * in different threads simultaneously. */
@@ -217,6 +229,8 @@ typedef curl_progress_callback s3_progress_func;
     S3_ERROR(Unknown), \
     S3_ERROR(UnresolvableGrantByEmailAddress), \
     S3_ERROR(UserKeyMustBeSpecified), \
+    S3_ERROR(RegionNotFound), \
+    S3_ERROR(NotFound), \
     S3_ERROR(END)
 
 typedef enum {
@@ -285,6 +299,7 @@ s3_init(void);
  */
 S3Handle *
 s3_open(const char * access_key, const char *secret_key,
+	const char *session_token,
 	const char *swift_account_id, const char *swift_access_key,
 	const char *host,
         const char *service_path, gboolean use_subdomain,
@@ -297,11 +312,14 @@ s3_open(const char * access_key, const char *secret_key,
 	const char *password,
 	const char *tenant_id,
 	const char *tenant_name,
+	const char *project_name,
+	const char *domain_name,
 	const char *client_id,
 	const char *client_secret,
 	const char *refresh_token,
 	const gboolean reuse_connection,
-	const long  timeout,
+	const gboolean read_from_glacier,
+	const long timeout,
         const char *reps,
         const char *reps_bucket);
 
@@ -426,7 +444,8 @@ s3_strerror(S3Handle *hdl);
 gboolean
 s3_upload(S3Handle *hdl,
           const char *bucket,
-          const char *key, 
+          const char *key,
+          const gboolean chunked,
           s3_read_func read_func,
           s3_reset_func reset_func,
           s3_size_func size_func,
@@ -435,6 +454,118 @@ s3_upload(S3Handle *hdl,
           s3_progress_func progress_func,
           gpointer progress_data);
 
+/* Perform a part upload.
+ *
+ * When this function returns, KEY and BUFFER remain the
+ * responsibility of the caller.
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to which the upload should be made
+ * @param key: the key to which the upload should be made
+ * @param uploadId: the UploadId
+ * @param partNumber: the part number
+ * @param etag: return the resulting etag.
+ * @param read_func: the callback for reading data
+ * @param reset_func: the callback for to reset reading data
+ * @param size_func: the callback to get the number of bytes to upload
+ * @param md5_func: the callback to get the MD5 hash of the data to upload
+ * @param read_data: pointer to pass to the above functions
+ * @param progress_func: the callback for progress information
+ * @param progress_data: pointer to pass to C{progress_func}
+ *
+ * @returns: false if an error ocurred
+ */
+gboolean
+s3_part_upload(S3Handle *hdl,
+          const char *bucket,
+          const char *key,
+	  const char *uploadID,
+	  int         partNumber,
+	  char      **etag,
+          s3_read_func read_func,
+          s3_reset_func reset_func,
+          s3_size_func size_func,
+          s3_md5_func md5_func,
+          gpointer read_data,
+          s3_progress_func progress_func,
+          gpointer progress_data);
+
+/* Initiate a multi part upload.
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to which the upload should be made
+ * @param key: the key to which the upload should be made
+ *
+ * @returns: string, the uploadId or NULL on failure
+ */
+char *
+s3_initiate_multi_part_upload(
+    S3Handle *hdl,
+    const char *bucket,
+    const char *key);
+
+/* Complete a multi part upload.
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to which the upload should be made
+ * @param key: the key to which the upload should be made
+ * @param read_func: the callback for reading data
+ *
+ * @returns: false if an error ocurred
+ */
+gboolean
+s3_complete_multi_part_upload(
+    S3Handle *hdl,
+    const char *bucket,
+    const char *key,
+    const char *uploadId,
+    s3_read_func read_func,
+    s3_reset_func reset_func,
+    s3_size_func size_func,
+    s3_md5_func md5_func,
+    gpointer read_data);
+
+gboolean
+s3_abort_multi_part_upload(
+    S3Handle *hdl,
+    const char *bucket,
+    const char *key,
+    const char *uploadId);
+
+typedef struct {
+    char    *key;
+    char    *uploadId;
+    char    *prefix;
+    guint64  size;
+    StorageClass storage_class;
+} s3_object;
+void free_s3_object(gpointer part);
+
+typedef struct {
+    char *key;
+    char *x_amz_expiration;
+    char *x_amz_restore;
+} s3_head_t;
+void free_s3_head(s3_head_t *head);
+
+typedef struct lifecycle_action {
+    gint  days;
+    char *date;
+    char *storage_class;
+} lifecycle_action;
+
+typedef struct lifecycle_rule {
+    char *id;
+    char *prefix;
+    char *status;
+    lifecycle_action *transition;
+    lifecycle_action *expiration;
+} lifecycle_rule;
+
+void free_lifecycle_rule(gpointer data);
+void free_lifecycle(GSList *lifecycle);
+
+
 /* List all of the files matching the pseudo-glob C{PREFIX*DELIMITER*},
  * returning only that portion which matches C{PREFIX*DELIMITER}.  S3 supports
  * this particular semantics, making it quite efficient.  The returned list
@@ -445,17 +576,42 @@ s3_upload(S3Handle *hdl,
  * @param prefix: the prefix
  * @param delimiter: delimiter (any length string)
  * @param list: (output) the list of files
- * @param total_size: (output) sum of size of files 
+ * @param total_size: (output) sum of size of files
  * @returns: FALSE if an error occurs
  */
 gboolean
 s3_list_keys(S3Handle *hdl,
               const char *bucket,
+              const char *subresource,
               const char *prefix,
               const char *delimiter,
               GSList **list,
               guint64 *total_size);
 
+/* Init a restore from s3 for an object
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to list
+ * @param key: the key to get the head from
+ * @returns: FALSE if an error occurs
+ */
+gboolean
+s3_init_restore(S3Handle *hdl,
+	        const char *bucket,
+	        const char *key);
+
+/* get the head of an object
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to list
+ * @param key: the key to get the head from
+ * @returns: FALSE if an error occurs
+ */
+s3_head_t *
+s3_head(S3Handle *hdl,
+	const char *bucket,
+	const char *key);
+
 /* Read an entire file, passing the contents to write_func buffer
  * by buffer.
  *
@@ -479,6 +635,33 @@ s3_read(S3Handle *hdl,
         s3_progress_func progress_func,
         gpointer progress_data);
 
+/* Read a range of bytes from a file, passing the contents to write_func buffer
+ * by buffer.
+ *
+ * @param hdl: the S3Handle object
+ * @param bucket: the bucket to read from
+ * @param key: the key to read from
+ * @param range_begin:
+ * @param range_ned:
+ * @param write_func: the callback for writing data
+ * @param reset_func: the callback for to reset writing data
+ * @param write_data: pointer to pass to C{write_func}
+ * @param progress_func: the callback for progress information
+ * @param progress_data: pointer to pass to C{progress_func}
+ * @returns: FALSE if an error occurs
+ */
+gboolean
+s3_read_range(S3Handle *hdl,
+        const char *bucket,
+        const char *key,
+	const guint64 range_begin,
+	const guint64 range_end,
+        s3_write_func write_func,
+        s3_reset_func reset_func,
+        gpointer write_data,
+        s3_progress_func progress_func,
+        gpointer progress_data);
+
 /* Delete a file.
  *
  * @param hdl: the S3Handle object
@@ -502,7 +685,7 @@ s3_delete(S3Handle *hdl,
 int
 s3_multi_delete(S3Handle *hdl,
                 const char *bucket,
-                const char **key);
+                GSList *objects);
 
 /* Create a bucket.
  *
@@ -524,6 +707,7 @@ s3_make_bucket(S3Handle *hdl,
 gboolean
 s3_is_bucket_exists(S3Handle *hdl,
                     const char *bucket,
+                    const char *prefix,
 		    const char *project_id);
 
 /* Delete a bucket
@@ -544,12 +728,42 @@ s3_delete_bucket(S3Handle *hdl,
 gboolean sts_refresh_token(char ** token, const char * directory);
 
 /* These functions are for if you want to use curl on your own. You get more
- * control, but it's a lot of work that way: */
+ * control, but it's a lot of work that way:
+ */
+/* There is two use, simple buffer and circle buffer
+ */
+/* simple buffer
+ * buffer: pointer to the buffer
+ * buffer_len: size of the allocated buffer
+ * buffer_pos: size use in the buffer (from buffer to buffer+buffer_pos)
+ * max_buffer_size: maximum size the buffer can be reallocated.
+ * end_of_buffer: unused
+ * mutex: NULL
+ * cond: unused
+ */
+/* circle buffer (use for chunked transfer-encodig)
+ * buffer: pointer to the buffer
+ * buffer_len: indice of the last byte+1 use in the buffer
+ * buffer_pos: indice of the first byte use in the buffer
+ * max_buffer_size: allocated size of the buffer
+ * end_of_buffer: TRUE once reach end of data.
+ * mutex: !NULL
+ * cond: !NULL
+ *
+ * buffer_len == buffer_pos: buffer is empty
+ * The data in use are
+ *     buffer_len > buffer_pos: from buffer_pos to buffer_len
+ *     buffer_len < buffer_pos: from buffer_pos to max_buffer_size
+ *                          and from 0 to buffer_len
+ */
 typedef struct {
     char *buffer;
     guint buffer_len;
     guint buffer_pos;
     guint max_buffer_size;
+    gboolean end_of_buffer;
+    GMutex   *mutex;
+    GCond    *cond;
 } CurlBuffer;
 
 #define S3_BUFFER_READ_FUNCS s3_buffer_read_func, s3_buffer_reset_func, s3_buffer_size_func, s3_buffer_md5_func
@@ -615,6 +829,12 @@ size_t
 s3_file_write_func(void *ptr, size_t size, size_t nmemb, void *stream);
 #endif
 
+gboolean
+s3_get_lifecycle(S3Handle *hdl, const char *bucket, GSList **lifecycle);
+
+gboolean
+s3_put_lifecycle(S3Handle *hdl, const char *bucket, GSList *lifecycle);
+
 /* Adds a null termination to a buffer. */
 void terminate_buffer(CurlBuffer *);
 
