@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,14 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
 #include "pipespawn.h"
 #include <string.h> /* memset() */
-#include "util.h"
+#include "amutil.h"
 #include "device.h"
 
 #ifdef HAVE_SYS_TAPE_H
@@ -193,9 +194,10 @@ static gboolean tape_device_set_read_block_size_fn(Device *p_self, DevicePropert
 static void tape_device_open_device (Device * self, char * device_name, char * device_type, char * device_node);
 static Device * tape_device_factory (char * device_name, char * device_type, char * device_node);
 static DeviceStatusFlags tape_device_read_label(Device * self);
-static gboolean tape_device_write_block(Device * self, guint size, gpointer data);
+static DeviceWriteResult tape_device_write_block(Device * self, guint size,
+						 gpointer data);
 static int tape_device_read_block(Device * self,  gpointer buf,
-                                       int * size_req);
+                                       int * size_req, int max_block);
 static gboolean tape_device_start (Device * self, DeviceAccessMode mode,
                                    char * label, char * timestamp);
 static gboolean tape_device_start_file (Device * self, dumpfile_t * ji);
@@ -204,6 +206,7 @@ static dumpfile_t * tape_device_seek_file (Device * self, guint file);
 static gboolean tape_device_seek_block (Device * self, guint64 block);
 static gboolean tape_device_eject (Device * self);
 static gboolean tape_device_finish (Device * self);
+static gboolean tape_device_check_writable(Device *self);
 static IoResult tape_device_robust_read (TapeDevice * self, void * buf,
                                                int * count, char **errmsg);
 static IoResult tape_device_robust_write (TapeDevice * self, void * buf, int count, char **errmsg);
@@ -219,7 +222,7 @@ GType tape_device_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (TapeDeviceClass),
             (GBaseInitFunc) tape_device_base_init,
@@ -385,6 +388,7 @@ tape_device_class_init (TapeDeviceClass * c)
     device_class->seek_block = tape_device_seek_block;
     device_class->eject = tape_device_eject;
     device_class->finish = tape_device_finish;
+    device_class->check_writable = tape_device_check_writable;
 
     g_object_class->finalize = tape_device_finalize;
 }
@@ -487,7 +491,7 @@ tape_device_set_feature_property_fn(Device *p_self, DevicePropertyBase *base,
 
 	if (old_surety == PROPERTY_SURETY_GOOD && old_source == PROPERTY_SOURCE_DETECTED) {
 	    if (new_bool != old_bool) {
-		device_set_error(p_self, vstrallocf(_(
+		device_set_error(p_self, g_strdup_printf(_(
 			   "Value for property '%s' was autodetected and cannot be changed"),
 			   base->name),
 		    DEVICE_STATUS_DEVICE_ERROR);
@@ -545,7 +549,7 @@ tape_device_set_final_filemarks_fn(Device *p_self, DevicePropertyBase *base,
 
 	if (old_surety == PROPERTY_SURETY_GOOD && old_source == PROPERTY_SOURCE_DETECTED) {
 	    if (new_int != old_int) {
-		device_set_error(p_self, vstrallocf(_(
+		device_set_error(p_self, g_strdup_printf(_(
 			   "Value for property '%s' was autodetected and cannot be changed"),
 			   base->name),
 		    DEVICE_STATUS_DEVICE_ERROR);
@@ -733,8 +737,21 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
     }
 #ifdef O_NONBLOCK
     /* Clear O_NONBLOCK for operations from now on. */
-    if (fd >= 0 && nonblocking)
-	fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NONBLOCK);
+    if (fd >= 0 && nonblocking) {
+	int r = fcntl(fd, F_GETFL, 0);
+	if (r < 0) {
+	    device_set_error(DEVICE(self),
+		g_strdup_printf("Can't fcntl(F_GETFL) on %s: %s", self->private->device_filename, strerror(errno)),
+		DEVICE_STATUS_DEVICE_BUSY | DEVICE_STATUS_DEVICE_ERROR);
+	}
+
+	r = fcntl(fd, F_SETFL, r & ~O_NONBLOCK);
+	if (r < 0) {
+	    device_set_error(DEVICE(self),
+		g_strdup_printf("Can't fcntl(F_SETFL) on %s: %s", self->private->device_filename, strerror(errno)),
+		DEVICE_STATUS_DEVICE_BUSY | DEVICE_STATUS_DEVICE_ERROR);
+	}
+    }
     errno = save_errno;
     /* function continues after #endif */
 
@@ -747,7 +764,7 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
 	else
 	    status_flag = DEVICE_STATUS_DEVICE_ERROR;
 	device_set_error(DEVICE(self),
-	    vstrallocf(_("Can't open tape device %s: %s"), self->private->device_filename, strerror(errno)),
+	    g_strdup_printf(_("Can't open tape device %s: %s"), self->private->device_filename, strerror(errno)),
 	    status_flag);
         return -1;
     }
@@ -756,14 +773,14 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
     new_status = tape_is_tape_device(fd);
     if (new_status & DEVICE_STATUS_DEVICE_ERROR) {
 	device_set_error(DEVICE(self),
-	    vstrallocf(_("File %s is not a tape device"), self->private->device_filename),
+	    g_strdup_printf(_("File %s is not a tape device"), self->private->device_filename),
 	    new_status);
         robust_close(fd);
         return -1;
     }
     if (new_status & DEVICE_STATUS_VOLUME_MISSING) {
 	device_set_error(DEVICE(self),
-	    vstrallocf(_("Tape device %s is not ready or is empty"), self->private->device_filename),
+	    g_strdup_printf(_("Tape device %s is not ready or is empty"), self->private->device_filename),
 	    new_status);
         robust_close(fd);
         return -1;
@@ -772,14 +789,14 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
     new_status = tape_is_ready(fd, self);
     if (new_status & DEVICE_STATUS_VOLUME_MISSING) {
 	device_set_error(DEVICE(self),
-	    vstrallocf(_("Tape device %s is empty"), self->private->device_filename),
+	    g_strdup_printf(_("Tape device %s is empty"), self->private->device_filename),
 	    new_status);
         robust_close(fd);
         return -1;
     }
     if (new_status != DEVICE_STATUS_SUCCESS) {
 	device_set_error(DEVICE(self),
-	    vstrallocf(_("Tape device %s is not ready or is empty"), self->private->device_filename),
+	    g_strdup_printf(_("Tape device %s is not ready or is empty"), self->private->device_filename),
 	    new_status);
         robust_close(fd);
         return -1;
@@ -822,7 +839,7 @@ tape_device_open_device (Device * dself, char * device_name,
     self = TAPE_DEVICE(dself);
 
     self->fd = -1;
-    self->private->device_filename = stralloc(device_node);
+    self->private->device_filename = g_strdup(device_node);
 
     /* Set tape drive/OS info */
     bzero(&val, sizeof(val));
@@ -905,7 +922,7 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
     /* Rewind it. */
     if (!tape_rewind(self->fd)) {
 	device_set_error(dself,
-	    vstrallocf(_("Error rewinding device %s to read label: %s"),
+	    g_strdup_printf(_("Error rewinding device %s to read label: %s"),
 		    self->private->device_filename, strerror(errno)),
 	      DEVICE_STATUS_DEVICE_ERROR
 	    | DEVICE_STATUS_VOLUME_ERROR);
@@ -929,7 +946,7 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
         /* I/O error. */
 	switch (result) {
 	case RESULT_NO_DATA:
-	    msg = stralloc(_("no data"));
+	    msg = g_strdup(_("no data"));
             new_status = (DEVICE_STATUS_VOLUME_ERROR |
 	                  DEVICE_STATUS_VOLUME_UNLABELED);
 	    header = dself->volume_header = g_new(dumpfile_t, 1);
@@ -937,7 +954,7 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
 	    break;
 
 	case RESULT_SMALL_BUFFER:
-	    msg = stralloc(_("block size too small"));
+	    msg = g_strdup(_("block size too small"));
             new_status = (DEVICE_STATUS_DEVICE_ERROR |
 	                  DEVICE_STATUS_VOLUME_ERROR);
 	    header = dself->volume_header = g_new(dumpfile_t, 1);
@@ -946,7 +963,8 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
 	    break;
 
 	default:
-	    msg = stralloc(_("unknown error"));
+	    msg = g_strdup(_("unknown error"));
+	    // fallthrough
 	case RESULT_ERROR:
             new_status = (DEVICE_STATUS_DEVICE_ERROR |
 	                  DEVICE_STATUS_VOLUME_ERROR |
@@ -977,7 +995,7 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
     amfree(header_buffer);
     if (header->type != F_TAPESTART) {
 	device_set_error(dself,
-		stralloc(_("No tapestart header -- unlabeled device?")),
+		g_strdup(_("No tapestart header -- unlabeled device?")),
 		DEVICE_STATUS_VOLUME_UNLABELED);
         return dself->status;
     }
@@ -991,7 +1009,7 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
     return dself->status;
 }
 
-static gboolean
+static DeviceWriteResult
 tape_device_write_block(Device * pself, guint size, gpointer data) {
     TapeDevice * self;
     char *replacement_buffer = NULL;
@@ -1001,7 +1019,7 @@ tape_device_write_block(Device * pself, guint size, gpointer data) {
     self = TAPE_DEVICE(pself);
 
     g_assert(self->fd >= 0);
-    if (device_in_error(self)) return FALSE;
+    if (device_in_error(self)) return WRITE_FAILED;
 
     /* zero out to the end of a short block -- tape devices only write
      * whole blocks. */
@@ -1011,7 +1029,7 @@ tape_device_write_block(Device * pself, guint size, gpointer data) {
 	    device_set_error(pself,
 		g_strdup(_("failed to allocate memory")),
 		DEVICE_STATUS_DEVICE_ERROR);
-	    return FALSE;
+	    return WRITE_FAILED;
 	}
         memcpy(replacement_buffer, data, size);
         bzero(replacement_buffer+size, pself->block_size-size);
@@ -1029,19 +1047,20 @@ tape_device_write_block(Device * pself, guint size, gpointer data) {
 
 	case RESULT_NO_SPACE:
 	    device_set_error(pself,
-		stralloc(_("No space left on device")),
+		g_strdup(_("No space left on device")),
 		DEVICE_STATUS_VOLUME_ERROR);
 	    pself->is_eom = TRUE;
-	    return FALSE;
+	    return WRITE_FAILED;
 
 	default:
-	    msg = stralloc(_("unknown error"));
+	    msg = g_strdup(_("unknown error"));
+	    // fallthrough
 	case RESULT_ERROR:
 	    device_set_error(pself,
 		g_strdup_printf(_("Error writing block: %s"), msg),
 		DEVICE_STATUS_DEVICE_ERROR);
 	    amfree(msg);
-	    return FALSE;
+	    return WRITE_FAILED;
     }
 
     pself->block++;
@@ -1049,11 +1068,11 @@ tape_device_write_block(Device * pself, guint size, gpointer data) {
     pself->bytes_written += size;
     g_mutex_unlock(pself->device_mutex);
 
-    return TRUE;
+    return WRITE_SUCCEED;
 }
 
 static int tape_device_read_block (Device * pself, gpointer buf,
-                                   int * size_req) {
+                                   int * size_req, int max_block G_GNUC_UNUSED) {
     TapeDevice * self;
     int size;
     IoResult result;
@@ -1119,15 +1138,16 @@ static int tape_device_read_block (Device * pself, gpointer buf,
 	pself->in_file = FALSE;
 	g_mutex_unlock(pself->device_mutex);
 	device_set_error(pself,
-	    stralloc(_("EOF")),
+	    g_strdup(_("EOF")),
 	    DEVICE_STATUS_SUCCESS);
         return -1;
 
     default:
-	msg = stralloc(_("unknown error"));
+	msg = g_strdup(_("unknown error"));
+	// fallthrough
     case RESULT_ERROR:
 	device_set_error(pself,
-	    vstrallocf(_("Error reading from tape device: %s"), msg),
+	    g_strdup_printf(_("Error reading from tape device: %s"), msg),
 	    DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
 	amfree(msg);
         return -1;
@@ -1152,7 +1172,7 @@ static gboolean write_tapestart_header(TapeDevice * self, char * label,
      header_buf = device_build_amanda_header(d_self, header, NULL);
      if (header_buf == NULL) {
 	 device_set_error(d_self,
-	    stralloc(_("Tapestart header won't fit in a single block!")),
+	    g_strdup(_("Tapestart header won't fit in a single block!")),
 	    DEVICE_STATUS_DEVICE_ERROR);
 	 dumpfile_free(header);
          return FALSE;
@@ -1181,7 +1201,7 @@ static gboolean write_tapestart_header(TapeDevice * self, char * label,
 
      if (!tape_weof(self->fd, 1)) {
 	device_set_error(d_self,
-			 vstrallocf(_("Error writing filemark: %s"),
+			 g_strdup_printf(_("Error writing filemark: %s"),
 				    strerror(errno)),
 			 DEVICE_STATUS_DEVICE_ERROR|DEVICE_STATUS_VOLUME_ERROR);
         /* can't tell if this was EOM or not, so assume it is */
@@ -1226,15 +1246,20 @@ tape_device_start (Device * d_self, DeviceAccessMode mode, char * label,
         if (self->write_open_errno != 0) {
             /* We tried and failed to open the device in write mode. */
 	    device_set_error(d_self,
-		vstrallocf(_("Can't open tape device %s for writing: %s"),
+		g_strdup_printf(_("Can't open tape device %s for writing: %s"),
 			    self->private->device_filename, strerror(self->write_open_errno)),
 		DEVICE_STATUS_DEVICE_ERROR | DEVICE_STATUS_VOLUME_ERROR);
             return FALSE;
-        } else if (!tape_rewind(self->fd)) {
-	    device_set_error(d_self,
-		vstrallocf(_("Error rewinding device to start: %s"), strerror(errno)),
-		DEVICE_STATUS_DEVICE_ERROR);
-	    return FALSE;
+        } else {
+	    if (!tape_device_check_writable(d_self)) {
+		return FALSE;
+	    }
+	    if (!tape_rewind(self->fd)) {
+		device_set_error(d_self,
+			g_strdup_printf(_("Error rewinding device to start: %s"), strerror(errno)),
+			DEVICE_STATUS_DEVICE_ERROR);
+		return FALSE;
+	    }
         }
     }
 
@@ -1248,7 +1273,7 @@ tape_device_start (Device * d_self, DeviceAccessMode mode, char * label,
 
         if (!tape_device_eod(self)) {
 	    device_set_error(d_self,
-		vstrallocf(_("Couldn't seek to end of tape: %s"), strerror(errno)),
+		g_strdup_printf(_("Couldn't seek to end of tape: %s"), strerror(errno)),
 		DEVICE_STATUS_DEVICE_ERROR);
             return FALSE;
 	}
@@ -1262,7 +1287,7 @@ tape_device_start (Device * d_self, DeviceAccessMode mode, char * label,
 
         if (!tape_rewind(self->fd)) {
 	    device_set_error(d_self,
-		vstrallocf(_("Error rewinding device after reading label: %s"), strerror(errno)),
+		g_strdup_printf(_("Error rewinding device after reading label: %s"), strerror(errno)),
 		DEVICE_STATUS_DEVICE_ERROR);
             return FALSE;
         }
@@ -1275,8 +1300,10 @@ tape_device_start (Device * d_self, DeviceAccessMode mode, char * label,
             return FALSE;
         }
 
-        d_self->volume_label = newstralloc(d_self->volume_label, label);
-        d_self->volume_time = newstralloc(d_self->volume_time, timestamp);
+        g_free(d_self->volume_label);
+        d_self->volume_label = g_strdup(label);
+        g_free(d_self->volume_time);
+        d_self->volume_time = g_strdup(timestamp);
 
 	/* unset the VOLUME_UNLABELED flag, if it was set */
 	device_set_error(d_self, NULL, DEVICE_STATUS_SUCCESS);
@@ -1310,7 +1337,7 @@ static gboolean tape_device_start_file(Device * d_self,
     amanda_header = device_build_amanda_header(d_self, info, NULL);
     if (amanda_header == NULL) {
 	device_set_error(d_self,
-	    stralloc(_("Amanda file header won't fit in a single block!")),
+	    g_strdup(_("Amanda file header won't fit in a single block!")),
 	    DEVICE_STATUS_DEVICE_ERROR);
 	return FALSE;
     }
@@ -1318,7 +1345,7 @@ static gboolean tape_device_start_file(Device * d_self,
     result = tape_device_robust_write(self, amanda_header, d_self->block_size, &msg);
     if (result != RESULT_SUCCESS) {
 	device_set_error(d_self,
-	    vstrallocf(_("Error writing file header: %s"),
+	    g_strdup_printf(_("Error writing file header: %s"),
 			(result == RESULT_ERROR)? msg : _("out of space")),
 	    DEVICE_STATUS_DEVICE_ERROR);
 
@@ -1359,7 +1386,7 @@ tape_device_finish_file (Device * d_self) {
 
     if (!tape_weof(self->fd, 1)) {
 	device_set_error(d_self,
-		vstrallocf(_("Error writing filemark: %s"), strerror(errno)),
+		g_strdup_printf(_("Error writing filemark: %s"), strerror(errno)),
 		DEVICE_STATUS_DEVICE_ERROR | DEVICE_STATUS_VOLUME_ERROR);
         /* can't tell if this was EOM or not, so assume it is */
         d_self->is_eom = TRUE;
@@ -1406,7 +1433,7 @@ reseek:
         if (!tape_device_fsf(self, difference)) {
             tape_rewind(self->fd);
 	    device_set_error(d_self,
-		vstrallocf(_("Could not seek forward to file %d"), file),
+		g_strdup_printf(_("Could not seek forward to file %d"), file),
 		DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
             return NULL;
         }
@@ -1419,7 +1446,7 @@ reseek:
 	    if (!tape_bsf(self->fd, -difference + 1)) {
 		tape_rewind(self->fd);
 		device_set_error(d_self,
-		    vstrallocf(_("Could not seek backward to file %d"), file),
+		    g_strdup_printf(_("Could not seek backward to file %d"), file),
 		    DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
 		return NULL;
 	    }
@@ -1429,7 +1456,7 @@ reseek:
 	    if (!tape_device_fsf(self, 1)) {
 		tape_rewind(self->fd);
 		device_set_error(d_self,
-		    vstrallocf(_("Could not seek forward to file %d"), file),
+		    g_strdup_printf(_("Could not seek forward to file %d"), file),
 		    DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
 		return NULL;
 	    }
@@ -1437,7 +1464,7 @@ reseek:
 	    /* no BSF, so just rewind and seek forward */
 	    if (!tape_rewind(self->fd)) {
 		device_set_error(d_self,
-		    vstrallocf(_("Could not rewind device while emulating BSF")),
+		    g_strdup(_("Could not rewind device while emulating BSF")),
 		    DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
 		return FALSE;
 	    }
@@ -1445,7 +1472,7 @@ reseek:
 	    if (!tape_device_fsf(self, file)) {
 		tape_rewind(self->fd);
 		device_set_error(d_self,
-		    vstrallocf(_("Could not seek forward to file %d"), file),
+		    g_strdup_printf(_("Could not seek forward to file %d"), file),
 		    DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
 		return NULL;
 	    }
@@ -1458,7 +1485,7 @@ reseek:
     got_file = tape_fileno(self->fd);
     if (got_file >= 0 && (guint)got_file != file) {
 	device_set_error(d_self,
-		vstrallocf(_("Could not seek to file %d correctly; got %d"),
+		g_strdup_printf(_("Could not seek to file %d correctly; got %d"),
 			    file, got_file),
 		DEVICE_STATUS_DEVICE_ERROR);
 	d_self->file = (guint)got_file;
@@ -1490,11 +1517,11 @@ reseek:
             return make_tapeend_header();
 
 	case RESULT_SMALL_BUFFER:
-	    msg = stralloc(_("block size too small"));
+	    msg = g_strdup(_("block size too small"));
 	    break;
 
 	default:
-	    msg = stralloc(_("unknown error"));
+	    msg = g_strdup(_("unknown error"));
 	case RESULT_ERROR:
 	    break;
         }
@@ -1534,7 +1561,7 @@ reseek:
     default:
         tape_rewind(self->fd);
 	device_set_error(d_self,
-	    stralloc(_("Invalid amanda header while reading file header")),
+	    g_strdup(_("Invalid amanda header while reading file header")),
 	    DEVICE_STATUS_VOLUME_ERROR);
         amfree(rval);
         return NULL;
@@ -1562,14 +1589,14 @@ tape_device_seek_block (Device * d_self, guint64 block) {
     if (difference > 0) {
         if (!tape_device_fsr(self, difference)) {
 	    device_set_error(d_self,
-		vstrallocf(_("Could not seek forward to block %ju: %s"), (uintmax_t)block, strerror(errno)),
+		g_strdup_printf(_("Could not seek forward to block %ju: %s"), (uintmax_t)block, strerror(errno)),
 		DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
             return FALSE;
 	}
     } else if (difference < 0) {
         if (!tape_device_bsr(self, difference, d_self->file, d_self->block)) {
 	    device_set_error(d_self,
-		vstrallocf(_("Could not seek backward to block %ju: %s"), (uintmax_t)block, strerror(errno)),
+		g_strdup_printf(_("Could not seek backward to block %ju: %s"), (uintmax_t)block, strerror(errno)),
 		DEVICE_STATUS_VOLUME_ERROR | DEVICE_STATUS_DEVICE_ERROR);
             return FALSE;
 	}
@@ -1582,6 +1609,8 @@ tape_device_seek_block (Device * d_self, guint64 block) {
 static gboolean
 tape_device_eject (Device * d_self) {
     TapeDevice * self;
+    gboolean eject_opened_tape = FALSE;
+    gboolean result = FALSE;
 
     self = TAPE_DEVICE(d_self);
 
@@ -1594,27 +1623,35 @@ tape_device_eject (Device * d_self) {
          * approppriate error status */
         if (self->fd == -1)
             return FALSE;
+	eject_opened_tape = TRUE;
     }
 
     /* Rewind it. */
     if (!tape_rewind(self->fd)) {
 	device_set_error(d_self,
-	    vstrallocf(_("Error rewinding device %s before ejecting: %s"),
+	    g_strdup_printf(_("Error rewinding device %s before ejecting: %s"),
 		       self->private->device_filename, strerror(errno)),
 	      DEVICE_STATUS_DEVICE_ERROR
 	    | DEVICE_STATUS_VOLUME_ERROR);
-	return FALSE;
+	result = FALSE;
+	goto eject_finish;
     }
 
-    if (tape_offl(self->fd))
-	return TRUE;
+    if (tape_offl(self->fd)) {
+	result = TRUE;
+	goto eject_finish;
+    }
 
     device_set_error(d_self,
-	vstrallocf(_("Error ejecting device %s: %s\n"),
+	g_strdup_printf(_("Error ejecting device %s: %s\n"),
 		   self->private->device_filename, strerror(errno)),
 	  DEVICE_STATUS_DEVICE_ERROR);
 
-    return FALSE;
+eject_finish:
+    if (eject_opened_tape) {
+	device_finish(d_self);
+    }
+    return result;
 }
 
 static gboolean
@@ -1663,7 +1700,7 @@ tape_device_finish (Device * d_self) {
 	header = device_build_amanda_header(d_self, &file, NULL);
 	if (!header) {
 	    device_set_error(d_self,
-		stralloc(_("Amanda file header won't fit in a single block!")),
+		g_strdup(_("Amanda file header won't fit in a single block!")),
 		DEVICE_STATUS_DEVICE_ERROR);
 	    goto finish_error;
 	}
@@ -1671,7 +1708,7 @@ tape_device_finish (Device * d_self) {
 	result = tape_device_robust_write(self, header, d_self->block_size, &msg);
 	if (result != RESULT_SUCCESS) {
 	    device_set_error(d_self,
-		vstrallocf(_("Error writing file header: %s"),
+		g_strdup_printf(_("Error writing file header: %s"),
 			    (result == RESULT_ERROR)? msg : _("out of space")),
 		DEVICE_STATUS_DEVICE_ERROR);
 	    amfree(header);
@@ -1684,7 +1721,7 @@ tape_device_finish (Device * d_self) {
     /* Rewind (the kernel will write a filemark first) */
     if (!tape_rewind(self->fd)) {
 	device_set_error(d_self,
-	    vstrallocf(_("Couldn't rewind device to finish: %s"), strerror(errno)),
+	    g_strdup_printf(_("Couldn't rewind device to finish: %s"), strerror(errno)),
 	    DEVICE_STATUS_DEVICE_ERROR);
 	goto finish_error;
     }
@@ -1708,6 +1745,38 @@ finish_error:
     return FALSE;
 }
 
+static gboolean
+tape_device_check_writable(Device * d_self)
+{
+    TapeDevice *self;
+#ifdef GMT_WR_PROT
+    struct mtget get;
+#endif
+
+    self = TAPE_DEVICE(d_self);
+
+    if (device_in_error(self))
+	return TRUE;
+
+#ifdef GMT_WR_PROT
+    if (ioctl(self->fd, MTIOCGET, &get) == 0) {
+	if (GMT_WR_PROT(get.mt_gstat)) {
+	    device_set_error(d_self,
+		g_strdup_printf("Device '%s' is write protected", self->private->device_filename),
+		DEVICE_STATUS_VOLUME_ERROR);
+	    return FALSE;
+	}
+    } else {
+	device_set_error(d_self,
+	    g_strdup_printf("ioctl failed on device '%s'", self->private->device_filename),
+	    DEVICE_STATUS_VOLUME_ERROR);
+	return FALSE;
+    }
+#endif
+
+    return TRUE;
+}
+
 /* Works just like read(), except for the following:
  * 1) Retries on EINTR & friends.
  * 2) Stores count in parameter, not return value.
@@ -1885,9 +1954,13 @@ tape_device_robust_write (TapeDevice * self, void * buf, int count, char **errms
             }
 #endif
             return RESULT_NO_SPACE;
-        } else {
+        } else if (errno == EPERM) {
+	    *errmsg = g_strdup_printf("write to the '%s' device failed: %s, maybe the tab on the tape is set at the read-only possition?",
+			    self->private->device_filename, strerror(errno));
+	    return RESULT_ERROR;
+	} else {
             /* WTF */
-	    *errmsg = vstrallocf(_("Kernel gave unexpected write() result of \"%s\" on device %s"),
+	    *errmsg = g_strdup_printf("write to the '%s' device failed: %s",
 			    strerror(errno), self->private->device_filename);
             return RESULT_ERROR;
         }
@@ -1955,7 +2028,12 @@ static int drain_tape_blocks(TapeDevice * self, int count) {
                     amfree(buffer);
                     return -1;
                 } else {
-                    buffer = realloc(buffer, buffer_size);
+                   char *new_buffer = realloc(buffer, buffer_size);
+		    if (new_buffer == NULL) {
+			amfree(buffer);
+			return -1;
+		    }
+		    buffer = new_buffer;
                     continue;
                 }
             }
@@ -2079,7 +2157,7 @@ tape_device_eod (TapeDevice * self) {
 static Device *
 tape_device_factory (char * device_name, char * device_type, char * device_node) {
     Device * rval;
-    g_assert(0 == strcmp(device_type, "tape"));
+    g_assert(g_str_equal(device_type, "tape"));
     rval = DEVICE(g_object_new(TYPE_TAPE_DEVICE, NULL));
     device_open_device(rval, device_name, device_type, device_node);
     return rval;
@@ -2174,7 +2252,7 @@ gint tape_eod(int fd) {
 
     /* Ignored result. This is just to flush buffers. */
     mt.mt_op = MTNOP;
-    ioctl(fd, MTIOCTOP, &mt);
+    (void)ioctl(fd, MTIOCTOP, &mt);
 
     if (0 != ioctl(fd, MTIOCGET, &get))
         return TAPE_POSITION_UNKNOWN;
