@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +17,13 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
 #include "amxfer.h"
 #include "xfer-device.h"
-#include "arglist.h"
 #include "conffile.h"
 
 /* A transfer destination that writes an entire dumpfile to one or more files
@@ -242,6 +242,8 @@ typedef struct XferDestTaperCacher {
 
     /* number of slabs in a part */
     guint64 slabs_per_part;
+
+    crc_t crc_before_part;
 } XferDestTaperCacher;
 
 static GType xfer_dest_taper_cacher_get_type(void);
@@ -312,7 +314,7 @@ alloc_slab(
 	    DBG(9, "waiting for available slab");
 	    g_cond_wait(self->slab_free_cond, self->slab_mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "alloc_slab done waiting");
 
         if (elt->cancelled)
             return NULL;
@@ -489,7 +491,7 @@ disk_cache_thread(
 	    DBG(9, "waiting for a disk slab");
 	    g_cond_wait(self->slab_cond, self->slab_mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "disk_cache_thread done waiting");
 	g_mutex_unlock(self->slab_mutex);
 
 	if (elt->cancelled)
@@ -506,7 +508,7 @@ disk_cache_thread(
             DBG(9, "waiting for the disk slab to become current and un-paused");
             g_cond_wait(self->state_cond, self->state_mutex);
         }
-	DBG(9, "done waiting");
+	DBG(9, "disk_cache_thread done waiting");
 
 	stop_serial = self->part_stop_serial;
 	g_mutex_unlock(self->state_mutex);
@@ -523,7 +525,7 @@ disk_cache_thread(
 		DBG(9, "waiting for the next disk slab");
 		g_cond_wait(self->slab_cond, self->slab_mutex);
 	    }
-	    DBG(9, "done waiting");
+	    DBG(9, "disk_cache_thread done waiting");
 
             if (elt->cancelled)
                 break;
@@ -616,7 +618,7 @@ slab_source_prebuffer(
 	DBG(9, "prebuffering wait");
 	g_cond_wait(self->slab_cond, self->slab_mutex);
     }
-    DBG(9, "done waiting");
+    DBG(9, "slab_source_prebuffer done waiting");
 
     if (elt->cancelled) {
 	self->last_part_successful = FALSE;
@@ -692,7 +694,7 @@ slab_source_setup(
 		DBG(9, "waiting for disk_cache_thread to set disk_cache_read_fd");
 		g_cond_wait(self->state_cond, self->state_mutex);
 	    }
-	    DBG(9, "done waiting");
+	    DBG(9, "slab_source_setup done waiting");
 	    g_mutex_unlock(self->state_mutex);
 
 	    if (elt->cancelled) {
@@ -746,13 +748,13 @@ slab_source_get_from_disk(
     /* NOTE: slab_mutex is held, but we don't need it here, so release it for the moment */
     g_mutex_unlock(self->slab_mutex);
 
-    bytes_read = full_read(self->disk_cache_read_fd,
-			   state->tmp_slab->base,
-			   self->slab_size);
-    if ((gsize)bytes_read < self->slab_size) {
+    bytes_read = read_fully(self->disk_cache_read_fd, state->tmp_slab->base,
+        self->slab_size, NULL);
+
+    if (bytes_read < self->slab_size) {
 	xfer_cancel_with_error(XFER_ELEMENT(xdt),
 	    _("Error reading disk cache: %s"),
-	    errno? strerror(errno) : _("Unexpected EOF"));
+	    errno ? strerror(errno) : _("Unexpected EOF"));
 	goto fatal_error;
     }
 
@@ -795,7 +797,7 @@ slab_source_get(
 	    DBG(9, "waiting for the next slab");
 	    g_cond_wait(self->slab_cond, self->slab_mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "slab_source_get done waiting");
 
 	if (elt->cancelled)
 	    goto fatal_error;
@@ -845,9 +847,9 @@ write_slab_to_device(
 
     while (remaining && !elt->cancelled) {
 	gsize write_size = MIN(self->block_size, remaining);
-	gboolean ok;
+	DeviceWriteResult ok;
 	ok = device_write_block(self->device, write_size, buf);
-	if (!ok) {
+	if (ok != WRITE_SUCCEED) {
             self->bytes_written += slab->size - remaining;
 
             /* TODO: handle an error without is_eom
@@ -878,6 +880,7 @@ static XMsg *
 device_thread_write_part(
     XferDestTaperCacher *self)
 {
+    XferElement *elt = XFER_ELEMENT(self);
     GTimer *timer = g_timer_new();
     XMsg *msg;
     slab_source_state src_state = {0, 0};
@@ -889,6 +892,7 @@ device_thread_write_part(
 
     self->last_part_successful = FALSE;
     self->bytes_written = 0;
+    self->crc_before_part = elt->crc;
 
     if (!device_start_file(self->device, self->part_header)) {
 	failed = 1;
@@ -949,6 +953,8 @@ part_done:
     if (!failed) {
 	self->last_part_successful = TRUE;
 	self->no_more_parts = eof;
+    } else {
+	elt->crc = self->crc_before_part;
     }
 
     g_timer_stop(timer);
@@ -1025,7 +1031,7 @@ device_thread(
 	    DBG(9, "waiting to be unpaused");
 	    g_cond_wait(self->state_cond, self->state_mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "device_thread done waiting");
 
         if (elt->cancelled)
 	    break;
@@ -1057,11 +1063,17 @@ device_thread(
     if (self->disk_cache_thread)
         g_thread_join(self->disk_cache_thread);
 
+    g_debug("sending XMSG_CRC message");
+    g_debug("xfer-dest-taper-cacher CRC %08x      size %lld",
+	    crc32_finish(&elt->crc), (long long)elt->crc.size);
+    msg = xmsg_new(XFER_ELEMENT(self), XMSG_CRC, 0);
+    msg->crc = crc32_finish(&elt->crc);
+    msg->size = elt->crc.size;
+    xfer_queue_message(elt->xfer, msg);
+
     /* tell the main thread we're done */
     xfer_queue_message(XFER_ELEMENT(self)->xfer, xmsg_new(XFER_ELEMENT(self), XMSG_DONE, 0));
 
-    g_debug("xfer-dest-taper-cacher CRC %08x:%lld",
-	    crc32_finish(&elt->crc), (long long)elt->crc.size);
     return NULL;
 }
 
@@ -1113,7 +1125,7 @@ add_reader_slab_to_train(
 }
 
 static void
-push_buffer_impl(
+push_buffer_static_impl(
     XferElement *elt,
     gpointer buf,
     size_t size)
@@ -1121,7 +1133,7 @@ push_buffer_impl(
     XferDestTaperCacher *self = (XferDestTaperCacher *)elt;
     char *p;
 
-    DBG(3, "push_buffer(%p, %ju)", buf, (uintmax_t)size);
+    DBG(3, "push_buffer_static(%p, %ju)", buf, (uintmax_t)size);
 
     /* do nothing if cancelled */
     if (G_UNLIKELY(elt->cancelled)) {
@@ -1129,7 +1141,7 @@ push_buffer_impl(
     }
 
     /* handle EOF */
-    if (G_UNLIKELY(buf == NULL)) {
+    if (G_UNLIKELY(buf == NULL) || size == 0) {
 	/* send off the last, probably partial slab */
 	g_mutex_lock(self->slab_mutex);
 
@@ -1191,6 +1203,17 @@ push_buffer_impl(
     }
 
 free_and_finish:
+    return;
+}
+
+static void
+push_buffer_impl(
+    XferElement *elt,
+    gpointer buf,
+    size_t size)
+{
+    push_buffer_static_impl(elt, buf, size);
+
     if (buf)
         g_free(buf);
 }
@@ -1417,13 +1440,15 @@ class_init(
     XferDestTaperClass *xdt_klass = XFER_DEST_TAPER_CLASS(selfc);
     GObjectClass *goc = G_OBJECT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, 1, 1},
-	{ XFER_MECH_NONE, XFER_MECH_NONE, 0, 0},
+	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(1), XFER_NTHREADS(1), XFER_NALLOC(0) },
+	{ XFER_MECH_PUSH_BUFFER_STATIC, XFER_MECH_NONE, XFER_NROPS(1), XFER_NTHREADS(1), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) }
     };
 
     klass->start = start_impl;
     klass->cancel = cancel_impl;
     klass->push_buffer = push_buffer_impl;
+    klass->push_buffer_static = push_buffer_static_impl;
     xdt_klass->start_part = start_part_impl;
     xdt_klass->use_device = use_device_impl;
     xdt_klass->get_part_bytes_written = get_part_bytes_written_impl;
@@ -1440,7 +1465,7 @@ xfer_dest_taper_cacher_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferDestTaperCacherClass),
             (GBaseInitFunc) NULL,
