@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,15 +17,15 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
 #include "amxfer.h"
 #include "xfer-device.h"
-#include "arglist.h"
 #include "conffile.h"
+#include "device.h"
 
 /* A transfer destination that writes an entire dumpfile to one or more files
  * on one or more devices, without any caching.  This destination supports both
@@ -92,24 +93,11 @@ typedef struct XferDestTaperSplitter {
      * for streaming */
     GThread *device_thread;
 
-    /* Ring Buffer
-     *
-     * This buffer holds MAX_MEMORY bytes of data (rounded up to the next
-     * blocksize), and serves as the interface between the device_thread and
-     * the thread calling push_buffer.  Ring_length is the total length of the
-     * buffer in bytes, while ring_count is the number of data bytes currently
-     * in the buffer.  The ring_add_cond is signalled when data is added to the
-     * buffer, while ring_free_cond is signalled when data is removed.  Both
-     * are governed by ring_mutex, and both are signalled when the transfer is
-     * cancelled.
-     */
-
+    /* Ring Buffer */
     GMutex *ring_mutex;
-    GCond *ring_add_cond, *ring_free_cond;
-    gchar *ring_buffer;
-    gsize ring_length, ring_count;
-    gsize ring_head, ring_tail;
-    gboolean ring_head_at_eof;
+    GCond *ring_cond;
+    mem_ring_t *mem_ring;
+    gboolean    ring_ready;
 
     /* Element State
      *
@@ -117,7 +105,7 @@ typedef struct XferDestTaperSplitter {
      * parameters).  Note that the device_thread holdes this mutex for the
      * entire duration of writing a part.
      *
-     * state_mutex should always be locked before ring_mutex, if both are to be
+     * state_mutex should always be locked before mem_ring->mutex, if both are to be
      * held simultaneously.
      */
     GMutex *state_mutex;
@@ -130,6 +118,7 @@ typedef struct XferDestTaperSplitter {
 
     /* bytes to read from cached slices before reading from the ring buffer */
     guint64 bytes_to_read_from_slices;
+    guint64 max_memory;
 
     /* part number in progress */
     volatile guint64 partnum;
@@ -153,8 +142,15 @@ typedef struct XferDestTaperSplitter {
      * the links in this list. */
     FileSlice *part_slices;
     GMutex *part_slices_mutex;
+
+    crc_t crc_before_part;
+    int more_space;
+
 } XferDestTaperSplitter;
 
+static DeviceWriteResult retry_write(XferDestTaperSplitter *self, gsize to_write,
+				     gpointer buf);
+
 static GType xfer_dest_taper_splitter_get_type(void);
 #define XFER_DEST_TAPER_SPLITTER_TYPE (xfer_dest_taper_splitter_get_type())
 #define XFER_DEST_TAPER_SPLITTER(obj) G_TYPE_CHECK_INSTANCE_CAST((obj), xfer_dest_taper_splitter_get_type(), XferDestTaperSplitter)
@@ -282,7 +278,7 @@ iterator_get_block(
 
     while (bytes_needed > 0) {
 	gsize read_size;
-	int bytes_read;
+	gsize bytes_read;
 
 	if (iter->cur_fd < 0) {
 	    guint64 offset;
@@ -310,14 +306,12 @@ iterator_get_block(
 	}
 
 	read_size = MIN(iter->slice_remaining, bytes_needed);
-	bytes_read = full_read(iter->cur_fd,
-			       (gchar *)buf + buf_offset,
-			       read_size);
-	if (bytes_read < 0 || (gsize)bytes_read < read_size) {
-	    xfer_cancel_with_error(elt,
-		_("Error reading '%s': %s"),
+	bytes_read = read_fully(iter->cur_fd, (gchar *)buf + buf_offset, read_size,
+	    NULL);
+	if (bytes_read < read_size) {
+	    xfer_cancel_with_error(elt, _("Error reading '%s': %s"),
 		iter->slice->filename,
-		errno? strerror(errno) : _("Unexpected EOF"));
+		errno ? strerror(errno) : _("Unexpected EOF"));
 	    return NULL;
 	}
 
@@ -362,56 +356,109 @@ iterator_free(
  * Called with the ring mutex held. */
 static gsize
 device_thread_wait_for_block(
-    XferDestTaperSplitter *self)
+    XferDestTaperSplitter *self,
+    gboolean *eof_flag)
 {
     XferElement *elt = XFER_ELEMENT(self);
     gsize bytes_needed = self->device->block_size;
-    gsize usable;
+    gsize usable = 0;
 
-    /* for any kind of streaming, we need to fill the entire buffer before the
-     * first byte */
-    if (self->part_bytes_written == 0 && self->streaming != STREAMING_REQUIREMENT_NONE)
-	bytes_needed = self->ring_length;
+    *eof_flag = FALSE;
 
-    while (1) {
-	/* are we ready? */
-	if (elt->cancelled)
-	    break;
+    if (self->mem_ring) {
+	gsize max_ring_block_size = MAX(self->mem_ring->producer_block_size,
+					self->mem_ring->consumer_block_size);
 
-	if (self->ring_count >= bytes_needed)
-	    break;
+	/* for any kind of streaming, we need to fill the entire buffer before the
+	 * first byte */
+	if (self->part_bytes_written == 0 && self->streaming != STREAMING_REQUIREMENT_NONE)
+	    bytes_needed = self->mem_ring->ring_size - max_ring_block_size;
 
-	if (self->ring_head_at_eof)
-	    break;
+	while (1) {
+	    /* are we ready? */
+	    if (elt->cancelled)
+		break;
 
-	/* nope - so wait */
-	g_cond_wait(self->ring_add_cond, self->ring_mutex);
+	    usable = self->mem_ring->written - self->mem_ring->readx;
+	    *eof_flag = self->mem_ring->eof_flag;
+	    if (usable >= bytes_needed) {
+		break;
+	    }
 
-	/* in STREAMING_REQUIREMENT_REQUIRED, once we decide to wait for more bytes,
-	 * we need to wait for the entire buffer to fill */
-	if (self->streaming == STREAMING_REQUIREMENT_REQUIRED)
-	    bytes_needed = self->ring_length;
+	    if (self->mem_ring->eof_flag)
+		break;
+
+	    /* nope - so wait */
+	    g_cond_wait(self->mem_ring->add_cond, self->mem_ring->mutex);
+
+	    /* in STREAMING_REQUIREMENT_REQUIRED, once we decide to wait for more bytes,
+	     * we need to wait for the entire buffer to fill */
+	    if (self->streaming == STREAMING_REQUIREMENT_REQUIRED)
+		bytes_needed = self->mem_ring->ring_size - max_ring_block_size;
+	}
+
+    } else { // shm_ring
+	gsize max_ring_block_size = MAX(elt->shm_ring->mc->producer_block_size,
+					elt->shm_ring->mc->consumer_block_size);
+	if (self->part_bytes_written == 0 && self->streaming != STREAMING_REQUIREMENT_NONE)
+	    bytes_needed = elt->shm_ring->ring_size - max_ring_block_size;
+
+	while (!elt->cancelled &&
+	       !elt->shm_ring->mc->cancelled) {
+
+	    if (shm_ring_sem_wait(elt->shm_ring, elt->shm_ring->sem_read) != 0)
+		break;
+	    usable = elt->shm_ring->mc->written - elt->shm_ring->mc->readx;
+	    *eof_flag = elt->shm_ring->mc->eof_flag;
+
+	    if (elt->cancelled || elt->shm_ring->mc->cancelled)
+		break;
+
+	    if (usable >= bytes_needed)
+		break;
+
+	    if (*eof_flag)
+		break;
+
+
+	    if (self->streaming == STREAMING_REQUIREMENT_REQUIRED)
+		bytes_needed = elt->shm_ring->ring_size - max_ring_block_size;
+	}
+	if (elt->shm_ring->mc->cancelled && !elt->cancelled) {
+	    xfer_cancel_with_error(elt, "shm_ring_cancelled");
+	}
     }
-
-    usable = MIN(self->ring_count, bytes_needed);
     if (self->part_size)
        usable = MIN(usable, self->part_size - self->part_bytes_written);
 
     return usable;
 }
 
-/* Mark WRITTEN bytes as free in the ring buffer.  Called with the ring mutex
+/* Mark readx bytes as free in the ring buffer.  Called with the ring mutex
  * held. */
 static void
 device_thread_consume_block(
     XferDestTaperSplitter *self,
-    gsize written)
+    gsize readx)
 {
-    self->ring_count -= written;
-    self->ring_tail += written;
-    if (self->ring_tail >= self->ring_length)
-	self->ring_tail -= self->ring_length;
-    g_cond_broadcast(self->ring_free_cond);
+    XferElement *elt = XFER_ELEMENT(self);
+    uint64_t read_offset;
+
+    if (self->mem_ring) {
+	read_offset = self->mem_ring->read_offset + readx;
+	if (read_offset >= self->mem_ring->ring_size)
+	    read_offset -= self->mem_ring->ring_size;
+	self->mem_ring->readx += readx;
+	self->mem_ring->read_offset = read_offset;
+	g_cond_broadcast(self->mem_ring->free_cond);
+    } else { // shm_ring
+	read_offset = elt->shm_ring->mc->read_offset + readx;
+	if (read_offset >= elt->shm_ring->ring_size)
+	    read_offset -= elt->shm_ring->ring_size;
+	elt->shm_ring->mc->readx += readx;
+	elt->shm_ring->mc->read_offset = read_offset;
+	sem_post(elt->shm_ring->sem_write);
+    }
 }
 
 /* Write an entire part.  Called with the state_mutex held */
@@ -425,8 +472,10 @@ device_thread_write_part(
     enum { PART_EOF, PART_LEOM, PART_EOP, PART_FAILED } part_status = PART_FAILED;
     int fileno = 0;
     XMsg *msg;
+    void *buf;
 
     self->part_bytes_written = 0;
+    self->crc_before_part = elt->crc;
 
     g_timer_start(timer);
 
@@ -457,7 +506,7 @@ device_thread_write_part(
 
 	iterate_slices(self, &iter);
 	while (bytes_from_slices) {
-	    gboolean ok;
+	    DeviceWriteResult ok;
 
 	    if (!iterator_get_block(self, &iter, buf, to_write)) {
 		part_status = PART_FAILED;
@@ -468,15 +517,28 @@ device_thread_write_part(
 	    /* note that it's OK to reference these ring_* vars here, as they
 	     * are static at this point */
 	    ok = device_write_block(self->device, (guint)to_write, buf);
+	    DBG(8, "writing %ju bytes from slice to device", (uintmax_t)to_write);
 
-	    if (!ok) {
+	    if (ok == WRITE_SPACE)
+		ok = retry_write(self, to_write, buf);
+
+	    if (ok == WRITE_FAILED) {
 		part_status = PART_FAILED;
 		successful = FALSE;
 		break;
+	    } else if (ok == WRITE_FULL) {
+		part_status = PART_EOP;
+		successful = FALSE;
+		break;
+	    } else if (ok == WRITE_SPACE) {
+		part_status = PART_EOP;
+		successful = FALSE;
+		break;
 	    }
 
 	    self->part_bytes_written += to_write;
 	    bytes_from_slices -= to_write;
+	    crc32_add((uint8_t *)buf, to_write, &elt->crc);
 
 	    if (self->part_size && self->part_bytes_written >= self->part_size) {
 		part_status = PART_EOP;
@@ -497,64 +559,110 @@ device_thread_write_part(
 	    goto part_done;
     }
 
-    g_mutex_lock(self->ring_mutex);
-    while (1) {
-	gsize to_write;
-	gboolean ok;
+    if (self->mem_ring)
+	g_mutex_lock(self->mem_ring->mutex);
+    while (!elt->cancelled &&
+	   (!elt->shm_ring || !elt->shm_ring->mc->cancelled)) {
+	DeviceWriteResult ok;
+	gboolean eof_flag;
 
 	/* wait for at least one block, and (if necessary) prebuffer */
-	to_write = device_thread_wait_for_block(self);
-	to_write = MIN(to_write, self->device->block_size);
-	if (elt->cancelled)
+	gsize to_writeX = device_thread_wait_for_block(self, &eof_flag);
+	if (elt->cancelled || (elt->shm_ring && elt->shm_ring->mc->cancelled))
 	    break;
 
-	if (to_write == 0) {
-	    part_status = PART_EOF;
-	    break;
-	}
+	while (to_writeX >= self->device->block_size || eof_flag) {
+	    //crc_t block_crc;
+	    gsize to_write = MIN(to_writeX, self->device->block_size);
+	    if (elt->cancelled)
+		goto part_done_unlock;
 
-	g_mutex_unlock(self->ring_mutex);
-	DBG(8, "writing %ju bytes to device", (uintmax_t)to_write);
+	    if (to_write == 0) {
+		part_status = PART_EOF;
+		goto part_done_unlock;
+	    }
 
-	/* note that it's OK to reference these ring_* vars here, as they
-	 * are static at this point */
-	ok = device_write_block(self->device, (guint)to_write,
-		self->ring_buffer + self->ring_tail);
-	g_mutex_lock(self->ring_mutex);
+	    if (self->mem_ring)
+		g_mutex_unlock(self->mem_ring->mutex);
+	    DBG(8, "writing %ju bytes to device", (uintmax_t)to_write);
 
-	if (!ok) {
-	    part_status = PART_FAILED;
-	    break;
-	}
+	    /* note that it's OK to reference these ring_* vars here, as they
+	     * are static at this point */
+	    if (self->mem_ring) {
+		buf = self->mem_ring->buffer + self->mem_ring->read_offset;
+	    } else {
+		buf = elt->shm_ring->data + elt->shm_ring->mc->read_offset;
+	    }
+	    ok = device_write_block(self->device, (guint)to_write, buf);
 
-	crc32_add((uint8_t *)(self->ring_buffer + self->ring_tail),
-		  to_write, &elt->crc);
-	self->part_bytes_written += to_write;
-	device_thread_consume_block(self, to_write);
+	    if (ok == WRITE_SPACE)
+	    ok = retry_write(self, to_write, buf);
 
-	if (self->part_size && self->part_bytes_written >= self->part_size) {
-	    part_status = PART_EOP;
-	    break;
-	} else if (self->device->is_eom) {
-	    part_status = PART_LEOM;
-	    break;
+	    if (self->mem_ring)
+		g_mutex_lock(self->mem_ring->mutex);
+
+	    if (ok == WRITE_FAILED) {
+		part_status = PART_FAILED;
+		goto part_done_unlock;
+	    } else if (ok == WRITE_FULL) {
+		part_status = PART_EOP;
+		goto part_done_unlock;
+	    } else if (ok == WRITE_SPACE) {
+		part_status = PART_EOP;
+		goto part_done_unlock;
+	    }
+
+	    crc32_add((uint8_t *)(buf),
+			 to_write, &elt->crc);
+	    self->part_bytes_written += to_write;
+	    device_thread_consume_block(self, to_write);
+
+	    if (self->part_size && self->part_bytes_written >= self->part_size) {
+		part_status = PART_EOP;
+		goto part_done_unlock;
+	    } else if (self->device->is_eom) {
+		part_status = PART_LEOM;
+		goto part_done_unlock;
+	    }
+	    to_writeX -= to_write;
 	}
     }
-    g_mutex_unlock(self->ring_mutex);
-part_done:
+part_done_unlock:
+    if (self->mem_ring)
+	g_mutex_unlock(self->mem_ring->mutex);
 
+part_done:
+    if (elt->shm_ring) {
+	if (elt->cancelled) {
+	    if (elt->shm_ring) {
+		g_debug("device_thread_write_part: cancelling shm-ring because xfer is cancelled");
+		elt->shm_ring->mc->cancelled = TRUE;
+	    }
+	} else if (elt->shm_ring->mc->cancelled) {
+	    part_status = PART_FAILED;
+	    xfer_cancel_with_error(elt, "shm_ring cancelled");
+	}
+	sem_post(elt->shm_ring->sem_read);
+	sem_post(elt->shm_ring->sem_read);
+	sem_post(elt->shm_ring->sem_read);
+	sem_post(elt->shm_ring->sem_write);
+    }
     /* if we write all of the blocks, but the finish_file fails, then likely
      * there was some buffering going on in the device driver, and the blocks
      * did not all make it to permanent storage -- so it's a failed part.  Note
      * that we try to finish_file even if the part failed, just to be thorough. */
     if (self->device->in_file) {
-	if (!device_finish_file(self->device))
+	if (!device_finish_file(self->device)) {
 	    if (!elt->cancelled) {
 		part_status = PART_FAILED;
 	    }
+	}
     }
 
     g_timer_stop(timer);
+    if (part_status == PART_FAILED) {
+	elt->crc = self->crc_before_part;
+    }
 
     msg = xmsg_new(XFER_ELEMENT(self), XMSG_PART_DONE, 0);
     msg->size = self->part_bytes_written;
@@ -570,7 +678,9 @@ part_done:
 
     if (msg->successful && msg->size > 0)
 	self->partnum++;
-    self->no_more_parts = msg->eof || (!msg->successful && !self->expect_cache_inform);
+    //self->no_more_parts = msg->eof || (!msg->successful && !self->expect_cache_inform);
+    self->no_more_parts = msg->eof;
+
 
     g_timer_destroy(timer);
 
@@ -587,24 +697,58 @@ device_thread(
 
     DBG(1, "(this is the device thread)");
 
+    if (elt->input_mech == XFER_MECH_PUSH_BUFFER) {
+	self->mem_ring = create_mem_ring();
+	init_mem_ring(self->mem_ring, self->max_memory, self->device->block_size);
+    } else if (elt->input_mech == XFER_MECH_MEM_RING) {
+	self->mem_ring = xfer_element_get_mem_ring(elt->upstream);
+	mem_ring_consumer_set_size(self->mem_ring, self->max_memory, self->device->block_size);
+    } else if (elt->input_mech == XFER_MECH_SHM_RING) {
+	shm_ring_consumer_set_size(elt->shm_ring, self->max_memory, self->device->block_size);
+	if (elt->input_mech == XFER_MECH_SHM_RING &&
+	    elt->shm_ring->mc->cancelled) {
+	    g_debug("A shm_ring is cancelled");
+	    if (!elt->cancelled) {
+		xfer_cancel_with_error(XFER_ELEMENT(self)->upstream,
+			_("failed to set shm-ring"));
+	    }
+	    goto device_thread_done;
+	}
+    }
+    crc32_init(&elt->crc);
+
     /* This is the outer loop, that loops once for each split part written to
      * tape. */
+    g_mutex_lock(self->ring_mutex);
+    self->ring_ready = TRUE;
+    g_cond_broadcast(self->ring_cond);
+    g_mutex_unlock(self->ring_mutex);
     g_mutex_lock(self->state_mutex);
     while (1) {
 	/* wait until the main thread un-pauses us, and check that we have
 	 * the relevant device info available (block_size) */
 	while (self->paused && !elt->cancelled) {
-	    DBG(9, "waiting to be unpaused");
+	    DBG(9, "device_thread waiting to be unpaused");
 	    g_cond_wait(self->state_cond, self->state_mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "device_thread done waiting");
 
         if (elt->cancelled)
 	    break;
 
-	DBG(2, "beginning to write part");
+	if (elt->input_mech == XFER_MECH_SHM_RING &&
+	    elt->shm_ring->mc->cancelled) {
+	    g_debug("B shm_ring is cancelled");
+	    if (!elt->cancelled) {
+		xfer_cancel_with_error(XFER_ELEMENT(self)->upstream,
+		    _("shm-ring cancelled"));
+	    }
+	    break;
+	}
+
+	DBG(2, "device_thread beginning to write part");
 	msg = device_thread_write_part(self);
-	DBG(2, "done writing part");
+	DBG(2, "device_thread done writing part");
 
 	if (!msg) /* cancelled */
 	    break;
@@ -625,10 +769,23 @@ device_thread(
     }
     g_mutex_unlock(self->state_mutex);
 
+    // notify the producer that everythinng is read
+    if (elt->input_mech == XFER_MECH_SHM_RING) {
+	sem_post(elt->shm_ring->sem_write);
+    }
+
+    g_debug("device_thread sending XMSG_CRC message");
+    DBG(2, "xfer-dest-taper-splitter CRC: %08x      size %lld",
+	   crc32_finish(&elt->crc), (long long)elt->crc.size);
+    msg = xmsg_new(XFER_ELEMENT(self), XMSG_CRC, 0);
+    msg->crc = crc32_finish(&elt->crc);
+    msg->size = elt->crc.size;
+    xfer_queue_message(elt->xfer, msg);
+
+device_thread_done:
     /* tell the main thread we're done */
     xfer_queue_message(XFER_ELEMENT(self)->xfer, xmsg_new(XFER_ELEMENT(self), XMSG_DONE, 0));
-    g_debug("xfer-dest-taper-splitter CRC: %08x:%lld",
-	    crc32_finish(&elt->crc), (long long)elt->crc.size);
+
     return NULL;
 }
 
@@ -652,53 +809,63 @@ push_buffer_impl(
         goto free_and_finish;
     }
 
+    if (G_UNLIKELY(!self->ring_ready)) {
+	g_mutex_lock(self->ring_mutex);
+	while (!self->ring_ready && !elt->cancelled) {
+	    g_cond_wait(self->ring_cond, self->ring_mutex);
+	}
+	if (elt->cancelled)
+	    goto unlock_and_free_and_finish;
+	g_mutex_unlock(self->ring_mutex);
+    }
+
     /* handle EOF */
     if (G_UNLIKELY(buf == NULL)) {
 	/* indicate EOF to the device thread */
-	g_mutex_lock(self->ring_mutex);
-	self->ring_head_at_eof = TRUE;
-	g_cond_broadcast(self->ring_add_cond);
-	g_mutex_unlock(self->ring_mutex);
+	g_mutex_lock(self->mem_ring->mutex);
+	self->mem_ring->eof_flag = TRUE;
+	g_cond_broadcast(self->mem_ring->add_cond);
+	g_mutex_unlock(self->mem_ring->mutex);
 	goto free_and_finish;
     }
 
     /* push the block into the ring buffer, in pieces if necessary */
-    g_mutex_lock(self->ring_mutex);
+    g_mutex_lock(self->mem_ring->mutex);
     while (size > 0) {
 	gsize avail;
 
 	/* wait for some space */
-	while (self->ring_count == self->ring_length && !elt->cancelled) {
-	    DBG(9, "waiting for any space to buffer pushed data");
-	    g_cond_wait(self->ring_free_cond, self->ring_mutex);
+	while (self->mem_ring->written - self->mem_ring->readx == self->mem_ring->ring_size && !elt->cancelled) {
+	    DBG(9, "push_buffer waiting for any space to buffer pushed data");
+	    g_cond_wait(self->mem_ring->free_cond, self->mem_ring->mutex);
 	}
-	DBG(9, "done waiting");
+	DBG(9, "push_buffer done waiting");
 
 	if (elt->cancelled)
 	    goto unlock_and_free_and_finish;
 
 	/* only copy to the end of the buffer, if the available space wraps
 	 * around to the beginning */
-	avail = MIN(size, self->ring_length - self->ring_count);
-	avail = MIN(avail, self->ring_length - self->ring_head);
+	avail = MIN(size, self->mem_ring->ring_size - (self->mem_ring->written - self->mem_ring->readx));
+	avail = MIN(avail, self->mem_ring->ring_size - self->mem_ring->write_offset);
 
 	/* copy AVAIL bytes into the ring buf (knowing it's contiguous) */
-	memmove(self->ring_buffer + self->ring_head, p, avail);
+	memmove(self->mem_ring->buffer + self->mem_ring->write_offset, p, avail);
 
 	/* reset the ring variables to represent this state */
-	self->ring_count += avail;
-	self->ring_head += avail; /* will, at most, hit ring_length */
-	if (self->ring_head == self->ring_length)
-	    self->ring_head = 0;
+	self->mem_ring->written += avail;
+	self->mem_ring->write_offset += avail; /* will, at most, hit mem_ring->ring_size */
+	if (self->mem_ring->write_offset == self->mem_ring->ring_size)
+	    self->mem_ring->write_offset = 0;
 	p = (gpointer)((guchar *)p + avail);
 	size -= avail;
 
 	/* and give the device thread a notice that data is ready */
-	g_cond_broadcast(self->ring_add_cond);
+	g_cond_broadcast(self->mem_ring->add_cond);
     }
 
 unlock_and_free_and_finish:
-    g_mutex_unlock(self->ring_mutex);
+    g_mutex_unlock(self->mem_ring->mutex);
 
 free_and_finish:
     if (buf)
@@ -739,10 +906,27 @@ cancel_impl(
     /* then signal all of our condition variables, so that threads waiting on them
      * wake up and see elt->cancelled. */
     g_mutex_lock(self->ring_mutex);
-    g_cond_broadcast(self->ring_add_cond);
-    g_cond_broadcast(self->ring_free_cond);
+    g_cond_broadcast(self->ring_cond);
     g_mutex_unlock(self->ring_mutex);
 
+    if (elt->shm_ring && !elt->shm_ring->mc->cancelled) {
+	if (!elt->shm_ring->mc->cancelled) {
+	    g_debug("XDTS:cancel_impl: cancelling shm-ring because xfer is cancelled");
+	    elt->shm_ring->mc->cancelled = TRUE;
+	}
+	sem_post(elt->shm_ring->sem_ready);
+	sem_post(elt->shm_ring->sem_start);
+	sem_post(elt->shm_ring->sem_read);
+	sem_post(elt->shm_ring->sem_write);
+    }
+    if (self->mem_ring) {
+	g_mutex_lock(self->mem_ring->mutex);
+	self->mem_ring->eof_flag = TRUE;
+	g_cond_broadcast(self->mem_ring->add_cond);
+	g_cond_broadcast(self->mem_ring->free_cond);
+	g_mutex_unlock(self->mem_ring->mutex);
+    }
+
     g_mutex_lock(self->state_mutex);
     g_cond_broadcast(self->state_cond);
     g_mutex_unlock(self->state_mutex);
@@ -757,24 +941,39 @@ start_part_impl(
     dumpfile_t *header)
 {
     XferDestTaperSplitter *self = XFER_DEST_TAPER_SPLITTER(xdt);
+    XferElement *elt = XFER_ELEMENT(self);
 
     g_assert(self->device != NULL);
     g_assert(!self->device->in_file);
     g_assert(header != NULL);
 
-    DBG(1, "start_part()");
+    DBG(1, "start_part() start_part_impl");
 
     /* we can only retry the part if we're getting slices via cache_inform's */
     if (retry_part) {
 	if (self->last_part_successful) {
 	    xfer_cancel_with_error(XFER_ELEMENT(self),
 		_("Previous part did not fail; cannot retry"));
+	    if (elt->shm_ring && !elt->shm_ring->mc->cancelled) {
+		elt->shm_ring->mc->cancelled = TRUE;
+		sem_post(elt->shm_ring->sem_ready);
+		sem_post(elt->shm_ring->sem_start);
+		sem_post(elt->shm_ring->sem_read);
+		sem_post(elt->shm_ring->sem_write);
+	    }
 	    return;
 	}
 
 	if (!self->expect_cache_inform) {
 	    xfer_cancel_with_error(XFER_ELEMENT(self),
 		_("No cache for previous failed part; cannot retry"));
+	    if (elt->shm_ring && !elt->shm_ring->mc->cancelled) {
+		elt->shm_ring->mc->cancelled = TRUE;
+		sem_post(elt->shm_ring->sem_ready);
+		sem_post(elt->shm_ring->sem_start);
+		sem_post(elt->shm_ring->sem_read);
+		sem_post(elt->shm_ring->sem_write);
+	    }
 	    return;
 	}
 
@@ -884,6 +1083,21 @@ get_part_bytes_written_impl(
     }
 }
 
+static void
+new_space_available_impl(
+    XferDestTaper *xdtself,
+    int            made_space)
+{
+    XferDestTaperSplitter *self;
+
+    self = XFER_DEST_TAPER_SPLITTER(xdtself);
+
+    self->more_space = made_space;
+    g_mutex_lock(self->state_mutex);
+    device_reset(self->device);
+    g_cond_broadcast(self->state_cond);
+    g_mutex_unlock(self->state_mutex);
+}
 static void
 instance_init(
     XferElement *elt)
@@ -891,11 +1105,10 @@ instance_init(
     XferDestTaperSplitter *self = XFER_DEST_TAPER_SPLITTER(elt);
     elt->can_generate_eof = FALSE;
 
+    self->ring_mutex = g_mutex_new();
+    self->ring_cond = g_cond_new();
     self->state_mutex = g_mutex_new();
     self->state_cond = g_cond_new();
-    self->ring_mutex = g_mutex_new();
-    self->ring_add_cond = g_cond_new();
-    self->ring_free_cond = g_cond_new();
     self->part_slices_mutex = g_mutex_new();
 
     self->device = NULL;
@@ -907,19 +1120,40 @@ instance_init(
     crc32_init(&elt->crc);
 }
 
+static gboolean
+setup_impl(
+    XferElement *elt)
+{
+    if (elt->input_mech == XFER_MECH_SHM_RING) {
+	elt->shm_ring = shm_ring_create(NULL);
+    }
+
+    return TRUE;
+}
+
 static void
 finalize_impl(
     GObject * obj_self)
 {
     XferDestTaperSplitter *self = XFER_DEST_TAPER_SPLITTER(obj_self);
+    XferElement *elt = XFER_ELEMENT(self);
     FileSlice *slice, *next_slice;
 
+    g_mutex_free(self->ring_mutex);
+    g_cond_free(self->ring_cond);
     g_mutex_free(self->state_mutex);
     g_cond_free(self->state_cond);
 
-    g_mutex_free(self->ring_mutex);
-    g_cond_free(self->ring_add_cond);
-    g_cond_free(self->ring_free_cond);
+    if (self->mem_ring) {
+	g_mutex_free(self->mem_ring->mutex);
+	g_cond_free(self->mem_ring->add_cond);
+	g_cond_free(self->mem_ring->free_cond);
+    }
+
+    if (elt->shm_ring) {
+	close_consumer_shm_ring(elt->shm_ring);
+	elt->shm_ring = NULL;
+    }
 
     g_mutex_free(self->part_slices_mutex);
 
@@ -930,8 +1164,8 @@ finalize_impl(
 	g_free(slice);
     }
 
-    if (self->ring_buffer)
-	g_free(self->ring_buffer);
+    if (self->mem_ring && self->mem_ring->buffer)
+	g_free(self->mem_ring->buffer);
 
     if (self->part_header)
 	dumpfile_free(self->part_header);
@@ -951,10 +1185,13 @@ class_init(
     XferDestTaperClass *xdt_klass = XFER_DEST_TAPER_CLASS(selfc);
     GObjectClass *goc = G_OBJECT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, 1, 1},
-	{ XFER_MECH_NONE, XFER_MECH_NONE, 0, 0},
+	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(2), XFER_NTHREADS(1), XFER_NALLOC(0) },
+	{ XFER_MECH_MEM_RING, XFER_MECH_NONE, XFER_NROPS(1), XFER_NTHREADS(2), XFER_NALLOC(1) },
+	{ XFER_MECH_SHM_RING, XFER_MECH_NONE, XFER_NROPS(1), XFER_NTHREADS(1), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
     };
 
+    klass->setup = setup_impl;
     klass->start = start_impl;
     klass->cancel = cancel_impl;
     klass->push_buffer = push_buffer_impl;
@@ -962,6 +1199,7 @@ class_init(
     xdt_klass->use_device = use_device_impl;
     xdt_klass->cache_inform = cache_inform_impl;
     xdt_klass->get_part_bytes_written = get_part_bytes_written_impl;
+    xdt_klass->new_space_available = new_space_available_impl;
     goc->finalize = finalize_impl;
 
     klass->perl_class = "Amanda::Xfer::Dest::Taper::Splitter";
@@ -975,7 +1213,7 @@ xfer_dest_taper_splitter_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferDestTaperSplitterClass),
             (GBaseInitFunc) NULL,
@@ -1026,16 +1264,7 @@ xfer_dest_taper_splitter(
     self->paused = TRUE;
     self->no_more_parts = FALSE;
 
-    /* set up a ring buffer of size max_memory */
-    self->ring_length = max_memory;
-    self->ring_buffer = g_try_malloc(max_memory);
-    if (!self->ring_buffer) {
-	g_critical("Can't allocate %llu KB (device-output-buffer-size) of memory", (long long)max_memory/1024);
-    }
-
-    self->ring_head = self->ring_tail = 0;
-    self->ring_count = 0;
-    self->ring_head_at_eof = 0;
+    self->max_memory = max_memory;
 
     /* get this new device's streaming requirements */
     bzero(&val, sizeof(val));
@@ -1053,3 +1282,37 @@ xfer_dest_taper_splitter(
 
     return XFER_ELEMENT(self);
 }
+
+static DeviceWriteResult
+retry_write(
+    XferDestTaperSplitter *self,
+    gsize to_write,
+    gpointer buf)
+{
+    XferElement *elt = XFER_ELEMENT(self);
+    XMsg *msg;
+    DeviceWriteResult result;
+
+    msg = xmsg_new(XFER_ELEMENT(self), XMSG_NO_SPACE, 0);
+    xfer_queue_message(elt->xfer, msg);
+
+    while(1) {
+	self->more_space = -1;
+	while (self->more_space == -1 && !elt->cancelled) {
+	    g_cond_wait(self->state_cond, self->state_mutex);
+	}
+
+	if (elt->cancelled) {
+	    return WRITE_FAILED;
+	}
+
+	result = device_write_block(self->device, (guint)to_write, buf);
+	if (result != WRITE_SPACE) {
+	    return result;   /* WRITE_SUCCEED, WRITE_FAILED or WRITE_FULL */
+	}
+	if (self->more_space == 0) {
+	    return result;   /* WRITE_SPACE */
+	}
+    }
+}
+
