@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -64,7 +65,7 @@ xfer_dest_taper_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferDestTaperClass),
             (GBaseInitFunc) NULL,
@@ -140,3 +141,16 @@ xfer_dest_taper_get_part_bytes_written(
     else
 	return 0;
 }
+
+void
+xfer_dest_taper_new_space_available(
+    XferElement *elt,
+    int          made_space)
+{
+    XferDestTaperClass *klass;
+    g_assert(IS_XFER_DEST_TAPER(elt));
+
+    klass = XFER_DEST_TAPER_GET_CLASS(elt);
+    if (klass->new_space_available)
+	klass->new_space_available(XFER_DEST_TAPER(elt), made_space);
+}
