@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -76,6 +77,7 @@ pull_buffer_impl(
     gpointer buf = NULL;
     int result;
     int devsize;
+    int max_block;
 
     /* indicate EOF on an cancel */
     if (elt->cancelled) {
@@ -98,7 +100,11 @@ pull_buffer_impl(
 	    return NULL;
 	}
 	devsize = (int)self->block_size;
-	result = device_read_block(self->device, buf, &devsize);
+	if (elt->size < 0)
+	    max_block = -1;
+	else
+	    max_block = (elt->size+self->block_size-1)/self->block_size;
+	result = device_read_block(self->device, buf, &devsize, max_block);
 	*size = devsize;
 
 	/* if the buffer was too small, loop around again */
@@ -141,8 +147,8 @@ class_init(
 {
     XferElementClass *klass = XFER_ELEMENT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_NONE, XFER_MECH_PULL_BUFFER, 0, 1},
-	{ XFER_MECH_NONE, XFER_MECH_NONE, 0, 0},
+	{ XFER_MECH_NONE, XFER_MECH_PULL_BUFFER, XFER_NROPS(0), XFER_NTHREADS(1), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) }
     };
 
     klass->pull_buffer = pull_buffer_impl;
@@ -158,7 +164,7 @@ xfer_source_device_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferSourceDeviceClass),
             (GBaseInitFunc) NULL,
