@@ -1,4 +1,5 @@
-# Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,16 +15,16 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
-use Test::More tests => 54;
+use Test::More tests => 48;
 use File::Path;
 use Data::Dumper;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck::Config;
 use Amanda::Paths;
 use Amanda::Device qw( :constants );;
@@ -252,7 +253,7 @@ if ($cfg_result != $CFGERR_OK) {
 
 # check out the relevant changer properties
 my $tlf = Amanda::Config::config_dir_relative(getconf($CNF_TAPELIST));
-my $tl = Amanda::Tapelist->new($tlf);
+my ($tl, $message) = Amanda::Tapelist->new($tlf);
 my $chg = Amanda::Changer->new("mychanger", tapelist => $tl);
 is($chg->{'config'}->get_property("testprop"), "testval",
     "changer properties are correctly represented");
@@ -260,9 +261,11 @@ is($chg->have_inventory(), 1, "changer have inventory");
 my @new_tape_label = $chg->make_new_tape_label();
 is_deeply(\@new_tape_label, [undef, "template is not set, you must set autolabel"], "no make_new_tape_label");
 is($chg->make_new_meta_label(), undef, "no make_new_meta_label");
+$chg->quit();
 
 $chg = Amanda::Changer->new("mychanger", tapelist => $tl,
-			    labelstr => "TESTCONF-[0-9][0-9][0-9]-[a-z][a-z][a-z]-[0-9][0-9][0-9]",
+			    labelstr => { match_autolabel => 0,
+					  template => "TESTCONF-[0-9][0-9][0-9]-[a-z][a-z][a-z]-[0-9][0-9][0-9]"},
 			    autolabel => { template => '$c-$m-$b-%%%',
 					   other_config => 1,
 					   non_amanda => 1,
@@ -274,11 +277,6 @@ is($meta, "001", "meta 001");
 my $label = $chg->make_new_tape_label(meta => $meta, barcode => 'aaa');
 is($label, 'TESTCONF-001-aaa-001', "label TESTCONF-001-aaa-001");
 
-is($chg->volume_is_labelable($DEVICE_STATUS_VOLUME_UNLABELED, $Amanda::Header::F_EMPTY),
-   1, "empty volume is labelable");
-is($chg->volume_is_labelable($DEVICE_STATUS_VOLUME_ERROR, undef),
-   0, "empty volume is labelable");
-
 # test loading by label
 {
     my @labels;
@@ -623,14 +621,6 @@ loadconfig(undef, "mychanger", "chg-disk:/bar", undef);
 is_deeply( Amanda::Changer->new(), [ "chg-disk:/bar", "cc" ],
     "default changer with global tpchanger naming a defined changer with a uri");
 
-loadconfig(undef, "mychanger", "chg-zd-mtx", undef);
-is_deeply( Amanda::Changer->new(), [ "chg-compat:chg-zd-mtx", "cc" ],
-    "default changer with global tpchanger naming a defined changer with a compat script");
-
-loadconfig(undef, "chg-zd-mtx", undef, undef);
-is_deeply( Amanda::Changer->new(), [ "chg-compat:chg-zd-mtx", undef ],
-    "default changer with global tpchanger naming a compat script");
-
 loadconfig("tape:/dev/foo", undef, undef, undef);
 is_deeply( Amanda::Changer->new(), [ "chg-single:tape:/dev/foo", undef ],
     "default changer with global tapedev naming a device and no tpchanger");
@@ -643,10 +633,6 @@ assert_invalid("tape:/dev/foo", "chg-disk:/foo", undef, undef, undef,
     qr/Cannot specify both 'tapedev' and 'tpchanger'/,
     "supplying a device for tapedev and a changer for tpchanger is invalid");
 
-loadconfig("tape:/dev/foo", 'chg-zd-mtx', undef, undef);
-is_deeply( Amanda::Changer->new(), [ "chg-compat:chg-zd-mtx", undef ],
-    "default changer with global tapedev naming a device and a global tpchanger naming a compat script");
-
 assert_invalid("chg-disk:/foo", "tape:/dev/foo", undef, undef, undef,
     qr/Cannot specify both 'tapedev' and 'tpchanger'/,
     "supplying a changer for tapedev and a device for tpchanger is invalid");
@@ -659,10 +645,6 @@ loadconfig("mychanger", undef, "chg-disk:/bar", undef);
 is_deeply( Amanda::Changer->new(), [ "chg-disk:/bar", "cc" ],
     "default changer with global tapedev naming a defined changer with a uri");
 
-loadconfig("mychanger", undef, "chg-zd-mtx", undef);
-is_deeply( Amanda::Changer->new(), [ "chg-compat:chg-zd-mtx", "cc" ],
-    "default changer with global tapedev naming a defined changer with a compat script");
-
 loadconfig(undef, undef, "chg-disk:/foo", undef);
 is_deeply( Amanda::Changer->new("mychanger"), [ "chg-disk:/foo", "cc" ],
     "named changer loads the proper definition");
