@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +15,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 19;
@@ -22,7 +23,7 @@ use File::Path;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck;
 use Installcheck::Config;
 use Installcheck::Changer;
@@ -59,9 +60,9 @@ sub reset_taperoot {
 sub is_pointing_to {
     my ($res, $slot, $msg) = @_;
 
-    my ($datalink) = ($res->{'device'}->device_name =~ /file:(.*)/);
-    $datalink .= "/data";
-    is(readlink($datalink), "../slot$slot", $msg);
+    is($res->{'device'}->device_name, "file:$taperoot/slot$slot", $msg);
+
+    # Should check the state file */
 }
 
 # Build a configuration that specifies Amanda::Changer::Disk
@@ -164,7 +165,7 @@ sub test_relative_slot {
     my $steps = define_steps
 	cb_ref => \$finished_cb;
 
-    # load the "current" slot, which should be 3
+    # load the "current" slot, which should be 5
     step load_current => sub {
 	$chg->load(relative_slot => "current", res_cb => $steps->{'check_current_cb'});
     };
