@@ -1,4 +1,5 @@
-# Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +15,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 39;
@@ -22,7 +23,7 @@ use File::Path;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck;
 use Installcheck::Config;
 use Data::Dumper;
@@ -36,6 +37,7 @@ use Amanda::Logfile qw(:logtype_t :program_t open_logfile get_logline
 use Amanda::Config qw( :init :getconf config_dir_relative );
 
 Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
 
 my $log_filename = "$Installcheck::TMP/Amanda_Logfile_test.log";
 
@@ -191,6 +193,7 @@ my $logdir = $testconf->{'logdir'};
     my $filename = "$logdir/log";
 
     -f "$filename" and unlink("$filename");
+    Amanda::Logfile::set_logname($filename);
     log_add($L_INFO, "This is my info");
     log_add($L_START, "blah blah blah date 20300405060708 blah blah");
 
@@ -219,7 +222,7 @@ print $tlf "20071109010002 TESTCONF003 reuse\n";
 print $tlf "20071109010002 TESTCONF002 reuse\n";
 print $tlf "20071108010001 TESTCONF001 reuse\n";
 close $tlf;
-Amanda::Tapelist->new($tapelist);
+my ($tl, $message) = Amanda::Tapelist->new($tapelist);
 
 # set up a number of logfiles in logdir.
 my $logf;
@@ -290,7 +293,7 @@ my @results;
 my @results_arr;
 
 @results = Amanda::Logfile::search_logfile(undef, "20071109010002",
-					   "$logdir/log.20071109010002.0", 1);
+					   "$logdir/log.20071109010002.0", 1, 1);
 is($#results+1, 17, "search_logfile returned 17 results");
 
 # sort by filenum so we can compare each to what it should be
@@ -401,7 +404,6 @@ is_deeply([ map { res2arr($_) } @filtered ],
 	  [ '20071109010002', 'thatbox', '/u_win',    3, 'TESTCONF004', 1,  'OK',      'OK'  , '',         3, 4 ],
 	  [ '20071109010002', 'thatbox', '/u_win',    3, 'TESTCONF004', 2,  'OK',      'OK'  , '',         4, 4 ],
 	], "filter with dumpspecs '.* /var thatbox' (union of two overlapping sets includes dupes)");
-
 @dumpspecs = Amanda::Cmdline::dumpspec_t->new('thatbox', undef, undef, undef, '20071109010002');
 @filtered = Amanda::Logfile::dumps_match_dumpspecs([@results], [@dumpspecs], 0);
 @filtered = sort { $a->{'label'} cmp $b->{'label'} ||
