@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2013 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 11;
@@ -23,7 +23,7 @@ use Data::Dumper;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck::Run;
 use Installcheck::Catalogs;
 use Amanda::Config qw( :init :getconf config_dir_relative );
@@ -43,7 +43,7 @@ Amanda::Debug::dbopen("installcheck");
 Installcheck::log_test_output();
 
 my $testconf;
-$testconf = Installcheck::Run->setup();
+$testconf = Installcheck::Run::setup();
 $testconf->write();
 
 # install the 'bigdb' catalog to test against
@@ -64,6 +64,7 @@ sub make_plan_sync {
     my $plan;
 
     Amanda::Recovery::Planner::make_plan(@_,
+	labelstr => ".*",
 	debug => 1,
 	plan_cb => sub {
 	    (my $err, $plan) = @_;
@@ -114,17 +115,13 @@ sub is_plan {
 	or diag("got:\n" . Dumper(\@got_abbrev));
 }
 
-my $changer = undef; # not needed yet
-
 is_plan(make_plan_sync(
-	    dumpspec => ds("no-box-at-all"),
-	    changer => $changer),
+	    dumpspec => ds("no-box-at-all")),
     [ ],
     "empty plan for nonexistent host");
 
 is_plan(make_plan_sync(
-	    dumpspec => ds("oldbox", "^/opt"),
-	    changer => $changer),
+	    dumpspec => ds("oldbox", "^/opt")),
     [
 	[   "oldbox", "/opt", "20080414144444", 0, [
 		'20080414144444/oldbox._opt',
@@ -134,8 +131,7 @@ is_plan(make_plan_sync(
     "simple plan for a dump on holding disk");
 
 is_plan(make_plan_sync(
-	    dumpspec => ds("somebox", "^/lib", "200801"),
-	    changer => $changer),
+	    dumpspec => ds("somebox", "^/lib", "200801")),
     [
 	[   "somebox", "/lib", "20080111000000", 0, [
 		'Conf-001' => 1,
@@ -145,8 +141,7 @@ is_plan(make_plan_sync(
     "simple plan for just one dump");
 
 is_plan(make_plan_sync(
-	    dumpspec => ds("somebox", "^/usr/bin"),
-	    changer => $changer),
+	    dumpspec => ds("somebox", "^/usr/bin")),
     [
         [   'somebox', '/usr/bin', '20080313133333', 1, [
 		'Conf-003' => 1,
@@ -164,8 +159,7 @@ is_plan(make_plan_sync(
     "plan for three dumps, in order by tape write time");
 
 is_plan(make_plan_sync(
-	    dumpspec => ds("otherbox", "^/lib"),
-	    changer => $changer),
+	    dumpspec => ds("otherbox", "^/lib")),
     [
 	[   "otherbox", "/lib", "20080414144444", 1, [
 		'20080414144444/otherbox._lib',
@@ -186,8 +180,7 @@ is_plan(make_plan_sync(
 	    dumpspecs => [
 		ds("somebox", "^/lib", "20080111"),
 		ds("somebox", "^/lib", "20080222"),
-	    ],
-	    changer => $changer),
+	    ]),
     [
 	[   "somebox", "/lib", "20080111000000", 0, [
 		'Conf-001' => 1,
@@ -203,8 +196,7 @@ is_plan(make_plan_sync(
 
 is_plan(make_plan_sync(
 	    dumpspec => ds("somebox", "^/lib", "200803"),
-	    one_dump_per_part => 1,
-	    changer => $changer),
+	    one_dump_per_part => 1),
     [
 	[   "somebox", "/lib", "20080313133333", 0, [
 		'Conf-003' => 2,
@@ -273,10 +265,9 @@ is_plan(make_plan_sync(
 is_plan(make_plan_sync(
 	    dumpspec => ds("somebox", "/lib", "20080515155555"),
 	    filelist => [
-		'Conf-006' => [2, 3, 4, 5,       8, 9, 10, 11],
+		'TESTCONF', 'Conf-006', [2, 3, 4, 5,       8, 9, 10, 11],
 		#  (make_plan should fill in files 6 and 7)
-	    ],
-	    changer => $changer),
+	    ]),
     [
 	[   'somebox', '/lib', '20080515155555', 0, [
 		'Conf-006' => 2,
@@ -296,10 +287,9 @@ is_plan(make_plan_sync(
 
 is_plan(make_plan_sync(
 	    filelist => [
-		'Conf-006' => [2, 3, 4, 5,       8, 9, 10, 11],
+		'TESTCONF', 'Conf-006', [2, 3, 4, 5,       8, 9, 10, 11],
 		#  (make_plan should fill in files 6 and 7)
-	    ],
-	    changer => $changer),
+	    ]),
     [
 	[   'somebox', '/lib', '20080515155555', 0, [
 		'Conf-006' => 2,
