@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,19 +15,21 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
-use Test::More tests => 58;
+use Test::More tests => 67;
 use strict;
 use warnings;
 
 use File::Path;
 use Data::Dumper;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 
 use Installcheck;
+use Installcheck::Run;
+use Amanda::Config qw( :init :getconf );
 use Amanda::Report;
 use Amanda::Debug;
 
@@ -39,6 +42,9 @@ my %LogfileData;
 my $logCount = 0;
 my $log_filename = "$Installcheck::TMP/Amanda_Report_test.log";
 
+my $testconf = Installcheck::Run::setup();;
+$testconf->write();
+
 # copy/pasted from installcheck/Amanda_Logfile.pl .  Maybe move this
 # to a module?
 sub write_logfile
@@ -65,11 +71,13 @@ $LogfileFlags{planner} = {
     normal_run => 1,
     results_missing => 1,
     dump_failed => 0,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 8,
 };
 
 $LogfileData{planner} = {
-    programs => { planner => { start => "20090728160530", }, },
+    programs => { planner => { start => "20090728160530",
+			       pid   => '12346' }, },
     disklist => {
         localhost => {
             "/root" => {
@@ -112,14 +120,17 @@ $LogfileFlags{driver} = {
     normal_run => 1,
     results_missing => 1,
     dump_failed => 0,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 8,
 };
 
 $LogfileData{driver} = {
     programs => {
-        planner => { start => "20090728122430", },
+        planner => { start => "20090728122430",
+		     pid   => '12346' },
         driver  => {
             start      => "20090728122430",
+	    pid        => '12347',
             start_time => "0.034",
             time       => "14.46",
         },
@@ -186,14 +197,17 @@ $LogfileFlags{dumper} = {
     normal_run => 1,
     results_missing => 0,
     dump_failed => 1,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 4,
 };
 
 $LogfileData{dumper} = {
     programs => {
-        planner => { start => "20090728122430", },
+        planner => { start => "20090728122430",
+		     pid   => '12346' },
         driver  => {
             start      => "20090728122430",
+	    pid        => '12345',
             start_time => "0.034",
             time       => "14.46",
         },
@@ -308,19 +322,22 @@ $LogfileFlags{chunker} = {
     normal_run => 1,
     results_missing => 0,
     dump_failed => 1,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 4,
 };
 
 $LogfileData{chunker} = {
     programs => {
-        planner => { start => "20090728122430", },
+        planner => { start => "20090728122430",
+		     pid   => '12346' },
         driver => {
             start      => "20090728122430",
+	    pid        => '12345',
             start_time => "0.034",
             time       => "14.46",
         },
         dumper  => {},
-        chunker => {}
+        chunker => { pid => '12350' }
     },
     disklist => {
         localhost => {
@@ -426,6 +443,7 @@ $LogfileData{chunker} = {
 
 
 $LogfileContents{taper} = <<EOF;
+INFO amdump amdump pid 28020
 DISK planner somebox /lib
 START planner date 20080111
 START driver date 20080111
@@ -445,6 +463,7 @@ PARTIAL taper somebox /lib 20080111 4 0 [sec 0.011669 kb 2016 kps 172765.446911]
 INFO taper tape TESTCONF01 kb 2016 fm 4 [OK]
 INFO taper pid-done 28023
 FINISH driver date 20080111 time 2167.581
+INFO amdump pid-done 28020
 EOF
 
 $LogfileFlags{taper} = {
@@ -452,7 +471,8 @@ $LogfileFlags{taper} = {
     normal_run => 1,
     results_missing => 0,
     dump_failed => 0,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 16,
 };
 
 $LogfileData{taper} = {
@@ -466,10 +486,12 @@ $LogfileData{taper} = {
             start_time => "0.051",
             time       => "2167.581",
         },
+        amdump  => { pid => '28020' },
         dumper  => {},
         chunker => {},
         taper   => {
             start => '20080111',
+	    pid   => '28023',
             notes => [
                 "tape TESTCONF01 kb 2016 fm 4 [OK]"
             ],
@@ -481,6 +503,8 @@ $LogfileData{taper} = {
                     kb      => "2016",
                     files   => "4",
                     dle     => "1",
+		    'storage' => "TESTCONF",
+		    'pool' => "TESTCONF"
                 },
             },
             tape_labels => ['TESTCONF01'],
@@ -520,6 +544,8 @@ $LogfileData{taper} = {
                             status => "partial",
                             parts  => [
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "TESTCONF01",
                                     date  => "20080111",
                                     file  => "1",
@@ -529,6 +555,8 @@ $LogfileData{taper} = {
                                     kps   => "135535.789920",
                                 },
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "TESTCONF01",
                                     date  => "20080111",
                                     file  => "2",
@@ -538,6 +566,8 @@ $LogfileData{taper} = {
                                     kps   => "186154.741129",
                                 },
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "TESTCONF01",
                                     date  => "20080111",
                                     file  => "3",
@@ -547,6 +577,8 @@ $LogfileData{taper} = {
                                     kps   => "218355.510065",
                                 },
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "TESTCONF01",
                                     date  => "20080111",
                                     file  => "4",
@@ -562,6 +594,8 @@ $LogfileData{taper} = {
                             kps     => "172765.446911",
                             orig_kb => undef,
                             error   => '',
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                     },
                   ]
@@ -573,6 +607,7 @@ $LogfileData{taper} = {
 
 
 $LogfileContents{simple} = <<EOF;
+INFO amdump amdump pid 28020
 DISK planner somebox /lib
 START planner date 20080111
 START driver date 20080111
@@ -586,6 +621,7 @@ STATS driver estimate somebox /lib 20080111 0 [sec 1 nkb 2002 ckb 480 kps 385]
 PART taper Conf-001 1 somebox /lib 20080111 1/1 0 [sec 4.813543 kb 419 kps 87.133307]
 DONE taper somebox /lib 20080111 1 0 [sec 4.813543 kb 419 kps 87.133307]
 FINISH driver date 20080111 time 2167.581
+INFO amdump pid-done 28020
 EOF
 
 $LogfileFlags{simple} = {
@@ -593,7 +629,8 @@ $LogfileFlags{simple} = {
     normal_run => 1,
     results_missing => 0,
     dump_failed => 0,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 0,
 };
 
 $LogfileData{simple} = {
@@ -616,11 +653,14 @@ $LogfileData{simple} = {
                     'time'  => '4.813543',
                     'date'  => '20080111',
                     'kb'    => 419,
-                    'dle'   => 1
+                    'dle'   => 1,
+		    'storage' => "TESTCONF",
+		    'pool' => "TESTCONF"
                 },
             },
             tape_labels => ["Conf-001"],
         },
+        amdump  => { pid => '28020' },
         dumper  => {},
         chunker => {},
     },
@@ -650,6 +690,8 @@ $LogfileData{simple} = {
                             status => "done",
                             parts  => [
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "Conf-001",
                                     date  => "20080111",
                                     file  => "1",
@@ -664,6 +706,8 @@ $LogfileData{simple} = {
                             kb      => "419",
                             kps     => "87.133307",
                             orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                         chunker => {
                             status => "success",
@@ -754,6 +798,7 @@ INFO dumper pid-done 9318
 INFO taper pid-done 9314
 FINISH driver date 20081002040002 time 663.574
 INFO driver pid-done 9313
+INFO amdump pid-done 9291
 
 EOF
 
@@ -762,25 +807,29 @@ $LogfileFlags{fullExample} = {
     normal_run => 1,
     results_missing => 0,
     dump_failed => 0,
-    dump_strange => 1
+    dump_strange => 1,
+    exit_status => 2,
 };
 
 $LogfileData{fullExample} = {
     programs => {
-        amdump => {},
-        dumper => {},
+        amdump => { pid => '9291'},
+        dumper => { pid => '9317'},
         driver => {
             time       => "663.574",
             start      => "20081002040002",
             start_time => "0.043",
+	    pid        => '9313'
         },
         planner => {
             time  => "32.689",
             start => "20081002040002",
+	    pid   => '9312'
         },
-        chunker => {},
+        chunker => { pid => '11792' },
         taper   => {
             start => '20081002040002',
+	    pid   => '9314',
             tapes => {
                 "FullBackup-14" => {
                     'label' => 'FullBackup-14',
@@ -788,7 +837,9 @@ $LogfileData{fullExample} = {
                     'time'  => '740.930972',
                     'date'  => '20081002040002',
                     'kb'    => 57568922,
-                    'dle'   => 6
+                    'dle'   => 6,
+		    'storage' => "TESTCONF",
+		    'pool' => "TESTCONF"
                 },
             },
             tape_labels => ["FullBackup-14"],
@@ -828,11 +879,15 @@ $LogfileData{fullExample} = {
                                     partnum  => "1",
                                     file  => "4",
                                     kb    => "8392",
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "FullBackup-14"
                                 },
                             ],
                             kb      => '8392',
-                            orig_kb => undef
+                            orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                         dumper => {
                             kps     => "2771.1",
@@ -876,6 +931,8 @@ $LogfileData{fullExample} = {
                             status => "done",
                             parts  => [
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => 'FullBackup-14',
                                     date  => '20081002040002',
                                     kps   => '77695.214669',
@@ -886,7 +943,9 @@ $LogfileData{fullExample} = {
                                 }
                             ],
                             kb      => "28776940",
-                            orig_kb => undef
+                            orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                     },
                   ],
@@ -899,6 +958,8 @@ $LogfileData{fullExample} = {
                             status => "done",
                             parts  => [
                                 {
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => 'FullBackup-14',
                                     date  => '20081002030002',
                                     kps   => '77695.214669',
@@ -909,7 +970,9 @@ $LogfileData{fullExample} = {
                                 }
                             ],
                             kb      => "28776940",
-                            orig_kb => undef
+                            orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
 		    },
 		  ]
@@ -947,11 +1010,15 @@ $LogfileData{fullExample} = {
                                     partnum  => "1",
                                     file  => "2",
                                     kb    => "10",
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "FullBackup-14"
                                 },
                             ],
                             'kb'    => '10',
-                            orig_kb => undef
+                            orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                         'dumper' => {
                             'kps'    => '250.8',
@@ -998,11 +1065,15 @@ $LogfileData{fullExample} = {
                                     partnum  => "1",
                                     file  => "3",
                                     kb    => "6630",
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "FullBackup-14"
                                 },
                             ],
                             kb      => "6630",
                             orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                         dumper => {
                             kps     => "16013.4",
@@ -1049,11 +1120,15 @@ $LogfileData{fullExample} = {
                                     partnum  => "1",
                                     file  => "1",
                                     kb    => "10",
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     label => "FullBackup-14",
                                 },
                             ],
                             kb      => "10",
-                            orig_kb => undef
+                            orig_kb => undef,
+			    storage => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                         dumper => {
                             kps     => "250.4",
@@ -1094,6 +1169,7 @@ INFO taper pid-done 26077
 FINISH driver date 20090622075550 time 177.708
 INFO driver pid-done 26076
 INFO amflush pid-done 26075
+INFO amflush pid-done 26036
 EOF
 
 $LogfileFlags{amflushExample} = {
@@ -1101,13 +1177,15 @@ $LogfileFlags{amflushExample} = {
     amflush_run => 1,
     results_missing => 0,
     dump_failed => 0,
-    dump_strange => 0
+    dump_strange => 0,
+    exit_status => 0,
 };
 
 $LogfileData{amflushExample} = {
     'programs' => {
         'taper' => {
             'start' => '20090622075550',
+	    'pid'   => '26077',
             'tapes' => {
                 'DailyTapeDataSet-017' => {
                     'label' => 'DailyTapeDataSet-017',
@@ -1115,16 +1193,20 @@ $LogfileData{amflushExample} = {
                     'time'  => '4.180007',
                     'date'  => '20090622075550',
                     'kb'    => 37289,
-                    'dle'   => 2
+                    'dle'   => 2,
+		    'storage' => "TESTCONF",
+		    'pool' => "TESTCONF"
                 },
             },
             tape_labels => ["DailyTapeDataSet-017"],
         },
-        'amflush' => { 'start' => '20090622075550' },
+        'amflush' => { 'start' => '20090622075550',
+		       'pid'   => '26036' },
         'driver'  => {
             'time'       => '177.708',
             'start_time' => '0.011',
-            'start'      => '20090622075550'
+            'start'      => '20090622075550',
+	    'pid'        => '26076'
         },
     },
     'disklist' => {
@@ -1147,11 +1229,15 @@ $LogfileData{amflushExample} = {
                                     'partnum'  => '1',
                                     'file'  => '1',
                                     'kb'    => '36980',
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     'label' => 'DailyTapeDataSet-017'
                                 },
                             ],
                             'kb'     => '36980',
-                            orig_kb  => undef
+                            orig_kb  => undef,
+			    storage  => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                     },
 		  ]
@@ -1179,11 +1265,15 @@ $LogfileData{amflushExample} = {
                                     'partnum'  => '1',
                                     'file'  => '2',
                                     'kb'    => '309',
+                                    'storage' => "TESTCONF",
+                                    'pool'  => "TESTCONF",
                                     'label' => 'DailyTapeDataSet-017'
                                 },
                             ],
                             'kb'     => '309',
-                            orig_kb  => undef
+                            orig_kb  => undef,
+			    storage  => "TESTCONF",
+			    'pool' => "TESTCONF"
                         },
                     }]
                 },
@@ -1200,6 +1290,13 @@ $LogfileData{amflushExample} = {
     },
 };
 
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+my ($cfgerr_level, @cfgerr_errors) = config_errors();
+if ($cfgerr_level >= $CFGERR_WARNINGS) {
+    config_print_errors();
+    die "config errors";
+}
+
 foreach my $test ( keys %LogfileContents ) {
 
     unless ( exists $LogfileData{$test} ) {
@@ -1208,14 +1305,14 @@ foreach my $test ( keys %LogfileContents ) {
 
     my $report =
       Amanda::Report->new( write_logfile( $LogfileContents{$test} ), 1 );
-    is_deeply( $report->{data}, $LogfileData{$test}, "data check: $test" );
+    is_deeply( $report->{data}, $LogfileData{$test}, "data check: $test" ) || diag(Data::Dumper::Dumper($report->{data}) ."\n\n" . Data::Dumper::Dumper($LogfileData{$test}));
 
     map {
         cmp_ok(
             $report->get_flag($_), "==",
             $LogfileFlags{$test}->{$_},
             "flag test: $_, $test"
-        );
+        ) || die();
     } keys %{ $LogfileFlags{$test} };
 }
 
@@ -1280,11 +1377,15 @@ is_deeply(
                             partnum  => "1",
                             file  => "3",
                             kb    => "6630",
+                            'storage' => 'TESTCONF',
+                            'pool' => 'TESTCONF',
                             label => "FullBackup-14"
                         },
                     ],
                     kb      => "6630",
                     orig_kb => undef,
+		    storage => "TESTCONF",
+		    'pool' => "TESTCONF"
                 },
                 dumper => {
                     kps     => "16013.4",
@@ -1319,6 +1420,7 @@ is_deeply(
     {
         time  => "32.689",
         start => "20081002040002",
+        pid   => '9312'
     },
     'check: Amanda::Report::get_program_info($program)'
 );
@@ -1353,5 +1455,8 @@ is(
 ok($report->get_flag('historical'),
     "historical flag processed correctly");
 
+is($report->get_flag('exit_status'), 2,
+   "exit_status");
+
 # clean up
 unlink($log_filename) if -f $log_filename;
