@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2013 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 29;
@@ -23,7 +23,7 @@ use Data::Dumper;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck::Config;
 use Amanda::Config qw( :init );
 use Amanda::Changer;
@@ -170,7 +170,8 @@ sub scan {
 }
 
 package Mock::Feedback;
-use base qw( Amanda::Taper::Scribe::Feedback );
+use Amanda::Taper::Scribe;
+use parent -norequire, qw( Amanda::Taper::Scribe::Feedback );
 use Test::More;
 use Data::Dumper;
 use Installcheck::Config;
