@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,26 +15,27 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
-use Test::More tests => 46;
+use Test::More tests => 68;
 use File::Path;
 use Data::Dumper;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck;
-use Installcheck::Run;
+use Installcheck::Run qw( load_vtape_res );
 use Installcheck::Mock;
 use Amanda::Xfer qw( :constants );
 use Amanda::Header;
 use Amanda::Debug;
 use Amanda::MainLoop;
 use Amanda::Paths;
-use Amanda::Config;
+use Amanda::Config qw( :init :getconf );
 use Amanda::Constants;
+use Fcntl 'SEEK_SET';
 
 # get Amanda::Device only when we're building for server
 BEGIN {
@@ -41,6 +43,8 @@ BEGIN {
     if (Amanda::Util::built_with_component("server")) {
 	eval "use Amanda::Device;";
 	die $@ if $@;
+	eval "use Amanda::Changer;";
+	die $@ if $@;
     }
 }
 
@@ -51,6 +55,8 @@ Installcheck::log_test_output();
 # and disable Debug's die() and warn() overrides
 Amanda::Debug::disable_die_override();
 
+my $r;
+
 {
     my $RANDOM_SEED = 0xD00D;
 
@@ -75,19 +81,22 @@ Amanda::Debug::disable_die_override();
 	}
     });
     Amanda::MainLoop::run();
+    $xfer = undef;
     pass("A simple transfer runs to completion");
     is($got_msg, "Is this thing on?",
 	"XMSG_INFO from Amanda::Xfer::Dest::Null has correct message");
 }
 
+my $xfer1;
+my $xfer2;
 {
     my $RANDOM_SEED = 0xDEADBEEF;
 
-    my $xfer1 = Amanda::Xfer->new([
+    $xfer1 = Amanda::Xfer->new([
 	Amanda::Xfer::Source::Random->new(1024*1024, $RANDOM_SEED),
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
     ]);
-    my $xfer2 = Amanda::Xfer->new([
+    $xfer2 = Amanda::Xfer->new([
 	Amanda::Xfer::Source::Random->new(1024*1024*3, $RANDOM_SEED),
 	Amanda::Xfer::Filter::Xor->new(0xf0),
 	Amanda::Xfer::Filter::Xor->new(0xf0),
@@ -112,6 +121,8 @@ Amanda::Debug::disable_die_override();
 # let the already-started transfers go out of scope before they 
 # complete, as a memory management test..
 Amanda::MainLoop::run();
+$xfer1 = undef;
+$xfer2 = undef;
 pass("Two simultaneous transfers run to completion");
 
 {
@@ -143,6 +154,7 @@ pass("Two simultaneous transfers run to completion");
     $xfer->start($cb);
 
     Amanda::MainLoop::run();
+    $xfer = undef;
     pass("One 10-element transfer runs to completion");
 }
 
@@ -180,20 +192,74 @@ pass("Two simultaneous transfers run to completion");
     };
 
     $xfer->start($cb);
-
     Amanda::MainLoop::run();
+    $xfer = undef;
 
-    close($wfh);
     close($rfh);
+    close($wfh);
 
     # now verify the file contents are identical
     open($rfh, "<", $read_filename);
     my $src = do { local $/; <$rfh> };
+    close($rfh);
 
     open($rfh, "<", $write_filename);
     my $dest = do { local $/; <$rfh> };
+    close($rfh);
 
-    is($src, $dest, "Source::Fd and Dest::Fd read and write files");
+    is($src,$dest,"read from Source::Fd identical");
+
+    unlink($read_filename);
+    unlink($write_filename);
+}
+
+{
+    my $read_filename = "$Installcheck::TMP/xfer-junk-src.tmp";
+    my $write_filename = "$Installcheck::TMP/xfer-junk-dest.tmp";
+    my ($rfh, $wfh);
+
+    mkdir($Installcheck::TMP) unless (-e $Installcheck::TMP);
+
+    # fill the file with some stuff
+    open($wfh, ">", $read_filename) or die("Could not open '$read_filename' for writing");
+    for my $i (1 .. 100) { print $wfh "line $i\n"; }
+    close($wfh);
+
+    open($wfh, ">", "$write_filename") or die("Could not open '$write_filename' for writing");
+
+    # now run a transfer out of it
+    my $xfer = Amanda::Xfer->new([
+	Amanda::Xfer::Source::File->new($read_filename),
+	Amanda::Xfer::Filter::Xor->new(0xde),
+	Amanda::Xfer::Filter::Xor->new(0xde),
+	Amanda::Xfer::Dest::Fd->new(fileno($wfh)),
+    ]);
+
+    my $cb = sub {
+	my ($src, $msg, $xfer) = @_;
+	if ($msg->{type} == $XMSG_ERROR) {
+	    die $msg->{elt} . " failed: " . $msg->{message};
+	} elsif ($msg->{'type'} == $XMSG_DONE) {
+	    Amanda::MainLoop::quit();
+	}
+    };
+
+    $xfer->start($cb);
+    Amanda::MainLoop::run();
+    $xfer = undef;
+
+    close($wfh);
+
+    # now verify the file contents are identical
+    open($rfh, "<", $read_filename);
+    my $src = do { local $/; <$rfh> };
+    close($rfh);
+
+    open($rfh, "<", $write_filename);
+    my $dest = do { local $/; <$rfh> };
+    close($rfh);
+
+    is($src,$dest,"read from Source::File identical");
 
     unlink($read_filename);
     unlink($write_filename);
@@ -226,6 +292,7 @@ pass("Two simultaneous transfers run to completion");
 	}
     });
     Amanda::MainLoop::run();
+    $xfer = undef;
     ok($got_timeout, "A neverending transfer finishes after being cancelled");
     # (note that this does not test all of the cancellation possibilities)
 }
@@ -256,6 +323,7 @@ pass("Two simultaneous transfers run to completion");
 	}
     });
     Amanda::MainLoop::run();
+    $xfer = undef;
     ok($got_error, "A transfer with an error cancels itself after sending an error");
 
     unlink($read_filename);
@@ -285,9 +353,68 @@ pass("Two simultaneous transfers run to completion");
     });
     $xfer->start();
     Amanda::MainLoop::run();
+    $xfer = undef;
     pass("compress | uncompress gets back the original stream");
 }
 
+# test the Process filter (gzip exit code 2 is not an error)
+{
+    my $RANDOM_SEED = 0xD00D;
+
+    # create a compressed file
+    my $write_filename = "$Installcheck::TMP/xfer-junk-dest.tmp";
+    my $wfh;
+    open($wfh, ">", "$write_filename") or die("Could not open '$write_filename' for writing");
+    my $xfer = Amanda::Xfer->new([
+	Amanda::Xfer::Source::Random->new(1024*1024, $RANDOM_SEED),
+	Amanda::Xfer::Filter::Process->new(
+	    [ $Amanda::Constants::COMPRESS_PATH, $Amanda::Constants::COMPRESS_BEST_OPT ], 0, 0, 0, 0),
+	Amanda::Xfer::Dest::Fd->new(fileno($wfh))
+    ]);
+
+    $xfer->get_source()->set_callback(sub {
+	my ($src, $msg, $xfer) = @_;
+	if ($msg->{type} == $XMSG_ERROR) {
+	    die $msg->{elt} . " failed: " . $msg->{message};
+	} elsif ($msg->{'type'} == $XMSG_DONE) {
+	    $src->remove();
+	    Amanda::MainLoop::quit();
+	}
+    });
+    $xfer->start();
+    Amanda::MainLoop::run();
+    $xfer = undef;
+    close($wfh);
+
+    # add non-compressed random byte to the compressed file
+    open($wfh, ">>", "$write_filename") or die("Could not open '$write_filename' for writing");
+    print $wfh "abcdefghijklmnopqrstuvwxyzA";
+    close($wfh);
+
+    open($wfh, "<", "$write_filename") or die("Could not open '$write_filename' for writing");
+    # uncompress it.
+    $xfer = Amanda::Xfer->new([
+	Amanda::Xfer::Source::Fd->new(fileno($wfh)),
+	Amanda::Xfer::Filter::Process->new(
+	    [ $Amanda::Constants::UNCOMPRESS_PATH, $Amanda::Constants::UNCOMPRESS_OPT ], 0, 0, 0, 0),
+	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
+    ]);
+
+    $xfer->get_source()->set_callback(sub {
+	my ($src, $msg, $xfer) = @_;
+	if ($msg->{type} == $XMSG_ERROR) {
+	    die $msg->{elt} . " failed: " . $msg->{message};
+	} elsif ($msg->{'type'} == $XMSG_DONE) {
+	    $src->remove();
+	    Amanda::MainLoop::quit();
+	}
+    });
+    $xfer->start();
+    Amanda::MainLoop::run();
+    $xfer = undef;
+    pass("uncompress with trailling bytes succeed");
+}
+
 {
     my $RANDOM_SEED = 0x5EAF00D;
 
@@ -322,6 +449,7 @@ pass("Two simultaneous transfers run to completion");
     });
     $xfer->start();
     Amanda::MainLoop::run();
+    $xfer = undef;
     ok($got_timeout, "Amanda::Xfer::Filter::Process can be cancelled");
     # (note that this does not test all of the cancellation possibilities)
 }
@@ -345,6 +473,7 @@ pass("Two simultaneous transfers run to completion");
     });
     $xfer->start();
     Amanda::MainLoop::run();
+    $xfer = undef;
 
     is($dest->get(), 'ABCDEFGH' x 128,
 	"buffer captures the right bytes");
@@ -370,12 +499,13 @@ pass("Two simultaneous transfers run to completion");
     });
     $xfer->start();
     Amanda::MainLoop::run();
+    $xfer = undef;
 
     ok($got_err, "buffer stops the xfer if it doesn't have space");
 }
 
 SKIP: {
-    skip "not built with server", 25 unless Amanda::Util::built_with_component("server");
+    skip "not built with server", 45 unless Amanda::Util::built_with_component("server");
 
     my $disk_cache_dir = "$Installcheck::TMP";
     my $RANDOM_SEED = 0xFACADE;
@@ -397,11 +527,20 @@ SKIP: {
 	# set up vtapes
 	my $testconf = Installcheck::Run::setup();
 	$testconf->write();
+	config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+	my ($cfgerr_level, @cfgerr_errors) = config_errors();
+	if ($cfgerr_level >= $CFGERR_WARNINGS) {
+	    config_print_errors();
+	    BAIL_OUT("config errors");
+	}
 
 	# set up a device for slot 1
-	my $device = Amanda::Device->new("file:" . Installcheck::Run::load_vtape(1));
+	my $chg = Amanda::Changer->new();
+	my $res = load_vtape_res($chg, 1);
+	my $device = $res->{'device'};
+
 	die("Could not open VFS device: " . $device->error())
-	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS);
+	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_VOLUME_UNLABELED);
 
 	# write to it
 	my $hdr = Amanda::Header->new();
@@ -423,6 +562,7 @@ SKIP: {
 	$xfer->start($quit_cb);
 
 	Amanda::MainLoop::run();
+	$xfer = undef;
 	pass("write to a device (completed succesfully; data may not be correct)");
 
 	# finish up the file and device
@@ -441,6 +581,13 @@ SKIP: {
 	$xfer->start($quit_cb);
 
 	Amanda::MainLoop::run();
+	$xfer = undef;
+
+	$res->release(finished_cb => sub { Amanda::MainLoop::quit() });
+	Amanda::MainLoop::run();
+	$xfer = undef;
+	$chg->quit();
+
 	pass("read from a device succeeded, too, and data was correct");
     }
 
@@ -448,15 +595,21 @@ SKIP: {
     #   cancel_after_partnum - after this partnum is completed, cancel the xfer
     #   do_not_retry - do not retry a failed part - cancel the xfer instead
     sub test_taper_dest {
-	my ($src, $dest_sub, $expected_messages, $msg_prefix, %params) = @_;
+	my ($src, $dest_sub, $expected_messages, $expected_crcs, $msg_prefix, %params) = @_;
 	my $xfer;
-	my $device;
 	my $vtape_num = 1;
+	my @crcs;
 	my @messages;
 
 	# set up vtapes
 	my $testconf = Installcheck::Run::setup();
 	$testconf->write();
+	config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+	my ($cfgerr_level, @cfgerr_errors) = config_errors();
+	if ($cfgerr_level >= $CFGERR_WARNINGS) {
+	    config_print_errors();
+	    BAIL_OUT("config errors");
+	}
 
 	my $hdr = Amanda::Header->new();
 	$hdr->{'type'} = $Amanda::Header::F_DUMPFILE;
@@ -466,19 +619,25 @@ SKIP: {
 	$hdr->{'program'} = "INSTALLCHECK";
 
 	# set up a device for the taper dest
-	$device = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($vtape_num++));
+	my $chg = Amanda::Changer->new();
+	my $res = load_vtape_res($chg, $vtape_num++);
+	my $device = $res->{'device'};
+
 	die("Could not open VFS device: " . $device->error())
-	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS);
-	$device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
-	$device->property_set("LEOM", $params{'disable_leom'}? 0 : 1);
+	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_VOLUME_UNLABELED);
+	$r = $device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
+	$r = $device->property_set("LEOM", $params{'disable_leom'}? 0 : 1);
 	$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
 	my $dest = $dest_sub->($device);
 
+	my ($successful, $eof, $partnum, $eom);
 	# and create the xfer
 	$xfer = Amanda::Xfer->new([ $src, $dest ]);
 
+	my $start_new_part_1;
+	my $start_new_part_2;
 	my $start_new_part = sub {
-	    my ($successful, $eof, $partnum, $eom) = @_;
+	    ($successful, $eof, $partnum, $eom) = @_;
 
 	    if (exists $params{'cancel_after_partnum'}
 		    and $params{'cancel_after_partnum'} == $partnum) {
@@ -490,15 +649,30 @@ SKIP: {
 	    if (!$device || $eom) {
 		# set up a device and start writing a part to it
 		$device->finish() if $device;
-		$device = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($vtape_num++));
-		die("Could not open VFS device: " . $device->error())
-		    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS);
-		$dest->use_device($device);
-		$device->property_set("LEOM", $params{'disable_leom'}? 0 : 1);
-		$device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
-		$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
+		if ($res) {
+		    return $res->release(finished_cb => $start_new_part_1);
+		}
+		return $start_new_part_1->();
 	    }
+	    $start_new_part_2->();
+	};
 
+	$start_new_part_1 = sub {
+	    $chg->load(slot => $vtape_num++,
+		res_cb => sub {
+			(my $err, $res) = @_;
+			my $device = $res->{'device'};
+			die("Could not open VFS device: " . $device->error())
+			    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_VOLUME_UNLABELED);
+			$dest->use_device($device);
+			$r = $device->property_set("LEOM", $params{'disable_leom'}? 0 : 1);
+			$r = $device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
+			$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
+			$start_new_part_2->();
+		});
+	};
+
+	$start_new_part_2 = sub {
 	    # bail out if we shouldn't retry this part
 	    if (!$successful and $params{'do_not_retry'}) {
 		push @messages, "NOT-RETRYING";
@@ -521,7 +695,7 @@ SKIP: {
 	    if ($msg->{'type'} == $XMSG_ERROR) {
 		die $msg->{'elt'} . " failed: " . $msg->{'message'};
 	    } elsif ($msg->{'type'} == $XMSG_PART_DONE) {
-		push @messages, "PART-" . $msg->{'partnum'} . '-' . ($msg->{'successful'}? "OK" : "FAILED");
+		push @messages, "PART-" . $msg->{'partnum'} . '-' . $msg->{'size'} . '-' . ($msg->{'successful'}? "OK" : "FAILED");
 		push @messages, "EOM" if $msg->{'eom'};
 		$start_new_part->($msg->{'successful'}, $msg->{'eof'}, $msg->{'partnum'}, $msg->{'eom'});
 	    } elsif ($msg->{'type'} == $XMSG_DONE) {
@@ -529,35 +703,68 @@ SKIP: {
 		Amanda::MainLoop::quit();
 	    } elsif ($msg->{'type'} == $XMSG_CANCEL) {
 		push @messages, "CANCELLED";
+	    } elsif ($msg->{'type'} == $XMSG_CRC) {
+		push @crcs, "$msg->{'crc'}:$msg->{'size'}";
 	    } else {
-		push @messages, "$msg";
+		push @messages, "$msg->{'type'}";
 	    }
 	});
 
+	if ($src->isa("Amanda::Xfer::Source::Holding")) {
+            $src->start_recovery();
+	}
 	Amanda::MainLoop::call_later(sub { $start_new_part->(1, 0, -1); });
 	Amanda::MainLoop::run();
 
+	$res->release(finished_cb => sub { Amanda::MainLoop::quit() });
+	Amanda::MainLoop::run();
+	$chg->quit();
+	$xfer = undef;
+
 	is_deeply([@messages],
 	    $expected_messages,
 	    "$msg_prefix: element produces the correct series of messages")
 	or diag(Dumper([@messages]));
+
+
+	if ($expected_crcs) {
+	    # we sort because the crc can come in different order
+	    @crcs = sort @crcs;
+	    @$expected_crcs = sort @$expected_crcs;
+	    is_deeply([@crcs],
+		$expected_crcs,
+		"$msg_prefix: element produces the correct crcs")
+	    or diag(Dumper([@crcs]));
+	}
     }
 
     sub run_recovery_source {
-	my ($dest, $files, $expected_messages, $finished_cb) = @_;
+	my ($dest, $files, $expected_messages, $expected_crcs, $finished_cb) = @_;
 	my $device;
 	my @filenums;
+	my @crcs;
 	my @messages;
 	my $xfer;
 	my $dev;
 	my $src;
+	my $chg;
+	my $res;
 
 	my $steps = define_steps
+	    finalize => sub { $xfer = undef; },
 	    cb_ref => \$finished_cb;
 
 	step setup => sub {
 	    # we need a device up front, so sneak a peek into @$files
-	    $dev = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($files->[0]));
+	    $chg = Amanda::Changer->new();
+	    $chg->load(slot => $files->[0],
+		res_cb => $steps->{'first_load'});
+	};
+
+	step first_load => sub {
+	    (my $err, $res) = @_;
+	    $dev = $res->{'device'};
+
 	    $src = Amanda::Xfer::Source::Recovery->new($dev);
 	    $xfer = Amanda::Xfer->new([ $src, $dest ]);
 
@@ -566,6 +773,7 @@ SKIP: {
 	};
 
 	step got_ready => sub {
+	    return $res->release(finished_cb => $steps->{'load_slot'}) if $res;
 	    $steps->{'load_slot'}->();
 	};
 
@@ -578,7 +786,12 @@ SKIP: {
 	    my $slot = shift @$files;
 	    @filenums = @{ shift @$files };
 
-	    $dev = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($slot));
+	    $chg->load(slot => $slot,
+		res_cb => $steps->{'loaded'});
+	};
+	step loaded => sub {
+	    (my $err, $res) = @_;
+	    $dev = $res->{'device'};
 	    if ($dev->status != $Amanda::Device::DEVICE_STATUS_SUCCESS) {
 		die $dev->error_or_status();
 	    }
@@ -594,7 +807,7 @@ SKIP: {
 
 	step seek_file => sub {
 	    if (!@filenums) {
-		return $steps->{'load_slot'}->();
+		return $steps->{'got_ready'}->();
 	    }
 
 	    my $hdr = $dev->seek_file(shift @filenums);
@@ -613,7 +826,7 @@ SKIP: {
 	    if ($msg->{'type'} == $XMSG_ERROR) {
 		die $msg->{'elt'} . " failed: " . $msg->{'message'};
 	    } elsif ($msg->{'type'} == $XMSG_PART_DONE) {
-		push @messages, "KB-" . ($msg->{'size'}/1024);
+		push @messages, "BYTES-" .  $msg->{'size'};
 		$steps->{'seek_file'}->();
 	    } elsif ($msg->{'type'} == $XMSG_DONE) {
 		push @messages, "DONE";
@@ -623,6 +836,8 @@ SKIP: {
 		$steps->{'got_ready'}->();
 	    } elsif ($msg->{'type'} == $XMSG_CANCEL) {
 		push @messages, "CANCELLED";
+	    } elsif ($msg->{'type'} == $XMSG_CRC) {
+		push @crcs, "$msg->{'crc'}:$msg->{'size'}";
 	    }
 	};
 
@@ -632,6 +847,16 @@ SKIP: {
 		"files read back and verified successfully with Amanda::Xfer::Recovery::Source")
 	    or diag(Dumper([@messages]));
 
+	    if ($expected_crcs) {
+		#@crcs = sort @crcs;
+		#@$expected_crcs = sort @$expected_crcs;
+		is_deeply([@crcs],
+			$expected_crcs,
+			"element produces the correct crcs")
+		or diag(Dumper([@crcs]));
+	    }
+
+	    $chg->quit();
 	    $finished_cb->();
 	};
     }
@@ -686,8 +911,9 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 						     520*1024, 0);
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-OK", "PART-4-OK",
-	  "DONE" ],
+	[ "PART-1-557056-OK", "PART-2-557056-OK", "PART-3-557056-OK",
+	  "PART-4-326656-OK", "DONE" ],
+	[ 'e896f9b1:1997824', 'e896f9b1:1997824' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - simple splitting");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -695,15 +921,21 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-544',
+	  'BYTES-557056',
 	  'PART',
-	  'KB-544',
+	  'BYTES-557056',
 	  'PART',
-	  'KB-544',
+	  'BYTES-557056',
 	  'PART',
-	  'KB-319',
+	  'BYTES-326656',
 	  'DONE'
-	]);
+	],
+	[ 'afce8b74:557056',
+	  'fe33b4e9:1114112',
+	  '869c563f:1671168',
+	  'e896f9b1:1997824',
+	  'e896f9b1:1997824' ]
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*3.1, $RANDOM_SEED),
@@ -712,9 +944,10 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 						     1024*1024, 0);
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-OK", "EOM",
-	  "PART-4-OK",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-294912-OK", "EOM",
+	  "PART-4-858521-OK",
 	  "DONE" ],
+	[ 'eda70336:3250585', 'eda70336:3250585' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - splitting and spanning with LEOM");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -722,15 +955,21 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-288',
+	  'BYTES-294912',
 	  'PART',
-	  'KB-838',
+	  'BYTES-858521',
 	  'DONE'
-	]);
+	],
+	[ 'd4b66333:1048576',
+	  'ab6d231b:2097152',
+	  '0e0acec2:2392064',
+	  'eda70336:3250585',
+	  'eda70336:3250585' ],
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*1.5, $RANDOM_SEED),
@@ -739,8 +978,9 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 						     0, 0);
 	},
-	[ "PART-1-OK",
+	[ "PART-1-1572864-OK",
 	  "DONE" ],
+	[ '102b1445:1572864', '102b1445:1572864' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - no splitting");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -748,9 +988,12 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-1536',
+	  'BYTES-1572864',
 	  'DONE'
-	]);
+	],
+	[ '102b1445:1572864',
+	  '102b1445:1572864' ],
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*3.1, $RANDOM_SEED),
@@ -759,9 +1002,10 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 						     2368*1024, 0);
 	},
-	[ "PART-1-OK", "PART-2-OK", "EOM",
-	  "PART-2-OK",
+	[ "PART-1-2424832-OK", "PART-2-0-OK", "EOM",
+	  "PART-2-825753-OK",
 	  "DONE" ],
+	[ 'eda70336:3250585', 'eda70336:3250585' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - LEOM hits in file 2 header");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -769,13 +1013,18 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-2368',
+	  'BYTES-2424832',
 	  'PART',
-	  'KB-0', # this wouldn't be in the catalog, but it's on the vtape
+	  'BYTES-0', # this wouldn't be in the catalog, but it's on the vtape
 	  'PART',
-	  'KB-806',
+	  'BYTES-825753',
 	  'DONE'
-	]);
+	],
+	[ '96ff2746:2424832',
+	  '96ff2746:2424832',
+	  'eda70336:3250585',
+	  'eda70336:3250585' ],
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*3.1, $RANDOM_SEED),
@@ -784,8 +1033,9 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 						     2368*1024, 0);
 	},
-	[ "PART-1-OK", "PART-2-FAILED", "EOM",
+	[ "PART-1-2424832-OK", "PART-2-98304-FAILED", "EOM",
 	  "NOT-RETRYING", "CANCELLED", "DONE" ],
+	[ '96ff2746:2424832', '626c1849:2621440' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - LEOM fails, PEOM => failure",
 	disable_leom => 1, do_not_retry => 1);
 
@@ -797,9 +1047,10 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024,
 						     1024*1024, 1, undef),
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-FAILED", "EOM",
-	  "PART-3-OK", "PART-4-OK", "PART-5-OK",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-393216-FAILED",
+	  "EOM", "PART-3-1048576-OK", "PART-4-1048576-OK", "PART-5-104857-OK",
 	  "DONE" ],
+	[ 'c201f5aa:4299161' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - mem cache");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -807,17 +1058,24 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-102',
+	  'BYTES-104857',
 	  'DONE'
-	]);
+	],
+	[ 'd4b66333:1048576',
+	  'ab6d231b:2097152',
+	  'ca8b7765:3145728',
+	  '8f6c1b34:4194304',
+	  'c201f5aa:4299161',
+	  'c201f5aa:4299161' ],
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*4.1, $RANDOM_SEED),
@@ -826,9 +1084,10 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024,
 					      1024*1024, 0, $disk_cache_dir),
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-FAILED", "EOM",
-	  "PART-3-OK", "PART-4-OK", "PART-5-OK",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-393216-FAILED",
+	  "EOM", "PART-3-1048576-OK", "PART-4-1048576-OK", "PART-5-104857-OK",
 	  "DONE" ],
+	[ 'c201f5aa:4299161' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - disk cache");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -836,17 +1095,24 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-102',
+	  'BYTES-104857',
 	  'DONE'
-	]);
+	],
+	[ 'd4b66333:1048576',
+	  'ab6d231b:2097152',
+	  'ca8b7765:3145728',
+	  '8f6c1b34:4194304',
+	  'c201f5aa:4299161',
+	  'c201f5aa:4299161' ]
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*2, $RANDOM_SEED),
@@ -855,8 +1121,9 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024,
 						    1024*1024, 0, undef),
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-OK",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-0-OK",
 	  "DONE" ],
+	[ 'ab6d231b:2097152' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - no cache (no failed parts; exact multiple of part size)");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -864,13 +1131,18 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-1024',
+	  'BYTES-1048576',
 	  'PART',
-	  'KB-0',
+	  'BYTES-0',
 	  'DONE'
-	]);
+	],
+	[ 'd4b66333:1048576',
+	  'ab6d231b:2097152',
+	  'ab6d231b:2097152',
+	  'ab6d231b:2097152' ]
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*2, $RANDOM_SEED),
@@ -878,7 +1150,8 @@ SKIP: {
 	    my ($first_dev) = @_;
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024, 0, 0, undef),
 	},
-	[ "PART-1-OK", "DONE" ],
+	[ "PART-1-2097152-OK", "DONE" ],
+	[ 'ab6d231b:2097152' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - no splitting (fits on volume)");
     test_recovery_source(
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
@@ -886,9 +1159,12 @@ SKIP: {
 	[
 	  'READY',
 	  'PART',
-	  'KB-2048',
+	  'BYTES-2097152',
 	  'DONE'
-	]);
+	],
+	[ 'ab6d231b:2097152',
+	  'ab6d231b:2097152' ]
+	);
 
     test_taper_dest(
 	Amanda::Xfer::Source::Random->new(1024*1024*4.1, $RANDOM_SEED),
@@ -896,8 +1172,9 @@ SKIP: {
 	    my ($first_dev) = @_;
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024, 0, 0, undef),
 	},
-	[ "PART-1-FAILED", "EOM",
+	[ "PART-1-2555904-FAILED", "EOM",
 	  "NOT-RETRYING", "CANCELLED", "DONE" ],
+	[ '00000000:0' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - no splitting (doesn't fit on volume -> fails)",
 	do_not_retry => 1);
 
@@ -908,9 +1185,10 @@ SKIP: {
 	    Amanda::Xfer::Dest::Taper::Cacher->new($first_dev, 128*1024,
 					    1024*1024, 0, $disk_cache_dir),
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-FAILED", "EOM",
-	  "PART-3-OK", "PART-4-OK", "CANCEL",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-393216-FAILED",
+	  "EOM", "PART-3-1048576-OK", "PART-4-1048576-OK", "CANCEL",
 	  "CANCELLED", "DONE" ],
+	[ '8f6c1b34:4194304' ],
 	"Amanda::Xfer::Dest::Taper::Cacher - cancellation after success",
 	cancel_after_partnum => 4);
 
@@ -918,17 +1196,19 @@ SKIP: {
 
     $holding_file = make_holding_files(3);
 
+    my $xfer_source_holding = Amanda::Xfer::Source::Holding->new($holding_file);
     test_taper_dest(
-	Amanda::Xfer::Source::Holding->new($holding_file),
+	$xfer_source_holding,
 	sub {
 	    my ($first_dev) = @_;
 	    Amanda::Xfer::Dest::Taper::Splitter->new($first_dev, 128*1024,
 					    1024*1024, 1);
 	},
-	[ "PART-1-OK", "PART-2-OK", "PART-3-FAILED", "EOM",
-	  "PART-3-OK", "PART-4-OK", "PART-5-FAILED", "EOM",
-	  "PART-5-OK", "PART-6-OK", "PART-7-OK",
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-393216-FAILED", "EOM",
+	  "PART-3-1048576-OK", "PART-4-1048576-OK", "PART-5-393216-FAILED", "EOM",
+	  "PART-5-1048576-OK", "PART-6-1048576-OK", "PART-7-0-OK",
 	  "DONE" ],
+	[ 'a27c85ce:6291456', 'a27c85ce:6291456' ],
 	"Amanda::Xfer::Dest::Taper::Splitter - Amanda::Xfer::Source::Holding "
 	. "acts as a source and supplies cache_inform",
 	disable_leom => 1);
@@ -945,6 +1225,7 @@ SKIP: {
 	my $file_size = $part_size * 4 + 100 * 1024;
 	my $cache_file = "$Installcheck::TMP/cache_file";
 	my $vtape_num = 1;
+	my @crcs;
 
 	# set up our "cache", cleverly using an Amanda::Xfer::Dest::Fd
 	open($fh, ">", "$cache_file") or die("Could not open '$cache_file' for writing");
@@ -962,9 +1243,10 @@ SKIP: {
 	    }
 	});
 	Amanda::MainLoop::run();
+	$xfer = undef;
 	close($fh);
 
-	# create a list of holding chuunks, some slab-aligned, some part-aligned,
+	# create a list of holding chunks, some slab-aligned, some part-aligned,
 	# some not
 	my @holding_chunks;
 	my $offset = 0;
@@ -989,6 +1271,13 @@ SKIP: {
 	# set up vtapes
 	my $testconf = Installcheck::Run::setup();
 	$testconf->write();
+	config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+	my ($cfgerr_level, @cfgerr_errors) = config_errors();
+	if ($cfgerr_level >= $CFGERR_WARNINGS) {
+	    config_print_errors();
+	    BAIL_OUT("config errors");
+	}
+
 
 	my $hdr = Amanda::Header->new();
 	$hdr->{'type'} = $Amanda::Header::F_DUMPFILE;
@@ -1001,11 +1290,13 @@ SKIP: {
 	open($fh, "<", "$cache_file") or die("Could not open '$cache_file' for reading");
 
 	# set up a device for writing
-	$device = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($vtape_num++));
+	my $chg = Amanda::Changer->new();
+	my $res = load_vtape_res($chg, $vtape_num++);
+	$device = $res->{'device'};
 	die("Could not open VFS device: " . $device->error())
-	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS);
-	$device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
-	$device->property_set("LEOM", 0);
+	    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_VOLUME_UNLABELED);
+	$r = $device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
+	$r = $device->property_set("LEOM", 0);
 	$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
 
 	my $dest = Amanda::Xfer::Dest::Taper::Splitter->new($device, 128*1024,
@@ -1015,21 +1306,40 @@ SKIP: {
 	    $dest,
 	]);
 
+	my ($successful, $eof, $last_partnum);
+
+	my $start_new_part_1;
+	my $start_new_part_2;
 	my $start_new_part = sub {
-	    my ($successful, $eof, $last_partnum) = @_;
+	    ($successful, $eof, $last_partnum) = @_;
 
 	    if (!$device || !$successful) {
 		# set up a device and start writing a part to it
 		$device->finish() if $device;
-		$device = Amanda::Device->new("file:" . Installcheck::Run::load_vtape($vtape_num++));
-		die("Could not open VFS device: " . $device->error())
-		    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS);
-		$dest->use_device($device);
-		$device->property_set("LEOM", 0);
-		$device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
-		$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
+		if ($res) {
+		    return $res->release(finished_cb => $start_new_part_1);
+		}
+		return $start_new_part_1->();
 	    }
+	    $start_new_part_2->();
+	};
 
+	$start_new_part_1 = sub {
+	    $chg->load(slot => $vtape_num++,
+		res_cb => sub {
+		    (my $err, $res) = @_;
+		$device = $res->{'device'};
+		die("Could not open VFS device: " . $device->error())
+		    unless ($device->status() == $Amanda::Device::DEVICE_STATUS_VOLUME_UNLABELED);
+		$dest->use_device($device);
+		$r = $device->property_set("LEOM", 0);
+		$r = $device->property_set("MAX_VOLUME_USAGE", 1024*1024*2.5);
+		$device->start($Amanda::Device::ACCESS_WRITE, "TESTCONF01", "20080102030405");
+		$start_new_part_2->();
+	    });
+	};
+
+	$start_new_part_2 = sub {
 	    # feed enough chunks to cache_inform
 	    my $upto = ($last_partnum+2) * $part_size;
 	    while (@holding_chunks and $holding_chunks[0]->[1] < $upto) {
@@ -1053,13 +1363,18 @@ SKIP: {
 	    if ($msg->{'type'} == $XMSG_ERROR) {
 		push @messages, "ERROR: $msg->{message}";
 	    } elsif ($msg->{'type'} == $XMSG_PART_DONE) {
-		push @messages, "PART-" . ($msg->{'successful'}? "OK" : "FAILED");
+		push @messages, "PART-" . $msg->{'partnum'} . '-' . $msg->{'size'} . '-' . ($msg->{'successful'}? "OK" : "FAILED");
 		$start_new_part->($msg->{'successful'}, $msg->{'eof'}, $msg->{'partnum'});
 	    } elsif ($msg->{'type'} == $XMSG_DONE) {
 		push @messages, "DONE";
-		Amanda::MainLoop::quit();
+		$res->release(finished_cb => sub {
+		    $chg->quit();
+		    Amanda::MainLoop::quit();
+		});
 	    } elsif ($msg->{'type'} == $XMSG_CANCEL) {
 		push @messages, "CANCELLED";
+	    } elsif ($msg->{'type'} == $XMSG_CRC) {
+		push @crcs, "$msg->{'crc'}:$msg->{'size'}";
 	    } else {
 		push @messages, $msg->{'type'};
 	    }
@@ -1067,16 +1382,17 @@ SKIP: {
 
 	Amanda::MainLoop::call_later(sub { $start_new_part->(1, 0, -1); });
 	Amanda::MainLoop::run();
+	$xfer = undef;
 
 	unlink($cache_file);
 
-	return @messages;
+	return ( @messages, @crcs );
     }
 
     is_deeply([ test_taper_dest_splitter_cache_inform() ],
-	[ "PART-OK", "PART-OK", "PART-FAILED",
-	  "PART-OK", "PART-OK", "PART-OK",
-	  "DONE" ],
+	[ "PART-1-1048576-OK", "PART-2-1048576-OK", "PART-3-393216-FAILED",
+	  "PART-3-1048576-OK", "PART-4-1048576-OK", "PART-5-102400-OK",
+	  "DONE", '9548ff3c:4296704', '9548ff3c:4296704'],
 	"cache_inform: splitter element produces the correct series of messages");
 
     rmtree($holding_base);
@@ -1099,10 +1415,10 @@ SKIP: {
     my $mkdevice = sub {
 	my $dev = Amanda::Device->new("ndmp:127.0.0.1:$ndmp_port\@$drive");
 	die "can't create device" unless $dev->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS;
-	$dev->property_set("indirect", 0) or die "can't set INDIRECT";
-	$dev->property_set("verbose", 1) or die "can't set VERBOSE";
-	$dev->property_set("ndmp_username", "ndmp") or die "can't set username";
-	$dev->property_set("ndmp_password", "ndmp") or die "can't set password";
+	$r = $dev->property_set("indirect", 0) and die "can't set INDIRECT: $r";
+	$r = $dev->property_set("verbose", 1) and die "can't set VERBOSE: $r";
+	$r = $dev->property_set("ndmp_username", "ndmp") and die "can't set username: $r";
+	$r = $dev->property_set("ndmp_password", "ndmp") and die "can't set password: $r";
 
 	return $dev;
     };
@@ -1151,7 +1467,7 @@ SKIP: {
 
 		$start_new_part->(1, 0); # start first part
 	    } elsif ($msg->{'type'} == $XMSG_PART_DONE) {
-		push @messages, "PART-" . $msg->{'partnum'} . '-' . ($msg->{'successful'}? "OK" : "FAILED");
+		push @messages, "PART-" . $msg->{'partnum'} . '-' . $msg->{'size'} . '-' . ($msg->{'successful'}? "OK" : "FAILED");
 		if ($do_cancel and $msg->{'partnum'} == 2) {
 		    $xfer->cancel();
 		} else {
@@ -1162,8 +1478,10 @@ SKIP: {
 		Amanda::MainLoop::quit();
 	    } elsif ($msg->{'type'} == $XMSG_CANCEL) {
 		push @messages, "CANCELLED";
+	    } elsif ($msg->{'type'} == $XMSG_CRC) {
+		#push @messages, "CRC";
 	    } else {
-		push @messages, "$msg";
+		push @messages, "$msg->{'type'}";
 	    }
 	};
 
@@ -1171,9 +1489,11 @@ SKIP: {
 	# in an error state.  NDMP devices do not support append, so starting in
 	# append mode should trigger the failure.
 	if ($do_cancel eq 'in_setup') {
+	    no warnings 'once';
 	    if ($dev->start($Amanda::Device::ACCESS_APPEND, "MYLABEL", undef)) {
 		die "successfully started NDMP device in ACCESS_APPEND?!";
 	    }
+	    use warnings 'once';
 	}
 
 	$xfer->start($xmsg_cb);
@@ -1189,12 +1509,14 @@ SKIP: {
 	};
 
 	Amanda::MainLoop::run();
+	$xfer = undef;
 
 	$dev->finish();
 
 	if (!$do_cancel) {
 	    is_deeply([@messages],
-		[ 'READY', 'PART-1-OK', 'PART-2-OK', 'PART-3-OK', 'DONE' ],
+		[ 'READY', 'PART-1-524288-OK', 'PART-2-524288-OK',
+		  'PART-3-65536-OK', 'DONE' ],
 		"Amanda::Xfer::Dest::Taper::DirectTCP element produces the correct series of messages")
 	    or diag(Dumper([@messages]));
 	} elsif ($do_cancel eq 'in_setup') {
@@ -1204,7 +1526,8 @@ SKIP: {
 	    or diag(Dumper([@messages]));
 	} else {
 	    is_deeply([@messages],
-		[ 'READY', 'PART-1-OK', 'PART-2-OK', 'CANCELLED', 'DONE' ],
+		[ 'READY', 'PART-1-524288-OK', 'PART-2-524288-OK', 'CANCELLED',
+		  'DONE' ],
 		"Amanda::Xfer::Dest::Taper::DirectTCP element produces the correct series of messages when cancelled in mid-xfer")
 	    or diag(Dumper([@messages]));
 	}
@@ -1217,6 +1540,17 @@ SKIP: {
 
     $ndmp->cleanup();
 }
+if (!Amanda::Util::built_with_component("server")) {
+    my $testconf = Installcheck::Run::setup();
+    $testconf->write();
+    config_init($CONFIG_INIT_EXPLICIT_NAME|$CONFIG_INIT_CLIENT, "TESTCONF");
+    my ($cfgerr_level, @cfgerr_errors) = config_errors();
+    if ($cfgerr_level >= $CFGERR_WARNINGS) {
+	config_print_errors();
+	BAIL_OUT("config errors");
+    }
+}
+
 
 # directtcp stuff
 
@@ -1283,6 +1617,9 @@ SKIP: {
     # let the already-started transfers go out of scope before they 
     # complete, as a memory management test..
     Amanda::MainLoop::run();
+    $xferA = undef;
+    $xferB = undef;
+    $xferC = undef;
     pass("Three xfers interlinked via DirectTCP complete successfully");
 }
 
@@ -1310,6 +1647,7 @@ SKIP: {
     $xfer->cancel();
 
     Amanda::MainLoop::run();
+    $xfer = undef;
     pass("A DirectTCP accept operation can be cancelled");
 }
 
