@@ -1,5 +1,6 @@
 # vim:ft=perl
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 package Installcheck::Application;
@@ -28,6 +29,7 @@ use IO::Handle;
 use POSIX qw( :errno_h :fcntl_h );
 use POSIX qw( EAGAIN );
 use Test::More;
+use JSON;
 
 use strict;
 use warnings;
@@ -484,7 +486,7 @@ sub _make_done_cb {
 sub _parse_size {
     my $sstr = shift @_;
 
-    confess "failed to parse size" unless ($sstr =~ /^(\d+)(\D?)$/i);
+    confess "failed to parse size" unless ($sstr =~ /^[\-]?(\d+)(\D?)$/i);
     my $size = 0 + $1;
     my $suf = lc($2);
 
@@ -555,8 +557,10 @@ sub backup {
             }
         } elsif ($l =~ /^sendbackup: (.*)$/) {
             my $rem = $1;
-            if ($rem =~ /^size (\d+)$/i) {
+            if ($rem =~ /^size ([\-]?\d+)$/i) {
                 $size = _parse_size($1);
+            } elsif ($rem =~ /error \[(.*)\]$/) {
+                push @errors, $1;
             } elsif (lc($rem) eq 'end') {
                 # do nothing
             } else {
@@ -597,6 +601,9 @@ sub restore {
 sub estimate {
     my $self = shift @_;
     my %nargs = @_;
+    my $level;
+    my $size;
+    my @errors;
 
     foreach my $k ( qw(device level) ) {
         confess "$k required" unless defined($nargs{$k});
@@ -614,12 +621,20 @@ sub estimate {
             1 => {'child_mode' => 'w', 'save_to' => \$est},
         }
     );
-    $est =~ /^(\d+) (\d+) 1\n$/;
-    my ($level, $size) = ($1, $2);
-    $level = 0 + $level;
-    $size = ($size eq '-1')? -1 : _parse_size($size);
+    if ($est =~ /ERROR "(.*)"$/m) {
+	push @errors, $1;
+    } elsif ($est =~ /ERROR \[(.*)\]$/m) {
+	push @errors, $1;
+    } elsif ($est =~ /ERROR (.*)$/m) {
+	push @errors, $1;
+    }
+    if ($est =~ /^(\d+) (\d+) 1\n$/m) {
+	($level, $size) = ($1, $2);
+	$level = 0 + $level;
+	$size = ($size eq '-1')? -1 : _parse_size($size);
+    }
 
-    {'level' => $level, 'size' => $size, 'exit_status' => $exit_status};
+    {'level' => $level, 'size' => $size, 'errors' => \@errors, 'exit_status' => $exit_status};
 }
 
 sub selfcheck {
@@ -656,6 +671,45 @@ sub selfcheck {
     {'oks' => \@oks, 'errors' => \@errors, 'exit_status' => $exit_status};
 }
 
+sub selfcheck_message {
+    my $self = shift @_;
+    my %nargs = @_;
+
+    foreach my $k ( qw(device) ) {
+        confess "$k required" unless defined($nargs{$k});
+    }
+    $nargs{'disk'} ||=  $nargs{'device'};
+
+    my @args = map {my $k = $_; ("--$k", $nargs{$k}) } keys(%nargs);
+
+    my $msg_str;
+    push @args, '--message', 'json';
+    my $exit_status = _exec($self, 'selfcheck', \@args,
+        {
+            1 => {'child_mode' => 'w', 'save_to' => \$msg_str},
+        }
+    );
+
+    my $json = JSON->new->allow_nonref;
+    my (@oks, @errors, $messages);
+    if ($msg_str =~ /^MESSAGE JSON/) {
+	$msg_str = substr $msg_str, 13;
+	$msg_str = '[ ' . $msg_str . ' ]';
+	$messages = $json->decode($msg_str);
+	foreach my $message (@{$messages}) {
+	    if ($message->{'severity'} eq "success" ||
+		$message->{'severity'} eq "info") {
+		push @oks, $message;
+	    } else {
+		push @errors, $message;
+	    }
+	}
+    } else {
+	confess "invalid message: $msg_str, expecting 'MESSAGE JSON'";
+    }
+    { 'oks' => \@oks, 'errors' => \@errors, 'messages' => $messages, 'exit_status' => $exit_status};
+}
+
 # XXX: print?
 
 1;
