@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +15,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 package Installcheck::ClientService;
@@ -496,10 +497,11 @@ sub _add_stream {
 
 	$async_read_cb = sub {
 	    my ($err, $data) = @_;
+	    return $self->{'got_eof'}{$name} = 1 if $err;
 	    die "on stream $name: $err" if $err;
 
 	    # log it
-	    $self->_log_data("<<", $name, $data);
+	    $self->_log_data("<<", "$name:$rfd", $data);
 
 	    # prep for next time
 	    if ($data) {
