@@ -1,5 +1,6 @@
 # vim:ft=perl
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 package Installcheck::Config;
@@ -24,6 +25,7 @@ use Amanda::Paths;
 use Amanda::Constants;
 use File::Path;
 use Carp;
+use Test::More;
 
 =head1 NAME
 
@@ -87,10 +89,12 @@ sub new {
 
 	    # These dirs are under CONFIG_DIR just for ease of destruction.
 	    # This is not a recommended layout!
+	    'inparallel' => "2",
 	    'infofile' => "\"$infofile\"",
 	    'logdir' => "\"$logdir\"",
 	    'indexdir' => "\"$indexdir\"",
 	    'org' => "\"$org\"",
+	    'send-amreport-on' =>'never',
 
 	    # (this is actually added while writing the config file, if not
 	    # overridden by the caller)
@@ -118,6 +122,8 @@ sub new {
 	'script' => [ ],
 	'devices' => [ ],
 	'changers' => [ ],
+	'policy' => [ ],
+	'storage' => [ ],
 	'text' => '',
 
 	'dles' => [ ],
@@ -148,6 +154,33 @@ sub add_param {
     push @{$self->{'params'}}, $param, $value;
 }
 
+=item C<rm_param($param, $value)>
+
+Remove the given parameter from the configuration file.  Note that strings which
+should be quoted in the configuration file itself must be double-quoted here,
+If the value is not provided, it remove all params of that name.
+e.g.,
+
+  $testconf->rm_param('org' => '"MyOrganization"');
+
+=cut
+
+sub rm_param {
+    my $self = shift;
+    my ($rm_param, $rm_value) = @_;
+
+    my @new_params;
+
+    while (@{$self->{'params'}}) {
+	$param = shift @{$self->{'params'}};
+	$value = shift @{$self->{'params'}};
+	if ($param ne $rm_param || (defined $rm_value && $value ne $rm_value)) {
+	    push @new_params, $param, $value;
+	}
+    }
+    @{$self->{'params'}} = @new_params;
+}
+
 =item C<add_client_param($param, $value)>, C<add_client_config_param($param, $value)>
 
 Add the given parameter to the client configuration file, as C<add_param> does
@@ -206,6 +239,8 @@ sub remove_param {
 =item C<add_changer($name, $values_arrayref)>
 =item C<add_interactivity($name, $values_arrayref)>
 =item C<add_taperscan($name, $values_arrayref)>
+=item C<add_policy($name, $values_arrayref)>
+=item C<add_storage($name, $values_arrayref)>
 
 Add the given subsection to the configuration file, including all values in the
 arrayref.  The values should be specified as alternating key/value pairs.
@@ -293,6 +328,18 @@ sub add_taperscan {
     $self->_add_subsec("taperscans", $name, 1, $values);
 }
 
+sub add_policy {
+    my $self = shift;
+    my ($name, $values) = @_;
+    $self->_add_subsec("policys", $name, 1, $values);
+}
+
+sub add_storage {
+    my $self = shift;
+    my ($name, $values) = @_;
+    $self->_add_subsec("storages", $name, 1, $values);
+}
+
 =item C<add_text($text)>
 
 Add arbitrary text to the config file.
@@ -414,6 +461,8 @@ sub _write_amanda_conf {
     $self->_write_amanda_conf_subsection($amanda_conf, "changer", $self->{"changers"});
     $self->_write_amanda_conf_subsection($amanda_conf, "interactivity", $self->{"interactivities"});
     $self->_write_amanda_conf_subsection($amanda_conf, "taperscan", $self->{"taperscans"});
+    $self->_write_amanda_conf_subsection($amanda_conf, "policy", $self->{"policys"});
+    $self->_write_amanda_conf_subsection($amanda_conf, "storage", $self->{"storages"});
     print $amanda_conf "\n", $self->{'text'}, "\n";
     print $amanda_conf "taperscan $taperscan\n" if $taperscan;
 
