@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,19 +15,23 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 18;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck;
 use Installcheck::Run qw( run run_get );
 use Amanda::Paths;
 use Amanda::Constants;
 use File::Path qw( mkpath rmtree );
+use Amanda::Debug;
+
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
 
 my $tmpdir = "$Installcheck::TMP/amarchiver-installcheck";
 my $archfile = "$tmpdir/test.amar";
@@ -93,7 +98,7 @@ run('amarchiver', '--create', '--file', $archfile, "test.tmp-1", "test.tmp-2")
 
 ok(run('amarchiver', '--list', '--file', $archfile),
     "archive listing succeeds");
-is($Installcheck::Run::stdout, "test.tmp-1\ntest.tmp-2\n",
+is($Installcheck::Run::stdout, "test.tmp-1\ntest.tmp-2\nsize: 4096\n",
     "..and output is correct");
 
 # test extracting archives
