@@ -1,4 +1,5 @@
-# Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,19 +15,24 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 17;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
+use Installcheck;
 use Installcheck::Config;
 use Installcheck::Run qw(run run_get run_err $diskname);
 use Installcheck::Dumpcache;
+use Amanda::Debug;
 use Amanda::Paths;
 
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
+
 my $testconf;
 
 ##
@@ -80,14 +86,14 @@ is_deeply([ sort split "\n", $Installcheck::Run::stdout],
 ok(run('amdevcheck', 'TESTCONF', "--properties", "CANONICAL_NAME"),
     "check canonical_name property value");
 is_deeply([ sort split "\n", $Installcheck::Run::stdout],
-	  [ sort "CANONICAL_NAME=file:" . Installcheck::Run::vtape_dir() ],
+	  [ sort "CANONICAL_NAME=file:" . Installcheck::Run::vtape_dir() . "/slot1" ],
     ".. and confirm it is set to default value");
 
 ok(run('amdevcheck', 'TESTCONF', "--properties", "BLOCK_SIZE,CANONICAL_NAME"),
     "check a list of properties");
 is_deeply([ sort split "\n", $Installcheck::Run::stdout],
 	  [ sort "BLOCK_SIZE=32768",
-	  	 "CANONICAL_NAME=file:" . Installcheck::Run::vtape_dir() ],
+		 "CANONICAL_NAME=file:" . Installcheck::Run::vtape_dir() . "/slot1" ],
     ".. with correct results");
 
 ok(run('amdevcheck', 'TESTCONF', 'null:null'),
