@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,21 +15,26 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 3;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
+use Installcheck;
 use Installcheck::Dumpcache;
 use Installcheck::Config;
 use Installcheck::Run qw(run run_err $diskname amdump_diag);
 use Amanda::Config qw( :init );
+use Amanda::Debug;
 use Amanda::Paths;
 use Sys::Hostname;
 
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
+
 my $testconf;
 
 # Just run amdump.
@@ -56,7 +62,7 @@ $testconf->write();
 
 ok(run('amdump_client', '--config', 'TESTCONF', 'list'), "'amdump_client list' runs successfully");
 is($Installcheck::Run::stdout,
-    "config: TESTCONF\ndiskname\n",
+    "diskname\n",
     "'amdump_client list' list diskname");
 ok(run('amdump_client', '--config', 'TESTCONF', 'dump'), "'amdump_client dump' runs successfully")
     or amdump_diag();
