@@ -1,4 +1,5 @@
-# Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,20 +15,24 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
-use Test::More tests => 86;
+use Test::More tests => 95;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use Installcheck;
 use Installcheck::Config;
 use Installcheck::Run qw(run run_get run_err);
+use Amanda::Debug;
 use Amanda::Paths;
 use Cwd;
 
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
+
 # this is re-created for each test
 my $testconf;
 
@@ -54,22 +59,22 @@ is(run_get('amgetconf', 'TESTCONF', "tapelist"), "tapelist",
     "tapelist defaults to 'tapelist'");
 is(run_get('amgetconf', 'TESTCONF', "usetimestamps"), "yes", 
     "usetimestamps defaults to 'yes'");
-is(run_get('amgetconf', 'TESTCONF', "send_amreport_on"), "ALL",
-    "send_amreport_on defaults to 'ALL'"); # (enum value is 0)
+is(run_get('amgetconf', 'TESTCONF', "send_amreport_on"), "NEVER",
+    "send_amreport_on set to 'NEVER'"); # (enum value is 0)
 is(run_get('amgetconf', 'TESTCONF', "taperalgo"), "FIRST",
     "taperalgo defaults to 'ALL'"); # (enum value is 0)
 is(run_get('amgetconf', 'TESTCONF', "printer"), "",
     "printer defaults to empty string, which is not an error");
 
-# test command-line parsing
+# test command-line parsing (amandates is in the global amanbda-client.conA)f
 is(run_get('amgetconf', 'TESTCONF', '--execute-where', 'client', 'amandates'),
-   $Amanda::Constants::DEFAULT_AMANDATES_FILE,
+    "$Installcheck::TMP/TESTCONF/amandates",
     "--execute-where client");
 is(run_get('amgetconf', 'TESTCONF', '--execute-where=client', 'amandates'),
-   $Amanda::Constants::DEFAULT_AMANDATES_FILE,
+    "$Installcheck::TMP/TESTCONF/amandates",
     "--execute-where=client");
 is(run_get('amgetconf', 'TESTCONF', '--client', 'amandates'),
-   $Amanda::Constants::DEFAULT_AMANDATES_FILE,
+    "$Installcheck::TMP/TESTCONF/amandates",
     "--client");
 
 is(run_get('amgetconf', 'TESTCONF', '--execute-where', 'server', 'reserve'), "100",
@@ -79,7 +84,7 @@ is(run_get('amgetconf', 'TESTCONF', '--execute-where=server', 'reserve'), "100",
 is(run_get('amgetconf', 'TESTCONF', '--execute-where=server', '--execute-where=server', 'reserve'), "100",
     "--execute-where=server --execute-where=server");
 is(run_get('amgetconf', 'TESTCONF', '--execute-where=client', '--execute-where=client', 'amandates'),
-   $Amanda::Constants::DEFAULT_AMANDATES_FILE,
+    "$Installcheck::TMP/TESTCONF/amandates",
     "--execute-where=client --execute-where=client");
 
 like(run_err('amgetconf', 'TESTCONF', '--execute-where=server', '--execute-where=client'),
@@ -130,6 +135,12 @@ is(run_get('amgetconf', 'TESTCONF', "build.CONFIG_DIR"), $CONFIG_DIR,
     "build.CONFIG_DIR is correct");
 is(run_get('amgetconf', 'TESTCONF', "build.__empty"), "",
     "empty build variables handled correctly");
+is(run_get('amgetconf', "build.amdatadir"), $amdatadir,
+    "empty build variables handled correctly");
+is(run_get('amgetconf', 'TESTCONF', "build.security_file"), $SECURITY_FILE,
+    "empty build variables handled correctly");
+is(run_get('amgetconf', "build.amandates_file"), $AMANDATES_FILE,
+    "empty build variables handled correctly");
 
 like(run_err('amgetconf', 'TESTCONF', "build.bogus-param"), qr(no such parameter),
     "bogus build parameters result in an error");
@@ -208,7 +219,7 @@ $testconf->add_param("device_property", '"turbo" "engaged"');
 $testconf->write();
 
 is_deeply([sort(+split(qr/\n/, run_get('amgetconf', 'TESTCONF', 'device_property')))],
-	  [sort('"power" "on"', '"turbo" "engaged"')],
+	  [sort('visible "power" "on"', 'visible "turbo" "engaged"')],
     "device_property can have multiple values");
 
 ##
@@ -287,9 +298,9 @@ is(run_get('amgetconf', 'TESTCONF', 'script_tOOl:my_script:eXECute-on'), 'PRE-DL
     "insensitive to case in subsec_key");
 is(run_get('amgetconf', 'TESTCONF', 'script-tool:my_script:execute_on'), 'PRE-DLE-AMCHECK',
     "insensitive to -/_ in subsec_key");
-is(run_get('amgetconf', 'TESTCONF', 'dumptype:testdump1:auth', '-odumptype:testdump:auth=SSH'), 'SSH',
+is(run_get('amgetconf', 'TESTCONF', 'dumptype:srvcompress:Auth', '-odumptype:srvcompress:auth=SSH'), 'SSH',
     "inherited setting are overrided");
-is(run_get('amgetconf', 'TESTCONF', 'dumptype:testdump1:compress', '-odumptype:testdump:compress=SERVER BEST'), 'SERVER BEST',
+is(run_get('amgetconf', 'TESTCONF', 'dumptype:srvcompress:compress', '-odumptype:srvcompress:compress=SERVER BEST'), 'SERVER BEST',
     "inherited default are overrided");
 
 is_deeply([sort(split(/\n/, run_get('amgetconf', 'TESTCONF', '--list', 'device')))],
@@ -344,12 +355,28 @@ $testconf->add_param("property", '"prop2" "value2"');
 $testconf->add_param("property", '"prop3" "value3"');
 $testconf->write();
 
-is(run_get('amgetconf', 'TESTCONF', "property:prop1"), "value1", 
+is(run_get('amgetconf', 'TESTCONF', "property:prop1"), "value1",
     "correctly returns property prop1 from the file");
-is(run_get('amgetconf', 'TESTCONF', "property:prop2"), "value2", 
+is(run_get('amgetconf', 'TESTCONF', "property:prop2"), "value2",
     "correctly returns property prop2 from the file");
-is(run_get('amgetconf', 'TESTCONF', "property:prop3"), "value3", 
+is(run_get('amgetconf', 'TESTCONF', "property:prop3"), "value3",
     "correctly returns property prop3 from the file");
-is(run_get('amgetconf', 'TESTCONF', "property"), "\"prop1\" \"value1\"\n\"prop2\" \"value2\"\n\"prop3\" \"value3\"", 
+is(run_get('amgetconf', 'TESTCONF', "property"), "hidden \"prop1\" \"value1\"\nhidden \"prop2\" \"value2\"\nhidden \"prop3\" \"value3\"",
     "correctly returns all propertiss from the file");
 
+isnt(run_get('amgetconf', '--platform'), "Unknown",
+    "correctly returns then platform");
+isnt(run_get('amgetconf', '--distro'), "Unknown",
+    "correctly returns then distro");
+
+$testconf->add_storage("STO", [ 'tpchanger' => '"/dev/nst0"' ]);
+$testconf->add_taperscan("SCAN", [ 'plugin' => '"lexical"' ]);
+$testconf->write();
+is(run_get('amgetconf', 'TESTCONF', 'storage:STO:tpchanger', '-ostorage:STO:tpchanger=toto'), 'toto',
+   "correctly returns STO tpchanger 1");
+is(run_get('amgetconf', 'TESTCONF', 'storage:STO:tpchanger', '-ostorage:STO:tpchanger=toto', '-ostorage=STO'), 'toto',
+   "correctly returns STO tpchanger 2");
+is(run_get('amgetconf', 'TESTCONF', 'taperscan:SCAN:plugin', '-otaperscan:SCAN:plugin=foobar'), 'foobar',
+   "correctly returns SCAN plugin 1");
+is(run_get('amgetconf', 'TESTCONF', 'taperscan:SCAN:plugin', '-otaperscan:SCAN:plugin=foobar', '-otaperscan=SCAN'), 'foobar',
+   "correctly returns SCAN plugin 2");
