@@ -1,4 +1,5 @@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,16 +15,17 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 15;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use strict;
 use warnings;
 use Installcheck;
 use Amanda::Constants;
+use Amanda::Debug;
 use Amanda::Paths;
 use Amanda::Tests;
 use File::Path;
@@ -37,6 +39,9 @@ unless ($Amanda::Constants::GNUTAR and -x $Amanda::Constants::GNUTAR) {
     exit 0;
 }
 
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
+
 my $app = Installcheck::Application->new('amraw');
 
 my $support = $app->support();
