@@ -1,4 +1,5 @@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,19 +15,19 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
-use Test::More tests => 167;
+use Test::More tests => 192;
 
 use strict;
 use warnings;
 use Errno;
 use Cwd qw(abs_path);
-use lib "@amperldir@";
+use lib '@amperldir@';
 
 use Installcheck;
-use Installcheck::Run qw( run run_get run_err );
+use Installcheck::Run qw( run run_get run_out run_err );
 use Installcheck::Catalogs;
 use Amanda::Paths;
 use Amanda::Constants;
@@ -81,6 +82,20 @@ sub setup_config {
       : $params{bogus_mailer} ? "\"$mail_mock.bogus\""
       :                         '""';                    #default
 
+    if ($params{bogus_mailer}) {
+	unlink "$mail_mock.bogus";
+	if ($params{bogus_mailer} == 2) {
+	    open "MAIL", ">$mail_mock.bogus";
+	    close MAIL;
+	}
+	if ($params{bogus_mailer} == 3) {
+	    # write garbage
+	    open "MAIL", ">$mail_mock.bogus";
+	    print MAIL "sdfsdn398gl32";
+	    close MAIL;
+	    chmod 0755, "$mail_mock.bogus";
+	}
+    }
     $testconf->add_param('mailer', $mailer);
     $testconf->add_param('mailto',
         $params{want_mailto} ? '"nobody\@localhost"' : '""');
@@ -106,6 +121,13 @@ sub setup_config {
 	$testconf->add_param('tapecycle', $params{'tapecycle'});
     }
 
+    if ($params{vault1}) {
+	$testconf->add_storage("vtapes");
+	$testconf->add_storage("vault");
+	$testconf->add_param('storage', "\"vtapes\"");
+	$testconf->add_param('vault-storage', "\"vault\"");
+    }
+
     $testconf->write();
 
     undef $cat;
@@ -120,6 +142,7 @@ sub cleanup {
     unlink $mail_output;
     unlink $printer_output;
     unlink $alternate_log_filename;
+    unlink "$mail_mock.bogus";
 }
 
 # compare two multiline strings, giving a diff if they do not match
@@ -227,7 +250,7 @@ results_match($mail_output,
 
 results_match($printer_output,
     $cat->get_text('postscript'),
-    "..printer output matches");
+    "..printer output matches A");
 
 ok(run($amreport, 'TESTCONF', '--from-amdump', '/garbage/directory/'),
     "amreport, as run from amdump, with mailer, mailto, and a template, and  bogus option")
@@ -238,7 +261,7 @@ results_match($mail_output,
     "..mail matches");
 results_match($printer_output,
     $cat->get_text('postscript'),
-    "..printer output matches");
+    "..printer output matches B");
 
 cleanup();
 
@@ -251,7 +274,7 @@ results_match($mail_output,
     "..mail matches");
 results_match($printer_output,
     $cat->get_text('postscript'),
-    "..printer output matches");
+    "..printer output matches C");
 
 cleanup();
 
@@ -442,7 +465,11 @@ ok(! -f $printer_output,
 
 cleanup();
 
-like(run_err($amreport, 'TESTCONF', '--from-amdump', '-o', 'mailto=ill\egal'),
+like(run_out($amreport, 'TESTCONF', '--from-amdump', '-o', 'mailto=ill\egal'),
+    qr/mail addresses have invalid characters/,
+    "amreport with illegal email in -o, with mailer but no mailto and no template, errors out");
+
+like(run_err($amreport, 'TESTCONF', '--mail-text=ill\egal'),
     qr/mail addresses have invalid characters/,
     "amreport with illegal email in -o, with mailer but no mailto and no template, errors out");
 
@@ -519,14 +546,43 @@ setup_config(catalog => 'normal',
 	bogus_mailer => 1, want_mailto => 1, want_template => 1);
 
 ok(run($amreport, 'TESTCONF', '--from-amdump'),
-    "amreport with bogus mailer; doesn't mail, still prints")
+    "amreport with bogus mailer (1); doesn't mail, still prints")
   or diag($Installcheck::Run::stderr);
 ok(!-f $mail_output, "..produces no mail output");
 is($Installcheck::Run::stdout, "", "..produces no stdout output");
 $! = &Errno::ENOENT;
 my $enoent = $!;
 like($Installcheck::Run::stderr,
-     qr/^error: the mailer '.*' is not an executable program\.$/,
+     qr/^error: the mailer '.*': $enoent$/,
+     "..produces correct stderr output");
+
+setup_config(catalog => 'normal',
+	bogus_mailer => 2, want_mailto => 1, want_template => 1);
+
+ok(run($amreport, 'TESTCONF', '--from-amdump'),
+    "amreport with bogus mailer (2); doesn't mail, still prints")
+  or diag($Installcheck::Run::stderr);
+ok(!-f $mail_output, "..produces no mail output");
+is($Installcheck::Run::stdout, "", "..produces no stdout output");
+$! = &Errno::ENOENT;
+#my $enoent = $!;
+like($Installcheck::Run::stderr,
+     qr/^error: the mailer '.*' is not an executable program/,
+     "..produces correct stderr output");
+
+
+setup_config(catalog => 'normal',
+	bogus_mailer => 3, want_mailto => 1, want_template => 1);
+
+ok(run($amreport, 'TESTCONF', '--from-amdump'),
+    "amreport with bogus mailer (3); doesn't mail, still prints")
+  or diag($Installcheck::Run::stderr);
+ok(!-f $mail_output, "..produces no mail output");
+is($Installcheck::Run::stderr, "", "..produces no stderr output");
+$! = &Errno::ENOENT;
+#my $enoent = $!;
+like($Installcheck::Run::stdout,
+     qr/^.*: sdfsdn398gl32: (command |)not found/,
      "..produces correct stderr output");
 results_match(
     $printer_output,
@@ -667,8 +723,8 @@ results_match($printer_output,
 setup_config(catalog => 'retried', want_template => 1);
 
 run($amreport, 'TESTCONF', '-f', $out_filename);
-is($Installcheck::Run::exit_code, 4,
-    "amreport with retried logfile exit==4");
+is($Installcheck::Run::exit_code, 0,
+    "amreport with retried logfile exit==0");
 results_match($out_filename, $cat->get_text('report'),
     "..result matches 12");
 results_match($printer_output,
@@ -678,16 +734,16 @@ results_match($printer_output,
 setup_config(catalog => 'retried-strange');
 
 run($amreport, 'TESTCONF', '-f', $out_filename);
-is($Installcheck::Run::exit_code, 6,
-    "amreport with retried logfile, with strange exit==6");
+is($Installcheck::Run::exit_code, 2,
+    "amreport with retried logfile, with strange exit==2");
 results_match($out_filename, $cat->get_text('report'),
     "..result matches 13");
 
 setup_config(catalog => 'retried-nofinish', want_template => 1);
 
 run($amreport, 'TESTCONF', '-f', $out_filename);
-is($Installcheck::Run::exit_code, 4,
-    "amreport with retried logfile where driver did not finish exit==4");
+is($Installcheck::Run::exit_code, 0,
+    "amreport with retried logfile where driver did not finish exit==0");
 results_match($out_filename, $cat->get_text('report'),
     "..result matches 14");
 
@@ -710,6 +766,22 @@ is($Installcheck::Run::exit_code, 16,
 results_match($out_filename, $cat->get_text('report-noholding'),
     "..result matches 16");
 
+setup_config(catalog => 'tapers', want_template => 1);
+burp($alternate_log_filename, $cat->get_file('log/log'));
+run($amreport, 'TESTCONF', '-f', $out_filename, '-l', $alternate_log_filename, '-odisplayunit=g');
+is($Installcheck::Run::exit_code, 16,
+    "amreport with tapers logfile exit==16");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 16s");
+
+setup_config(catalog => 'tapers1', want_template => 1);
+burp($alternate_log_filename, $cat->get_file('log/log'));
+run($amreport, 'TESTCONF', '-f', $out_filename, '-l', $alternate_log_filename);
+is($Installcheck::Run::exit_code, 16,
+    "amreport with taperss logfile exit==16");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 16s1");
+
 setup_config(catalog => 'spanned', want_template => 1);
 
 run($amreport, 'TESTCONF', '-f', $out_filename);
@@ -782,4 +854,59 @@ is($Installcheck::Run::exit_code, 0,
 results_match($out_filename, $cat->get_text('report'),
     "..result matches 24");
 
+setup_config(catalog => 'dumper-tryagain', want_template => 0);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 4,
+    "amreport correctly report try-again (error)");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 25");
+
+setup_config(catalog => 'dumper-chunker-taper-success', want_template => 0);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 0,
+    "amreport correctly report dumper-chunker-taper-success");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 26");
+
+setup_config(catalog => 'dumper-taper-success', want_template => 0);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 0,
+    "amreport correctly report dumper-taper-success");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 27");
+
+setup_config(catalog => 'vault1', want_template => 0, vault1 => 1);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 4,
+    "amreport correctly report vault1");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 28");
+
+setup_config(catalog => 'vault2', want_template => 0, vault1 => 1);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 0,
+    "amreport correctly report vault2");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 29");
+
+setup_config(catalog => 'vault3', want_template => 0, vault1 => 1);
+
+config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
+run($amreport, 'TESTCONF', '-f', $out_filename);
+is($Installcheck::Run::exit_code, 0,
+    "amreport correctly report vault3");
+results_match($out_filename, $cat->get_text('report'),
+    "..result matches 30");
+exit;
+
 cleanup();
