@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,14 +15,14 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 41;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use File::Find;
 use Amanda::Config qw( :init :getconf config_dir_relative );
 use Amanda::Device qw( :constants );
@@ -65,7 +66,7 @@ my ($idx_count_pre, $idx_count_post);
 
 
 ## test config overrides
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 config_init($CONFIG_INIT_EXPLICIT_NAME, 'TESTCONF');
 
@@ -76,16 +77,16 @@ cmp_ok(
 
 cmp_ok(
     $Installcheck::Run::stdout, "=~",
-    qr/label 'TESTCONF01' not found in \/this\/is\/a\/fake\/tapelist/,
+    qr/label 'TESTCONF01' not found in tapelist file '\/this\/is\/a\/fake\/tapelist'/,
     "config overrides handled correctly"
 ) or proc_diag();
 
 ## test
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 config_init($CONFIG_INIT_EXPLICIT_NAME, 'TESTCONF');
-my $tapelist = Amanda::Tapelist->new(config_dir_relative("tapelist"));
+my ($tapelist, $message) = Amanda::Tapelist->new(config_dir_relative("tapelist"));
 ok($tapelist->lookup_tapelabel('TESTCONF01'), "looked up tape after dump");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
@@ -112,7 +113,7 @@ ok($dev->finish(),
 
 # test --cleanup
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 my $diskpath = Amanda::Util::sanitise_filename($Installcheck::Run::diskname);
 system ("touch -mt 201401020304.05 " . getconf($CNF_INDEXDIR) . "/localhost/$diskpath/*");
 
@@ -123,7 +124,7 @@ ok(run('amrmtape', '--cleanup', 'TESTCONF', 'TESTCONF01'),
      or proc_diag();
 
 $idx_count_post = dir_file_count($CNF_INDEXDIR);
-isnt($idx_count_post, $idx_count_pre, "number of index files before and after is different");
+isnt($idx_count_post, $idx_count_pre, "number of index files before ($idx_count_pre) and after ($idx_count_post) is different");
 
 $tapelist->reload();
 ok(!$tapelist->lookup_tapelabel('TESTCONF01'),
@@ -141,7 +142,7 @@ ok($dev->finish(),
 
 # test --erase
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
 
@@ -169,7 +170,7 @@ ok($dev->finish(),
 
 # test --keep-label
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
 
@@ -197,7 +198,7 @@ ok($dev->finish(),
 
 # test --keep-label --erase
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
 
@@ -223,7 +224,7 @@ is($dev->volume_label, 'TESTCONF01', "label is correct");
 
 # test --keep-label --erase
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
 
@@ -249,7 +250,9 @@ is($dev->volume_label, 'TESTCONF01', "label is correct");
 
 # test --dryrun --erase --cleanup
 
-Installcheck::Dumpcache::load("notimestamps");
+Installcheck::Dumpcache::load("basic");
+$diskpath = Amanda::Util::sanitise_filename($Installcheck::Run::diskname);
+system ("touch -mt 201401020304.05 " . getconf($CNF_INDEXDIR) . "/localhost/$diskpath/*");
 
 $idx_count_pre = dir_file_count($CNF_INDEXDIR);
 
