@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,21 +15,21 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 11;
 use strict;
 use warnings;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use File::Path;
 use Data::Dumper;
 use Installcheck;
 use Installcheck::Dumpcache;
 use Installcheck::Config;
 use Installcheck::Mock;
-use Installcheck::Run qw(run run_err run_get $diskname);
+use Installcheck::Run qw(run run_err run_out run_get $diskname);
 use Amanda::DB::Catalog;
 use Amanda::Paths;
 use Amanda::Config qw( :init );
@@ -36,6 +37,7 @@ use Amanda::Changer;
 use Amanda::Debug;
 
 Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
 
 my $vtape_root = "$Installcheck::TMP/tertiary";
 sub setup_chg_disk {
@@ -44,8 +46,29 @@ sub setup_chg_disk {
     return "chg-disk:$vtape_root";
 }
 
+sub add_storage {
+    my $tertiary_chg = setup_chg_disk();
+
+    my $amanda_conf_filename = "$CONFIG_DIR/TESTCONF/amanda.conf";
+    open(my $fh, ">>", $amanda_conf_filename);
+    print $fh <<EOF;
+define policy "amvault-policy" {
+    comment "amvault-policy-comment"
+    retention-tapes 15
+}
+define storage "amvault-test" {
+    comment "amvault-storage-comment"
+    tpchanger "$tertiary_chg"
+    policy "amvault-policy"
+    autolabel "TESTCONF%%" any
+}
+vault-storage "amvault-test"
+EOF
+
+}
 # set up a basic dump
 Installcheck::Dumpcache::load("basic");
+add_storage();
 
 config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
 my ($cfgerr_level, @cfgerr_errors) = config_errors();
@@ -58,31 +81,22 @@ if ($cfgerr_level >= $CFGERR_WARNINGS) {
 my $tertiary_chg = setup_chg_disk();
 
 # try a few failures first
-like(run_err("$sbindir/amvault",
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
+like(run_out("$sbindir/amvault",
 		'--src-timestamp', 'latest',
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF', 'someotherhost'),
     qr/No dumps to vault/,
     "amvault with a non-matching dumpspec dumps nothing")
-    or diag($Installcheck::Run::stderr);
+    or diag($Installcheck::Run::stdout);
 
-like(run_err("$sbindir/amvault",
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
+like(run_out("$sbindir/amvault",
 		'--src-timestamp', 'latest',
 		'--fulls-only',
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF', '*', '*', '*', '1-3'),
     qr/No dumps to vault/,
     "amvault with --fulls-only but specifying non-full dumpspecs dumps nothing")
     or diag($Installcheck::Run::stderr);
 
 like(run_err("$sbindir/amvault",
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF'),
     qr/specify something to select/,
     "amvault without any limiting factors is an error"),
@@ -90,13 +104,11 @@ like(run_err("$sbindir/amvault",
 
 # now a successful vaulting
 ok(run("$sbindir/amvault",
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
 		'--src-timestamp', 'latest',
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF'),
     "amvault runs!")
     or diag($Installcheck::Run::stderr);
+
 my @tert_files = glob("$vtape_root/slot1/0*");
 ok(@tert_files > 0,
     "..and files appear on the tertiary volume!");
@@ -124,6 +136,7 @@ Installcheck::Run::cleanup();
 
 # try the multi dump, to get a better idea of the filtering possibilities
 Installcheck::Dumpcache::load("multi");
+add_storage();
 config_init($CONFIG_INIT_EXPLICIT_NAME, "TESTCONF");
 ($cfgerr_level, @cfgerr_errors) = config_errors();
 if ($cfgerr_level >= $CFGERR_WARNINGS) {
@@ -141,20 +154,17 @@ sub get_dry_run {
     my @rv;
     for my $line (split /\n/, $stdout) {
 	next if ($line =~ /^Total Size:/);
-	my ($tape, $file, $host, $disk, $datestamp, $level) =
-	    ($line =~ /^(\S+) (\d*) (\S+) (.+) (\d+) (\d+)$/);
+	my ($tape, $file, $host, $disk, $datestamp, $level, $kb) =
+	    ($line =~ /^(\S+) (\d*) (\S+) (.+) (\d+) (\d+) (\d+)$/);
 	$tape = 'holding' if $file eq '';
-	push @rv, [$tape, $file, $host, $disk,   $level]; # note: no datestamp
+	push @rv, [$tape, $file, $host, $disk,   $level]; # note: no datestamp, no kb
     }
     return @rv;
 }
 
 is_deeply([ get_dry_run("$sbindir/amvault",
 		'--dry-run',
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
 		'--fulls-only',
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF') ], [
     [ "TESTCONF01", "1", "localhost", "$diskname/dir", "0" ],
     [ "TESTCONF01", "2", "localhost", "$diskname",     "0" ],
@@ -163,9 +173,6 @@ is_deeply([ get_dry_run("$sbindir/amvault",
 
 is_deeply([ get_dry_run("$sbindir/amvault",
 		'--dry-run',
-		'--autolabel=any',
-		'--label-template', "TESTCONF%%",
-		'--dst-changer', $tertiary_chg,
 		'TESTCONF', "localhost", "$diskname/dir") ], [
     [ "holding", "",     "localhost", "$diskname/dir",     "1" ],
     [ "TESTCONF01", "1", "localhost", "$diskname/dir",     "0" ],
@@ -206,15 +213,21 @@ define changer "tertiary" {
     device-property "indirect" "FALSE"
     changerfile "$chg_dir-changerfile"
 }
+define policy "tertiary-policy" {
+    retention-tapes 15
+}
+define storage "tertiary-storage" {
+    tpchanger "tertiary"
+    policy "tertiary-policy"
+    autolabel "TESTCONF%%" any
+}
 EOF
 
     $tertiary_chg = "tertiary";
     ok(run("$sbindir/amvault",
 		    '--export',
-		    '--autolabel=any',
-		    '--label-template', "TESTCONF%%",
 		    '--src-timestamp', 'latest',
-		    '--dst-changer', $tertiary_chg,
+		    '-ovault-storage=tertiary-storage',
 		    'TESTCONF'),
 	"amvault runs with an NDMP device as secondary and tertiary, with --export")
 	or diag($Installcheck::Run::stderr);
