@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,20 +15,24 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use Test::More tests => 2;
 
-use lib "@amperldir@";
+use lib '@amperldir@';
 use File::Path;
 use Installcheck;
 use Installcheck::Config;
 use Installcheck::Run;
+use Amanda::Debug;
 use Amanda::Paths;
 use Amanda::Util qw( slurp burp );
 use Amanda::Config qw( :init );
 
+Amanda::Debug::dbopen("installcheck");
+Installcheck::log_test_output();
+
 # set up a basic TESTCONF, and then burp the example configs over amanda.conf
 my $testconf = Installcheck::Config->new();
 $testconf->write();
