@@ -1,5 +1,6 @@
 #! @PERL@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 # This utility is useful for setting up a long-running NDMP tape service while
@@ -23,8 +24,8 @@
 # installchecks.  Note that you will need to run the =setupcache installcheck
 # first, to generate the cached NDMP dump.
 
-use lib "@top_srcdir@/installcheck";
-use lib "@amperldir@";
+use lib '@top_srcdir@/installcheck';
+use lib '@amperldir@';
 
 use Installcheck;
 use Installcheck::Mock;
