@@ -35,7 +35,7 @@
  */
 
 #include "ndmjob.h"
-#include "util.h"
+#include "amutil.h"
 
 #ifndef NDMOS_OPTION_NO_CONTROL_AGENT
 int
