@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -36,7 +37,7 @@
 #include "amrecover.h"
 #include "getfsent.h"
 #include "dgram.h"
-#include "util.h"
+#include "amutil.h"
 #include "conffile.h"
 
 extern int process_line(char *line);
@@ -106,7 +107,8 @@ get_line(void)
 	}
 	if((len = strlen(line)) > 0 && line[len-1] == '\r') {
 	    line[len-1] = '\0';
-	    server_line = newstralloc(server_line, line);
+	    g_free(server_line);
+	    server_line = g_strdup(line);
 	    amfree(line);
 	    return 0;
 	}
@@ -258,11 +260,11 @@ clean_pathname(
 	s[length-1]='\0';
 
     /* change "/." to "/" */
-    if(strcmp(s,"/.")==0)
+    if(g_str_equal(s, "/."))
 	s[1]='\0';
 
     /* remove "/." at end of path */
-    if(strcmp(&(s[length-2]),"/.")==0)
+    if(g_str_equal(&(s[length - 2]), "/."))
 	s[length-2]='\0';
 }
 
@@ -312,17 +314,20 @@ guess_disk (
 		  fsent.fsname ? fsent.fsname : _("(fsname null)"));
 	if ((current_length > longest_match)
 	    && (current_length <= cwd_length)
-	    && (strncmp(fsent.mntdir, cwd, current_length) == 0))
+	    && (g_str_has_prefix(cwd, fsent.mntdir)))
 	{
 	    longest_match = current_length;
-	    *mpt_guess = newstralloc(*mpt_guess, fsent.mntdir);
+	    g_free(*mpt_guess);
+	    *mpt_guess = g_strdup(fsent.mntdir);
 	    if(strncmp(fsent.fsname,DEV_PREFIX,(strlen(DEV_PREFIX))))
 	    {
-	        fsname = newstralloc(fsname, fsent.fsname);
+	        g_free(fsname);
+	        fsname = g_strdup(fsent.fsname);
             }
 	    else
 	    {
-	        fsname = newstralloc(fsname,fsent.fsname+strlen(DEV_PREFIX));
+	        g_free(fsname);
+	        fsname = g_strdup(fsent.fsname + strlen(DEV_PREFIX));
 	    }
 	    local_disk = is_local_fstype(&fsent);
 	    dbprintf(_("guess_disk: local_disk = %d, fsname = \"%s\"\n"),
@@ -348,24 +353,24 @@ guess_disk (
     /* disk name may be specified by mount point (logical name) or
        device name, have to determine */
     g_printf(_("Trying disk %s ...\n"), *mpt_guess);
-    disk_try = stralloc2("DISK ", *mpt_guess);		/* try logical name */
+    disk_try = g_strconcat("DISK ", *mpt_guess, NULL);		/* try logical name */
     if (exchange(disk_try) == -1)
 	exit(1);
     amfree(disk_try);
     if (server_happy())
     {
-	*dn_guess = stralloc(*mpt_guess);		/* logical is okay */
+	*dn_guess = g_strdup(*mpt_guess);		/* logical is okay */
 	amfree(fsname);
 	return 1;
     }
     g_printf(_("Trying disk %s ...\n"), fsname);
-    disk_try = stralloc2("DISK ", fsname);		/* try device name */
+    disk_try = g_strconcat("DISK ", fsname, NULL);		/* try device name */
     if (exchange(disk_try) == -1)
 	exit(1);
     amfree(disk_try);
     if (server_happy())
     {
-	*dn_guess = stralloc(fsname);			/* dev name is okay */
+	*dn_guess = g_strdup(fsname);			/* dev name is okay */
 	amfree(fsname);
 	return 1;
     }
@@ -409,6 +414,9 @@ main(
     char *service_name;
     char *line = NULL;
     struct tm *tm;
+    char *stream_msg = NULL;
+
+    glib_init();
 
     /*
      * Configure program for internationalization:
@@ -428,28 +436,29 @@ main(
 
     dbopen(DBG_SUBDIR_CLIENT);
 
-    localhost = alloc(MAX_HOSTNAME_LENGTH+1);
+    localhost = g_malloc(MAX_HOSTNAME_LENGTH+1);
     if (gethostname(localhost, MAX_HOSTNAME_LENGTH) != 0) {
 	error(_("cannot determine local host name\n"));
 	/*NOTREACHED*/
     }
     localhost[MAX_HOSTNAME_LENGTH] = '\0';
 
-    config = newstralloc(config, DEFAULT_CONFIG);
+    g_free(config);
+    config = g_strdup(DEFAULT_CONFIG);
 
     check_running_as(RUNNING_AS_ROOT);
 
     amfree(server_name);
     server_name = getenv("AMANDA_SERVER");
     if(!server_name) server_name = DEFAULT_SERVER;
-    server_name = stralloc(server_name);
+    server_name = g_strdup(server_name);
 
     amfree(tape_server_name);
     tape_server_name = getenv("AMANDA_TAPESERVER");
     if(!tape_server_name) tape_server_name = DEFAULT_TAPE_SERVER;
-    tape_server_name = stralloc(tape_server_name);
+    tape_server_name = g_strdup(tape_server_name);
 
-    config_init(CONFIG_INIT_CLIENT, NULL);
+    config_init(CONFIG_INIT_CLIENT|CONFIG_INIT_GLOBAL, NULL);
 
     if (config_errors(NULL) >= CFGERR_WARNINGS) {
 	config_print_errors();
@@ -467,7 +476,7 @@ main(
 	 */
 	char **new_argv;
 
-	new_argv = (char **) alloc((size_t)((argc + 1 + 1) * sizeof(*new_argv)));
+	new_argv = (char **) g_malloc((size_t)((argc + 1 + 1) * sizeof(*new_argv)));
 	new_argv[0] = argv[0];
 	new_argv[1] = "-C";
 	for (i = 1; i < argc; i++)
@@ -483,19 +492,23 @@ main(
 	switch (i)
 	{
 	    case 'C':
-		config = newstralloc(config, optarg);
+		g_free(config);
+		config = g_strdup(optarg);
 		break;
 
 	    case 's':
-		server_name = newstralloc(server_name, optarg);
+		g_free(server_name);
+		server_name = g_strdup(optarg);
 		break;
 
 	    case 't':
-		tape_server_name = newstralloc(tape_server_name, optarg);
+		g_free(tape_server_name);
+		tape_server_name = g_strdup(optarg);
 		break;
 
 	    case 'd':
-		tape_device_name = newstralloc(tape_device_name, optarg);
+		g_free(tape_device_name);
+		tape_device_name = g_strdup(optarg);
 		break;
 
 	    case 'U':
@@ -532,7 +545,7 @@ main(
 	/*NOTREACHED*/
     }
 
-    service_name = stralloc2("amandaidx", SERVICE_SUFFIX);
+    service_name = "amandaidx";
 
     g_printf(_("AMRECOVER Version %s. Contacting server on %s ...\n"),
 	   VERSION, server_name);
@@ -540,13 +553,17 @@ main(
 	error(_("%s/tcp unknown protocol"), service_name);
 	/*NOTREACHED*/
     }
-    amfree(service_name);
-    server_socket = stream_client_privileged(server_name,
+    server_socket = stream_client_privileged(NULL,
+					     server_name,
 					     (in_port_t)ntohs((in_port_t)sp->s_port),
 					     0,
 					     0,
 					     &my_port,
-					     0);
+					     0, &stream_msg);
+    if (stream_msg) {
+	error(_("cannot connect to %s: %s"), server_name, stream_msg);
+	/*NOTREACHED*/
+    }
     if (server_socket < 0) {
 	error(_("cannot connect to %s: %s"), server_name, strerror(errno));
 	/*NOTREACHED*/
@@ -559,24 +576,24 @@ main(
 
     /* get server's banner */
     if (grab_reply(1) == -1) {
-        aclose(server_socket);
+        aaclose(server_socket);
 	exit(1);
     }
     if (!server_happy())
     {
 	dbclose();
-	aclose(server_socket);
+	aaclose(server_socket);
 	exit(1);
     }
 
     /* do the security thing */
     line = get_security();
     if (converse(line) == -1) {
-        aclose(server_socket);
+        aaclose(server_socket);
 	exit(1);
     }
     if (!server_happy()) {
-        aclose(server_socket);
+        aaclose(server_socket);
 	exit(1);
     }
     memset(line, '\0', strlen(line));
@@ -589,9 +606,9 @@ main(
 
 	our_features = am_init_feature_set();
 	our_feature_string = am_feature_to_string(our_features);
-	line = stralloc2("FEATURES ", our_feature_string);
+	line = g_strconcat("FEATURES ", our_feature_string, NULL);
 	if(exchange(line) == 0) {
-	    their_feature_string = stralloc(server_line+13);
+	    their_feature_string = g_strdup(server_line+13);
 	    indexsrv_features = am_string_to_feature(their_feature_string);
 	}
 	else {
@@ -611,16 +628,16 @@ main(
 	error(_("BAD DATE"));
 
     g_printf(_("Setting restore date to today (%s)\n"), dump_date);
-    line = stralloc2("DATE ", dump_date);
+    line = g_strconcat("DATE ", dump_date, NULL);
     if (converse(line) == -1) {
-        aclose(server_socket);
+        aaclose(server_socket);
 	exit(1);
     }
     amfree(line);
 
-    line = stralloc2("SCNF ", config);
+    line = g_strconcat("SCNF ", config, NULL);
     if (converse(line) == -1) {
-        aclose(server_socket);
+        aaclose(server_socket);
 	exit(1);
     }
     amfree(line);
@@ -642,7 +659,7 @@ main(
 			   cwd, dn_guess, mpt_guess);
 		    set_disk(dn_guess, mpt_guess);
 		    set_directory(cwd);
-		    if (server_happy() && strcmp(cwd, mpt_guess) != 0)
+		    if (server_happy() && !g_str_equal(cwd, mpt_guess))
 		        g_printf(_("WARNING: not on root of selected filesystem, check man-page!\n"));
 		    amfree(dn_guess);
 		    amfree(mpt_guess);
@@ -685,7 +702,7 @@ main(
 
     dbclose();
 
-    aclose(server_socket);
+    aaclose(server_socket);
     return 0;
 }
 
@@ -698,5 +715,5 @@ get_security(void)
 	error(_("can't get login name for my uid %ld"), (long)getuid());
 	/*NOTREACHED*/
     }
-    return stralloc2("SECURITY USER ", pwptr->pw_name);
+    return g_strconcat("SECURITY USER ", pwptr->pw_name, NULL);
 }
