@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -41,7 +42,7 @@
 #ifdef SAMBA_CLIENT
 #include "findpass.h"
 #endif
-#include "util.h"
+#include "amutil.h"
 
 typedef struct EXTRACT_LIST_ITEM {
     char *path;
@@ -331,14 +332,14 @@ clean_tape_list(
 	fn2 = fn1->next;
 	while (fn2 != NULL && remove_fn1 == 0) {
 	    remove_fn2 = 0;
-	    if(strcmp(fn1->path, fn2->path) == 0) {
+	    if(g_str_equal(fn1->path, fn2->path)) {
 		remove_fn2 = 1;
-	    } else if (strncmp(fn1->path, fn2->path, strlen(fn1->path)) == 0 &&
+	    } else if (g_str_has_prefix(fn2->path, fn1->path) &&
 		       ((strlen(fn2->path) > strlen(fn1->path) &&
 			 fn2->path[strlen(fn1->path)] == '/') ||
 		       (fn1->path[strlen(fn1->path)-1] == '/'))) {
 		remove_fn2 = 1;
-	    } else if (strncmp(fn2->path, fn1->path, strlen(fn2->path)) == 0 &&
+	    } else if (g_str_has_prefix(fn1->path, fn2->path) &&
 		       ((strlen(fn1->path) > strlen(fn2->path) &&
 			 fn1->path[strlen(fn2->path)] == '/')  ||
 		       (fn2->path[strlen(fn2->path)-1] == '/'))) {
@@ -403,26 +404,27 @@ add_extract_item(
     EXTRACT_LIST_ITEM *that, *curr;
     char *ditem_path = NULL;
 
-    ditem_path = stralloc(ditem->path);
+    ditem_path = g_strdup(ditem->path);
     clean_pathname(ditem_path);
 
     for (this = extract_list; this != NULL; this = this->next)
     {
 	/* see if this is the list for the tape */	
-	if (this->level == ditem->level && strcmp(this->tape, ditem->tape) == 0)
+	if (this->level == ditem->level && g_str_equal(this->tape,
+                                                       ditem->tape))
 	{
 	    /* yes, so add to list */
 	    curr=this->files;
 	    while(curr!=NULL)
 	    {
-		if (strcmp(curr->path, ditem_path) == 0) {
+		if (g_str_equal(curr->path, ditem_path)) {
 		    amfree(ditem_path);
 		    return 1;
 		}
 		curr=curr->next;
 	    }
-	    that = (EXTRACT_LIST_ITEM *)alloc(sizeof(EXTRACT_LIST_ITEM));
-            that->path = stralloc(ditem_path);
+	    that = (EXTRACT_LIST_ITEM *)g_malloc(sizeof(EXTRACT_LIST_ITEM));
+            that->path = g_strdup(ditem_path);
 	    that->next = this->files;
 	    this->files = that;		/* add at front since easiest */
 	    amfree(ditem_path);
@@ -431,13 +433,13 @@ add_extract_item(
     }
 
     /* so this is the first time we have seen this tape */
-    this = (EXTRACT_LIST *)alloc(sizeof(EXTRACT_LIST));
-    this->tape = stralloc(ditem->tape);
+    this = (EXTRACT_LIST *)g_malloc(sizeof(EXTRACT_LIST));
+    this->tape = g_strdup(ditem->tape);
     this->level = ditem->level;
     this->fileno = ditem->fileno;
-    this->date = stralloc(ditem->date);
-    that = (EXTRACT_LIST_ITEM *)alloc(sizeof(EXTRACT_LIST_ITEM));
-    that->path = stralloc(ditem_path);
+    this->date = g_strdup(ditem->date);
+    that = (EXTRACT_LIST_ITEM *)g_malloc(sizeof(EXTRACT_LIST_ITEM));
+    that->path = g_strdup(ditem_path);
     that->next = NULL;
     this->files = that;
 
@@ -481,17 +483,18 @@ delete_extract_item(
     EXTRACT_LIST_ITEM *that, *prev;
     char *ditem_path = NULL;
 
-    ditem_path = stralloc(ditem->path);
+    ditem_path = g_strdup(ditem->path);
     clean_pathname(ditem_path);
 
     for (this = extract_list; this != NULL; this = this->next)
     {
 	/* see if this is the list for the tape */	
-	if (this->level == ditem->level && strcmp(this->tape, ditem->tape) == 0)
+	if (this->level == ditem->level && g_str_equal(this->tape,
+                                                       ditem->tape))
 	{
 	    /* yes, so find file on list */
 	    that = this->files;
-	    if (strcmp(that->path, ditem_path) == 0)
+	    if (g_str_equal(that->path, ditem_path))
 	    {
 		/* first on list */
 		this->files = that->next;
@@ -507,7 +510,7 @@ delete_extract_item(
 	    that = that->next;
 	    while (that != NULL)
 	    {
-		if (strcmp(that->path, ditem_path) == 0)
+		if (g_str_equal(that->path, ditem_path))
 		{
 		    prev->next = that->next;
                     amfree(that->path);
@@ -549,7 +552,7 @@ add_glob(
          * $, but we need to match an optional trailing /, so tack that on
          * the end.
          */
-        regex_path = stralloc(regex + 1);
+        regex_path = g_strdup(regex + 1);
         regex_path[strlen(regex_path) - 1] = '\0';
         strappend(regex_path, "[/]*$");
         add_file(uqglob, regex_path);
@@ -603,10 +606,10 @@ void add_file(
 
     dbprintf(_("add_file: Looking for \"%s\"\n"), regex);
 
-    if(strcmp(regex, "/[/]*$") == 0) {	/* "/" behave like "." */
+    if(g_str_equal(regex, "/[/]*$")) {	/* "/" behave like "." */
 	regex = "\\.[/]*$";
     }
-    else if(strcmp(regex, "[^/]*[/]*$") == 0) {		/* "*" */
+    else if(g_str_equal(regex, "[^/]*[/]*$")) {		/* "*" */
 	regex = "([^/.]|\\.[^/]+|[^/.][^/]*)[/]*$";
     } else {
 	/* remove "/" at end of path */
@@ -616,21 +619,21 @@ void add_file(
     }
 
     /* convert path (assumed in cwd) to one on disk */
-    if (strcmp(disk_path, "/") == 0) {
+    if (g_str_equal(disk_path, "/")) {
         if (*regex == '/') {
 	    /* No mods needed if already starts with '/' */
-	    path_on_disk = stralloc(regex);
+	    path_on_disk = g_strdup(regex);
 	} else {
 	    /* Prepend '/' */
-	    path_on_disk = stralloc2("/", regex);
+	    path_on_disk = g_strconcat("/", regex, NULL);
 	}
     } else {
 	char *clean_disk_path = clean_regex(disk_path, 0);
-	path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+	path_on_disk = g_strjoin(NULL, clean_disk_path, "/", regex, NULL);
 	amfree(clean_disk_path);
     }
 
-    path_on_disk_slash = stralloc2(path_on_disk, "/");
+    path_on_disk_slash = g_strconcat(path_on_disk, "/", NULL);
 
     dbprintf(_("add_file: Converted path=\"%s\" to path_on_disk=\"%s\"\n"),
 	      regex, path_on_disk);
@@ -651,10 +654,12 @@ void add_file(
 	    if((j > 0 && ditem->path[j-1] == '/')
 	       || (j > 1 && ditem->path[j-2] == '/' && ditem->path[j-1] == '.'))
 	    {	/* It is a directory */
-		ditem_path = newstralloc(ditem_path, ditem->path);
+		g_free(ditem_path);
+		ditem_path = g_strdup(ditem->path);
 		clean_pathname(ditem_path);
 
-		cmd = newstralloc2(cmd, "ORLD ", ditem_path);
+		g_free(cmd);
+		cmd = g_strconcat("ORLD ", ditem_path, NULL);
 		if(send_command(cmd) == -1) {
 		    amfree(cmd);
 		    amfree(ditem_path);
@@ -675,13 +680,17 @@ void add_file(
 		    amfree(ditem_path);
 		    amfree(path_on_disk);
 		    amfree(path_on_disk_slash);
+		    amfree(lditem.path);
+		    amfree(lditem.date);
+		    amfree(lditem.tape);
 		    l = reply_line();
 		    g_printf(_("%s\n"), l);
 		    return;
 		}
 		dir_undo = NULL;
 		added=0;
-                lditem.path = newstralloc(lditem.path, ditem->path);
+                g_free(lditem.path);
+                lditem.path = g_strdup(ditem->path);
 		/* skip the last line -- duplicate of the preamble */
 
 		while ((i = get_reply_line()) != 0) {
@@ -695,7 +704,7 @@ void add_file(
 			if(cmd == NULL) {
 			    if(dir_undo) *dir_undo = dir_undo_ch;
 			    dir_undo = NULL;
-			    cmd = stralloc(l);	/* save for error report */
+			    cmd = g_strdup(l);	/* save for error report */
 			}
 			continue;	/* throw the rest of the lines away */
 		    }
@@ -720,7 +729,8 @@ void add_file(
                     fp = s-1;
                     skip_non_whitespace(s, ch);
                     s[-1] = '\0';
-                    lditem.date = newstralloc(lditem.date, fp);
+                    g_free(lditem.date);
+                    lditem.date = g_strdup(fp);
                     s[-1] = (char)ch;
 
 		    skip_whitespace(s, ch);
@@ -738,7 +748,8 @@ void add_file(
                     fp = s-1;
                     skip_non_whitespace(s, ch);
                     s[-1] = '\0';
-                    lditem.tape = newstralloc(lditem.tape, fp);
+                    g_free(lditem.tape);
+                    lditem.tape = g_strdup(fp);
                     s[-1] = (char)ch;
 
 		    if(am_has_feature(indexsrv_features, fe_amindexd_fileno_in_ORLD)) {
@@ -827,6 +838,9 @@ void add_file(
     amfree(ditem_path);
     amfree(path_on_disk);
     amfree(path_on_disk_slash);
+    amfree(lditem.path);
+    amfree(lditem.date);
+    amfree(lditem.tape);
 
     if(! found_one) {
 	quoted = quote_string(path);
@@ -860,7 +874,7 @@ delete_glob(
          * $, but we need to match an optional trailing /, so tack that on
          * the end.
          */
-        regex_path = stralloc(regex + 1);
+        regex_path = g_strdup(regex + 1);
         regex_path[strlen(regex_path) - 1] = '\0';
         strappend(regex_path, "[/]*$");
         delete_file(uqglob, regex_path);
@@ -918,7 +932,7 @@ delete_file(
 
     dbprintf(_("delete_file: Looking for \"%s\"\n"), path);
 
-    if (strcmp(regex, "[^/]*[/]*$") == 0) {
+    if (g_str_equal(regex, "[^/]*[/]*$")) {
 	/* Looking for * find everything but single . */
 	regex = "([^/.]|\\.[^/]+|[^/.][^/]*)[/]*$";
     } else {
@@ -928,26 +942,26 @@ delete_file(
     }
 
     /* convert path (assumed in cwd) to one on disk */
-    if (strcmp(disk_path, "/") == 0) {
+    if (g_str_equal(disk_path, "/")) {
         if (*regex == '/') {
-	    if (strcmp(regex, "/[/]*$") == 0) {
+	    if (g_str_equal(regex, "/[/]*$")) {
 		/* We want "/" to match the directory itself: "/." */
-		path_on_disk = stralloc("/\\.[/]*$");
+		path_on_disk = g_strdup("/\\.[/]*$");
 	    } else {
 		/* No mods needed if already starts with '/' */
-		path_on_disk = stralloc(regex);
+		path_on_disk = g_strdup(regex);
 	    }
 	} else {
 	    /* Prepend '/' */
-	    path_on_disk = stralloc2("/", regex);
+	    path_on_disk = g_strconcat("/", regex, NULL);
 	}
     } else {
 	char *clean_disk_path = clean_regex(disk_path, 0);
-	path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+	path_on_disk = g_strjoin(NULL, clean_disk_path, "/", regex, NULL);
 	amfree(clean_disk_path);
     }
 
-    path_on_disk_slash = stralloc2(path_on_disk, "/");
+    path_on_disk_slash = g_strconcat(path_on_disk, "/", NULL);
 
     dbprintf(_("delete_file: Converted path=\"%s\" to path_on_disk=\"%s\"\n"),
 	      regex, path_on_disk);
@@ -965,10 +979,12 @@ delete_file(
 	    if((j > 0 && ditem->path[j-1] == '/')
 	       || (j > 1 && ditem->path[j-2] == '/' && ditem->path[j-1] == '.'))
 	    {	/* It is a directory */
-		ditem_path = newstralloc(ditem_path, ditem->path);
+		g_free(ditem_path);
+		ditem_path = g_strdup(ditem->path);
 		clean_pathname(ditem_path);
 
-		cmd = newstralloc2(cmd, "ORLD ", ditem_path);
+		g_free(cmd);
+		cmd = g_strconcat("ORLD ", ditem_path, NULL);
 		if(send_command(cmd) == -1) {
 		    amfree(cmd);
 		    amfree(ditem_path);
@@ -989,12 +1005,16 @@ delete_file(
 		    amfree(ditem_path);
 		    amfree(path_on_disk);
 		    amfree(path_on_disk_slash);
+		    amfree(lditem.path);
+		    amfree(lditem.date);
+		    amfree(lditem.tape);
 		    l = reply_line();
 		    g_printf("%s\n", l);
 		    return;
 		}
 		deleted=0;
-                lditem.path = newstralloc(lditem.path, ditem->path);
+                g_free(lditem.path);
+                lditem.path = g_strdup(ditem->path);
 		amfree(cmd);
 		tape_undo = dir_undo = NULL;
 		/* skip the last line -- duplicate of the preamble */
@@ -1011,7 +1031,7 @@ delete_file(
 			    if(tape_undo) *tape_undo = tape_undo_ch;
 			    if(dir_undo) *dir_undo = dir_undo_ch;
 			    tape_undo = dir_undo = NULL;
-			    cmd = stralloc(l);	/* save for the error report */
+			    cmd = g_strdup(l);	/* save for the error report */
 			}
 			continue;	/* throw the rest of the lines away */
 		    }
@@ -1076,9 +1096,11 @@ delete_file(
 		    dir_undo_ch = *dir_undo;
 		    *dir_undo = '\0';
 
-                    lditem.date = newstralloc(lditem.date, date);
+                    g_free(lditem.date);
+                    lditem.date = g_strdup(date);
 		    lditem.level=level;
-                    lditem.tape = newstralloc(lditem.tape, tape);
+                    g_free(lditem.tape);
+                    lditem.tape = g_strdup(tape);
 		    switch(delete_extract_item(&lditem)) {
 		    case -1:
 			g_printf(_("System error\n"));
@@ -1134,6 +1156,9 @@ delete_file(
     amfree(ditem_path);
     amfree(path_on_disk);
     amfree(path_on_disk_slash);
+    amfree(lditem.path);
+    amfree(lditem.date);
+    amfree(lditem.tape);
 
     if(! found_one) {
 	g_printf(_("File %s doesn't exist in directory\n"), path);
@@ -1165,7 +1190,7 @@ display_extract_list(
 	 * Set up the pager command so if the pager is terminated, we do
 	 * not get a SIGPIPE back.
 	 */
-	pager_command = stralloc2(pager, " ; /bin/cat > /dev/null");
+	pager_command = g_strconcat(pager, " ; /bin/cat > /dev/null", NULL);
 	if ((fp = popen(pager_command, "w")) == NULL)
 	{
 	    g_printf(_("Warning - can't pipe through %s\n"), pager);
@@ -1297,7 +1322,7 @@ send_to_tape_server(
     int		tss,
     char *	cmd)
 {
-    char *msg = stralloc2(cmd, "\r\n");
+    char *msg = g_strconcat(cmd, "\r\n", NULL);
 
     if (full_write(tss, msg, strlen(msg)) < strlen(msg))
     {
@@ -1325,27 +1350,31 @@ extract_files_setup(
     char *clean_datestamp, *ch, *ch1;
     char *our_feature_string = NULL;
     char *tt = NULL;
+    char *stream_msg = NULL;
 
-    service_name = stralloc2("amidxtape", SERVICE_SUFFIX);
+    service_name = "amidxtape";
 
     /* get tape server details */
     if ((sp = getservbyname(service_name, "tcp")) == NULL)
     {
 	g_printf(_("%s/tcp unknown protocol - config error?\n"), service_name);
-	amfree(service_name);
 	return -1;
     }
-    amfree(service_name);
     if (seteuid(0) != 0) { error("Can't set euid"); }; /* it either works ... */
     if (setegid(0) != 0) { error("Can't set egid"); };
-    tape_control_sock = stream_client_privileged(tape_server_name,
+    tape_control_sock = stream_client_privileged(NULL,
+						  tape_server_name,
 						  (in_port_t)ntohs((in_port_t)sp->s_port),
 						  0,
 						  STREAM_BUFSIZE,
 						  &my_port,
-						  0);
-    if (tape_control_sock < 0)
-    {
+						  0, &stream_msg);
+    if (stream_msg) {
+	g_printf(_("cannot connect to %s: %s\n"), tape_server_name, stream_msg);
+	g_free(stream_msg);
+	return -1;
+    }
+    if (tape_control_sock < 0) {
 	g_printf(_("cannot connect to %s: %s\n"), tape_server_name, strerror(errno));
 	return -1;
     }
@@ -1364,7 +1393,7 @@ extract_files_setup(
     memset(line, '\0', strlen(line));
     amfree(line);
 
-    disk_regex = alloc(strlen(disk_name) * 2 + 3);
+    disk_regex = g_malloc(strlen(disk_name) * 2 + 3);
 
     ch = disk_name;
     ch1 = disk_regex;
@@ -1388,7 +1417,7 @@ extract_files_setup(
 
     *ch1 = '\0';
 
-    host_regex = alloc(strlen(dump_hostname) * 2 + 3);
+    host_regex = g_malloc(strlen(dump_hostname) * 2 + 3);
 
     ch = dump_hostname;
     ch1 = host_regex;
@@ -1412,7 +1441,7 @@ extract_files_setup(
 
     *ch1 = '\0';
 
-    clean_datestamp = stralloc(dump_datestamp);
+    clean_datestamp = g_strdup(dump_datestamp);
     for(ch=ch1=clean_datestamp;*ch1 != '\0';ch1++) {
 	if(*ch1 != '-') {
 	    *ch = *ch1;
@@ -1427,7 +1456,8 @@ extract_files_setup(
 	char buffer[32768] = "\0";
 
 	our_feature_string = am_feature_to_string(our_features);
-	tt = newstralloc2(tt, "FEATURES=", our_feature_string);
+	g_free(tt);
+	tt = g_strconcat("FEATURES=", our_feature_string, NULL);
 	send_to_tape_server(tape_control_sock, tt);
 	if (read(tape_control_sock, buffer, sizeof(buffer)) <= 0) {
 	    error(_("Could not read features from control socket\n"));
@@ -1445,28 +1475,35 @@ extract_files_setup(
        am_has_feature(indexsrv_features, fe_amidxtaped_datestamp)) {
 
 	if(am_has_feature(indexsrv_features, fe_amidxtaped_config)) {
-	    tt = newstralloc2(tt, "CONFIG=", config);
+	    g_free(tt);
+	    tt = g_strconcat("CONFIG=", config, NULL);
 	    send_to_tape_server(tape_control_sock, tt);
 	}
 	if(am_has_feature(indexsrv_features, fe_amidxtaped_label) &&
 	   label && label[0] != '/') {
-	    tt = newstralloc2(tt,"LABEL=",label);
+	    g_free(tt);
+	    tt = g_strconcat("LABEL=", label, NULL);
 	    send_to_tape_server(tape_control_sock, tt);
 	}
 	if(am_has_feature(indexsrv_features, fe_amidxtaped_fsf)) {
 	    char v_fsf[100];
 	    g_snprintf(v_fsf, 99, "%lld", (long long)fsf);
-	    tt = newstralloc2(tt, "FSF=",v_fsf);
+	    g_free(tt);
+	    tt = g_strconcat("FSF=", v_fsf, NULL);
 	    send_to_tape_server(tape_control_sock, tt);
 	}
 	send_to_tape_server(tape_control_sock, "HEADER");
-	tt = newstralloc2(tt, "DEVICE=", dump_device_name);
+	g_free(tt);
+	tt = g_strconcat("DEVICE=", dump_device_name, NULL);
 	send_to_tape_server(tape_control_sock, tt);
-	tt = newstralloc2(tt, "HOST=", host_regex);
+	g_free(tt);
+	tt = g_strconcat("HOST=", host_regex, NULL);
 	send_to_tape_server(tape_control_sock, tt);
-	tt = newstralloc2(tt, "DISK=", disk_regex);
+	g_free(tt);
+	tt = g_strconcat("DISK=", disk_regex, NULL);
 	send_to_tape_server(tape_control_sock, tt);
-	tt = newstralloc2(tt, "DATESTAMP=", clean_datestamp);
+	g_free(tt);
+	tt = g_strconcat("DATESTAMP=", clean_datestamp, NULL);
 	send_to_tape_server(tape_control_sock, tt);
 	send_to_tape_server(tape_control_sock, "END");
 	amfree(tt);
@@ -1500,11 +1537,12 @@ extract_files_setup(
      */
 
     if(am_has_feature(tapesrv_features, fe_recover_splits)){
-	char buffer[32768];
+	char buffer[32768+1];
 	in_port_t data_port = (in_port_t)-1;
         ssize_t nread;
+	char *stream_msg = NULL;
 
-        nread = read(tape_control_sock, buffer, sizeof(buffer));
+        nread = read(tape_control_sock, buffer, sizeof(buffer)-1);
 
 	if (nread <= 0) {
 	    error(_("Could not read from control socket: %s\n"),
@@ -1520,14 +1558,19 @@ extract_files_setup(
 	    /*NOTREACHED*/
         }
 
-	tape_data_sock = stream_client_privileged(server_name,
+	tape_data_sock = stream_client_privileged(NULL,
+						  server_name,
 						  data_port,
 						  0,
 						  STREAM_BUFSIZE,
 						  &my_data_port,
-						  0);
-	if(tape_data_sock == -1){
-	    error(_("Unable to make data connection to server: %s\n"),
+						  0, &stream_msg);
+	if (stream_msg) {
+	    error("Unable to make data connection to server: %s\n", stream_msg);
+	    /*NOTREACHED*/
+	}
+	if (tape_data_sock == -1) {
+	    error("Unable to make data connection to server: %s\n",
 		      strerror(errno));
 	    /*NOTREACHED*/
 	}
@@ -1634,21 +1677,21 @@ extract_files_child(
 	/*NOTREACHED*/
     }
 
-    if (file.program != NULL) {
+    if (file.program[0] != '\0') {
 #ifdef GNUTAR
-	if (strcmp(file.program, GNUTAR) == 0)
+	if (g_str_equal(file.program, GNUTAR))
 	    dumptype = IS_GNUTAR;
 #endif
 
 	if (dumptype == IS_UNKNOWN) {
 	    len_program = strlen(file.program);
 	    if(len_program >= 3 &&
-	       strcmp(&file.program[len_program-3],"tar") == 0)
+	       g_str_equal(&file.program[len_program - 3], "tar"))
 		dumptype = IS_TAR;
 	}
 
 #ifdef SAMBA_CLIENT
-	if (dumptype == IS_UNKNOWN && strcmp(file.program, SAMBA_CLIENT) ==0) {
+	if (dumptype == IS_UNKNOWN && g_str_equal(file.program, SAMBA_CLIENT)) {
 	    if (samba_extract_method == SAMBA_TAR)
 	      dumptype = IS_SAMBA_TAR;
 	    else
@@ -1662,57 +1705,57 @@ extract_files_child(
     switch(dumptype) {
 #ifdef SAMBA_CLIENT
     case IS_SAMBA:
-	g_ptr_array_add(argv_ptr, stralloc("smbclient"));
+	g_ptr_array_add(argv_ptr, g_strdup("smbclient"));
 	smbpass = findpass(file.disk, &domain);
 	if (smbpass) {
-	    g_ptr_array_add(argv_ptr, stralloc(file.disk));
-	    g_ptr_array_add(argv_ptr, stralloc("-U"));
+	    g_ptr_array_add(argv_ptr, g_strdup(file.disk));
+	    g_ptr_array_add(argv_ptr, g_strdup("-U"));
 	    passwd_field = argv_ptr->len;
-	    g_ptr_array_add(argv_ptr, stralloc(smbpass));
+	    g_ptr_array_add(argv_ptr, g_strdup(smbpass));
 	    if (domain) {
-		g_ptr_array_add(argv_ptr, stralloc("-W"));
-		g_ptr_array_add(argv_ptr, stralloc(domain));
+		g_ptr_array_add(argv_ptr, g_strdup("-W"));
+		g_ptr_array_add(argv_ptr, g_strdup(domain));
 	    }
 	}
-	g_ptr_array_add(argv_ptr, stralloc("-d0"));
-	g_ptr_array_add(argv_ptr, stralloc("-Tx"));
-	g_ptr_array_add(argv_ptr, stralloc("-"));	/* data on stdin */
+	g_ptr_array_add(argv_ptr, g_strdup("-d0"));
+	g_ptr_array_add(argv_ptr, g_strdup("-Tx"));
+	g_ptr_array_add(argv_ptr, g_strdup("-"));	/* data on stdin */
 	break;
 #endif
     case IS_TAR:
     case IS_GNUTAR:
-	g_ptr_array_add(argv_ptr, stralloc("tar"));
-	g_ptr_array_add(argv_ptr, stralloc("--numeric-owner"));
-	g_ptr_array_add(argv_ptr, stralloc("-xpGvf"));
-	g_ptr_array_add(argv_ptr, stralloc("-"));	/* data on stdin */
+	g_ptr_array_add(argv_ptr, g_strdup("tar"));
+	g_ptr_array_add(argv_ptr, g_strdup("--numeric-owner"));
+	g_ptr_array_add(argv_ptr, g_strdup("-xpGvf"));
+	g_ptr_array_add(argv_ptr, g_strdup("-"));	/* data on stdin */
 	break;
     case IS_SAMBA_TAR:
-	g_ptr_array_add(argv_ptr, stralloc("tar"));
-	g_ptr_array_add(argv_ptr, stralloc("-xpvf"));
-	g_ptr_array_add(argv_ptr, stralloc("-"));	/* data on stdin */
+	g_ptr_array_add(argv_ptr, g_strdup("tar"));
+	g_ptr_array_add(argv_ptr, g_strdup("-xpvf"));
+	g_ptr_array_add(argv_ptr, g_strdup("-"));	/* data on stdin */
 	break;
     case IS_UNKNOWN:
     case IS_DUMP:
-	g_ptr_array_add(argv_ptr, stralloc("restore"));
+	g_ptr_array_add(argv_ptr, g_strdup("restore"));
 #ifdef AIX_BACKUP
-	g_ptr_array_add(argv_ptr, stralloc("-xB"));
+	g_ptr_array_add(argv_ptr, g_strdup("-xB"));
 #else
 #if defined(XFSDUMP)
-	if (strcmp(file.program, XFSDUMP) == 0) {
-	    g_ptr_array_add(argv_ptr, stralloc("-v"));
-	    g_ptr_array_add(argv_ptr, stralloc("silent"));
+	if (g_str_equal(file.program, XFSDUMP)) {
+	    g_ptr_array_add(argv_ptr, g_strdup("-v"));
+	    g_ptr_array_add(argv_ptr, g_strdup("silent"));
 	} else
 #endif
 #if defined(VDUMP)
-	if (strcmp(file.program, VDUMP) == 0) {
-	    g_ptr_array_add(argv_ptr, stralloc("xf"));
-	    g_ptr_array_add(argv_ptr, stralloc("-"));	/* data on stdin */
+	if (g_str_equal(file.program, VDUMP)) {
+	    g_ptr_array_add(argv_ptr, g_strdup("xf"));
+	    g_ptr_array_add(argv_ptr, g_strdup("-"));	/* data on stdin */
 	} else
 #endif
 	{
-	g_ptr_array_add(argv_ptr, stralloc("xbf"));
-	g_ptr_array_add(argv_ptr, stralloc("2")); /* read in units of 1K */
-	g_ptr_array_add(argv_ptr, stralloc("-")); /* data on stdin */
+	g_ptr_array_add(argv_ptr, g_strdup("xbf"));
+	g_ptr_array_add(argv_ptr, g_strdup("2")); /* read in units of 1K */
+	g_ptr_array_add(argv_ptr, g_strdup("-")); /* data on stdin */
 	}
 #endif
     }
@@ -1726,32 +1769,32 @@ extract_files_child(
 #ifdef SAMBA_CLIENT
 	case IS_SAMBA:
 #endif
-	    if (strcmp(fn->path, "/") == 0)
-		g_ptr_array_add(argv_ptr, stralloc("."));
+	    if (g_str_equal(fn->path, "/"))
+		g_ptr_array_add(argv_ptr, g_strdup("."));
 	    else
-		g_ptr_array_add(argv_ptr, stralloc2(".", fn->path));
+		g_ptr_array_add(argv_ptr, g_strconcat(".", fn->path, NULL));
 	    break;
 	case IS_UNKNOWN:
 	case IS_DUMP:
 #if defined(XFSDUMP)
-	    if (strcmp(file.program, XFSDUMP) == 0) {
+	    if (g_str_equal(file.program, XFSDUMP)) {
 		/*
 		 * xfsrestore needs a -s option before each file to be
 		 * restored, and also wants them to be relative paths.
 		 */
-		g_ptr_array_add(argv_ptr, stralloc("-s"));
-		g_ptr_array_add(argv_ptr, stralloc(fn->path + 1));
+		g_ptr_array_add(argv_ptr, g_strdup("-s"));
+		g_ptr_array_add(argv_ptr, g_strdup(fn->path + 1));
 	    } else
 #endif
 	    {
-		g_ptr_array_add(argv_ptr, stralloc(fn->path));
+		g_ptr_array_add(argv_ptr, g_strdup(fn->path));
 	    }
 	}
     }
 #if defined(XFSDUMP)
-    if (strcmp(file.program, XFSDUMP) == 0) {
-	g_ptr_array_add(argv_ptr, stralloc("-"));
-	g_ptr_array_add(argv_ptr, stralloc("."));
+    if (g_str_equal(file.program, XFSDUMP)) {
+	g_ptr_array_add(argv_ptr, g_strdup("-"));
+	g_ptr_array_add(argv_ptr, g_strdup("."));
     }
 #endif
     g_ptr_array_add(argv_ptr, NULL);
@@ -1759,7 +1802,7 @@ extract_files_child(
     switch (dumptype) {
 #ifdef SAMBA_CLIENT
     case IS_SAMBA:
-	cmd = stralloc(SAMBA_CLIENT);
+	cmd = g_strdup(SAMBA_CLIENT);
 	break;
 #else
 	/* fall through to ... */
@@ -1769,38 +1812,38 @@ extract_files_child(
     case IS_SAMBA_TAR:
 #ifndef GNUTAR
 	g_fprintf(stderr, _("warning: GNUTAR program not available.\n"));
-	cmd = stralloc("tar");
+	cmd = g_strdup("tar");
 #else
-  	cmd = stralloc(GNUTAR);
+  	cmd = g_strdup(GNUTAR);
 #endif
 	break;
     case IS_UNKNOWN:
     case IS_DUMP:
 	cmd = NULL;
 #if defined(DUMP)
-	if (strcmp(file.program, DUMP) == 0) {
-	    cmd = stralloc(RESTORE);
+	if (g_str_equal(file.program, DUMP)) {
+	    cmd = g_strdup(RESTORE);
 	}
 #endif
 #if defined(VDUMP)
-	if (strcmp(file.program, VDUMP) == 0) {
-	    cmd = stralloc(VRESTORE);
+	if (g_str_equal(file.program, VDUMP)) {
+	    cmd = g_strdup(VRESTORE);
 	}
 #endif
 #if defined(VXDUMP)
-	if (strcmp(file.program, VXDUMP) == 0) {
-	    cmd = stralloc(VXRESTORE);
+	if (g_str_equal(file.program, VXDUMP)) {
+	    cmd = g_strdup(VXRESTORE);
 	}
 #endif
 #if defined(XFSDUMP)
-	if (strcmp(file.program, XFSDUMP) == 0) {
-	    cmd = stralloc(XFSRESTORE);
+	if (g_str_equal(file.program, XFSDUMP)) {
+	    cmd = g_strdup(XFSRESTORE);
 	}
 #endif
 	if (cmd == NULL) {
 	    g_fprintf(stderr, _("warning: restore program for %s not available.\n"),
 		    file.program);
-	    cmd = stralloc("restore");
+	    cmd = g_strdup("restore");
 	}
     }
     if (cmd) {
@@ -1934,12 +1977,14 @@ writer_intermediary(
                         fflush(stdout);
 
                         input = agets(stdin); /* strips \n */
-                        if (strcasecmp("", input) == 0||
-                            strcasecmp("y", input) == 0||
-                            strcasecmp("yes", input) == 0) {
+                        if (input &&
+			    (strcasecmp("", input) == 0 ||
+                             strcasecmp("y", input) == 0 ||
+                             strcasecmp("yes", input) == 0)) {
                             send_to_tape_server(tape_control_sock, "OK");
                             done = 1;
-                        } else if (strcasecmp("n", input) == 0||
+                        } else if (!input ||
+				   strcasecmp("n", input) == 0 ||
                                    strcasecmp("no", input) == 0) {
                             send_to_tape_server(tape_control_sock, "ERROR");
                             /* Abort!
@@ -2048,7 +2093,8 @@ extract_files(void)
 
     /* get tape device name from index server if none specified */
     if (tape_server_name == NULL) {
-	tape_server_name = newstralloc(tape_server_name, server_name);
+	g_free(tape_server_name);
+	tape_server_name = g_strdup(server_name);
     }
     if (tape_device_name == NULL) {
 	if (send_command("TAPE") == -1)
@@ -2062,10 +2108,11 @@ extract_files(void)
 	    exit(1);
 	}
 	/* skip reply number */
-	tape_device_name = newstralloc(tape_device_name, l+4);
+	g_free(tape_device_name);
+	tape_device_name = g_strdup(l + 4);
     }
 
-    if (strcmp(tape_device_name, "/dev/null") == 0)
+    if (g_str_equal(tape_device_name, "/dev/null"))
     {
 	g_printf(_("amrecover: warning: using %s as the tape device will not work\n"),
 	       tape_device_name);
@@ -2082,7 +2129,9 @@ extract_files(void)
 	    }
 	    else
 		g_printf("                               ");
-	    tlist = unmarshal_tapelist_str(elist->tape);
+	    tlist = unmarshal_tapelist_str(elist->tape,
+			am_has_feature(indexsrv_features,
+				       fe_amrecover_storage_in_marshall));
 	    for( ; tlist != NULL; tlist = tlist->next)
 		g_printf(" %s", tlist->label);
 	    g_printf("\n");
@@ -2100,7 +2149,9 @@ extract_files(void)
 	    }
 	    else
 		g_printf("                               ");
-	    tlist = unmarshal_tapelist_str(elist->tape);
+	    tlist = unmarshal_tapelist_str(elist->tape,
+			am_has_feature(indexsrv_features,
+				       fe_amrecover_storage_in_marshall));
 	    for( ; tlist != NULL; tlist = tlist->next)
 		g_printf(" %s", tlist->label);
 	    g_printf("\n");
@@ -2126,9 +2177,12 @@ extract_files(void)
     while ((elist = first_tape_list()) != NULL)
     {
 	if(elist->tape[0]=='/') {
-	    dump_device_name = newstralloc(dump_device_name, elist->tape);
+	    g_free(dump_device_name);
+	    dump_device_name = g_strdup(elist->tape);
 	    g_printf(_("Extracting from file "));
-	    tlist = unmarshal_tapelist_str(dump_device_name);
+	    tlist = unmarshal_tapelist_str(dump_device_name,
+			am_has_feature(indexsrv_features,
+				       fe_amrecover_storage_in_marshall));
 	    for( ; tlist != NULL; tlist = tlist->next)
 		g_printf(" %s", tlist->label);
 	    g_printf("\n");
@@ -2137,8 +2191,11 @@ extract_files(void)
 	else {
 	    g_printf(_("Extracting files using tape drive %s on host %s.\n"),
 		   tape_device_name, tape_server_name);
-	    tlist = unmarshal_tapelist_str(elist->tape);
+	    tlist = unmarshal_tapelist_str(elist->tape,
+			am_has_feature(indexsrv_features,
+				       fe_amrecover_storage_in_marshall));
 	    g_printf(_("Load tape %s now\n"), tlist->label);
+	    amfree(tlist->label);
 	    amfree(tlist);
 	    otc = okay_to_continue(1,1,0);
 	    if (otc == 0)
@@ -2147,9 +2204,11 @@ extract_files(void)
 		delete_tape_list(elist); /* skip this tape */
 		continue;
 	    }
-	    dump_device_name = newstralloc(dump_device_name, tape_device_name);
+	    g_free(dump_device_name);
+	    dump_device_name = g_strdup(tape_device_name);
 	}
-	dump_datestamp = newstralloc(dump_datestamp, elist->date);
+	g_free(dump_datestamp);
+	dump_datestamp = g_strdup(elist->date);
 
 	/* connect to the tape handler daemon on the tape drive server */
 	if ((tape_control_sock = extract_files_setup(elist->tape, elist->fileno)) == -1)
