@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -32,7 +33,7 @@
 
 #include "amanda.h"
 #include "match.h"
-#include "util.h"
+#include "amutil.h"
 #include "amrecover.h"
 
 #ifdef SAMBA_CLIENT
@@ -48,7 +49,7 @@ set_date(
 
     clear_dir_list();
 
-    cmd = stralloc2("DATE ", date);
+    cmd = g_strconcat("DATE ", date, NULL);
     if (converse(cmd) == -1)
 	exit(1);
 
@@ -56,7 +57,8 @@ set_date(
        is still valid at the new date, and if not set directory to
        mount_point */
     if (disk_path != NULL) {
-	cmd = newstralloc2(cmd, "OISD ", disk_path);
+	g_free(cmd);
+	cmd = g_strconcat("OISD ", disk_path, NULL);
 	if (exchange(cmd) == -1)
 	    exit(1);
 	if (server_happy())
@@ -67,7 +69,8 @@ set_date(
 	{
 	    g_printf(_("No index records for cwd on new date\n"));
 	    g_printf(_("Setting cwd to mount point\n"));
-	    disk_path = newstralloc(disk_path, "/");	/* fake it */
+	    g_free(disk_path);
+	    disk_path = g_strdup("/");	/* fake it */
 	    clear_dir_list();
 	}
     }
@@ -84,7 +87,7 @@ set_host(
     struct hostent *hp;
     char **hostp;
     int found_host = 0;
-    char *uqhost = unquote_string(host);
+    char *uqhost;
 
     if (is_extract_list_nonempty())
     {
@@ -92,7 +95,8 @@ set_host(
 	return;
     }
 
-    cmd = stralloc2("HOST ", uqhost);
+    uqhost = unquote_string(host);
+    cmd = g_strconcat("HOST ", uqhost, NULL);
     if (converse(cmd) == -1)
 	exit(1);
     if (server_happy())
@@ -108,7 +112,8 @@ set_host(
 	if ((hp = gethostbyname(uqhost)) != NULL) {
 	    host = hp->h_name;
 	    g_printf(_("Trying host %s ...\n"), host);
-	    cmd = newstralloc2(cmd, "HOST ", host);
+	    g_free(cmd);
+	    cmd = g_strconcat("HOST ", host, NULL);
 	    if (converse(cmd) == -1)
 		exit(1);
 	    if(server_happy())
@@ -120,7 +125,8 @@ set_host(
 	        for (hostp = hp->h_aliases; (host = *hostp) != NULL; hostp++)
 	        {
 		    g_printf(_("Trying host %s ...\n"), host);
-		    cmd = newstralloc2(cmd, "HOST ", host);
+		    g_free(cmd);
+		    cmd = g_strconcat("HOST ", host, NULL);
 		    if (converse(cmd) == -1)
 		        exit(1);
 		    if(server_happy())
@@ -134,7 +140,8 @@ set_host(
     }
     if(found_host)
     {
-	dump_hostname = newstralloc(dump_hostname, host);
+	g_free(dump_hostname);
+	dump_hostname = g_strdup(host);
 	amfree(disk_name);
 	amfree(mount_point);
 	amfree(disk_path);
@@ -150,7 +157,7 @@ list_host(void)
 {
     char *cmd = NULL;
 
-    cmd = stralloc("LISTHOST");
+    cmd = g_strdup("LISTHOST");
     if (converse(cmd) == -1)
         exit(1);
     amfree(cmd);
@@ -183,33 +190,40 @@ set_disk(
 
     clear_dir_list();
     uqdsk = unquote_string(dsk);
-    cmd = stralloc2("DISK ", uqdsk);
+    cmd = g_strconcat("DISK ", uqdsk, NULL);
     if (converse(cmd) == -1)
 	exit(1);
     amfree(cmd);
 
-    if (!server_happy())
+    if (!server_happy()) {
+	amfree(uqdsk);
+	amfree(uqmtpt);
 	return;
+    }
 
-    disk_name = newstralloc(disk_name, uqdsk);
+    g_free(disk_name);
+    disk_name = g_strdup(uqdsk);
     if (mtpt == NULL)
     {
 	/* mount point not specified */
 	if (*uqdsk == '/')
 	{
 	    /* disk specified by mount point, hence use it */
-	    mount_point = newstralloc(mount_point, uqdsk);
+	    g_free(mount_point);
+	    mount_point = g_strdup(uqdsk);
 	}
 	else
 	{
 	    /* device name given, use '/' because nothing better */
-	    mount_point = newstralloc(mount_point, "/");
+	    g_free(mount_point);
+	    mount_point = g_strdup("/");
 	}
     }
     else
     {
 	/* mount point specified */
-	mount_point = newstralloc(mount_point, uqmtpt);
+	g_free(mount_point);
+	mount_point = g_strdup(uqmtpt);
     }
 
     /* set the working directory to the mount point */
@@ -221,14 +235,16 @@ set_disk(
 	exit(1);
     if (server_happy())
     {
-	disk_path = newstralloc(disk_path, "/");
+	g_free(disk_path);
+	disk_path = g_strdup("/");
 	suck_dir_list_from_server();	/* get list of directory contents */
     }
     else
     {
 	g_printf(_("No index records for disk for specified date\n"));
 	g_printf(_("If date correct, notify system administrator\n"));
-	disk_path = newstralloc(disk_path, "/");	/* fake it */
+	g_free(disk_path);
+	disk_path = g_strdup("/");	/* fake it */
 	clear_dir_list();
     }
     amfree(uqmtpt);
@@ -244,14 +260,14 @@ list_disk(
 
     if(amdevice) {
 	uqamdevice = unquote_string(amdevice);
-	cmd = stralloc2("LISTDISK ", uqamdevice);
+	cmd = g_strconcat("LISTDISK ", uqamdevice, NULL);
 	amfree(uqamdevice);
 	if (converse(cmd) == -1)
 	    exit(1);
 	amfree(cmd);
     }
     else {
-	cmd = stralloc("LISTDISK");
+	cmd = g_strdup("LISTDISK");
 	if (converse(cmd) == -1)
 	    exit(1);
 	amfree(cmd);
@@ -292,6 +308,7 @@ cd_glob(
         g_printf(_("\"%s\" is not a valid shell wildcard pattern: "), glob);
         puts(s);
 	amfree(regex);
+	amfree(uqglob);
         return;
     }
     /*
@@ -300,7 +317,7 @@ cd_glob(
      * in add_file, so strip that off.  Also, it anchors the end with
      * $, but we need to match a trailing /, add it if it is not there
      */
-    regex_path = stralloc(regex + 1);
+    regex_path = g_strdup(regex + 1);
     amfree(regex);
     if(regex_path[strlen(regex_path) - 2] != '/' ) {
 	regex_path[strlen(regex_path) - 1] = '\0';
@@ -308,11 +325,11 @@ cd_glob(
     }
 
     /* convert path (assumed in cwd) to one on disk */
-    if (strcmp(disk_path, "/") == 0)
-        path_on_disk = stralloc2("/", regex_path);
+    if (g_str_equal(disk_path, "/"))
+        path_on_disk = g_strconcat("/", regex_path, NULL);
     else {
         char *clean_disk_path = clean_regex(disk_path, 0);
-        path_on_disk = vstralloc(clean_disk_path, "/", regex_path, NULL);
+        path_on_disk = g_strjoin(NULL, clean_disk_path, "/", regex_path, NULL);
         amfree(clean_disk_path);
     }
 
@@ -346,11 +363,11 @@ cd_regex(
     }
 
     /* convert path (assumed in cwd) to one on disk */
-    if (strcmp(disk_path, "/") == 0)
-        path_on_disk = stralloc2("/", regex);
+    if (g_str_equal(disk_path, "/"))
+        path_on_disk = g_strconcat("/", regex, NULL);
     else {
         char *clean_disk_path = clean_regex(disk_path, 0);
-        path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+        path_on_disk = g_strjoin(NULL, clean_disk_path, "/", regex, NULL);
         amfree(clean_disk_path);
     }
 
@@ -373,7 +390,7 @@ cd_dir(
 
     DIR_ITEM *ditem;
 
-    path_on_disk_slash = stralloc2(path_on_disk, "/");
+    path_on_disk_slash = g_strconcat(path_on_disk, "/", NULL);
 
     nb_found = 0;
 
@@ -389,14 +406,15 @@ cd_dir(
             {   /* It is a directory */
 		char *dir1, *dir2;
 		nb_found++;
-		dir = newstralloc(dir,ditem->path);
+		g_free(dir);
+		dir = g_strdup(ditem->path);
 		if(dir[strlen(dir)-1] == '/')
 		    dir[strlen(dir)-1] = '\0'; /* remove last / */
 		/* remove everything before the last / */
 		dir1 = strrchr(dir,'/');
 		if (dir1) {
 		    dir1++;
-		    dir2 = stralloc(dir1);
+		    dir2 = g_strdup(dir1);
 		    amfree(dir);
 		    dir = dir2;
 		}
@@ -427,7 +445,7 @@ set_directory(
     char *ldir = NULL;
 
     /* do nothing if "." */
-    if(strcmp(dir,".")==0) {
+    if(g_str_equal(dir, ".")) {
 	show_directory();		/* say where we are */
 	return;
 	/*NOTREACHED*/
@@ -439,16 +457,16 @@ set_directory(
 	/*NOTREACHED*/
     }
 
-    ldir = stralloc(dir);
+    ldir = g_strdup(dir);
     clean_pathname(ldir);
 
     /* convert directory into absolute path relative to disk mount point */
     if (ldir[0] == '/')
     {
 	/* absolute path specified, must start with mount point */
-	if (strcmp(mount_point, "/") == 0)
+	if (g_str_equal(mount_point, "/"))
 	{
-	    new_dir = stralloc(ldir);
+	    new_dir = g_strdup(ldir);
 	}
 	else
 	{
@@ -460,19 +478,20 @@ set_directory(
 		return;
 		/*NOTREACHED*/
 	    }
-	    new_dir = stralloc(ldir+strlen(mount_point));
+	    new_dir = g_strdup(ldir+strlen(mount_point));
 	    if (strlen(new_dir) == 0) {
-		new_dir = newstralloc(new_dir, "/");
+		g_free(new_dir);
+		new_dir = g_strdup("/");
 					/* i.e. ldir == mount_point */
 	    }
 	}
     }
     else
     {
-	new_dir = stralloc(disk_path);
+	new_dir = g_strdup(disk_path);
 	dp = ldir;
 	/* strip any leading ..s */
-	while (strncmp(dp, "../", 3) == 0)
+	while (g_str_has_prefix(dp, "../"))
 	{
 	    de = strrchr(new_dir, '/');	/* always at least 1 */
 	    if (de == new_dir)
@@ -487,8 +506,8 @@ set_directory(
 		dp = dp + 3;
 	    }
 	}
-	if (strcmp(dp, "..") == 0) {
-	    if (strcmp(new_dir, "/") == 0) {
+	if (g_str_equal(dp, "..")) {
+	    if (g_str_equal(new_dir, "/")) {
 		/* at top of disk */
 		g_printf(_("Invalid directory - Can't cd outside mount point \"%s\"\n"),
 		       mount_point);
@@ -511,7 +530,7 @@ set_directory(
  	    }
 	} else {
 	    /*@ignore@*/
-	    if (strcmp(new_dir, "/") != 0) {
+	    if (!g_str_equal(new_dir, "/")) {
 		strappend(new_dir, "/");
 	    }
 	    strappend(new_dir, ldir);
@@ -519,7 +538,7 @@ set_directory(
 	}
     }
 
-    cmd = stralloc2("OISD ", new_dir);
+    cmd = g_strconcat("OISD ", new_dir, NULL);
     if (exchange(cmd) == -1) {
 	exit(1);
 	/*NOTREACHED*/
@@ -528,7 +547,8 @@ set_directory(
 
     if (server_happy())
     {
-	disk_path = newstralloc(disk_path, new_dir);
+	g_free(disk_path);
+	disk_path = g_strdup(new_dir);
 	suck_dir_list_from_server();	/* get list of directory contents */
 	show_directory();		/* say where we moved to */
     }
@@ -550,9 +570,9 @@ show_directory(void)
 {
     if (mount_point == NULL || disk_path == NULL)
         g_printf(_("Must select disk first\n"));
-    else if (strcmp(mount_point, "/") == 0)
+    else if (g_str_equal(mount_point, "/"))
 	g_printf("%s\n", disk_path);
-    else if (strcmp(disk_path, "/") == 0)
+    else if (g_str_equal(disk_path, "/"))
 	g_printf("%s\n", mount_point);
     else
 	g_printf("%s%s\n", mount_point, disk_path);
@@ -571,15 +591,16 @@ set_tape(
     {
 	if (tapedev != uqtape) {
 	    if((strchr(tapedev+1, ':') == NULL) &&
-	       (strncmp(uqtape, "null:", 5) == 0 ||
-		strncmp(uqtape, "rait:", 5) == 0 ||
-		strncmp(uqtape, "file:", 5) == 0 ||
-		strncmp(uqtape, "tape:", 5) == 0)) {
+	       (g_str_has_prefix(uqtape, "null:") ||
+		g_str_has_prefix(uqtape, "rait:") ||
+		g_str_has_prefix(uqtape, "file:") ||
+		g_str_has_prefix(uqtape, "tape:"))) {
 		tapedev = uqtape;
 	    }
 	    else {
 		*tapedev = '\0';
-		tape_server_name = newstralloc(tape_server_name, uqtape);
+		g_free(tape_server_name);
+		tape_server_name = g_strdup(uqtape);
 		++tapedev;
 	    }
 	} else { /* reset server_name if start with : */
@@ -588,13 +609,15 @@ set_tape(
 	}
     } else
 	tapedev = uqtape;
-    
+
     if (tapedev[0])
     {
-	if (strcmp(tapedev, "default") == 0)
+	if (g_str_equal(tapedev, "default"))
 	    amfree(tape_device_name);
-	else
-	    tape_device_name = newstralloc(tape_device_name, tapedev);
+	else {
+	        g_free(tape_device_name);
+	        tape_device_name = g_strdup(tapedev);
+	}
     }
 
     if (tape_device_name)
@@ -607,6 +630,7 @@ set_tape(
     else
 	g_printf (_(".\nTape server unspecified, assumed to be %s.\n"),
 		server_name);
+    amfree(uqtape);
 }
 
 void
