@@ -1,4 +1,5 @@
 # Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
 package Amanda::Changer::aggregate;
 
@@ -27,7 +28,7 @@ use vars qw( @ISA );
 
 use File::Glob qw( :glob );
 use File::Path;
-use Amanda::Config qw( :getconf );
+use Amanda::Config qw( :init :getconf string_to_boolean );
 use Amanda::Paths;
 use Amanda::Debug qw( debug warning );
 use Amanda::Util qw( :alternates );
@@ -54,7 +55,7 @@ See the amanda-changers(7) manpage for usage information.
 
 sub new {
     my $class = shift;
-    my ($config, $tpchanger) = @_;
+    my ($config, $tpchanger, %params) = @_;
     my ($kidspecs) = ( $tpchanger =~ /chg-aggregate:(.*)/ );
 
     my @kidspecs = Amanda::Util::expand_braced_alternates($kidspecs);
@@ -64,14 +65,14 @@ sub new {
     }
 
     my @children = map {
-	($_ eq "ERROR")? "ERROR" : Amanda::Changer->new($_)
+	($_ eq "ERROR")? "ERROR" : Amanda::Changer->new($_, %params)
     } @kidspecs;
 
-    my $fail_on_error;
-    if (defined $config->{'properties'}->{'fail-on-error'}) {
-	$fail_on_error = string_to_boolean($config->{'properties'}->{'fail-on-error'}->{'values'}[0]);
+    my $allow_missing_changer;
+    if (defined $config->{'properties'}->{'allow-missing-changer'}) {
+	$allow_missing_changer = string_to_boolean($config->{'properties'}->{'allow-missing-changer'}->{'values'}[0]);
     } else {
-	$fail_on_error = 1;
+	$allow_missing_changer = 0;
     }
 
     if (grep { $_->isa("Amanda::Changer::Error") } @children) {
@@ -85,12 +86,16 @@ sub new {
 		$valid++;
 	    }
 	}
-	if ($valid == 0 || $fail_on_error) {
+	if ($valid == 0 || !$allow_missing_changer) {
 	    return Amanda::Changer->make_combined_error(
 		"fatal", [ @annotated_errs ]);
 	}
     }
 
+    @children = map {
+	($_->isa("Amanda::Changer::Error")? "ERROR" : $_);
+    } @children;
+
     my $state_filename;
     my $state_filename_prop = $config->{'properties'}->{'state_filename'};
     my $lock_timeout = $config->{'properties'}->{'lock-timeout'};
@@ -101,6 +106,7 @@ sub new {
     if (!defined $state_filename) {
 	$state_filename = $Amanda::Paths::CONFIG_DIR . '/' . $config->{'name'} . ".state";
     }
+    debug("chg-aggragte: Using statefile '$state_filename'");
 
     my $self = {
 	config => $config,
@@ -120,7 +126,9 @@ sub quit {
 
     # quit each child
     foreach my $child (@{$self->{'children'}}) {
-        $child->quit();
+	if ($child ne "ERROR") {
+	    $child->quit();
+	}
     }
 
     $self->SUPER::quit();
@@ -133,8 +141,11 @@ sub _get_current_slot
 
     $self->with_locked_state($self->{'state_filename'}, $cb, sub {
 	my ($state, $cb) = @_;
-	$self->{'current_slot'} = $state->{'current_slot'};
-	$self->{'current_slot'} = "0:first" if !defined $self->{'current_slot'};
+	my $storage = $self->{'storage'}->{'storage_name'};
+	my $changer = $self->{'chg_name'};
+	$self->{'current_slot'} = $state->{'current_slot_csc'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer};
+	$self->{'current_slot'} = $state->{'current_slot'} if !defined $self->{'current_slot'};
+	$self->{'current_slot'} = "0:1" if !defined $self->{'current_slot'};
 	$cb->();
     });
 }
@@ -146,11 +157,39 @@ sub _set_current_slot
 
     $self->with_locked_state($self->{'state_filename'}, $cb, sub {
 	my ($state, $cb) = @_;
+	my $storage = $self->{'storage'}->{'storage_name'};
+	my $changer = $self->{'chg_name'};
+	$state->{'current_slot_csc'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer} = $self->{'current_slot'};
 	$state->{'current_slot'} = $self->{'current_slot'};
 	$cb->();
     });
 }
 
+sub slot_in_same_changer {
+    my $self  = shift;
+    my $slot1 = shift;
+    my $slot2 = shift;
+
+    (my $kid1 = $slot1) =~ s/:.*$//g;
+    (my $kid2 = $slot2) =~ s/:.*$//g;
+
+    if ($kid1 != $kid2) {
+	return 0;
+    }
+
+    my $child = $self->{'children'}[$kid1];
+    if (!defined $child) {
+	return 0;
+    }
+
+    if ($child->can("slot_in_same_changer")) {
+	my $kid_slot1 =~ s/[^:]*://g;
+	my $kid_slot2 =~ s/[^:]*://g;
+	return $child->slot_in_same_changer($kid_slot1, $kid_slot2);
+    }
+    return 1;
+}
+
 sub load {
     my $self = shift;
     my %params = @_;
@@ -231,7 +270,7 @@ sub load {
 	for my $i (0.. scalar(@$inv)-1) {
 	    my $slot = @$inv[$i]->{'slot'};
 	    my $label = @$inv[$i]->{'label'};
-	    if ($label eq $params{'label'}) {
+	    if (defined $label && $label eq $params{'label'}) {
 		$orig_slot = $slot;
 		return $steps->{'slot_set'}->();
 	    }
@@ -271,6 +310,12 @@ sub load {
 	    }
 	    return $res_cb->($err, undef);
 	};
+	if ($child eq "ERROR") {
+	    return $self->make_error("failed", $res_cb,
+		reason => "invalid",
+		message => "no changer $kid");
+	}
+
 	return $child->load(%params);
     };
 
@@ -337,6 +382,35 @@ sub info_key {
 	    errsub => undef,
 	    parent_cb => $all_kids_done_cb,
 	);
+    } elsif ($key eq 'slots') {
+	my $all_kids_done_cb = sub {
+	    my ($kid_results) = @_;
+	    return if ($check_and_report_errors->($kid_results));
+
+	    # Sum the result
+	    my @slots;
+	    my $kid = 0;
+	    for (@$kid_results) {
+		my ($err, %kid_info) = @$_;
+		if (!exists($kid_info{'slots'})){
+		    $kid++;
+		    next;
+		}
+		my $kid_slots = $kid_info{'slots'};
+		push @slots, map "$kid:$_", @{$kid_slots};
+		$kid++;
+	    }
+	    $params{'info_cb'}->(undef, slots => \@slots) if $params{'info_cb'};
+	};
+
+	$self->_for_each_child(
+	    oksub => sub {
+		my ($kid_chg, $kid_cb) = @_;
+		$kid_chg->info(info => [ 'slots' ], info_cb => $kid_cb);
+	    },
+	    errsub => undef,
+	    parent_cb => $all_kids_done_cb,
+	);
     } elsif ($key eq "vendor_string") {
 	my $all_kids_done_cb = sub {
 	    my ($kid_results) = @_;
@@ -440,15 +514,29 @@ sub _mk_simple_op {
     };
 }
 
+my $_reset = _mk_simple_op("reset");
 {
     # perl doesn't like that these symbols are only mentioned once
     no warnings;
 
-    *reset = _mk_simple_op("reset");
     *clean = _mk_simple_op("clean");
     *eject = _mk_simple_op("eject");
 }
 
+sub reset {
+    my $self = shift;
+    my %params = @_;
+
+    my $finished_cb = $params{'finished_cb'};
+    delete $params{'finished_cb'};
+
+    $_reset->($self, %params, finished_cb => sub {
+	my @args = @_;
+	$self->{'current_slot'} = "0:1";
+	$self->_set_current_slot(sub { $finished_cb->(@args) });
+    });
+}
+
 sub update {
     my $self = shift;
     my %params = @_;
@@ -458,7 +546,7 @@ sub update {
     my $user_msg_fn = $params{'user_msg_fn'};
     $user_msg_fn ||= sub { Amanda::Debug::info("chg-aggregate: " . $_[0]); };
 
-    if (exists $params{'changed'}) {
+    if (defined $params{'changed'}) {
 	for my $range (split ',', $params{'changed'}) {
 	    my ($first, $last) = ($range =~ /([:\d]+)(?:-([:\d]+))?/);
 	    $last = $first unless defined($last);
@@ -604,7 +692,7 @@ sub set_meta_label {
 
     my ($kid, $slot) = split(':', $orig_slot, 2);
     my $child = $self->{'children'}[$kid];
-    if (!defined $child) {
+    if (!defined $child || $child eq "ERROR") {
 	return $self->make_error("failed", $finished_cb,
 	    reason => "invalid",
 	    message => "no changer $kid");
@@ -632,7 +720,7 @@ sub get_meta_label {
 
     my ($kid, $slot) = split(':', $orig_slot, 2);
     my $child = $self->{'children'}[$kid];
-    if (!defined $child) {
+    if (!defined $child || $child eq "ERROR") {
 	return $self->make_error("failed", $finished_cb,
 	    reason => "invalid",
 	    message => "no changer $kid");
@@ -705,7 +793,7 @@ sub _merge_inventories {
 
 	for my $x (@$kid_inv) {
 	    my $slotname = "$nb:" . $x->{'slot'};
-	    my $current = $slotname eq $self->{'current_slot'};
+	    my $current = ($slotname eq $self->{'current_slot'})?1:undef;
 	    push @combined, {
 		    state => $x->{'state'},
 		    device_status => $x->{'device_status'},
@@ -767,3 +855,37 @@ sub set_meta_label {
 
     $self->{'kid_res'}->set_meta_label(%params);
 }
+
+sub make_new_tape_label {
+    my $self = shift;
+    my %params = @_;
+
+    $params{'barcode'} = $self->{'barcode'} if !defined $params{'barcode'};
+    $params{'meta'} = $self->{'meta'} if !defined $params{'meta'};
+    if (!defined $params{'slot'}) {
+	$params{'slot'} = $self->{'this_slot'};
+	$params{'slot'} =~ s/^.*://g;
+    }
+    return $self->{'kid_res'}->make_new_tape_label(%params);
+}
+
+sub make_new_meta_label {
+    my $self = shift;
+    my %params = @_;
+
+    return $self->{'kid_res'}->make_new_meta_label(%params);
+}
+
+sub set_label {
+    my $self = shift;
+    my %params = @_;
+
+    $self->{'kid_res'}->set_label(%params);
+}
+
+sub set_device_error {
+    my $self = shift;
+    my %params = @_;
+
+    $self->{'kid_res'}->set_device_error(%params);
+}
