@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
 package Amanda::Changer::disk;
 
@@ -28,8 +29,7 @@ use vars qw( @ISA );
 use File::Glob qw( :glob );
 use File::Path;
 use File::Basename;
-use Errno;
-use Amanda::Config qw( :getconf string_to_boolean );
+use Amanda::Config qw( :getconf :init string_to_boolean );
 use Amanda::Debug qw( debug warning );
 use Amanda::Changer;
 use Amanda::MainLoop;
@@ -45,10 +45,6 @@ This changer operates within a root directory, specified in the changer
 string, which it arranges as follows:
 
   $dir -|
-        |- drive0/ -|
-        |           | data -> '../slot4'
-        |- drive1/ -|
-        |           | data -> '../slot1'
         |- data -> slot5
         |- slot1/
         |- slot2/
@@ -56,14 +52,10 @@ string, which it arranges as follows:
         |- slot$n/
 
 The user should create the desired number of C<slot$n> subdirectories.  The
-changer will take care of dynamically creating the drives as needed, and track
+changer track
 the current slot using a "data" symlink.  This allows use of "file:$dir" as a
 device operating on the current slot, although note that it is unlocked.
 
-Drives are dynamically allocated as Amanda applications request access to
-particular slots.  Each drive is represented as a subdirectory containing a
-'data' symlink pointing to the "loaded" slot.
-
 See the amanda-changers(7) manpage for usage information.
 
 =cut
@@ -72,26 +64,29 @@ See the amanda-changers(7) manpage for usage information.
 #
 # The device state is shared between all changers accessing the same changer.
 # It is a hash with keys:
-#   drives - see below
+#   current - the slot directory of the current slot
+#   current_slot->{'config'}->{$config_name}->{'storage'}->{$storage_name}->{'changer'}->{$changer}
+#   meta    - meta label of the vtapes
+#   drives  - see below
 #
 # The 'drives' key is a hash, with drive as keys and hashes
 # as values.  Each drive's hash has keys:
-#   pid - the pid that reserved that drive.
+#   slot - slot directory
+#   pid  - the pid that reserved that drive.
 #
 
 
 sub new {
     my $class = shift;
-    my ($config, $tpchanger) = @_;
+    my ($config, $tpchanger, %params) = @_;
     my ($dir) = ($tpchanger =~ /chg-disk:(.*)/);
     my $properties = $config->{'properties'};
 
-    # note that we don't track outstanding Reservation objects -- we know
-    # they're gone when they delete their drive directory
     my $self = {
 	dir => $dir,
 	config => $config,
 	state_filename => "$dir/state",
+	global_space => 1,
 
 	# list of all reservations
 	reservation => {},
@@ -108,7 +103,7 @@ sub new {
     }
     $self->{'lock-timeout'} = $config->get_property('lock-timeout');
 
-    $self->{'num-slot'} = $config->get_property('num-slot');
+    $self->{'num-slot'} = $config->get_property('num-slot') || 1;
     $self->{'auto-create-slot'} = $config->get_boolean_property(
 					'auto-create-slot', 0);
     $self->{'removable'} = $config->get_boolean_property('removable', 0);
@@ -120,7 +115,10 @@ sub new {
 	$self->{'fl'} = Amanda::Util::file_lock->new($self->{'umount_lockfile'})
     }
 
-    $self->_validate();
+    # the following are set in Changer.pm, but as _validate need them, they
+    # must be set here.
+    $self->{'runtapes'} = $params{'storage'}->{'runtapes'} || 1;
+
     debug("chg-disk: Dir $dir");
     debug("chg-disk: Using statefile '$self->{state_filename}'");
     return $self->{'fatal_error'} if defined $self->{'fatal_error'};
@@ -137,9 +135,36 @@ sub DESTROY {
 sub quit {
     my $self = shift;
 
-    $self->force_unlock();
-    delete $self->{'fl'};
-    $self->SUPER::quit();
+    if (defined $self->{'dir'}) {
+	$self->force_unlock();
+	delete $self->{'fl'};
+	delete $self->{'dir'};
+	$self->SUPER::quit();
+    }
+}
+
+sub create {
+    my $self = shift;
+    my %params = @_;
+
+    return if $self->check_error($params{'finished_cb'});
+
+    if (!mkdir($self->{'dir'}, 0700)) {
+	return $self->make_error("failed", $params{'finished_cb'},
+		source_filename => __FILE__,
+		source_line     => __LINE__,
+		code    => 1100026,
+		severity => $Amanda::Message::ERROR,
+		dir     => $self->{'dir'},
+		error   => $!,
+		reason  => "unknown");
+    }
+    return $params{'finished_cb'}->(undef, Amanda::Changer::Message->new(
+		source_filename => __FILE__,
+		source_line     => __LINE__,
+		code    => 1100027,
+		severity => $Amanda::Message::SUCCESS,
+		dir     => $self->{'dir'}));
 }
 
 sub load {
@@ -190,6 +215,9 @@ sub info_key {
 	if ($key eq 'num_slots') {
 	    my @slots = $self->_all_slots();
 	    $results{$key} = scalar @slots;
+	} elsif ($key eq 'slots') {
+	    my @slots = $self->_all_slots();
+	    $results{$key} = \@slots;
 	} elsif ($key eq 'vendor_string') {
 	    $results{$key} = 'chg-disk'; # mostly just for testing
 	} elsif ($key eq 'fast_search') {
@@ -213,7 +241,7 @@ sub reset {
 	my ($state, $finished_cb) = @_;
 
 	$slot = (scalar @slots)? $slots[0] : 0;
-	$self->_set_current($slot);
+	$self->_set_current($state, $slot);
 
 	$finished_cb->();
     });
@@ -229,8 +257,9 @@ sub inventory {
 	my ($state, $finished_cb) = @_;
 	my @inventory;
 
+	my $nb_empty = 0;
 	my @slots = $self->_all_slots();
-	my $current = $self->_get_current();
+	my $current = $self->_get_current($state);
 	for my $slot (@slots) {
 	    my $s = { slot => $slot, state => Amanda::Changer::SLOT_FULL };
 	    $s->{'reserved'} = $self->_is_slot_in_use($state, $slot);
@@ -243,10 +272,26 @@ sub inventory {
 		$s->{'label'} = undef;
 		$s->{'f_type'} = "".$Amanda::Header::F_EMPTY;
 		$s->{'device_status'} = "".$DEVICE_STATUS_VOLUME_UNLABELED;
+		$nb_empty++;
 	    }
 	    $s->{'current'} = 1 if $slot eq $current;
 	    push @inventory, $s;
 	}
+	# Add up to runtapes slots
+	my $last_slot = $slots[-1];
+	if ($nb_empty < $self->{'runtapes'} && $self->{'num-slot'} &&
+	    $self->{'auto-create-slot'} && $last_slot < $self->{'num-slot'}) {
+	    my $to_add = $self->{'runtapes'} - $nb_empty;
+	    for (my $i = 1; $i <= $to_add; $i++) {
+		my $slot = $last_slot + $i;
+		last if $slot > $self->{'num-slot'};
+		push @inventory, { slot => $slot,
+				   state => Amanda::Changer::SLOT_FULL,
+				   label => undef,
+				   f_type => "".$Amanda::Header::F_EMPTY,
+				   device_status => "".$DEVICE_STATUS_VOLUME_UNLABELED };
+	    }
+	}
 	$finished_cb->(undef, \@inventory);
     });
 }
@@ -309,15 +354,15 @@ sub _load_by_slot {
 
     if (exists $params{'relative_slot'}) {
 	if ($params{'relative_slot'} eq "current") {
-	    $slot = $self->_get_current();
+	    $slot = $self->_get_current($params{'state'});
 	} elsif ($params{'relative_slot'} eq "next") {
 	    if (exists $params{'slot'}) {
 		$slot = $params{'slot'};
 	    } else {
-		$slot = $self->_get_current();
+		$slot = $self->_get_current($params{'state'});
 	    }
 	    $slot = $self->_get_next($slot);
-	    $self->_set_current($slot) if ($params{'set_current'});
+	    $self->_set_current($params{'state'}, $slot) if ($params{'set_current'});
 	} else {
 	    return $self->make_error("failed", $params{'res_cb'},
 		reason => "invalid",
@@ -347,11 +392,9 @@ sub _load_by_slot {
 	    message => "Slot $slot is already in use by drive '$drive' and process '$params{state}->{drives}->{$drive}->{pid}'");
     }
 
-    $drive = $self->_alloc_drive($params{'res_cb'});
-    return if ref($drive) ne '';
-
-    $self->_load_drive($drive, $slot);
-    $self->_set_current($slot) if ($params{'set_current'});
+    $drive = $self->_alloc_drive($params{state});
+    $self->_load_drive($params{state}, $drive, $slot);
+    $self->_set_current($params{state}, $slot) if ($params{'set_current'});
 
     $self->_make_res($params{'state'}, $params{'res_cb'}, $drive, $slot);
 }
@@ -374,37 +417,41 @@ sub _load_by_label {
 	return $self->make_error("failed", $params{'res_cb'},
 	    reason => "volinuse",
 	    message => "Slot $slot, containing '$label', is already " .
-			"in use by drive '$drive'");
+			"in use by drive '$drive' and process '$params{state}->{drives}->{$drive}->{pid}'");
     }
 
-    $drive = $self->_alloc_drive($params{'res_cb'});
-    return if ref($drive) ne '';
-
-    $self->_load_drive($drive, $slot);
-    $self->_set_current($slot) if ($params{'set_current'});
+    $drive = $self->_alloc_drive($params{state});
+    $self->_load_drive($params{state}, $drive, $slot);
+    $self->_set_current($params{state}, $slot) if ($params{'set_current'});
 
     $self->_make_res($params{'state'}, $params{'res_cb'}, $drive, $slot);
 }
 
 sub _make_res {
     my $self = shift;
-    my ($state, $res_cb, $drive, $slot) = @_;
+    my ($state, $res_cb, $drive, $slot, $meta) = @_;
     my $res;
 
-    my $device = Amanda::Device->new("file:$drive");
+    my $slot_path = "$self->{'dir'}/slot$slot";
+    my $device_name = "file:$slot_path";
+    my $device = Amanda::Device->new($device_name);
     if ($device->status != $DEVICE_STATUS_SUCCESS) {
 	return $self->make_error("failed", $res_cb,
 		reason => "device",
-		message => "opening 'file:$drive': " . $device->error_or_status());
+		message => "opening '$device_name': " . $device->error_or_status());
     }
-
-    if (my $err = $self->{'config'}->configure_device($device)) {
+    my ($use_data, $surety, $source) = $device->property_get("USE-DATA");
+    if ($source == $PROPERTY_SOURCE_DEFAULT) {
+	my $r = $device->property_set("USE-DATA", "NO");
+	$use_data = !$r;
+    }
+    if (my $err = $self->{'config'}->configure_device($device, $self->{'storage'})) {
 	return $self->make_error("failed", $res_cb,
 		reason => "device",
 		message => $err);
     }
 
-    $res = Amanda::Changer::disk::Reservation->new($self, $device, $drive, $slot);
+    $res = Amanda::Changer::disk::Reservation->new($self, $device, $drive, $slot, $state->{'meta'});
     $state->{drives}->{$drive}->{pid} = $$;
     $device->read_label();
 
@@ -414,23 +461,16 @@ sub _make_res {
 # Internal function to find an unused (nonexistent) driveN subdirectory and
 # create it.  Note that this does not add a 'data' symlink inside the directory.
 sub _alloc_drive {
-    my $self = shift;
-    my $res_cb = shift;
+    my ($self, $state) = @_;
     my $n = 0;
 
     while (1) {
-	my $drive = $self->{'dir'} . "/drive$n";
+	my $drive = "drive$n";
 	$n++;
 
-	warn "$drive is not a directory; please remove it" if (-e $drive and ! -d $drive);
-	next if (-e $drive);
-	if (mkdir($drive)) { # TODO probably not a very effective locking mechanism..
-	    return $drive;
-	}
-	next if ($! == &Errno::EEXIST);
-	return $self->make_error("failed", $res_cb,
-		reason => "device",
-		message => "Can't make directory '$drive': $!");
+	next if exists $state->{'drives'}->{$drive};
+
+	return $drive;
     }
 }
 
@@ -453,6 +493,11 @@ sub _all_slots {
 # Internal function to determine whether a slot exists.
 sub _slot_exists {
     my ($self, $slot) = @_;
+
+    return 0 if $slot !~ /^\d*$/;
+    if ($self->{'num-slot'} && $slot <= $self->{'num-slot'} && $self->{'auto-create-slot'}) {
+	mkdir $self->{'dir'} . "/slot$slot";
+    }
     return (-d $self->{'dir'} . "/slot$slot");
 }
 
@@ -462,36 +507,24 @@ sub _is_slot_in_use {
     my ($self, $state, $slot) = @_;
     my $dir = _quote_glob($self->{'dir'});
 
-    for my $symlink (bsd_glob("$dir/drive*/data")) {
-	if (! -l $symlink) {
-	    warn "'$symlink' is not a symlink; please remove it";
+    foreach my $drive (keys %{$state->{'drives'}}) {
+	my $adrive = $state->{'drives'}->{$drive};
+	if (!defined $adrive->{'slot'} or !defined $adrive->{'pid'}) {
+	    delete $state->{'drives'}->{$drive};
 	    next;
 	}
 
-	my $target = readlink($symlink);
-	if (!$target) {
-	    warn "could not read '$symlink': $!";
-	    next;
-	}
-
-	my $tslot;
-	if (!(($tslot) = ($target =~ /..\/slot([0-9]+)/))) {
-	    warn "invalid changer symlink '$symlink' -> '$target'";
+	my $tslot = $adrive->{'slot'};
+	if (!(($tslot) = ($adrive->{'slot'} =~ /slot([0-9]+)/))) {
+	    warn "invalid slot '$adrive->{'slot'}' for drive '$drive'";
 	    next;
 	}
 
 	if ($tslot+0 == $slot) {
-	    my $drive = $symlink;
-	    $drive =~ s{/data$}{}; # strip the trailing '/data'
-
 	    #check if process is alive
-	    my $pid = $state->{drives}->{$drive}->{pid};
+	    my $pid = $state->{'drives'}->{$drive}->{'pid'};
 	    if (!defined $pid or !Amanda::Util::is_pid_alive($pid)) {
-		unlink("$drive/data")
-		    or warn("Could not unlink '$drive/data': $!");
-		rmdir("$drive")
-		    or warn("Could not rmdir '$drive': $!");
-		delete $state->{drives}->{$drive}->{pid};
+		delete $state->{'drives'}->{$drive};
 		next;
 	    }
 	    return $drive;
@@ -515,15 +548,9 @@ sub _get_slot_label {
 
 # Internal function to point a drive to a slot
 sub _load_drive {
-    my ($self, $drive, $slot) = @_;
+    my ($self, $state, $drive, $slot) = @_;
 
-    confess "'$drive' does not exist" unless (-d $drive);
-    if (-e "$drive/data") {
-	unlink("$drive/data");
-    }
-
-    symlink("../slot$slot", "$drive/data");
-    # TODO: read it to be sure??
+    $state->{'drives'}->{$drive}->{'slot'} = "slot$slot";
 }
 
 # Internal function to return the slot containing a volume with the given
@@ -553,6 +580,7 @@ sub _get_next {
 
     # Try just incrementing the slot number
     $next_slot = $slot+1;
+    return $next_slot if $next_slot <= $self->{'num-slot'} && $self->{'auto-create-slot'};
     return $next_slot if (-d $self->{'dir'} . "/slot$next_slot");
 
     # Otherwise, search through all slots
@@ -569,7 +597,24 @@ sub _get_next {
 
 # Get the 'current' slot, represented as a symlink named 'data'
 sub _get_current {
-    my ($self) = @_;
+    my $self = shift;
+    my $state = shift;
+
+    my $storage = $self->{'storage'}->{'storage_name'};
+    my $changer = $self->{'chg_name'};
+    my $current_slot = $state->{'current_slot'}->{'config'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer};
+    if (defined $current_slot) {
+	if ($current_slot =~ "^slot([0-9]+)/?") {
+	    return $1;
+	}
+    }
+
+    if ($state->{'current'}) {
+        if ($state->{'current'} =~ "^slot([0-9]+)/?") {
+            return $1;
+        }
+    }
+
     my $curlink = $self->{'dir'} . "/data";
 
     # for 2.6.1-compatibility, also parse a "current" symlink
@@ -593,7 +638,14 @@ sub _get_current {
 
 # Set the 'current' slot
 sub _set_current {
-    my ($self, $slot) = @_;
+    my $self  = shift;
+    my $state = shift;
+    my $slot  = shift;
+
+    $state->{'current'} = "slot$slot";
+    my $storage = $self->{'storage'}->{'storage_name'};
+    my $changer = $self->{'chg_name'};
+    $state->{'current_slot'}->{'config'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer} = "slot$slot";
     my $curlink = $self->{'dir'} . "/data";
 
     if (-l $curlink or -e $curlink) {
@@ -638,22 +690,7 @@ sub _validate() {
 	}
     }
 
-    if ($self->{'num-slot'}) {
-	for my $i (1..$self->{'num-slot'}) {
-	    my $slot_dir = "$dir/slot$i";
-	    if (!-e $slot_dir) {
-		if ($self->{'auto-create-slot'}) {
-		    if (!mkdir ($slot_dir)) {
-			return $self->make_error("fatal", undef,
-			    message => "Can't create '$slot_dir': $!");
-		    }
-		} else {
-		    return $self->make_error("fatal", undef,
-			message => "slot $i doesn't exists '$slot_dir'");
-		}
-	    }
-	}
-    } else {
+    if (!$self->{'num-slot'}) {
 	if ($self->{'auto-create-slot'}) {
 	    return $self->make_error("fatal", undef,
 		message => "property 'auto-create-slot' set but property 'num-slot' is not set");
@@ -708,8 +745,7 @@ sub try_lock {
     };
 
     step lock_done => sub {
-	my $err = $self->_validate();
-	$cb->($err);
+	$cb->(undef);
     };
 }
 
@@ -791,7 +827,7 @@ use vars qw( @ISA );
 
 sub new {
     my $class = shift;
-    my ($chg, $device, $drive, $slot) = @_;
+    my ($chg, $device, $drive, $slot, $meta) = @_;
     my $self = Amanda::Changer::Reservation::new($class);
 
     $self->{'chg'} = $chg;
@@ -799,6 +835,7 @@ sub new {
 
     $self->{'device'} = $device;
     $self->{'this_slot'} = $slot;
+    $self->{'meta'} = $meta;
 
     $self->{'chg'}->{'reservation'}->{$slot} += 1;
     return $self;
@@ -809,11 +846,6 @@ sub do_release {
     my %params = @_;
     my $drive = $self->{'drive'};
 
-    unlink("$drive/data")
-	or warn("Could not unlink '$drive/data': $!");
-    rmdir("$drive")
-	or warn("Could not rmdir '$drive': $!");
-
     # unref the device, for good measure
     $self->{'device'} = undef;
     my $slot = $self->{'this_slot'};
@@ -838,7 +870,7 @@ sub do_release {
 				    $finish, sub {
 	my ($state, $finished_cb) = @_;
 
-	delete $state->{drives}->{$drive}->{pid};
+	delete $state->{drives}->{$drive};
 
 	$finished_cb->();
     });
@@ -858,4 +890,5 @@ sub set_meta_label {
 
     $params{'slot'} = $self->{'this_slot'};
     $self->{'chg'}->set_meta_label(%params);
+    $self->{'meta'} = $params{'meta'};
 }
