@@ -1,4 +1,5 @@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,7 +15,7 @@
 # along with this library; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 package Amanda::Changer::robot;
@@ -28,9 +29,8 @@ use vars qw( @ISA );
 use Data::Dumper;
 use File::Path;
 use Amanda::Paths;
-use Amanda::Config qw( :init );
+use Amanda::Config qw( :init :getconf );
 use Amanda::MainLoop qw( :GIOCondition make_cb define_steps step );
-use Amanda::Config qw( :getconf );
 use Amanda::Debug qw( debug warning );
 use Amanda::Device qw( :constants );
 use Amanda::Changer;
@@ -62,6 +62,8 @@ See the amanda-changers(7) manpage for usage information.
 #   drive_lru - recently used drives, least recent first
 #   bc2lb - hash mapping known barcodes to known labels
 #   current_slot - hash of the current slot
+#       ->{$config_name}    #deprecated
+#       ->{'config'}->{$config_name}->{'storage'}->{$storage_name}->{'changer'}->{$changer_name}
 #   last_operation_time - time the last operation finished
 #   last_operation_delay - required delay for that operation
 #   last_status - last time a 'status' command finished
@@ -86,8 +88,6 @@ See the amanda-changers(7) manpage for usage information.
 #   barcode - volume barcode
 #   orig_slot - slot from which this tape was loaded
 #
-## The 'current_slot' key is also a hash by config name, the values of
-# which are the current slot for the config.
 # #
 # LOCKING
 #
@@ -319,25 +319,6 @@ sub new {
     return $self;
 }
 
-sub slot_can_have_label {
-    my $self = shift;
-    my $slot = shift;
-    my $label = shift;
-
-    return 1 if !$self->{'state'};
-    my $search_barcode;
-    while (my($barcode, $blabel) = each %{$self->{'state'}->{'bc2lb'}}) {
-	$search_barcode = $barcode if $blabel eq $label;
-    }
-    my $slot_barcode = $self->{'state'}->{'slots'}->{$slot}->{'barcode'};
-    return 1 if !$search_barcode;
-    return 1 if !$slot_barcode;
-    return 0 if $search_barcode ne $slot_barcode;
-    return 1 if $search_barcode eq $slot_barcode;
-
-    return 0;
-}
-
 sub load {
     my $self = shift;
     my %params = @_;
@@ -348,11 +329,35 @@ sub load {
     $self->_with_updated_state(\%params, 'res_cb', sub { $self->load_unlocked(@_) });
 }
 
+sub _get_current {
+    my $self = shift;
+    my %params = @_;
+    my $state = $params{'state'};
+
+    my $storage = $self->{'storage'}->{'storage_name'};
+    my $changer = $self->{'chg_name'};
+    my $current_slot = $state->{'current_slot'}->{'config'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer};
+    return $current_slot if defined $current_slot;
+    return $state->{'current_slot'}{get_config_name()};
+}
+
+sub _set_current {
+    my $self = shift;
+    my %params = @_;
+    my $state = $params{'state'};
+    my $slot = $params{'slot'};
+
+    my $storage = $self->{'storage'}->{'storage_name'};
+    my $changer = $self->{'chg_name'};
+    $state->{'current_slot'}->{'config'}->{get_config_name()}->{'storage'}->{$storage}->{'changer'}->{$changer} = $slot;
+    #$state->{'current_slot'}{get_config_name()} = $slot;
+}
+
 sub set_error_to_unknown {
     my $self  = shift;
     my %params = @_;
     my $state;
-    $self->with_locked_state($self->{'state_filename'},
+    $self->with_locked_state($self->{'statefile'},
 			     $params{'set_to_unknown_cb'}, sub {
 	my ($state, $set_to_unknown_cb) = @_;
 	for my $slot (keys %{ $state->{'slots'} }) {
@@ -400,7 +405,8 @@ sub load_unlocked {
 			$self->_debug("no next slot relative to $params{slot}");
 		    }
 		} else {
-		    $slot = $self->_get_next_slot($state, $state->{'current_slot'}{get_config_name()}, $params{'except_slots'});
+		    my $current_slot = $self->_get_current(state => $state);
+		    $slot = $self->_get_next_slot($state, $current_slot, $params{'except_slots'});
 		    if (defined $slot) {
 			$self->_debug("loading next relative to current slot: $slot");
 		    } else {
@@ -413,16 +419,17 @@ sub load_unlocked {
 			    message => "could not find next slot");
 		}
             } elsif ($params{'relative_slot'} eq "current") {
-                $slot = $state->{'current_slot'}{get_config_name()};
-		if (defined $slot && $slot == -1) {
+		my $current_slot = $self->_get_current(state => $state);
+		if (defined $current_slot && $current_slot == -1) {
 		    # seek to the first slot
-		    $slot = $self->_get_next_slot($state, $state->{'current_slot'}{get_config_name()}, $params{'except_slots'});
+		    $slot = $self->_get_next_slot($state, $current_slot, $params{'except_slots'});
 		}
-		if (defined $slot && $slot == -1) {
+		if (defined $current_slot && $current_slot == -1) {
 		    return $self->make_error("failed", $params{'res_cb'},
 			    reason => "invalid",
 			    message => "no current slot");
 		}
+		$slot = $current_slot;
 	    } else {
 		return $self->make_error("failed", $params{'res_cb'},
 			reason => "invalid",
@@ -565,7 +572,9 @@ sub load_unlocked {
 
 	# here is where we implement each of the drive-selection algorithms
 	my @check_order;
-	if ($self->{'drive_choice'} eq 'lru') {
+	if ($params{'drive'}) {
+	    @check_order = ($params{'drive'});
+	} elsif ($self->{'drive_choice'} eq 'lru') {
             my %lru = map { $_, 1 } @{$state->{'drive_lru'}};
             my @unused = grep { ! exists $lru{$_} } @{$self->{'driveorder'}};
 
@@ -765,6 +774,7 @@ sub load_unlocked {
 
 	    return $self->make_error("failed", $params{'res_cb'},
 		    reason => "notfound",
+		    slot   => $slot,
 		    message => "Found unexpected tape '$label' while looking " .
 			       "for '$params{label}'");
 	}
@@ -792,26 +802,11 @@ sub load_unlocked {
 		$self->_debug("Expected label '$params{label}', but got an unlabeled tape");
 		return $self->make_error("failed", $params{'res_cb'},
 		    reason => "notfound",
+		    slot   => $slot,
 		    message => "Found unlabeled tape while looking for '$params{label}'");
 	    }
 	}
 
-	if (defined $self->{'tapelist'}) {
-	    my $tle = $self->{'tapelist'}->lookup_tapelabel($label);
-	    if (defined $tle and defined $tle->{'barcode'} and
-		defined $state->{'slots'}->{$slot}->{'barcode'} and
-		$state->{'slots'}->{$slot}->{'barcode'} ne $tle->{'barcode'}) {
-		return $self->make_error("failed", $params{'res_cb'},
-		    reason => "device",
-		    message => "Slot $slot, label '$label', mismatch barcode between changer '$state->{'slots'}->{$slot}->{'barcode'}' and tapelist file '$tle->{'barcode'}'");
-	    }
-	}
-        my $res = Amanda::Changer::robot::Reservation->new($self, $slot, $drive,
-                                $device, $state->{'slots'}->{$slot}->{'barcode'});
-
-	# mark this as reserved
-	$state->{'drives'}->{$drive}->{'res_info'} = $self->_res_info_new();
-
 	# update our state before returning
 	$state->{'slots'}->{$slot}->{'loaded_in'} = $drive;
 	$state->{'drives'}->{$drive}->{'orig_slot'} = $slot;
@@ -841,9 +836,26 @@ sub load_unlocked {
 	}
 	if ($params{'set_current'}) {
 	    $self->_debug("setting current slot to $slot");
-	    $state->{'current_slot'}{get_config_name()} = $slot;
+	    $self->_set_current(state => $state, slot => $slot);
 	}
 
+	if (defined $self->{'tapelist'} && $label) {
+	    my $tle = $self->{'tapelist'}->lookup_tapelabel($label);
+	    if (defined $tle and defined $tle->{'barcode'} and
+		defined $state->{'slots'}->{$slot}->{'barcode'} and
+		$state->{'slots'}->{$slot}->{'barcode'} ne $tle->{'barcode'}) {
+		return $self->make_error("failed", $params{'res_cb'},
+		    reason => "device",
+		    slot   => $slot,
+		    message => "Slot $slot, label '$label', mismatch barcode between changer '$state->{'slots'}->{$slot}->{'barcode'}' and tapelist file '$tle->{'barcode'}'");
+	    }
+	}
+        my $res = Amanda::Changer::robot::Reservation->new($self, $slot, $drive,
+                                $device, $state->{'slots'}->{$slot}->{'barcode'});
+
+	# mark this as reserved
+	$state->{'drives'}->{$drive}->{'res_info'} = $self->_res_info_new();
+
         return $params{'res_cb'}->(undef, $res);
     };
 }
@@ -858,6 +870,8 @@ sub info_key {
 	$self->info_key_vendor_string(%params);
     } elsif ($key eq 'num_slots') {
 	$self->info_key_num_slots(%params);
+    } elsif ($key eq 'slots') {
+	$self->info_key_slots(%params);
     }
 }
 
@@ -908,6 +922,25 @@ sub info_key_num_slots_unlocked {
     $params{'info_cb'}->(undef, num_slots => scalar @allowed_slots);
 }
 
+sub info_key_slots {
+    my $self = shift;
+    my %params = @_;
+
+    $self->_with_updated_state(\%params, 'info_cb',
+	sub { $self->info_key_slots_unlocked(@_) });
+}
+
+sub info_key_slots_unlocked {
+    my $self = shift;
+    my %params = @_;
+    my $state = $params{'state'};
+
+    my @allowed_slots = sort {$a <=> $b} grep { $self->_is_slot_allowed($_) }
+			keys %{$state->{'slots'}};
+
+    $params{'info_cb'}->(undef, slots => \@allowed_slots);
+}
+
 sub get_interface { # (overridden by subclasses)
     my $self = shift;
     my ($device_name, $ignore_barcodes) = @_;
@@ -943,7 +976,7 @@ sub get_device { # (overridden by subclasses)
 		message => "opening '$device_name': " . $device->error_or_status());
     }
 
-    if (my $err = $self->{'config'}->configure_device($device)) {
+    if (my $err = $self->{'config'}->configure_device($device, $self->{'storage'})) {
 	return Amanda::Changer->make_error("fatal", undef,
 		reason => "device",
 		message => $err);
@@ -1003,6 +1036,39 @@ sub _set_label_unlocked {
     $params{'finished_cb'}->(undef);
 }
 
+sub _set_device_error {
+    my $self = shift;
+    my %params = @_;
+
+    return if $self->check_error($params{'finished_cb'});
+
+    $self->_with_updated_state(\%params, 'finished_cb',
+	sub { $self->_set_device_error_unlocked(@_); });
+}
+
+sub _set_device_error_unlocked {
+    my $self = shift;
+    my %params = @_;
+    my $state = $params{'state'};
+
+    # update all of the various pieces of cached information
+    my $drive = $params{'drive'};
+    my $slot = $state->{'drives'}->{$drive}->{'orig_slot'};
+    my $dev = $params{dev};
+
+    if (defined $slot) {
+	$state->{'slots'}->{$slot}->{'state'} = Amanda::Changer::SLOT_FULL;
+	$state->{'slots'}->{$slot}->{'device_status'} = "".$dev->status;
+	if ($dev->status != $DEVICE_STATUS_SUCCESS) {
+	    $state->{'slots'}->{$slot}->{'device_error'} = $dev->error;
+	} else {
+	    $state->{'slots'}->{$slot}->{'device_error'} = undef;
+	}
+    }
+
+    $params{'finished_cb'}->(undef);
+}
+
 sub _release {
     my $self = shift;
     my %params = @_;
@@ -1055,7 +1121,7 @@ sub reset_unlocked {
     my %params = @_;
     my $state = $params{'state'};
 
-    $state->{'current_slot'}{get_config_name()} = $self->_get_next_slot($state, -1);
+    $self->_set_current(state => $state, slot => $self->_get_next_slot($state, -1));
 
     $params{'finished_cb'}->();
 }
@@ -1221,7 +1287,7 @@ sub update_unlocked {
 
     step handle_assignment => sub {
 	# check for the SL=LABEL format, and handle it here
-	if (exists $params{'changed'} and $params{'changed'} =~ /^\d+=\S+$/) {
+	if (defined $params{'changed'} and $params{'changed'} =~ /^\d+=\S+$/) {
 	    my ($slot, $label) = ($params{'changed'} =~ /^(\d+)=(\S+)$/);
 
 	    # let's list the reasons we *can't* do what the user has asked
@@ -1242,7 +1308,13 @@ sub update_unlocked {
 			reason => "unknown", message => $whynot);
 	    }
 
-	    $user_msg_fn->("recoding volume '$label' in slot $slot");
+	    $user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code  => 1100020,
+			severity => $Amanda::Message::INFO,
+			slot  => $slot,
+			label => $label));
 	    # ok, now erase all knowledge of that label
 	    while (my ($bc, $lb) = each %{$state->{'bc2lb'}}) {
 		if ($lb eq $label) {
@@ -1270,11 +1342,23 @@ sub update_unlocked {
 
 	    # that's it -- no changer motion required
 	    return $params{'finished_cb'}->(undef);
-	} elsif (exists $params{'changed'} and
+	} elsif (defined $params{'changed'} and
 		 $params{'changed'} =~ /^(.+)=$/) {
 	    $params{'changed'} = $1;
 	    $set_to_unknown = 1;
 	    $steps->{'calculate_slots'}->($steps->{'set_to_unknown'});
+	} elsif (!defined $params{'changed'}) {
+	    $steps->{'calculate_slots'}->($steps->{'update_slot'});
+	} elsif ($params{'changed'} and
+		 $params{'changed'} =~ /^.+=.+$/) {
+	    my ($slot, $label) = ($params{'changed'} =~ /^(.+)=(.+)$/);
+	    $user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code  => 1100068,
+			severity => $Amanda::Message::ERROR,
+			slot => $slot));
+	    return $params{'finished_cb'}->(undef);
 	} else {
 	    $steps->{'calculate_slots'}->($steps->{'update_slot'});
 	}
@@ -1282,22 +1366,44 @@ sub update_unlocked {
 
     step calculate_slots => sub {
 	my ($update_slot_cb) = shift @_;
-	if (exists $params{'changed'}) {
+	if (defined $params{'changed'}) {
 	    # parse the string just like use-slots, using a hash for uniqueness
 	    my %changed;
 	    for my $range (split ',', $params{'changed'}) {
 		if ($range eq 'error') {
+		    my $error_range;
 		    while (my ($sl, $inf) = each %{$state->{'slots'}}) {
 			if (defined $inf->{'device_status'} &&
 			    $inf->{'device_status'} != $DEVICE_STATUS_SUCCESS) {
+			    $error_range++;
 			    $changed{$sl} = undef;
 			}
 		    }
+
+		    if (!defined $error_range) {
+			$user_msg_fn->(Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code  => 1100070,
+				severity => $Amanda::Message::SUCCESS,
+				reason => "unknown",
+				slot => $range));
+		    }
 		} else {
-		   my ($first, $last) = ($range =~ /(\d+)(?:-(\d+))?/);
-		   $last = $first unless defined($last);
-		   for ($first .. $last) {
-			$changed{$_} = undef;
+		    my ($first, $last) = ($range =~ /(\d+)(?:-(\d+))?/);
+		    $last = $first unless defined($last);
+		    if (defined $first and
+			$first =~ /^\d+$/ and $last =~ /^\d+$/) {
+			for ($first .. $last) {
+			    $changed{$_} = undef;
+			}
+		    } else {
+			$user_msg_fn->(Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code  => ($range =~ /\-/ ? 1100069 : 1100068),
+				severity => $Amanda::Message::ERROR,
+				slot => $range));
 		    }
 		}
 	    }
@@ -1314,6 +1420,10 @@ sub update_unlocked {
 	@slots_to_check = grep { $state->{'slots'}->{$_}->{'state'} == Amanda::Changer::SLOT_FULL} @slots_to_check;
 	@slots_to_check = sort { $a <=> $b } @slots_to_check;
 
+	if (@slots_to_check == 0) {
+	    return $steps->{'done'}->() if (!@slots_to_check);
+	}
+
 	$update_slot_cb->();
     };
 
@@ -1321,7 +1431,12 @@ sub update_unlocked {
 	return $steps->{'done'}->() if (!@slots_to_check);
 
 	my $slot = shift @slots_to_check;
-	$user_msg_fn->("Removing entry for slot $slot");
+	$user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code  => 1100021,
+			severity => $Amanda::Message::SUCCESS,
+			slot  => $slot));
 	$state->{'slots'}->{$slot}->{'label'} = undef;
 	$state->{'slots'}->{$slot}->{'device_status'} = undef;
 	$state->{'slots'}->{$slot}->{'device_error'} = undef;
@@ -1332,7 +1447,7 @@ sub update_unlocked {
 	    $state->{'drives'}->{$drive}->{'state'} =
 					Amanda::Changer::SLOT_FULL;
 	}
-	$steps->{'set_to_unknown'}->();
+	return $steps->{'set_to_unknown'}->();
     };
 
     # TODO: parallelize this if multiple drives are available
@@ -1341,7 +1456,12 @@ sub update_unlocked {
 	return $steps->{'done'}->() if (!@slots_to_check);
 
 	my $slot = shift @slots_to_check;
-	$user_msg_fn->("scanning slot $slot");
+	$user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code => 1100019,
+			severity => $Amanda::Message::INFO,
+			slot => $slot));
 
 	$self->load_unlocked(
 		slot => $slot,
@@ -1352,11 +1472,34 @@ sub update_unlocked {
     step slot_loaded => sub {
 	my ($err, $res) = @_;
 	if ($err) {
-	    return $params{'finished_cb'}->($err);
+	    $user_msg_fn->($err);
+	    return $steps->{'update_slot'}->();
 	}
 
 	# load() already fixed up the metadata, so just release; but we have to
 	# be careful to do an unlocked release.
+	my $slot = $res->{'this_slot'};
+	my $dev = $res->{device};
+	if ($dev->status() == $DEVICE_STATUS_SUCCESS) {
+	    my $label = $dev->volume_label;
+	    $user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code  => 1100020,
+			severity => $Amanda::Message::SUCCESS,
+			slot  => $slot,
+			label => $label));
+	} else {
+	    my $status = $dev->error_or_status;
+	    $user_msg_fn->(Amanda::Changer::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code  => 1100023,
+			severity => $Amanda::Message::SUCCESS,
+			slot  => $slot,
+			dev_status => $status));
+	}
+
 	$res->release(
 	    finished_cb => $steps->{'released'},
 	    unlocked => 1,
@@ -1420,7 +1563,7 @@ sub inventory_unlocked {
 	    if $slot->{'ie'};
 
 	$i->{'current'} = 1
-	    if $slot_name eq $state->{'current_slot'}{get_config_name()};
+	    if $slot_name eq $self->_get_current(state => $state);
 
 	push @inv, $i;
     }
@@ -1707,18 +1850,43 @@ sub verify_unlocked {
 
 	if ($device->status & $DEVICE_STATUS_VOLUME_MISSING) {
 	    debug("ERROR: Drive $drive is not device $device_name");
-	    push @results, "ERROR: Drive $drive is not device $device_name";
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100009,
+				severity => $Amanda::Message::WARNING,
+				drive => $drive,
+				device_name => $device_name);
 	} elsif ($device->status == $DEVICE_STATUS_DEVICE_ERROR) {
 	    debug("ERROR: Drive $drive: " . $device->error());
-	    push @results, "ERROR: Drive $drive: " . $device->error();
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100025,
+				severity => $Amanda::Message::INFO,
+				drive => $drive,
+				device_name => $device_name,
+				error => $device->error());
 	} else {
 	    debug("GOOD : Drive $drive is device $device_name");
-	    push @results, "GOOD : Drive $drive is device $device_name";
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100006,
+				severity => $Amanda::Message::INFO,
+				drive => $drive,
+				device_name => $device_name);
 	    push @tape_devices, "$drive=$device_name";
 	}
 	if ($hint) {
 	    debug("HINT : Drive $drive looks to be device $device_name2");
-	    push @results, "HINT : Drive $drive looks to be device $device_name2";
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100007,
+				severity => $Amanda::Message::INFO,
+				drive => $drive,
+				device_name => $device_name2);
 	    push @tape_devices, "$drive=$device_name2";
 	}
 
@@ -1761,7 +1929,12 @@ sub verify_unlocked {
 	for my $drive (@{$self->{'driveorder'}}) {
 	    if (!exists $state->{'drives'}->{$drive}) {
 		debug("ERROR: Drive $drive: no such drive in changer");
-		push @results, "ERROR: Drive $drive: no such drive in changer";
+		push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100024,
+				severity => $Amanda::Message::WARNING,
+				drive => $drive);
 	    }
 	}
 
@@ -1769,9 +1942,18 @@ sub verify_unlocked {
 	    $tape_devices .= " \"$tape_device\"";
 	}
 	if (defined $tape_devices) {
-	    push @results, "property \"TAPE-DEVICE\"$tape_devices";
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100008,
+				severity => $Amanda::Message::INFO,
+				tape_devices => $tape_devices);
 	} else {
-	    push @results, "ERROR: Found no valid tape device";
+	    push @results, Amanda::Changer::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1100065,
+				severity => $Amanda::Message::ERROR);
 	}
 	$params{'finished_cb'}->(undef, @results);
     };
@@ -1906,8 +2088,6 @@ sub _with_updated_state {
 	}
 
 	$params{'state'} = $state;
-	# keep a copy of the latest state
-	$self->{'state'} = $state;
 	# finally, call through to the user's method; $params{$cbname} has been
 	# properly patched to release the state lock when this method is done.
 	$sub->(%params);
@@ -1931,13 +2111,14 @@ sub _get_state {
 	    $state->{'drives'} = {};
 	    $state->{'drive_lru'} = [];
 	    $state->{'bc2lb'} = {};
-	    $state->{'current_slot'}{get_config_name()} = -1;
+	    $self->_set_current(state => $state, slot => -1);
 	}
 
 	if (defined $state->{'current_slot'} &&
 	    ref(\$state->{'current_slot'}) eq "SCALAR") {
 	    my $current_slot = $state->{'current_slot'};
-	    $state->{'current_slot'} = {get_config_name() => $current_slot};
+	    delete $state->{'current_slot'};
+	    $self->_set_current(state => $state, slot => $current_slot);
 	}
 
 	# this is for testing ONLY!
@@ -2014,7 +2195,7 @@ sub _get_state {
 		    debug("MISMATCH label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
 		}
 		if (!defined $label && defined $tl_label) {
-		    if (!defined $info->{'state'} ||
+		    if (defined $info->{'state'} and
 			$info->{'state'} == Amanda::Changer::SLOT_UNKNOWN) {
 			$label = $tl_label;
 			$state->{'bc2lb'}->{$info->{'barcode'}} = $tl_label;
@@ -2207,9 +2388,9 @@ sub _get_state {
 	    }
 	}
 
-	if (!defined $state->{'current_slot'}{get_config_name()} ||
-	    $state->{'current_slot'}{get_config_name()} == -1) {
-	    $state->{'current_slot'}{get_config_name()} = $self->_get_next_slot($state, -1);
+	my $current_slot = $self->_get_current(state => $state);
+	if (!defined $current_slot || $current_slot == -1) {
+	    $self->_set_current(state => $state, slot => $self->_get_next_slot($state, -1));
 	}
 
 	$steps->{'done'}->();
@@ -2303,6 +2484,15 @@ sub set_label {
 			       dev => $self->{device}, %params);
 }
 
+sub set_device_error {
+    my $self = shift;
+    my %params = @_;
+
+    return unless $self->{'chg'};
+    $self->{'chg'}->_set_device_error(drive => $self->{'drive'},
+			       dev => $self->{device}, %params);
+}
+
 package Amanda::Changer::robot::Interface;
 
 # The physical interface to the changer is abstracted out to allow several
@@ -2354,6 +2544,7 @@ package Amanda::Changer::robot::Interface;
 
 package Amanda::Changer::robot::Interface::MTX;
 
+use Carp;
 use Amanda::Paths;
 use Amanda::Config qw( :getconf );
 use Amanda::Debug qw( debug warning );
@@ -2429,6 +2620,7 @@ sub status {
 	    if ($exitstatus != 0) {
 		my $err = $output;
 		for my $line (split '\n', $output) {
+		    $line =~ s/\s*$//g;
 		    debug("mtx: $line");
 		}
 		# if it's a regular SCSI error, just show the sense key
@@ -2443,6 +2635,7 @@ sub status {
 	    } else {
 		my %status;
 		for my $line (split '\n', $output) {
+		    $line =~ s/\s*$//g;
 		    debug("mtx: $line");
 		    my ($slot, $ie, $slinfo);
 
