@@ -1,4 +1,5 @@
-# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
 # Contact information: Zmanda Inc, 505 N Mathlida Ave, Suite 120
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
 package Amanda::DB::Catalog;
 
@@ -115,6 +116,21 @@ have an associated dump row.
 (integer) -- size (in kb) of the complete dump (before compression or encryption); undef
 if not available
 
+=item native_crc
+
+(string) -- CRC of the complete dump (before compression or encryption); undef
+if not available
+
+=item client_crc
+
+(string) -- CRC of the complete dump (after client compression or client encryption); undef
+if not available
+
+=item server_crc
+
+(string) -- CRC of the complete dump (after server compression or server encryption); undef
+if not available
+
 =item sec
 
 (integer) -- time (in seconds) spent writing this part
@@ -452,7 +468,7 @@ sub get_latest_write_timestamp {
 
     if (@timestamps) {
 	# if we're not looking for a particular type, then this is easy
-	if (!exists $params{'types'}) {
+	if (!defined $params{'types'}) {
 	    return $timestamps[-1];
 	}
 
@@ -508,18 +524,20 @@ sub get_parts_and_dumps {
 
     # pre-process params by appending all of the "singular" parameters to the "plurals"
     push @{$params{'write_timestamps'}}, map { zeropad($_) } $params{'write_timestamp'} 
-	if exists($params{'write_timestamp'});
+	if defined($params{'write_timestamp'});
     push @{$params{'dump_timestamps'}}, map { zeropad($_) } $params{'dump_timestamp'} 
-	if exists($params{'dump_timestamp'});
+	if defined($params{'dump_timestamp'});
     push @{$params{'hostnames'}}, $params{'hostname'} 
-	if exists($params{'hostname'});
+	if defined($params{'hostname'});
     push @{$params{'disknames'}}, $params{'diskname'} 
-	if exists($params{'diskname'});
+	if defined($params{'diskname'});
     push @{$params{'levels'}}, $params{'level'} 
-	if exists($params{'level'});
+	if defined($params{'level'});
+    push @{$params{'storages'}}, $params{'storage'}
+	if defined($params{'storage'});
     if ($get_what eq 'parts') {
-	push @{$params{'labels'}}, $params{'label'} 
-	    if exists($params{'label'});
+	push @{$params{'labels'}}, $params{'label'}
+	    if defined($params{'label'});
     } else {
 	delete $params{'labels'};
     }
@@ -544,7 +562,7 @@ sub get_parts_and_dumps {
     my @logfiles;
     if ($params{'holding'}) {
 	@logfiles = ( 'holding', );
-    } elsif (exists($params{'write_timestamps'})) {
+    } elsif (defined($params{'write_timestamps'})) {
 	# if we have specific write_timestamps, the job is pretty easy.
 	my %timestamps_hash = map { ($_, undef) } @{$params{'write_timestamps'}};
 	for my $logfile (Amanda::Logfile::find_log()) {
@@ -552,7 +570,7 @@ sub get_parts_and_dumps {
 	    next unless (exists($timestamps_hash{zeropad($timestamp)}));
 	    push @logfiles, $logfile;
 	}
-    } elsif (exists($params{'dump_timestamps'})) {
+    } elsif (defined($params{'dump_timestamps'})) {
 	# otherwise, we need only look in logfiles at or after the earliest dump timestamp
 	my @sorted_timestamps = sort @{$params{'dump_timestamps'}};
 	my $earliest_timestamp = $sorted_timestamps[0];
@@ -567,23 +585,25 @@ sub get_parts_and_dumps {
     }
 
     # Set up some hash tables for speedy lookups of various attributes
-    my (%dump_timestamps_hash, %hostnames_hash, %disknames_hash, %levels_hash, %labels_hash);
+    my (%dump_timestamps_hash, %hostnames_hash, %disknames_hash, %levels_hash, %storages_hash, %labels_hash);
     %dump_timestamps_hash = map { ($_, undef) } @{$params{'dump_timestamps'}}
-	if (exists($params{'dump_timestamps'}));
+	if (defined($params{'dump_timestamps'}));
     %hostnames_hash = map { ($_, undef) } @{$params{'hostnames'}}
-	if (exists($params{'hostnames'}));
+	if (defined($params{'hostnames'}));
     %disknames_hash = map { ($_, undef) } @{$params{'disknames'}}
-	if (exists($params{'disknames'}));
+	if (defined($params{'disknames'}));
     %levels_hash = map { ($_, undef) } @{$params{'levels'}}
-	if (exists($params{'levels'}));
+	if (defined($params{'levels'}));
+    %storages_hash = map { ($_, undef) } @{$params{'storages'}}
+	if (defined($params{'storages'}));
     %labels_hash = map { ($_, undef) } @{$params{'labels'}}
-	if (exists($params{'labels'}));
+	if (defined($params{'labels'}));
 
     my %dumps;
     my @parts;
 
     # *also* scan holding if the holding param wasn't specified
-    if (!exists $params{'holding'}) {
+    if (!defined $params{'holding'}) {
 	push @logfiles, 'holding';
     }
 
@@ -596,21 +616,21 @@ sub get_parts_and_dumps {
 	# $logfile is undef
 	if ($logfile ne 'holding') {
 	    @find_results = Amanda::Logfile::search_logfile(undef, undef,
-							"$logfile_dir/$logfile", 1);
+							"$logfile_dir/$logfile", 1, 1);
 	    # convert to dumpfile hashes, including the write_timestamp from the logfile name
 	    my ($timestamp) = $logfile =~ /^log\.([0-9]+)(?:\.[0-9]+|\.amflush)?$/;
 	    $write_timestamp = zeropad($timestamp);
 
 	} else {
-	    @find_results = Amanda::Logfile::search_holding_disk();
+	    @find_results = Amanda::Logfile::search_holding_disk(1);
 	    $write_timestamp = '00000000000000';
 	}
 
 	# filter against *_match with dumps_match
 	@find_results = Amanda::Logfile::dumps_match([@find_results],
-	    exists($params{'hostname_match'})? $params{'hostname_match'} : undef,
-	    exists($params{'diskname_match'})? $params{'diskname_match'} : undef,
-	    exists($params{'dump_timestamp_match'})? $params{'dump_timestamp_match'} : undef,
+	    defined($params{'hostname_match'})? $params{'hostname_match'} : undef,
+	    defined($params{'diskname_match'})? $params{'diskname_match'} : undef,
+	    defined($params{'dump_timestamp_match'})? $params{'dump_timestamp_match'} : undef,
 	    undef,
 	    0);
 
@@ -621,22 +641,28 @@ sub get_parts_and_dumps {
 	    next unless (defined $find_result->{'label'});
 
 	    # bail out on this result early, if possible
-	    next if (%dump_timestamps_hash 
+	    next if (%dump_timestamps_hash
 		and !exists($dump_timestamps_hash{zeropad($find_result->{'timestamp'})}));
-	    next if (%hostnames_hash 
+	    next if (%hostnames_hash
 		and !exists($hostnames_hash{$find_result->{'hostname'}}));
-	    next if (%disknames_hash 
+	    next if (%disknames_hash
 		and !exists($disknames_hash{$find_result->{'diskname'}}));
-	    next if (%levels_hash 
+	    next if (%levels_hash
 		and !exists($levels_hash{$find_result->{'level'}}));
-	    next if (%labels_hash 
+	    next if (%storages_hash
+		and !exists($storages_hash{$find_result->{'storage'}}));
+	    next if (%labels_hash
 		and !exists($labels_hash{$find_result->{'label'}}));
 	    next if (defined $params{'labelstr'}
 		and !match_labelstr_expr($params{'labelstr'},$find_result->{'label'}));
 	    if ($get_what eq 'parts') {
-		next if (exists($params{'status'}) 
+		next if (exists($params{'status'}) and defined $params{'status'}
 		    and defined $find_result->{'status'}
 		    and $find_result->{'status'} ne $params{'status'});
+	    } elsif ($get_what eq 'dumps') {
+		next if (    exists($params{'status'}) and defined $params{'status'}
+			 and defined $find_result->{'dump_status'}
+			 and $find_result->{'dump_status'} ne $params{'status'});
 	    }
 
 	    # filter each result against dumpspecs, to avoid dumps_match_dumpspecs'
@@ -648,6 +674,7 @@ sub get_parts_and_dumps {
 	    my $dump_timestamp = zeropad($find_result->{'timestamp'});
 
 	    my $dumpkey = join("\0", $find_result->{'hostname'}, $find_result->{'diskname'},
+				     $find_result->{'storage'},
 			             $write_timestamp, $find_result->{'level'}, $dump_timestamp);
 	    my $dump = $dumps{$dumpkey};
 	    if (!defined $dump) {
@@ -656,8 +683,13 @@ sub get_parts_and_dumps {
 		    write_timestamp => $write_timestamp,
 		    hostname => $find_result->{'hostname'},
 		    diskname => $find_result->{'diskname'},
+		    storage  => $find_result->{'storage'},
+		    pool => $find_result->{'pool'},
 		    level => $find_result->{'level'}+0,
 		    orig_kb => $find_result->{'orig_kb'},
+		    native_crc => $find_result->{'native_crc'},
+		    client_crc => $find_result->{'client_crc'},
+		    server_crc => $find_result->{'server_crc'},
 		    status => $find_result->{'dump_status'},
 		    message => $find_result->{'message'},
 		    # the rest of these params are unknown until we see a taper
@@ -668,6 +700,17 @@ sub get_parts_and_dumps {
 		    kb => -1,    # $find_result->{'kb'}
 		    sec => -1,   # $find_result->{'sec'}
 		};
+	    } elsif (defined $find_result->{'partnum'} and $find_result->{'partnum'} == 1) {
+		$dumps{$dumpkey}->{'orig_kb'} = $find_result->{'orig_kb'},
+		$dumps{$dumpkey}->{'native_crc'} = $find_result->{'native_crc'},
+		$dumps{$dumpkey}->{'client_crc'} = $find_result->{'client_crc'},
+		$dumps{$dumpkey}->{'server_crc'} = $find_result->{'server_crc'},
+		$dumps{$dumpkey}->{'status'} = $find_result->{'dump_status'};
+		$dumps{$dumpkey}->{'message'} = $find_result->{'message'};
+		$dumps{$dumpkey}->{'nparts'} = 0;
+		$dumps{$dumpkey}->{'bytes'} = -1;
+		$dumps{$dumpkey}->{'kb'} = -1;
+		$dumps{$dumpkey}->{'sec'} = -1;
 	    }
 
 	    # start setting up a part hash for this result
@@ -682,6 +725,9 @@ sub get_parts_and_dumps {
 		    sec => $find_result->{'sec'},
 		    kb => $find_result->{'kb'},
 		    orig_kb => $find_result->{'orig_kb'},
+		    native_crc => $find_result->{'native_crc'},
+		    client_crc => $find_result->{'client_crc'},
+		    server_crc => $find_result->{'server_crc'},
 		    partnum => $find_result->{'partnum'},
 		);
 	    } else {
@@ -693,10 +739,13 @@ sub get_parts_and_dumps {
 		    sec => 0.0,
 		    kb => $find_result->{'kb'},
 		    orig_kb => $find_result->{'orig_kb'},
+		    native_crc => $find_result->{'native_crc'},
+		    client_crc => $find_result->{'client_crc'},
+		    server_crc => $find_result->{'server_crc'},
 		    partnum => 1,
 		);
 		# and fix up the dump, too
-		$dump->{'status'} = $find_result->{'status'} || 'FAILED';
+		$dump->{'status'} = $find_result->{'dump_status'} || 'FAILED';
 		$dump->{'bytes'} = $find_result->{'bytes'};
 		$dump->{'kb'} = $find_result->{'kb'};
 		$dump->{'sec'} = $find_result->{'sec'};
@@ -708,7 +757,7 @@ sub get_parts_and_dumps {
 
 	    # count the number of successful parts in the dump
 	    $dump->{'nparts'}++ if $part{'status'} eq 'OK';
-	    
+
 	    # and add a ref to the array of parts; if we're getting
 	    # parts, then this is a weak ref
 	    $dump->{'parts'}[$part{'partnum'}] = \%part;
@@ -742,8 +791,25 @@ sub get_parts_and_dumps {
 
 	    # now extract the appropriate info; luckily these log lines have the same
 	    # format, more or less
-	    my ($hostname, $diskname, $dump_timestamp, $nparts, $level, $secs, $kb, $bytes, $message);
-	    ($hostname, $str) = Amanda::Util::skip_quoted_string($str);
+	    my ($storage, $pool, $hostname, $diskname, $dump_timestamp, $nparts, $level, $secs, $kb, $bytes, $message);
+	    ($storage, $str) = Amanda::Util::skip_quoted_string($str);
+	    $storage = Amanda::Util::unquote_string($storage);
+	    if ($storage =~ /^ST:/) {
+		$storage =~ s/^ST://;
+		($pool, $str) = Amanda::Util::skip_quoted_string($str);
+		$pool = Amanda::Util::unquote_string($pool);
+		if ($pool =~ /^POOL/) {
+		    $pool =~ s/^POOL//;
+		    ($hostname, $str) = Amanda::Util::skip_quoted_string($str);
+		} else {
+		    $hostname = $pool;
+		    $pool = Amanda::Config::get_config_name();
+		}
+	    } else {
+		$hostname = $storage;
+		$storage = Amanda::Config::get_config_name();
+		$pool = $storage;
+	    }
 	    ($diskname, $str) = Amanda::Util::skip_quoted_string($str);
 	    ($dump_timestamp, $str) = Amanda::Util::skip_quoted_string($str);
 	    if ($status ne 'FAIL' and $type != $L_SUCCESS) { # nparts is not in SUCCESS lines
@@ -753,12 +819,21 @@ sub get_parts_and_dumps {
 		} else { # nparts is not in all PARTIAL lines
 		    $nparts = 0;
 		}
-		
+
 	    } else {
 		$nparts = 0;
 	    }
 	    ($level, $str) = Amanda::Util::skip_quoted_string($str);
 	    if ($status ne 'FAIL') {
+		if ($str !~ /^\[sec/) {
+		    (my $crc1, $str) = Amanda::Util::skip_quoted_string($str);
+		}
+		if ($str !~ /^\[sec/) {
+		    (my $crc2, $str) = Amanda::Util::skip_quoted_string($str);
+		}
+		if ($str !~ /^\[sec/) {
+		    (my $crc3, $str) = Amanda::Util::skip_quoted_string($str);
+		}
 		my $s = $str;
 		my $b_unit;
 		($secs, $b_unit, $kb, $str) = ($str =~ /^\[sec ([-0-9.]+) (kb|bytes) ([-0-9]+).*\] ?(.*)$/)
@@ -784,21 +859,25 @@ sub get_parts_and_dumps {
 	    # filter against dump criteria
 	    next if ($params{'dump_timestamp_match'}
 		and !match_datestamp($params{'dump_timestamp_match'}, zeropad($dump_timestamp)));
-	    next if (%dump_timestamps_hash 
+	    next if (%dump_timestamps_hash
 		and !exists($dump_timestamps_hash{zeropad($dump_timestamp)}));
 
 	    next if ($params{'hostname_match'}
 		and !match_host($params{'hostname_match'}, $hostname));
-	    next if (%hostnames_hash 
+	    next if (%hostnames_hash
 		and !exists($hostnames_hash{$hostname}));
 
 	    next if ($params{'diskname_match'}
 		and !match_disk($params{'diskname_match'}, $diskname));
-	    next if (%disknames_hash 
+	    next if (%disknames_hash
 		and !exists($disknames_hash{$diskname}));
 
-	    next if (%levels_hash 
+	    next if (%levels_hash
 		and !exists($levels_hash{$level}));
+
+	    next if (%storages_hash
+		and !exists($storages_hash{$storage}));
+
 	    # get_dumps filters on status
 
 	    if ($params{'dumpspecs'}) {
@@ -822,7 +901,7 @@ sub get_parts_and_dumps {
 		next unless $ok;
 	    }
 
-	    my $dumpkey = join("\0", $hostname, $diskname, $write_timestamp,
+	    my $dumpkey = join("\0", $hostname, $diskname, $storage, $write_timestamp,
 				     $level, zeropad($dump_timestamp));
 	    my $dump = $dumps{$dumpkey};
 	    if (!defined $dump) {
@@ -832,8 +911,12 @@ sub get_parts_and_dumps {
 		    write_timestamp => $write_timestamp,
 		    hostname => $hostname,
 		    diskname => $diskname,
+		    storage  => $storage,
 		    level => $level+0,
 		    orig_kb => undef,
+		    native_crc => undef,
+		    client_crc => undef,
+		    server_crc => undef,
 		    status => "FAILED",
 		    # message set below
 		    nparts => $nparts, # hopefully 0?
@@ -869,7 +952,7 @@ sub get_dumps {
     my ($dumps, $parts) = get_parts_and_dumps("dumps", @_);
     my @dumps = @$dumps;
 
-    if (exists $params{'status'}) {
+    if (exists $params{'status'} and defined $params{'status'}) {
 	@dumps = grep { $_->{'status'} eq $params{'status'} } @dumps;
     }
 
@@ -965,7 +1048,7 @@ sub add_part {
 	    # write timestamp matches; now check the label
 	    LOGFILE_DUMP:
 	    for $find_result (Amanda::Logfile::search_logfile(undef, undef,
-					"$logdir/$lf", 1)) {
+					"$logdir/$lf", 1, 1)) {
 		next unless (defined $find_result->{'label'});
 
 		if ($find_result->{'label'} eq $dump->{'label'}) {
@@ -996,7 +1079,7 @@ sub add_part {
 	# package is rewritten to parse logfiles on its own (or access a relational
 	# database), this implementation detail will no longer be relevant.
 	my @find_results = reverse Amanda::Logfile::search_logfile(undef, undef,
-						    "$logdir/$logfile", 1);
+						    "$logdir/$logfile", 1, 1);
 	for $find_result (@find_results) {
 	    # filter out the non-dump error messages that find.c produces
 	    next unless (defined $find_result->{'label'});
@@ -1047,7 +1130,7 @@ sub add_part {
 		    and zeropad($_->{'datestamp'}) eq zeropad($dump->{'write_timestamp'})
 		} @{$tapelist->{tles}}) {
 	    $tapelist->reload(1);
-	    $tapelist->add_tapelabel($write_timestamp, $dump->{'label'}, undef, 1);
+	    $tapelist->add_tapelabel($write_timestamp, $dump->{'label'}, undef, 1,undef,undef,undef,undef,undef, Amanda::Config::get_config_name());
 	    $tapelist->write();
 	}
     }
@@ -1092,10 +1175,10 @@ sub add_part {
 }
 
 sub _load_tapelist {
-    if (!defined $tapelist) {
+#    if (!defined $tapelist) {
 	my $tapelist_filename = config_dir_relative(getconf($CNF_TAPELIST));
-	$tapelist = Amanda::Tapelist->new($tapelist_filename);
-    }
+	($tapelist, my $message) = Amanda::Tapelist->new($tapelist_filename);
+#    }
 }
 
 sub _clear_cache { # (used by installcheck)
