@@ -1,4 +1,5 @@
 # Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,9 +15,34 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
+package Amanda::Process::Message;
+use strict;
+use warnings;
+
+use Amanda::Message;
+use vars qw( @ISA );
+@ISA = qw( Amanda::Message );
+
+sub local_message {
+    my $self = shift;
+
+    if ($self->{'code'} == 3500000) {
+	return "pid $self->{'pid'} doesn't match: $self->{'pname'} != $self->{'ps_pname'}";
+    } elsif ($self->{'code'} == 3500001) {
+	return "Sending $self->{'signal'} signal to process '$self->{'process'}' (pid $self->{'pid'})";
+    } elsif ($self->{'code'} == 3500002) {
+	return "pid $self->{'pid'} is done";
+    } elsif ($self->{'code'} == 3500003) {
+	return "pid $self->{'pid'} is dead";
+    } elsif ($self->{'code'} == 3500004) {
+	return "Can't open trace_log file '$self->{'trace_log'}': $self->{'errnostr'}";
+    } else {
+	return "no message for code $self->{'code'}";
+    }
+}
 package Amanda::Process;
 
 use strict;
@@ -27,6 +53,8 @@ use Exporter;
 use vars qw( @ISA @EXPORT_OK );
 use File::Basename;
 use Amanda::Constants;
+use Amanda::Debug qw( debug );
+use Amanda::Config qw( :getconf config_dir_relative );
 @ISA = qw( Exporter );
 
 =head1 NAME
@@ -126,11 +154,13 @@ Return 1 if the process is still alive.
 sub new {
     my $class = shift;
     my ($verbose) = shift;
+    my ($user_message) = shift;
 
     my $self = {
 	verbose => $verbose,
 	master_name => "",
 	master_pid => "",
+	user_message => $user_message,
 	pids => {},
 	pstable => {},
 	ppid => {},
@@ -178,15 +208,17 @@ sub load_ps_table() {
 	my $psline = <PSTABLE>; #header line
 	while($psline = <PSTABLE>) {
 	    chomp $psline;
-	    my ($pid, $ppid, $pname, $arg1, $arg2) = split " ", $psline;
+	    next if $psline =~ /\<defunct\>/;
+	    my ($pid, $ppid, $pname, @args) = split " ", $psline;
+	    next if !defined $pid;
+	    next if !defined $ppid;
+	    next if !defined $pname;
 	    $pname = basename($pname);
-	    if ($pname =~ /^perl/ && defined $arg1) {
-		if ($arg1 !~ /^\-/) {
-		    $pname = $arg1;
-		} elsif (defined $arg2) {
-		    if ($arg2 !~ /^\-/) {
-			$pname = $arg2;
-		    }
+	    if ($pname =~ /perl/ && @args) {
+		foreach my $arg (@args) {
+		    next if $arg =~ /^\-/;
+		    $pname = $arg;
+		    last;
 		}
 		$pname = basename($pname);
 	    }
@@ -220,7 +252,20 @@ sub scan_log($) {
     my $first = 1;
     my($line);
 
-    open(LOGFILE, "<", $logfile) || die("$logfile: $!");
+    if ($logfile !~ /^\//) {
+	$logfile = config_dir_relative(Amanda::Config::getconf($CNF_LOGDIR)) . '/' . $logfile;
+    }
+    if (!open(LOGFILE, "<", $logfile)) {
+	$self->{'user_message'}(Amanda::Process::Message->new(
+			source_filename	=> __FILE__,
+                        source_line	=> __LINE__,
+                        code		=> 3500004,
+                        severity	=> $Amanda::Message::ERROR,
+			trace_log	=> $logfile,
+			errno		=> $!));
+	return;
+    }
+
     while($line = <LOGFILE>) {
 	if ($line =~ /^INFO .* (.*) pid (\d*)$/) {
 	    my ($pname, $pid) = ($1, $2);
@@ -231,16 +276,31 @@ sub scan_log($) {
 	    }
 	    if (defined $self->{pstable}->{$pid} && $pname eq $self->{pstable}->{$pid}) {
 		$self->{pids}->{$pid} = $pname;
-	    } elsif (defined $self->{pstable}->{$pid} && $self->{pstable}->{$pid} =~ /^perl/) {
+	    } elsif (defined $self->{pstable}->{$pid} && $self->{pstable}->{$pid} =~ /perl/) {
 		# We can get 'perl' for a perl script.
 		$self->{pids}->{$pid} = $pname;
+	    } elsif (defined $self->{pstable}->{$pid} && $self->{pstable}->{$pid} =~ /^$pname\d*$/) {
+		$self->{pids}->{$pid} = $pname;
+	    } elsif (defined $self->{pstable}->{$pid} && $self->{pstable}->{$pid} =~ /^am$pname\d*$/) {
+		$self->{pids}->{$pid} = $pname;
 	    } elsif (defined $self->{pstable}->{$pid}) {
-		print "pid $pid doesn't match: ", $pname, " != ", $self->{pstable}->{$pid}, "\n" if $self->{verbose};
+		$self->{'user_message'}(Amanda::Process::Message->new(
+			source_filename	=> __FILE__,
+                        source_line	=> __LINE__,
+                        code		=> 3500000,
+                        severity	=> $Amanda::Message::WARNING,
+			pname		=> $pname,
+			ps_pname	=> $self->{pstable}->{$pid},
+			pid		=> $pid)) if $self->{verbose};;
 	    }
 	} elsif ($line =~ /^INFO .* pid-done (\d*)$/) {
 	    my $pid = $1;
-	    print "pid $pid is done\n" if $self->{verbose};
-	    delete $self->{pids}->{$pid};
+	    $self->{'user_message'}(Amanda::Process::Message->new(
+			source_filename	=> __FILE__,
+                        source_line	=> __LINE__,
+                        code		=> 3500002,
+                        severity	=> $Amanda::Message::INFO,
+			pid		=> $pid)) if $self->{verbose};;
 	}
     }
     close(LOGFILE);
@@ -249,7 +309,12 @@ sub scan_log($) {
     if ($self->{verbose}) {
 	for my $pid (keys %{$self->{pids}}) {
 	    if (!defined $self->{pstable}->{$pid}) {
-		print "pid $pid is dead\n";
+	        $self->{'user_message'}(Amanda::Process::Message->new(
+			source_filename	=> __FILE__,
+                        source_line	=> __LINE__,
+                        code		=> 3500003,
+                        severity	=> $Amanda::Message::INFO,
+			pid		=> $pid));
 	    }
 	}
     }
@@ -359,7 +424,14 @@ sub kill_process($) {
     my $signal = shift;
 
     foreach my $pid (keys %{$self->{amprocess}}) {
-	print "Sendding $signal signal to pid $pid\n" if $self->{verbose};
+	$self->{'user_message'}(Amanda::Process::Message->new(
+			source_filename	=> __FILE__,
+                        source_line	=> __LINE__,
+                        code		=> 3500001,
+                        severity	=> $Amanda::Message::INFO,
+			signal		=> $signal,
+			process		=> $self->{'pids'}->{$pid},
+			pid		=> $pid));
 	kill $signal, $pid;
     }
 }
@@ -386,6 +458,22 @@ sub process_running() {
     return $count;
 }
 
+sub which_process_running() {
+    my $self = shift;
+
+    $self->load_ps_table();
+    my $count = 0;
+    my @pids;
+    foreach my $pid (keys %{$self->{amprocess}}) {
+	if (defined $self->{pstable}->{$pid}) {
+	    $count++;
+	    push @pids, $pid;
+	}
+    }
+
+    return ( $count, \@pids );
+}
+
 # Count the number of processes in %amprocess.
 #
 # Prerequisites:
@@ -416,7 +504,8 @@ sub process_alive() {
     my $pname = shift;
 
     if (defined $pname && defined $self->{pstable}->{$pid}) {
-	return $self->{pstable}->{$pid} eq $pname;
+	return $self->{pstable}->{$pid} eq $pname ||
+	       $self->{pstable}->{$pid} eq "starman";
     } else {
 	return defined $self->{pstable}->{$pid};
     }
