@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,7 +15,7 @@
 # along with this library; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 =head1 NAME
@@ -29,7 +30,11 @@ Amanda::Recovery::Planner - use the catalog to plan recoveries
 	Amanda::Recovery::Planner::make_plan(
 	    dumpspecs => [ $ds1, $ds2 ],
 	    algorithm => $algo,
+	    labelstr => $labelstr,
+	    storage_list => \@storage_list,
 	    changer => $changer,
+	    all_copy => $all_copy,
+	    only_in_storage => $only_in_storage,
 	    plan_cb => $subs{'plan_cb'});
     };
 
@@ -153,7 +158,7 @@ performed.  Callers should shift dumps off this list to present to the Clerk.
 To get a list of volumes that the plan requires, in order, use
 C<get_volume_list>.  Each volume is represented as a hash:
 
-  { label => 'DATA182', available => 1 }
+  { label => 'DATA182', storage => 'STORAGE-1', pool => 'POOL-1', available => 1 }
 
 where C<available> is false if the planner did not find this volume in the
 changer.  Planners which do not consult the changer will have a false value for
@@ -179,9 +184,10 @@ sub make_plan {
 
     my $plan = Amanda::Recovery::Planner::Plan->new({
 	algo => $params{'algorithm'},
-	chg => $params{'changer'},
+	storage => $params{'storage'},
 	debug => $params{'debug'},
 	one_dump_per_part => $params{'one_dump_per_part'},
+	all_copy => $params{'all_copy'},
     });
 
     if (exists $params{'holding_file'}) {
@@ -229,171 +235,197 @@ sub make_plan {
     my $self = shift;
     my %params = @_;
 
-    for my $rq_param (qw(changer plan_cb dumpspecs)) {
-	croak "required parameter '$rq_param' mising"
+    for my $rq_param (qw(plan_cb )) {
+	croak "required parameter '$rq_param' missing"
 	    unless exists $params{$rq_param};
     }
+    my $status = $params{'status'};
+    my $dumpspecs = $params{'dumpspecs'};
+    my $hostname = $params{'hostname'};
+    my $diskname = $params{'diskname'};
+    my $dump_timestamp = $params{'dump_timestamp'};
+    my $level = $params{'level'};
+    my $src_labelstr = $params{'src_labelstr'};
 
-    my $steps = define_steps
-	cb_ref => \$params{'plan_cb'};
+    # first, get the set of dumps that match these dumpspecs
+    my @dumps = Amanda::DB::Catalog::get_dumps(dumpspecs => $dumpspecs,
+					       hostname => $hostname,
+					       diskname => $diskname,
+					       dump_timestamp => $dump_timestamp,
+					       level => $level,
+					       status => $status,
+					       labelstr  => $src_labelstr);
 
-    step get_inventory => sub {
-	if (defined $params{'changer'} and $params{'changer'}->have_inventory()) {
-	    return $params{'changer'}->inventory( inventory_cb => $steps->{'got_inventory'});
-	} else {
-	    return $steps->{'got_inventory'}->(undef, undef);
-	}
-    };
-    step got_inventory => sub {
-	my ($err, $inventory) = @_;
-
-	my $dumpspecs = $params{'dumpspecs'};
-
-	# first, get the set of dumps that match these dumpspecs
-	my @dumps = Amanda::DB::Catalog::get_dumps(dumpspecs => $dumpspecs);
-
-	# Create a hash of the latest datestamp of each dle.
-	my %datestamp;
-	if ($params{'latest_fulls'}) {
-	    for my $dump (@dumps) {
-		my $kd = join("\0", $dump->{'hostname'}, $dump->{'diskname'});
-		if (exists $datestamp{$kd}) {
-		    if ($dump->{'dump_timestamp'} > $datestamp{$kd}) {
-			$datestamp{$kd} = $dump->{'dump_timestamp'};
-		    }
-		} else {
+    # Create a hash of the latest datestamp of each dle.
+    my %datestamp;
+    if ($params{'latest_fulls'}) {
+	for my $dump (@dumps) {
+	    my $kd = join("\0", $dump->{'hostname'}, $dump->{'diskname'});
+	    if (exists $datestamp{$kd}) {
+		if ($dump->{'dump_timestamp'} > $datestamp{$kd}) {
 		    $datestamp{$kd} = $dump->{'dump_timestamp'};
 		}
-	    }
-	}
-
-	# now "bin" those by host/disk/dump_ts/level
-	my %dumps;
-	for my $dump (@dumps) {
-	    next if !defined $dump;
-	    my $k = join("\0", $dump->{'hostname'}, $dump->{'diskname'},
-			       $dump->{'dump_timestamp'}, $dump->{'level'});
-	    if ($params{'latest_fulls'}) {
-		my $kd = join("\0", $dump->{'hostname'}, $dump->{'diskname'});
-		next if defined $datestamp{$kd} and
-			        $datestamp{$kd} != $dump->{'dump_timestamp'};
-	    }
-	    $dumps{$k} = [] unless exists $dumps{$k};
-	    push @{$dumps{$k}}, $dump;
-	}
-
-	# now select the "best" of each set of dumps, and put that in @dumps
-	@dumps = ();
-	for my $options (values %dumps) {
-	    my @options = @$options;
-
-	    # remove dump without parts
-	    @options = grep { defined $_->{'parts'} } @options;
-
-	    if (@options == 0) {
-		next;
-	    }
-
-	    # if there's only one option, the choice is easy
-	    if (@options == 1) {
-		push @dumps, $options[0];
-		next;
-	    }
-
-	    # if there are several, narrow to those with an OK status or barring that,
-	    # those with a PARTIAL status.  FAIL need not apply.
-	    my @ok_options = grep { $_->{'status'} eq 'OK' } @options;
-	    my @partial_options = grep { $_->{'status'} eq 'PARTIAL' } @options;
-
-	    if (@ok_options) {
-		@options = @ok_options;
 	    } else {
-		@options = @partial_options;
+		$datestamp{$kd} = $dump->{'dump_timestamp'};
 	    }
+	}
+    }
 
-	    if (@options == 0) {
-		next;
-	    }
+    # now "bin" those by host/disk/dump_ts/level
+    my %dumps;
+    for my $dump (@dumps) {
+	next if !defined $dump;
+	my $k = join("\0", $dump->{'hostname'}, $dump->{'diskname'},
+			   $dump->{'dump_timestamp'}, $dump->{'level'});
+	if ($params{'latest_fulls'}) {
+	    my $kd = join("\0", $dump->{'hostname'}, $dump->{'diskname'});
+	    next if defined $datestamp{$kd} and
+		    $datestamp{$kd} != $dump->{'dump_timestamp'};
+	}
+	$dumps{$k} = [] unless exists $dumps{$k};
+	push @{$dumps{$k}}, $dump;
+    }
 
-	    # now, take the one written longest ago - this gets us the dump on secondary
-	    # media if it hasn't been overwritten, otherwise the dump on tertiary media,
-	    # etc.  Note that this also prefers dumps on holding disk, since they are
-	    # tagged with a write_timestamp of 0
-	    @options = Amanda::DB::Catalog::sort_dumps(['write_timestamp'], @options);
+    # all_copy  storage_list only_in_storage
+    #     0            0         0		first
+    #     0            0         1		-------
+    #     0            1         0		first in storage_list or first
+    #     0            1         1		first in storage_list
+    #     1            0         0		all
+    #     1            0         1		-------
+    #     1            1         0		all
+    #     1            1         1		all in storage_list
+    # now select the "best" of each set of dumps, and put that in @dumps
+    @dumps = ();
+    for my $options (values %dumps) {
+	my @options = @$options;
 
-	    my $found;
-	    for my $option (@options) {
-		foreach my $sl (@$inventory) {
-		    if(defined $sl->{'label'} and $sl->{'label'} eq $option->{'parts'}[1]{'label'}) {
-			push @dumps, $option;
-			$found = 1;
-		    }
+	# remove if not in a valid storage
+	if ($params{'storage_list'} && $params{'only_in_storage'}) {
+	    @options = grep { my $storage = $_->{'storage'};
+			      grep { $_ eq $storage } @{$params{'storage_list'}}
+			    } @options;
+	}
+
+	# remove dump without parts
+	@options = grep { defined $_->{'parts'} } @options;
+
+	if ($params{'all_copy'}) {
+	    push @dumps, @options;
+	    next;
+	}
+
+	# limit to the one in order of the storage_list
+	if ($params{'storage_list'}) {
+	    for my $storage (@{$params{'storage_list'}}) {
+		my @options1 = grep { $_->{'storage'} eq $storage } @options;
+		if (@options1) {
+		    @options = @options1;
+		    last;
 		}
 	    }
-	    if (!$found) {
-		push @dumps, $options[0];
-	    }
 	}
 
-	# at this point we have exactly one instance of each dump in @dumps.
-
-	# If one_dump_per_part was specified, rearrange @dumps to have a distinct
-	# dump object for each part.
-	if ($self->{'one_dump_per_part'}) {
-	    @dumps = $self->split_dumps_per_part(\@dumps);
+	if (@options == 0) {
+	    next;
 	}
 
-	# now sort the dumps in order by their constituent parts.  This sorts based
-	# on write_timestamp, then on the label of the first part of the dump,
-	# using the tapelist to order the labels.  Where labels match, it sorts on
-	# the part's filenum.  This should sort the dumps into the order in which
-	# they were written, with holding dumps coming in at the head of the list.
-	my $tapelist_filename = config_dir_relative(getconf($CNF_TAPELIST));
-	my $tapelist = Amanda::Tapelist->new($tapelist_filename);
+	# if there's only one option, the choice is easy
+	if (@options == 1) {
+	    push @dumps, $options[0];
+	    next;
+	}
 
-	my $sortfn = sub {
-	    my $rv;
-	    my $tle;
+	# if there are several, narrow to those with an OK status or barring that,
+	# those with a PARTIAL status.  FAIL need not apply.
+	my @ok_options = grep { $_->{'status'} eq 'OK' } @options;
+	my @partial_options = grep { $_->{'status'} eq 'PARTIAL' } @options;
 
-	    return $rv
-		if ($rv = $a->{'write_timestamp'} cmp $b->{'write_timestamp'});
+	if (@ok_options) {
+	    @options = @ok_options;
+	} else {
+	    @options = @partial_options;
+	}
 
-	    # above will take care of comparing a holding dump to an on-media dump, but
-	    # if both are on holding then we need to compare them lexically
-	    if (exists $a->{'parts'}[1]{'holding_file'}
-	    and exists $b->{'parts'}[1]{'holding_file'}) {
-		return $a->{'parts'}[1]{'holding_file'} cmp $b->{'parts'}[1]{'holding_file'};
+	# now, take the one written longest ago - this gets us the dump on secondary
+	# media if it hasn't been overwritten, otherwise the dump on tertiary media,
+	# etc.  Note that this also prefers dumps on holding disk, since they are
+	# tagged with a write_timestamp of 0
+	@options = Amanda::DB::Catalog::sort_dumps(['write_timestamp'], @options);
+	if ($options[0]->{'write_timestamp'} =~ /^0+$/) {
+	    # use holding disk if available
+	    push @dumps, $options[0];
+	} else {
+	    # find one that match the labelstr
+	    my $j = 0;
+	    for (my $i = 0; $i < @options; $i++) {
+		if ($options[$i]->{'parts'}[1]{'label'} =~ /^$params{'labelstr'}$/) {
+		    $j = $i;
+		    last;
+		}
 	    }
+	    push @dumps, $options[$j];
+	}
+    }
 
-	    my ($alabel, $blabel) = (
-		$a->{'parts'}[1]{'label'},
-		$b->{'parts'}[1]{'label'},
-	    );
+    # at this point we have exactly one instance of each dump in @dumps.
 
-	    my ($apos, $bpos);
-	    $apos = $tle->{'position'}
-		if (($tle = $tapelist->lookup_tapelabel($alabel)));
-	    $bpos = $tle->{'position'}
-		if (($tle = $tapelist->lookup_tapelabel($blabel)));
-	    return ($bpos <=> $apos) # not: reversed for "oldest to newest"
-		if defined $bpos && defined $apos && ($bpos <=> $apos);
+    # If one_dump_per_part was specified, rearrange @dumps to have a distinct
+    # dump object for each part.
+    if ($self->{'one_dump_per_part'}) {
+	@dumps = $self->split_dumps_per_part(\@dumps);
+    }
 
-	    # if a tape wasn't in the tapelist, just sort the labels lexically (this
-	    # really shouldn't happen)
-	    if (!defined $bpos || !defined $apos) {
-		return $alabel cmp $blabel
-		    if defined $alabel and defined $blabel and $alabel cmp $blabel ;
-	    }
+    # now sort the dumps in order by their constituent parts.  This sorts based
+    # on write_timestamp, then on the label of the first part of the dump,
+    # using the tapelist to order the labels.  Where labels match, it sorts on
+    # the part's filenum.  This should sort the dumps into the order in which
+    # they were written, with holding dumps coming in at the head of the list.
+    my $tapelist_filename = config_dir_relative(getconf($CNF_TAPELIST));
+    my ($tapelist, $message) = Amanda::Tapelist->new($tapelist_filename);
 
-	    # finally, the dumps are on the same volume, so just sort by filenum
-	    return $a->{'parts'}[1]{'filenum'} <=> $b->{'parts'}[1]{'filenum'};
-	};
-	@dumps = sort $sortfn @dumps;
+    my $sortfn = sub {
+	my $rv;
+	my $tle;
 
-	$self->{'dumps'} = \@dumps;
+	return $rv
+	    if ($rv = $a->{'write_timestamp'} cmp $b->{'write_timestamp'});
 
-	Amanda::MainLoop::call_later($params{'plan_cb'}, undef, $self);
+	# above will take care of comparing a holding dump to an on-media dump, but
+	# if both are on holding then we need to compare them lexically
+	if (exists $a->{'parts'}[1]{'holding_file'}
+	and exists $b->{'parts'}[1]{'holding_file'}) {
+	    return $a->{'parts'}[1]{'holding_file'} cmp $b->{'parts'}[1]{'holding_file'};
+	}
+
+	my ($alabel, $blabel) = (
+	    $a->{'parts'}[1]{'label'},
+	    $b->{'parts'}[1]{'label'},
+	);
+
+	my ($apos, $bpos);
+	$apos = $tle->{'position'}
+	    if (($tle = $tapelist->lookup_tapelabel($alabel)));
+	$bpos = $tle->{'position'}
+	    if (($tle = $tapelist->lookup_tapelabel($blabel)));
+	return ($bpos <=> $apos) # not: reversed for "oldest to newest"
+	    if defined $bpos && defined $apos && ($bpos <=> $apos);
+
+	# if a tape wasn't in the tapelist, just sort the labels lexically (this
+	# really shouldn't happen)
+	if (!defined $bpos || !defined $apos) {
+	    return $alabel cmp $blabel
+		if defined $alabel and defined $blabel and $alabel cmp $blabel ;
+	}
+
+	# finally, the dumps are on the same volume, so just sort by filenum
+	return $a->{'parts'}[1]{'filenum'} <=> $b->{'parts'}[1]{'filenum'};
     };
+    @dumps = sort $sortfn @dumps;
+
+    $self->{'dumps'} = \@dumps;
+
+    Amanda::MainLoop::call_later($params{'plan_cb'}, undef, $self);
 }
 
 sub make_holding_plan {
@@ -401,7 +433,7 @@ sub make_holding_plan {
     my %params = @_;
 
     for my $rq_param (qw(holding_file plan_cb)) {
-	croak "required parameter '$rq_param' mising"
+	croak "required parameter '$rq_param' missing"
 	    unless exists $params{$rq_param};
     }
 
@@ -452,7 +484,7 @@ sub make_plan_from_filelist {
     my %params = @_;
 
     for my $rq_param (qw(filelist plan_cb)) {
-	croak "required parameter '$rq_param' mising"
+	croak "required parameter '$rq_param' missing"
 	    unless exists $params{$rq_param};
     }
 
@@ -477,14 +509,16 @@ sub make_plan_from_filelist {
 	my @labels;
 	my %files;
 	my @filelist = @{$params{'filelist'}};
+	my $storage;
 	while (@filelist) {
+	    $storage = shift @filelist;
 	    my $label = shift @filelist;
 	    push @labels, $label;
 	    $files{$label} = shift @filelist;
 	}
-
 	my @parts = Amanda::DB::Catalog::get_parts(
 		$params{'dumpspec'}? (dumpspecs => [ $params{'dumpspec'} ]) : (),
+		storage => $storage,
 		labels => [ @labels ]);
 
 	# filter down to the parts that match filelist (using %files)
@@ -552,6 +586,9 @@ sub make_plan_from_filelist {
 	    }
 	}
 
+	if ($storage && $storage ne $dumps[0]->{'storage'}) {
+	    die("bad storage $storage : $dumps[0]->{'storage'}");
+	}
 	# now, because of the weak linking used by Amanda::DB::Catalog, we need to
 	# re-query for this dump.  If we don't do this, the parts will all be
 	# garbage-collected when we hand back the plan.  This is, chartiably, "less
@@ -563,6 +600,7 @@ sub make_plan_from_filelist {
 	    level => $dumps[0]->{'level'},
 	    dump_timestamp => $dumps[0]->{'dump_timestamp'},
 	    write_timestamp => $dumps[0]->{'write_timestamp'},
+	    storage => $dumps[0]->{'storage'},
 	    dumpspecs => $params{'dumpspecs'});
 
 	# sanity check
@@ -614,7 +652,7 @@ sub get_volume_list {
 	    next unless defined $part->{'label'}; # skip holding parts
 	    if (!defined $last_label || $part->{'label'} ne $last_label) {
 		$last_label = $part->{'label'};
-		push @volumes, { label => $last_label, available => 0 };
+		push @volumes, { label => $last_label, storage => $dump->{'storage'}, pool => $dump->{'pool'}, available => 0 };
 	    }
 	}
     }
