@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
 package Amanda::Recovery::Scan;
 
@@ -35,6 +36,7 @@ use Amanda::Debug qw( debug );
 use Amanda::Changer;
 use Amanda::MainLoop;
 use Amanda::Interactivity;
+use Amanda::Recovery;
 
 use constant SCAN_ASK  => 1;     # call Amanda::Interactivity module
 use constant SCAN_POLL => 2;     # wait 'poll_delay' and retry the scan.
@@ -140,7 +142,7 @@ Initiate the scan of the slot $slot:
 Initiate the scan of the slot $slot which should have the label $label:
   $self->user_msg_fn(scan_slot => 1,
                      slot      => $slot,
-                     label     => $label);   
+                     label     => $label);
 
 The result of scanning slot $slot:
   $self->user_msg_fn(slot_result => 1,
@@ -178,7 +180,7 @@ sub new {
 sub DESTROY {
     my $self = shift;
 
-    die("Recovery::Scan detroyed without quit") if defined $self->{'scan_conf'};
+    die("Recovery::Scan destroyed without quit") if defined $self->{'scan_conf'};
 }
 
 sub quit {
@@ -224,7 +226,7 @@ sub find_volume {
 	Amanda::Debug::debug("find_volume labeled '$label'");
 
 	$scan_running = 1;
-	$self->{'chg'}->inventory(inventory_cb => $steps->{'got_first_inventory'});
+	$self->{'chg'}->inventory(all_slots => 1, inventory_cb => $steps->{'got_first_inventory'});
     };
 
     step got_first_inventory => sub {
@@ -268,7 +270,7 @@ sub find_volume {
 	    Amanda::Debug::debug("update the changer");
 	    $self->{'chg'}->update();
 	}
-	$self->{'chg'}->inventory(inventory_cb => $steps->{'parse_inventory'});
+	$self->{'chg'}->inventory(all_slots => 1, inventory_cb => $steps->{'parse_inventory'});
     };
 
     step parse_inventory => sub {
@@ -296,13 +298,18 @@ sub find_volume {
 		    return $steps->{'handle_error'}->(
 			    Amanda::Changer::Error->new('failed',
 				reason => 'volinuse',
-				message => "Volume '$label' in slot $slot_scanned is reserved"),
+				message => "Source volume '$label' in slot $slot_scanned is reserved"),
 			    undef);
 		}
 		Amanda::Debug::debug("parse_inventory: load slot $slot_scanned with label '$label'");
-		$user_msg_fn->(scan_slot => 1,
+		$user_msg_fn->(Amanda::Recovery::Message->new(
+			       source_filename => __FILE__,
+			       source_line     => __LINE__,
+			       code => 1200000,
+			       severity => $Amanda::Message::INFO,
+			       scan_slot => 1,
 			       slot      => $slot_scanned,
-			       label     => $label);
+			       label     => $label));
 		$seen{$slot_scanned} = { device_status => $sl->{'device_status'},
 					 f_type        => $sl->{'f_type'},
 					 label         => $sl->{'label'} };
@@ -311,12 +318,14 @@ sub find_volume {
 				  res_cb => $steps->{'slot_loaded'},
 				  set_current => $params{'set_current'});
 	    }
+	    $inventory->[$i] = undef if defined $sl->{'allowed'} && !$sl->{'allowed'};
 	}
 
 	# Remove from seen all slot that have state == SLOT_UNKNOWN
 	# It is done when as scan is restarted from interactivity object.
 	if ($remove_undef_state) {
 	    for my $i (0..(scalar(@$inventory)-1)) {
+		next if !defined $inventory->[$i];
 		my $slot = $inventory->[$i]->{slot};
 		if (exists($seen{$slot}) &&
 		    !defined($inventory->[$i]->{state})) {
@@ -329,6 +338,7 @@ sub find_volume {
 	# remove any slots where the state has changed from the list of seen slots
 	for my $i (0..(scalar(@$inventory)-1)) {
 	    my $sl = $inventory->[$i];
+	    next if !defined $sl;
 	    my $slot = $sl->{slot};
 	    if ($seen{$slot} &&
 		defined($sl->{'state'}) &&
@@ -348,12 +358,19 @@ sub find_volume {
 	# scan any unseen slot already in a drive, if configured to do so
 	if ($self->{'scan_conf'}->{'scan_drive'}) {
 	    for my $sl (@$inventory) {
+		next if !defined $sl;
 		my $slot = $sl->{'slot'};
 		if (defined $sl->{'loaded_in'} &&
 		    !$sl->{'reserved'} &&
 		    !$seen{$slot}) {
 		    $slot_scanned = $slot;
-		    $user_msg_fn->(scan_slot => 1, slot => $slot_scanned);
+		    $user_msg_fn->(Amanda::Recovery::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1200000,
+				severity => $Amanda::Message::INFO,
+				scan_slot => 1,
+				slot => $slot_scanned));
 		    $seen{$slot_scanned} = { device_status => $sl->{'device_status'},
 					     f_type        => $sl->{'f_type'},
 					     label         => $sl->{'label'} };
@@ -370,6 +387,7 @@ sub find_volume {
 	    #find index for current slot
 	    my $current_index = undef;
 	    for my $i (0..(scalar(@$inventory)-1)) {
+		next if !defined $inventory->[$i];
 		my $slot = $inventory->[$i]->{slot};
 		if ($slot eq $current) {
                     $current_index = $i;
@@ -380,6 +398,7 @@ sub find_volume {
 	    $current_index = 0 if !defined $current_index;
 	    for my $i ($current_index..(scalar(@$inventory)-1), 0..($current_index-1)) {
 		my $sl = $inventory->[$i];
+		next if !$sl;
 		my $slot = $sl->{slot};
 		# skip slots we've seen
 		next if defined($seen{$slot});
@@ -396,7 +415,13 @@ sub find_volume {
 		$current = $slot;
 		$slot_scanned = $current;
 		Amanda::Debug::debug("parse_inventory: load slot $current");
-		$user_msg_fn->(scan_slot => 1, slot => $slot_scanned);
+		$user_msg_fn->(Amanda::Recovery::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1200000,
+				severity => $Amanda::Message::INFO,
+				scan_slot => 1,
+				slot => $slot_scanned));
 		$seen{$slot_scanned} = { device_status => $sl->{'device_status'},
 					 f_type        => $sl->{'f_type'},
 					 label         => $sl->{'label'} };
@@ -411,13 +436,10 @@ sub find_volume {
 	if ($last_err) {
 	    return $steps->{'handle_error'}->($last_err, undef);
 	} else {
-Amanda::Debug::debug("notfound 1");
-Amanda::Debug::debug("label $label");
-Amanda::Debug::debug("inventory: " . Data::Dumper::Dumper($inventory));
 	    return $steps->{'handle_error'}->(
 		    Amanda::Changer::Error->new('failed',
 			    reason => 'notfound',
-			    message => "Volume '$label' not found"),
+			    message => "Source Volume '$label' not found"),
 		    undef);
 	}
     };
@@ -428,16 +450,31 @@ Amanda::Debug::debug("inventory: " . Data::Dumper::Dumper($inventory));
 	# we don't responsd to abort_scan or restart_scan here, since we
 	# have an open reservation that we should deal with.
 
-	$user_msg_fn->(slot_result => 1,
-		       slot => $slot_scanned,
-		       err  => $err,
-		       res  => $res);
 	if ($res) {
 	    if ($res->{device}->status == $DEVICE_STATUS_SUCCESS &&
-		$res->{device}->volume_label &&
-		$res->{device}->volume_label eq $label) {
+		$res->{device}->volume_label) {
 		my $volume_label = $res->{device}->volume_label;
-		return $steps->{'call_res_cb'}->(undef, $res);
+		$user_msg_fn->(Amanda::Recovery::Message->new(
+			source_filename => __FILE__,
+			source_line     => __LINE__,
+			code => 1200001,
+			severity => $Amanda::Message::INFO,
+			slot_result => 1,
+			slot => $slot_scanned,
+			label      => $volume_label));
+		if ($volume_label eq $label) {
+		    return $steps->{'call_res_cb'}->(undef, $res);
+		}
+	    } else {
+		$user_msg_fn->(Amanda::Recovery::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1200002,
+				severity => $Amanda::Message::ERROR,
+				slot_result => 1,
+				slot => $slot_scanned,
+				dev_status => $res->{'device'},
+				dev_error  => $res->{'device'}->error_or_status()));
 	    }
 	    my $f_type;
 	    if (defined $res->{device}->volume_header) {
@@ -463,6 +500,14 @@ Amanda::Debug::debug("inventory: " . Data::Dumper::Dumper($inventory));
 	    }
 	    return $res->release(finished_cb => $steps->{'load_released'});
 	} else {
+	    $user_msg_fn->(Amanda::Recovery::Message->new(
+				source_filename => __FILE__,
+				source_line     => __LINE__,
+				code => 1200003,
+				severity => $Amanda::Message::ERROR,
+				slot_result => 1,
+				slot => $slot_scanned,
+				err  => $err));
 	    if ($load_for_label == 0 && $err->volinuse) {
 		# Scan semantics for volinuse is different than changer.
 		# If a slot with unknown label is loaded then we map
@@ -474,6 +519,10 @@ Amanda::Debug::debug("inventory: " . Data::Dumper::Dumper($inventory));
 		# volinuse is an error
 		return $steps->{'handle_error'}->($err, $steps->{'load_released'});
 	    }
+	    if ($load_for_label == 1 && $err->failed && $err->invalid) {
+		# invalid is an error
+		return $steps->{'handle_error'}->($err, $steps->{'load_released'});
+	    }
 	    return $steps->{'load_released'}->();
 	}
     };
@@ -717,7 +766,7 @@ sub _find_volume_no_inventory {
 }
 
 sub _user_msg_fn {
-    my %params = @_;
+    my $msg = shift;
 }
 
 package Amanda::Recovery::Scan::Config;
@@ -739,6 +788,7 @@ sub new {
     $self->{'unknown'} = Amanda::Recovery::Scan::SCAN_FAIL;
     $self->{'notimpl'} = Amanda::Recovery::Scan::SCAN_FAIL;
     $self->{'invalid'} = Amanda::Recovery::Scan::SCAN_CONTINUE;
+    #$self->{'invalid'} = Amanda::Recovery::Scan::SCAN_ASK_POLL;
 
     return $self;
 }
