@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,18 +15,43 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
+
+package Amanda::Report::Message;
+use strict;
+use warnings;
+
+use Amanda::Message;
+use vars qw( @ISA );
+@ISA = qw( Amanda::Message );
+
+sub local_message {
+    my $self = shift;
+
+    if ($self->{'code'} == 1900000) {
+	return "Failed to make report: $self->{'error'}";
+    } elsif ($self->{'code'} == 1900001) {
+	return "The report";
+    } elsif ($self->{'code'} == 1900002) {
+	return "Log file is for a checkdump run";
+    } elsif ($self->{'code'} == 1900003) {
+	return "Log file is for a fetchdump run";
+    }
+}
 
 package Amanda::Report;
 use strict;
 use warnings;
 use Data::Dumper;
 
+use Amanda::Config qw( :getconf );
 use Amanda::Disklist;
 use Amanda::Logfile qw/:logtype_t :program_t/;
 use Amanda::Util;
 use Amanda::Debug qw( debug warning );
+use Amanda::Storage;
+use Amanda::Process;
 
 =head1 NAME
 
@@ -422,6 +448,7 @@ sub new
     my $class = shift @_;
     my ($logfname, $historical) = @_;
 
+    debug("Amanda::Report::new logfname: $logfname");
     my $self = {
         data => {},
 
@@ -436,7 +463,13 @@ sub new
     };
     bless $self, $class;
 
-    $self->read_file();
+    $self->{'Amanda_process'} = Amanda::Process->new();
+    $self->{'Amanda_process'}->load_ps_table();
+
+    my $result = $self->read_file();
+    if (defined $result and $result->isa("Amanda::Message")) {
+	return $result;
+    }
     return $self;
 }
 
@@ -461,26 +494,113 @@ sub read_file
     $self->{flags}{results_missing} = 0;
     $self->{flags}{dump_failed} = 0;
     $self->{flags}{dump_strange} = 0;
+    $self->{flags}{status} = "running";
 
     while ( my ( $type, $prog, $str ) = Amanda::Logfile::get_logline($logfh) ) {
         $self->read_line( $type, $prog, $str );
     }
+    if (defined $self->get_program_info("checkdump")) {
+	return Amanda::Report::Message->new(
+		source_filename => __FILE__,
+		source_line     => __LINE__,
+		code   => 1900002,
+		severity => $Amanda::Message::ERROR);
+    }
+    if (defined $self->get_program_info("fetchdump")) {
+	return Amanda::Report::Message->new(
+		source_filename => __FILE__,
+		source_line     => __LINE__,
+		code   => 1900003,
+		severity => $Amanda::Message::ERROR);
+    }
 
     ## set post-run flags
 
     $self->{flags}{historical} = $self->{_historical};
     $self->{flags}{amflush_run} = 0;
     $self->{flags}{amvault_run} = 0;
+    $self->{flags}{ambackupd_run} = 0;
+    my $il;
     if (!$self->get_flag("normal_run")) {
         if (   ( defined $self->get_program_info("amflush") )
             && ( scalar %{ $self->get_program_info("amflush") } ) ) {
 	    debug("detected an amflush run");
 	    $self->{flags}{amflush_run} = 1;
+	    $il = getconf($CNF_STORAGE);
+	    $self->{'storage_list'} = $il;
+	    my $info = $self->get_program_info("amflush");
+	    my $pid = $info->{pid};
+	    if ($self->{pids}->{$pid} eq 'running') {
+		if (!$self->{'Amanda_process'}->process_alive($pid, "amflush")) {
+		    $self->{pids}->{$pid} = 'aborted';
+		}
+	    }
+	    $self->{flags}{status} = $self->{pids}->{$pid};
 	} elsif (   ( defined $self->get_program_info("amvault") )
                  && ( scalar %{ $self->get_program_info("amvault") } ) ) {
 	    debug("detected an amvault run");
 	    $self->{flags}{amvault_run} = 1;
+	    my $storage_names = getconf($CNF_VAULT_STORAGE);
+	    my $storage_name = $storage_names->[0] if defined $storage_names->[0];
+	    if (!$storage_name) {
+	         my $il = getconf($CNF_STORAGE);
+	         $storage_name = $il->[0];
+	    }
+	    $il = [ $storage_name ];
+	    my $info = $self->get_program_info("amvault");
+	    my $pid = $info->{pid};
+	    if ($self->{pids}->{$pid} eq 'running') {
+		if (!$self->{'Amanda_process'}->process_alive($pid, "amvault")) {
+		    $self->{pids}->{$pid} = 'aborted';
+		}
+	    }
+	    $self->{flags}{status} = $self->{pids}->{$pid};
+	} elsif (   ( defined $self->get_program_info("ambackupd") )
+                 && ( scalar %{ $self->get_program_info("ambackupd") } ) ) {
+	    debug("detected an ambackupd run");
+	    $self->{flags}{ambackupd_run} = 1;
+	    my $info = $self->get_program_info("ambackupd");
+	    my $pid = $info->{pid};
+	    if ($self->{pids}->{$pid} eq 'running') {
+		if (!$self->{'Amanda_process'}->process_alive($pid, "ambackupd")) {
+		    $self->{pids}->{$pid} = 'aborted';
+		}
+	    }
+	    $self->{flags}{status} = $self->{pids}->{$pid};
+	} else {
+	    debug("detected an unknown run");
+	    #$il = getconf($CNF_STORAGE);
 	}
+    } else {
+	    debug("detected an amdump run");
+	    $self->{flags}{amdump_run} = 1;
+	    my $info = $self->get_program_info("amdump");
+	    my $pid = $info->{pid};
+	    if ($self->{pids}->{$pid} eq 'running') {
+		if (!$self->{'Amanda_process'}->process_alive($pid, "amdump")) {
+		    $self->{pids}->{$pid} = 'aborted';
+		}
+	    }
+	    $self->{flags}{status} = $self->{pids}->{$pid};
+	my $il1 = getconf($CNF_STORAGE);
+
+	my $il2 = getconf($CNF_VAULT_STORAGE);
+
+	push @{$il}, @{$il1} if $il1;
+
+	push @{$il}, @{$il2} if $il2;
+
+    }
+    $self->{'storage_list'} = $il;
+
+    my $i;
+    $il = getconf($CNF_VAULT_STORAGE);
+    foreach $i (@$il) {
+	$self->{'flush_or_vault'}->{$i} = "VAULT";
+    }
+    $il = getconf($CNF_STORAGE);
+    foreach $i (@$il) {
+	$self->{'flush_or_vault'}->{$i} = "FLUSH";
     }
 
     # check for missing, fail and strange results
@@ -552,14 +672,42 @@ sub read_line
     } elsif ( $prog == $P_AMDUMP ) {
         return $self->_handle_amdump_line( $type, $str );
 
-    } elsif ( $prog == $P_REPORTER ) {
-        return $self->_handle_reporter_line( $type, $str );
+    } elsif ( $prog == $P_AMCHECKDUMP ) {
+        return $self->_handle_checkdump_line( $type, $str );
+
+    } elsif ( $prog == $P_AMFETCHDUMP ) {
+        return $self->_handle_fetchdump_line( $type, $str );
+
+    } elsif ( $prog == $P_AMREPORT ) {
+        return $self->_handle_global_line( $prog, $type, $str );
+    } elsif ( $prog == $P_AMTRMIDX ) {
+        return $self->_handle_global_line( $prog, $type, $str );
+    } elsif ( $prog == $P_AMTRMLOG ) {
+        return $self->_handle_global_line( $prog, $type, $str );
+
+    } elsif ( $prog == $P_AMCLEANUP ) {
+        return $self->_handle_amcleanup_line( $type, $str );
+
+    } elsif ( $prog == $P_AMBACKUPD ) {
+        return $self->_handle_ambackupd_line( $type, $str );
 
     } else {
         return $self->_handle_bogus_line( $prog, $type, $str );
     }
 }
 
+sub get_exit_status
+{
+    my $self = shift;
+    return $self->{'flags'}{'exit_status'};
+}
+
+sub get_status
+{
+    my $self = shift;
+    return $self->{'flags'}{'status'};
+}
+
 sub get_timestamp
 {
     my $self = shift;
@@ -784,7 +932,6 @@ sub _handle_dumper_line
     my $self = shift @_;
     my ( $type, $str ) = @_;
     my $data     = $self->{data};
-    my $disklist = $data->{disklist};
     my $programs = $data->{programs};
     my $dumper_p = $programs->{dumper} ||= {};
 
@@ -795,11 +942,17 @@ sub _handle_dumper_line
 
         my @info = Amanda::Util::split_quoted_strings($str);
         my ( $hostname, $disk, $level ) = @info[ 0 .. 2 ];
-        my ( $sec, $kb, $kps, $orig_kb ) = @info[ 4, 6, 8, 10 ];
-	$kb = int($kb/1024) if $info[4] eq 'bytes';
+	my $x;
+	if ($info[3] eq '[sec') {
+	    $x = 4;
+	} else {
+	    $x = 6; #native and client crc
+	}
+        my ( $sec, $kb, $kps, $orig_kb ) = @info[ $x, $x+2, $x+4, $x+6 ];
+	$kb = int($kb/1024) if $info[$x+1] eq 'bytes';
         $orig_kb =~ s{\]$}{};
 
-        my $dle    = $disklist->{$hostname}->{$disk};
+        my $dle    = $self->_get_disklist($hostname, $disk);
         my $try    = $self->_get_try( $dle, "dumper", $self->{'run_timestamp'});
         my $dumper = $try->{dumper} ||= {};
 	$dumper->{level} = $level;
@@ -826,11 +979,19 @@ sub _handle_dumper_line
 
         my @info = Amanda::Util::split_quoted_strings($str);
         my ( $hostname, $disk, $timestamp, $level ) = @info[ 0 .. 3 ];
-        my ( $sec, $kb, $kps, $orig_kb ) = @info[ 5, 7, 9, 11 ];
-	$kb = int($kb/1024) if $info[6] eq 'bytes';
+	my $x;
+	if ($info[4] eq '[sec') {
+	    $x = 5;
+	} elsif ($info[6] eq '[sec') {
+	    $x = 7; #native and client crc
+	} else {
+	    $x = 8; #native, client crc and server crc
+	}
+        my ( $sec, $kb, $kps, $orig_kb ) = @info[ $x, $x+2, $x+4, $x+6 ];
+	$kb = int($kb/1024) if $info[$x+1] eq 'bytes';
         $orig_kb =~ s{\]$}{};
 
-        my $dle    = $disklist->{$hostname}->{$disk};
+        my $dle    = $self->_get_disklist($hostname, $disk);
         my $try    = $self->_get_try( $dle, "dumper", $timestamp );
         my $dumper = $try->{dumper} ||= {};
 
@@ -843,6 +1004,21 @@ sub _handle_dumper_line
 
         return $dumper->{status} = "success";
 
+    } elsif ( $type == $L_RETRY ) {
+        my @info = Amanda::Util::split_quoted_strings($str);
+        my ( $hostname, $disk, $timestamp, $level ) = @info[ 0 .. 3 ];
+
+        my $dle    = $self->_get_disklist($hostname, $disk);
+        my $try    = $self->_get_try( $dle, "dumper", $timestamp );
+        my $dumper = $try->{dumper} ||= {};
+	$dumper->{date}      = $timestamp;
+	$dumper->{level}     = $level;
+	$dumper->{status}    = 'retry';
+	$try->{retry} = 1;
+	$try->{retry_delay} = $info[5];
+	$try->{retry_level} = $info[7];
+	$try->{retry_message} = $info[9];
+
     } elsif ( $type == $L_ERROR ) {
         return $self->_handle_error_line( "dumper", $str );
 
@@ -863,7 +1039,6 @@ sub _handle_chunker_line
     my $self = shift @_;
     my ( $type, $str ) = @_;
     my $data      = $self->{data};
-    my $disklist  = $data->{disklist};
     my $programs  = $data->{programs};
     my $chunker_p = $programs->{chunker} ||= {};
 
@@ -874,11 +1049,17 @@ sub _handle_chunker_line
 
         my @info = Amanda::Util::split_quoted_strings($str);
         my ( $hostname, $disk, $timestamp, $level ) = @info[ 0 .. 3 ];
-        my ( $sec, $kb, $kps ) = @info[ 5, 7, 9 ];
-	$kb = int($kb/1024) if $info[6] eq 'bytes';
+	my $x;
+	if ($info[4] eq '[sec') {
+	    $x = 5;
+	} else {
+	    $x = 6; #server-src
+	}
+        my ( $sec, $kb, $kps, $orig_kb ) = @info[ $x, $x+2, $x+4, $x+6 ];
+	$kb = int($kb/1024) if $info[$x+1] eq 'bytes';
         $kps =~ s{\]$}{};
 
-        my $dle     = $disklist->{$hostname}->{$disk};
+        my $dle     = $self->_get_disklist($hostname, $disk);
         my $try     = $self->_get_try( $dle, "chunker", $timestamp );
         my $chunker = $try->{chunker} ||= {};
 
@@ -911,18 +1092,35 @@ sub _handle_taper_line
     my $self = shift @_;
     my ( $type, $str ) = @_;
     my $data     = $self->{data};
-    my $disklist = $data->{disklist};
     my $programs = $data->{programs};
     my $taper_p  = $programs->{taper} ||= {};
 
     if ( $type == $L_START ) {
         # format is:
-        # START taper datestamp <start> label <label> tape <tapenum>
+        # START taper [ST:storage [POOL:pool]] datestamp <start> label <label> tape <tapenum>
         my @info = Amanda::Util::split_quoted_strings($str);
-        my ($datestamp, $label, $tapenum) = @info[ 1, 3, 5 ];
+        my ($datestamp, $label, $tapenum, $storage, $pool);
+	$datestamp = $info[1];
+	if ($info[2] =~ /^ST:/) {
+	    $storage = $info[2];
+	    $storage =~ s/^ST://g;
+	    if ($info[3] =~ /^POOL:/) {
+		$pool = $info[3];
+		($label, $tapenum) = @info[ 5, 7 ];
+	    } else {
+		($label, $tapenum) = @info[ 4, 6 ];
+		$pool = $storage;
+	    }
+	} else {
+            ($label, $tapenum) = @info[ 3, 5 ];
+	    $storage = Amanda::Config::get_config_name();
+	    $pool = $storage;
+	}
         my $tape = $self->get_tape($label);
         $tape->{date} = $datestamp;
+        $tape->{pool} = $pool;
         $tape->{label} = $label;
+        $tape->{storage} = $storage;
 
 	# keep this tape for later
 	$self->{'_current_tape'} = $tape;
@@ -932,10 +1130,24 @@ sub _handle_taper_line
     } elsif ( $type == $L_PART || $type == $L_PARTPARTIAL ) {
 
 # format is:
-# <label> <tapefile> <hostname> <disk> <timestamp> <currpart>/<predparts> <level> [sec <sec> kb <kb> kps <kps>]
+# [ST:<storage> [POOL:<pool>]] <label> <tapefile> <hostname> <disk> <timestamp> <currpart>/<predparts> <level> [sec <sec> kb <kb> kps <kps>]
 #
 # format for $L_PARTPARTIAL is the same as $L_PART, plus <err> at the end
         my @info = Amanda::Util::split_quoted_strings($str);
+	my $storage = $info[0];
+	if ($storage =~ /^ST:/) {
+	    $storage =~ s/^ST://g;
+	    shift @info;
+	} else {
+	    $storage = Amanda::Config::get_config_name();
+	}
+	my $pool = $info[0];
+	if ($pool =~ /^POOL:/) {
+	    $pool =~ s/^POOL://g;
+	    shift @info;
+	} else {
+	    $pool = $storage;
+	}
         my ($label, $tapefile, $hostname, $disk, $timestamp) = @info[ 0 .. 4 ];
 
         $info[5] =~ m{^(\d+)\/(-?\d+)$};
@@ -946,18 +1158,20 @@ sub _handle_taper_line
         $kps =~ s{\]$}{};
         $orig_kb =~ s{\]$}{} if defined($orig_kb);
 
-        my $dle   = $disklist->{$hostname}{$disk};
+        my $dle   = $self->_get_disklist($hostname, $disk);
         my $try   = $self->_get_try($dle, "taper", $timestamp);
         my $taper = $try->{taper} ||= {};
         my $parts = $taper->{parts} ||= [];
 
         my $part = {
-            label => $label,
-            date  => $timestamp,
-            file  => $tapefile,
-            sec   => $sec,
-            kb    => $kb,
-            kps   => $kps,
+            storage  => $storage,
+            pool     => $pool,
+            label    => $label,
+            date     => $timestamp,
+            file     => $tapefile,
+            sec      => $sec,
+            kb       => $kb,
+            kps      => $kps,
             partnum  => $currpart,
         };
 
@@ -976,23 +1190,46 @@ sub _handle_taper_line
 
 # format is:
 # $type = DONE | PARTIAL
-# $type taper <hostname> <disk> <timestamp> <part> <level> [sec <sec> kb <kb> kps <kps>]
+# $type taper [ST:<storage> [POOL:<pool>]] <hostname> <disk> <timestamp> <part> <level> [native-crc client-crc server-crc] [sec <sec> kb <kb> kps <kps>]
         my @info = Amanda::Util::split_quoted_strings($str);
+	my $storage = $info[0];
+	if ($storage =~ /^ST:/) {
+	    $storage =~ s/^ST://g;
+	    shift @info;
+	} else {
+	    $storage = Amanda::Config::get_config_name();
+	}
+	my $pool = $info[0];
+	if ($pool =~ /^POOL/) {
+	    $pool =~ s/^POOL://g;
+	    shift @info;
+	} else {
+	    $pool = $storage
+	}
+
         my ( $hostname, $disk, $timestamp, $part_ct, $level ) = @info[ 0 .. 4 ];
-        my ( $sec, $kb, $kps, $orig_kb ) = @info[ 6, 8, 10, 12 ];
-	$kb = int($kb/1024) if $info[7] eq 'bytes';
+	$part_ct = 1 if !defined $part_ct || $part_ct eq '';
+	my $x;
+	if ($info[5] eq '[sec') {
+	    $x = 6;
+	} else {
+	    $x = 9; #native, client and server crc
+	}
+
+        my ( $sec, $kb, $kps, $orig_kb ) = @info[ $x, $x+2, $x+4, $x+6 ];
+	$kb = int($kb/1024) if $info[$x+1] eq 'bytes';
 	my $error;
 	if ($type == $L_PARTIAL) {
 	    if ($kps =~ /\]$/) {
-	        $error = join " ", @info[ 11 .. $#info ];
+	        $error = join " ", @info[ $x+5 .. $#info ];
 	    } else {
-	        $error = join " ", @info[ 13 .. $#info ];
+	        $error = join " ", @info[ $x+7 .. $#info ];
 	    }
 	}
         $kps =~ s{\]$}{};
         $orig_kb =~ s{\]$}{} if defined $orig_kb;
 
-        my $dle   = $disklist->{$hostname}->{$disk};
+        my $dle   = $self->_get_disklist($hostname, $disk);
         my $try   = $self->_get_try($dle, "taper", $timestamp);
         my $taper = $try->{taper} ||= {};
         my $parts = $taper->{parts};
@@ -1001,13 +1238,15 @@ sub _handle_taper_line
             ## this should always be true; do nothing right now
         }
 
-        $taper->{level} = $level;
-        $taper->{sec}   = $sec;
-        $taper->{kb}    = $kb;
-        $taper->{kps}   = $kps;
+        $taper->{storage} = $storage;
+        $taper->{pool}    = $pool;
+        $taper->{level}   = $level;
+        $taper->{sec}     = $sec;
+        $taper->{kb}      = $kb;
+        $taper->{kps}     = $kps;
 
-        $taper->{status} = ( $type == $L_DONE ) ? "done" : "partial";
-	$taper->{error} = $error if $type == $L_PARTIAL;
+        $taper->{status}  = ( $type == $L_DONE ) ? "done" : "partial";
+	$taper->{error}   = $error if $type == $L_PARTIAL;
 
     } elsif ( $type == $L_INFO ) {
         $self->_handle_info_line("taper", $str);
@@ -1062,6 +1301,11 @@ sub _handle_amflush_line
     } elsif ( $type == $L_INFO ) {
         return $self->_handle_info_line( "amflush", $str );
 
+    } elsif ( $type == $L_FINISH ) {
+        my @info = Amanda::Util::split_quoted_strings($str);
+        $self->{flags}{got_finish} = 1;
+        return $amflush_p->{time} = $info[3];
+
     } else {
         return $self->_handle_bogus_line( $P_AMFLUSH, $type, $str );
     }
@@ -1079,6 +1323,14 @@ sub _handle_amvault_line
     if ( $type == $L_START ) {
         return $self->_handle_start_line( "amvault", $str );
 
+    } elsif ( $type == $L_STATS ) {
+
+        my @info = Amanda::Util::split_quoted_strings($str);
+        if ( $info[0] eq "hostname" ) {
+
+            return $self->{hostname} = $info[1];
+	}
+
     } elsif ( $type == $L_INFO ) {
         return $self->_handle_info_line( "amvault", $str );
 
@@ -1091,8 +1343,13 @@ sub _handle_amvault_line
     } elsif ( $type == $L_DISK ) {
         return $self->_handle_disk_line( "amvault", $str );
 
+    } elsif ( $type == $L_FINISH ) {
+        my @info = Amanda::Util::split_quoted_strings($str);
+        $self->{flags}{got_finish} = 1;
+        return $amvault_p->{time} = $info[3];
+
     } else {
-        return $self->_handle_bogus_line( $P_AMFLUSH, $type, $str );
+        return $self->_handle_bogus_line( $P_AMVAULT, $type, $str );
     }
 }
 
@@ -1120,12 +1377,155 @@ sub _handle_amdump_line
     }
 }
 
+sub _handle_checkdump_line
+{
+    my $self = shift;
+    my ( $type, $str ) = @_;
+    my $data     = $self->{data};
+    my $disklist = $data->{disklist};
+    my $programs = $data->{programs};
+    my $checkdump = $programs->{checkdump} ||= {};
+
+    if ( $type == $L_INFO ) {
+        $self->_handle_info_line("checkdump", $str);
+
+    } elsif ( $type == $L_ERROR ) {
+        $self->_handle_error_line("checkdump", $str);
+
+    } else {
+        return $self->_handle_bogus_line( $P_AMFETCHDUMP, $type, $str );
+    }
+}
+
+sub _handle_fetchdump_line
+{
+    my $self = shift;
+    my ( $type, $str ) = @_;
+    my $data     = $self->{data};
+    my $disklist = $data->{disklist};
+    my $programs = $data->{programs};
+    my $fetchdump = $programs->{amcleanup} ||= {};
+
+    if ( $type == $L_INFO ) {
+        $self->_handle_info_line("fetchdump", $str);
+
+    } elsif ( $type == $L_ERROR ) {
+        $self->_handle_error_line("fetchdump", $str);
+
+    } else {
+        return $self->_handle_bogus_line( $P_AMCHECKDUMP, $type, $str );
+    }
+}
+
+
+sub _handle_amcleanup_line
+{
+    my $self = shift;
+    my ( $type, $str ) = @_;
+    my $data     = $self->{data};
+    my $disklist = $data->{disklist};
+    my $programs = $data->{programs};
+    my $amcleanup = $programs->{amcleanup} ||= {};
+
+    if ( $type == $L_INFO ) {
+        $self->_handle_info_line("amcleanup", $str);
+
+    } elsif ( $type == $L_ERROR ) {
+        $self->_handle_error_line("amcleanup", $str);
+
+    } else {
+        return $self->_handle_bogus_line( $P_AMCLEANUP, $type, $str );
+    }
+}
+
+
+sub _handle_global_line
+{
+    my $self = shift;
+    my ( $p_prog, $type, $str ) = @_;
+    my $data     = $self->{data};
+    my $disklist = $data->{disklist};
+    my $programs = $data->{programs};
+    my $program_str = Amanda::Logfile::get_program_str($p_prog);
+    my $amcleanup = $programs->{$program_str} ||= {};
+
+    if ( $type == $L_INFO ) {
+        $self->_handle_info_line($program_str, $str);
+
+    } elsif ( $type == $L_ERROR ) {
+        $self->_handle_error_line($program_str, $str);
+
+    } else {
+        return $self->_handle_bogus_line( $p_prog, $type, $str );
+    }
+}
+
+
+sub _handle_ambackupd_line
+{
+    my $self = shift;
+    my ( $type, $str ) = @_;
+    my $data     = $self->{data};
+    my $disklist = $data->{disklist};
+    my $programs = $data->{programs};
+    my $ambackupd = $programs->{amcleanup} ||= {};
+
+    if ( $type == $L_INFO ) {
+        $self->_handle_info_line("ambackupd", $str);
+
+    } elsif ( $type == $L_START ) {
+        $self->_handle_start_line("ambackupd", $str);
+
+    } elsif ( $type == $L_FINISH ) {
+        my @info = Amanda::Util::split_quoted_strings($str);
+        $self->{flags}{got_finish} = 1;
+        return $ambackupd->{time} = $info[3];
+
+    } elsif ( $type == $L_STATS ) {
+        my @info = Amanda::Util::split_quoted_strings($str);
+        if ( $info[0] eq "hostname" ) {
+            return $self->{hostname} = $info[1];
+	}
+
+    } elsif ( $type == $L_DISK ) {
+        return $self->_handle_disk_line( "ambackupd", $str );
+
+    } elsif ( $type == $L_ERROR ) {
+        $self->_handle_error_line("ambackupd", $str);
+
+    } else {
+        return $self->_handle_bogus_line( $P_AMCLEANUP, $type, $str );
+    }
+}
+
 
 sub _handle_fail_line
 {
     my ($self, $program, $str) = @_;
 
     my @info = Amanda::Util::split_quoted_strings($str);
+    my $VAULT = $info[0];
+    if ($VAULT eq "VAULT") {
+	shift @info;
+    } else {
+	$VAULT = undef;
+    }
+    my $storage = $info[0];
+    if ($storage =~ /^ST:/) {
+	$storage =~ s/^ST://;
+	shift @info;
+    } else {
+	$storage = undef;
+    }
+
+    my $pool = $info[0];
+    if ($pool =~ /^POOL:/) {
+	$pool =~ s/^POOL://;
+	shift @info;
+    } else {
+	$pool = undef;
+    }
+
     my ($hostname, $disk, $timestamp, $level) = @info;
     my $error;
     my $failure_from;
@@ -1135,6 +1535,9 @@ sub _handle_fail_line
     } else {
 	$error = join " ", @info[ 4 .. $#info ];
     }
+    if (substr($error,0,1) eq '[' and substr($error,-1,1) eq ']') {
+	$error = substr($error, 1, length($error)-2);
+    }
 
     #TODO: verify that this reaches the right try.  Also, DLE or
     #program?
@@ -1144,11 +1547,16 @@ sub _handle_fail_line
     if ($program eq "planner" ||
         $program eq "driver") {
 	$program_d = $dle->{$program} ||= {};
+	$self->{flags}{dump_failed} = 1;
+	$self->{flags}{exit_status} |= STATUS_FAILED;
     } else {
-        my $try = $self->_get_try($dle, $program, $timestamp);
+        my $try = $self->_get_try($dle, $program, $timestamp, $storage, $VAULT);
         $program_d = $try->{$program} ||= {};
     }
 
+    $program_d->{vault} = $VAULT;
+    $program_d->{storage} = $storage;
+    $program_d->{pool}   = $pool;
     $program_d->{level}  = $level;
     $program_d->{status} = "fail";
     $program_d->{failure_from}  = $failure_from;
@@ -1157,7 +1565,6 @@ sub _handle_fail_line
     my $errors = $self->get_program_info("program", "errors", []);
     push @$errors, $error;
 
-    $self->{flags}{exit_status} |= STATUS_FAILED;
     if ($program eq "dumper") {
         $self->{contline} = $program_d->{errors} ||= [];
 	$program_d->{nb_errors} = 0;
@@ -1225,32 +1632,41 @@ sub _handle_start_line
 }
 
 
-sub _handle_disk_line
+sub _get_disklist
 {
     my $self = shift @_;
-    my ($program, $str) = @_;
+    my $hostname = shift @_;
+    my $disk = shift @_;
 
     my $data     = $self->{data};
     my $disklist = $data->{disklist};
     my $hosts    = $self->{cache}{hosts} ||= [];
     my $dles     = $self->{cache}{dles}  ||= [];
 
-    my @info = Amanda::Util::split_quoted_strings($str);
-    my ($hostname, $disk) = @info;
-
     if (!exists $disklist->{$hostname}) {
-
         $disklist->{$hostname} = {};
         push @$hosts, $hostname;
     }
 
     if (!exists $disklist->{$hostname}{$disk}) {
-
         push @$dles, [ $hostname, $disk ];
         my $dle = $disklist->{$hostname}{$disk} = {};
         $dle->{'estimate'} = undef;
         $dle->{'dumps'}    = {};
     }
+    return $disklist->{$hostname}{$disk};
+}
+
+sub _handle_disk_line
+{
+    my $self = shift @_;
+    my ($program, $str) = @_;
+
+    my @info = Amanda::Util::split_quoted_strings($str);
+    my ($hostname, $disk) = @info;
+
+    $self->{dump_disk}->{$hostname}->{$disk} = 1;
+    $self->_get_disklist($hostname, $disk);
     return;
 }
 
@@ -1285,11 +1701,29 @@ sub _handle_info_line
 
     my $program_p = $programs->{$program} ||= {};
 
-    if ( $str =~ m/^\w+ pid \d+/ || $str =~ m/^pid-done \d+/ ) {
-
+    if ( $str =~ m/^([^ ]+) pid (\d+)/ ) {
+	my $pp = $1;
+	my $pid = $2;
+	if ($pp eq $program) {
+	    $program_p->{pid} = $pid;
+	}
         #do not report pid lines
+        $self->{'pids'}->{$pid} = 'running';
+        return;
+    } elsif ( $str =~ m/^fork ([^ ]+) (\d+) (\d+)/ ) {
+	my $pp = $1;
+	my $pid = $3;
+	if ($pp eq $program) {
+	    $program_p->{pid} = $pid;
+	}
+        #do not report pid lines
+        $self->{'pids'}->{$pid} = 'running';
+        return;
+    } elsif ( $str =~ m/^pid-done (\d+)/ ) {
+	my $pid = $1;
+        #do not report pid-done lines
+        $self->{'pids'}->{$pid} = 'done';
         return;
-
     } else {
         my $notes = $program_p->{notes} ||= [];
         push @$notes, $str;
@@ -1326,6 +1760,7 @@ sub check_missing_fail_strange
 
 	if ($planner && $planner->{'status'} eq 'fail') {
 	    $self->{flags}{dump_failed} = 1;
+	    $self->{flags}{exit_status} |= STATUS_FAILED;
 	} elsif ($planner && $planner->{'status'} eq 'skipped') {
 	    # We don't want these to be counted as missing below
 	} elsif (!defined $alldumps->{$self->{'run_timestamp'}} and
@@ -1340,17 +1775,40 @@ sub check_missing_fail_strange
 
 	    if (exists $try->{dumper} && $try->{dumper}->{status} eq 'fail') {
 		$self->{flags}{dump_failed} = 1;
-	    } elsif ((defined($try->{'chunker'}) &&
-		 $try->{'chunker'}->{status} eq 'success') ||
-		(defined($try->{'taper'}) &&
-		 $try->{'taper'}->{status} eq 'done')) {
-		#chunker or taper success, use dumper status
-		if (exists $try->{dumper} && $try->{dumper}->{status} eq 'strange') {
-		    $self->{flags}{dump_strange} = 1;
-		}
-	    } else {
-		#chunker or taper failed, the dump is not valid.
-		$self->{flags}{dump_failed} = 1;
+		$self->{flags}{exit_status} |= STATUS_FAILED;
+	    } elsif (defined($try->{'dumper'})) {
+		    if ((defined($try->{'chunker'}) &&
+			 $try->{'chunker'}->{status} eq 'success') ||
+			(defined($try->{'taper'}) &&
+			 $try->{'taper'}->{status} eq 'done')) {
+			#chunker or taper success, use dumper status
+			if (exists $try->{dumper} && $try->{dumper}->{status} eq 'strange') {
+			    $self->{flags}{dump_strange} = 1;
+			}
+			if (defined($try->{'taper'}) &&
+			    $try->{'taper'}->{status} ne 'done') {
+				$self->{flags}{exit_status} |= STATUS_TAPE;
+			}
+		    } elsif ((defined($try->{'chunker'}) &&
+			 $try->{'chunker'}->{status} ne 'success') ||
+			(defined($try->{'taper'}) &&
+			 $try->{'taper'}->{status} ne 'done') ||
+			(!defined($try->{'chunker'}) and !defined($try->{'taper'}))) {
+			#chunker or taper failed, the dump is not valid.
+			$self->{flags}{dump_failed} = 1;
+			$self->{flags}{exit_status} |= STATUS_FAILED;
+		    }
+	    } elsif (defined($try->{'chunker'}) &&
+		$try->{'chunker'}->{status} ne 'done') {
+		    $self->{flags}{exit_status} |= STATUS_FAILED;
+	    } elsif (defined($try->{'taper'}) &&
+		$try->{'taper'}->{status} ne 'done') {
+		    $self->{flags}{exit_status} |= STATUS_FAILED;
+	    } elsif (!exists $try->{dumper} &&
+		     !exists $try->{chunker} &&
+		     !exists $try->{taper} &&
+		     $driver->{'status'} eq 'fail') {
+		$self->{flags}{exit_status} |= STATUS_FAILED;
 	    }
 	}
     }
@@ -1363,18 +1821,44 @@ sub check_missing_fail_strange
 sub _get_try
 {
     my $self = shift @_;
-    my ( $dle, $program, $timestamp ) = @_;
+    my ( $dle, $program, $timestamp, $storage, $vault ) = @_;
+    $storage = "" if !defined $storage;
+    $vault = "" if !defined $vault;
     my $tries = $dle->{'dumps'}{$timestamp} ||= [];
 
-    if (
-        !@$tries    # no tries
-        || defined $tries->[-1]->{$program}->{status}
-        && $self->_program_finished(    # program has finished
-            $program, $tries->[-1]->{$program}->{status}
-        )
-      ) {
-        push @$tries, {};
+    foreach my $i (reverse 0 .. $#$tries) {
+	my $try = $tries->[$i];
+    #foreach my $try (@$tries) {
+	next if (exists $try->{$program} &&
+		 exists $try->{$program}->{status} &&
+		 defined $try->{$program}->{status} &&
+		 $self->_program_finished($program,  $try->{$program}->{status}));
+	if ($program eq "taper" && !exists $try->{$program}) {
+	    if (exists $try->{chunker} &&
+		exists $try->{chunker}->{status} &&
+		defined $try->{chunker}->{status} &&
+		$try->{chunker}->{status} eq "fail") {
+		next;
+	    }
+	}
+
+	next if ($program eq "taper" && $vault eq "VAULT" &&
+		 (exists $try->{chunker} ||
+		  exists $try->{dumper} ||
+		 (exists $try->{taper} &&
+		  defined $try->{taper}->{vault} &&
+		  $try->{taper}->{vault} ne 'VAULT' &&
+		  defined $try->{taper}->{storage} &&
+		  $try->{taper}->{storage} ne $storage)));
+
+	next if ($program ne "taper" &&
+		 exists $try->{taper} &&
+		 defined $try->{taper}->{vault} &&
+                 $try->{taper}->{vault} eq 'VAULT');
+	return $try;
     }
+
+    push @$tries, {};
     return $tries->[-1];
 }
 
