@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
 # Contact information: Zmanda Inc., 465 S Mathlida Ave, Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 #
 
 package Amanda::Report::human;
@@ -30,6 +31,9 @@ use Data::Dumper;
 use Amanda::Config qw(:getconf config_dir_relative);
 use Amanda::Util qw(:constants quote_string );
 use Amanda::Holding;
+use Amanda::Policy;
+use Amanda::Storage;
+use Amanda::Changer;
 use Amanda::Tapelist;
 use Amanda::Debug qw( debug );
 use Amanda::Util qw( quote_string );
@@ -47,13 +51,14 @@ use constant COLSPEC_FORMAT    => 5;    # sprintf format
 use constant COLSPEC_TITLE     => 6;    # column title
 
 use constant PROGRAM_ORDER =>
-  qw(amdump planner amflush amvault driver dumper chunker taper reporter);
+  qw(amcleanup amdump planner amflush amvault driver dumper chunker taper amreport amtrmidx amtrmlog);
 
 
 ## helper functions
 
 sub divzero
 {
+    my ($self) = shift;
     my ( $a, $b ) = @_;
     my $q;
     return
@@ -65,6 +70,7 @@ sub divzero
 
 sub divzero_wide
 {
+    my ($self) = shift;
     my ( $a, $b ) = @_;
     my $q;
     return
@@ -76,6 +82,7 @@ sub divzero_wide
 
 sub divzero_col
 {
+    my ($self) = shift;
     my ( $a, $b, $col ) = @_;
     return ( $b == 0 )
       ? "-- "
@@ -113,6 +120,7 @@ sub min
 
 sub hrmn
 {
+    my ($self) = shift;
     my ($sec) = @_;
     $sec += 30; # round up
     my ( $hr, $mn ) = ( int( $sec / ( 60 * 60 ) ), int( $sec / 60 ) % 60 );
@@ -121,6 +129,7 @@ sub hrmn
 
 sub mnsc
 {
+    my ($self) = shift;
     my ($sec) = @_;
     $sec += 0.5; # round up
     my ( $mn, $sc ) = ( int( $sec / (60) ), int( $sec % 60 ) );
@@ -175,6 +184,7 @@ sub new
         @errors =
           map { @{ $report->get_program_info($_, "errors", []) }; }
           PROGRAM_ORDER;
+
         ## prepend program name to notes lines.
         foreach my $program (PROGRAM_ORDER) {
             push @notes,
@@ -203,7 +213,7 @@ sub calculate_stats
     my $total_stats = $self->{total_stats};
     my $dumpdisks   = $self->{dumpdisks};
     my $tapedisks   = $self->{tapedisks};
-    my $tapeparts  = $self->{tapeparts};
+    my $tapeparts   = $self->{tapeparts};
 
     ## initialize all relevant fields to 0
     map { $incr_stats->{$_} = $full_stats->{$_} = 0; }
@@ -219,6 +229,10 @@ sub calculate_stats
 	while( my ($timestamp, $tries) = each %$alldumps ) {
 	    foreach my $try ( @$tries ) {
 
+		if (exists $try->{'retry'}) {
+		    next;
+		}
+
 		my $level = exists $try->{dumper} ? $try->{dumper}{'level'} :
 			    exists $try->{taper} ? $try->{taper}{'level'} :
 			    0;
@@ -242,7 +256,15 @@ sub calculate_stats
 		my $origsize = 0;
 		if ( exists $try->{dumper}
 		    && (   $try->{dumper}{status} eq 'success'
-			|| $try->{dumper}{status} eq 'strange')) {
+			|| $try->{dumper}{status} eq 'strange')
+		    && (   (   exists $try->{chunker}
+			    && exists $try->{chunker}{status}
+		            && (   $try->{chunker}{status} eq 'success'
+		                || $try->{chunker}{status} eq 'partial'))
+			|| (   exists $try->{taper}
+			    && exists $try->{taper}{status}
+		            && (   $try->{taper}{status} eq 'done'
+			        || $try->{taper}{status} eq 'partial')))) {
 
 		    $origsize = $try->{dumper}{orig_kb};
 		    $stats->{dumper_time} += $try->{dumper}{sec};
@@ -387,13 +409,11 @@ sub print_header
 @<<<<<<<: ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<...
 EOF
 
-    if ($hostname) {
-	$self->zprint(swrite($header_format, "Hostname", $hostname));
-	$self->zprint(swrite($header_format, "Org",      $org));
-	$self->zprint(swrite($header_format, "Config",   $config_name));
-	$self->zprint(swrite($header_format, "Date",     $date));
-	$self->zprint("\n");
-    }
+    $self->zprint(swrite($header_format, "Hostname", $hostname)) if $hostname;
+    $self->zprint(swrite($header_format, "Org",      $org)) if $org;
+    $self->zprint(swrite($header_format, "Config",   $config_name)) if $config_name;
+    $self->zprint(swrite($header_format, "Date",     $date)) if $date;
+    $self->zprint("\n");
 
     return;
 }
@@ -413,20 +433,41 @@ sub output_tapeinfo
     my %incr_stats  = %{ $self->{incr_stats} };
     my %total_stats = %{ $self->{total_stats} };
 
-    if (getconf($CNF_REPORT_USE_MEDIA) and @$tape_labels > 0) {
+    for my $storage_n (@{$report->{'storage_list'}}) {
+	my $st = Amanda::Config::lookup_storage($storage_n);
+	if (!$st) {
+	    debug("Storage '%s' not found", $storage_n);
+	    next;
+	}
+	if (storage_getconf($st, $STORAGE_REPORT_USE_MEDIA)) {
+	    # find and count label use for the storage
 
-	# slightly different sentence depending on the run type
-        my $tapelist_str;
-	if ($report->get_flag("amflush_run")) {
-	    $tapelist_str = "The dumps were flushed ";
-	} elsif ($report->get_flag("amvault_run")) {
-	    $tapelist_str = "The dumps were vaulted ";
-	} else {
-	    $tapelist_str = "These dumps were ";
+	    my @storage_tape_labels;
+	    foreach my $tape_label (@$tape_labels) {
+		my $tape = $tapes->{$tape_label};
+		if ($tape->{'storage'} eq $storage_n) {
+		    push @storage_tape_labels, $tape_label;
+		}
+	    }
+
+	    if (@storage_tape_labels > 0) {
+
+		my $to_storage = '';
+		$to_storage = " to storage '$storage_n'" if @{$report->{'storage_list'}} > 1;
+		# slightly different sentence depending on the run type
+	        my $tapelist_str;
+		if ($report->get_flag("amflush_run")) {
+		    $tapelist_str = "The dumps" . $to_storage . " were flushed ";
+		} elsif ($report->get_flag("amvault_run")) {
+		    $tapelist_str = "The dumps" . $to_storage . " were vaulted ";
+		} else {
+		    $tapelist_str = "These dumps" . $to_storage . " were ";
+		}
+	        $tapelist_str .= (@storage_tape_labels > 1) ? "to tapes " : "to tape ";
+	        $tapelist_str .= join(", ", @storage_tape_labels) . ".\n";
+	        $self->zprint($tapelist_str);
+	    }
 	}
-        $tapelist_str .= (@$tape_labels > 1) ? "to tapes " : "to tape ";
-        $tapelist_str .= join(", ", @$tape_labels) . ".\n";
-        $self->zprint($tapelist_str);
     }
 
     if (my $tape_error =
@@ -452,7 +493,7 @@ sub output_tapeinfo
 
     } else {
 
-        my @holding_list = Amanda::Holding::get_files_for_flush();
+        my @holding_list = Amanda::Holding::get_files_for_flush(0);
         my $h_size = 0;
         foreach my $holding_file (@holding_list) {
             $h_size += (0 + Amanda::Holding::file_size($holding_file, 1));
@@ -466,57 +507,90 @@ sub output_tapeinfo
               "There are $h_size_u of dumps left in the holding disk.\n");
 
             (getconf($CNF_AUTOFLUSH))
-              ? $self->zprint("They will be flushed on the next run.\n\n")
-              : $self->zprint("Run amflush to flush them to tape.\n\n");
+              ? $self->zprint("They will be flushed on the next run.\n")
+              : $self->zprint("Run amflush to flush them to tape.\n");
 
         } elsif ($report->get_flag("degraded_mode")) {
-            $self->zprint("No dumps are left in the holding disk.\n\n");
+            $self->zprint("No dumps are left in the holding disk.\n");
         }
     }
 
-    if (getconf($CNF_REPORT_NEXT_MEDIA)) {
-	my $nb_new_tape = 0;
-	my $run_tapes   = getconf($CNF_RUNTAPES);
-
-	my $text;
-	if ($run_tapes) {
-            $text = ($run_tapes > 1)
-	          ? "The next $run_tapes tapes Amanda expects to use are: "
-	          : "The next tape Amanda expects to use is: ";
+    for my $storage_n (@{$report->{'storage_list'}}) {
+	my $st = Amanda::Config::lookup_storage($storage_n);
+	if (!$st) {
+	    debug("Storage '%s' not found", $storage_n);
+	    next;
 	}
+	if (storage_getconf($st, $STORAGE_REPORT_NEXT_MEDIA)) {
+	    my $run_tapes   = storage_getconf($st, $STORAGE_RUNTAPES);
+	    my $nb_new_tape = 0;
+	    my $first = 1;
 
-	my $first = 1;
-	foreach my $i ( 0 .. ( $run_tapes - 1 ) ) {
+	    my $for_storage = '';
+	    $for_storage = " for storage '$storage_n'" if @{$report->{'storage_list'}} > 1;
+	    my $text;
+	    if ($run_tapes) {
+		$text = ($run_tapes > 1)
+	          ? "The next $run_tapes tapes Amanda expects to use" . $for_storage . " are: "
+	          : "The next tape Amanda expects to use" . $for_storage . " is: ";
+	    }
 
-            if ( my $tape_label =
-		Amanda::Tapelist::get_last_reusable_tape_label($i) ) {
+	    my $tlf = Amanda::Config::config_dir_relative(getconf($CNF_TAPELIST));
+	    my ($tl, $message) = Amanda::Tapelist->new($tlf);
 
-		if ($nb_new_tape) {
-		    $text .= ", " if !$first;
-		    $text .= "$nb_new_tape new tape"
-			    . ( $nb_new_tape > 1 ? "s" : "" );
-		    $nb_new_tape = 0;
-		    $first = 0;
-		}
+	    my $labelstr = storage_getconf($st, $STORAGE_LABELSTR);
+	    my $tapepool = storage_getconf($st, $STORAGE_TAPEPOOL);
+	    my $policy = Amanda::Policy->new(policy => storage_getconf($st, $STORAGE_POLICY));
+	    my $retention_tapes = $policy->{'retention_tapes'};
+	    my $retention_days = $policy->{'retention_days'};
+	    my $retention_recover = $policy->{'retention_recover'};
+	    my $retention_full = $policy->{'retention_full'};
 
-		$text .=
+	    foreach my $i ( 0 .. ( $run_tapes - 1 ) ) {
+
+		if ( my $tape_label =
+		    Amanda::Tapelist::get_last_reusable_tape_label(
+					$labelstr->{'template'},
+					$tapepool,
+					$storage_n,
+					$retention_tapes,
+					$retention_days,
+					$retention_recover,
+					$retention_full,
+					$i) ) {
+		    if ($nb_new_tape) {
+			$text .= ", " if !$first;
+			$text .= "$nb_new_tape new tape"
+				. ( $nb_new_tape > 1 ? "s" : "" );
+			$nb_new_tape = 0;
+			$first = 0;
+		    }
+
+		    $text .=
 		    ($first ? "" : ", ") .
 		    $tape_label;
-		$first = 0;
-            } else {
-		$nb_new_tape++;
-            }
-	}
+		    $first = 0;
+		} else {
+		    $nb_new_tape++;
+		}
+	    }
 
-	if ($nb_new_tape) {
-            $text .= ", " if !$first;
-            $text .= "$nb_new_tape new tape"
-              . ( $nb_new_tape > 1 ? "s" : "" );
-	}
-	$self->zprint("$text.\n");
+	    if ($nb_new_tape) {
+		$text .= ", " if !$first;
+		$text .= "$nb_new_tape new tape"
+			. ( $nb_new_tape > 1 ? "s" : "" );
+	    }
+	    $self->zprint("$text.\n");
 
-	my $new_tapes = Amanda::Tapelist::list_new_tapes(getconf($CNF_RUNTAPES));
-	$self->zprint("$new_tapes\n") if $new_tapes;
+	    my @new_tapes = Amanda::Tapelist::list_new_tapes(
+						$storage_n,
+						$run_tapes);
+	    if (@new_tapes == 1) {
+		$self->zprint("The next new tape already labelled is: $new_tapes[0].");
+	    } elsif (@new_tapes > 1) {
+		$self->zprint("The next " . @new_tapes . " tape already labelled are: " . join(',', @new_tapes) . ".");
+	    }
+	}
     }
 
     return;
@@ -557,6 +631,7 @@ sub output_error_summaries
 
 	if ($report->get_flag('results_missing') and
 	    !defined($alldumps->{$report->{run_timestamp}}) and
+	    $report->{dump_disk}->{$hostname}->{$disk} &&
 	    !$dle->{driver} and
 	    !$dle->{planner}) {
 	    push @missing_failures, "$hostname $qdisk RESULTS MISSING";
@@ -564,45 +639,61 @@ sub output_error_summaries
 
 	if (   exists $dle->{driver}
 	    && exists $dle->{driver}->{error}) {
-	    push @driver_failures, "$hostname $qdisk lev $dle->{driver}->{level}  FAILED $dle->{driver}->{error}";
+	    push @driver_failures, "$hostname $qdisk lev $dle->{driver}->{level}  FAILED [$dle->{driver}->{error}]";
 	}
 
 	if (   exists $dle->{planner}
 	    && exists $dle->{planner}->{error}) {
-	    push @planner_failures, "$hostname $qdisk lev $dle->{planner}->{level}  FAILED $dle->{planner}->{error}";
+	    push @planner_failures, "$hostname $qdisk lev $dle->{planner}->{level}  FAILED [$dle->{planner}->{error}]";
 	}
 
 	while( my ($timestamp, $tries) = each %$alldumps ) {
 	    my $failed = 0;
+	    my $datestr = '';
+	    $datestr = "date $timestamp " if $timestamp != $report->{run_timestamp};
 	    foreach my $try (@$tries) {
+
+		if (exists $try->{'retry'}) {
+		    push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{dumper}->{level}  FAILED [$try->{'retry_message'}: Will retry at level $try->{'retry_level'}]";
+		    next;
+		}
 		if (exists $try->{dumper} &&
 		    $try->{dumper}->{status} &&
-		    $try->{dumper}->{status} eq 'fail') {
-		    push @dump_failures, "$hostname $qdisk lev $try->{dumper}->{level}  FAILED $try->{dumper}->{error}";
+		    $try->{dumper}->{status} eq 'fail' &&
+		    $try->{dumper}->{error} ne 'Aborted by driver') {
+		    push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{dumper}->{level}  FAILED [$try->{dumper}->{error}]";
 		    $failed = 1;
 		}
-		if (exists $try->{chunker} &&
+		if (exists $try->{chunker} && !exists $dle->{driver} &&
 		    $try->{chunker}->{status} eq 'fail') {
-		    push @dump_failures, "$hostname $qdisk lev $try->{chunker}->{level}  FAILED $try->{chunker}->{error}";
+		    push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{chunker}->{level}  FAILED [$try->{chunker}->{error}]";
 		    $failed = 1;
 		}
 		if (   exists $try->{taper}
+		    && ((exists $try->{dumper} && !exists $dle->{driver})
+			|| (!exists $try->{dumper} && !exists $dle->{driver}))
 		    && (   $try->{taper}->{status} eq 'fail'
 			|| (   $try->{taper}->{status} eq 'partial'))) {
 		    my $flush = "FLUSH";
+		    $flush = "VAULT" if $report->get_flag("amvault_run");
+		    $flush = "VAULT" if $try->{taper}->{vault};
+		    if (defined $report->{'flush_or_vault'}->{$try->{taper}->{storage}}) {
+			$flush = "$report->{'flush_or_vault'}->{$try->{taper}->{storage}}($try->{taper}->{storage})";
+		    }
 		    $flush = "FAILED" if exists $try->{dumper} && !exists $try->{chunker};
-		    if ($flush ne "FLUSH" or !defined $try->{taper}->{failure_from}
-					  or $try->{taper}->{failure_from} ne 'config') {
+		    $flush = "FAILED" if !exists $try->{dumper} && exists $try->{chunker};
+		    if ($flush eq "FAILED" or !defined $try->{taper}->{failure_from}
+					   or $try->{taper}->{failure_from} ne 'config') {
 		        if ($try->{taper}->{status} eq 'partial') {
 			    # if the error message is omitted, then the taper only got a partial
 			    # dump from the dumper/chunker, rather than failing with a taper error
 			    my $errmsg = $try->{taper}{error} || "successfully taped a partial dump";
 			    $flush = "partial taper: $errmsg";
 		        } else {
-			    $flush .= " " . $try->{taper}{error};
+			    $flush .= " [" . $try->{taper}{error} . "]";
 		        }
 
-		        push @dump_failures, "$hostname $qdisk lev $try->{taper}->{level}  $flush";
+		        push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{taper}->{level}  $flush";
 		        $failed = 1;
 		    }
 		}
@@ -616,22 +707,22 @@ sub output_error_summaries
 			|| $try->{chunker}->{status} eq "success")
 		    && (   !exists $try->{taper}
 			|| $try->{taper}->{status} eq "done")) {
-		    push @dump_failures, "$hostname $qdisk lev $try->{dumper}->{level}  was successfully retried";
+		    push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{dumper}->{level}  was successfully retried";
 		    $failed = 0;
 		}
 
 		# detect dumps re-flushed from holding
 		if (   $failed
-		    && !exists $try->{dumper}
+		    && !exists $try->{dumper} && !exists $dle->{driver}
 		    && !exists $try->{chunker}
 		    && exists $try->{taper}
 		    && $try->{taper}->{status} eq "done") {
-		    push @dump_failures, "$hostname $qdisk lev $try->{taper}->{level}  was successfully re-flushed";
+		    push @dump_failures, "$hostname $qdisk ${datestr}lev $try->{taper}->{level}  was successfully re-flushed";
 		    $failed = 0;
 		}
 
 		push @stranges,
-    "$hostname $qdisk lev $try->{dumper}->{level}  STRANGE (see below)"
+    "$hostname $qdisk ${datestr}lev $try->{dumper}->{level}  STRANGE (see below)"
 		  if (defined $try->{dumper}
 		    && $try->{dumper}->{status} eq 'strange');
 	    }
@@ -681,14 +772,19 @@ EOF
     my $incr_stats  = $self->{incr_stats};
     my $total_stats = $self->{total_stats};
 
-    my ( $ttyp, $tt, $tapesize, $marksize );
-    $ttyp = getconf($CNF_TAPETYPE);
-    $tt = lookup_tapetype($ttyp) if $ttyp;
+    my ($tapesize, $marksize );
 
-    if ( $ttyp && $tt ) {
+    if ($report->{'storage_list'}[0]) {
+	my $st = Amanda::Config::lookup_storage($report->{'storage_list'}[0]);
+	if (!$st) {
+	}
+	my $tapetype_name = storage_getconf($st, $STORAGE_TAPETYPE);
+	my $tt = lookup_tapetype($tapetype_name) if $tapetype_name;
 
-        $tapesize = "".tapetype_getconf( $tt, $TAPETYPE_LENGTH );
-        $marksize = "".tapetype_getconf( $tt, $TAPETYPE_FILEMARK );
+	if ( $tapetype_name && $tt ) {
+            $tapesize = "".tapetype_getconf( $tt, $TAPETYPE_LENGTH );
+            $marksize = "".tapetype_getconf( $tt, $TAPETYPE_FILEMARK );
+	}
     }
 
     # these values should never be zero; assign defaults
@@ -700,23 +796,23 @@ EOF
     $self->zprint(swrite(
         $st_format,
         "Estimate Time (hrs:min)",
-        hrmn( $total_stats->{planner_time} ),
+        $self->hrmn( $total_stats->{planner_time} ),
         "", "", ""
     ));
 
     $self->zprint(swrite(
         $st_format,
         "Run Time (hrs:min)",
-        hrmn( $total_stats->{total_time} ),
+        $self->hrmn( $total_stats->{total_time} ),
         "", "", ""
     ));
 
     $self->zprint(swrite(
         $st_format,
         "Dump Time (hrs:min)",
-        hrmn( $total_stats->{dumper_time} ),
-        hrmn( $full_stats->{dumper_time} ),
-        hrmn( $incr_stats->{dumper_time} ),
+        $self->hrmn( $total_stats->{dumper_time} ),
+        $self->hrmn( $full_stats->{dumper_time} ),
+        $self->hrmn( $incr_stats->{dumper_time} ),
 	""
     ));
 
@@ -740,7 +836,7 @@ EOF
 
     my $comp_size = sub {
         my ($stats) = @_;
-        return divzero(100 * $stats->{outsize}, $stats->{origsize});
+        return $self->divzero(100 * $stats->{outsize}, $stats->{origsize});
     };
 
     $self->zprint(swrite(
@@ -764,9 +860,9 @@ EOF
     $self->zprint(swrite(
         $st_format,
         "Avg Dump Rate (k/s)",
-        divzero_wide( $total_stats->{outsize}, $total_stats->{dumper_time} ),
-        divzero_wide( $full_stats->{outsize},  $full_stats->{dumper_time} ),
-        divzero_wide( $incr_stats->{outsize},  $incr_stats->{dumper_time} ),
+        $self->divzero_wide( $total_stats->{outsize}, $total_stats->{dumper_time} ),
+        $self->divzero_wide( $full_stats->{outsize},  $full_stats->{dumper_time} ),
+        $self->divzero_wide( $incr_stats->{outsize},  $incr_stats->{dumper_time} ),
         ""
     ));
     $self->zprint("\n");
@@ -774,9 +870,9 @@ EOF
     $self->zprint(swrite(
         $st_format,
         "Tape Time (hrs:min)",
-        hrmn( $total_stats->{taper_time} ),
-        hrmn( $full_stats->{taper_time} ),
-        hrmn( $incr_stats->{taper_time} ),
+        $self->hrmn( $total_stats->{taper_time} ),
+        $self->hrmn( $full_stats->{taper_time} ),
+        $self->hrmn( $incr_stats->{taper_time} ),
 	""
     ));
 
@@ -791,7 +887,7 @@ EOF
 
     my $tape_usage = sub {
         my ($stat_ref) = @_;
-        return divzero(
+        return $self->divzero(
             100 * (
                 $marksize *
                   ($stat_ref->{tapedisk_count} + $stat_ref->{tapepart_count}) +
@@ -813,7 +909,7 @@ EOF
     my $nb_incr_dle = 0;
     my @incr_dle = @{$self->{tapedisks}};
     foreach my $level (1 .. $#incr_dle) {
-	$nb_incr_dle += $incr_dle[$level];
+	$nb_incr_dle += $incr_dle[$level] if defined $incr_dle[$level];
     }
     $self->zprint(swrite(
         $st_format,
@@ -846,13 +942,12 @@ EOF
     $self->zprint(swrite(
         $st_format,
         "Avg Tp Write Rate (k/s)",
-        divzero_wide( $total_stats->{tapesize}, $total_stats->{taper_time} ),
-        divzero_wide( $full_stats->{tapesize},  $full_stats->{taper_time} ),
-        divzero_wide( $incr_stats->{tapesize},  $incr_stats->{taper_time} ),
+        $self->divzero_wide( $total_stats->{tapesize}, $total_stats->{taper_time} ),
+        $self->divzero_wide( $full_stats->{tapesize},  $full_stats->{taper_time} ),
+        $self->divzero_wide( $incr_stats->{tapesize},  $incr_stats->{taper_time} ),
         ""
     ));
 
-    $self->zprint("\n");
     return;
 }
 
@@ -861,7 +956,7 @@ sub has_incrementals
     my $array = shift;
 
     for ($a = 1; $a < @$array; $a+=1) {
-	return 1 if $array->[$a] > 0;
+	return 1 if defined $array->[$a] and $array->[$a] > 0;
     }
     return 0;
 }
@@ -887,10 +982,13 @@ sub output_tape_stats
       . '<' x ($label_length - 1)
       . " @>>>>> @>>>>>>>>>>> @>>>>> @>>>> @>>>>\n";
 
-    $self->zsprint("USAGE BY TAPE:\n");
+    $self->zsprint("\n\nUSAGE BY TAPE:\n");
     $self->zprint(swrite($ts_format, "Label", "Time", "Size", "%", "DLEs", "Parts"));
 
-    my $tapetype_name = getconf($CNF_TAPETYPE);
+    my $st = Amanda::Config::lookup_storage($report->{'storage_list'}[0]);
+    if (!$st) {
+    }
+    my $tapetype_name = storage_getconf($st, $STORAGE_TAPETYPE);
     my $tapetype      = lookup_tapetype($tapetype_name);
     my $tapesize      = "" . tapetype_getconf($tapetype, $TAPETYPE_LENGTH);
     my $marksize      = "" . tapetype_getconf($tapetype, $TAPETYPE_FILEMARK);
@@ -905,14 +1003,13 @@ sub output_tape_stats
         $self->zprint(swrite(
             $ts_format,
             $label,
-            hrmn($tape->{time}),                               # time
+            $self->hrmn($tape->{time}),                               # time
             sprintf("%.0f", $self->tounits($tape->{kb})) . $self->{disp_unit},  # size
-            divzero(100 * $tapeused, $tapesize),    # % usage
+            $self->divzero(100 * $tapeused, $tapesize),    # % usage
             int($tape->{dle}),                        # # of dles
             int($tape->{files})                       # # of parts
         ));
     }
-    $self->zprint("\n");
     return;
 }
 
@@ -948,10 +1045,11 @@ sub output_details
 		# check for failed dumper details
 		#
 		if (defined $try->{dumper}
-		    && $try->{dumper}->{status} eq 'fail') {
+		    && $try->{dumper}->{status} eq 'fail'
+		    && $try->{dumper}->{error} ne 'Aborted by driver') {
 
 		    push @failed_dump_details,
-    "/-- $hostname $qdisk lev $try->{dumper}->{level} FAILED $try->{dumper}->{error}",
+    "/-- $hostname $qdisk lev $try->{dumper}->{level} FAILED [$try->{dumper}->{error}]",
 		      @{ $try->{dumper}->{errors} },
 		      "\\--------";
 
@@ -1020,12 +1118,10 @@ sub output_details
                 && ($est->{ckb} - $outsize > 1.0e5));
         }
     }
-
     $self->print_if_def(\@failed_dump_details,  "FAILED DUMP DETAILS:");
     $self->print_if_def(\@strange_dump_details, "STRANGE DUMP DETAILS:");
     $self->print_if_def($notes,                 "NOTES:");
 
-    $self->zprint("\n");
     return;
 }
 
@@ -1066,6 +1162,7 @@ sub output_summary
     my $nodump_PARTIAL_format = get_summary_format($col_spec, 'nodump-PARTIAL', @summary_linedata);
     my $nodump_FAILED_format = get_summary_format($col_spec, 'nodump-FAILED', @summary_linedata);
     my $nodump_FLUSH_format = get_summary_format($col_spec, 'nodump-FLUSH', @summary_linedata);
+    my $nodump_VAULT_format = get_summary_format($col_spec, 'nodump-VAULT', @summary_linedata);
     my $nodump_NOT_FLUSHED_format = get_summary_format($col_spec, 'nodump-NOT FLUSHED', @summary_linedata);
     my $skipped_format = get_summary_format($col_spec, 'skipped', @summary_linedata);
 
@@ -1112,7 +1209,7 @@ sub output_summary
       . ' ' x $col_spec->[8]->[COLSPEC_PRE_SPACE]
       . '-' x $ts . "\n";
 
-    $self->zsprint("DUMP SUMMARY:\n");
+    $self->zsprint("\n\nDUMP SUMMARY:\n");
     $self->zprint($summary_header);
     $self->zprint(sprintf($title_format, map { $_->[COLSPEC_TITLE] } @$col_spec));
     $self->zprint($summary_dashes);
@@ -1128,6 +1225,8 @@ sub output_summary
 	    $self->zprint(sprintf($nodump_FAILED_format, @data));
 	} elsif ($type eq 'nodump-FLUSH') {
 	    $self->zprint(sprintf($nodump_FLUSH_format, @data));
+	} elsif ($type eq 'nodump-VAULT') {
+	    $self->zprint(sprintf($nodump_VAULT_format, @data));
 	} elsif ($type eq 'nodump-NOT FLUSHED') {
 	    $self->zprint(sprintf($nodump_NOT_FLUSHED_format, @data));
 	} elsif ($type eq 'missing') {
@@ -1136,6 +1235,8 @@ sub output_summary
 	    $self->zprint(sprintf($noflush_format, @data[0..2]));
 	} elsif ($type eq 'skipped') {
 	    $self->zprint(sprintf($skipped_format, @data[0..2]));
+	} else {
+	    die("type eq $type");
 	}
     }
 
@@ -1151,7 +1252,7 @@ sub output_summary
 ##  ('missing', host, disk, '' ..) # MISSING -----
 ##  ('noflush', host, disk, '' ..) # NO FILE TO FLUSH ------
 ##  ('nodump-$msg', host, disk, level, '', out, '--', '',
-##	    '', tapetime, taperate, taperpartial)  # ... {FLUSH|NOT FLUSHED|FAILED|PARTIAL} ...
+##	    '', tapetime, taperate, taperpartial)  # ... {VAULT|FLUSH|NOT FLUSHED|FAILED|PARTIAL} ...
 ##  ('skipped', host, disk, '' ..) # SKIPPED -----
 ##
 ## the taperpartial column is not covered by the columnspec, and "hangs off"
@@ -1198,13 +1299,13 @@ sub get_summary_info
          $dle_info->{'planner'}->{'status'} eq 'fail') or
 	($dle_info->{'driver'} &&
          $dle_info->{'driver'}->{'status'} eq 'fail')) {
-
 	# Do not report driver error if we have a try with dumper
 	my $tries = $alldumps->{$report->{'run_timestamp'}};
 	my $to_report = !defined $tries || !@$tries;
 	foreach my $try ( @$tries ) {
 	    $to_report = 1 if !defined $try->{'dumper'};
 	}
+	#if (!exists $alldumps->{$report->{'run_timestamp'}}) {
 	if ($to_report) {
 	    my @rv;
 	    push @rv, 'nodump-FAILED';
@@ -1239,6 +1340,8 @@ sub get_summary_info
 	  :                               $last_try->{dumper}{level};
 
 	my $orig_size = undef;
+	my $storage = undef;
+	my $flush_or_vault = undef;
 
 	# find the try with the successful dumper entry
 	my $dumper = undef;
@@ -1248,9 +1351,12 @@ sub get_summary_info
 		&& (   $try->{dumper}{status} eq "success"
 		    || $try->{dumper}{status} eq "strange")) {
 		$dumper = $try->{dumper};
+		$storage = $try->{taper}->{storage} if defined $try->{taper} and defined $try->{taper}->{storage};
 		last;
 	    }
 	}
+	# Do not report taper/chunker error if no dumper for this run and driver error
+	next if (!$dumper && $timestamp eq $report->get_timestamp() && $dle_info->{'driver'} && $dle_info->{'driver'}->{'status'} eq 'fail');
 	$orig_size = $dumper->{orig_kb}
 	    if defined $dumper;
 
@@ -1259,40 +1365,48 @@ sub get_summary_info
 	my ($dumper_status) = "";
 	my $saw_dumper = 0; # no dumper will mean this was a flush
 	my $taper_partial = 0; # was the last taper run partial?
+	my $taper_try = undef;
 
 	## Use this loop to set values
 	foreach my $try ( @$tries ) {
-
 	    ## find the outsize for the output summary
-
 	    if (
-		exists $try->{taper}
+		exists $try->{taper} && defined $try->{taper}->{storage}
+		&& (!$storage || $try->{taper}->{storage} eq $storage)
 		&& (   $try->{taper}{status} eq "done"
 		    || $try->{taper}{status} eq "part+partial" )
 	      ) {
 		$taper_partial = 0;
+		$taper_try = $try;
 		$orig_size = $try->{taper}{orig_kb} if !defined($orig_size);
 		$out_size  = $try->{taper}{kb};
 		$tape_time = $try->{taper}{sec};
 		$tape_rate = $try->{taper}{kps};
-		$tape_failure_from = $try->{taper}{failure_from};
-	    } elsif ( exists $try->{taper}
+	    } elsif ( exists $try->{taper} && defined $try->{taper}->{storage}
+		&& (!$storage || $try->{taper}->{storage} eq $storage)
 		&& ( $try->{taper}{status} eq "partial" ) ) {
 
 		$taper_partial = 1;
+		$taper_try = $try;
 		$orig_size = $try->{taper}{orig_kb} if !defined($orig_size);
 		$out_size  = $try->{taper}{kb};
 		$tape_time = $try->{taper}{sec} if !$tape_time;
 		$tape_rate = $try->{taper}{kps} if !$tape_rate;
-		$tape_failure_from = $try->{taper}{failure_from};
-	    } elsif (exists $try->{taper} && ( $try->{taper}{status} eq "fail")) {
+	    } elsif (exists $try->{taper} && defined $try->{taper}->{storage}
+		&& (!$storage || $try->{taper}->{storage} eq $storage)
+		&& ( $try->{taper}{status} eq "fail")) {
+		$taper_try = $try;
 		$tape_time = undef;
 		$tape_rate = undef;
-		$tape_failure_from = $try->{taper}{failure_from};
 	    }
 
+	    if (exists $try->{taper}->{failure_from}) {
+		$tape_failure_from = $try->{taper}->{failure_from};
+	    } else {
+		$tape_failure_from = '';
+	    }
 	    if (!$out_size &&
-		exists $try->{chunker}
+		exists $try->{chunker} && defined $try->{chunker}->{status}
 		&& (   $try->{chunker}{status} eq "success"
 		    || $try->{chunker}{status} eq "partial" )
 	      ) {
@@ -1300,17 +1414,17 @@ sub get_summary_info
 	    }
 
 	    if (!$out_size &&
-		exists $try->{dumper}) {
+		exists $try->{dumper} && defined $try->{dumper}{kb}) {
 		$out_size = $try->{dumper}{kb};
 	    }
 
-	    if ( exists $try->{dumper}) {
+	    if ( exists $try->{dumper} && defined $try->{dumper}{status}) {
 		$saw_dumper = 1;
 		$dumper_status = $try->{dumper}{status};
 	    }
 
 	    ## find the dump time
-	    if ( exists $try->{dumper}
+	    if ( exists $try->{dumper} && defined $try->{dumper}{status}
 		&& exists $try->{dumper}{status}
 		&& (   $try->{dumper}{status} eq "success"
 		    || $try->{dumper}{status} eq "strange")) {
@@ -1318,6 +1432,7 @@ sub get_summary_info
 		$dump_time = $try->{dumper}{sec};
 		$dump_rate = $try->{dumper}{kps};
 	    }
+	    last if defined $taper_try;
 	}
 
 	# sometimes the driver logs an orig_size of -1, which makes the
@@ -1328,24 +1443,33 @@ sub get_summary_info
 	# no compression)
 	my $compression;
 	if (!defined $orig_size || $orig_size == 0 ||
+	    $taper_partial ||
 	    ($out_size/$orig_size > 0.99  && $out_size/$orig_size < 1.01)) {
 	    $compression = '--';
+	    $compression = $self->divzero(0, 0);
 	} else {
 	    $compression =
-	      divzero_col((100 * $out_size), $orig_size, $col_spec->[5]);
+	      $self->divzero_col((100 * $out_size), $orig_size, $col_spec->[5]);
 	}
 
 	## simple formatting macros
+	my $fmt_col_field;
+	if ($self->isa("Amanda::Report::json")) {
+	    $fmt_col_field = sub {
+		my ( $column, $data ) = @_;
+		return $data;
+	    };
+	} else {
+	    $fmt_col_field = sub {
+		my ( $column, $data ) = @_;
 
-	my $fmt_col_field = sub {
-	    my ( $column, $data ) = @_;
-
-	    return sprintf(
-		$col_spec->[$column]->[COLSPEC_FORMAT],
-		$col_spec->[$column]->[COLSPEC_WIDTH],
-		$col_spec->[$column]->[COLSPEC_PREC], $data
-	    );
-	};
+		return sprintf(
+		    $col_spec->[$column]->[COLSPEC_FORMAT],
+		    $col_spec->[$column]->[COLSPEC_WIDTH],
+		    $col_spec->[$column]->[COLSPEC_PREC], $data
+		);
+	    };
+	}
 
 	my $format_space = sub {
 	    my ( $column, $data ) = @_;
@@ -1368,7 +1492,7 @@ sub get_summary_info
 	    push @rv, $orig_size ? $fmt_col_field->(3, $self->tounits($orig_size)) : '';
 	    push @rv, $out_size ? $fmt_col_field->(4, $self->tounits($out_size)) : '';
 	    push @rv, $compression;
-	    push @rv, $dump_time ? $fmt_col_field->(6, mnsc($dump_time)) : "PARTIAL";
+	    push @rv, $dump_time ? $fmt_col_field->(6, $self->mnsc($dump_time)) : "PARTIAL";
 	    push @rv, $dump_rate ? $fmt_col_field->(7, $dump_rate) : "";
 	    if (defined $tape_failure_from and $tape_failure_from eq 'config') {
 		push @rv, $format_space->(8,"");
@@ -1376,7 +1500,7 @@ sub get_summary_info
 	    } else {
 		push @rv, $fmt_col_field->(8,
 		    (defined $tape_time) ?
-			    $tape_time ? mnsc($tape_time) : ""
+			    $tape_time ? $self->mnsc($tape_time) : ""
 			  : "FAILED");
 		push @rv, (defined $tape_rate) ?
 			  $tape_rate ?
@@ -1385,12 +1509,63 @@ sub get_summary_info
 			  : $format_space->(9, "FAILED");
 	    }
 	    push @rv, $taper_partial? " PARTIAL" : ""; # column 10
+	    # if others try with successful taper (sent to another storage)
+	    foreach my $try ( @$tries ) {
+		next if !$try->{'taper'};
+		next if !defined $try->{'taper'}->{status};
+		next if defined $taper_try && $try == $taper_try;
+		#next if $try->{taper}{status} ne "done";
+		push @rvs, [@rv];
+		if ($report->get_flag("amvault_run")) {
+		     @rv = 'nodump-VAULT';
+		} else {
+		    if ($try->{taper}->{storage}) {
+			$storage = $try->{taper}->{storage};
+			$flush_or_vault = $report->{'flush_or_vault'}->{$storage};
+		    }
+		    if (defined $flush_or_vault) {
+			@rv = "nodump-$flush_or_vault";
+		    } else {
+			@rv = 'nodump-FLUSH';
+		    }
+		}
+		#@rv = 'nodump-FLUSH';
+		#@rv = 'nodump-VAULT' if $report->get_flag("amvault_run");
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		$tape_time = $try->{taper}{sec};
+		$tape_rate = $try->{taper}{kps};
+	        push @rv, $fmt_col_field->(8,
+		       (defined $tape_time) ?
+			       $tape_time ? $self->mnsc($tape_time) : ""
+			     : "FAILED");
+	        push @rv, (defined $tape_rate) ?
+		   $tape_rate ?
+		       $fmt_col_field->(9, $tape_rate)
+		     : $format_space->(9, "")
+	           : $format_space->(9, "FAILED");
+	        push @rv, '';
+
+	    }
 	} else {
+	    if ($taper_try && $taper_try->{taper}->{storage} && $report->{'flush_or_vault'}->{$taper_try->{taper}->{storage}}) {
+		$storage = $taper_try->{taper}->{storage};
+		$flush_or_vault = $report->{'flush_or_vault'}->{$storage};
+	    }
 	    my $message = $saw_dumper?
 			    ($dumper_status eq 'failed') ? 'FAILED' : 'PARTIAL'
 			  : (defined $tape_failure_from and
 			     $tape_failure_from eq 'config') ? 'NOT FLUSHED'
-							     : 'FLUSH';
+			  : $report->get_flag("amvault_run") ? 'VAULT'
+			  : $report->get_flag("amflush_run") ? 'FLUSH'
+			  : defined $flush_or_vault          ? $flush_or_vault
+							     : 'FAILED';
 	    push @rv, "nodump-$message";
 	    push @rv, $hostname;
 	    push @rv, $disk_out;
@@ -1407,7 +1582,7 @@ sub get_summary_info
 	    } else {
 	       push @rv, $fmt_col_field->(8,
 		       (defined $tape_time) ?
-			       $tape_time ? mnsc($tape_time) : ""
+			       $tape_time ? $self->mnsc($tape_time) : ""
 			     : "FAILED");
 	       push @rv, (defined $tape_rate) ?
 		   $tape_rate ?
@@ -1416,6 +1591,43 @@ sub get_summary_info
 	         : $format_space->(9, "FAILED");
 	    }
 	    push @rv, $taper_partial? " PARTIAL" : "";
+	    # if others try with successful taper (sent to another storage)
+	    foreach my $try ( @$tries ) {
+		next if !$try->{'taper'};
+		next if defined $taper_try and $try eq $taper_try;
+		next if !defined $try->{taper}{status} or $try->{taper}{status} ne "done";
+		push @rvs, [@rv];
+
+		if ($try->{taper}->{storage} && $report->{'flush_or_vault'}->{$try->{taper}->{storage}}) {
+		    $storage = $try->{taper}->{storage};
+		    $flush_or_vault = $report->{'flush_or_vault'}->{$storage};
+		    @rv = "nodump-$flush_or_vault";
+		} else {
+		    @rv = 'nodump-FLUSH';
+		    @rv = 'nodump-VAULT' if $report->get_flag("amvault_run");
+		}
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		push @rv, '';
+		$tape_time = $try->{taper}{sec};
+		$tape_rate = $try->{taper}{kps};
+	        push @rv, $fmt_col_field->(8,
+		       (defined $tape_time) ?
+			       $tape_time ? $self->mnsc($tape_time) : ""
+			     : "FAILED");
+	        push @rv, (defined $tape_rate) ?
+		   $tape_rate ?
+		       $fmt_col_field->(9, $tape_rate)
+		     : $format_space->(9, "")
+	           : $format_space->(9, "FAILED");
+	        push @rv, " ";
+
+	    }
 	}
 	push @rvs, [@rv];
     }
@@ -1669,7 +1881,7 @@ sub print_if_def
     foreach my $msg (@$msgs) {
         $self->zprint("  $msg\n");
     }
-    $self->zprint("\n");
 }
 
+
 1;
