@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
+# Sunnyvale, CA 94085, or: http://www.zmanda.com
 
 package Amanda::ScanInventory;
 
@@ -41,7 +42,9 @@ use base qw(Exporter);
 our @EXPORT_OK = qw($DEFAULT_CHANGER);
 
 use Amanda::Paths;
-use Amanda::Util;
+use Amanda::Tapelist;
+use Amanda::Config qw( :getconf );
+use Amanda::Util qw( match_labelstr );
 use Amanda::Device qw( :constants );
 use Amanda::Debug qw( debug );
 use Amanda::Changer;
@@ -79,12 +82,11 @@ sub new {
 	tapelist    => $tapelist,
         interactivity => $interactivity,
 	seen        => {},
-	scan_num    => 0
+	scan_num    => 0,
     };
     return bless ($self, $class);
 }
 
-
 sub scan {
     my $self = shift;
     my %params = @_;
@@ -118,14 +120,13 @@ sub _scan {
     my $action_slot;
     my $res;
     my $label;
-    my %seen = ();
     my $inventory;
     my $current;
     my $new_slot;
     my $poll_src;
     my $scan_running = 0;
     my $interactivity_running = 0;
-    my $restart_scan = 0;
+    my $restart_scan = 0;		# if a scan must be restarted
     my $restart_scan_changer = undef;
     my $abort_scan = undef;
     my $last_err = undef; # keep the last meaningful error, the one reported
@@ -135,33 +136,31 @@ sub _scan {
     my $slot_scanned;
     my $remove_undef_state = 0;
     my $result_cb = $params{'result_cb'};
+    my $new_inventory = 0;
+    my $restart_scan_running = 0;	# if restart_scan is running
 
     my $steps = define_steps
 	cb_ref => \$result_cb;
 
-    step get_first_inventory => sub {
+    step init => sub {
 	$scan_running = 1;
-	$self->{'chg'}->inventory(inventory_cb => $steps->{'got_first_inventory'});
+	$steps->{'should_get_inventory'}->();
     };
 
-    step got_first_inventory => sub {
-	(my $err, $inventory) = @_;
-
-	if ($err && $err->notimpl) {
-	    #inventory not implemented
-	    die("no inventory");
-	} elsif ($err and $err->fatal) {
-	    #inventory fail
-	    return $steps->{'call_result_cb'}->($err, undef);
+    step should_get_inventory => sub {
+	$new_inventory = 0;
+	if ($res || ($self->{'slots'} && @{$self->{'slots'}} && defined $self->{'slots'}->[0])) {
+	    return $steps->{'action'}->();
 	}
-
-	# continue parsing the inventory
-	$steps->{'parse_inventory'}->($err, $inventory);
+	return $steps->{'get_inventory'}->();
     };
 
     step restart_scan => sub {
 	$restart_scan = 0;
 
+	return if $restart_scan_running;
+	$restart_scan_running = 1;
+
 	# Reload the tapelist at every scan.
 	$self->{'tapelist'}->reload(0);
 
@@ -202,9 +201,9 @@ sub _scan {
 	if ($remove_undef_state) {
 	    for my $i (0..(scalar(@$inventory)-1)) {
 		my $slot = $inventory->[$i]->{slot};
-		if (exists($seen{$slot}) &&
+		if (exists($self->{seen}->{$slot}) &&
 		    !defined($inventory->[$i]->{state})) {
-		    delete $seen{$slot}
+		    delete $self->{seen}->{$slot};
 		}
 	    }
 	    $remove_undef_state = 0;
@@ -214,30 +213,80 @@ sub _scan {
 	for my $i (0..(scalar(@$inventory)-1)) {
 	    my $sl = $inventory->[$i];
 	    my $slot = $sl->{slot};
-	    if ($seen{$slot} &&
-		!defined ($seen{$slot}->{'failed'}) &&
+	    if ($self->{seen}->{$slot} &&
+		!defined ($self->{seen}->{$slot}->{'failed'}) &&
 		defined($sl->{'state'}) &&
-		(($seen{$slot}->{'device_status'} != $sl->{'device_status'}) ||
-		 (defined $seen{$slot}->{'device_status'} &&
-		  $seen{$slot}->{'device_status'} == $DEVICE_STATUS_SUCCESS &&
-		  $seen{$slot}->{'f_type'} != $sl->{'f_type'}) ||
-		 (defined $seen{$slot}->{'device_status'} &&
-		  $seen{$slot}->{'device_status'} == $DEVICE_STATUS_SUCCESS &&
-		  defined $seen{$slot}->{'f_type'} &&
-		  $seen{$slot}->{'f_type'} == $Amanda::Header::F_TAPESTART &&
-		  $seen{$slot}->{'label'} ne $sl->{'label'}))) {
-		delete $seen{$slot};
+		!($self->{seen}->{$slot}->{'device_status'} == $DEVICE_STATUS_SUCCESS &&
+		  ($sl->{'device_status'} & $DEVICE_STATUS_DEVICE_ERROR ||
+		   $sl->{'device_status'} & $DEVICE_STATUS_VOLUME_ERROR)) &&
+		(($self->{seen}->{$slot}->{'device_status'} != $sl->{'device_status'}) ||
+		 (defined $self->{seen}->{$slot}->{'device_status'} &&
+		  $self->{seen}->{$slot}->{'device_status'} == $DEVICE_STATUS_SUCCESS &&
+		  $self->{seen}->{$slot}->{'f_type'} != $sl->{'f_type'}) ||
+		 (defined $self->{seen}->{$slot}->{'device_status'} &&
+		  $self->{seen}->{$slot}->{'device_status'} == $DEVICE_STATUS_SUCCESS &&
+		  defined $self->{seen}->{$slot}->{'f_type'} &&
+		  $self->{seen}->{$slot}->{'f_type'} == $Amanda::Header::F_TAPESTART &&
+		  $self->{seen}->{$slot}->{'label'} ne $sl->{'label'}))) {
+		delete $self->{seen}->{$slot};
 	    }
 	}
+	$new_inventory = 1;
+	$steps->{'action'}->();
+    };
 
-	($action, $action_slot) = $self->analyze($inventory, \%seen, $res);
+    step action => sub {
+	$self->{'slot-error-message'} = undef;
 
-	if ($action == Amanda::ScanInventory::SCAN_DONE) {
-	    return $steps->{'call_result_cb'}->(undef, $res);
+	if ($res) {
+	    my $dev = $res->{'device'};
+	    if ($dev) {
+		my $volume_header = $dev->volume_header;
+		if ($dev->status == $DEVICE_STATUS_SUCCESS) {
+		    my $label = $volume_header->{'name'};
+		    if ($self->is_reusable_volume(label => $label, new_label_ok => 1)) {
+			    $action = Amanda::ScanInventory::SCAN_DONE;
+			    return $steps->{'call_result_cb'}->(undef, $res);
+		    } else {
+			my $vol_tle = $self->{'tapelist'}->lookup_tapelabel($label);
+			if ($vol_tle) {
+			    if ($self->volume_is_new_labelled($vol_tle, {label => $label, barcode => $res->{barcode}})) {
+				$action = Amanda::ScanInventory::SCAN_DONE;
+				return $steps->{'call_result_cb'}->(undef, $res);
+			    }
+			}
+		   }
+		} else {
+		    if ($self->volume_is_labelable({ device_status => $dev->status,
+						     f_type  => $volume_header->{'type'},
+						     label   => $label,
+						     slot    => $res->{'this_slot'},
+						     barcode => $res->{'barcode'},
+						     meta    => $res->{'meta'} })) {
+			$action = Amanda::ScanInventory::SCAN_DONE;
+			return $steps->{'call_result_cb'}->(undef, $res);
+		    }
+		}
+	    }
+	}
+	delete $self->{'use_sl'};
+	if (!$self->{'slots'} || !@{$self->{'slots'}} || !defined $self->{'slots'}->[0]) {
+	    $self->{'slots'} = $self->analyze($inventory, $self->{seen});
+	}
+	if (@{$self->{'slots'}}) {
+	    $self->{'use_sl'} = shift @{$self->{'slots'}};
+	}
+	if ($self->{'use_sl'} && defined $self->{'use_sl'}->{'slot'}) {
+	    $action = Amanda::ScanInventory::SCAN_LOAD;
+	    $action_slot = $self->{'use_sl'}->{'slot'};
+	} elsif ($self->{'scan_conf'}->{'ask'}) {
+	    $action = Amanda::ScanInventory::SCAN_ASK_POLL;
+	} else {
+	    $action = Amanda::ScanInventory::SCAN_FAIL;
 	}
 
 	if (defined $res) {
-	    $res->release(finished_cb => $steps->{'released'});
+	    $res->release(need_another => 1, finished_cb => $steps->{'released'});
 	    $res = undef;
 	} else {
 	    $steps->{'released'}->();
@@ -249,12 +298,19 @@ sub _scan {
 	    $slot_scanned = $action_slot;
 	    $self->_user_msg(scan_slot => 1,
 			     slot => $slot_scanned);
+	    $self->{'slot-error-message'} = $self->{seen}->{$slot_scanned}->{'device_error'};
+
 	    return $self->{'chg'}->load(
 			slot => $slot_scanned,
 			set_current => $params{'set_current'},
 			res_cb => $steps->{'slot_loaded'});
 	}
 
+	if (!$new_inventory) {
+	    delete $self->{'slots'};
+	    return $steps->{'get_inventory'}->();
+	}
+
 	my $err;
 	if ($last_err) {
 	    $err = $last_err;
@@ -274,15 +330,46 @@ sub _scan {
     step slot_loaded => sub {
 	(my $err, $res) = @_;
 
+	$self->{'slot_loaded_err'} = $err;
+
 	# we don't responsd to abort_scan or restart_scan here, since we
 	# have an open reservation that we should deal with.
 
+	# change status of slot in error if that one succeeded.
+	if (defined $self->{'slot-error-message'} and
+	    $res and defined $res->{'device'} and
+	    $self->{'slot-error-message'} ne $res->{'device'}->error) {
+	    # mark all unseen slots with that error message as unknown state
+	    for my $i (0..(scalar(@$inventory)-1)) {
+		my $sl = $inventory->[$i];
+		next if $self->{seen}->{$sl->{slot}};
+		next if !defined $self->{'slot-error-message'} ||
+			!defined $sl->{'device_error'} ||
+			$self->{'slot-error-message'} ne $sl->{'device_error'};
+		# mark the slot as unknown
+		$inventory->[$i] = { slot  => $sl->{'slot'},
+				     state => $sl->{'state'}};
+	    }
+	    if ($self->{'chg'}->can("set_error_to_unknown")) {
+		$self->{'chg'}->set_error_to_unknown(
+			error_message => $self->{'slot-error-message'},
+			set_to_unknown_cb => $steps->{'set_to_unknown_cb'});
+	    }
+	} else {
+	    return $steps->{'set_to_unknown_cb'}->();
+	}
+    };
+
+    step set_to_unknown_cb => sub {
+	my $err = $self->{'slot_loaded_err'};
+	$self->{'slot_loaded_err'} = undef;
+
 	my $label;
 	if ($res && defined $res->{device} &&
 	    $res->{device}->status == $DEVICE_STATUS_SUCCESS) {
 	    $label = $res->{device}->volume_label;
 	}
-	my $relabeled = !defined($label) || $label !~ /$self->{'labelstr'}/;
+	my $relabeled = !defined($label) || !match_labelstr($self->{'labelstr'}, $self->{'autolabel'}, $label, $res->{'barcode'}, $res->{'meta'}, $self->{'chg'}->{'storage'}->{'storage_name'});
 	$self->_user_msg(slot_result => 1,
 			 slot => $slot_scanned,
 			 label => $label,
@@ -297,8 +384,9 @@ sub _scan {
 
 	    # The slot did not contain the volume we wanted, so mark it
 	    # as seen and try again.
-	    $seen{$slot_scanned} = {
+	    $self->{seen}->{$slot_scanned} = {
 			device_status => $res->{device}->status,
+			device_error => $res->{device}->error_or_status,
 			f_type => $f_type,
 			label  => $res->{device}->volume_label
 	    };
@@ -311,7 +399,7 @@ sub _scan {
 				message => $res->{device}->error_or_status());
 	    }
 	} else {
-	    $seen{$slot_scanned} = { failed => 1 };
+	    $self->{seen}->{$slot_scanned} = { failed => 1 };
 	    if ($err->volinuse) {
 		# Scan semantics for volinuse is different than changer.
 		# If a slot with unknown label is loaded then we map
@@ -327,14 +415,13 @@ sub _scan {
 	my ($err) = @_;
 
 	# TODO: handle error
-
 	# throw out the inventory result and move on if the situation has
 	# changed while we were loading a volume
 	return $steps->{'abort_scan'}->() if $abort_scan;
 	return $steps->{'restart_scan'}->() if $restart_scan;
 
 	$new_slot = $current;
-	$steps->{'get_inventory'}->();
+	$steps->{'should_get_inventory'}->();
     };
 
     step handle_error => sub {
@@ -390,6 +477,7 @@ sub _scan {
 
     step scan_next => sub {
 	my ($scan_method, $err, $continue_cb) = @_;
+	$restart_scan_running = 0;
 
 	if ($scan_method == SCAN_ASK && !defined $self->{'interactivity'}) {
 	    $scan_method = SCAN_FAIL;
@@ -425,6 +513,9 @@ sub _scan {
 	if ($poll_src) {
 	    $poll_src->remove();
 	    $poll_src = undef;
+	    if ($self->{'chg'}->{'scan-require-update'}) {
+		$remove_undef_state = 1;
+	    }
 	    return $steps->{'restart_scan'}->();
 	}
     };
@@ -482,7 +573,7 @@ sub _scan {
 		return $steps->{'scan_interactivity'}->("$new_chg");
 	    }
 	    $restart_scan_changer = $new_chg;
-	    %seen = ();
+	    $self->{seen} = {};
 	} else {
 	    $remove_undef_state = 1;
 	}
@@ -525,7 +616,8 @@ sub _scan {
 	    return $result_cb->($err, $res);
 	}
 	$label = $res->{'device'}->volume_label;
-	if (!defined($label) || $label !~ /$self->{'labelstr'}/) {
+	if (!defined($label) ||
+	    !match_labelstr($self->{'labelstr'}, $self->{'autolabel'}, $label, $res->{'barcode'}, $res->{'meta'}, $self->{'chg'}->{'storage'}->{'storage_name'})) {
 	    $res->get_meta_label(finished_cb => $steps->{'got_meta_label'});
 	    return;
 	}
@@ -538,17 +630,48 @@ sub _scan {
 	if (defined $err) {
 	    return $result_cb->($err, $res);
 	}
-	($label, my $make_err) = $res->make_new_tape_label(meta => $meta);
+	($label, my $make_err, my $not_fatal) = $res->make_new_tape_label(meta => $meta);
 	if (!defined $label) {
-	    # make this fatal, rather than silently skipping new tapes
-	    $self->{'scanning'} = 0;
-	    return $result_cb->($make_err, $res);
+	    if ($not_fatal) {
+		# must be logged
+		$self->_user_msg(slot_result => 1,
+				 slot => $slot_scanned,
+				 err => "Can't label slot $slot_scanned: $make_err");
+		my $res1 = $res;
+		$res = undef;
+		return $res1->release(need_another => 1, finished_cb => $steps->{'get_inventory'});
+	    } else {
+		# make this fatal, rather than silently skipping new tapes
+		$self->{'scanning'} = 0;
+		return $result_cb->($make_err, $res);
+	    }
 	}
 	$self->{'scanning'} = 0;
 	return $result_cb->(undef, $res, $label, $ACCESS_WRITE, 1);
     };
 }
 
+sub volume_is_new_labelled {
+    my $self = shift;
+    my $tle = shift;
+    my $sl = shift;
+
+    if ($tle->{'pool'} && $tle->{'pool'} ne $self->{'tapepool'}) {
+	return 0;
+    }
+    if (!$tle->{'pool'} &&
+	     !match_labelstr($self->{'labelstr'}, $self->{'autolabel'}, $sl->{'label'}, $sl->{'barcode'}, $sl->{'meta'}, $self->{'chg'}->{'storage'}->{'storage_name'})) {
+	return 0;
+    }
+    if ($tle->{'datestamp'} ne '0') {
+	return 0;
+    }
+    if (!$tle->{'reuse'}) {
+	return 0;
+    }
+    return 1;
+}
+
 sub volume_is_labelable {
     my $self = shift;
     my $sl = shift;
@@ -556,6 +679,8 @@ sub volume_is_labelable {
     my $f_type = $sl->{'f_type'};
     my $label = $sl->{'label'};
     my $slot = $sl->{'slot'};
+    my $barcode = $sl->{'barcode'};
+    my $meta = $sl->{'meta'};
     my $chg = $self->{'chg'};
     my $autolabel = $chg->{'autolabel'};
 
@@ -565,59 +690,64 @@ sub volume_is_labelable {
 	     defined $f_type and
 	     $f_type == $Amanda::Header::F_EMPTY) {
 	if (!$autolabel->{'empty'}) {
-	    $self->_user_msg(slot_result  => 1,
-			     empty        => 1,
-			     slot         => $slot);
+#	    $self->_user_msg(slot_result  => 1,
+#			     empty        => 1,
+#			     slot         => $slot);
 	    return 0;
 	}
     } elsif ($dev_status & $DEVICE_STATUS_VOLUME_UNLABELED and
 	     defined $f_type and
 	     $f_type == $Amanda::Header::F_WEIRD) {
 	if (!$autolabel->{'non_amanda'}) {
-	    $self->_user_msg(slot_result  => 1,
-			     non_amanda   => 1,
-			     slot         => $slot);
+#	    $self->_user_msg(slot_result  => 1,
+#			     non_amanda   => 1,
+#			     slot         => $slot);
 	    return 0;
 	}
     } elsif ($dev_status & $DEVICE_STATUS_VOLUME_ERROR) {
 	if (!$autolabel->{'volume_error'}) {
-	    $self->_user_msg(slot_result  => 1,
-			     volume_error => 1,
-			     err          => $sl->{'device_error'},
-			     slot         => $slot);
+#	    $self->_user_msg(slot_result  => 1,
+#			     volume_error => 1,
+#			     err          => $sl->{'device_error'},
+#			     slot         => $slot);
 	    return 0;
 	}
     } elsif ($dev_status != $DEVICE_STATUS_SUCCESS) {
-	    $self->_user_msg(slot_result  => 1,
-			     not_success  => 1,
-			     err          => $sl->{'device_error'},
-			     slot         => $slot);
+#	    $self->_user_msg(slot_result  => 1,
+#			     not_success  => 1,
+#			     err          => $sl->{'device_error'},
+#			     slot         => $slot);
+	return 0;
+    } elsif ($dev_status == $DEVICE_STATUS_SUCCESS and
+	     $f_type != $Amanda::Header::F_TAPESTART) {
 	return 0;
     } elsif ($dev_status == $DEVICE_STATUS_SUCCESS and
 	     $f_type == $Amanda::Header::F_TAPESTART) {
-	if ($label !~ /$self->{'labelstr'}/) {
+	if (!match_labelstr($self->{'labelstr'}, $autolabel, $label,
+			    $barcode, $meta, $self->{'chg'}->{'storage'}->{'storage_name'})) {
 	    if (!$autolabel->{'other_config'}) {
-		$self->_user_msg(slot_result  => 1,
-				 label        => $label,
-				 labelstr     => $self->{'labelstr'},
-				 does_not_match_labelstr => 1,
-				 slot         => $slot);
+#	        $self->_user_msg(slot_result  => 1,
+#			         label        => $label,
+#			         labelstr     => $self->{'labelstr'}->{'template'},
+#			         does_not_match_labelstr => 1,
+#			         slot         => $slot);
 		return 0;
 	    }
 	} else {
-	   my $vol_tle = $self->{'tapelist'}->lookup_tapelabel($label);
-	   if (!$vol_tle) {
-		$self->_user_msg(slot_result  => 1,
-				 label        => $label,
-				 not_in_tapelist => 1,
-				 slot         => $slot);
+	    my $vol_tle = $self->{'tapelist'}->lookup_tapelabel($label);
+	    if (!$vol_tle) {
+#		$self->_user_msg(slot_result     => 1,
+#				 label           => $label,
+#				 not_in_tapelist => 1,
+#				 slot            => $slot);
 		return 0;
-	   }
+	    }
 	}
     }
 
     return 1;
 }
+
 package Amanda::ScanInventory::Config;
 
 sub new {
