@@ -14,7 +14,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 =head1 NAME
@@ -31,17 +31,19 @@ controls one or more workers (L<Amanda::Taper::Worker>).
 
 The controller create an L<Amanda::Taper::Worker> object for each
 START_TAPER command it receive. It dispatch the following commands
-to the correct worker.  
+to the correct worker.
 
 =cut
 
-use lib '@amperldir@';
 use strict;
 use warnings;
 
 package Amanda::Taper::Controller;
 
 use POSIX qw( :errno_h );
+use Amanda::Policy;
+use Amanda::Debug qw( debug );
+use Amanda::Storage;
 use Amanda::Changer;
 use Amanda::Config qw( :getconf config_dir_relative );
 use Amanda::Header;
@@ -67,7 +69,7 @@ sub new {
 	# filled in at start
 	proto => undef,
 	tapelist => $params{'tapelist'},
-
+	storage_name => $params{'storage_name'},
 	worker => {},
     }, $class;
     return $self;
@@ -116,9 +118,33 @@ sub start {
 	message_cb => $message_cb,
 	message_obj => $self,
 	debug => $Amanda::Config::debug_taper?'taper/driver':'',
+	no_read => 1,
     );
 
-    my $changer = Amanda::Changer->new(undef, tapelist => $self->{'tapelist'});
+    my ($storage) = Amanda::Storage->new(
+					storage_name => $self->{'storage_name'},
+					tapelist => $self->{'tapelist'});
+    if ($storage->isa("Amanda::Changer::Error")) {
+	$self->{'proto'}->send(Amanda::Taper::Protocol::TAPE_ERROR,
+		worker_name => "SETUP",
+		message => "$storage");
+
+	# log the error (note that the message is intentionally not quoted)
+	log_add($L_ERROR, "no-tape error [$storage]");
+
+	# wait for it to be transmitted, then exit
+	$self->{'proto'}->stop(finished_cb => sub {
+	    Amanda::MainLoop::quit();
+	});
+
+	# don't finish start()ing
+	return;
+    }
+    if ($storage->{'erase_volume'}) {
+	$storage->erase_no_retention();
+	$self->{'tapelist'}->reload();
+    }
+    my $changer = $storage->{'chg'};
     if ($changer->isa("Amanda::Changer::Error")) {
 	# send a TAPE_ERROR right away
 	$self->{'proto'}->send(Amanda::Taper::Protocol::TAPE_ERROR,
@@ -133,17 +159,22 @@ sub start {
 	    Amanda::MainLoop::quit();
 	});
 
+	$storage->quit();
+
 	# don't finish start()ing
 	return;
     }
 
     my $interactivity = Amanda::Interactivity->new(
-					name => getconf($CNF_INTERACTIVITY));
-    my $scan_name = getconf($CNF_TAPERSCAN);
+					name => $storage->{'interactivity'});
+    my $scan_name = $storage->{'taperscan_name'};
+    $self->{'storage'} = $storage;
     $self->{'taperscan'} = Amanda::Taper::Scan->new(algorithm => $scan_name,
+					    storage => $storage,
 					    changer => $changer,
 					    interactivity => $interactivity,
 					    tapelist => $self->{'tapelist'});
+    $self->{'proto'}->start_read();
 }
 
 sub quit {
@@ -151,6 +182,7 @@ sub quit {
     my %params = @_;
     my @errors = ();
     my @worker = ();
+    my $worker;
 
     my $steps = define_steps
 	cb_ref => \$params{'finished_cb'};
@@ -162,17 +194,34 @@ sub quit {
     };
 
     step quit_scribe => sub {
-	my $worker = shift @worker;
-	if (defined $worker and defined $worker->{'scribe'}) {
-	    $worker->{'scribe'}->quit(finished_cb => sub {
-	        my ($err) = @_;
-	        push @errors, $err if ($err);
+	$worker = shift @worker;
+	if (defined $worker) {
+	    if (defined $worker->{'scribe'}) {
+		return $worker->{'scribe'}->quit(finished_cb => sub {
+	            my ($err) = @_;
+	            push @errors, $err if ($err);
 
-	        $steps->{'quit_scribe'}->();
-	    });
-	} else {
-	    $steps->{'stop_proto'}->();
+	            $steps->{'quit_clerk'}->();
+		});
+	    }
+	    $steps->{'quit_clerk'}->();
 	}
+	$steps->{'stop_proto'}->();
+    };
+
+    step quit_clerk => sub {
+	if (defined $worker->{'src'}->{'clerk'}) {
+	    return $worker->{'src'}->{'clerk'}->quit(finished_cb => sub {
+	            my ($err) = @_;
+	            push @errors, $err if ($err);
+		    if (defined $worker->{'src'}->{'storage'}) {
+			$worker->{'src'}->{'storage'}->quit();
+		    }
+
+	            $steps->{'quit_scribe'}->();
+	    });
+	}
+	$steps->{'quit_scribe'}->();
     };
 
     step stop_proto => sub {
@@ -186,6 +235,7 @@ sub quit {
 
     step done => sub {
 	$self->{'taperscan'}->quit() if defined $self->{'taperscan'};
+	$self->{'storage'}->quit() if defined $self->{'storage'};
 	if (@errors) {
 	    $params{'finished_cb'}->(join("; ", @errors));
 	} else {
@@ -201,7 +251,7 @@ sub msg_START_TAPER {
     my $self = shift;
     my ($msgtype, %params) = @_;
 
-    my $worker = new Amanda::Taper::Worker($params{'worker_name'}, $self,
+    my $worker = new Amanda::Taper::Worker($params{'taper_name'}, $params{'worker_name'}, $self,
 				  $params{'timestamp'});
 
     $self->{'worker'}->{$params{'worker_name'}} = $worker;
@@ -226,6 +276,22 @@ sub msg_PORT_WRITE {
     $worker->PORT_WRITE(@_);
 }
 
+sub msg_SHM_WRITE {
+    my $self = shift;
+    my ($msgtype, %params) = @_;
+
+    my $worker = $self->{'worker'}->{$params{'worker_name'}};
+    $worker->SHM_WRITE(@_);
+}
+
+sub msg_VAULT_WRITE {
+    my $self = shift;
+    my ($msgtype, %params) = @_;
+
+    my $worker = $self->{'worker'}->{$params{'worker_name'}};
+    $worker->VAULT_WRITE(@_);
+}
+
 sub msg_START_SCAN {
     my $self = shift;
     my ($msgtype, %params) = @_;
@@ -274,6 +340,14 @@ sub msg_CLOSE_VOLUME {
     $worker->CLOSE_VOLUME(@_);
 }
 
+sub msg_CLOSE_SOURCE_VOLUME {
+    my $self = shift;
+    my ($msgtype, %params) = @_;
+
+    my $worker = $self->{'worker'}->{$params{'worker_name'}};
+    $worker->CLOSE_SOURCE_VOLUME(@_);
+}
+
 sub msg_TAKE_SCRIBE_FROM {
     my $self = shift;
     my ($msgtype, %params) = @_;
