@@ -1,4 +1,5 @@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,7 +15,7 @@
 # along with this library; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 package Amanda::Taper::Scan::traditional;
@@ -39,6 +40,7 @@ use Amanda::Device qw( :constants );
 use Amanda::Header;
 use Amanda::Debug qw( :logging );
 use Amanda::MainLoop;
+use Amanda::Util qw( match_labelstr );
 
 sub new {
     my $class = shift;
@@ -114,7 +116,7 @@ sub scan_result {
 	# we may not ever have looked for this, the oldest reusable volume, if
 	# the changer is not fast-searchable.  But we'll tell the user about it
 	# anyway.
-	my $oldest_reusable = $self->oldest_reusable_volume(new_label_ok => 0);
+	my $oldest_reusable = $self->oldest_reusable_volume();
 	$self->_user_msg(scan_failed => 1,
 			 expected_label => $oldest_reusable,
 			 expected_new => 1);
@@ -139,7 +141,6 @@ sub stage_1 {
     step setup => sub {
 	debug("Amanda::Taper::Scan::traditional stage 1: search for oldest reusable volume");
 	$oldest_reusable = $self->oldest_reusable_volume(
-	    new_label_ok => 0,      # stage 1 never selects new volumes
 	);
 
 	if (!defined $oldest_reusable) {
@@ -194,6 +195,15 @@ sub stage_1 {
 	    if ($err->failed and $err->notfound) {
 		debug("Amanda::Taper::Scan::traditional oldest reusable volume not found");
 		return $self->stage_2($result_cb);
+	    } elsif ($err->failed and $err->volinuse) {
+		debug("Amanda::Taper::Scan::traditional oldest reusable volume in use");
+		return $self->stage_2($result_cb);
+	    } elsif ($err->failed and $err->invalid) {
+		debug("Amanda::Taper::Scan::traditional oldest reusable volume is not accessible");
+		return $self->stage_2($result_cb);
+	    } elsif ($err->failed and $err->invalid) {
+		debug("Amanda::Taper::Scan::traditional oldest reusable volume is in an invalid slot");
+		return $self->stage_2($result_cb);
 	    } else {
 		return $self->scan_result(error => $err,
 			res => $res, result_cb => $result_cb);
@@ -210,12 +220,22 @@ sub stage_1 {
 
         # go on to stage 2 if we didn't get the expected volume
         my $label = $res->{'device'}->volume_label;
-        my $labelstr = $self->{'labelstr'};
-        if ($label !~ /$labelstr/) {
+	if ($label ne $oldest_reusable) {
             warning "Searched for label '$oldest_reusable' but found a volume labeled '$label'";
             return $self->release_and_stage_2($res, $result_cb);
         }
 
+        my $labelstr = $self->{'labelstr'};
+        my $autolabel = $self->{'autolabel'};
+	my $barcode = $res->{'barcode'} || "";
+	my $meta = $res->{'meta'} || "";
+	if (!match_labelstr($labelstr, $autolabel, $label,
+			    $barcode, $meta, $self->{'chg'}->{'storage'})) {
+            warning "Oldest reusable volume '$oldest_reusable' do not match the labelstr '" .
+				$labelstr->{'match_autolabel'} ? $autolabel->{'template'} : $labelstr->{'template'} . "'";
+            return $self->release_and_stage_2($res, $result_cb);
+        }
+
 	# great! -- volume found
 	return $self->scan_result(res => $res, label => $oldest_reusable,
 		    mode => $ACCESS_WRITE, is_new => 0, result_cb => $result_cb);
@@ -313,6 +333,7 @@ sub stage_2 {
 	    # or if we loaded the 'current' slot and it was invalid (this happens if
 	    # the user changes 'use-slots', for example
 	    $ignore_error = 1 if ($loaded_current && $err->invalid);
+	    $ignore_error = 1 if (defined($err->{'slot'}) && $err->invalid);
 	    $ignore_error = 1 if ($err->empty);
 
 	    if ($ignore_error) {
@@ -342,58 +363,96 @@ sub stage_2 {
 	my $status = $dev->status;
 	my $labelstr = $res->{'chg'}->{'labelstr'};
 	my $label;
+	my $barcode = $res->{'barcode'} || "";
+	my $meta = $res->{'meta'} || "";
 	my $autolabel = $res->{'chg'}->{'autolabel'};
 
 	if ($status == $DEVICE_STATUS_SUCCESS) {
             $label = $dev->volume_label;
 
-            if ($label !~ /$labelstr/) {
-	        if (!$autolabel->{'other_config'}) {
-		    $self->_user_msg(slot_result             => 1,
-				     does_not_match_labelstr => 1,
-				     labelstr                => $labelstr,
-				     slot                    => $slot,
-				     label                   => $label,
-				     res                     => $res);
-		    return $steps->{'try_continue'}->();
-	        }
-            } else {
-	        # verify that the label is in the tapelist
-	        my $tle = $self->{'tapelist'}->lookup_tapelabel($label);
-	        if (!$tle) {
+	    # verify that the label is in the tapelist
+	    my $tle = $self->{'tapelist'}->lookup_tapelabel($label);
+	    if (!$tle) {
+		if (!match_labelstr($labelstr, $autolabel, $label, $barcode,
+                                    $meta, $self->{'chg'}->{'storage'})) {
+		    if (!$autolabel->{'other_config'}) {
+		        $self->_user_msg(slot_result             => 1,
+				         does_not_match_labelstr => 1,
+				         labelstr                => $labelstr->{'match_autolabel'} ? $autolabel->{'template'} : $labelstr->{'template'},,
+				         slot                    => $slot,
+				         label                   => $label,
+				         res                     => $res);
+		        return $steps->{'try_continue'}->();
+		    }
+		} else {
 		    $self->_user_msg(slot_result     => 1,
 				     not_in_tapelist => 1,
 				     slot            => $slot,
 				     label           => $label,
 				     res             => $res);
 		    return $steps->{'try_continue'}->();
-	        }
-
-	        # see if it's reusable
-	        if (!$self->is_reusable_volume(label => $label, new_label_ok => 1)) {
-		    $self->_user_msg(slot_result => 1,
-				     active      => 1,
-				     slot        => $slot,
-				     label       => $label,
-				     res         => $res);
+		}
+	    } else {
+		if ($tle->{'config'} &&
+		    $tle->{'config'} ne Amanda::Config::get_config_name()) {
+		    $self->_user_msg(slot_result     => 1,
+				     other_config    => 1,
+				     config          => $tle->{'config'},
+				     slot            => $slot,
+				     label           => $label,
+				     res             => $res);
 		    return $steps->{'try_continue'}->();
-	        }
-	        $self->_user_msg(slot_result => 1,
+		}
+
+		if ($tle->{'pool'} &&
+		    $tle->{'pool'} ne $self->{'tapepool'}) {
+		    $self->_user_msg(slot_result     => 1,
+				     other_pool      => 1,
+				     pool            => $tle->{'pool'},
+				     slot            => $slot,
+				     label           => $label,
+				     res             => $res);
+		    return $steps->{'try_continue'}->();
+		}
+
+		if (!$tle->{'pool'} &&
+		    !match_labelstr($labelstr, $autolabel, $label, $barcode,
+				    $meta, $self->{'chg'}->{'storage'})) {
+		    $self->_user_msg(slot_result             => 1,
+				     does_not_match_labelstr => 1,
+				     labelstr                => $labelstr->{'match_autolabel'} ? $autolabel->{'template'} : $labelstr->{'template'},
+				     slot                    => $slot,
+				     label                   => $label,
+				     res                     => $res);
+		    return $steps->{'try_continue'}->();
+		}
+
+		# see if it's reusable
+		if (!$self->is_reusable_volume(label => $label, new_label_ok => 1)) {
+		    $self->_user_msg(slot_result => 1,
+				 active      => 1,
+				 slot        => $slot,
+				 label       => $label,
+				 res         => $res);
+		    return $steps->{'try_continue'}->();
+		} else {
+	            $self->_user_msg(slot_result => 1,
 			         slot        => $slot,
 			         label       => $label,
 			         res         => $res);
-	        $self->scan_result(res => $res, label => $label,
+	            $self->scan_result(res => $res, label => $label,
 				   mode => $ACCESS_WRITE, is_new => 0,
 				   result_cb => $result_cb);
-	        return;
+	            return;
+		}
 	    }
 	}
 
 	if (!defined $autolabel->{'template'} ||
 	    $autolabel->{'template'} eq "") {
 	    if ($status & $DEVICE_STATUS_VOLUME_UNLABELED and
-		$dev->volume_header and
-		$dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY) {
+		(!$dev->volume_header or
+		 $dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY)) {
 		$self->_user_msg(slot_result   => 1,
 			         not_autolabel => 1,
 				 empty         => 1,
@@ -431,8 +490,8 @@ sub stage_2 {
 	}
 
 	if ($status & $DEVICE_STATUS_VOLUME_UNLABELED and
-	    $dev->volume_header and
-	    $dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY) {
+	    (!$dev->volume_header or
+	     $dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY)) {
 	    if (!$autolabel->{'empty'}) {
 	        $self->_user_msg(slot_result  => 1,
 			         empty        => 1,
