@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amglue.h"
@@ -113,7 +114,7 @@ amglue_newSVu64(guint64 v)
  * @returns: signed integer
  */
 static gint64
-bigint2int64(SV *bigint)
+bigint2int64(SV *bigint, gchar **err)
 {
     SV *sv;
     char *str;
@@ -123,8 +124,10 @@ bigint2int64(SV *bigint)
     dSP;
 
     /* first, see if it's a BigInt */
-    if (!sv_isobject(bigint) || !sv_derived_from(bigint, "Math::BigInt"))
-	croak("Expected an integer or a Math::BigInt; cannot convert");
+    if (!sv_isobject(bigint) || !sv_derived_from(bigint, "Math::BigInt")) {
+	*err = g_strdup("Expected an integer or a Math::BigInt; cannot convert");
+	return 0;
+    }
 
     ENTER;
     SAVETMPS;
@@ -177,7 +180,7 @@ bigint2int64(SV *bigint)
  * @returns: unsigned integer
  */
 static guint64
-bigint2uint64(SV *bigint)
+bigint2uint64(SV *bigint, gchar **err)
 {
     SV *sv;
     char *str;
@@ -186,8 +189,10 @@ bigint2uint64(SV *bigint)
     dSP;
 
     /* first, see if it's a BigInt */
-    if (!sv_isobject(bigint) || !sv_derived_from(bigint, "Math::BigInt"))
-	croak("Expected an integer or a Math::BigInt; cannot convert");
+    if (!sv_isobject(bigint) || !sv_derived_from(bigint, "Math::BigInt")) {
+	*err = g_strdup("Expected an integer or a Math::BigInt; cannot convert");
+	return 0;
+    }
 
     ENTER;
     SAVETMPS;
@@ -247,7 +252,7 @@ bigint2uint64(SV *bigint)
     return rv;
 }
 
-gint64 amglue_SvI64(SV *sv)
+gint64 amglue_SvI64(SV *sv, gchar **err)
 {
     if (SvIOK(sv)) {
 	if (SvIsUV(sv)) {
@@ -263,23 +268,23 @@ gint64 amglue_SvI64(SV *sv)
 	 * also error out on integer truncation .. which is probably OK */
 	gint64 iv = (gint64)dv;
 	if (dv != (double)iv) {
-	    croak("Expected a signed 64-bit value or smaller; value '%.0f' out of range", (float)dv);
+	    *err = g_strdup_printf("Expected a signed 64-bit value or smaller; value '%.0f' out of range", (float)dv);
 	    return 0;
 	} else {
 	    return iv;
 	}
     } else {
-	return bigint2int64(sv);
+	return bigint2int64(sv, err);
     }
 }
 
-guint64 amglue_SvU64(SV *sv)
+guint64 amglue_SvU64(SV *sv, gchar **err)
 {
     if (SvIOK(sv)) {
 	if (SvIsUV(sv)) {
 	    return SvUV(sv);
 	} else if (SvIV(sv) < 0) {
-	    croak("Expected an unsigned value, got a negative integer");
+	    *err = g_strdup("Expected an unsigned value, got a negative integer");
 	    return 0;
 	} else {
 	    return (guint64)SvIV(sv);
@@ -287,79 +292,79 @@ guint64 amglue_SvU64(SV *sv)
     } else if (SvNOK(sv)) {
 	double dv = SvNV(sv);
 	if (dv < 0.0) {
-	    croak("Expected an unsigned value, got a negative integer");
+	    *err = g_strdup("Expected an unsigned value, got a negative integer");
 	    return 0;
 	} else if (dv > (double)G_MAXUINT64) {
-	    croak("Expected an unsigned 64-bit value or smaller; value out of range");
+	    *err = g_strdup("Expected an unsigned 64-bit value or smaller; value out of range");
 	    return 0;
 	} else {
 	    return (guint64)dv;
 	}
     } else {
-	return bigint2uint64(sv);
+	return bigint2uint64(sv, err);
     }
 }
 
-gint32 amglue_SvI32(SV *sv)
+gint32 amglue_SvI32(SV *sv, gchar **err)
 {
-    gint64 v64 = amglue_SvI64(sv);
+    gint64 v64 = amglue_SvI64(sv, err);
     if (v64 < G_MININT32 || v64 > G_MAXINT32) {
-	croak("Expected a 32-bit integer; value out of range");
+	*err = g_strdup("Expected a 32-bit integer; value out of range");
 	return 0;
     } else {
 	return (gint32)v64;
     }
 }
 
-guint32 amglue_SvU32(SV *sv)
+guint32 amglue_SvU32(SV *sv, gchar **err)
 {
-    guint64 v64 = amglue_SvU64(sv);
+    guint64 v64 = amglue_SvU64(sv, err);
     if (v64 > G_MAXUINT32) {
-	croak("Expected a 32-bit unsigned integer; value out of range");
+	*err = g_strdup("Expected a 32-bit unsigned integer; value out of range");
 	return 0;
     } else {
 	return (guint32)v64;
     }
 }
 
-gint16 amglue_SvI16(SV *sv)
+gint16 amglue_SvI16(SV *sv, gchar **err)
 {
-    gint64 v64 = amglue_SvI64(sv);
+    gint64 v64 = amglue_SvI64(sv, err);
     if (v64 < G_MININT16 || v64 > G_MAXINT16) {
-	croak("Expected a 16-bit integer; value out of range");
+	*err = g_strdup("Expected a 16-bit integer; value out of range");
 	return 0;
     } else {
 	return (gint16)v64;
     }
 }
 
-guint16 amglue_SvU16(SV *sv)
+guint16 amglue_SvU16(SV *sv, gchar **err)
 {
-    guint64 v64 = amglue_SvU64(sv);
+    guint64 v64 = amglue_SvU64(sv, err);
     if (v64 > G_MAXUINT16) {
-	croak("Expected a 16-bit unsigned integer; value out of range");
+	*err = g_strdup("Expected a 16-bit unsigned integer; value out of range");
 	return 0;
     } else {
 	return (guint16)v64;
     }
 }
 
-gint8 amglue_SvI8(SV *sv)
+gint8 amglue_SvI8(SV *sv, gchar **err)
 {
-    gint64 v64 = amglue_SvI64(sv);
+    gint64 v64 = amglue_SvI64(sv, err);
     if (v64 < G_MININT8 || v64 > G_MAXINT8) {
-	croak("Expected a 8-bit integer; value out of range");
+	*err = g_strdup("Expected a 8-bit integer; value out of range");
 	return 0;
     } else {
 	return (gint8)v64;
     }
 }
 
-guint8 amglue_SvU8(SV *sv)
+guint8 amglue_SvU8(SV *sv, gchar **err)
 {
-    guint64 v64 = amglue_SvU64(sv);
+    guint64 v64 = amglue_SvU64(sv, err);
     if (v64 > G_MAXUINT8) {
-	croak("Expected a 8-bit unsigned integer; value out of range");
+	*err = g_strdup("Expected a 8-bit unsigned integer; value out of range");
 	return 0;
     } else {
 	return (guint8)v64;
