@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -32,7 +33,7 @@
 
 #include "amanda.h"
 #include "amrecover.h"
-#include "util.h"
+#include "amutil.h"
 
 gboolean translate_mode = TRUE;
 
@@ -106,16 +107,15 @@ add_dir_list_item(
     dbprintf(_("add_dir_list_item: Adding \"%s\" \"%d\" \"%s\" \"%lld\" \"%s\"\n"),
 	      date, level, tape, (long long)fileno, path);
 
-    next = (DIR_ITEM *)alloc(sizeof(DIR_ITEM));
+    next = (DIR_ITEM *)g_malloc(sizeof(DIR_ITEM));
     memset(next, 0, sizeof(DIR_ITEM));
 
-    next->date = stralloc(date);
+    next->date = g_strdup(date);
     next->level = level;
-    next->tape = stralloc(tape);
+    next->tape = g_strdup(tape);
     next->fileno = fileno;
-    next->path = stralloc(path);
+    next->path = g_strdup(path);
     next->tpath = translate_octal(g_strdup(path));
-
     next->next = dir_list;
     dir_list = next;
 
@@ -152,16 +152,16 @@ suck_dir_list_from_server(void)
     if (disk_path == NULL) {
 	g_printf(_("Directory must be set before getting listing\n"));
 	return;
-    } else if(strcmp(disk_path, "/") == 0) {
-	disk_path_slash = stralloc(disk_path);
+    } else if(g_str_equal(disk_path, "/")) {
+	disk_path_slash = g_strdup(disk_path);
     } else {
-	disk_path_slash = stralloc2(disk_path, "/");
+	disk_path_slash = g_strconcat(disk_path, "/", NULL);
     }
 
     clear_dir_list();
 
     qdisk_path = quote_string(disk_path);
-    cmd = stralloc2("OLSD ", qdisk_path);
+    cmd = g_strconcat("OLSD ", qdisk_path, NULL);
     amfree(qdisk_path);
     if (send_command(cmd) == -1) {
 	amfree(cmd);
@@ -181,7 +181,7 @@ suck_dir_list_from_server(void)
 	g_printf("%s\n", l);
 	return;
     }
-    disk_path_slash_dot = stralloc2(disk_path_slash, ".");
+    disk_path_slash_dot = g_strconcat(disk_path_slash, ".", NULL);
     amfree(cmd);
     amfree(err);
     tape_undo = NULL;
@@ -197,7 +197,7 @@ suck_dir_list_from_server(void)
 	    if(cmd == NULL) {
 		if(tape_undo) *tape_undo = tape_undo_ch;
 		tape_undo = NULL;
-		cmd = stralloc(l);	/* save for the error report */
+		cmd = g_strdup(l);	/* save for the error report */
 	    }
 	    continue;			/* throw the rest of the lines away */
 	}
@@ -246,6 +246,7 @@ suck_dir_list_from_server(void)
 	    skip_whitespace(s, ch);
 	    if(ch == '\0' || sscanf(s - 1, "%lld", &fileno_) != 1) {
 		err = _("bad reply: cannot parse fileno field");
+		amfree(tape);
 		continue;
 	    }
 	    fileno = (off_t)fileno_;
@@ -258,15 +259,16 @@ suck_dir_list_from_server(void)
 	skip_whitespace(s, ch);
 	if(ch == '\0') {
 	    err = _("bad reply: missing directory field");
+	    amfree(tape);
 	    continue;
 	}
 	qdir = s - 1;
 	dir = unquote_string(qdir);
 
 	/* add a '.' if it a the entry for the current directory */
-	if((strcmp(disk_path,dir)==0) || (strcmp(disk_path_slash,dir)==0)) {
+	if((g_str_equal(disk_path, dir)) || (g_str_equal(disk_path_slash, dir))) {
 	    amfree(dir);
-	    dir = stralloc(disk_path_slash_dot);
+	    dir = g_strdup(disk_path_slash_dot);
 	}
 	add_dir_list_item(date, level, tape, fileno, dir);
 	amfree(tape);
@@ -311,7 +313,7 @@ list_directory(void)
      * Set up the pager command so if the pager is terminated, we do
      * not get a SIGPIPE back.
      */
-    pager_command = stralloc2(pager, " ; /bin/cat > /dev/null");
+    pager_command = g_strconcat(pager, " ; /bin/cat > /dev/null", NULL);
     if ((fp = popen(pager_command, "w")) == NULL)
     {
 	g_printf(_("Warning - can't pipe through %s\n"), pager);
