@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -36,7 +37,12 @@
 #include "amanda.h"
 #include "conffile.h"
 
+char *getstatefname(char *host, char *disk, char *date, int level);
 char *getindexfname(char *host, char *disk, char *date, int level);
+char *getindex_unsorted_fname(char *host, char *disk, char *date, int level);
+char *getindex_unsorted_gz_fname(char *host, char *disk, char *date, int level);
+char *getindex_sorted_fname(char *host, char *disk, char *date, int level);
+char *getindex_sorted_gz_fname(char *host, char *disk, char *date, int level);
 char *getheaderfname(char *host, char *disk, char *date, int level);
 char *getoldindexfname(char *host, char *disk, char *date, int level);
 
