@@ -1,5 +1,6 @@
 #! @PERL@
-# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -59,6 +60,7 @@ my ($opt_mailto, $opt_filename, $opt_logfname, $opt_psfname, $opt_xml);
 my $from_amdump = 1;
 my ($config_name, $report, $outfh);
 my $mode = MODE_NONE;
+my $tl;
 
 # list of [ report-spec, output-spec ]
 my (@outputs, @output_queue);
@@ -93,7 +95,11 @@ sub error
 {
     my ( $error_msg, $exit_code ) = @_;
     warning("error: $error_msg");
-    print STDERR "$error_msg\n";
+    if ($mode == MODE_SCRIPT) {
+	print STDOUT "$error_msg\n";
+    } else {
+	print STDERR "$error_msg\n";
+    }
     exit $exit_code;
 }
 
@@ -197,8 +203,11 @@ sub calculate_legacy_outputs {
     # Part of the "options" is the configuration.  Do we have a template?  And a
     # mailto? And mailer?
 
-    my $ttyp = getconf($CNF_TAPETYPE);
-    my $tt = lookup_tapetype($ttyp) if $ttyp;
+    my $storage = Amanda::Storage->new(
+			storage_name => $report->{'storage_list'}[0],
+			tapelist => $tl);
+    my $tapetype_name = $storage->{'tapetype_name'};
+    my $tt = lookup_tapetype($tapetype_name) if $tapetype_name;
     my $cfg_template = "" . tapetype_getconf($tt, $TAPETYPE_LBL_TEMPL) if $tt;
 
     my $cfg_mailer  = getconf($CNF_MAILER);
@@ -251,6 +260,7 @@ sub calculate_legacy_outputs {
     if ($opt_psfname and $cfg_template) {
 	push @outputs, [ [ 'postscript', $cfg_template ], [ 'file', $opt_psfname ] ];
     }
+    $storage->quit();
 }
 
 sub legacy_send_amreport
@@ -339,28 +349,35 @@ sub open_printer_output
 
     # redirect stdout/stderr to stderr, which is usually the amdump log
     my ($pid, $fh);
-    if (!-f $Amanda::Constants::LPR || !-x $Amanda::Constants::LPR) {
-	my $errstr = "error: the mailer '$Amanda::Constants::LPR' is not an executable program.";
-	print STDERR "$errstr\n";
-        if ($mode == MODE_SCRIPT) {
-            debug($errstr);
-        } else {
-            error($errstr, 1);
-        }
+    if (!-e $Amanda::Constants::LPR) {
+	my $errstr = "error: the printer command '$Amanda::Constants::LPR': $!";
+	report_error($errstr);
+    } elsif (!-f $Amanda::Constants::LPR) {
+	my $errstr = "error: the printer command '$Amanda::Constants::LPR' is not a file";
+	report_error($errstr);
+    } elsif (!-x $Amanda::Constants::LPR) {
+	my $errstr = "error: the printer command '$Amanda::Constants::LPR' is not an executable program";
+	report_error($errstr);
     } else {
-	eval { $pid = open3($fh, ">&2", ">&2", @cmd); } or do {
-            ($pid, $fh) = (0, undef);
-            chomp $@;
-            my $errstr = "error: $@: $!";
+	if ($mode == MODE_SCRIPT) {
+	    eval { $pid = open3($fh, ">&1", ">&1", @cmd); } or do {
+		($pid, $fh) = (0, undef);
+		chomp $@;
+		my $errstr = "error: $@: $!";
+
+		report_error($errstr);
+            };
+	} else {
+	    eval { $pid = open3($fh, ">&1", ">&2", @cmd); } or do {
+		($pid, $fh) = (0, undef);
+		chomp $@;
+		my $errstr = "error: $@: $!";
+
+		report_error($errstr);
+            };
+	}
+    };
 
-	    print STDERR "$errstr\n";
-            if ($mode == MODE_SCRIPT) {
-		debug($errstr);
-            } else {
-		error($errstr, 1);
-            }
-        };
-    }
     return ($pid, $fh);
 }
 
@@ -375,8 +392,9 @@ sub open_mail_output
 
     my $datestamp =
       $report->get_program_info(
-        $report->get_flag("amflush_run") ? "amflush" : 
-	$report->get_flag("amvault_run") ? "amvault" : "planner", "start" );
+        $report->get_flag("amflush_run") ? "amflush" :
+	$report->get_flag("amvault_run") ? "amvault" :
+	$report->get_flag("ambackupd_run") ? "ambackupd" : "planner", "start" );
 
     $datestamp /= 1000000 if $datestamp > 99999999;
     $datestamp = int($datestamp);
@@ -399,7 +417,8 @@ sub open_mail_output
     my $subj_str =
         getconf($CNF_ORG) . $done
       . ( $report->get_flag("amflush_run") ? " AMFLUSH" :
-	  $report->get_flag("amvault_run") ? " AMVAULT" : " AMANDA" )
+	  $report->get_flag("amvault_run") ? " AMVAULT" :
+	  $report->get_flag("ambackupd_run") ? " AMBACKUPD" : " AMANDA" )
       . " MAIL REPORT FOR "
       . $date;
 
@@ -408,35 +427,50 @@ sub open_mail_output
     my @cmd = ("$cfg_mailer", "-s", $subj_str, split(/ +/, $mailto));
     debug("invoking mail app: " . join(" ", @cmd));
 
-
     my ($pid, $fh);
-    if (!-f $cfg_mailer || !-x $cfg_mailer) {
-	my $errstr = "error: the mailer '$cfg_mailer' is not an executable program.";
-	print STDERR "$errstr\n";
-        if ($mode == MODE_SCRIPT) {
-            debug($errstr);
-        } else {
-            error($errstr, 1);
-        }
-	
+    if (!-e $cfg_mailer) {
+	my $errstr = "error: the mailer '$cfg_mailer': $!";
+	report_error($errstr);
+    } elsif (!-f $cfg_mailer) {
+	my $errstr = "error: the mailer '$cfg_mailer' is not a file";
+	report_error($errstr);
+    } elsif (!-x $cfg_mailer) {
+	my $errstr = "error: the mailer '$cfg_mailer' is not an executable program";
+	report_error($errstr);
     } else {
-	eval { $pid = open3($fh, ">&2", ">&2", @cmd) } or do {
-            ($pid, $fh) = (0, undef);
-            chomp $@;
-            my $errstr = "error: $@: $!";
+	if ($mode == MODE_SCRIPT) {
+	    eval { $pid = open3($fh, ">&1", ">&1", @cmd) } or do {
+		($pid, $fh) = (0, undef);
+		chomp $@;
+		my $errstr = "error: $@: $!";
 
-	    print STDERR "$errstr\n";
-            if ($mode == MODE_SCRIPT) {
-		debug($errstr);
-            } else {
-		error($errstr, 1);
-            }
-	};
-    }
+		report_error($errstr);
+	    };
+	} else {
+	    eval { $pid = open3($fh, ">&1", ">&2", @cmd) } or do {
+		($pid, $fh) = (0, undef);
+		chomp $@;
+		my $errstr = "error: $@: $!";
+
+		report_error($errstr);
+	    };
+	}
+    };
 
     return ($pid, $fh);
 }
 
+sub report_error {
+    my $errstr = shift;
+
+    if ($mode == MODE_SCRIPT) {
+	print STDERR "$errstr\n";
+	debug($errstr);
+    } else {
+	error($errstr, 1);
+    }
+}
+
 sub run_output {
     my ($output) = @_;
     my ($reportspec, $outputspec) = @$output;
@@ -501,7 +535,7 @@ sub run_output {
 
 ## Application initialization
 
-Amanda::Util::setup_application("amreport", "server", $CONTEXT_CMDLINE);
+Amanda::Util::setup_application("amreport", "server", $CONTEXT_CMDLINE, "amanda", "amanda");
 
 my $config_overrides = new_config_overrides( scalar(@ARGV) + 1 );
 
@@ -553,7 +587,7 @@ $config_name = shift @ARGV;    # only use first argument
 $config_name ||= '.';          # default config is current dir
 
 set_config_overrides($config_overrides);
-config_init( $CONFIG_INIT_EXPLICIT_NAME, $config_name );
+config_init_with_global( $CONFIG_INIT_EXPLICIT_NAME, $config_name );
 
 my ( $cfgerr_level, @cfgerr_errors ) = config_errors();
 if ( $cfgerr_level >= $CFGERR_WARNINGS ) {
@@ -567,11 +601,14 @@ Amanda::Util::finish_setup($RUNNING_AS_DUMPUSER);
 
 # read the tapelist
 my $tl_file = config_dir_relative(getconf($CNF_TAPELIST));
-my $tl = Amanda::Tapelist->new($tl_file);
+($tl, my $message) = Amanda::Tapelist->new($tl_file);
+if (defined $message) {
+    debug("Could not read the tapelist: $message");
+}
 
 # read the disklist
 my $diskfile = config_dir_relative(getconf($CNF_DISKFILE));
-$cfgerr_level += Amanda::Disklist::read_disklist('filename' => $diskfile);
+$cfgerr_level = Amanda::Disklist::read_disklist('filename' => $diskfile);
 ($cfgerr_level < $CFGERR_ERRORS) || die "Errors processing disklist";
 
 # shim for installchecks
@@ -585,6 +622,15 @@ my $logfile = $opt_logfname || get_default_logfile();
 my $historical = defined $opt_logfname;
 debug("using logfile: $logfile" . ($historical? " (historical)" : ""));
 
+## Parse the report & set output
+
+$report = Amanda::Report->new($logfile, $historical);
+if ($report->isa("Amanda::Message")) {
+    print $report, "\n";
+    exit(1);
+}
+my $exit_status = $report->get_flag("exit_status");
+
 if ($mode == MODE_CMDLINE) {
     debug("operating in cmdline mode");
     apply_output_defaults();
@@ -606,11 +652,6 @@ if ($mode == MODE_CMDLINE) {
     }
 }
 
-## Parse the report & set output
-
-$report = Amanda::Report->new($logfile, $historical);
-my $exit_status = $report->get_flag("exit_status");
-
 ## filter outputs by errors & stranges
 
 @outputs = grep { legacy_send_amreport($_) } @outputs;
