@@ -1,5 +1,6 @@
 #! @PERL@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -54,7 +55,7 @@ EOF
 ##
 # main
 
-Amanda::Util::setup_application("amrestore", "server", $CONTEXT_CMDLINE);
+Amanda::Util::setup_application("amrestore", "server", $CONTEXT_CMDLINE, "amanda", "amanda");
 
 my $config_overrides = new_config_overrides($#ARGV+1);
 
@@ -108,9 +109,9 @@ $opt_header = 1 if $opt_raw;
 
 set_config_overrides($config_overrides);
 if ($opt_config) {
-    config_init($CONFIG_INIT_EXPLICIT_NAME, $opt_config);
+    config_init_with_global($CONFIG_INIT_EXPLICIT_NAME, $opt_config);
 } else {
-    config_init(0, undef);
+    config_init($CONFIG_INIT_GLOBAL, undef);
 }
 my ($cfgerr_level, @cfgerr_errors) = config_errors();
 if ($cfgerr_level >= $CFGERR_WARNINGS) {
@@ -166,7 +167,10 @@ sub main {
 	    $steps->{'read_header'}->();
 	} else {
 	    my $tlf = Amanda::Config::config_dir_relative(getconf($CNF_TAPELIST));
-	    my $tl = Amanda::Tapelist->new($tlf);
+	    my ($tl, $message) = Amanda::Tapelist->new($tlf);
+	    if (defined $message) {
+		return failure($message, $finished_cb);
+	    }
 	    $chg = Amanda::Changer->new($opt_restore_src, tapelist => $tl);
 	    if ($chg->isa("Amanda::Changer::Error")) {
 		return failure($chg, $finished_cb);
@@ -184,8 +188,8 @@ sub main {
 	$dev = $res->{'device'};
 
 	if ($opt_blocksize) {
-	    if ( !$dev->property_set("BLOCK_SIZE", $opt_blocksize)) {
-		return failure($dev->error_or_status, $finished_cb);
+	    if ((my $r = $dev->property_set("BLOCK_SIZE", $opt_blocksize))) {
+		return failure($r, $finished_cb);
 	    }
 
 	    # re-read the label with the correct blocksize
@@ -274,6 +278,7 @@ sub main {
 	# set up the source..
 	if ($opt_holding) {
 	    $src = Amanda::Xfer::Source::Holding->new($opt_restore_src);
+	    $src->start_recovery();
 	} else {
 	    $src = Amanda::Xfer::Source::Device->new($dev);
 	}
