@@ -1,5 +1,6 @@
 #! @PERL@
-# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -29,7 +30,8 @@ use Text::Wrap;
 use Amanda::Device qw( :constants );
 use Amanda::Debug qw( :logging );
 use Amanda::Config qw( :init :getconf config_dir_relative );
-use Amanda::Util qw( :constants );
+use Amanda::Util qw( :constants match_labelstr );
+use Amanda::Storage;
 use Amanda::Changer;
 use Amanda::Constants;
 use Amanda::MainLoop;
@@ -37,6 +39,7 @@ use Amanda::Taper::Scan;
 use Amanda::Recovery::Scan;
 use Amanda::Interactivity;
 use Amanda::Tapelist;
+use Amanda::Message qw( :severity );
 
 my $exit_status = 0;
 my $tl;
@@ -92,10 +95,11 @@ subcommand("reset", "reset", "reset changer to known state",
 sub {
     my ($finished_cb, @args) = @_;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     $chg->reset(finished_cb => sub {
 	    my ($err) = @_;
+	    $storage->quit();
 	    $chg->quit();
 	    return failure($err, $finished_cb) if $err;
 
@@ -109,7 +113,7 @@ sub {
     my ($finished_cb, @args) = @_;
     my @drive_args;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     if (@args) {
 	@drive_args = (drive => shift @args);
@@ -117,6 +121,7 @@ sub {
     $chg->eject(@drive_args,
 	finished_cb => sub {
 	    my ($err) = @_;
+	    $storage->quit();
 	    $chg->quit();
 	    return failure($err, $finished_cb) if $err;
 
@@ -130,7 +135,7 @@ sub {
     my ($finished_cb, @args) = @_;
     my @drive_args;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     if (@args == 1) {
 	@drive_args = (drive => shift @args);
@@ -141,6 +146,7 @@ sub {
     $chg->clean(@drive_args,
 	finished_cb => sub {
 	    my ($err) = @_;
+	    $storage->quit();
 	    $chg->quit();
 	    return failure($err, $finished_cb) if $err;
 
@@ -154,123 +160,27 @@ sub {
     my ($finished_cb, @args) = @_;
     my $last_slot;
     my %seen_slots;
-    my $chg;
 
     if (@args > 1) {
 	return usage($finished_cb);
     }
 
-    my $what = $args[0];
-    my @slots;
+    my $user_msg = sub {
+	my $msg = shift;
+	print STDERR $msg->message() . "\n";
+    };
 
-    if (defined $what) {
-	my @what1 = split /,/, $what;
-	foreach my $what1 (@what1) {
-	    if ($what1 =~ /^(\d*)-(\d*)$/) {
-		my $begin = $1;
-		my $end = $2;
-		$end = $begin if $begin > $end;
-		while ($begin <= $end) {
-		    push @slots, $begin;
-		    $begin++;
-		}
-	    } else {
-		push @slots, $what1;
-	    }
-	}
-    }
-
-    my $use_slots = @slots > 0;
-
-    $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     my $steps = define_steps
 	cb_ref => \$finished_cb,
-	finalize => sub { $chg->quit() if defined $chg };
+	finalize => sub { $storage->quit() if defined $storage;
+			  $chg->quit() if defined $chg };
 
     step start => sub {
-	$chg->info(info => [ 'num_slots' ], info_cb => $steps->{'info_cb'});
-    };
-
-    step info_cb => sub {
-	my ($err, %info) = @_;
-	return failure($err, $finished_cb) if $err;
-
-	if ($use_slots) {
-	   my $slot = shift @slots;
-	   $chg->load(slot => $slot,
-		      mode => "read",
-		      res_cb => $steps->{'loaded'});
-
-	} else {
-	    print STDERR "amtape: scanning all $info{num_slots} slots in changer:\n";
-
-	    $chg->load(relative_slot => 'current',
-		       mode => "read",
-		       res_cb => $steps->{'loaded'});
-	}
-    };
-
-    step loaded => sub {
-	my ($err, $res) = @_;
-	if ($err) {
-	    if ($err->notfound) {
-		# no more interesting slots
-		$finished_cb->();
-		return;
-	    } elsif ($err->volinuse and defined $err->{'slot'}) {
-		$last_slot = $err->{'slot'};
-	        print STDERR sprintf("slot %3s: in use\n", $last_slot);
-	    } elsif ($err->empty and defined $err->{'slot'}) {
-		$last_slot = $err->{'slot'};
-	        print STDERR sprintf("slot %3s: empty\n", $last_slot);
-	    } elsif ($err->invalid and defined $err->{'slot'}) {
-		$last_slot = $err->{'slot'};
-	        print STDERR sprintf("slot %3s: %s\n", $last_slot, "$err");
-	    } else {
-		return failure($err, $finished_cb) if $err;
-	    }
-	} else {
-	    $last_slot = $res->{'this_slot'};
-	}
-
-	$seen_slots{$last_slot} = 1;
-
-	if ($res) {
-	    my $dev = $res->{'device'};
-	    my $st = $dev->read_label();
-	    if ($st == $DEVICE_STATUS_SUCCESS) {
-		print STDERR sprintf("slot %3s: date %-14s label %s\n",
-			$last_slot, $dev->volume_time(),
-			$dev->volume_label());
-	    } elsif ($st == $DEVICE_STATUS_VOLUME_UNLABELED) {
-		print STDERR sprintf("slot %3s: unlabeled volume\n", $last_slot);
-	    } else {
-		print STDERR sprintf("slot %3s: %s\n", $last_slot, $dev->error_or_status());
-	    }
-	}
-
-	if ($res) {
-	    $res->release(finished_cb => $steps->{'released'});
-	} else {
-	    $steps->{'released'}->();
-	}
-    };
-
-    step released => sub {
-	if ($use_slots) {
-	   return $finished_cb->() if @slots == 0;
-	   my $slot = shift @slots;
-	   $chg->load(slot => $slot,
-		      mode => "read",
-		      res_cb => $steps->{'loaded'});
-
-	} else {
-	    $chg->load(relative_slot => 'next',
-		       slot => $last_slot,
-		       except_slots => { %seen_slots },
-		       res_cb => $steps->{'loaded'});
-	}
+	$chg->show(slots => $args[0],
+		   user_msg => $user_msg,
+		   finished_cb => $finished_cb);
     };
 });
 
@@ -278,12 +188,14 @@ subcommand("inventory", "inventory", "show inventory of changer slots",
 sub {
     my ($finished_cb, @args) = @_;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     if (@args != 0) {
 	return usage($finished_cb);
     }
 
+    Amanda::Tapelist::compute_retention();
+
     # TODO -- support an --xml option
 
     my $inventory_cb = make_cb(inventory_cb => sub {
@@ -299,6 +211,7 @@ sub {
 		print STDERR "$err\n";
 	    }
 
+	    $storage->quit();
 	    $chg->quit();
 	    return $finished_cb->();
 	}
@@ -306,6 +219,7 @@ sub {
 	for my $sl (@$inv) {
 	    my $line = "slot $sl->{slot}:";
 	    my $tle;
+	    my $meta;
 	    if ($sl->{'state'} == Amanda::Changer::SLOT_EMPTY) {
 		$line .= " empty";
 	    } elsif (!defined($sl->{device_status}) && !defined($sl->{label})) {
@@ -317,19 +231,14 @@ sub {
 		    if (defined $tle) {
 			if ($tle->{'meta'}) {
 				$line .= " ($tle->{'meta'})";
+				$meta = $tle->{'meta'};
 			}
 		    }
 		} elsif ($sl->{'device_status'} == $DEVICE_STATUS_VOLUME_UNLABELED) {
 		    $line .= " blank";
-		} elsif ($sl->{'device_status'} != $DEVICE_STATUS_SUCCESS) {
-		    if (defined $sl->{'device_error'}) {
-			$line .= " " . $sl->{'device_error'};
-		    } else {
-			$line .= "device error";
-		    }
 		} elsif ($sl->{'f_type'} != $Amanda::Header::F_TAPESTART) {
 		    $line .= " blank";
-		} else {
+		} elsif (!defined $sl->{label}) {
 		    $line .= " unknown";
 		}
 	    }
@@ -348,7 +257,26 @@ sub {
 	    if ($sl->{'current'}) {
 		$line .= " (current)";
 	    }
+	    if ($sl->{'device_status'} != $DEVICE_STATUS_SUCCESS &&
+		$sl->{'device_status'} != $DEVICE_STATUS_VOLUME_UNLABELED) {
+		if (defined $sl->{'device_error'}) {
+		    $line .= " [" . $sl->{'device_error'} . "]";
+		} else {
+		    $line .= " [device error]";
+		}
+	    }
+	    if ($sl->{'label'}) {
+		if (!match_labelstr($storage->{'labelstr'},
+				    $storage->{'autolabel'},
+				    $sl->{label},
+				    $sl->{'barcode'}, $meta,
+				    $storage->{'storage_name'})) {
+		    $line .= " (label do not match labelstr)";
+		}
+	    }
 	    if (defined $tle) {
+		my $retention_type = Amanda::Tapelist::get_retention_type($tle->{pool}, $tle->{label});
+		$line .= " [" . $tl->get_retention_name($retention_type) . "]";
 		if (defined $sl->{'barcode'} and
 		    defined $tle->{'barcode'} and
 		    $sl->{'barcode'} ne $tle->{'barcode'}) {
@@ -360,17 +288,52 @@ sub {
 	    print "$line\n";
 	}
 
+	$storage->quit();
 	$chg->quit();
 	$finished_cb->();
     });
     $chg->inventory(inventory_cb => $inventory_cb);
 });
 
+subcommand("create", "create", "create the changer root directory",
+sub {
+    my ($finished_cb, @args) = @_;
+
+    my ($storage, $chg) = load_changer($finished_cb) or return;
+
+    if (@args != 0) {
+	return usage($finished_cb);
+    }
+
+    # TODO -- support an --xml option
+
+    my $create_cb = make_cb(create => sub {
+	my ($err, @results) = @_;
+	if ($err) {
+	    if ($err->notimpl) {
+		if ($err->{'message'}) {
+		    print STDERR "create not supported by this changer: $err->{'message'}\n";
+		} else {
+		    print STDERR "create not supported by this changer\n";
+		}
+	    } else {
+		print STDERR "$err\n";
+	    }
+	} else {
+	    print STDERR "Created\n";
+	}
+	$storage->quit();
+	$chg->quit();
+	return $finished_cb->();
+    });
+    $chg->create(finished_cb => $create_cb);
+});
+
 subcommand("verify", "verify", "verify the changer is correctly configured",
 sub {
     my ($finished_cb, @args) = @_;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     if (@args != 0) {
 	return usage($finished_cb);
@@ -391,9 +354,18 @@ sub {
 		print STDERR "$err\n";
 	    }
 	} else {
-	    print STDERR join("\n", @results);
-	    print STDERR "\n";
+	    foreach my $result (@results) {
+		if ($result->isa("Amanda::Message")) {
+		    print "GOOD : " if $result->{'code'} == 1100006;
+		    print "HINT : " if $result->{'code'} == 1100007;
+		    print "ERROR: " if $result->{'code'} == 1100009;
+		    print "ERROR: " if $result->{'code'} == 1100024;
+		    print "ERROR: " if $result->{'code'} == 1100025;
+		}
+		print STDERR $result . "\n";
+	    }
 	}
+	$storage->quit();
 	$chg->quit();
 	return $finished_cb->();
     });
@@ -410,25 +382,31 @@ sub {
     return invoke_subcommand("slot", $finished_cb, "current");
 });
 
-subcommand("slot", "slot <slot>",
-	   "load the volume in slot <slot>; <slot> can also be 'current', 'next', 'first', or 'last'",
+subcommand("slot", "slot <slot> [drive <drive>]",
+	   "load the volume in slot <slot> in drive <drive>; <slot> can also be 'current', 'next', 'first', or 'last'",
 sub {
     my ($finished_cb, @args) = @_;
     my @slotarg;
+    my $storage;
     my $chg;
 
     my $steps = define_steps
 	cb_ref => \$finished_cb,
-	finalize => sub { $chg->quit() if defined $chg };
+	finalize => sub { $storage->quit() if defined $storage;
+			  $chg->quit() if defined $chg };
 
     # NOTE: the syntax of this subcommand precludes actual slots named
     # 'current' or 'next' ..  when we have a changer using such slot names,
     # this subcommand will need to support a --literal flag
 
-    return usage($finished_cb) unless (@args == 1);
+    return usage($finished_cb) unless (@args == 1 || @args == 3);
     my $slot = shift @args;
+    my $drive = shift @args;
+    if (defined $drive) {	# check drive keyword
+	$drive = shift @args;
+    }
 
-    $chg = load_changer($finished_cb) or return;
+    ($storage, $chg) = load_changer($finished_cb) or return;
 
     step get_slot => sub {
 	if ($slot eq 'current' or $slot eq 'next') {
@@ -462,7 +440,7 @@ sub {
     };
 
     step do_load => sub {
-	$chg->load(@slotarg, set_current => 1,
+	$chg->load(@slotarg, drive => $drive, set_current => 1,
 	    res_cb => $steps->{'done_load'});
     };
 
@@ -490,6 +468,7 @@ sub {
     my ($finished_cb, @args) = @_;
     my $interactivity;
     my $scan;
+    my $storage;
     my $chg;
 
     return usage($finished_cb) unless (@args == 1);
@@ -498,33 +477,26 @@ sub {
     my $steps = define_steps
 	cb_ref => \$finished_cb,
 	finalize => sub { $scan->quit() if defined $scan;
+			  $storage->quit() if defined $storage;
 			  $chg->quit() if defined $chg };
 
     step start => sub {
 	my $_user_msg_fn = sub {
-	    my %params = @_;
+	    my $msg = shift;
 
-	    if (exists($params{'scan_slot'})) {
-		print "slot $params{'slot'}:";
-	    } elsif (exists($params{'slot_result'})) {
-		if (defined($params{'err'})) {
-		    print " $params{'err'}\n";
-		} else { # res must be defined
-		    my $res = $params{'res'};
-		    my $dev = $res->{'device'};
-		    if ($dev->status == $DEVICE_STATUS_SUCCESS) {
-			my $volume_label = $res->{device}->volume_label;
-			print " $volume_label\n";
-		    } else {
-			my $errmsg = $res->{device}->error_or_status();
-			print " $errmsg\n";
-		    }
-		}
+	    if ($msg->{'code'} == 1200000) {
+		printf "slot %3s:", $msg->{'slot'};
+	    } elsif ($msg->{'code'} == 1200001) {
+		print " " . $msg->message() . "\n";
+	    } elsif ($msg->{'code'} == 1200002) {
+		print " " . $msg->message() . "\n";
+	    } elsif ($msg->{'code'} == 1200003) {
+		print " " . $msg->message() . "\n";
 	    }
 	};
 
 	$interactivity = Amanda::Interactivity->new(name => 'stdin');
-	$chg = load_changer($finished_cb) or return;
+	($storage, $chg) = load_changer($finished_cb) or return;
 	$scan = Amanda::Recovery::Scan->new(chg => $chg,
 					    interactivity => $interactivity);
 	return failure("$scan", $finished_cb)
@@ -559,6 +531,10 @@ sub {
 subcommand("taper", "taper", "perform the taperscan algorithm and display the result",
 sub {
     my ($finished_cb, @args) = @_;
+    return usage($finished_cb) unless (@args == 0);
+    my $label = shift @args;
+
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     my $taper_user_msg_fn = sub {
 	my %params = @_;
@@ -576,58 +552,66 @@ sub {
 		    print STDERR "slot $params{'err'}->{'slot'}:";
 		}
 		print STDERR " $params{'err'}\n";
-	    } elsif ($params{'res'}) {
-		my $res = $params{'res'};
-		my $dev = $res->{'device'};
-		if (exists($params{'search_result'})) {
-		    print STDERR " found in slot $res->{'this_slot'}:";
-		}
-		if ($dev->status == $DEVICE_STATUS_SUCCESS) {
-		    my $volume_label = $res->{device}->volume_label;
-		    if ($params{'active'}) {
-			print STDERR " volume '$volume_label' is still active and cannot be overwritten\n";
-		    } elsif ($params{'does_not_match_labelstr'}) {
-			print STDERR " volume '$volume_label' does not match labelstr '$params{'labelstr'}'\n";
-		    } elsif ($params{'not_in_tapelist'}) {
-			print STDERR " volume '$volume_label' is not in the tapelist\n";
-		    } elsif ($params{'relabeled'}) {
-			print STDERR " volume '$volume_label' from another config will be relabeled\n";
-		    } else {
-			print STDERR " volume '$volume_label'\n";
-		    }
-		} elsif ($dev->status & $DEVICE_STATUS_VOLUME_UNLABELED and
-			 $dev->volume_header and
-			 $dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY) {
-		    print STDERR " contains an empty volume\n";
-		} elsif ($dev->status & $DEVICE_STATUS_VOLUME_UNLABELED and
-			 $dev->volume_header and
-			 $dev->volume_header->{'type'} == $Amanda::Header::F_WEIRD) {
-		    my $autolabel = getconf($CNF_AUTOLABEL);
-		    if ($autolabel->{'non_amanda'}) {
-			print STDERR " contains a non-Amanda volume\n";
-		    } else {
-			print STDERR " contains a non-Amanda volume; check and relabel it with 'amlabel -f'\n";
-		    }
-		} elsif ($dev->status & $DEVICE_STATUS_VOLUME_ERROR) {
-		    my $message = $dev->error_or_status();
-		    print STDERR " can't read label: $message\n";
+	    } elsif (!$params{'res'}) {
+		my $volume_label = $params{'label'};
+		if ($params{'active'}) {
+		    print STDERR " volume '$volume_label' is still active and cannot be overwritten\n";
+		} elsif ($params{'does_not_match_labelstr'}) {
+		    print STDERR " volume '$volume_label' does not match labelstr '$params{'labelstr'}'\n";
+		} elsif ($params{'not_in_tapelist'}) {
+		    print STDERR " volume '$volume_label' is not in the tapelist\n"
 		} else {
-		    my $errmsg = $res->{device}->error_or_status();
-		    print STDERR " $errmsg\n";
+		    print STDERR " volume '$volume_label'\n";
 		}
+	    } else { # res must be defined
+                my $res = $params{'res'};
+                my $dev = $res->{'device'};
+                if (exists($params{'search_result'})) {
+                    print STDERR " found in slot $res->{'this_slot'}:";
+                }
+                if ($dev->status == $DEVICE_STATUS_SUCCESS) {
+                    my $volume_label = $res->{device}->volume_label;
+                    if ($params{'active'}) {
+                        print STDERR " volume '$volume_label' is still active and cannot be overwritten\n";
+                    } elsif ($params{'does_not_match_labelstr'}) {
+                        print STDERR " volume '$volume_label' does not match labelstr '$params{'labelstr'}'\n";
+                    } elsif ($params{'not_in_tapelist'}) {
+                        print STDERR " volume '$volume_label' is not in the tapelist\n"
+                    } elsif ($params{'relabeled'}) {
+                        print STDERR " volume '$volume_label' from another config will be relabeled\n";
+                    } else {
+                        print STDERR " volume '$volume_label'\n";
+                    }
+                } elsif ($dev->status & $DEVICE_STATUS_VOLUME_UNLABELED and
+                         $dev->volume_header and
+                         $dev->volume_header->{'type'} == $Amanda::Header::F_EMPTY) {
+                    print STDERR " contains an empty volume\n";
+                } elsif ($dev->status & $DEVICE_STATUS_VOLUME_UNLABELED and
+                         $dev->volume_header and
+                         $dev->volume_header->{'type'} == $Amanda::Header::F_WEIRD) {
+                    my $autolabel = storage_getconf($storage, $STORAGE_AUTOLABEL);
+                    if ($autolabel->{'non_amanda'}) {
+                        print STDERR " contains a non-Amanda volume\n";
+                    } else {
+                        print STDERR " contains a non-Amanda volume; check and relabel it with 'amlabel -f'\n";
+                    }
+                } elsif ($dev->status & $DEVICE_STATUS_VOLUME_ERROR) {
+                    my $message = $dev->error_or_status();
+                    print STDERR " can't read label: $message\n";
+                } else {
+                    my $errmsg = $res->{device}->error_or_status();
+                    print STDERR " $errmsg\n";
+                }
 	    }
 	} else {
 	    print STDERR "UNKNOWN\n";
 	}
     };
 
-    return usage($finished_cb) unless (@args == 0);
-    my $label = shift @args;
-
-    my $chg = load_changer($finished_cb) or return;
     my $interactivity = Amanda::Interactivity->new(name => 'tty');
-    my $scan_name = getconf($CNF_TAPERSCAN);
+    my $scan_name = $chg->{'storage'}->{'taperscan_name'};
     my $taperscan = Amanda::Taper::Scan->new(algorithm => $scan_name,
+					     storage => $chg->{'storage'},
 					     changer => $chg,
 					     tapelist => $tl);
 
@@ -636,11 +620,13 @@ sub {
 	if ($err) {
 	    if ($res) {
 		$res->release(finished_cb => sub {
+		    $storage->quit() if defined $storage;
 		    $taperscan->quit() if defined $taperscan;
 		    return failure($err, $finished_cb);
 		});
 		return;
 	    } else {
+		$storage->quit() if defined $storage;
 		$taperscan->quit() if defined $taperscan;
 		return failure($err, $finished_cb);
 	    }
@@ -654,7 +640,7 @@ sub {
 	    print STDERR "Will $modestr label '$label' to '" . $res->{'device'}->volume_label() . "' labelled volume in slot $slot.\n";
 	} else {
 	    my $header = $res->{'device'}->volume_header();
-	    if ($header->{'type'} == $Amanda::Header::F_WEIRD) {
+	    if (!defined $header || $header->{'type'} == $Amanda::Header::F_WEIRD) {
 		print STDERR "Will $modestr label '$label' to non-Amanda volume in slot $slot.\n";
 	    } else {
 		print STDERR "Will $modestr label '$label' to new volume in slot $slot.\n";
@@ -664,6 +650,7 @@ sub {
 	    my ($err) = @_;
 	    die "$err" if $err;
 
+	    $storage->quit() if defined $storage;
 	    $taperscan->quit() if defined $taperscan;
 	    $finished_cb->();
 	});
@@ -679,22 +666,55 @@ subcommand("update", "update [WHAT]", "update the changer's state; see changer d
 sub {
     my ($finished_cb, @args) = @_;
     my @changed_args;
+    my $got_success;
+    my $got_error;
 
-    my $chg = load_changer($finished_cb) or return;
+    my ($storage, $chg) = load_changer($finished_cb) or return;
 
     if (@args) {
 	@changed_args = (changed => shift @args);
     }
     $chg->update(@changed_args,
 	user_msg_fn => sub {
-	    print STDERR "$_[0]\n";
+	    if ($_[0]->{'code'} == 1100019) {
+		print STDERR "$_[0]";
+	    } else {
+		print STDERR "$_[0]\n";
+	    }
+	    $got_success++ if $_[0]->{'severity'} eq $Amanda::Message::SUCCESS;
+	    $got_error++ if $_[0]->{'severity'} eq $Amanda::Message::ERROR;
 	},
 	finished_cb => sub {
 	    my ($err) = @_;
+	    $storage->quit();
 	    $chg->quit();
 	    return failure($err, $finished_cb) if $err;
 
-	    print STDERR "update complete\n";
+	    print STDERR "update complete\n" if $got_success;
+	    print STDERR "update failed\n" if !$got_success;
+	    $finished_cb->();
+	});
+});
+
+subcommand("sync-catalog", "sync-catalog [request] [wait]", "sync the catalog whith the devices.",
+sub {
+    my ($finished_cb, $request, $wait) = @_;
+
+    my ($storage, $chg) = load_changer($finished_cb) or return;
+
+    $chg->sync_catalog(
+	request => $request,
+	wait => $wait,
+	user_msg_fn => sub {
+	    print STDERR "$_[0]\n";
+	},
+	sync_catalog_cb => sub {
+	    my ($err) = @_;
+	    $storage->quit();
+	    $chg->quit();
+	    return failure($err, $finished_cb) if $err;
+
+	    print STDERR "sync-catalog complete\n";
 	    $finished_cb->();
 	});
 });
@@ -705,9 +725,14 @@ sub {
 sub load_changer {
     my ($finished_cb) = @_;
 
-    my $chg = Amanda::Changer->new(undef, tapelist => $tl);
-    return failure($chg, $finished_cb) if ($chg->isa("Amanda::Changer::Error"));
-    return $chg;
+    my $storage  = Amanda::Storage->new(tapelist => $tl);
+    return failure("$storage", $finished_cb) if $storage->isa("Amanda::Changer::Error");
+    my $chg = $storage->{'chg'};
+    if ($chg->isa("Amanda::Changer::Error")) {
+	$storage->quit();
+	return failure($chg, $finished_cb);
+    }
+    return ($storage, $chg );
 }
 
 sub failure {
@@ -739,7 +764,7 @@ sub show_slot {
 ##
 # main
 
-Amanda::Util::setup_application("amtape", "server", $CONTEXT_CMDLINE);
+Amanda::Util::setup_application("amtape", "server", $CONTEXT_CMDLINE, "amanda", "amanda");
 
 my $config_overrides = new_config_overrides($#ARGV+1);
 
@@ -755,7 +780,7 @@ usage() if (@ARGV < 1);
 
 my $config_name = shift @ARGV;
 set_config_overrides($config_overrides);
-config_init($CONFIG_INIT_EXPLICIT_NAME, $config_name);
+config_init_with_global($CONFIG_INIT_EXPLICIT_NAME, $config_name);
 my ($cfgerr_level, @cfgerr_errors) = config_errors();
 if ($cfgerr_level >= $CFGERR_WARNINGS) {
     config_print_errors();
@@ -767,7 +792,14 @@ if ($cfgerr_level >= $CFGERR_WARNINGS) {
 Amanda::Util::finish_setup($RUNNING_AS_DUMPUSER);
 
 my $tlf = Amanda::Config::config_dir_relative(getconf($CNF_TAPELIST));
-$tl = Amanda::Tapelist->new($tlf);
+($tl, my $message) = Amanda::Tapelist->new($tlf);
+if (defined $message) {
+    if ($message->{'severity'} >= $Amanda::Message::CRITICAL) {
+	die("error loading tapelist: $message");
+    }
+    print STDERR "ERROR: $message\n";
+}
+
 
 #make STDOUT not line buffered
 my $previous_fh = select(STDOUT);
