@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  * 
- * Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
  * Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
  *
  * Author: Dustin J. Mitchell <dustin@zmanda.com>
@@ -43,11 +44,11 @@ dumpspec_new(
     dumpspec_t *rv;
 
     rv = g_new0(dumpspec_t, 1);
-    if (host) rv->host = stralloc(host);
-    if (disk) rv->disk = stralloc(disk);
-    if (datestamp) rv->datestamp = stralloc(datestamp);
-    if (level) rv->level = stralloc(level);
-    if (write_timestamp) rv->write_timestamp = stralloc(write_timestamp);
+    if (host) rv->host = g_strdup(host);
+    if (disk) rv->disk = g_strdup(disk);
+    if (datestamp) rv->datestamp = g_strdup(datestamp);
+    if (level) rv->level = g_strdup(level);
+    if (write_timestamp) rv->write_timestamp = g_strdup(write_timestamp);
 
     return rv;
 }
@@ -105,13 +106,13 @@ cmdline_parse_dumpspecs(
 
             case ARG_GET_DISK:
                 arg_state = ARG_GET_DATESTAMP;
-                dumpspec->disk = stralloc(name);
+                dumpspec->disk = g_strdup(name);
                 break;
 
             case ARG_GET_DATESTAMP:
                 arg_state = ARG_GET_LEVEL;
 		if (!(flags & CMDLINE_PARSE_DATESTAMP)) continue;
-                dumpspec->datestamp = stralloc(name);
+                dumpspec->datestamp = g_strdup(name);
                 break;
 
             case ARG_GET_LEVEL:
@@ -122,7 +123,7 @@ cmdline_parse_dumpspecs(
                     && (errstr=validate_regexp(name)) != NULL) {
                     error(_("bad level regex \"%s\": %s\n"), name, errstr);
                 }
-                dumpspec->level = stralloc(name);
+                dumpspec->level = g_strdup(name);
                 break;
         }
 	amfree(new_name);
@@ -168,7 +169,7 @@ quote_dumpspec_string(char *str)
     int need_single_quotes = 0;
 
     if (!str[0])
-	return stralloc("''"); /* special-case the empty string */
+	return g_strdup("''"); /* special-case the empty string */
 
     for (p = str; *p; p++) {
         if (!isalnum((int)*p) && *p != '.' && *p != '/') need_single_quotes=1;
@@ -196,33 +197,43 @@ cmdline_format_dumpspec_components(
     char *datestamp,
     char *level)
 {
-    char *rv = NULL;
+    GPtrArray *array = g_ptr_array_new();
+    gchar **strings;
+    char *ret = NULL;
 
-    host = host? quote_dumpspec_string(host):NULL;
-    disk = disk? quote_dumpspec_string(disk):NULL;
-    datestamp = datestamp? quote_dumpspec_string(datestamp):NULL;
-    level = level? quote_dumpspec_string(level):NULL;
+    if (!host)
+        goto out;
 
-    if (host) {
-        rv = host;
-	host = NULL;
-        if (disk) {
-            rv = newvstralloc(rv, rv, " ", disk, NULL);
-            if (datestamp) {
-                rv = newvstralloc(rv, rv, " ", datestamp, NULL);
-		if (level) {
-		    rv = newvstralloc(rv, rv, " ", level, NULL);
-		}
-            }
-        }
-    }
+    g_ptr_array_add(array, quote_dumpspec_string(host));
 
-    if (host) amfree(host);
-    if (disk) amfree(disk);
-    if (datestamp) amfree(datestamp);
-    if (level) amfree(level);
+    if (!disk)
+        goto out;
 
-    return rv;
+    g_ptr_array_add(array, quote_dumpspec_string(disk));
+
+    if (!datestamp)
+        goto out;
+
+    g_ptr_array_add(array, quote_dumpspec_string(datestamp));
+
+    if (level)
+        g_ptr_array_add(array, quote_dumpspec_string(level));
+
+out:
+    g_ptr_array_add(array, NULL);
+    strings = (gchar **) g_ptr_array_free(array, FALSE);
+
+    /*
+     * Note that if the only element of the GPtrArray is a NULL pointer, the
+     * result will be an array with only a NULL pointer, in which case
+     * g_strjoinv() will return an empty string. But we want to return NULL to
+     * the caller in that case.
+     */
+
+    ret = (*strings) ? g_strjoinv(" ", strings) : NULL;
+    g_strfreev(strings);
+
+    return ret;
 }
 
 GSList *
@@ -235,7 +246,7 @@ cmdline_match_holding(
     GSList *matching_files = NULL;
     dumpfile_t file;
 
-    holding_files = holding_get_files(NULL, 1);
+    holding_files = holding_get_files(NULL, 1, 0);
 
     for (hi = holding_files; hi != NULL; hi = hi->next) {
 	/* TODO add level */
