@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -52,16 +53,15 @@ typedef struct amhost_s {
     int maxdumps;			/* maximum dumps in parallel */
     netif_t *netif;			/* network interface this host is on */
     time_t start_t;			/* time last dump was started on this host */
-    char *up;				/* generic user pointer */
     am_feature_t *features;		/* feature set */
     int	 pre_script;
     int  post_script;
+    int  status;
 } am_host_t;
 
 typedef struct disk_s {
     int		line;			/* line number of last definition */
     char       *filename;               /* the filename where it is read */
-    struct disk_s *prev, *next;		/* doubly linked disk list */
 
     am_host_t	*host;			/* host list */
     struct disk_s *hostnext;
@@ -78,6 +78,14 @@ typedef struct disk_s {
     char	*clnt_encrypt;		/* custom encryption client filter */
     char	*amandad_path;		/* amandad path on the client */
     char	*client_username;	/* username to connect on the client */
+    char	*ssl_fingerprint_file;	/* */
+    char	*ssl_cert_file;		/* */
+    char	*ssl_key_file;		/* */
+    char	*ssl_ca_cert_file;	/* */
+    char	*ssl_cipher_list;	/* */
+    int		 ssl_check_host;	/* */
+    int		 ssl_check_certificate_host;	/* */
+    int		 ssl_check_fingerprint;	/* */
     char	*client_port;		/* port to connect on the client */
     char	*ssh_keys;		/* ssh_key file to use */
     am_sl_t	*exclude_file;		/* file exclude spec */
@@ -89,7 +97,6 @@ typedef struct disk_s {
     int		priority;		/* priority of disk */
     int		allow_split;
     int         max_warnings;
-    off_t	splitsize;
     off_t	tape_splitsize;         /* size of dumpfile chunks on tape */
     char	*split_diskbuffer;      /* place where we can buffer PORT-WRITE dumps other than RAM */
     off_t	fallback_splitsize;     /* size for in-RAM PORT-WRITE buffers */
@@ -122,18 +129,22 @@ typedef struct disk_s {
     int		index;			/* produce an index ? */
     data_path_t	data_path;		/* defined data-path */
     char       *dataport_list;		/* list of address to send data */
+    char       *shm_name;		/* shm-ring name to put data to */
     int		spindle;		/* spindle # - for parallel dumps */
     int		inprogress;		/* being dumped now? */
     int		todo;
     char       *application;
     identlist_t pp_scriptlist;
     host_limit_t *dump_limit;
-    void	*up;			/* generic user pointer */
+    int		retry_dump;
+    identlist_t tags;
+    int         status;			/* for amcheck */
 } disk_t;
 
 typedef struct disklist_s {
-    disk_t *head, *tail;
+    GList *head, *tail;
 } disklist_t;
+#define get_disk(dlist)((disk_t *)((dlist)->data))
 
 #define empty(dlist)	((dlist).head == NULL)
 
@@ -141,6 +152,7 @@ typedef struct disklist_s {
  * value just as you would the return of config_init() */
 cfgerr_level_t read_diskfile(const char *, disklist_t *);
 
+disklist_t * get_disklist(void);
 am_host_t *get_hostlist(void);
 am_host_t *lookup_host(const char *hostname);
 disk_t *lookup_disk(const char *hostname, const char *diskname);
@@ -164,12 +176,17 @@ char *optionstr(disk_t *dp);
  *           application, eg. driver to dumper.
  *           It must be set to 0 if the result is sent to the client.
  */
-GPtrArray *validate_optionstr(disk_t *dp);
+gchar **validate_optionstr(disk_t *dp);
 char *xml_optionstr(disk_t *dp, int to_server);
+char *xml_optionstr_disk(char *hostname, char *diskname);
 char *xml_estimate(estimatelist_t estimatelist, am_feature_t *their_features);
+char *xml_estimate_disk(char *hostname, char *diskname,
+			am_feature_t *their_features);
 char *clean_dle_str_for_client(char *dle_str, am_feature_t *their_features);
 char *xml_application(disk_t *dp, application_t *application,
 		      am_feature_t *their_features);
+char *xml_application_disk(char *hostname, char *diskname,
+			   am_feature_t *their_features);
 char *xml_scripts(identlist_t pp_scriptlist, am_feature_t *their_features);
 char *xml_dumptype_properties(disk_t *dp);
 
@@ -180,10 +197,11 @@ char *xml_dumptype_properties(disk_t *dp);
  */
 void disable_skip_disk(disklist_t *origqp);
 
-char *match_disklist(disklist_t *origqp, gboolean exact_match, int sargc,
+GPtrArray *match_disklist(disklist_t *origqp, gboolean exact_match, int sargc,
 		     char **sargv);
 gboolean match_dumpfile(dumpfile_t *file, gboolean exact_match, int sargc,
 			char **sargv);
+void free_disklist(disklist_t *dl);
 void unload_disklist(void);
 
 netif_t *disklist_netifs(void);
