@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -44,14 +45,19 @@ int find_match(char *host, char *disk);
 static char *find_nicedate(char *datestamp);
 static int len_find_nicedate(char *datestamp);
 static int find_compare(const void *, const void *);
-static int parse_taper_datestamp_log(char *logline, char **datestamp, char **level);
+static int parse_taper_datestamp_log(char *logline, char **datestamp,
+				     char **level, char **storage, char **pool);
 static gboolean logfile_has_tape(char * label, char * datestamp,
                                  char * logfile);
 
 static char *find_sort_order = NULL;
 static GStringChunk *string_chunk = NULL;
 
-find_result_t * find_dump(disklist_t* diskqp) {
+find_result_t *
+find_dump(
+    disklist_t *diskqp,
+    int added_todo)
+{
     char *conf_logdir, *logfile = NULL;
     int tape, maxtape, logs;
     unsigned seq;
@@ -76,7 +82,6 @@ find_result_t * find_dump(disklist_t* diskqp) {
 	}
 	g_hash_table_insert(tape_seen, tp->datestamp, GINT_TO_POINTER(1));
 
-
 	/* search log files */
 
 	logs = 0;
@@ -86,34 +91,36 @@ find_result_t * find_dump(disklist_t* diskqp) {
 	for(seq = 0; 1; seq++) {
 	    char seq_str[NUM_STR_SIZE];
 
-	    g_snprintf(seq_str, SIZEOF(seq_str), "%u", seq);
-	    logfile = newvstralloc(logfile,
-			conf_logdir, "/log.", tp->datestamp, ".", seq_str, NULL);
+	    g_snprintf(seq_str, sizeof(seq_str), "%u", seq);
+	    g_free(logfile);
+	    logfile = g_strconcat(conf_logdir, "/log.", tp->datestamp, ".",
+	        seq_str, NULL);
 	    if(access(logfile, R_OK) != 0) break;
 	    if (search_logfile(&output_find, NULL, tp->datestamp,
-                               logfile, diskqp)) {
+                               logfile, diskqp, added_todo)) {
                 logs ++;
             }
 	}
 
 	/* search old-style amflush log, if any */
 
-	logfile = newvstralloc(logfile, conf_logdir, "/log.",
-                               tp->datestamp, ".amflush", NULL);
+	g_free(logfile);
+	logfile = g_strconcat(conf_logdir, "/log.", tp->datestamp, ".amflush",
+	    NULL);
 	if(access(logfile,R_OK) == 0) {
 	    if (search_logfile(&output_find, NULL, tp->datestamp,
-                               logfile, diskqp)) {
+                               logfile, diskqp, added_todo)) {
                 logs ++;
             }
         }
-        
+
 	/* search old-style main log, if any */
 
-	logfile = newvstralloc(logfile, conf_logdir, "/log.", tp->datestamp,
-                               NULL);
+	g_free(logfile);
+	logfile = g_strconcat(conf_logdir, "/log.", tp->datestamp, NULL);
 	if(access(logfile,R_OK) == 0) {
 	    if (search_logfile(&output_find, NULL, tp->datestamp,
-                               logfile, diskqp)) {
+                               logfile, diskqp, added_todo)) {
                 logs ++;
             }
 	}
@@ -122,7 +129,7 @@ find_result_t * find_dump(disklist_t* diskqp) {
     amfree(logfile);
     amfree(conf_logdir);
 
-    search_holding_disk(&output_find, diskqp);
+    search_holding_disk(&output_find, diskqp, added_todo);
 
     return(output_find);
 }
@@ -141,7 +148,7 @@ find_log(void)
     conf_logdir = config_dir_relative(getconf_str(CNF_LOGDIR));
     maxtape = lookup_nb_tape();
 
-    output_find_log = alloc((maxtape*5+10) * SIZEOF(char *));
+    output_find_log = g_malloc((maxtape*5+10) * sizeof(char *));
     current_log = output_find_log;
 
     for(tape = 1; tape <= maxtape; tape++) {
@@ -158,13 +165,17 @@ find_log(void)
 	for(seq = 0; 1; seq++) {
 	    char seq_str[NUM_STR_SIZE];
 
-	    g_snprintf(seq_str, SIZEOF(seq_str), "%u", seq);
-	    logfile = newvstralloc(logfile, "log.", tp->datestamp, ".", seq_str, NULL);
-	    pathlogfile = newvstralloc(pathlogfile, conf_logdir, "/", logfile, NULL);
+	    g_snprintf(seq_str, sizeof(seq_str), "%u", seq);
+	    g_free(logfile);
+	    logfile = g_strconcat("log.", tp->datestamp, ".", seq_str, NULL);
+
+	    g_free(pathlogfile);
+	    pathlogfile = g_strconcat(conf_logdir, "/", logfile, NULL);
 	    if (access(pathlogfile, R_OK) != 0) break;
 	    if (logfile_has_tape(tp->label, tp->datestamp, pathlogfile)) {
-		if (current_log == output_find_log || strcmp(*(current_log-1), logfile)) {
-		    *current_log = stralloc(logfile);
+		if (current_log == output_find_log || !g_str_equal(*(current_log - 1),
+                                                                   logfile)) {
+		    *current_log = g_strdup(logfile);
 		    current_log++;
 		}
 		logs++;
@@ -174,12 +185,16 @@ find_log(void)
 
 	/* search old-style amflush log, if any */
 
-	logfile = newvstralloc(logfile, "log.", tp->datestamp, ".amflush", NULL);
-	pathlogfile = newvstralloc(pathlogfile, conf_logdir, "/", logfile, NULL);
+	g_free(logfile);
+	logfile = g_strconcat("log.", tp->datestamp, ".amflush", NULL);
+
+	g_free(pathlogfile);
+	pathlogfile = g_strconcat(conf_logdir, "/", logfile, NULL);
 	if (access(pathlogfile, R_OK) == 0) {
 	    if (logfile_has_tape(tp->label, tp->datestamp, pathlogfile)) {
-		if (current_log == output_find_log || strcmp(*(current_log-1), logfile)) {
-		    *current_log = stralloc(logfile);
+		if (current_log == output_find_log || !g_str_equal(*(current_log - 1),
+                                                                   logfile)) {
+		    *current_log = g_strdup(logfile);
 		    current_log++;
 		}
 		logs++;
@@ -188,19 +203,22 @@ find_log(void)
 
 	/* search old-style main log, if any */
 
-	logfile = newvstralloc(logfile, "log.", tp->datestamp, NULL);
-	pathlogfile = newvstralloc(pathlogfile, conf_logdir, "/", logfile, NULL);
+	g_free(logfile);
+	logfile = g_strconcat("log.", tp->datestamp, NULL);
+	g_free(pathlogfile);
+	pathlogfile = g_strconcat(conf_logdir, "/", logfile, NULL);
 	if (access(pathlogfile, R_OK) == 0) {
 	    if (logfile_has_tape(tp->label, tp->datestamp, pathlogfile)) {
-		if (current_log == output_find_log || strcmp(*(current_log-1), logfile)) {
-		    *current_log = stralloc(logfile);
+		if (current_log == output_find_log || !g_str_equal(*(current_log - 1),
+                                                                   logfile)) {
+		    *current_log = g_strdup(logfile);
 		    current_log++;
 		}
 		logs++;
 	    }
 	}
 
-	if(logs == 0 && strcmp(tp->datestamp,"0") != 0)
+	if(logs == 0 && !g_str_equal(tp->datestamp, "0"))
 	    g_fprintf(stderr, _("Warning: no log files found for tape %s written %s\n"),
 		   tp->label, find_nicedate(tp->datestamp));
     }
@@ -211,10 +229,32 @@ find_log(void)
     return(output_find_log);
 }
 
+GHashTable *
+hash_find_log(void)
+{
+    int tape, maxtape;
+    tape_t *tp;
+    char *prefix;
+    GHashTable *hash_output_find_log = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+
+    maxtape = lookup_nb_tape();
+
+    for (tape = 1; tape <= maxtape; tape++) {
+
+	tp = lookup_tapepos(tape);
+	if (tp == NULL) continue;
+
+	prefix = g_strconcat("log.", tp->datestamp, NULL);
+	g_hash_table_insert(hash_output_find_log, prefix, prefix);
+    }
+    return hash_output_find_log;
+}
+
 void
 search_holding_disk(
     find_result_t **output_find,
-    disklist_t * dynamic_disklist)
+    disklist_t * dynamic_disklist,
+    int added_todo)
 {
     GSList *holding_file_list;
     GSList *e;
@@ -222,7 +262,7 @@ search_holding_disk(
     disk_t *dp;
     char   *orig_name;
 
-    holding_file_list = holding_get_files(NULL, 1);
+    holding_file_list = holding_get_files(NULL, 1, 0);
 
     if (string_chunk == NULL) {
 	string_chunk = g_string_chunk_new(32768);
@@ -243,15 +283,14 @@ search_holding_disk(
 
 	dp = NULL;
 	orig_name = g_strdup(file.name);
-	for(;;) {
+	for (;;) {
 	    char *s;
-	    if((dp = lookup_disk(file.name, file.disk)))
+	    if ((dp = lookup_disk(orig_name, file.disk)))
 		break;
-	    if((s = strrchr(file.name,'.')) == NULL)
+	    if ((s = strrchr(orig_name,'.')) == NULL)
 		break;
 	    *s = '\0';
 	}
-	strcpy(file.name, orig_name); /* restore munged string */
 	g_free(orig_name);
 
 	if ( dp == NULL ) {
@@ -260,6 +299,7 @@ search_holding_disk(
 		continue;
 	    }
 	    dp = add_disk(dynamic_disklist, file.name, file.disk);
+	    dp->todo = added_todo;
 	}
 
 	if(find_match(file.name,file.disk)) {
@@ -269,6 +309,8 @@ search_holding_disk(
 	    new_output_find->write_timestamp = g_string_chunk_insert_const(string_chunk, "00000000000000");
 	    new_output_find->hostname = g_string_chunk_insert_const(string_chunk, file.name);
 	    new_output_find->diskname = g_string_chunk_insert_const(string_chunk, file.disk);
+	    new_output_find->storage = g_string_chunk_insert_const(string_chunk, "HOLDING");
+	    new_output_find->pool = g_string_chunk_insert_const(string_chunk, "HOLDING");
 	    new_output_find->level=file.dumplevel;
 	    new_output_find->label=g_string_chunk_insert_const(string_chunk, holding_file);
 	    new_output_find->partnum = -1;
@@ -316,8 +358,8 @@ find_compare(
         } else {
             i = *(find_result_t **)i1;
             j = *(find_result_t **)j1;
-        }            
-        
+        }
+
 	switch (sort_key) {
 	case 'h' : compare=strcmp(i->hostname,j->hostname);
 		   break;
@@ -338,6 +380,8 @@ find_compare(
 	case 'p' :
 		   compare=i->partnum - j->partnum;
 		   break;
+	case 's' : compare=i->storage_id - j->storage_id;
+		   break;
 	}
 	if(compare != 0)
 	    return compare;
@@ -349,11 +393,22 @@ void
 sort_find_result(
     char *sort_order,
     find_result_t **output_find)
+{
+    sort_find_result_with_storage(sort_order, NULL, output_find);
+}
+
+void
+sort_find_result_with_storage(
+    char *sort_order,
+    char **storage_list,
+    find_result_t **output_find)
 {
     find_result_t *output_find_result;
     find_result_t **array_find_result = NULL;
     size_t nb_result=0;
     size_t no_result;
+    int    i;
+    identlist_t il;
 
     find_sort_order = sort_order;
     /* qsort core dump if nothing to sort */
@@ -365,10 +420,28 @@ sort_find_result(
 	output_find_result;
 	output_find_result=output_find_result->next) {
 	nb_result++;
+	if (!storage_list) {
+	    for (i = 1, il = getconf_identlist(CNF_STORAGE); il != NULL;
+		i++, il = il->next) {
+		char *storage_n = il->data;
+		if (g_str_equal(output_find_result->storage,
+				storage_n)) {
+		    output_find_result->storage_id = i;
+		}
+	    }
+	} else {
+	    char **storage_l;
+	    for (i=1, storage_l = storage_list; *storage_l != NULL;
+		 storage_l++, i++) {
+		if (g_str_equal(output_find_result->storage, *storage_l)) {
+		    output_find_result->storage_id = i;
+		}
+	    }
+	}
     }
 
     /* put the list in an array */
-    array_find_result=alloc(nb_result * SIZEOF(find_result_t *));
+    array_find_result=g_malloc(nb_result * sizeof(find_result_t *));
     for(output_find_result=*output_find,no_result=0;
 	output_find_result;
 	output_find_result=output_find_result->next,no_result++) {
@@ -376,7 +449,7 @@ sort_find_result(
     }
 
     /* sort the array */
-    qsort(array_find_result,nb_result,SIZEOF(find_result_t *),
+    qsort(array_find_result,nb_result,sizeof(find_result_t *),
 	  find_compare);
 
     /* put the sorted result in the list */
@@ -398,6 +471,8 @@ print_find_result(
     int max_len_hostname  = 4;
     int max_len_diskname  = 4;
     int max_len_level     = 2;
+    int max_len_storage   = 7;
+    int max_len_pool      = 4;
     int max_len_label     =12;
     int max_len_filenum   = 4;
     int max_len_part      = 4;
@@ -427,6 +502,18 @@ print_find_result(
                 max_len_label = (int)len;
         }
 
+        if (output_find_result->storage != NULL) {
+	    len = len_quote_string(output_find_result->storage);
+            if((int)len > max_len_storage)
+                max_len_storage = (int)len;
+        }
+
+        if (output_find_result->pool != NULL) {
+	    len = len_quote_string(output_find_result->pool);
+            if((int)len > max_len_pool)
+                max_len_pool = (int)len;
+        }
+
 	len=strlen(output_find_result->status) + 1 + strlen(output_find_result->dump_status);
 	if((int)len > max_len_status)
 	    max_len_status = (int)len;
@@ -450,11 +537,13 @@ print_find_result(
 	g_printf(_("\nNo dump to list\n"));
     }
     else {
-	g_printf(_("\ndate%*s host%*s disk%*s lv%*s tape or file%*s file%*s part%*s status\n"),
+	g_printf(_("\ndate%*s host%*s disk%*s lv%*s storage%*s pool%*s tape or file%*s file%*s part%*s status\n"),
 	       max_len_datestamp-4,"",
 	       max_len_hostname-4 ,"",
 	       max_len_diskname-4 ,"",
 	       max_len_level-2    ,"",
+	       max_len_storage-7  ,"",
+	       max_len_pool-4     ,"",
 	       max_len_label-12   ,"",
 	       max_len_filenum-4  ,"",
 	       max_len_part-4  ,"");
@@ -468,16 +557,16 @@ print_find_result(
 
 	    qdiskname = quote_string(output_find_result->diskname);
             if (output_find_result->label == NULL)
-                formatted_label = stralloc("");
+                formatted_label = g_strdup("");
 	    else
 		formatted_label = quote_string(output_find_result->label);
 
-	    if (strcmp(output_find_result->status, "OK") != 0 ||
-		strcmp(output_find_result->dump_status, "OK") != 0) {
-		status = vstralloc(output_find_result->status, " ",
+	    if (!g_str_equal(output_find_result->status, "OK") ||
+		!g_str_equal(output_find_result->dump_status, "OK")) {
+		status = g_strjoin(NULL, output_find_result->status, " ",
 				   output_find_result->dump_status, NULL);
 	    } else {
-		status = stralloc(output_find_result->status);
+		status = g_strdup(output_find_result->status);
 	    }
 
 	    /*@ignore@*/
@@ -485,12 +574,14 @@ print_find_result(
 	     * output from 'amadmin' */
 	    s = g_strdup_printf("%d/%d", output_find_result->partnum,
 					 output_find_result->totalparts);
-	    g_printf("%-*s %-*s %-*s %*d %-*s %*lld %*s %s %s\n",
-                     max_len_datestamp, 
+	    g_printf("%-*s %-*s %-*s %*d %-*s %-*s %-*s %*lld %*s %s %s\n",
+                     max_len_datestamp,
                      find_nicedate(output_find_result->timestamp),
                      max_len_hostname,  output_find_result->hostname,
                      max_len_diskname,  qdiskname,
                      max_len_level,     output_find_result->level,
+                     max_len_storage,   output_find_result->storage,
+                     max_len_pool,      output_find_result->pool,
                      max_len_label,     formatted_label,
                      max_len_filenum,   (long long)output_find_result->filenum,
                      max_len_part,      s,
@@ -550,7 +641,7 @@ find_nicedate(
     day   = numdate % 100;
 
     if(strlen(datestamp) <= 8) {
-	g_snprintf(nice, SIZEOF(nice), "%4d-%02d-%02d",
+	g_snprintf(nice, sizeof(nice), "%4d-%02d-%02d",
 		year, month, day);
     }
     else {
@@ -561,7 +652,7 @@ find_nicedate(
 	minutes = (numtime / 100) % 100;
 	seconds = numtime % 100;
 
-	g_snprintf(nice, SIZEOF(nice), "%4d-%02d-%02d %02d:%02d:%02d",
+	g_snprintf(nice, sizeof(nice), "%4d-%02d-%02d %02d:%02d:%02d",
 		year, month, day, hours, minutes, seconds);
     }
 
@@ -583,10 +674,13 @@ static int
 parse_taper_datestamp_log(
     char *logline,
     char **datestamp,
-    char **label)
+    char **label,
+    char **storage,
+    char **pool)
 {
     char *s;
     int ch;
+    char *qnext_token, *uqnext_token;
 
     s = logline;
     ch = *s++;
@@ -611,9 +705,56 @@ parse_taper_datestamp_log(
     if(ch == '\0') {
 	return 0;
     }
-    if(strncmp_const_skip(s - 1, "label", s, ch) != 0) {
+    qnext_token = s - 1;
+    skip_quoted_string(s, ch);
+    s[-1] = '\0';
+
+    *storage = g_strdup(get_config_name());
+    uqnext_token = unquote_string(qnext_token);
+    if (strncmp(uqnext_token, "ST:", 3) == 0) {
+	skip_whitespace(s, ch);
+	if(ch == '\0') {
+	    return 0;
+	}
+	g_free(*storage);
+	*storage = g_strdup(uqnext_token+3);
+	g_free(uqnext_token);
+	skip_whitespace(s, ch);
+	if(ch == '\0') {
+	    return 0;
+	}
+	qnext_token = s - 1;
+	skip_quoted_string(s, ch);
+	s[-1] = '\0';
+
+	uqnext_token = unquote_string(qnext_token);
+    }
+
+    *pool = g_strdup(*storage);
+    if (strncmp(uqnext_token, "POOL:", 5) == 0) {
+	skip_whitespace(s, ch);
+	if(ch == '\0') {
+	    return 0;
+	}
+	g_free(*pool);
+	*pool = g_strdup(uqnext_token+5);
+	g_free(uqnext_token);
+	skip_whitespace(s, ch);
+	if(ch == '\0') {
+	    return 0;
+	}
+	qnext_token = s - 1;
+	skip_quoted_string(s, ch);
+	s[-1] = '\0';
+
+	uqnext_token = unquote_string(qnext_token);
+    }
+
+    if (strncmp(uqnext_token, "label", 5) != 0) {
+	g_free(uqnext_token);
 	return 0;
     }
+    g_free(uqnext_token);
 
     skip_whitespace(s, ch);
     if(ch == '\0') {
@@ -624,6 +765,7 @@ parse_taper_datestamp_log(
     s[-1] = '\0';
 
     *label = unquote_string(*label);
+
     return 1;
 }
 
@@ -631,7 +773,7 @@ parse_taper_datestamp_log(
 static gboolean logfile_has_tape(char * label, char * datestamp,
                                  char * logfile) {
     FILE * logf;
-    char * ck_datestamp, *ck_label = NULL;
+    char * ck_datestamp, *ck_label = NULL, *ck_storage = NULL, *ck_pool = NULL;
     if((logf = fopen(logfile, "r")) == NULL) {
 	error(_("could not open logfile %s: %s"), logfile, strerror(errno));
 	/*NOTREACHED*/
@@ -640,16 +782,20 @@ static gboolean logfile_has_tape(char * label, char * datestamp,
     while(get_logline(logf)) {
 	if(curlog == L_START && curprog == P_TAPER) {
 	    if(parse_taper_datestamp_log(curstr,
-					 &ck_datestamp, &ck_label) == 0) {
+					 &ck_datestamp, &ck_label, &ck_storage, &ck_pool) == 0) {
 		g_printf(_("strange log line \"start taper %s\" curstr='%s'\n"),
                          logfile, curstr);
-	    } else if(strcmp(ck_datestamp, datestamp) == 0
-		      && strcmp(ck_label, label) == 0) {
+	    } else if(g_str_equal(ck_datestamp, datestamp)
+		      && g_str_equal(ck_label, label)) {
 		amfree(ck_label);
+		amfree(ck_storage);
+		amfree(ck_pool);
                 afclose(logf);
                 return TRUE;
 	    }
 	    amfree(ck_label);
+	    amfree(ck_storage);
+	    amfree(ck_pool);
 	}
     }
 
@@ -669,13 +815,13 @@ volume_matches(
 	return TRUE;
 
     if (label1)
-	return (strcmp(label1, label2) == 0);
+	return (g_str_equal(label1, label2));
 
     /* check in tapelist */
     if (!(tp = lookup_tapelabel(label2)))
 	return FALSE;
 
-    if (strcmp(tp->datestamp, datestamp) != 0)
+    if (!g_str_equal(tp->datestamp, datestamp))
 	return FALSE;
 
     return TRUE;
@@ -689,20 +835,25 @@ search_logfile(
     const char *label,
     const char *passed_datestamp,
     const char *logfile,
-    disklist_t * dynamic_disklist)
+    disklist_t * dynamic_disklist,
+    int added_todo)
 {
     FILE *logf;
-    char *host, *host_undo;
-    char *disk, *qdisk, *disk_undo;
-    char *date, *date_undo;
+    char *host = NULL;
+    char *disk = NULL, *qdisk, *disk_undo;
+    char *date = NULL, *date_undo;
     int  partnum;
     int  totalparts;
     int  maxparts = -1;
     char *number;
     int fileno;
-    char *current_label = stralloc("");
+    char *current_label;
+    char *current_storage;
+    char *current_pool;
     char *rest, *rest_undo;
     char *ck_label=NULL;
+    char *ck_storage=NULL;
+    char *ck_pool=NULL;
     int level = 0;
     off_t filenum;
     char *ck_datestamp=NULL;
@@ -721,10 +872,17 @@ search_logfile(
     off_t bytes;
     off_t orig_kb;
     int   taper_part = 0;
+    crc_t crc1, crc2, crc3;
+    crc_t native_crc;
+    crc_t client_crc;
+    crc_t server_crc;
 
     g_return_val_if_fail(output_find != NULL, 0);
     g_return_val_if_fail(logfile != NULL, 0);
 
+    current_label = g_strdup("");
+    current_storage = g_strdup("");
+    current_pool = g_strdup("");
     if (string_chunk == NULL) {
 	string_chunk = g_string_chunk_new(32768);
     }
@@ -733,6 +891,9 @@ search_logfile(
     datestamp = g_strdup(passed_datestamp);
 
     if((logf = fopen(logfile, "r")) == NULL) {
+	if (errno != ENOENT) {
+	    g_debug("could not open logfile %s: %s", logfile, strerror(errno));
+	}
 	error(_("could not open logfile %s: %s"), logfile, strerror(errno));
 	/*NOTREACHED*/
     }
@@ -741,15 +902,17 @@ search_logfile(
     while(get_logline(logf)) {
 	if (curlog == L_START && curprog == P_TAPER) {
 	    amfree(ck_label);
+	    amfree(ck_storage);
+	    amfree(ck_pool);
 	    ck_datestamp = NULL;
 	    if(parse_taper_datestamp_log(curstr, &ck_datestamp,
-                                         &ck_label) == 0) {
+                                         &ck_label, &ck_storage, &ck_pool) == 0) {
 		g_printf(_("strange log line in %s \"start taper %s\"\n"),
                          logfile, curstr);
                 continue;
 	    }
             if (datestamp != NULL) {
-                if (strcmp(datestamp, ck_datestamp) != 0) {
+                if (!g_str_equal(datestamp, ck_datestamp)) {
                     g_printf(_("Log file %s stamped %s, expecting %s!\n"),
                              logfile, ck_datestamp, datestamp);
 		    amfree(ck_label);
@@ -768,6 +931,12 @@ search_logfile(
             amfree(current_label);
             current_label = ck_label;
 	    ck_label = NULL;
+            amfree(current_storage);
+            current_storage = ck_storage;
+	    ck_storage = NULL;
+            amfree(current_pool);
+            current_pool = ck_pool;
+	    ck_pool = NULL;
             if (datestamp == NULL) {
                 datestamp = g_strdup(ck_datestamp);
             }
@@ -800,51 +969,111 @@ search_logfile(
 		continue;
 	    }
 
-	    if (curlog == L_PART || curlog == L_PARTPARTIAL) {
+	    if (curlog == L_PART || curlog == L_PARTPARTIAL ||
+		curlog == L_DONE || curlog == L_FAIL || curlog == L_PARTIAL) {
+		char *part_storage;
+		char *qpart_storage;
+		char *part_pool;
 		char *part_label;
-		char *qpart_label = s - 1;
+		char *qnext_string;
+		char *next_string;
 		taper_part++;
+
+		qpart_storage = s - 1;
 		skip_quoted_string(s, ch);
 		s[-1] = '\0';
-
-		part_label = unquote_string(qpart_label);
-		if (!g_hash_table_lookup(valid_label, part_label)) {
-		    amfree(part_label);
-		    continue;
+		part_storage = unquote_string(qpart_storage);
+		if (strcmp(part_storage, "VAULT") == 0) {
+		    g_free(part_storage);
+		    skip_whitespace(s, ch);
+		    qpart_storage = s - 1;
+		    skip_quoted_string(s, ch);
+		    s[-1] = '\0';
+		    part_storage = unquote_string(qpart_storage);
 		}
-		amfree(current_label);
-		current_label = part_label;
-
-		skip_whitespace(s, ch);
-		if(ch == '\0') {
-		    g_printf("strange log line in %s \"%s\"\n",
-			   logfile, curstr);
-		    continue;
+		if (strncmp(part_storage, "ST:", 3) == 0) {
+		    char *ps = part_storage;
+		    part_storage = g_strdup(ps+3);
+		    g_free(ps);
+		    skip_whitespace(s, ch);
+		    qnext_string = s - 1;
+		    skip_quoted_string(s, ch);
+		    s[-1] = '\0';
+		    next_string = unquote_string(qnext_string);
+		} else {
+		    next_string = part_storage;
+		    part_storage = g_strdup(get_config_name());
 		}
 
-		number = s - 1;
-		skip_non_whitespace(s, ch);
-		s[-1] = '\0';
-		fileno = atoi(number);
-		filenum = fileno;
-		if (filenum == 0)
-		    continue;
+		if (strncmp(next_string, "POOL:", 5) == 0) {
+		    char *pp = next_string;
+		    part_pool = g_strdup(next_string+5);
+		    g_free(pp);
+		    skip_whitespace(s, ch);
+		    qnext_string = s - 1;
+		    skip_quoted_string(s, ch);
+		    s[-1] = '\0';
+		    next_string = unquote_string(qnext_string);
+		} else {
+		    part_pool = g_strdup(part_storage);
+		}
 
-		skip_whitespace(s, ch);
-		if(ch == '\0') {
-		    g_printf("strange log line in %s \"%s\"\n",
+		if (curlog == L_PART || curlog == L_PARTPARTIAL) {
+		    part_label = next_string;
+		    if (!g_hash_table_lookup(valid_label, part_label)) {
+			amfree(part_label);
+			amfree(part_storage);
+			amfree(part_pool);
+			continue;
+		    }
+		    amfree(current_label);
+		    current_label = part_label;
+		    amfree(current_storage);
+		    current_storage = part_storage;
+		    amfree(current_pool);
+		    current_pool = part_pool;
+
+		    skip_whitespace(s, ch);
+		    if (ch == '\0') {
+			g_printf("strange log line in %s \"%s\"\n",
 			   logfile, curstr);
-		    continue;
+			continue;
+		    }
+
+		    number = s - 1;
+		    skip_non_whitespace(s, ch);
+		    s[-1] = '\0';
+		    fileno = atoi(number);
+		    filenum = fileno;
+		    if (filenum == 0)
+			continue;
+
+		    skip_whitespace(s, ch);
+		    if(ch == '\0') {
+			g_printf("strange log line in %s \"%s\"\n",
+			   logfile, curstr);
+			continue;
+		    }
+		    amfree(host);
+		    host = s - 1;
+		    skip_non_whitespace(s, ch);
+		    s[-1] = '\0';
+		    host = g_strdup(host);
+		} else {
+		    amfree(part_storage);
+		    amfree(part_pool);
+		    amfree(host);
+		    host = next_string;
 		}
 	    } else {
 		taper_part = 0;
+		    amfree(host);
+		host = s - 1;
+		skip_non_whitespace(s, ch);
+		s[-1] = '\0';
+		host = g_strdup(host);
 	    }
 
-	    host = s - 1;
-	    skip_non_whitespace(s, ch);
-	    host_undo = s - 1;
-	    *host_undo = '\0';
-
 	    skip_whitespace(s, ch);
 	    if(ch == '\0') {
 		g_printf(_("strange log line in %s \"%s\"\n"),
@@ -855,22 +1084,26 @@ search_logfile(
 	    skip_quoted_string(s, ch);
 	    disk_undo = s - 1;
 	    *disk_undo = '\0';
+	    amfree(disk);
 	    disk = unquote_string(qdisk);
 
 	    skip_whitespace(s, ch);
 	    if(ch == '\0') {
 		g_printf(_("strange log line in %s \"%s\"\n"),
                          logfile, curstr);
+		amfree(disk);
 		continue;
 	    }
+	    amfree(date);
 	    date = s - 1;
 	    skip_non_whitespace(s, ch);
 	    date_undo = s - 1;
 	    *date_undo = '\0';
+	    date = g_strdup(date);
 
 	    if(strlen(date) < 3) { /* old log didn't have datestamp */
 		level = atoi(date);
-		date = stralloc(datestamp);
+		date = g_strdup(datestamp);
 		partnum = 1;
 		totalparts = 1;
 	    } else {
@@ -903,6 +1136,7 @@ search_logfile(
 		if(ch == '\0' || sscanf(s - 1, "%d", &level) != 1) {
 		    g_printf(_("Fstrange log line in %s \"%s\"\n"),
 		    logfile, s-1);
+		    amfree(disk);
 		    continue;
 		}
 		skip_integer(s, ch);
@@ -912,17 +1146,70 @@ search_logfile(
 	    if(ch == '\0') {
 		g_printf(_("strange log line in %s \"%s\"\n"),
 		    logfile, curstr);
+		amfree(disk);
 		continue;
 	    }
 	    rest = s - 1;
 	    skip_non_whitespace(s, ch);
 	    rest_undo = s - 1;
 	    *rest_undo = '\0';
-	    if (strcmp(rest, "[sec") == 0) {
+	    crc1.crc = 0;
+	    crc1.size = 0;
+	    crc2.crc = 0;
+	    crc2.size = 0;
+	    crc3.crc = 0;
+	    crc3.size = 0;
+	    if (curlog == L_DONE) {
+		if (!g_str_equal(rest, "[sec")) {
+		    // CRC
+		    parse_crc(rest, &crc1);
+		    skip_whitespace(s, ch);
+		    rest = s - 1;
+		    skip_non_whitespace(s, ch);
+		    rest_undo = s - 1;
+		    *rest_undo = '\0';
+		}
+		if (!g_str_equal(rest, "[sec")) {
+		    // CRC
+		    parse_crc(rest, &crc2);
+		    skip_whitespace(s, ch);
+		    rest = s - 1;
+		    skip_non_whitespace(s, ch);
+		    rest_undo = s - 1;
+		    *rest_undo = '\0';
+		}
+		if (!g_str_equal(rest, "[sec")) {
+		    // CRC
+		    parse_crc(rest, &crc3);
+		    skip_whitespace(s, ch);
+		    rest = s - 1;
+		    skip_non_whitespace(s, ch);
+		    rest_undo = s - 1;
+		    *rest_undo = '\0';
+		}
+	    }
+	    native_crc.crc = 0;
+	    native_crc.size = 0;
+	    client_crc.crc = 0;
+	    client_crc.size = 0;
+	    server_crc.crc = 0;
+	    server_crc.size = 0;
+	    if (curprog == P_DUMPER) {
+		native_crc = crc1;
+		client_crc = crc2;
+	    } else if (curprog == P_CHUNKER) {
+		server_crc = crc1;
+	    } else if (curprog == P_TAPER) {
+		native_crc = crc1;
+		client_crc = crc2;
+		server_crc = crc3;
+	    }
+	    if (g_str_equal(rest, "[sec")) {
 		skip_whitespace(s, ch);
 		if(ch == '\0') {
 		    g_printf(_("strange log line in %s \"%s\"\n"),
 			     logfile, curstr);
+		    amfree(disk);
 		    continue;
 		}
 		sec = atof(s - 1);
@@ -932,10 +1219,11 @@ search_logfile(
 		skip_non_whitespace(s, ch);
 		rest_undo = s - 1;
 		*rest_undo = '\0';
-		if (strcmp(rest, "kb") != 0 &&
-		    strcmp(rest, "bytes") != 0) {
+		if (!g_str_equal(rest, "kb") &&
+		    !g_str_equal(rest, "bytes")) {
 		    g_printf(_("Bstrange log line in %s \"%s\"\n"),
 			     logfile, curstr);
+		    amfree(disk);
 		    continue;
 		}
 
@@ -943,9 +1231,10 @@ search_logfile(
 		if (ch == '\0') {
 		     g_printf(_("strange log line in %s \"%s\"\n"),
 			      logfile, curstr);
+		     amfree(disk);
 		     continue;
 		}
-		if (strcmp(rest, "kb") == 0) {
+		if (g_str_equal(rest, "kb")) {
 		    kb = atof(s - 1);
 		    bytes = 0;
 		} else {
@@ -958,9 +1247,10 @@ search_logfile(
 		skip_non_whitespace(s, ch);
 		rest_undo = s - 1;
 		*rest_undo = '\0';
-		if (strcmp(rest, "kps") != 0) {
+		if (!g_str_equal(rest, "kps")) {
 		    g_printf(_("Cstrange log line in %s \"%s\"\n"),
 			     logfile, curstr);
+		    amfree(disk);
 		    continue;
 		}
 
@@ -968,6 +1258,7 @@ search_logfile(
 		if (ch == '\0') {
 		    g_printf(_("strange log line in %s \"%s\"\n"),
 			     logfile, curstr);
+		    amfree(disk);
 		    continue;
 		}
 		/* kps = atof(s - 1); */
@@ -977,7 +1268,7 @@ search_logfile(
 		skip_non_whitespace(s, ch);
 		rest_undo = s - 1;
 		*rest_undo = '\0';
-		if (strcmp(rest, "orig-kb") != 0) {
+		if (!g_str_equal(rest, "orig-kb")) {
 		    orig_kb = 0;
 		} else {
 
@@ -985,6 +1276,7 @@ search_logfile(
 		    if(ch == '\0') {
 			g_printf(_("strange log line in %s \"%s\"\n"),
 				 logfile, curstr);
+			amfree(disk);
 			continue;
 		    }
 		    orig_kb = atof(s - 1);
@@ -997,15 +1289,17 @@ search_logfile(
 		*rest_undo = ' ';
 	    }
 
-	    if (strncmp(rest, "error", 5) == 0) rest += 6;
-	    if (strncmp(rest, "config", 6) == 0) rest += 7;
+	    if (g_str_has_prefix(rest, "error")) rest += 6;
+	    if (g_str_has_prefix(rest, "config")) rest += 7;
 
 	    dp = lookup_disk(host,disk);
 	    if ( dp == NULL ) {
 		if (dynamic_disklist == NULL) {
+		    amfree(disk);
 		    continue;
 		}
 		dp = add_disk(dynamic_disklist, host, disk);
+		dp->todo = added_todo;
 	    }
             if (find_match(host, disk)) {
 		if(curprog == P_TAPER) {
@@ -1033,6 +1327,12 @@ search_logfile(
 		    new_output_find->partnum = partnum;
 		    new_output_find->totalparts = totalparts;
 		    new_output_find->label=g_string_chunk_insert_const(string_chunk, current_label);
+		    if (current_storage) {
+			new_output_find->storage=g_string_chunk_insert_const(string_chunk, current_storage);
+		    }
+		    if (current_pool) {
+			new_output_find->pool=g_string_chunk_insert_const(string_chunk, current_pool);
+		    }
 		    new_output_find->status=NULL;
 		    new_output_find->dump_status=NULL;
 		    new_output_find->message="";
@@ -1042,6 +1342,9 @@ search_logfile(
 		    new_output_find->bytes=bytes;
 		    new_output_find->orig_kb=orig_kb;
 		    new_output_find->next=NULL;
+		    new_output_find->native_crc = native_crc;
+		    new_output_find->client_crc = client_crc;
+		    new_output_find->server_crc = server_crc;
 		    if (curlog == L_SUCCESS) {
 			new_output_find->status = "OK";
 			new_output_find->dump_status = "OK";
@@ -1072,7 +1375,7 @@ search_logfile(
 				     a_part_find;
 				     a_part_find = a_part_find->next) {
 				    if (a_part_find->partnum == num_part &&
-					strcmp(a_part_find->status, "OK") == 0)
+					g_str_equal(a_part_find->status, "OK"))
 					num_part--;
 			        }
 				/* set dump_status of each part */
@@ -1099,6 +1402,9 @@ search_logfile(
 				if (a_part_find->orig_kb == 0) {
 				    a_part_find->orig_kb = orig_kb;
 				}
+				a_part_find->native_crc = native_crc;
+				a_part_find->client_crc = client_crc;
+				a_part_find->server_crc = server_crc;
 			    }
 			}
 			if (part_find) { /* find last element */
@@ -1146,9 +1452,15 @@ search_logfile(
 		    find_result_t *new_output_find = g_new0(find_result_t, 1);
 		    new_output_find->next = *output_find;
 		    new_output_find->timestamp = g_string_chunk_insert_const(string_chunk, date);
-		    new_output_find->write_timestamp = g_strdup("00000000000000"); /* dump was not written.. */
+		    new_output_find->write_timestamp = g_string_chunk_insert_const(string_chunk, "00000000000000"); /* dump was not written.. */
 		    new_output_find->hostname=g_string_chunk_insert_const(string_chunk, host);
 		    new_output_find->diskname=g_string_chunk_insert_const(string_chunk, disk);
+		    if (current_storage != NULL) {
+			new_output_find->storage=g_string_chunk_insert_const(string_chunk, current_storage);
+		    }
+		    if (current_pool != NULL) {
+			new_output_find->pool=g_string_chunk_insert_const(string_chunk, current_pool);
+		    }
 		    new_output_find->level=level;
 		    new_output_find->label=NULL;
 		    new_output_find->partnum=partnum;
@@ -1158,7 +1470,7 @@ search_logfile(
 		    new_output_find->kb=kb;
 		    new_output_find->bytes=bytes;
 		    new_output_find->orig_kb=orig_kb;
-		    status_failed = vstralloc(
+		    status_failed = g_strjoin(NULL,
 			 "FAILED (",
 			 program_str[(int)curprog],
 			 ") ",
@@ -1173,14 +1485,20 @@ search_logfile(
 		    maxparts = -1;
 		}
 	    }
-	    amfree(disk);
 	}
     }
+    amfree(host);
+    amfree(date);
+    amfree(disk);
 
     g_hash_table_destroy(valid_label);
+    g_hash_table_destroy(part_by_dle);
     afclose(logf);
     amfree(datestamp);
     amfree(current_label);
+    amfree(current_storage);
+    amfree(current_pool);
+    amfree(disk);
 
     return found_something;
 }
@@ -1210,16 +1528,16 @@ dumps_match(
 	cur_result;
 	cur_result=cur_result->next) {
 	char level_str[NUM_STR_SIZE];
-	g_snprintf(level_str, SIZEOF(level_str), "%d", cur_result->level);
+	g_snprintf(level_str, sizeof(level_str), "%d", cur_result->level);
 	if((!hostname || *hostname == '\0' || match_host(hostname, cur_result->hostname)) &&
 	   (!diskname || *diskname == '\0' || match_disk(diskname, cur_result->diskname)) &&
 	   (!datestamp || *datestamp== '\0' || match_datestamp(datestamp, cur_result->timestamp)) &&
 	   (!level || *level== '\0' || match_level(level, level_str)) &&
-	   (!ok || !strcmp(cur_result->status, "OK")) &&
-	   (!ok || !strcmp(cur_result->dump_status, "OK"))){
+	   (!ok || g_str_equal(cur_result->status, "OK")) &&
+	   (!ok || g_str_equal(cur_result->dump_status, "OK"))){
 
 	    find_result_t *curmatch = g_new0(find_result_t, 1);
-	    memcpy(curmatch, cur_result, SIZEOF(find_result_t));
+	    memcpy(curmatch, cur_result, sizeof(find_result_t));
 
 	    curmatch->timestamp = cur_result->timestamp;
 	    curmatch->write_timestamp = cur_result->write_timestamp;
@@ -1269,7 +1587,7 @@ dumps_match_dumpspecs(
 	char level_str[NUM_STR_SIZE];
 	char *zeropad_ts = NULL;
 	char *zeropad_w_ts = NULL;
-	g_snprintf(level_str, SIZEOF(level_str), "%d", cur_result->level);
+	g_snprintf(level_str, sizeof(level_str), "%d", cur_result->level);
 
 	/* get the timestamp padded to full width */
 	if (strlen(cur_result->timestamp) < 14) {
@@ -1294,16 +1612,17 @@ dumps_match_dumpspecs(
 			|| match_datestamp(ds->write_timestamp, cur_result->write_timestamp)
 			|| (zeropad_w_ts && match_datestamp(ds->write_timestamp, zeropad_w_ts))) &&
 	       (!ds->level || *ds->level== '\0' || match_level(ds->level, level_str)) &&
-	       (!ok || !strcmp(cur_result->status, "OK")) &&
-	       (!ok || !strcmp(cur_result->dump_status, "OK"))) {
+	       (!ok || g_str_equal(cur_result->status, "OK")) &&
+	       (!ok || g_str_equal(cur_result->dump_status, "OK"))) {
 
-		find_result_t *curmatch = alloc(SIZEOF(find_result_t));
-		memcpy(curmatch, cur_result, SIZEOF(find_result_t));
+		find_result_t *curmatch = g_malloc(sizeof(find_result_t));
+		memcpy(curmatch, cur_result, sizeof(find_result_t));
 
 		curmatch->timestamp = cur_result->timestamp;
 		curmatch->write_timestamp = cur_result->write_timestamp;
 		curmatch->hostname = cur_result->hostname;
 		curmatch->diskname = cur_result->diskname;
+		curmatch->storage = cur_result->storage;
 		curmatch->level = cur_result->level;
 		curmatch->label = cur_result->label? cur_result->label : NULL;
 		curmatch->filenum = cur_result->filenum;
@@ -1338,9 +1657,9 @@ dump_exist(
     for(output_find_result=output_find;
 	output_find_result;
 	output_find_result=output_find_result->next) {
-	if( !strcmp(output_find_result->hostname, hostname) &&
-	    !strcmp(output_find_result->diskname, diskname) &&
-	    !strcmp(output_find_result->timestamp, datestamp) &&
+	if( g_str_equal(output_find_result->hostname, hostname) &&
+	    g_str_equal(output_find_result->diskname, diskname) &&
+	    g_str_equal(output_find_result->timestamp, datestamp) &&
 	    output_find_result->level == level) {
 
 	    return output_find_result;
@@ -1348,3 +1667,75 @@ dump_exist(
     }
     return(NULL);
 }
+
+GHashTable *
+make_dump_hash(
+    find_result_t *output_find)
+{
+    find_result_t *output_find_result;
+    GHashTable *dump_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, (GDestroyNotify)g_hash_table_destroy);
+    GHashTable *host_hash;
+    GHashTable *disk_hash;
+    GHashTable *time_hash;
+
+    for (output_find_result=output_find;
+         output_find_result;
+         output_find_result=output_find_result->next) {
+
+	host_hash = g_hash_table_lookup(dump_hash, output_find_result->hostname);
+	if (!host_hash) {
+	    host_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, (GDestroyNotify)g_hash_table_destroy);
+	    g_hash_table_insert(dump_hash, output_find_result->hostname, host_hash);
+	}
+
+	disk_hash = g_hash_table_lookup(host_hash, output_find_result->diskname);
+	if (!disk_hash) {
+	    disk_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, (GDestroyNotify)g_hash_table_destroy);
+	    g_hash_table_insert(host_hash, output_find_result->diskname, disk_hash);
+	}
+
+	time_hash = g_hash_table_lookup(disk_hash, output_find_result->timestamp);
+	if (!time_hash) {
+	    time_hash = g_hash_table_new(g_int_hash, g_int_equal);
+	    g_hash_table_insert(disk_hash, output_find_result->timestamp, time_hash);
+	}
+
+	g_hash_table_insert(time_hash, &output_find_result->level, output_find_result);
+    }
+
+    return dump_hash;
+}
+
+
+find_result_t *
+dump_hash_exist(
+    GHashTable *dump_hash,
+    char *hostname,
+    char *diskname,
+    char *timestamp,
+    int level)
+{
+    GHashTable *host_hash;
+    GHashTable *disk_hash;
+    GHashTable *time_hash;
+    find_result_t *output_find_result;
+
+    host_hash = g_hash_table_lookup(dump_hash, hostname);
+    if (!host_hash) return NULL;
+
+    disk_hash = g_hash_table_lookup(host_hash, diskname);
+    if (!disk_hash) return NULL;
+
+    time_hash = g_hash_table_lookup(disk_hash, timestamp);
+    if (!time_hash) return NULL;
+
+    output_find_result = g_hash_table_lookup(time_hash, &level);
+
+    return output_find_result;
+}
+
+void free_dump_hash(GHashTable *dump_hash G_GNUC_UNUSED)
+{
+    //It fail on some machine
+    //g_hash_table_destroy(dump_hash);
+}
