@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -71,7 +72,8 @@ holding_get_disks(void);
  */
 GSList *
 holding_get_files(char *hdir,
-                  int fullpaths);
+                  int fullpaths,
+		  int take_pid_lock);
 
 /* Get a list of holding files chunks in the given holding
  * file.  Always returns full paths.
@@ -166,7 +168,9 @@ holding_cleanup(corrupt_dle_fn corrupt_dle,
  * @param orig_size: the size.
  */
 void
-holding_set_origsize(char *holding_file, off_t orig_size);
+holding_set_from_driver(char *holding_file, off_t orig_size,
+			crc_t native_crc, crc_t client_crc,
+			crc_t server_crc);
 
 /* Rename holding files from the temporary names used during
  * creation.
