@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -33,7 +34,7 @@
 #include "amanda.h"
 #include "conffile.h"
 #include "infofile.h"
-#include "util.h"
+#include "amutil.h"
 
 static void zero_info(info_t *);
 
@@ -65,7 +66,7 @@ open_txinfofile(
     myhost = sanitise_filename(host);
     mydisk = sanitise_filename(disk);
 
-    infofile = vstralloc(infodir,
+    infofile = g_strjoin(NULL, infodir,
 			 "/", myhost,
 			 "/", mydisk,
 			 "/info",
@@ -82,7 +83,7 @@ open_txinfofile(
 	}
     }
 
-    newinfofile = stralloc2(infofile, ".new");
+    newinfofile = g_strconcat(infofile, ".new", NULL);
 
     if(writing) {
 	infof = fopen(newinfofile, mode);
@@ -236,24 +237,27 @@ read_txinfofile(
 	else if (strncmp_const(line,"history:") == 0) {
 	    break;				/* normal */
 	}
-	memset(&onestat, 0, SIZEOF(onestat));
+	memset(&onestat, 0, sizeof(onestat));
 
 	s = line;
 	ch = *s++;
 
 	/* from here on, we had better be parsing a 'stats' line */
 	if(strncmp_const_skip(line, "stats:", s, ch) != 0) {
+	    amfree(line);
 	    return -1;
 	}
 
 	skip_whitespace(s, ch);
 	if(ch == '\0' || sscanf((s - 1), "%d", &level) != 1) {
+	    amfree(line);
 	    return -1;
 	}
 	skip_integer(s, ch);
 
 	skip_whitespace(s, ch);
 	if(ch == '\0' || sscanf((s - 1), "%lld", &off_t_tmp) != 1) {
+	    amfree(line);
 	    return -1;
 	}
 	onestat.size = (off_t)off_t_tmp;
@@ -261,6 +265,7 @@ read_txinfofile(
 
 	skip_whitespace(s, ch);
 	if(ch == '\0' || sscanf((s - 1), "%lld", &off_t_tmp) != 1) {
+	    amfree(line);
 	    return -1;
 	}
 	onestat.csize = (off_t)off_t_tmp;
@@ -269,6 +274,7 @@ read_txinfofile(
 	/* assume that the time fits in a long long */
 	skip_whitespace(s, ch);
 	if(ch == '\0' || sscanf((s - 1), "%lld", &off_t_tmp) != 1) {
+	    amfree(line);
 	    return -1;
 	}
         onestat.secs = (time_t)off_t_tmp;
@@ -276,6 +282,7 @@ read_txinfofile(
 
 	skip_whitespace(s, ch);
 	if(ch == '\0' || sscanf((s - 1), "%lld", &off_t_tmp) != 1) {
+	    amfree(line);
 	    return -1;
 	}
 	onestat.date = (time_t)off_t_tmp;
@@ -284,6 +291,7 @@ read_txinfofile(
 	skip_whitespace(s, ch);
 	if(ch != '\0') {
 	    if(sscanf((s - 1), "%lld", &off_t_tmp) != 1) {
+		amfree(line);
 		return -1;
 	    }
 	    onestat.filenum = (off_t)off_t_tmp;
@@ -291,18 +299,21 @@ read_txinfofile(
 
 	    skip_whitespace(s, ch);
 	    if(ch == '\0') {
+		amfree(line);
 		return -1;
 	    }
-	    strncpy(onestat.label, s-1, SIZEOF(onestat.label)-1);
-	    onestat.label[SIZEOF(onestat.label)-1] = '\0';
+	    strncpy(onestat.label, s-1, sizeof(onestat.label)-1);
+	    onestat.label[sizeof(onestat.label)-1] = '\0';
 	}
 
-	if(level < 0 || level > DUMP_LEVELS-1)
+	if (level < 0 || level > DUMP_LEVELS-1) {
+	    amfree(line);
 	    return -1;
+	}
 
 	info->inf[level] = onestat;
     }
-   
+
     if(line == NULL) return -1;
 
     rc = sscanf(line, "last_level: %d %d", 
@@ -313,16 +324,17 @@ read_txinfofile(
     rc = 0;
 
     nb_history = 0;
-    for(i=0;i<=NB_HISTORY;i++) {
+    for(i=0;i<NB_HISTORY;i++) {
 	info->history[i].level = -2;
     }
 
-    for(rc = -2; (line = agets(infof)) != NULL; free(line)) {
+    while ((line = agets(infof)) != NULL) {
 	history_t onehistory;	/* one history record */
 	long long off_t_tmp;
 
 	if (line[0] == '\0')
 	    continue;
+
 	if(line[0] == '/' && line[1] == '/') {
 	    info->history[nb_history].level = -2;
 	    rc = 0;
@@ -330,7 +342,9 @@ read_txinfofile(
 	    return 0;				/* normal end of record */
 	}
 
-	memset(&onehistory, 0, SIZEOF(onehistory));
+	if (nb_history >= NB_HISTORY) break;
+
+	memset(&onehistory, 0, sizeof(onehistory));
 
 	s = line;
 	ch = *s++;
@@ -383,6 +397,7 @@ read_txinfofile(
 	}
 
 	info->history[nb_history++] = onehistory;
+	amfree(line);
     }
     amfree(line);
 
@@ -391,8 +406,6 @@ read_txinfofile(
 	    break;
 	amfree(line);
     }
-    if (line == NULL) return -1;
-    amfree(line);
 
     return rc;
 }
@@ -454,7 +467,7 @@ write_txinfofile(
 
     g_fprintf(infof, _("last_level: %d %d\n"), info->last_level, info->consecutive_runs);
 
-    for(i=0;info->history[i].level > -1;i++) {
+    for(i=0;i < NB_HISTORY && info->history[i].level > -1;i++) {
 	g_fprintf(infof, _("history: %d %lld %lld %jd %jd\n"),
 		info->history[i].level,
 		(long long)info->history[i].size,
@@ -479,12 +492,12 @@ delete_txinfofile(
 
     myhost = sanitise_filename(host);
     mydisk = sanitise_filename(disk);
-    fn = vstralloc(infodir,
+    fn = g_strjoin(NULL, infodir,
 		   "/", myhost,
 		   "/", mydisk,
 		   "/info",
 		   NULL);
-    fn_new = stralloc2(fn, ".new");
+    fn_new = g_strconcat(fn, ".new", NULL);
 
     amfree(myhost);
     amfree(mydisk);
@@ -504,7 +517,7 @@ open_infofile(
 {
     assert(infodir == NULL);
 
-    infodir = stralloc(filename);
+    infodir = g_strdup(filename);
 
     return 0; /* success! */
 }
@@ -536,7 +549,7 @@ get_dumpdate(
     }
 
     t = gmtime(&last);
-    g_snprintf(stamp, SIZEOF(stamp), "%d:%d:%d:%d:%d:%d",
+    g_snprintf(stamp, sizeof(stamp), "%d:%d:%d:%d:%d:%d",
 		t->tm_year+1900, t->tm_mon+1, t->tm_mday,
 		t->tm_hour, t->tm_min, t->tm_sec);
 
@@ -577,7 +590,7 @@ zero_info(
 {
     int i;
 
-    memset(info, '\0', SIZEOF(info_t));
+    memset(info, '\0', sizeof(info_t));
 
     for(i = 0; i < AVG_COUNT; i++) {
 	info->full.comp[i] = info->incr.comp[i] = -1.0;
@@ -591,7 +604,7 @@ zero_info(
     info->last_level = -1;
     info->consecutive_runs = -1;
 
-    for(i=0;i<=NB_HISTORY;i++) {
+    for(i=0;i<NB_HISTORY;i++) {
 	info->history[i].level = -2;
 	info->history[i].size = (off_t)0;
 	info->history[i].csize = (off_t)0;
@@ -716,6 +729,8 @@ main(
 {
   int i;
 
+  glib_init();
+
   /*
    * Configure program for internationalization:
    *   1) Only set the message locale for now.
