@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -51,14 +52,15 @@ typedef enum logtype_e {
     L_DONE, L_PART, L_PARTPARTIAL, L_SUCCESS, L_PARTIAL, L_FAIL, L_STRANGE,
     L_CHUNK, L_CHUNKSUCCESS,                            /* ... continued */
     L_STATS,						   /* statistics */
-    L_MARKER,					  /* marker for reporter */
-    L_CONT			 /* continuation line, used when reading */
+    L_CONT,			 /* continuation line, used when reading */
+    L_RETRY,
+    L_MARKER,			    /* marker for reporter, must be last */
 } logtype_t;
 
 typedef enum program_e {
-    P_UNKNOWN, P_PLANNER, P_DRIVER, P_REPORTER, P_DUMPER, P_CHUNKER,
+    P_UNKNOWN, P_PLANNER, P_DRIVER, P_AMREPORT, P_DUMPER, P_CHUNKER,
     P_TAPER, P_AMFLUSH, P_AMDUMP, P_AMIDXTAPED, P_AMFETCHDUMP, P_AMCHECKDUMP,
-    P_AMVAULT, P_SENTINEL
+    P_AMVAULT, P_AMCLEANUP, P_AMBACKUPD, P_AMTRMIDX, P_AMTRMLOG, P_SENTINEL
 } program_t;
 #define P_LAST (P_SENTINEL-1)
 
@@ -70,11 +72,17 @@ extern program_t curprog;
 extern char *curstr;
 extern char *program_str[];
 
+char *get_logtype_str(logtype_t logtype);
+char *get_program_str(program_t program);
+
 void amanda_log_trace_log(GLogLevelFlags log_level, const gchar *message);
 void log_add(logtype_t typ, char * format, ...) G_GNUC_PRINTF(2, 3);
 void log_add_full(logtype_t typ, char *pname, char *format, ...) G_GNUC_PRINTF(3, 4);
 void log_start_multiline(void);
 void log_end_multiline(void);
+char *make_logname(char *process, char *datestamp);
+char *get_logname(void);
+void set_logname(char *filename);
 void log_rename(char *datestamp);
 int get_logline(FILE *);
 
