@@ -1,7 +1,8 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -31,14 +32,14 @@
 
 #include "amanda.h"
 #include "server_util.h"
-#include "arglist.h"
 #include "logfile.h"
-#include "util.h"
+#include "amutil.h"
 #include "conffile.h"
 #include "diskfile.h"
 #include "pipespawn.h"
 #include "conffile.h"
 #include "infofile.h"
+#include "backup_support_option.h"
 #include "sys/wait.h"
 
 const char *cmdstr[] = {
@@ -47,11 +48,14 @@ const char *cmdstr[] = {
     "FAILED", "TRY-AGAIN", "NO-ROOM", "RQ-MORE-DISK",	/* dumper results */
     "ABORT-FINISHED", "BAD-COMMAND",			/* dumper results */
     "START-TAPER", "FILE-WRITE", "NEW-TAPE", "NO-NEW-TAPE",
-
-    "PARTDONE", "PORT-WRITE", "DUMPER-STATUS",		    /* taper cmds */
+    "SHM-WRITE", "SHM-DUMP", "SHM-NAME",
+    "PARTDONE", "PORT-WRITE", "VAULT-WRITE", "DUMPER-STATUS", /* taper cmds */
     "PORT", "TAPE-ERROR", "TAPER-OK",			 /* taper results */
     "REQUEST-NEW-TAPE", "DIRECTTCP-PORT", "TAKE-SCRIBE-FROM",
-    "START-SCAN", "CLOSE-VOLUME", "READY", "LAST_TOK",
+    "START-SCAN", "CLOSE-VOLUME", "CLOSED-VOLUME",
+    "OPENED-SOURCE-VOLUME",
+    "CLOSE-SOURCE-VOLUME", "CLOSED-SOURCE-VOLUME",
+    "RETRY", "READY", "LAST_TOK",
     NULL
 };
 
@@ -68,10 +72,10 @@ getcmd(void)
 	fflush(stdout);
         line = agets(stdin);
     } else {
-        line = agets(stdin);
+        line = areads(0);
     }
     if (line == NULL) {
-	line = stralloc("QUIT");
+	line = g_strdup("QUIT");
     }
 
     dbprintf(_("getcmd: %s\n"), line);
@@ -87,7 +91,7 @@ getcmd(void)
     }
 
     for(cmd_i=BOGUS; cmdstr[cmd_i] != NULL; cmd_i++)
-	if(strcmp(cmdargs->argv[0], cmdstr[cmd_i]) == 0) {
+	if(g_str_equal(cmdargs->argv[0], cmdstr[cmd_i])) {
 	    cmdargs->cmd = cmd_i;
 	    return cmdargs;
 	}
@@ -97,21 +101,9 @@ getcmd(void)
 struct cmdargs *
 get_pending_cmd(void)
 {
-    SELECT_ARG_TYPE ready;
-    struct timeval  to;
-    int             nfound;
-
-    FD_ZERO(&ready);
-    FD_SET(0, &ready);
-    to.tv_sec = 0;
-    to.tv_usec = 0;
-
-    nfound = select(1, &ready, NULL, NULL, &to);
-    if (nfound && FD_ISSET(0, &ready)) {
-        return getcmd();
-    } else {
+    if (!areads_dataready(0))
 	return NULL;
-    }
+    return getcmd();
 }
 
 void
@@ -125,43 +117,92 @@ free_cmdargs(
     g_free(cmdargs);
 }
 
-printf_arglist_function1(void putresult, cmd_t, result, const char *, format)
+void putresult(cmd_t result, const char *format, ...)
 {
     va_list argp;
+    char *msg;
 
     arglist_start(argp, format);
-    dbprintf(_("putresult: %d %s\n"), result, cmdstr[result]);
-    g_printf("%s ", cmdstr[result]);
-    g_vprintf(format, argp);
-    fflush(stdout);
+    msg = g_strdup_vprintf(format, argp);
     arglist_end(argp);
+    g_debug("putresult: %d %s %s", result, cmdstr[result], msg);
+    g_printf("%s %s", cmdstr[result], msg);
+    fflush(stdout);
+    g_free(msg);
 }
 
 char *
 amhost_get_security_conf(
-    char *	string,
-    void *	arg)
+    char *string,
+    void *arg G_GNUC_UNUSED)
 {
+    char *result = NULL;
+    disk_t *dp;
+
     if(!string || !*string)
 	return(NULL);
 
-    if(strcmp(string, "krb5principal")==0)
-	return(getconf_str(CNF_KRB5PRINCIPAL));
-    else if(strcmp(string, "krb5keytab")==0)
-	return(getconf_str(CNF_KRB5KEYTAB));
+    if (g_str_equal(string, "krb5principal"))
+	result = getconf_str(CNF_KRB5PRINCIPAL);
+    else if (g_str_equal(string, "krb5keytab"))
+	result = getconf_str(CNF_KRB5KEYTAB);
+    if (result) {
+	if (strlen(result) == 0)
+	    result = NULL;
+	return result;
+    }
 
-    if(!arg || !((am_host_t *)arg)->disks) return(NULL);
+    if (!arg || !((am_host_t *)arg)->disks) return(NULL);
 
-    if(strcmp(string, "amandad_path")==0)
-	return ((am_host_t *)arg)->disks->amandad_path;
-    else if(strcmp(string, "client_username")==0)
-	return ((am_host_t *)arg)->disks->client_username;
-    else if(strcmp(string, "client_port")==0)
-	return ((am_host_t *)arg)->disks->client_port;
-    else if(strcmp(string, "ssh_keys")==0)
-	return ((am_host_t *)arg)->disks->ssh_keys;
+    for (dp = ((am_host_t *)arg)->disks; dp != NULL; dp = dp->hostnext) {
+	if (dp->todo)
+	    break;
+    }
+    if (!dp) return(NULL);
 
-    return(NULL);
+    if (g_str_equal(string, "amandad_path"))
+	result =  dp->amandad_path;
+    else if (g_str_equal(string, "client_username"))
+	result =  dp->client_username;
+    else if (g_str_equal(string, "client_port"))
+	result =  dp->client_port;
+    else if (g_str_equal(string, "src_ip")) {
+	char *result = interface_get_src_ip(((am_host_t *)arg)->netif->config);
+	if (g_str_equal(result, "NULL"))
+	    result = NULL;
+    } else if(g_str_equal(string, "ssh_keys"))
+	result =  dp->ssh_keys;
+    else if (g_str_equal(string, "ssl_fingerprint_file"))
+	result =  dp->ssl_fingerprint_file;
+    else if (g_str_equal(string, "ssl_cert_file"))
+	result =  dp->ssl_cert_file;
+    else if (g_str_equal(string, "ssl_key_file"))
+	result =  dp->ssl_key_file;
+    else if (g_str_equal(string, "ssl_ca_cert_file"))
+	result =  dp->ssl_ca_cert_file;
+    else if (g_str_equal(string, "ssl_cipher_list"))
+	result =  dp->ssl_cipher_list;
+    else if (g_str_equal(string, "ssl_check_certificate_host")) {
+	if (dp->ssl_check_certificate_host)
+	    result = "1";
+	else
+	    result = "0";
+    } else if (g_str_equal(string, "ssl_check_host")) {
+	if (dp->ssl_check_host)
+	    result = "1";
+	else
+	    result = "0";
+    } else if (g_str_equal(string, "ssl_check_fingerprint")) {
+	if (dp->ssl_check_fingerprint)
+	    result = "1";
+	else
+	    result = "0";
+    }
+
+    if (result && strlen(result) == 0)
+	result = NULL;
+
+    return(result);
 }
 
 int check_infofile(
@@ -169,10 +210,11 @@ int check_infofile(
     disklist_t  *dl,
     char       **errmsg)
 {
+    GList       *dlist, *dlist1;
     disk_t      *dp, *diskp;
-    char        *hostinfodir, *old_hostinfodir, *Xhostinfodir;
-    char        *diskdir,     *old_diskdir,     *Xdiskdir;
-    char        *infofile,    *old_infofile,    *Xinfofile;
+    char        *hostinfodir, *old_hostinfodir;
+    char        *diskdir,     *old_diskdir;
+    char        *infofile,    *old_infofile;
     struct stat  statbuf;
     int other_dle_match;
 
@@ -180,41 +222,64 @@ int check_infofile(
 	return 0;
     }
 
-    for (dp = dl->head; dp != NULL; dp = dp->next) {
+//    for (dp = dl->head; dp != NULL; dp = dp->next) {
+    for (dlist = dl->head; dlist != NULL; dlist = dlist->next) {
+	dp = dlist->data;
 	hostinfodir = sanitise_filename(dp->host->hostname);
 	diskdir     = sanitise_filename(dp->name);
-	infofile = vstralloc(infodir, "/", hostinfodir, "/", diskdir,
+	infofile = g_strjoin(NULL, infodir, "/", hostinfodir, "/", diskdir,
 			     "/info", NULL);
 	if (stat(infofile, &statbuf) == -1 && errno == ENOENT) {
 	    old_hostinfodir = old_sanitise_filename(dp->host->hostname);
 	    old_diskdir     = old_sanitise_filename(dp->name);
-	    old_infofile    = vstralloc(infodir, old_hostinfodir, "/",
+	    old_infofile    = g_strjoin(NULL, infodir, old_hostinfodir, "/",
 					old_diskdir, "/info", NULL);
 	    if (stat(old_infofile, &statbuf) == 0) {
 		other_dle_match = 0;
-		diskp = dl->head;
-		while (diskp != NULL) {
+		dlist1 = dl->head;
+		while (dlist1 != NULL) {
+		    char *Xhostinfodir;
+		    char *Xdiskdir;
+		    char *Xinfofile;
+
+		    diskp = dlist1->data;
 		    Xhostinfodir = sanitise_filename(diskp->host->hostname);
 		    Xdiskdir     = sanitise_filename(diskp->name);
-		    Xinfofile = vstralloc(infodir, "/", Xhostinfodir, "/",
+		    Xinfofile = g_strjoin(NULL, infodir, "/", Xhostinfodir, "/",
 					  Xdiskdir, "/info", NULL);
-		    if (strcmp(old_infofile, Xinfofile) == 0) {
+		    if (g_str_equal(old_infofile, Xinfofile)) {
 			other_dle_match = 1;
-			diskp = NULL;
+			dlist1 = NULL;
 		    }
 		    else {
-			diskp = diskp->next;
+			dlist1 = dlist1->next;
 		    }
+		    amfree(Xhostinfodir);
+		    amfree(Xdiskdir);
+		    amfree(Xinfofile);
 		}
 		if (other_dle_match == 0) {
 		    if(mkpdir(infofile, (mode_t)0755, (uid_t)-1,
 			      (gid_t)-1) == -1) {
-			*errmsg = vstralloc("Can't create directory for ",
+			*errmsg = g_strjoin(NULL, "Can't create directory for ",
 					    infofile, NULL);
+			amfree(hostinfodir);
+			amfree(diskdir);
+			amfree(infofile);
+			amfree(old_hostinfodir);
+			amfree(old_diskdir);
+			amfree(old_infofile);
 			return -1;
 		    }
-		    if(copy_file(infofile, old_infofile, errmsg) == -1) 
+		    if(copy_file(infofile, old_infofile, errmsg) == -1) {
+			amfree(hostinfodir);
+			amfree(diskdir);
+			amfree(infofile);
+			amfree(old_hostinfodir);
+			amfree(old_diskdir);
+			amfree(old_infofile);
 			return -1;
+		    }
 		}
 	    }
 	    amfree(old_hostinfodir);
@@ -233,6 +298,7 @@ run_server_script(
     pp_script_t  *pp_script,
     execute_on_t  execute_on,
     char         *config,
+    char         *timestamp,
     disk_t	 *dp,
     int           level)
 {
@@ -240,43 +306,43 @@ run_server_script(
     int        scriptin, scriptout, scripterr;
     char      *cmd;
     char      *command = NULL;
-    GPtrArray *argv_ptr = g_ptr_array_new();
+    GPtrArray *argv_ptr;
+    GPtrArray *errarray = NULL;
     FILE      *streamout;
     char      *line;
     char      *plugin;
     char       level_number[NUM_STR_SIZE];
     struct stat cmd_stat;
     int         result;
+    backup_support_option_t *bsu;
 
     if ((pp_script_get_execute_on(pp_script) & execute_on) == 0)
 	return;
-    if (pp_script_get_execute_where(pp_script) != ES_SERVER)
+    if (pp_script_get_execute_where(pp_script) != EXECUTE_WHERE_SERVER)
 	return;
 
     plugin = pp_script_get_plugin(pp_script);
 
-    cmd = vstralloc(APPLICATION_DIR, "/", plugin, NULL);
+    cmd = g_strjoin(NULL, APPLICATION_DIR, "/", plugin, NULL);
     result = stat(cmd, &cmd_stat);
     if (result == -1) {
 	dbprintf("Can't stat script '%s': %s\n", cmd, strerror(errno));
 	amfree(cmd);
-	cmd = vstralloc(get_config_dir(), "/application/", plugin, NULL);
+	cmd = g_strjoin(NULL, get_config_dir(), "/application/", plugin, NULL);
 	result = stat(cmd, &cmd_stat);
 	if (result == -1) {
 	    dbprintf("Can't stat script '%s': %s\n", cmd, strerror(errno));
 	    amfree(cmd);
-	    cmd = vstralloc(CONFIG_DIR, "/application/", plugin, NULL);
+	    cmd = g_strjoin(NULL, CONFIG_DIR, "/application/", plugin, NULL);
 	    result = stat(cmd, &cmd_stat);
 	    if (result == -1) {
 		dbprintf("Can't stat script '%s': %s\n", cmd, strerror(errno));
 		amfree(cmd);
-		cmd = vstralloc(APPLICATION_DIR, "/", plugin, NULL);
+		cmd = g_strjoin(NULL, APPLICATION_DIR, "/", plugin, NULL);
 	    }
 	}
     }
 
-    g_ptr_array_add(argv_ptr, stralloc(plugin));
-
     switch (execute_on) {
     case EXECUTE_ON_PRE_AMCHECK:
 	command = "PRE-AMCHECK";
@@ -339,34 +405,59 @@ run_server_script(
     case EXECUTE_ON_INTER_LEVEL_RECOVER:
 	{
 	     // ERROR these script can't be executed on server.
+	     amfree(cmd);
 	     return;
 	}
+    default:
+	amfree(cmd);
+	return;
     }
 
-    g_ptr_array_add(argv_ptr, stralloc(command));
-    g_ptr_array_add(argv_ptr, stralloc("--execute-where"));
-    g_ptr_array_add(argv_ptr, stralloc("server"));
+    bsu = backup_support_option(plugin, &errarray);
+    if (!bsu) {
+        guint  i;
+        for (i=0; i < errarray->len; i++) {
+            char *line = g_ptr_array_index(errarray, i);
+	    g_debug("Script: '%s': %s", plugin, line);
+        }
+        if (i == 0) { /* nothing in errarray */
+	    g_debug("Script: '%s': cannot execute support command", plugin);
+        }
+        g_ptr_array_free_full(errarray);
+    }
 
-    if (config) {
-	g_ptr_array_add(argv_ptr, stralloc("--config"));
-	g_ptr_array_add(argv_ptr, stralloc(config));
+    argv_ptr = g_ptr_array_new();
+    g_ptr_array_add(argv_ptr, g_strdup(plugin));
+    g_ptr_array_add(argv_ptr, g_strdup(command));
+    if (!bsu || bsu->execute_where) {
+	g_ptr_array_add(argv_ptr, g_strdup("--execute-where"));
+	g_ptr_array_add(argv_ptr, g_strdup("server"));
     }
-    if (dp->host->hostname) {
-	g_ptr_array_add(argv_ptr, stralloc("--host"));
-	g_ptr_array_add(argv_ptr, stralloc(dp->host->hostname));
+
+    if (config && (!bsu || bsu->config)) {
+	g_ptr_array_add(argv_ptr, g_strdup("--config"));
+	g_ptr_array_add(argv_ptr, g_strdup(config));
     }
-    if (dp->name) {
-	g_ptr_array_add(argv_ptr, stralloc("--disk"));
-	g_ptr_array_add(argv_ptr, stralloc(dp->name));
+    if (timestamp && (!bsu || bsu->timestamp)) {
+	g_ptr_array_add(argv_ptr, g_strdup("--timestamp"));
+	g_ptr_array_add(argv_ptr, g_strdup(timestamp));
+    }
+    if (dp->host->hostname && (!bsu || bsu->host)) {
+	g_ptr_array_add(argv_ptr, g_strdup("--host"));
+	g_ptr_array_add(argv_ptr, g_strdup(dp->host->hostname));
+    }
+    if (dp->name && (!bsu || bsu->disk)) {
+	g_ptr_array_add(argv_ptr, g_strdup("--disk"));
+	g_ptr_array_add(argv_ptr, g_strdup(dp->name));
     }
     if (dp->device) {
-	g_ptr_array_add(argv_ptr, stralloc("--device"));
-	g_ptr_array_add(argv_ptr, stralloc(dp->device));
+	g_ptr_array_add(argv_ptr, g_strdup("--device"));
+	g_ptr_array_add(argv_ptr, g_strdup(dp->device));
     }
     if (level >= 0) {
-	g_snprintf(level_number, SIZEOF(level_number), "%d", level);
-	g_ptr_array_add(argv_ptr, stralloc("--level"));
-	g_ptr_array_add(argv_ptr, stralloc(level_number));
+	g_snprintf(level_number, sizeof(level_number), "%d", level);
+	g_ptr_array_add(argv_ptr, g_strdup("--level"));
+	g_ptr_array_add(argv_ptr, g_strdup(level_number));
     }
 
     property_add_to_argv(argv_ptr, pp_script_get_property(pp_script));
@@ -384,11 +475,12 @@ run_server_script(
 	    dbprintf("script: %s\n", line);
 	    amfree(line);
 	}
+	fclose(streamout);
     }
-    fclose(streamout);
     waitpid(scriptpid, NULL, 0);
     g_ptr_array_free_full(argv_ptr);
     amfree(cmd);
+    g_free(bsu);
 }
 
 
@@ -396,6 +488,7 @@ void
 run_server_dle_scripts(
     execute_on_t  execute_on,
     char         *config,
+    char         *timestamp,
     disk_t	 *dp,
     int           level)
 {
@@ -405,7 +498,7 @@ run_server_dle_scripts(
 	 pp_scriptlist = pp_scriptlist->next) {
 	pp_script_t *pp_script = lookup_pp_script((char *)pp_scriptlist->data);
 	g_assert(pp_script != NULL);
-	run_server_script(pp_script, execute_on, config, dp, level);
+	run_server_script(pp_script, execute_on, config, timestamp, dp, level);
     }
 }
 
@@ -413,6 +506,7 @@ void
 run_server_host_scripts(
     execute_on_t  execute_on,
     char         *config,
+    char         *timestamp,
     am_host_t	 *hostp)
 {
     identlist_t pp_scriptlist;
@@ -433,7 +527,8 @@ run_server_host_scripts(
 			   == NULL;
 		}
 		if (todo) {
-		    run_server_script(pp_script, execute_on, config, dp, -1);
+		    run_server_script(pp_script, execute_on, config, timestamp,
+				      dp, -1);
 		    if (pp_script_get_single_execution(pp_script)) {
 			g_hash_table_insert(executed,
 					    pp_script_get_plugin(pp_script),
@@ -450,7 +545,8 @@ run_server_host_scripts(
 void
 run_server_global_scripts(
     execute_on_t  execute_on,
-    char         *config)
+    char         *config,
+    char         *timestamp)
 {
     identlist_t  pp_scriptlist;
     disk_t      *dp;
@@ -473,7 +569,7 @@ run_server_global_scripts(
 		    }
 		    if (todo) {
 			run_server_script(pp_script, execute_on, config,
-					  dp, -1);
+					  timestamp, dp, -1);
 			if (pp_script_get_single_execution(pp_script)) {
 			    g_hash_table_insert(executed,
 					pp_script_get_plugin(pp_script),
@@ -491,21 +587,24 @@ void
 run_amcleanup(
     char *config_name)
 {
-    pid_t amcleanup_pid;
-    char *amcleanup_program;
-    char *amcleanup_options[4];
+    pid_t  amcleanup_pid;
+    char  *amcleanup_program;
+    char  *amcleanup_options[4];
+    char **env;
 
     switch(amcleanup_pid = fork()) {
 	case -1:
 	    return;
 	    break;
 	case  0: /* child process */
-	    amcleanup_program = vstralloc(sbindir, "/", "amcleanup", NULL);
+	    amcleanup_program = g_strjoin(NULL, sbindir, "/", "amcleanup", NULL);
 	    amcleanup_options[0] = amcleanup_program;
 	    amcleanup_options[1] = "-p";
 	    amcleanup_options[2] = config_name;
 	    amcleanup_options[3] = NULL;
-	    execve(amcleanup_program, amcleanup_options, safe_env());
+	    env = safe_env();
+	    execve(amcleanup_program, amcleanup_options, env);
+	    free_env(env);
 	    error("exec %s: %s", amcleanup_program, strerror(errno));
 	    /*NOTREACHED*/
 	default:
@@ -525,9 +624,9 @@ get_master_process(
 
     log = fopen(logfile, "r");
     if (!log)
-	return stralloc("UNKNOWN");
+	return g_strdup("UNKNOWN");
 
-    while(fgets(line, 1024, log)) {
+    while (untaint_fgets(line, 1024, log)) {
 	if (strncmp_const(line, "INFO ") == 0) {
 	    s = line+5;
 	    ch = *s++;
@@ -539,14 +638,14 @@ get_master_process(
 	    s[-1] = '\0';
 	    skip_whitespace(s, ch);
 	    if (strncmp_const(s-1, "pid ") == 0) {
-		process_name = stralloc(process_name);
+		process_name = g_strdup(process_name);
 		fclose(log);
 		return process_name;
 	    }
 	}
     }
     fclose(log);
-    return stralloc("UNKNOWN");
+    return g_strdup("UNKNOWN");
 }
 
 
@@ -555,7 +654,8 @@ internal_server_estimate(
     disk_t *dp,
     info_t *info,
     int     level,
-    int    *stats)
+    int    *stats,
+    tapetype_t *tapetype)
 {
     int    j;
     gint64 size = 0;
@@ -580,11 +680,9 @@ internal_server_estimate(
 	    size = info->inf[level].size;
 	    *stats = 1;
 	} else {
-	    char *conf_tapetype = getconf_str(CNF_TAPETYPE);
-	    tapetype_t *tape = lookup_tapetype(conf_tapetype);
 	    size = (gint64)1000000;
-	    if (size > tapetype_get_length(tape)/2)
-		size = tapetype_get_length(tape)/2;
+	    if (size > tapetype_get_length(tapetype)/2)
+		size = tapetype_get_length(tapetype)/2;
 	    *stats = 0;
 	}
     } else if (level == info->last_level) {
@@ -626,13 +724,12 @@ internal_server_estimate(
 	else {
 	    int level0_stat;
 	    gint64 level0_size;
-	    char *conf_tapetype = getconf_str(CNF_TAPETYPE);
-	    tapetype_t *tape = lookup_tapetype(conf_tapetype);
 
-            level0_size = internal_server_estimate(dp, info, 0, &level0_stat);
+            level0_size = internal_server_estimate(dp, info, 0, &level0_stat,
+						   tapetype);
 	    size = (gint64)10000;
-	    if (size > tapetype_get_length(tape)/2)
-		size = tapetype_get_length(tape)/2;
+	    if (size > tapetype_get_length(tapetype)/2)
+		size = tapetype_get_length(tapetype)/2;
 	    if (level0_size > 0 && dp->strategy != DS_NOFULL) {
 		if (size > level0_size/2)
 		    size = level0_size/2;
@@ -662,13 +759,12 @@ internal_server_estimate(
 	} else {
 	    int level0_stat;
 	    gint64 level0_size;
-	    char *conf_tapetype = getconf_str(CNF_TAPETYPE);
-	    tapetype_t *tape = lookup_tapetype(conf_tapetype);
 
-            level0_size = internal_server_estimate(dp, info, 0, &level0_stat);
+            level0_size = internal_server_estimate(dp, info, 0, &level0_stat,
+						   tapetype);
 	    size = (gint64)100000;
-	    if (size > tapetype_get_length(tape)/2)
-		size = tapetype_get_length(tape)/2;
+	    if (size > tapetype_get_length(tapetype)/2)
+		size = tapetype_get_length(tapetype)/2;
 	    if (level0_size > 0 && dp->strategy != DS_NOFULL) {
 		if (size > level0_size/2)
 		    size = level0_size/2;
@@ -676,11 +772,9 @@ internal_server_estimate(
 	    *stats = 0;
 	}
     } else {
-	char *conf_tapetype = getconf_str(CNF_TAPETYPE);
-	tapetype_t *tape = lookup_tapetype(conf_tapetype);
 	size = (gint64)100000;
-	if (size > tapetype_get_length(tape)/2)
-	    size = tapetype_get_length(tape)/2;
+	if (size > tapetype_get_length(tapetype)/2)
+	    size = tapetype_get_length(tapetype)/2;
     }
 
     return size;
@@ -690,11 +784,12 @@ int
 server_can_do_estimate(
     disk_t *dp,
     info_t *info,
-    int     level)
+    int     level,
+    tapetype_t *tapetype)
 {
     int     stats;
 
-    internal_server_estimate(dp, info, level, &stats);
+    internal_server_estimate(dp, info, level, &stats, tapetype);
     return stats;
 }
 
