@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2009-2013 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
+# Contact information: Carbonite Inc., 756 N Pastoria Ave
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
 use lib '@amperldir@';
@@ -31,15 +31,18 @@ use Amanda::Debug qw( debug );
 use Amanda::Taper::Controller;
 use Getopt::Long;
 
-Amanda::Util::setup_application("taper", "server", $CONTEXT_DAEMON);
+Amanda::Util::setup_application("taper", "server", $CONTEXT_DAEMON, "amanda", "amanda");
 
 my $config_overrides = new_config_overrides($#ARGV+1);
+my $opt_storage_name;
 
 debug("Arguments: " . join(' ', @ARGV));
 Getopt::Long::Configure(qw{bundling});
 GetOptions(
     'version' => \&Amanda::Util::version_opt,
     'o=s' => sub { add_config_override_opt($config_overrides, $_[1]); },
+    'storage-name=s' => \$opt_storage_name,
+    'log-filename=s' => sub { Amanda::Logfile::set_logname($_[1]); },
 ) or usage();
 
 if (@ARGV != 1) {
@@ -47,7 +50,7 @@ if (@ARGV != 1) {
 }
 
 set_config_overrides($config_overrides);
-config_init($CONFIG_INIT_EXPLICIT_NAME, $ARGV[0]);
+config_init_with_global($CONFIG_INIT_EXPLICIT_NAME, $ARGV[0]);
 my ($cfgerr_level, @cfgerr_errors) = config_errors();
 if ($cfgerr_level >= $CFGERR_WARNINGS) {
     config_print_errors();
@@ -68,9 +71,14 @@ Amanda::Debug::add_amanda_log_handler($amanda_log_trace_log);
 Amanda::Util::finish_setup($RUNNING_AS_DUMPUSER);
 
 my $tlf = Amanda::Config::config_dir_relative(getconf($CNF_TAPELIST));
-my $tl = Amanda::Tapelist->new($tlf);
+my ($tl, $message) = Amanda::Tapelist->new($tlf);
+if (defined $message) {
+    die "Could not read the tapelist: $message";
+}
 # transfer control to the Amanda::Taper::Controller class implemented above
-my $controller = Amanda::Taper::Controller->new(tapelist => $tl);
+my $controller = Amanda::Taper::Controller->new(
+					storage_name => $opt_storage_name,
+					tapelist => $tl);
 $controller->start();
 Amanda::MainLoop::run();
 
