@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,7 +18,7 @@
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
  * Contact information: Zmanda Inc., 465 N Mathlida Ave, Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -119,6 +120,41 @@ push_buffer_impl(
     amfree(buf);
 }
 
+static void
+push_buffer_static_impl(
+    XferElement *elt,
+    gpointer buf,
+    size_t len)
+{
+    XferDestBuffer *self = (XferDestBuffer *)elt;
+
+    if (!buf || len == 0)
+	return;
+
+    /* make sure this isn't too much data */
+    if (self->max_size && self->len + len > self->max_size) {
+	xfer_cancel_with_error(elt,
+	    _("illegal attempt to transfer more than %zd bytes"), self->max_size);
+	wait_until_xfer_cancelled(elt->xfer);
+	return;
+    }
+
+    /* expand the buffer if necessary */
+    if (self->len + len > self->allocated) {
+	gsize new_size = self->allocated * 2;
+	if (new_size < self->len+len)
+	    new_size = self->len+len;
+	if (self->max_size && new_size > self->max_size)
+	    new_size = self->max_size;
+
+	self->buf = g_realloc(self->buf, new_size);
+	self->allocated = new_size;
+    }
+
+    g_memmove(((guint8 *)self->buf)+self->len, buf, len);
+    self->len += len;
+}
+
 static void
 finalize_impl(
     GObject * obj_self)
@@ -140,12 +176,14 @@ class_init(
     XferElementClass *klass = XFER_ELEMENT_CLASS(selfc);
     GObjectClass *goc = G_OBJECT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0) },
-	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0) },
+	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
+	{ XFER_MECH_PUSH_BUFFER_STATIC, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
     };
 
     selfc->get = get_impl;
     klass->push_buffer = push_buffer_impl;
+    klass->push_buffer_static = push_buffer_static_impl;
     goc->finalize = finalize_impl;
 
     klass->perl_class = "Amanda::Xfer::Dest::Buffer";
@@ -159,7 +197,7 @@ xfer_dest_buffer_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferDestBufferClass),
             (GBaseInitFunc) NULL,
