@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -75,8 +76,15 @@ push_buffer_impl(
 {
     XferDestNull *self = (XferDestNull *)elt;
 
-    if (!buf)
+    if (buf) {
+	crc32_add(buf, len, &elt->crc);
+    } else {
+	XMsg *msg = xmsg_new((XferElement *)self, XMSG_CRC, 0);
+	msg->crc = crc32_finish(&elt->crc);
+	msg->size = elt->crc.size;
+	xfer_queue_message(XFER_ELEMENT(self)->xfer, msg);
 	return;
+    }
 
     if (self->do_verify && !elt->cancelled) {
 	if (!simpleprng_verify_buffer(&self->prng, buf, len)) {
@@ -93,7 +101,7 @@ push_buffer_impl(
 	/* send a superfluous message (this is a testing XferElement,
 	 * after all) */
 	XMsg *msg = xmsg_new((XferElement *)self, XMSG_INFO, 0);
-	msg->message = stralloc("Is this thing on?");
+	msg->message = g_strdup("Is this thing on?");
 	xfer_queue_message(XFER_ELEMENT(self)->xfer, msg);
 	self->sent_info = TRUE;
     }
@@ -101,17 +109,57 @@ push_buffer_impl(
     amfree(buf);
 }
 
+static void
+push_buffer_static_impl(
+    XferElement *elt,
+    gpointer buf,
+    size_t len)
+{
+    XferDestNull *self = (XferDestNull *)elt;
+
+    if (buf && len != 0) {
+	crc32_add(buf, len, &elt->crc);
+    } else {
+	XMsg *msg = xmsg_new((XferElement *)self, XMSG_CRC, 0);
+	msg->crc = crc32_finish(&elt->crc);
+	msg->size = elt->crc.size;
+	xfer_queue_message(XFER_ELEMENT(self)->xfer, msg);
+	return;
+    }
+
+    if (self->do_verify && !elt->cancelled) {
+	if (!simpleprng_verify_buffer(&self->prng, buf, len)) {
+	    xfer_cancel_with_error(elt,
+		"verification of incoming bytestream failed; see stderr for details"),
+	    wait_until_xfer_cancelled(elt->xfer);
+	    return;
+	}
+    }
+
+    self->byte_position += len;
+    if (!self->sent_info) {
+	/* send a superfluous message (this is a testing XferElement,
+	 * after all) */
+	XMsg *msg = xmsg_new((XferElement *)self, XMSG_INFO, 0);
+	msg->message = g_strdup("Is this thing on?");
+	xfer_queue_message(XFER_ELEMENT(self)->xfer, msg);
+	self->sent_info = TRUE;
+    }
+}
+
 static void
 class_init(
     XferDestNullClass * selfc)
 {
     XferElementClass *klass = XFER_ELEMENT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0) },
-	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0) },
+	{ XFER_MECH_PUSH_BUFFER, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
+	{ XFER_MECH_PUSH_BUFFER_STATIC, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
     };
 
     klass->push_buffer = push_buffer_impl;
+    klass->push_buffer_static = push_buffer_static_impl;
 
     klass->perl_class = "Amanda::Xfer::Dest::Null";
     klass->mech_pairs = mech_pairs;
@@ -124,7 +172,7 @@ xfer_dest_null_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferDestNullClass),
             (GBaseInitFunc) NULL,
@@ -158,6 +206,7 @@ xfer_dest_null(
     } else {
 	self->do_verify = FALSE;
     }
+    crc32_init(&elt->crc);
 
     return elt;
 }
