@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -100,6 +101,33 @@ pull_buffer_impl(
     return buf;
 }
 
+static gpointer
+pull_buffer_static_impl(
+    XferElement *elt,
+    gpointer buf,
+    size_t block_size,
+    size_t *size)
+{
+    XferSourceRandom *self = (XferSourceRandom *)elt;
+
+    if (elt->cancelled || (self->limited_length && self->length == 0)) {
+	*size = 0;
+	return NULL;
+    }
+
+    if (self->limited_length) {
+        *size = MIN(10240, self->length);
+        *size = MIN(*size, block_size);
+        self->length -= *size;
+    } else {
+	*size = 10240;
+    }
+
+    simpleprng_fill_buffer(&self->prng, buf, *size);
+
+    return buf;
+}
+
 static void
 instance_init(
     XferElement *elt)
@@ -113,12 +141,14 @@ class_init(
 {
     XferElementClass *klass = XFER_ELEMENT_CLASS(selfc);
     static xfer_element_mech_pair_t mech_pairs[] = {
-	{ XFER_MECH_NONE, XFER_MECH_PULL_BUFFER, XFER_NROPS(1), XFER_NTHREADS(0) },
-	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0) },
+	{ XFER_MECH_NONE, XFER_MECH_PULL_BUFFER, XFER_NROPS(1), XFER_NTHREADS(0), XFER_NALLOC(1) },
+	{ XFER_MECH_NONE, XFER_MECH_PULL_BUFFER_STATIC, XFER_NROPS(1), XFER_NTHREADS(0), XFER_NALLOC(0) },
+	{ XFER_MECH_NONE, XFER_MECH_NONE, XFER_NROPS(0), XFER_NTHREADS(0), XFER_NALLOC(0) },
     };
 
     selfc->get_seed = get_seed_impl;
     klass->pull_buffer = pull_buffer_impl;
+    klass->pull_buffer_static = pull_buffer_static_impl;
 
     klass->perl_class = "Amanda::Xfer::Source::Random";
     klass->mech_pairs = mech_pairs;
@@ -131,7 +161,7 @@ xfer_source_random_get_type (void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferSourceRandomClass),
             (GBaseInitFunc) NULL,
