@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -43,6 +44,8 @@ xfer_element_init(
     xe->_input_fd = xe->_output_fd = -1;
     xe->repr = NULL;
     xe->must_drain = FALSE;
+    xe->cancel_on_success = FALSE;
+    xe->ignore_broken_pipe = FALSE;
 }
 
 static gboolean
@@ -53,15 +56,55 @@ xfer_element_setup_impl(
 }
 
 static gboolean
-xfer_element_set_size_impl(
-    XferElement *elt G_GNUC_UNUSED,
-    gint64       size G_GNUC_UNUSED)
+xfer_element_set_offset_impl(
+    XferElement *elt,
+    gint64       offset)
 {
+
+    elt->offset = offset;
+
+    return TRUE; /* success */
+}
+
+static gboolean
+xfer_element_set_size_impl(
+    XferElement *elt,
+    gint64       size)
+{
+    elt->orig_size = size;
     elt->size = size;
 
     return TRUE; /* success */
 }
 
+static off_t
+xfer_element_get_offset_impl(
+    XferElement *elt)
+{
+    return elt->offset;
+}
+
+static off_t
+xfer_element_get_orig_size_impl(
+    XferElement *elt)
+{
+    return elt->orig_size;
+}
+
+static off_t
+xfer_element_get_size_impl(
+    XferElement *elt)
+{
+    return elt->size;
+}
+
+static size_t
+xfer_element_get_block_size_impl(
+    XferElement *elt)
+{
+    return elt->block_size;
+}
+
 static gboolean
 xfer_element_start_impl(
     XferElement *elt G_GNUC_UNUSED)
@@ -84,6 +127,18 @@ xfer_element_pull_buffer_impl(
     XferElement *elt G_GNUC_UNUSED,
     size_t *size G_GNUC_UNUSED)
 {
+    *size = 0;
+    return NULL;
+}
+
+static gpointer
+xfer_element_pull_buffer_static_impl(
+    XferElement *elt G_GNUC_UNUSED,
+    gpointer buf G_GNUC_UNUSED,
+    size_t block_size G_GNUC_UNUSED,
+    size_t *size G_GNUC_UNUSED)
+{
+    *size = 0;
     return NULL;
 }
 
@@ -95,6 +150,14 @@ xfer_element_push_buffer_impl(
 {
 }
 
+static void
+xfer_element_push_buffer_static_impl(
+    XferElement *elt G_GNUC_UNUSED,
+    gpointer buf G_GNUC_UNUSED,
+    size_t size G_GNUC_UNUSED)
+{
+}
+
 static xfer_element_mech_pair_t *
 xfer_element_get_mech_pairs_impl(
     XferElement *elt)
@@ -107,9 +170,9 @@ xfer_element_repr_impl(
     XferElement *elt)
 {
     if (!elt->repr) {
-	elt->repr = newvstrallocf(elt->repr, "<%s@%p>",
-		G_OBJECT_TYPE_NAME(G_OBJECT(elt)),
-		elt);
+        g_free(elt->repr);
+        elt->repr = g_strdup_printf("<%s@%p>",
+                                    G_OBJECT_TYPE_NAME(G_OBJECT(elt)), elt);
     }
 
     return elt->repr;
@@ -146,11 +209,18 @@ xfer_element_class_init(
 
     klass->repr = xfer_element_repr_impl;
     klass->setup = xfer_element_setup_impl;
+    klass->set_offset = xfer_element_set_offset_impl;
     klass->set_size = xfer_element_set_size_impl;
+    klass->get_offset = xfer_element_get_offset_impl;
+    klass->get_orig_size = xfer_element_get_orig_size_impl;
+    klass->get_size = xfer_element_get_size_impl;
+    klass->get_block_size = xfer_element_get_block_size_impl;
     klass->start = xfer_element_start_impl;
     klass->cancel = xfer_element_cancel_impl;
     klass->pull_buffer = xfer_element_pull_buffer_impl;
+    klass->pull_buffer_static = xfer_element_pull_buffer_static_impl;
     klass->push_buffer = xfer_element_push_buffer_impl;
+    klass->push_buffer_static = xfer_element_push_buffer_static_impl;
     klass->get_mech_pairs = xfer_element_get_mech_pairs_impl;
 
     goc->finalize = xfer_element_finalize;
@@ -166,7 +236,7 @@ xfer_element_get_type(void)
 {
     static GType type = 0;
 
-    if G_UNLIKELY(type == 0) {
+    if (G_UNLIKELY(type == 0)) {
         static const GTypeInfo info = {
             sizeof (XferElementClass),
             (GBaseInitFunc) NULL,
@@ -212,6 +282,14 @@ xfer_element_setup(
     return XFER_ELEMENT_GET_CLASS(elt)->setup(elt);
 }
 
+gboolean
+xfer_element_set_offset(
+    XferElement *elt,
+    gint64       offset)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->set_offset(elt, offset);
+}
+
 gboolean
 xfer_element_set_size(
     XferElement *elt,
@@ -220,6 +298,34 @@ xfer_element_set_size(
     return XFER_ELEMENT_GET_CLASS(elt)->set_size(elt, size);
 }
 
+off_t
+xfer_element_get_offset(
+    XferElement *elt)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->get_offset(elt);
+}
+
+off_t
+xfer_element_get_orig_size(
+    XferElement *elt)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->get_orig_size(elt);
+}
+
+off_t
+xfer_element_get_size(
+    XferElement *elt)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->get_size(elt);
+}
+
+size_t
+xfer_element_get_block_size(
+    XferElement *elt)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->get_block_size(elt);
+}
+
 gboolean
 xfer_element_start(
     XferElement *elt)
@@ -254,6 +360,27 @@ xfer_element_pull_buffer(
     return XFER_ELEMENT_GET_CLASS(elt)->pull_buffer(elt, size);
 }
 
+gpointer
+xfer_element_pull_buffer_static(
+    XferElement *elt,
+    gpointer buf,
+    size_t block_size,
+    size_t *size)
+{
+    xfer_status status;
+    /* Make sure that the xfer is running before calling upstream's
+     * pull_bufferi_static method; this avoids a race condition where upstream
+     * hasn't finished its xfer_element_start yet, and isn't ready for
+     * a pull */
+    g_mutex_lock(elt->xfer->status_mutex);
+    status = elt->xfer->status;
+    g_mutex_unlock(elt->xfer->status_mutex);
+    if (status == XFER_START)
+	wait_until_xfer_running(elt->xfer);
+
+    return XFER_ELEMENT_GET_CLASS(elt)->pull_buffer_static(elt, buf, block_size, size);
+}
+
 void
 xfer_element_push_buffer(
     XferElement *elt,
@@ -265,6 +392,17 @@ xfer_element_push_buffer(
     XFER_ELEMENT_GET_CLASS(elt)->push_buffer(elt, buf, size);
 }
 
+void
+xfer_element_push_buffer_static(
+    XferElement *elt,
+    gpointer buf,
+    size_t size)
+{
+    /* There is no race condition with push_buffer, because downstream
+     * elements are started first. */
+    XFER_ELEMENT_GET_CLASS(elt)->push_buffer_static(elt, buf, size);
+}
+
 xfer_element_mech_pair_t *
 xfer_element_get_mech_pairs(
 	XferElement *elt)
@@ -296,9 +434,29 @@ xfer_element_drain_fd(
     char buf[1024];
 
     while (1) {
-	len = full_read(fd, buf, sizeof(buf));
+	len = read_fully(fd, buf, sizeof(buf), NULL);
 	if (len < sizeof(buf))
 	    return;
     }
 }
 
+mem_ring_t *
+xfer_element_get_mem_ring(
+    XferElement *elt)
+{
+    return XFER_ELEMENT_GET_CLASS(elt)->get_mem_ring(elt);
+}
+
+shm_ring_t *
+xfer_element_get_shm_ring(
+    XferElement *elt)
+{
+    if (elt->shm_ring) {
+	return elt->shm_ring;
+    } else if (elt->downstream) {
+	return xfer_element_get_shm_ring(elt->downstream);
+    } else {
+	return NULL;
+    }
+}
+
