@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 /* Base classes for transfer elements.
@@ -28,9 +29,9 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include "xfer.h"
 #include "amanda.h"
-#include "util.h"
+#include "amutil.h"
+#include "xfer.h"
 #include "directtcp.h"
 
 typedef enum {
@@ -52,10 +53,18 @@ typedef enum {
      * pull a buffer.  EOF is indicated by returning a NULL buffer */
     XFER_MECH_PULL_BUFFER,
 
+    /* downstream element will call elt->upstream->pull_buffer_static() to
+     * pull a buffer.  EOF is indicated by returning a NULL buffer */
+    XFER_MECH_PULL_BUFFER_STATIC,
+
     /* upstream element will call elt->downstream->push_buffer(buf) to push
      * a buffer.  EOF is indicated by passing a NULL buffer. */
     XFER_MECH_PUSH_BUFFER,
 
+    /* upstream element will call elt->downstream->push_buffer_static(buf) to push
+     * a buffer.  EOF is indicated by passing a NULL buffer. */
+    XFER_MECH_PUSH_BUFFER_STATIC,
+
     /* DirectTCP: downstream sends an array of IP:PORT addresses to which a TCP
      * connection should be made, then upstream connects to one of the addreses
      * and sends the data over that connection */
@@ -66,6 +75,12 @@ typedef enum {
      * receives the data over that connection */
     XFER_MECH_DIRECTTCP_CONNECT,
 
+    /* MemRing: Use a memory ring between element */
+    XFER_MECH_MEM_RING,
+
+    /* MemRing: Use a shared memory ring between element */
+    XFER_MECH_SHM_RING,
+
     /* (sentinel value) */
     XFER_MECH_MAX,
 } xfer_mech;
@@ -82,12 +97,14 @@ typedef enum {
 
 #define XFER_NROPS(x) (x)
 #define XFER_NTHREADS(x) (x)
+#define XFER_NALLOC(x) (x)
 
 typedef struct {
     xfer_mech input_mech;
     xfer_mech output_mech;
     guint8 ops_per_byte;	/* number of byte copies or other operations */
     guint8 nthreads;		/* number of additional threads created */
+    guint8 nalloc;		/* number of alloc for each block */
 } xfer_element_mech_pair_t;
 
 /***********************
@@ -152,8 +169,16 @@ typedef struct XferElement {
     char *repr;
 
     /* maximum size to transfer */
+    gint64 offset;
+    gint64 orig_size;
     gint64 size;
 
+    /* required block_size, 0 for no requirement */
+    size_t block_size;
+
+    /* shm_ring */
+    shm_ring_t *shm_ring;
+
     /* for crc computation */
     crc_t crc;
 
@@ -210,7 +235,17 @@ typedef struct {
      * @param size: the size of data to transfer
      * @return: TRUE
      */
+    gboolean (*set_offset)(XferElement *elt, gint64 offset);
     gboolean (*set_size)(XferElement *elt, gint64 size);
+    off_t (*get_offset)(XferElement *elt);
+    off_t (*get_orig_size)(XferElement *elt);
+    off_t (*get_size)(XferElement *elt);
+
+    /* the the required block_size for pull_buffer_static
+     */
+    size_t (*get_block_size)(XferElement *elt);
+
+    mem_ring_t *(*get_mem_ring)(XferElement *elt);
 
     /* Start transferring data.  The element downstream of this one will
      * already be started, while the upstream element will not, so data will
@@ -263,8 +298,8 @@ typedef struct {
 
     /* Get a buffer full of data from this element.  This function is called by
      * the downstream element under XFER_MECH_PULL_CALL.  It can block indefinitely,
-     * and must only return NULL on EOF.  Responsibility to free the buffer transfers
-     * to the caller.
+     * and must only return NULL on EOF. This function must allocate the buffer.
+     * Responsibility to free the buffer transfers to the caller.
      *
      * @param elt: the XferElement
      * @param size (output): size of resulting buffer
@@ -272,6 +307,17 @@ typedef struct {
      */
     gpointer (*pull_buffer)(XferElement *elt, size_t *size);
 
+    /* Get a buffer full of data from this element.  This function is called by
+     * the downstream element under XFER_MECH_PULL_STATIC_CALL.  It can block indefinitely,
+     * and must only return NULL on EOF. This function must use the pre-allocated buffer.
+     *
+     * @param elt: the XferElement
+     * @param buf (input-output): Where to put the buffer
+     * @param block_size (input): size of allocated buffer
+     * @param size (output): size of resulting buffer
+     */
+    gpointer (*pull_buffer_static)(XferElement *elt, gpointer buf, size_t block_size, size_t *size);
+
     /* A buffer full of data is being sent to this element for processing; this
      * function is called by the upstream element under XFER_MECH_PUSH_CALL.
      * It can block indefinitely if the data cannot be processed immediately.
@@ -284,6 +330,18 @@ typedef struct {
      */
     void (*push_buffer)(XferElement *elt, gpointer buf, size_t size);
 
+    /* Same as push_buffer, but the callee must not free the buffer
+     * A buffer full of data is being sent to this element for processing; this
+     * function is called by the upstream element under XFER_MECH_PUSH_CALL.
+     * It can block indefinitely if the data cannot be processed immediately.
+     * An EOF condition is signaled by call with a NULL buffer.
+     *
+     * @param elt: the XferElement
+     * @param buf: buffer
+     * @param size: size of buffer
+     */
+    void (*push_buffer_static)(XferElement *elt, gpointer buf, size_t size);
+
     /* Returns the mech_pairs that this element supports.  The default
      * implementation just returns the class attribute 'mech_pairs', but
      * subclasses can dynamically select the available mechanisms by overriding
@@ -322,12 +380,21 @@ void xfer_element_unref(XferElement *elt);
 gboolean xfer_element_link_to(XferElement *elt, XferElement *successor);
 char *xfer_element_repr(XferElement *elt);
 gboolean xfer_element_setup(XferElement *elt);
+gboolean xfer_element_set_offset(XferElement *elt, gint64 offset);
 gboolean xfer_element_set_size(XferElement *elt, gint64 size);
+off_t xfer_element_get_offset(XferElement *elt);
+off_t xfer_element_get_orig_size(XferElement *elt);
+off_t xfer_element_get_size(XferElement *elt);
+size_t xfer_element_get_block_size(XferElement *elt);
 gboolean xfer_element_start(XferElement *elt);
 void xfer_element_push_buffer(XferElement *elt, gpointer buf, size_t size);
+void xfer_element_push_buffer_static(XferElement *elt, gpointer buf, size_t size);
 gpointer xfer_element_pull_buffer(XferElement *elt, size_t *size);
+gpointer xfer_element_pull_buffer_static(XferElement *elt, gpointer buf, size_t block_size, size_t *size);
 gboolean xfer_element_cancel(XferElement *elt, gboolean expect_eof);
 xfer_element_mech_pair_t *xfer_element_get_mech_pairs(XferElement *elt);
+mem_ring_t *xfer_element_get_mem_ring(XferElement *elt);
+shm_ring_t *xfer_element_get_shm_ring(XferElement *elt);
 
 /****
  * Subclass utilities
@@ -414,6 +481,17 @@ XferElement *xfer_source_pattern(guint64 length, void * pattern,
 XferElement * xfer_source_fd(
     int fd);
 
+XferElement *xfer_source_file(
+     char *filename);
+
+/* A transfer source that get the bytes from a shm-ring
+ *
+ * Implemented in source-shm-ring.c
+ *
+ * @return: new element
+ */
+XferElement * xfer_source_shm_ring(void);
+
 /* A transfer source that exposes its listening DirectTCPAddrs (via
  * elt->input_listen_addrs) for external use
  *
@@ -433,7 +511,7 @@ XferElement * xfer_source_directtcp_listen(void);
  */
 XferElement * xfer_source_directtcp_connect(DirectTCPAddr *addrs);
 
-/* A transfer filter that copy its input to its output but contimue to read
+/* A transfer filter that copy its input to its output but continue to read
  *  the input if it get a Broken pipe to the output.
  *
  * Implemented in filter-drain.c
@@ -460,6 +538,10 @@ XferElement *xfer_filter_process(gchar **argv,
     gboolean cancel_on_success,
     gboolean ignore_broken_pipe);
 
+/* return the stderr of the filter */
+
+int filter_process_get_err_fd(XferElement *elt);
+
 /* A transfer filter that just applies a bytewise XOR transformation to the data
  * that passes through it.
  *
@@ -471,6 +553,15 @@ XferElement *xfer_filter_process(gchar **argv,
 XferElement *xfer_filter_xor(
     unsigned char xor_key);
 
+/* A transfer filter that compute the crc32 of to the data
+ * that passes through it.
+ *
+ * Implemented in filter-crc.c
+ *
+ * @return: new element
+ */
+XferElement *xfer_filter_crc(void);
+
 /* A transfer destination that consumes all bytes it is given, optionally
  * validating that they match those produced by source_random
  *
@@ -541,4 +632,27 @@ XferElement * xfer_dest_directtcp_listen(void);
 
 int get_err_fd(XferElement *elt);
 
+/* A transfer dest that execute an application,
+ * It can be used for restore, index and validate
+ *
+ * Implemented in dest-application.c
+ *
+ * @return: new element
+ */
+XferElement * xfer_dest_application(
+    gchar **argv,
+    gboolean need_root,
+    gboolean must_drain,
+    gboolean cancel_on_success,
+    gboolean ignore_broken_pipe);
+
+/* return the stderr of the application */
+int dest_application_get_err_fd(XferElement *elt);
+
+/* return the stdout of the application */
+int dest_application_get_out_fd(XferElement *elt);
+
+/* return the dar fd of the application */
+int dest_application_get_dar_fd(XferElement *elt);
+
 #endif
