@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
 #include "amxfer.h"
 #include "element-glue.h"
-#include "arglist.h"
 
 /* XMsgSource objects are GSource "subclasses" which manage
  * a queue of messages, delivering those messages via callback
@@ -112,6 +112,7 @@ xfer_unref(
     while ((msg = (XMsg *)g_async_queue_try_pop(xfer->queue))) {
 	g_warning("Dropping XMsg from %s because the XMsgSource is being destroyed", 
 	    xfer_element_repr(msg->elt));
+	g_debug("MSG: %s", xmsg_repr(msg));
 	xmsg_free(msg);
     }
     g_async_queue_unref(xfer->queue);
@@ -152,6 +153,7 @@ xfer_queue_message(
     g_assert(xfer != NULL);
     g_assert(msg != NULL);
 
+    g_debug("xfer_queue_message: MSG: %s", xmsg_repr(msg));
     g_async_queue_push(xfer->queue, (gpointer)msg);
 
     /* TODO: don't do this if we're in the main thread */
@@ -162,16 +164,21 @@ char *
 xfer_repr(
     Xfer *xfer)
 {
+    char *tmpbuf;
     unsigned int i;
 
     if (!xfer->repr) {
-	xfer->repr = newvstrallocf(xfer->repr, "<Xfer@%p (", xfer);
+	g_free(xfer->repr);
+	xfer->repr = g_strdup_printf("<Xfer@%p (", xfer);
 	for (i = 0; i < xfer->elements->len; i++) {
 	    XferElement *elt = (XferElement *)g_ptr_array_index(xfer->elements, i);
-	    xfer->repr = newvstralloc(xfer->repr,
-		xfer->repr, (i==0)?"":" -> ", xfer_element_repr(elt), NULL);
+	    tmpbuf = g_strconcat(xfer->repr, (i==0)?"":" -> ", xfer_element_repr(elt), NULL);
+	    g_free(xfer->repr);
+	    xfer->repr = tmpbuf;
 	}
-	xfer->repr = newvstralloc(xfer->repr, xfer->repr, ")>", NULL);
+	tmpbuf = g_strconcat(xfer->repr, ")>", NULL);
+	g_free(xfer->repr);
+	xfer->repr = tmpbuf;
     }
 
     return xfer->repr;
@@ -180,7 +187,7 @@ xfer_repr(
 void
 xfer_start(
     Xfer *xfer,
-    gint64 offset G_GNUC_UNUSED,
+    gint64 offset,
     gint64 size)
 {
     unsigned int len;
@@ -188,9 +195,8 @@ xfer_start(
     gboolean setup_ok;
 
     g_assert(xfer != NULL);
-    g_assert(xfer->status == XFER_INIT);
+    g_assert(xfer->status == XFER_INIT || xfer->status == XFER_DONE);
     g_assert(xfer->elements->len >= 2);
-    g_assert(offset == 0);
 
     g_debug("Starting %s", xfer_repr(xfer));
     /* set the status to XFER_START and add a reference to our count, so that
@@ -232,9 +238,10 @@ xfer_start(
 		elt->downstream = g_ptr_array_index(xfer->elements, i+1);
 	}
 
-	/* Set size for first element */
-	if (size) {
+	/* Set offset and size for first element */
+	{
 	    XferElement *xe = (XferElement *)g_ptr_array_index(xfer->elements, 0);
+	    xfer_element_set_offset(xe, offset);
 	    xfer_element_set_size(xe, size);
 	}
 
@@ -264,6 +271,18 @@ xfer_start(
     }
 }
 
+void
+xfer_set_offset_and_size(
+    Xfer *xfer,
+    gint64 offset,
+    gint64 size)
+{
+    /* Set offset for first element */
+    XferElement *xe = (XferElement *)g_ptr_array_index(xfer->elements, 0);
+    xfer_element_set_offset(xe, offset);
+    xfer_element_set_size(xe, size);
+}
+
 void
 xfer_cancel(
     Xfer *xfer)
@@ -289,7 +308,7 @@ xfer_set_status(
     /* check that this state transition is valid */
     switch (status) {
     case XFER_START:
-        g_assert(xfer->status == XFER_INIT);
+        g_assert(xfer->status == XFER_INIT || xfer->status == XFER_DONE);
         break;
     case XFER_RUNNING:
         g_assert(xfer->status == XFER_START);
@@ -346,8 +365,12 @@ xfer_mech_name(
 	case XFER_MECH_WRITEFD: return "WRITEFD";
 	case XFER_MECH_PULL_BUFFER: return "PULL_BUFFER";
 	case XFER_MECH_PUSH_BUFFER: return "PUSH_BUFFER";
+	case XFER_MECH_PULL_BUFFER_STATIC: return "PULL_BUFFER_STATIC";
+	case XFER_MECH_PUSH_BUFFER_STATIC: return "PUSH_BUFFER_STATIC";
 	case XFER_MECH_DIRECTTCP_LISTEN: return "DIRECTTCP_LISTEN";
 	case XFER_MECH_DIRECTTCP_CONNECT: return "DIRECTTCP_CONNECT";
+	case XFER_MECH_MEM_RING: return "MEM_RING";
+	case XFER_MECH_SHM_RING: return "SHM_RING";
 	default: return "UNKNOWN";
     }
 }
@@ -358,7 +381,7 @@ xfer_mech_name(
  *
  * PAIR will be evaluated multiple times.
  */
-#define PAIR_COST(pair) (((pair).ops_per_byte << 8) + (pair).nthreads)
+#define PAIR_COST(pair) (((pair).ops_per_byte << 16) + ((pair).nalloc << 8) + (pair).nthreads)
 
 /* maximum cost */
 #define MAX_COST 0xffffff
@@ -434,6 +457,7 @@ static void
 link_elements(
     Xfer *xfer)
 {
+    char *tmpbuf;
     GPtrArray *new_elements;
     XferElement *elt;
     char *linkage_str;
@@ -497,15 +521,21 @@ link_elements(
 
     /* debug-log the xfer's linkage */
     len = xfer->elements->len;
-    linkage_str = stralloc("Final linkage: ");
+    linkage_str = g_strdup("Final linkage: ");
     for (i = 0; i < len; i++) {
 	XferElement *elt = g_ptr_array_index(xfer->elements, i);
 
-	if (i == 0)
-	    linkage_str = newvstralloc(linkage_str, linkage_str, xfer_element_repr(elt), NULL);
-	else
-	    linkage_str = newvstrallocf(linkage_str, "%s -(%s)-> %s",
+	if (i == 0) {
+	    tmpbuf = g_strconcat(linkage_str, xfer_element_repr(elt), NULL);
+	    g_free(linkage_str);
+	    linkage_str = tmpbuf;
+	}
+	else {
+	    tmpbuf = g_strdup_printf("%s -(%s)-> %s",
 		linkage_str, xfer_mech_name(elt->input_mech), xfer_element_repr(elt));
+	    g_free(linkage_str);
+	    linkage_str = tmpbuf;
+	}
     }
     g_debug("%s", linkage_str);
     amfree(linkage_str);
@@ -664,8 +694,9 @@ wait_until_xfer_cancelled(
     g_assert(xfer != NULL);
 
     g_mutex_lock(xfer->status_mutex);
-    while (xfer->status != XFER_CANCELLED && xfer->status != XFER_DONE)
+    while (xfer->status != XFER_CANCELLED && xfer->status != XFER_DONE) {
 	g_cond_wait(xfer->status_cond, xfer->status_mutex);
+    }
     seen_status = xfer->status;
     g_mutex_unlock(xfer->status_mutex);
 
@@ -706,6 +737,8 @@ xfer_cancel_with_error(
     msg->message = g_strdup_vprintf(fmt, argp);
     arglist_end(argp);
 
+    g_debug("xfer_cancel_with_error: %s", msg->message);
+
     /* send the XMSG_ERROR */
     xfer_queue_message(elt->xfer, msg);
 
