@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2013-2016 Carbonite, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +16,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * Contact information: Zmanda Inc., 465 S. Mathilda Ave., Suite 300
- * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+ * Contact information: Carbonite Inc., 756 N Pastoria Ave
+ * Sunnyvale, CA 94085, or: http://www.zmanda.com
  */
 
 #include "amanda.h"
@@ -77,10 +78,14 @@ xmsg_repr(
 	    case XMSG_CANCEL: typ = "CANCEL"; break;
 	    case XMSG_PART_DONE: typ = "PART_DONE"; break;
 	    case XMSG_READY: typ = "READY"; break;
+	    case XMSG_CHUNK_DONE: typ = "CHUNK_DONE"; break;
+	    case XMSG_CRC: typ = "CRC"; break;
+	    case XMSG_NO_SPACE: typ = "NO_SPACE"; break;
+	    case XMSG_SEGMENT_DONE: typ = "SEGMENT_DONE"; break;
 	    default: typ = "**UNKNOWN**"; break;
 	}
 
-	msg->repr = vstrallocf("<XMsg@%p type=XMSG_%s elt=%s version=%d>",
+	msg->repr = g_strdup_printf("<XMsg@%p type=XMSG_%s elt=%s version=%d>",
 	    msg, typ, xfer_element_repr(msg->elt), msg->version);
     }
 
